@@ -26,20 +26,19 @@ import org.apache.hadoop.fs.ByteBufferReadable;
 import org.apache.hadoop.fs.CanUnbuffer;
 import org.apache.hadoop.fs.Seekable;
 import org.apache.hadoop.hdds.client.BlockID;
+import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ChunkInfo;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerCommandRequestProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerCommandResponseProto;
-import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ChunkInfo;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ReadChunkResponseProto;
 import org.apache.hadoop.hdds.scm.XceiverClientFactory;
 import org.apache.hadoop.hdds.scm.XceiverClientSpi;
-import org.apache.hadoop.hdds.scm.container.common.helpers.StorageContainerException;
+import org.apache.hadoop.hdds.scm.XceiverClientSpi.Validator;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.ozone.common.Checksum;
 import org.apache.hadoop.ozone.common.ChecksumData;
 import org.apache.hadoop.ozone.common.OzoneChecksumException;
 import org.apache.hadoop.ozone.common.utils.BufferUtils;
 import org.apache.hadoop.security.token.Token;
-import org.apache.hadoop.security.token.TokenIdentifier;
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
 
 import java.io.EOFException;
@@ -49,8 +48,6 @@ import java.nio.ByteBuffer;
 import java.util.Arrays;
 import java.util.List;
 import java.util.function.Supplier;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 
 /**
  * An {@link InputStream} called from BlockInputStream to read a chunk from the
@@ -60,16 +57,13 @@ import org.slf4j.LoggerFactory;
 public class ChunkInputStream extends InputStream
     implements Seekable, CanUnbuffer, ByteBufferReadable {
 
-  private static final Logger LOG =
-      LoggerFactory.getLogger(ChunkInputStream.class);
-
-  private ChunkInfo chunkInfo;
+  private final ChunkInfo chunkInfo;
   private final long length;
   private final BlockID blockID;
   private final XceiverClientFactory xceiverClientFactory;
   private XceiverClientSpi xceiverClient;
   private final Supplier<Pipeline> pipelineSupplier;
-  private boolean verifyChecksum;
+  private final boolean verifyChecksum;
   private boolean allocated = false;
   // Buffers to store the chunk data read from the DN container
   private ByteBuffer[] buffers;
@@ -101,21 +95,24 @@ public class ChunkInputStream extends InputStream
   // retry.  Once the chunk is read, this variable is reset.
   private long chunkPosition = -1;
 
-  private final Token<? extends TokenIdentifier> token;
+  private final Supplier<Token<?>> tokenSupplier;
 
   private static final int EOF = -1;
+  private final List<Validator> validators;
 
   ChunkInputStream(ChunkInfo chunkInfo, BlockID blockId,
       XceiverClientFactory xceiverClientFactory,
       Supplier<Pipeline> pipelineSupplier,
-      boolean verifyChecksum, Token<? extends TokenIdentifier> token) {
+      boolean verifyChecksum,
+      Supplier<Token<?>> tokenSupplier) {
     this.chunkInfo = chunkInfo;
     this.length = chunkInfo.getLen();
     this.blockID = blockId;
     this.xceiverClientFactory = xceiverClientFactory;
     this.pipelineSupplier = pipelineSupplier;
     this.verifyChecksum = verifyChecksum;
-    this.token = token;
+    this.tokenSupplier = tokenSupplier;
+    validators = ContainerProtocolCalls.toValidatorList(this::validateChunk);
   }
 
   public synchronized long getRemaining() {
@@ -276,7 +273,7 @@ public class ChunkInputStream extends InputStream
   }
 
   @Override
-  public boolean seekToNewSource(long targetPos) throws IOException {
+  public boolean seekToNewSource(long targetPos) {
     return false;
   }
 
@@ -325,7 +322,7 @@ public class ChunkInputStream extends InputStream
       if (buffersHaveData()) {
         // Data is available from buffers
         ByteBuffer bb = buffers[bufferIndex];
-        return len > bb.remaining() ? bb.remaining() : len;
+        return Math.min(len, bb.remaining());
       } else if (dataRemainingInChunk()) {
         // There is more data in the chunk stream which has not
         // been read into the buffers yet.
@@ -423,22 +420,10 @@ public class ChunkInputStream extends InputStream
   @VisibleForTesting
   protected ByteBuffer[] readChunk(ChunkInfo readChunkInfo)
       throws IOException {
-    ReadChunkResponseProto readChunkResponse;
 
-    try {
-      List<CheckedBiFunction> validators =
-          ContainerProtocolCalls.getValidatorList();
-      validators.add(validator);
-
-      readChunkResponse = ContainerProtocolCalls.readChunk(xceiverClient,
-          readChunkInfo, blockID, validators, token);
-
-    } catch (IOException e) {
-      if (e instanceof StorageContainerException) {
-        throw e;
-      }
-      throw new IOException("Unexpected OzoneException: " + e.toString(), e);
-    }
+    ReadChunkResponseProto readChunkResponse =
+        ContainerProtocolCalls.readChunk(xceiverClient,
+            readChunkInfo, blockID, validators, tokenSupplier.get());
 
     if (readChunkResponse.hasData()) {
       return readChunkResponse.getData().asReadOnlyByteBufferList()
@@ -453,56 +438,57 @@ public class ChunkInputStream extends InputStream
     }
   }
 
-  private CheckedBiFunction<ContainerCommandRequestProto,
-      ContainerCommandResponseProto, IOException> validator =
-          (request, response) -> {
-            final ChunkInfo reqChunkInfo =
-                request.getReadChunk().getChunkData();
+  private void validateChunk(
+      ContainerCommandRequestProto request,
+      ContainerCommandResponseProto response
+  ) throws OzoneChecksumException {
+    final ChunkInfo reqChunkInfo =
+        request.getReadChunk().getChunkData();
 
-            ReadChunkResponseProto readChunkResponse = response.getReadChunk();
-            List<ByteString> byteStrings;
-            boolean isV0 = false;
+    ReadChunkResponseProto readChunkResponse = response.getReadChunk();
+    List<ByteString> byteStrings;
+    boolean isV0 = false;
 
-            if (readChunkResponse.hasData()) {
-              ByteString byteString = readChunkResponse.getData();
-              if (byteString.size() != reqChunkInfo.getLen()) {
-                // Bytes read from chunk should be equal to chunk size.
-                throw new OzoneChecksumException(String.format(
-                    "Inconsistent read for chunk=%s len=%d bytesRead=%d",
-                    reqChunkInfo.getChunkName(), reqChunkInfo.getLen(),
-                    byteString.size()));
-              }
-              byteStrings = new ArrayList<>();
-              byteStrings.add(byteString);
-              isV0 = true;
-            } else {
-              byteStrings = readChunkResponse.getDataBuffers().getBuffersList();
-              long buffersLen = BufferUtils.getBuffersLen(byteStrings);
-              if (buffersLen != reqChunkInfo.getLen()) {
-                // Bytes read from chunk should be equal to chunk size.
-                throw new OzoneChecksumException(String.format(
-                    "Inconsistent read for chunk=%s len=%d bytesRead=%d",
-                    reqChunkInfo.getChunkName(), reqChunkInfo.getLen(),
-                    buffersLen));
-              }
-            }
+    if (readChunkResponse.hasData()) {
+      ByteString byteString = readChunkResponse.getData();
+      if (byteString.size() != reqChunkInfo.getLen()) {
+        // Bytes read from chunk should be equal to chunk size.
+        throw new OzoneChecksumException(String.format(
+            "Inconsistent read for chunk=%s len=%d bytesRead=%d",
+            reqChunkInfo.getChunkName(), reqChunkInfo.getLen(),
+            byteString.size()));
+      }
+      byteStrings = new ArrayList<>();
+      byteStrings.add(byteString);
+      isV0 = true;
+    } else {
+      byteStrings = readChunkResponse.getDataBuffers().getBuffersList();
+      long buffersLen = BufferUtils.getBuffersLen(byteStrings);
+      if (buffersLen != reqChunkInfo.getLen()) {
+        // Bytes read from chunk should be equal to chunk size.
+        throw new OzoneChecksumException(String.format(
+            "Inconsistent read for chunk=%s len=%d bytesRead=%d",
+            reqChunkInfo.getChunkName(), reqChunkInfo.getLen(),
+            buffersLen));
+      }
+    }
 
-            if (verifyChecksum) {
-              ChecksumData checksumData = ChecksumData.getFromProtoBuf(
-                  chunkInfo.getChecksumData());
+    if (verifyChecksum) {
+      ChecksumData checksumData = ChecksumData.getFromProtoBuf(
+          chunkInfo.getChecksumData());
 
-              // ChecksumData stores checksum for each 'numBytesPerChecksum'
-              // number of bytes in a list. Compute the index of the first
-              // checksum to match with the read data
+      // ChecksumData stores checksum for each 'numBytesPerChecksum'
+      // number of bytes in a list. Compute the index of the first
+      // checksum to match with the read data
 
-              long relativeOffset = reqChunkInfo.getOffset() -
-                  chunkInfo.getOffset();
-              int bytesPerChecksum = checksumData.getBytesPerChecksum();
-              int startIndex = (int) (relativeOffset / bytesPerChecksum);
-              Checksum.verifyChecksum(byteStrings, checksumData, startIndex,
-                  isV0);
-            }
-          };
+      long relativeOffset = reqChunkInfo.getOffset() -
+          chunkInfo.getOffset();
+      int bytesPerChecksum = checksumData.getBytesPerChecksum();
+      int startIndex = (int) (relativeOffset / bytesPerChecksum);
+      Checksum.verifyChecksum(byteStrings, checksumData, startIndex,
+          isV0);
+    }
+  }
 
   /**
    * Return the offset and length of bytes that need to be read from the
@@ -652,15 +638,7 @@ public class ChunkInputStream extends InputStream
    * Check if current buffer had been read till the end.
    */
   private boolean bufferEOF() {
-    if (!allocated) {
-      // Chunk data has not been read yet
-      return false;
-    }
-
-    if (!buffers[bufferIndex].hasRemaining()) {
-      return true;
-    }
-    return false;
+    return allocated && !buffers[bufferIndex].hasRemaining();
   }
 
   /**
