@@ -22,8 +22,6 @@ import java.io.IOException;
 import java.util.Map;
 import java.util.Random;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.ExecutionException;
-import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicInteger;
 
 import org.apache.hadoop.hdds.client.BlockID;
@@ -45,10 +43,11 @@ import org.apache.hadoop.hdds.scm.pipeline.MockPipeline;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+
 /**
  * UNIT test for BlockOutputStream.
  * <p>
@@ -105,7 +104,7 @@ public class TestBlockOutputStreamCorrectness {
     config.setChecksumType(ChecksumType.NONE);
     config.setBytesPerChecksum(256 * 1024);
 
-    BlockOutputStream outputStream = new RatisBlockOutputStream(
+    return new RatisBlockOutputStream(
         new BlockID(1L, 1L),
         xcm,
         pipeline,
@@ -113,7 +112,6 @@ public class TestBlockOutputStreamCorrectness {
         config,
         null,
         ContainerClientMetrics.acquire());
-    return outputStream;
   }
 
   /**
@@ -123,9 +121,9 @@ public class TestBlockOutputStreamCorrectness {
 
     private final Pipeline pipeline;
 
-    private Random expectedRandomStream = new Random(SEED);
+    private final Random expectedRandomStream = new Random(SEED);
 
-    private AtomicInteger counter = new AtomicInteger();
+    private final AtomicInteger counter = new AtomicInteger();
 
     MockXceiverClientSpi(Pipeline pipeline) {
       super();
@@ -133,12 +131,7 @@ public class TestBlockOutputStreamCorrectness {
     }
 
     @Override
-    public void connect() throws Exception {
-
-    }
-
-    @Override
-    public void connect(String encodedToken) throws Exception {
+    public void connect() {
 
     }
 
@@ -155,9 +148,7 @@ public class TestBlockOutputStreamCorrectness {
     @Override
     public XceiverClientReply sendCommandAsync(
         ContainerCommandRequestProto request
-    )
-        throws IOException, ExecutionException, InterruptedException {
-
+    ) {
       final ContainerCommandResponseProto.Builder builder =
           ContainerCommandResponseProto.newBuilder()
               .setResult(Result.SUCCESS)
@@ -178,10 +169,9 @@ public class TestBlockOutputStreamCorrectness {
       case WriteChunk:
         ByteString data = request.getWriteChunk().getData();
         final byte[] writePayload = data.toByteArray();
-        for (int i = 0; i < writePayload.length; i++) {
+        for (byte b : writePayload) {
           byte expectedByte = (byte) expectedRandomStream.nextInt();
-          Assert.assertEquals(expectedByte,
-              writePayload[i]);
+          assertEquals(expectedByte, b);
         }
         break;
       default:
@@ -200,9 +190,7 @@ public class TestBlockOutputStreamCorrectness {
     }
 
     @Override
-    public XceiverClientReply watchForCommit(long index)
-        throws InterruptedException, ExecutionException, TimeoutException,
-        IOException {
+    public XceiverClientReply watchForCommit(long index) {
       final ContainerCommandResponseProto.Builder builder =
           ContainerCommandResponseProto.newBuilder()
               .setCmdType(Type.WriteChunk)
@@ -220,10 +208,9 @@ public class TestBlockOutputStreamCorrectness {
 
     @Override
     public Map<DatanodeDetails, ContainerCommandResponseProto>
-        sendCommandOnAllNodes(ContainerCommandRequestProto request
-    ) throws IOException, InterruptedException {
+        sendCommandOnAllNodes(ContainerCommandRequestProto request) {
       return null;
     }
   }
 
-}
\ No newline at end of file
+}
