@@ -19,7 +19,7 @@
 package org.apache.hadoop.hdds.ratis;
 
 import java.io.IOException;
-import java.security.cert.X509Certificate;
+import java.nio.ByteBuffer;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -29,6 +29,7 @@ import java.util.UUID;
 import java.util.function.BiFunction;
 import java.util.stream.Collectors;
 
+import com.google.common.annotations.VisibleForTesting;
 import org.apache.hadoop.hdds.StringUtils;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
@@ -38,7 +39,7 @@ import org.apache.hadoop.hdds.ratis.conf.RatisClientConfig;
 import org.apache.hadoop.hdds.ratis.retrypolicy.RetryPolicyCreator;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
-import org.apache.hadoop.hdds.security.x509.SecurityConfig;
+import org.apache.hadoop.hdds.security.SecurityConfig;
 
 import org.apache.hadoop.hdfs.DFSConfigKeys;
 import org.apache.ratis.RaftConfigKeys;
@@ -46,20 +47,29 @@ import org.apache.ratis.client.RaftClient;
 import org.apache.ratis.client.RaftClientConfigKeys;
 import org.apache.ratis.conf.Parameters;
 import org.apache.ratis.conf.RaftProperties;
+import org.apache.ratis.datastream.SupportedDataStreamType;
 import org.apache.ratis.grpc.GrpcConfigKeys;
 import org.apache.ratis.grpc.GrpcTlsConfig;
+import org.apache.ratis.netty.NettyConfigKeys;
 import org.apache.ratis.proto.RaftProtos;
+import org.apache.ratis.protocol.GroupInfoReply;
+import org.apache.ratis.protocol.RaftClientReply;
 import org.apache.ratis.protocol.RaftGroup;
 import org.apache.ratis.protocol.RaftGroupId;
 import org.apache.ratis.protocol.RaftPeer;
 import org.apache.ratis.protocol.RaftPeerId;
+import org.apache.ratis.protocol.RoutingTable;
 import org.apache.ratis.retry.RetryPolicy;
 import org.apache.ratis.rpc.RpcType;
 import org.apache.ratis.rpc.SupportedRpcType;
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
+import org.apache.ratis.thirdparty.io.netty.buffer.ByteBuf;
+import org.apache.ratis.util.JvmPauseMonitor;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import javax.net.ssl.TrustManager;
+
 /**
  * Ratis helper methods.
  */
@@ -81,9 +91,18 @@ public final class RatisHelper {
   private static final RaftGroup EMPTY_GROUP = RaftGroup.valueOf(DUMMY_GROUP_ID,
       Collections.emptyList());
 
+  // Used for OM/SCM HA transfer leadership
+  @VisibleForTesting
+  public static final int NEUTRAL_PRIORITY = 0;
+  private static final int HIGHER_PRIORITY = 1;
+
   private RatisHelper() {
   }
 
+  public static JvmPauseMonitor newJvmPauseMonitor(String name) {
+    return JvmPauseMonitor.newBuilder().setName(name).build();
+  }
+
   private static String toRaftPeerIdString(DatanodeDetails id) {
     return id.getUuidString();
   }
@@ -134,7 +153,9 @@ public final class RatisHelper {
         .setId(toRaftPeerId(dn))
         .setAddress(toRaftPeerAddress(dn, Port.Name.RATIS_SERVER))
         .setAdminAddress(toRaftPeerAddress(dn, Port.Name.RATIS_ADMIN))
-        .setClientAddress(toRaftPeerAddress(dn, Port.Name.RATIS));
+        .setClientAddress(toRaftPeerAddress(dn, Port.Name.RATIS))
+        .setDataStreamAddress(
+            toRaftPeerAddress(dn, Port.Name.RATIS_DATASTREAM));
   }
 
   private static List<RaftPeer> toRaftPeers(Pipeline pipeline) {
@@ -183,11 +204,23 @@ public final class RatisHelper {
         toRaftPeers(pipeline));
   }
 
+  /**
+   * Create a Raft client used primarily for Ozone client communications with
+   * the Ratis pipeline.
+   * @param rpcType rpc type
+   * @param pipeline pipeline
+   * @param retryPolicy retry policy
+   * @param tlsConfig tls config
+   * @param ozoneConfiguration configuration
+   * @return Raft client
+   * @throws IOException IOException
+   */
   public static RaftClient newRaftClient(RpcType rpcType, Pipeline pipeline,
       RetryPolicy retryPolicy, GrpcTlsConfig tlsConfig,
       ConfigurationSource ozoneConfiguration) throws IOException {
     return newRaftClient(rpcType,
         toRaftPeerId(pipeline.getLeaderNode()),
+        toRaftPeer(pipeline.getClosestNode()),
         newRaftGroup(RaftGroupId.valueOf(pipeline.getId().getId()),
             pipeline.getNodes()), retryPolicy, tlsConfig, ozoneConfiguration);
   }
@@ -207,7 +240,7 @@ public final class RatisHelper {
   public static RaftClient newRaftClient(RpcType rpcType, RaftPeer leader,
       RetryPolicy retryPolicy, GrpcTlsConfig tlsConfig,
       ConfigurationSource configuration) {
-    return newRaftClient(rpcType, leader.getId(),
+    return newRaftClient(rpcType, leader.getId(), leader,
         newRaftGroup(Collections.singletonList(leader)), retryPolicy,
         tlsConfig, configuration);
   }
@@ -215,20 +248,21 @@ public final class RatisHelper {
   public static RaftClient newRaftClient(RpcType rpcType, RaftPeer leader,
       RetryPolicy retryPolicy,
       ConfigurationSource ozoneConfiguration) {
-    return newRaftClient(rpcType, leader.getId(),
+    return newRaftClient(rpcType, leader.getId(), leader,
         newRaftGroup(Collections.singletonList(leader)), retryPolicy, null,
         ozoneConfiguration);
   }
 
   @SuppressWarnings("checkstyle:ParameterNumber")
   private static RaftClient newRaftClient(RpcType rpcType, RaftPeerId leader,
-      RaftGroup group, RetryPolicy retryPolicy,
+      RaftPeer primary, RaftGroup group, RetryPolicy retryPolicy,
       GrpcTlsConfig tlsConfig, ConfigurationSource ozoneConfiguration) {
     if (LOG.isTraceEnabled()) {
       LOG.trace("newRaftClient: {}, leader={}, group={}",
           rpcType, leader, group);
     }
     final RaftProperties properties = newRaftProperties(rpcType);
+    enableNettyStreaming(properties);
 
     // Set the ratis client headers which are matching with regex.
     createRaftClientProperties(ozoneConfiguration, properties);
@@ -236,6 +270,7 @@ public final class RatisHelper {
     return RaftClient.newBuilder()
         .setRaftGroup(group)
         .setLeaderId(leader)
+        .setPrimaryDataStreamServer(primary)
         .setProperties(properties)
         .setParameters(setClientTlsConf(rpcType, tlsConfig))
         .setRetryPolicy(retryPolicy)
@@ -265,6 +300,7 @@ public final class RatisHelper {
       GrpcTlsConfig tlsConfig) {
     if (tlsConfig != null) {
       GrpcConfigKeys.Client.setTlsConf(parameters, tlsConfig);
+      NettyConfigKeys.DataStream.Client.setTlsConf(parameters, tlsConfig);
     }
   }
 
@@ -275,6 +311,8 @@ public final class RatisHelper {
       GrpcConfigKeys.Server.setTlsConf(parameters, serverConf);
       GrpcConfigKeys.TLS.setConf(parameters, serverConf);
       setAdminTlsConf(parameters, serverConf);
+
+      NettyConfigKeys.DataStream.Server.setTlsConf(parameters, serverConf);
     }
     setClientTlsConf(parameters, clientConf);
     return parameters;
@@ -296,12 +334,16 @@ public final class RatisHelper {
     return properties;
   }
 
+  public static RaftProperties enableNettyStreaming(RaftProperties properties) {
+    RaftConfigKeys.DataStream.setType(properties,
+        SupportedDataStreamType.NETTY);
+    return properties;
+  }
+
   /**
    * Set all client properties matching with regex
    * {@link RatisHelper#HDDS_DATANODE_RATIS_PREFIX_KEY} in
    * ozone configuration object and configure it to RaftProperties.
-   * @param ozoneConf
-   * @param raftProperties
    */
   public static void createRaftClientProperties(ConfigurationSource ozoneConf,
       RaftProperties raftProperties) {
@@ -310,7 +352,8 @@ public final class RatisHelper {
     Map<String, String> ratisClientConf =
         getDatanodeRatisPrefixProps(ozoneConf);
     ratisClientConf.forEach((key, val) -> {
-      if (isClientConfig(key) || isGrpcClientConfig(key)) {
+      if (isClientConfig(key) || isGrpcClientConfig(key)
+              || isNettyStreamConfig(key)) {
         raftProperties.set(key, val);
       }
     });
@@ -326,12 +369,19 @@ public final class RatisHelper {
         !key.startsWith(GrpcConfigKeys.Admin.PREFIX) &&
         !key.startsWith(GrpcConfigKeys.Server.PREFIX);
   }
+
+  private static boolean isNettyStreamConfig(String key) {
+    return key.startsWith(NettyConfigKeys.DataStream.PREFIX);
+  }
+
+  private static boolean isStreamClientConfig(String key) {
+    return key.startsWith(RaftClientConfigKeys.DataStream.PREFIX);
+  }
+
   /**
    * Set all server properties matching with prefix
    * {@link RatisHelper#HDDS_DATANODE_RATIS_PREFIX_KEY} in
    * ozone configuration object and configure it to RaftProperties.
-   * @param ozoneConf
-   * @param raftProperties
    */
   public static void createRaftServerProperties(ConfigurationSource ozoneConf,
        RaftProperties raftProperties) {
@@ -340,7 +390,8 @@ public final class RatisHelper {
         getDatanodeRatisPrefixProps(ozoneConf);
     ratisServerConf.forEach((key, val) -> {
       // Exclude ratis client configuration.
-      if (!isClientConfig(key)) {
+      if (isNettyStreamConfig(key) || isStreamClientConfig(key) ||
+          !isClientConfig(key)) {
         raftProperties.set(key, val);
       }
     });
@@ -356,11 +407,10 @@ public final class RatisHelper {
   // For External gRPC client to server with gRPC TLS.
   // No mTLS for external client as SCM CA does not issued certificates for them
   public static GrpcTlsConfig createTlsClientConfig(SecurityConfig conf,
-      List<X509Certificate> caCerts) {
+      TrustManager trustManager) {
     GrpcTlsConfig tlsConfig = null;
     if (conf.isSecurityEnabled() && conf.isGrpcTlsEnabled()) {
-      tlsConfig = new GrpcTlsConfig(null, null,
-          caCerts, false);
+      tlsConfig = new GrpcTlsConfig(null, trustManager, false);
     }
     return tlsConfig;
   }
@@ -403,4 +453,154 @@ public final class RatisHelper {
       throw new RuntimeException(e);
     }
   }
+  
+  public static RoutingTable getRoutingTable(Pipeline pipeline) {
+    RaftPeerId primaryId = null;
+    List<RaftPeerId> raftPeers = new ArrayList<>();
+
+    for (DatanodeDetails dn : pipeline.getNodesInOrder()) {
+      final RaftPeerId raftPeerId = RaftPeerId.valueOf(dn.getUuidString());
+      try {
+        if (dn == pipeline.getClosestNode()) {
+          primaryId = raftPeerId;
+        }
+      } catch (IOException e) {
+        LOG.error("Can not get primary node from the pipeline: {} with " +
+            "exception: {}", pipeline, e.getLocalizedMessage());
+        return null;
+      }
+      raftPeers.add(raftPeerId);
+    }
+
+    RoutingTable.Builder builder = RoutingTable.newBuilder();
+    RaftPeerId previousId = primaryId;
+    for (RaftPeerId peerId : raftPeers) {
+      if (peerId.equals(primaryId)) {
+        continue;
+      }
+      builder.addSuccessor(previousId, peerId);
+      previousId = peerId;
+    }
+
+    return builder.build();
+  }
+
+  public static void debug(ByteBuffer buffer, String name, Logger log) {
+    if (!log.isDebugEnabled()) {
+      return;
+    }
+    buffer = buffer.duplicate();
+    final StringBuilder builder = new StringBuilder();
+    for (int i = 1; buffer.remaining() > 0; i++) {
+      builder.append(buffer.get()).append(i % 20 == 0 ? "\n  " : ", ");
+    }
+    log.debug("{}: {}\n  {}", name, buffer, builder);
+  }
+
+  public static void debug(ByteBuf buf, String name, Logger log) {
+    if (!log.isDebugEnabled()) {
+      return;
+    }
+    buf = buf.duplicate();
+    final StringBuilder builder = new StringBuilder();
+    for (int i = 1; buf.readableBytes() > 0; i++) {
+      builder.append(buf.readByte()).append(i % 20 == 0 ? "\n  " : ", ");
+    }
+    log.debug("{}: {}\n  {}", name, buf, builder);
+  }
+
+  static RaftPeer newRaftPeer(RaftPeer peer, RaftPeerId target) {
+    final int priority = peer.getId().equals(target) ?
+        HIGHER_PRIORITY : NEUTRAL_PRIORITY;
+    return RaftPeer.newBuilder(peer).setPriority(priority).build();
+  }
+
+
+  /**
+   * Use raft client to send admin request, transfer the leadership.
+   * 1. Set priority and send setConfiguration request
+   * 2. Trigger transferLeadership API.
+   *
+   * @param group         the Raft group
+   * @param targetPeerId  the target expected leader
+   */
+  public static void transferRatisLeadership(ConfigurationSource conf,
+      RaftGroup group, RaftPeerId targetPeerId, GrpcTlsConfig tlsConfig)
+      throws IOException {
+    if (group.getPeer(targetPeerId) == null) {
+      throw new IOException("Target " + targetPeerId + " not found in group "
+          + group.getPeers().stream().map(RaftPeer::getId)
+              .collect(Collectors.toList()) + ".");
+    }
+
+    LOG.info("Start transferring leadership to {}", targetPeerId);
+    try (RaftClient client = newRaftClient(SupportedRpcType.GRPC, null,
+        null, group, createRetryPolicy(conf), tlsConfig, conf)) {
+      final GroupInfoReply info = client.getGroupManagementApi(targetPeerId)
+          .info(group.getGroupId());
+      if (!info.isSuccess()) {
+        throw new IOException("Failed to get info for " + group.getGroupId()
+            + " from " + targetPeerId);
+      }
+
+      final RaftGroup remote = info.getGroup();
+      if (!group.equals(remote)) {
+        throw new IOException("Group mismatched: the given group " + group
+          + " and the remote group from " + targetPeerId + " are not equal."
+          + "\n Given: " + group
+          + "\n Remote: " + remote);
+      }
+
+      RaftClientReply setConf = null;
+      try {
+        // Set priority
+        final List<RaftPeer> peersWithNewPriorities = group.getPeers().stream()
+            .map(peer -> newRaftPeer(peer, targetPeerId))
+            .collect(Collectors.toList());
+        // Set new configuration
+        setConf = client.admin().setConfiguration(peersWithNewPriorities);
+        if (setConf.isSuccess()) {
+          LOG.info("Successfully set priority: {}", peersWithNewPriorities);
+        } else {
+          throw new IOException("Failed to set priority.",
+              setConf.getException());
+        }
+
+        // Trigger the transferLeadership
+        final RaftClientReply reply = client.admin()
+            .transferLeadership(targetPeerId, 60_000);
+        if (reply.isSuccess()) {
+          LOG.info("Successfully transferred leadership to {}.", targetPeerId);
+        } else {
+          LOG.warn("Failed to transfer leadership to {}. Ratis reply: {}",
+              targetPeerId, reply);
+          throw new IOException(reply.getException());
+        }
+      } finally {
+        // Reset peers regardless of the result of transfer leadership
+        if (setConf != null && setConf.isSuccess()) {
+          resetPriorities(remote, client);
+        }
+      }
+    }
+  }
+
+  private static void resetPriorities(RaftGroup original, RaftClient client) {
+    final List<RaftPeer> resetPeers = original.getPeers().stream()
+        .map(originalPeer -> RaftPeer.newBuilder(originalPeer)
+            .setPriority(NEUTRAL_PRIORITY).build())
+        .collect(Collectors.toList());
+    LOG.info("Resetting Raft peers priorities to {}", resetPeers);
+    try {
+      RaftClientReply reply = client.admin().setConfiguration(resetPeers);
+      if (reply.isSuccess()) {
+        LOG.info("Successfully reset priorities: {}", original);
+      } else {
+        LOG.warn("Failed to reset priorities: {}, reply: {}", original, reply);
+      }
+    } catch (IOException e) {
+      LOG.error("Failed to reset priorities for " + original, e);
+      // Not re-thrown in order to keep the main exception, if there is any.
+    }
+  }
 }
