@@ -32,15 +32,24 @@ import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerC
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerCommandResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
-import org.apache.hadoop.hdds.scm.storage.CheckedBiFunction;
 
 import com.google.common.annotations.VisibleForTesting;
+import org.apache.ratis.util.function.CheckedBiConsumer;
 
 /**
  * A Client for the storageContainer protocol.
  */
 public abstract class XceiverClientSpi implements Closeable {
 
+  /**
+   * Validator for container command request/response.
+   */
+  public interface Validator extends
+      CheckedBiConsumer<ContainerCommandRequestProto,
+          ContainerCommandResponseProto, IOException> {
+    // just a shortcut to avoid having to repeat long list of generic parameters
+  }
+
   private final AtomicInteger referenceCount;
   private boolean isEvicted;
 
@@ -82,12 +91,6 @@ public abstract class XceiverClientSpi implements Closeable {
    */
   public abstract void connect() throws Exception;
 
-  /**
-   * Connects to the leader in the pipeline using encoded token. To be used
-   * in a secure cluster.
-   */
-  public abstract void connect(String encodedToken) throws Exception;
-
   @Override
   public abstract void close();
 
@@ -103,7 +106,6 @@ public abstract class XceiverClientSpi implements Closeable {
    * Sends a given command to server and gets the reply back.
    * @param request Request
    * @return Response to the command
-   * @throws IOException
    */
   public ContainerCommandResponseProto sendCommand(
       ContainerCommandRequestProto request) throws IOException {
@@ -126,17 +128,17 @@ public abstract class XceiverClientSpi implements Closeable {
    * @param request Request
    * @param validators functions to validate the response
    * @return Response to the command
-   * @throws IOException
    */
   public ContainerCommandResponseProto sendCommand(
-      ContainerCommandRequestProto request, List<CheckedBiFunction> validators)
+      ContainerCommandRequestProto request,
+      List<Validator> validators)
       throws IOException {
     try {
       XceiverClientReply reply;
       reply = sendCommandAsync(request);
       ContainerCommandResponseProto responseProto = reply.getResponse().get();
-      for (CheckedBiFunction function : validators) {
-        function.apply(request, responseProto);
+      for (Validator function : validators) {
+        function.accept(request, responseProto);
       }
       return responseProto;
     } catch (InterruptedException e) {
@@ -158,7 +160,6 @@ public abstract class XceiverClientSpi implements Closeable {
    *
    * @param request Request
    * @return Response to the command
-   * @throws IOException
    */
   public abstract XceiverClientReply
       sendCommandAsync(ContainerCommandRequestProto request)
@@ -176,10 +177,6 @@ public abstract class XceiverClientSpi implements Closeable {
    * @param index index to watch for
    * @return reply containing the min commit index replicated to all or majority
    *         servers in case of a failure
-   * @throws InterruptedException
-   * @throws ExecutionException
-   * @throws TimeoutException
-   * @throws IOException
    */
   public abstract XceiverClientReply watchForCommit(long index)
       throws InterruptedException, ExecutionException, TimeoutException,
