@@ -133,5 +133,12 @@ public class SCMException extends IOException {
     CONTAINER_REPLICA_NOT_FOUND,
     FAILED_TO_CONNECT_TO_CRL_SERVICE,
     FAILED_TO_ADD_CRL_CLIENT,
+    INVALID_PIPELINE_STATE,
+    DUPLICATED_PIPELINE_ID,
+    TIMEOUT,
+    CA_ROTATION_IN_PROGRESS,
+    CA_ROTATION_IN_POST_PROGRESS,
+    CONTAINER_ALREADY_CLOSED,
+    CONTAINER_ALREADY_CLOSING
   }
 }
