@@ -1,4 +1,4 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
  * distributed with this work for additional information
@@ -26,12 +26,14 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.LinkedHashMap;
+import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.UUID;
 
+import com.fasterxml.jackson.annotation.JsonIgnore;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.commons.lang3.builder.EqualsBuilder;
 import org.apache.commons.lang3.builder.HashCodeBuilder;
@@ -41,6 +43,10 @@ import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.DatanodeDetailsProto;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType;
+import org.apache.hadoop.hdds.utils.db.Codec;
+import org.apache.hadoop.hdds.utils.db.DelegatedCodec;
+import org.apache.hadoop.hdds.utils.db.Proto2Codec;
+import org.apache.hadoop.ozone.ClientVersion;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -50,12 +56,26 @@ import com.google.common.base.Preconditions;
  * Represents a group of datanodes which store a container.
  */
 public final class Pipeline {
+  /**
+   * This codec is inconsistent since
+   * deserialize(serialize(original)) does not equal to original
+   * -- the creation time may change.
+   */
+  private static final Codec<Pipeline> CODEC = new DelegatedCodec<>(
+      Proto2Codec.get(HddsProtos.Pipeline.getDefaultInstance()),
+      Pipeline::getFromProtobufSetCreationTimestamp,
+      p -> p.getProtobufMessage(ClientVersion.CURRENT_VERSION),
+      DelegatedCodec.CopyType.UNSUPPORTED);
+
+  public static Codec<Pipeline> getCodec() {
+    return CODEC;
+  }
 
   private static final Logger LOG = LoggerFactory.getLogger(Pipeline.class);
   private final PipelineID id;
   private final ReplicationConfig replicationConfig;
 
-  private PipelineState state;
+  private final PipelineState state;
   private Map<DatanodeDetails, Long> nodeStatus;
   private Map<DatanodeDetails, Integer> replicaIndexes;
   // nodes with ordered distance to client
@@ -67,10 +87,21 @@ public final class Pipeline {
   // suggested leader id with high priority
   private final UUID suggestedLeaderId;
 
+  private final Instant stateEnterTime;
+
   /**
    * The immutable properties of pipeline object is used in
    * ContainerStateManager#getMatchingContainerByPipeline to take a lock on
    * the container allocations for a particular pipeline.
+   * <br><br>
+   * Since the Pipeline class is immutable, if we want to change the state of
+   * the Pipeline we should create a new Pipeline object with the new state.
+   * Make sure that you set the value of <i>creationTimestamp</i> properly while
+   * creating the new Pipeline object.
+   * <br><br>
+   * There is no need to worry about the value of <i>stateEnterTime</i> as it's
+   * set to <i>Instant.now</i> when you crate the Pipeline object as part of
+   * state change.
    */
   private Pipeline(PipelineID id,
       ReplicationConfig replicationConfig, PipelineState state,
@@ -82,6 +113,7 @@ public final class Pipeline {
     this.creationTimestamp = Instant.now();
     this.suggestedLeaderId = suggestedLeaderId;
     this.replicaIndexes = new HashMap<>();
+    this.stateEnterTime = Instant.now();
   }
 
   /**
@@ -120,6 +152,10 @@ public final class Pipeline {
     return creationTimestamp;
   }
 
+  public Instant getStateEnterTime() {
+    return stateEnterTime;
+  }
+
   /**
    * Return the suggested leaderId which has a high priority among DNs of the
    * pipeline.
@@ -153,6 +189,11 @@ public final class Pipeline {
     this.leaderId = leaderId;
   }
 
+  /** @return the number of datanodes in this pipeline. */
+  public int size() {
+    return nodeStatus.size();
+  }
+
   /**
    * Returns the list of nodes which form this pipeline.
    *
@@ -167,6 +208,7 @@ public final class Pipeline {
    *
    * @return Set of DatanodeDetails
    */
+  @JsonIgnore
   public Set<DatanodeDetails> getNodeSet() {
     return Collections.unmodifiableSet(nodeStatus.keySet());
   }
@@ -216,24 +258,54 @@ public final class Pipeline {
   }
 
   public DatanodeDetails getFirstNode() throws IOException {
+    return getFirstNode(null);
+  }
+
+  public DatanodeDetails getFirstNode(Set<DatanodeDetails> excluded)
+      throws IOException {
+    if (excluded == null) {
+      excluded = Collections.emptySet();
+    }
     if (nodeStatus.isEmpty()) {
       throw new IOException(String.format("Pipeline=%s is empty", id));
     }
-    return nodeStatus.keySet().iterator().next();
+    for (DatanodeDetails d : nodeStatus.keySet()) {
+      if (!excluded.contains(d)) {
+        return d;
+      }
+    }
+    throw new IOException(String.format(
+        "All nodes are excluded: Pipeline=%s, excluded=%s", id, excluded));
   }
 
   public DatanodeDetails getClosestNode() throws IOException {
-    if (nodesInOrder.get() == null || nodesInOrder.get().isEmpty()) {
-      LOG.debug("Nodes in order is empty, delegate to getFirstNode");
-      return getFirstNode();
-    }
-    return nodesInOrder.get().get(0);
+    return getClosestNode(null);
   }
 
+  public DatanodeDetails getClosestNode(Set<DatanodeDetails> excluded)
+      throws IOException {
+    if (excluded == null) {
+      excluded = Collections.emptySet();
+    }
+    if (nodesInOrder.get() == null || nodesInOrder.get().isEmpty()) {
+      LOG.debug("Nodes in order is empty, delegate to getFirstNode");
+      return getFirstNode(excluded);
+    }
+    for (DatanodeDetails d : nodesInOrder.get()) {
+      if (!excluded.contains(d)) {
+        return d;
+      }
+    }
+    throw new IOException(String.format(
+        "All nodes are excluded: Pipeline=%s, excluded=%s", id, excluded));
+  }
+
+  @JsonIgnore
   public boolean isClosed() {
     return state == PipelineState.CLOSED;
   }
 
+  @JsonIgnore
   public boolean isOpen() {
     return state == PipelineState.OPEN;
   }
@@ -265,7 +337,7 @@ public final class Pipeline {
 
   public boolean isHealthy() {
     // EC pipelines are not reported by the DN and do not have a leader. If a
-    // node goes stale or dead, EC pipelines will by closed like RATIS pipelines
+    // node goes stale or dead, EC pipelines will be closed like RATIS pipelines
     // but at the current time there are not other health metrics for EC.
     if (replicationConfig.getReplicationType() == ReplicationType.EC) {
       return true;
@@ -348,6 +420,14 @@ public final class Pipeline {
     return builder.build();
   }
 
+  static Pipeline getFromProtobufSetCreationTimestamp(
+      HddsProtos.Pipeline proto) throws UnknownPipelineStateException {
+    final Pipeline pipeline = getFromProtobuf(proto);
+    // When SCM is restarted, set Creation time with current time.
+    pipeline.setCreationTimestamp(Instant.now());
+    return pipeline;
+  }
+
   public static Pipeline getFromProtobuf(HddsProtos.Pipeline pipeline)
       throws UnknownPipelineStateException {
     Preconditions.checkNotNull(pipeline, "Pipeline is null");
@@ -408,7 +488,7 @@ public final class Pipeline {
     return new EqualsBuilder()
         .append(id, that.id)
         .append(replicationConfig, that.replicationConfig)
-        .append(getNodes(), that.getNodes())
+        .append(nodeStatus.keySet(), that.nodeStatus.keySet())
         .isEquals();
   }
 
@@ -508,6 +588,12 @@ public final class Pipeline {
     public Builder setNodes(List<DatanodeDetails> nodes) {
       this.nodeStatus = new LinkedHashMap<>();
       nodes.forEach(node -> nodeStatus.put(node, -1L));
+      if (nodesInOrder != null) {
+        // nodesInOrder may belong to another pipeline, avoid overwriting it
+        nodesInOrder = new LinkedList<>(nodesInOrder);
+        // drop nodes no longer part of the pipeline
+        nodesInOrder.retainAll(nodes);
+      }
       return this;
     }
 
