@@ -20,10 +20,14 @@ package org.apache.hadoop.hdds.scm.storage;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ExecutionException;
+import java.util.function.Function;
 
 import org.apache.hadoop.hdds.annotation.InterfaceStability;
 import org.apache.hadoop.hdds.client.BlockID;
@@ -47,15 +51,18 @@ import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.PutBlockRe
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.PutSmallFileRequestProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.PutSmallFileResponseProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ReadChunkRequestProto;
+import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ReadChunkResponseProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ReadContainerRequestProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ReadContainerResponseProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.Type;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.WriteChunkRequestProto;
 import org.apache.hadoop.hdds.scm.XceiverClientReply;
 import org.apache.hadoop.hdds.scm.XceiverClientSpi;
+import org.apache.hadoop.hdds.scm.XceiverClientSpi.Validator;
 import org.apache.hadoop.hdds.scm.container.common.helpers.BlockNotCommittedException;
 import org.apache.hadoop.hdds.scm.container.common.helpers.ContainerNotOpenException;
 import org.apache.hadoop.hdds.scm.container.common.helpers.StorageContainerException;
+import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.security.token.OzoneBlockTokenIdentifier;
 import org.apache.hadoop.ozone.common.Checksum;
 import org.apache.hadoop.ozone.common.ChecksumData;
@@ -63,12 +70,19 @@ import org.apache.hadoop.security.token.Token;
 
 import org.apache.hadoop.security.token.TokenIdentifier;
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
+import org.apache.ratis.util.function.CheckedFunction;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import static java.util.Collections.singletonList;
 
 /**
  * Implementation of all container protocol calls performed by Container
  * clients.
  */
 public final class ContainerProtocolCalls  {
+  private static final Logger LOG =
+      LoggerFactory.getLogger(ContainerProtocolCalls.class);
 
   /**
    * There is no need to instantiate this class.
@@ -120,36 +134,78 @@ public final class ContainerProtocolCalls  {
     return response.getListBlock();
   }
 
+  static <T> T tryEachDatanode(Pipeline pipeline,
+      CheckedFunction<DatanodeDetails, T, IOException> op,
+      Function<DatanodeDetails, String> toErrorMessage)
+      throws IOException {
+    final Set<DatanodeDetails> excluded = new HashSet<>();
+    for (; ;) {
+      final DatanodeDetails d = pipeline.getClosestNode(excluded);
+
+      try {
+        return op.apply(d);
+      } catch (IOException e) {
+        excluded.add(d);
+        if (excluded.size() < pipeline.size()) {
+          LOG.warn(toErrorMessage.apply(d)
+              + "; will try another datanode.", e);
+        } else {
+          throw e;
+        }
+      }
+    }
+  }
+
   /**
    * Calls the container protocol to get a container block.
    *
    * @param xceiverClient client to perform call
+   * @param validators functions to validate the response
    * @param datanodeBlockID blockID to identify container
    * @param token a token for this block (may be null)
    * @return container protocol get block response
    * @throws IOException if there is an I/O error while performing the call
    */
   public static GetBlockResponseProto getBlock(XceiverClientSpi xceiverClient,
+      List<Validator> validators,
       DatanodeBlockID datanodeBlockID,
       Token<? extends TokenIdentifier> token) throws IOException {
     GetBlockRequestProto.Builder readBlockRequest = GetBlockRequestProto
         .newBuilder()
         .setBlockID(datanodeBlockID);
-    String id = xceiverClient.getPipeline().getFirstNode().getUuidString();
-
     ContainerCommandRequestProto.Builder builder = ContainerCommandRequestProto
         .newBuilder()
         .setCmdType(Type.GetBlock)
         .setContainerID(datanodeBlockID.getContainerID())
-        .setDatanodeUuid(id)
         .setGetBlock(readBlockRequest);
     if (token != null) {
       builder.setEncodedToken(token.encodeToUrlString());
     }
 
-    ContainerCommandRequestProto request = builder.build();
+    return tryEachDatanode(xceiverClient.getPipeline(),
+        d -> getBlock(xceiverClient, validators, builder, d),
+        d -> toErrorMessage(datanodeBlockID, d));
+  }
+
+  static String toErrorMessage(DatanodeBlockID blockId, DatanodeDetails d) {
+    return String.format("Failed to get block #%s in container #%s from %s",
+        blockId.getLocalID(), blockId.getContainerID(), d);
+  }
+
+  public static GetBlockResponseProto getBlock(XceiverClientSpi xceiverClient,
+      DatanodeBlockID datanodeBlockID,
+      Token<? extends TokenIdentifier> token) throws IOException {
+    return getBlock(xceiverClient, getValidatorList(), datanodeBlockID, token);
+  }
+
+  private static GetBlockResponseProto getBlock(XceiverClientSpi xceiverClient,
+      List<Validator> validators,
+      ContainerCommandRequestProto.Builder builder,
+      DatanodeDetails datanode) throws IOException {
+    final ContainerCommandRequestProto request = builder
+        .setDatanodeUuid(datanode.getUuidString()).build();
     ContainerCommandResponseProto response =
-        xceiverClient.sendCommand(request, getValidatorList());
+        xceiverClient.sendCommand(request, validators);
     return response.getGetBlock();
   }
 
@@ -226,18 +282,24 @@ public final class ContainerProtocolCalls  {
    * @param token a token for this block (may be null)
    * @return putBlockResponse
    * @throws IOException if there is an error while performing the call
-   * @throws InterruptedException
-   * @throws ExecutionException
    */
   public static XceiverClientReply putBlockAsync(XceiverClientSpi xceiverClient,
       BlockData containerBlockData, boolean eof,
       Token<? extends TokenIdentifier> token)
       throws IOException, InterruptedException, ExecutionException {
+    final ContainerCommandRequestProto request = getPutBlockRequest(
+        xceiverClient.getPipeline(), containerBlockData, eof, token);
+    return xceiverClient.sendCommandAsync(request);
+  }
+
+  public static ContainerCommandRequestProto getPutBlockRequest(
+      Pipeline pipeline, BlockData containerBlockData, boolean eof,
+      Token<? extends TokenIdentifier> token) throws IOException {
     PutBlockRequestProto.Builder createBlockRequest =
         PutBlockRequestProto.newBuilder()
             .setBlockData(containerBlockData)
             .setEof(eof);
-    String id = xceiverClient.getPipeline().getFirstNode().getUuidString();
+    final String id = pipeline.getFirstNode().getUuidString();
     ContainerCommandRequestProto.Builder builder =
         ContainerCommandRequestProto.newBuilder().setCmdType(Type.PutBlock)
             .setContainerID(containerBlockData.getBlockID().getContainerID())
@@ -246,8 +308,7 @@ public final class ContainerProtocolCalls  {
     if (token != null) {
       builder.setEncodedToken(token.encodeToUrlString());
     }
-    ContainerCommandRequestProto request = builder.build();
-    return xceiverClient.sendCommandAsync(request);
+    return builder.build();
   }
 
   /**
@@ -263,58 +324,60 @@ public final class ContainerProtocolCalls  {
    */
   public static ContainerProtos.ReadChunkResponseProto readChunk(
       XceiverClientSpi xceiverClient, ChunkInfo chunk, BlockID blockID,
-      List<CheckedBiFunction> validators,
+      List<Validator> validators,
       Token<? extends TokenIdentifier> token) throws IOException {
     ReadChunkRequestProto.Builder readChunkRequest =
         ReadChunkRequestProto.newBuilder()
             .setBlockID(blockID.getDatanodeBlockIDProtobuf())
             .setChunkData(chunk)
             .setReadChunkVersion(ContainerProtos.ReadChunkVersion.V1);
-    String id = xceiverClient.getPipeline().getClosestNode().getUuidString();
     ContainerCommandRequestProto.Builder builder =
         ContainerCommandRequestProto.newBuilder().setCmdType(Type.ReadChunk)
             .setContainerID(blockID.getContainerID())
-            .setDatanodeUuid(id).setReadChunk(readChunkRequest);
+            .setReadChunk(readChunkRequest);
     if (token != null) {
       builder.setEncodedToken(token.encodeToUrlString());
     }
-    ContainerCommandRequestProto request = builder.build();
-    ContainerCommandResponseProto reply =
-        xceiverClient.sendCommand(request, validators);
-    return reply.getReadChunk();
+
+    return tryEachDatanode(xceiverClient.getPipeline(),
+        d -> readChunk(xceiverClient, chunk, blockID,
+            validators, builder, d),
+        d -> toErrorMessage(chunk, blockID, d));
   }
 
-  /**
-   * Calls the container protocol to write a chunk.
-   *
-   * @param xceiverClient client to perform call
-   * @param chunk information about chunk to write
-   * @param blockID ID of the block
-   * @param data the data of the chunk to write
-   * @param token a token for this block (may be null)
-   * @throws IOException if there is an error while performing the call
-   */
-  public static void writeChunk(XceiverClientSpi xceiverClient, ChunkInfo chunk,
-      BlockID blockID, ByteString data,
-      Token<OzoneBlockTokenIdentifier> token)
-      throws IOException {
-    WriteChunkRequestProto.Builder writeChunkRequest = WriteChunkRequestProto
-        .newBuilder()
-        .setBlockID(blockID.getDatanodeBlockIDProtobuf())
-        .setChunkData(chunk)
-        .setData(data);
-    String id = xceiverClient.getPipeline().getFirstNode().getUuidString();
-    ContainerCommandRequestProto.Builder builder = ContainerCommandRequestProto
-        .newBuilder()
-        .setCmdType(Type.WriteChunk)
-        .setContainerID(blockID.getContainerID())
-        .setDatanodeUuid(id)
-        .setWriteChunk(writeChunkRequest);
-    if (token != null) {
-      builder.setEncodedToken(token.encodeToUrlString());
+  private static ContainerProtos.ReadChunkResponseProto readChunk(
+      XceiverClientSpi xceiverClient, ChunkInfo chunk, BlockID blockID,
+      List<Validator> validators,
+      ContainerCommandRequestProto.Builder builder,
+      DatanodeDetails d) throws IOException {
+    final ContainerCommandRequestProto request = builder
+        .setDatanodeUuid(d.getUuidString()).build();
+    ContainerCommandResponseProto reply =
+        xceiverClient.sendCommand(request, validators);
+    final ReadChunkResponseProto response = reply.getReadChunk();
+    final long readLen = getLen(response);
+    if (readLen != chunk.getLen()) {
+      throw new IOException(toErrorMessage(chunk, blockID, d)
+          + ": readLen=" + readLen);
+    }
+    return response;
+  }
+
+  static String toErrorMessage(ChunkInfo chunk, BlockID blockId,
+      DatanodeDetails d) {
+    return String.format("Failed to read chunk %s (len=%s) %s from %s",
+        chunk.getChunkName(), chunk.getLen(), blockId, d);
+  }
+
+  static long getLen(ReadChunkResponseProto response) {
+    if (response.hasData()) {
+      return response.getData().size();
+    } else if (response.hasDataBuffers()) {
+      return response.getDataBuffers() .getBuffersList().stream()
+          .mapToLong(ByteString::size).sum();
+    } else {
+      return -1;
     }
-    ContainerCommandRequestProto request = builder.build();
-    xceiverClient.sendCommand(request, getValidatorList());
   }
 
   /**
@@ -368,7 +431,6 @@ public final class ContainerProtocolCalls  {
    * @param data - Data to be written into the container.
    * @param token a token for this block (may be null)
    * @return container protocol writeSmallFile response
-   * @throws IOException
    */
   public static PutSmallFileResponseProto writeSmallFile(
       XceiverClientSpi client, BlockID blockID, byte[] data,
@@ -426,13 +488,12 @@ public final class ContainerProtocolCalls  {
    * @param containerID - ID of container
    * @param encodedToken - encodedToken if security is enabled
    * @param replicaIndex - index position of the container replica
-   * @throws IOException
    */
   @InterfaceStability.Evolving
   public static void createRecoveringContainer(XceiverClientSpi client,
       long containerID, String encodedToken, int replicaIndex)
       throws IOException {
-    createContainerInternal(client, containerID, encodedToken,
+    createContainer(client, containerID, encodedToken,
         ContainerProtos.ContainerDataProto.State.RECOVERING, replicaIndex);
   }
 
@@ -441,11 +502,10 @@ public final class ContainerProtocolCalls  {
    * @param client  - client
    * @param containerID - ID of container
    * @param encodedToken - encodedToken if security is enabled
-   * @throws IOException
    */
   public static void createContainer(XceiverClientSpi client, long containerID,
       String encodedToken) throws IOException {
-    createContainerInternal(client, containerID, encodedToken, null, 0);
+    createContainer(client, containerID, encodedToken, null, 0);
   }
   /**
    * createContainer call that creates a container on the datanode.
@@ -454,9 +514,8 @@ public final class ContainerProtocolCalls  {
    * @param encodedToken - encodedToken if security is enabled
    * @param state - state of the container
    * @param replicaIndex - index position of the container replica
-   * @throws IOException
    */
-  private static void createContainerInternal(XceiverClientSpi client,
+  public static void createContainer(XceiverClientSpi client,
       long containerID, String encodedToken,
       ContainerProtos.ContainerDataProto.State state, int replicaIndex)
       throws IOException {
@@ -487,10 +546,8 @@ public final class ContainerProtocolCalls  {
   /**
    * Deletes a container from a pipeline.
    *
-   * @param client
    * @param force whether or not to forcibly delete the container.
    * @param encodedToken - encodedToken if security is enabled
-   * @throws IOException
    */
   public static void deleteContainer(XceiverClientSpi client, long containerID,
       boolean force, String encodedToken) throws IOException {
@@ -514,10 +571,7 @@ public final class ContainerProtocolCalls  {
   /**
    * Close a container.
    *
-   * @param client
-   * @param containerID
    * @param encodedToken - encodedToken if security is enabled
-   * @throws IOException
    */
   public static void closeContainer(XceiverClientSpi client,
       long containerID, String encodedToken)
@@ -541,7 +595,6 @@ public final class ContainerProtocolCalls  {
    *
    * @param client       - client
    * @param encodedToken - encodedToken if security is enabled
-   * @throws IOException
    */
   public static ReadContainerResponseProto readContainer(
       XceiverClientSpi client, long containerID, String encodedToken)
@@ -566,11 +619,9 @@ public final class ContainerProtocolCalls  {
   /**
    * Reads the data given the blockID.
    *
-   * @param client
    * @param blockID - ID of the block
    * @param token a token for this block (may be null)
    * @return GetSmallFileResponseProto
-   * @throws IOException
    */
   public static GetSmallFileResponseProto readSmallFile(XceiverClientSpi client,
       BlockID blockID,
@@ -623,13 +674,24 @@ public final class ContainerProtocolCalls  {
         response.getMessage(), response.getResult());
   }
 
-  public static List<CheckedBiFunction> getValidatorList() {
-    List<CheckedBiFunction> validators = new ArrayList<>(1);
-    CheckedBiFunction<ContainerProtos.ContainerCommandRequestProto,
-        ContainerProtos.ContainerCommandResponseProto, IOException>
-        validator = (request, response) -> validateContainerResponse(response);
+  public static List<Validator> getValidatorList() {
+    return VALIDATORS;
+  }
+
+  private static final List<Validator> VALIDATORS = createValidators();
+
+  private static List<Validator> createValidators() {
+    return singletonList(
+        (request, response) -> validateContainerResponse(response));
+  }
+
+  public static List<Validator> toValidatorList(Validator validator) {
+    final List<Validator> defaults = getValidatorList();
+    final List<Validator> validators
+        = new ArrayList<>(defaults.size() + 1);
+    validators.addAll(defaults);
     validators.add(validator);
-    return validators;
+    return Collections.unmodifiableList(validators);
   }
 
   public static HashMap<DatanodeDetails, GetBlockResponseProto>
@@ -662,4 +724,30 @@ public final class ContainerProtocolCalls  {
     }
     return datanodeToResponseMap;
   }
+
+  public static HashMap<DatanodeDetails, ReadContainerResponseProto>
+      readContainerFromAllNodes(XceiverClientSpi client, long containerID,
+      String encodedToken) throws IOException, InterruptedException {
+    String id = client.getPipeline().getFirstNode().getUuidString();
+    HashMap<DatanodeDetails, ReadContainerResponseProto> datanodeToResponseMap
+        = new HashMap<>();
+    ContainerCommandRequestProto.Builder request =
+        ContainerCommandRequestProto.newBuilder();
+    request.setCmdType(Type.ReadContainer);
+    request.setContainerID(containerID);
+    request.setReadContainer(ReadContainerRequestProto.getDefaultInstance());
+    request.setDatanodeUuid(id);
+    if (encodedToken != null) {
+      request.setEncodedToken(encodedToken);
+    }
+    Map<DatanodeDetails, ContainerCommandResponseProto> responses =
+        client.sendCommandOnAllNodes(request.build());
+    for (Map.Entry<DatanodeDetails, ContainerCommandResponseProto> entry :
+        responses.entrySet()) {
+      datanodeToResponseMap.put(entry.getKey(),
+          entry.getValue().getReadContainer());
+    }
+    return datanodeToResponseMap;
+  }
+
 }
