@@ -26,9 +26,9 @@ public interface Cache<K, V> {
 
   V get(K key);
 
-  V put(K key, V value) throws InterruptedException;
+  void put(K key, V value) throws InterruptedException;
 
-  V remove(K key);
+  void remove(K key);
 
   void removeIf(Predicate<K> predicate);
 
