@@ -29,6 +29,7 @@ import java.io.IOException;
 import java.lang.management.ManagementFactory;
 import java.util.Iterator;
 
+import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 import org.mockito.MockitoAnnotations;
@@ -50,6 +51,11 @@ public class TestAbstractLayoutVersionManager {
     MockitoAnnotations.initMocks(this);
   }
 
+  @AfterEach
+  public void close() {
+    versionManager.close();
+  }
+
   @Test
   public void testInitializationWithFeaturesToBeFinalized() throws  Exception {
     versionManager.init(1, getTestLayoutFeatures(3));
@@ -122,13 +128,19 @@ public class TestAbstractLayoutVersionManager {
   }
 
   @Test
-  public void testFeatureFinalizationFailsIfFeatureIsAlreadyFinalized()
+  public void testFeatureFinalizationIfFeatureIsAlreadyFinalized()
       throws IOException {
+    /*
+     * Feature finalization call is idempotent, it should not have any
+     * side effects even if it's executed again.
+     */
     LayoutFeature[] lfs = getTestLayoutFeatures(3);
-    versionManager.init(1, lfs);
-
-    assertThrows(IllegalArgumentException.class,
-        () -> versionManager.finalized(lfs[0]));
+    versionManager.init(2, lfs);
+    assertEquals(2, versionManager.getMetadataLayoutVersion());
+    versionManager.finalized(lfs[0]);
+    assertEquals(2, versionManager.getMetadataLayoutVersion());
+    versionManager.finalized(lfs[1]);
+    assertEquals(2, versionManager.getMetadataLayoutVersion());
   }
 
   @Test
