@@ -17,6 +17,9 @@
  */
 package org.apache.hadoop.hdds.conf;
 
+import java.time.Duration;
+import java.time.temporal.ChronoUnit;
+import java.time.temporal.TemporalUnit;
 import java.util.concurrent.TimeUnit;
 
 import org.slf4j.Logger;
@@ -44,6 +47,11 @@ public final class TimeDurationUtil {
    */
   public static long getTimeDurationHelper(String name, String vStr,
       TimeUnit unit) {
+    final long millis = getDuration(name, vStr, unit).toMillis();
+    return unit.convert(millis, TimeUnit.MILLISECONDS);
+  }
+
+  public static Duration getDuration(String name, String vStr, TimeUnit unit) {
     vStr = vStr.trim();
     vStr = vStr.toLowerCase();
     ParsedTimeDuration vUnit = ParsedTimeDuration.unitFor(vStr);
@@ -58,12 +66,7 @@ public final class TimeDurationUtil {
     }
 
     long raw = Long.parseLong(vStr);
-    long converted = unit.convert(raw, vUnit.unit());
-    if (vUnit.unit().convert(converted, unit) < raw) {
-      LOG.warn("Possible loss of precision converting " + vStr
-          + vUnit.suffix() + " to " + unit + " for " + name);
-    }
-    return converted;
+    return Duration.of(raw, vUnit.temporalUnit());
   }
 
   enum ParsedTimeDuration {
@@ -77,6 +80,11 @@ public final class TimeDurationUtil {
       String suffix() {
         return "ns";
       }
+
+      @Override
+      TemporalUnit temporalUnit() {
+        return ChronoUnit.NANOS;
+      }
     },
     US {
       @Override
@@ -88,6 +96,11 @@ public final class TimeDurationUtil {
       String suffix() {
         return "us";
       }
+
+      @Override
+      TemporalUnit temporalUnit() {
+        return ChronoUnit.MICROS;
+      }
     },
     MS {
       @Override
@@ -99,6 +112,11 @@ public final class TimeDurationUtil {
       String suffix() {
         return "ms";
       }
+
+      @Override
+      TemporalUnit temporalUnit() {
+        return ChronoUnit.MILLIS;
+      }
     },
     S {
       @Override
@@ -110,6 +128,11 @@ public final class TimeDurationUtil {
       String suffix() {
         return "s";
       }
+
+      @Override
+      TemporalUnit temporalUnit() {
+        return ChronoUnit.SECONDS;
+      }
     },
     M {
       @Override
@@ -121,6 +144,11 @@ public final class TimeDurationUtil {
       String suffix() {
         return "m";
       }
+
+      @Override
+      TemporalUnit temporalUnit() {
+        return ChronoUnit.MINUTES;
+      }
     },
     H {
       @Override
@@ -132,6 +160,11 @@ public final class TimeDurationUtil {
       String suffix() {
         return "h";
       }
+
+      @Override
+      TemporalUnit temporalUnit() {
+        return ChronoUnit.HOURS;
+      }
     },
     D {
       @Override
@@ -143,12 +176,19 @@ public final class TimeDurationUtil {
       String suffix() {
         return "d";
       }
+
+      @Override
+      TemporalUnit temporalUnit() {
+        return ChronoUnit.DAYS;
+      }
     };
 
     abstract TimeUnit unit();
 
     abstract String suffix();
 
+    abstract TemporalUnit temporalUnit();
+
     static ParsedTimeDuration unitFor(String s) {
       for (ParsedTimeDuration ptd : values()) {
         // iteration order is in decl order, so SECONDS matched last
@@ -167,5 +207,14 @@ public final class TimeDurationUtil {
       }
       return null;
     }
+
+    static ParsedTimeDuration unitFor(TemporalUnit unit) {
+      for (ParsedTimeDuration ptd : values()) {
+        if (ptd.temporalUnit() == unit) {
+          return ptd;
+        }
+      }
+      return null;
+    }
   }
 }
