@@ -24,7 +24,7 @@ import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
-import java.util.Optional;
+import java.util.OptionalLong;
 import java.util.Queue;
 import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
@@ -40,6 +40,7 @@ import java.util.concurrent.atomic.AtomicReference;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.function.Consumer;
+import java.util.stream.Collectors;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.protobuf.Descriptors.Descriptor;
@@ -141,7 +142,7 @@ public class StateContext {
    *
    * For non-HA mode, term of SCMCommand will be 0.
    */
-  private Optional<Long> termOfLeaderSCM = Optional.empty();
+  private OptionalLong termOfLeaderSCM = OptionalLong.empty();
 
   /**
    * Starting with a 2 sec heartbeat frequency which will be updated to the
@@ -151,6 +152,11 @@ public class StateContext {
   private final AtomicLong heartbeatFrequency = new AtomicLong(2000);
 
   private final AtomicLong reconHeartbeatFrequency = new AtomicLong(2000);
+
+  private final int maxCommandQueueLimit;
+
+  private final String threadNamePrefix;
+
   /**
    * Constructs a StateContext.
    *
@@ -160,8 +166,11 @@ public class StateContext {
    */
   public StateContext(ConfigurationSource conf,
       DatanodeStateMachine.DatanodeStates
-          state, DatanodeStateMachine parent) {
+          state, DatanodeStateMachine parent, String threadNamePrefix) {
     this.conf = conf;
+    DatanodeConfiguration dnConf =
+        conf.getObject(DatanodeConfiguration.class);
+    maxCommandQueueLimit = dnConf.getCommandQueueLimit();
     this.state = state;
     this.parentDatanodeStateMachine = parent;
     commandQueue = new LinkedList<>();
@@ -181,6 +190,7 @@ public class StateContext {
     isFullReportReadyToBeSent = new HashMap<>();
     fullReportTypeList = new ArrayList<>();
     type2Reports = new HashMap<>();
+    this.threadNamePrefix = threadNamePrefix;
     initReportTypeCollection();
   }
 
@@ -505,8 +515,6 @@ public class StateContext {
         int size = actions.size();
         int limit = size > maxLimit ? maxLimit : size;
         for (int count = 0; count < limit; count++) {
-          // we need to remove the action from the containerAction queue
-          // as well
           ContainerAction action = actions.poll();
           Preconditions.checkNotNull(action);
           containerActionList.add(action);
@@ -570,6 +578,7 @@ public class StateContext {
       InetSocketAddress endpoint,
       int maxLimit) {
     List<PipelineAction> pipelineActionList = new ArrayList<>();
+    List<PipelineAction> persistPipelineAction = new ArrayList<>();
     synchronized (pipelineActions) {
       if (!pipelineActions.isEmpty() &&
           CollectionUtils.isNotEmpty(pipelineActions.get(endpoint))) {
@@ -578,8 +587,21 @@ public class StateContext {
         int size = actionsForEndpoint.size();
         int limit = size > maxLimit ? maxLimit : size;
         for (int count = 0; count < limit; count++) {
-          pipelineActionList.add(actionsForEndpoint.poll());
+          // Add closePipeline back to the pipelineAction queue until
+          // pipeline is closed and removed from the DN.
+          PipelineAction action = actionsForEndpoint.poll();
+          if (action.hasClosePipeline()) {
+            if (parentDatanodeStateMachine.getContainer().getPipelineReport()
+                .getPipelineReportList().stream().noneMatch(
+                    report -> action.getClosePipeline().getPipelineID()
+                        .equals(report.getPipelineID()))) {
+              continue;
+            }
+            persistPipelineAction.add(action);
+          }
+          pipelineActionList.add(action);
         }
+        actionsForEndpoint.addAll(persistPipelineAction);
       }
       return pipelineActionList;
     }
@@ -719,10 +741,9 @@ public class StateContext {
 
     // if commandQueue is not empty, init termOfLeaderSCM
     // with the largest term found in commandQueue
-    commandQueue.stream()
+    termOfLeaderSCM = commandQueue.stream()
         .mapToLong(SCMCommand::getTerm)
-        .max()
-        .ifPresent(term -> termOfLeaderSCM = Optional.of(term));
+        .max();
   }
 
   /**
@@ -730,12 +751,27 @@ public class StateContext {
    * Always record the latest term that has seen.
    */
   private void updateTermOfLeaderSCM(SCMCommand<?> command) {
+    updateTermOfLeaderSCM(command.getTerm());
+  }
+
+  public void updateTermOfLeaderSCM(final long newTerm) {
     if (!termOfLeaderSCM.isPresent()) {
-      LOG.error("should init termOfLeaderSCM before update it.");
       return;
     }
-    termOfLeaderSCM = Optional.of(
-        Long.max(termOfLeaderSCM.get(), command.getTerm()));
+
+    final long currentTerm = termOfLeaderSCM.getAsLong();
+    if (currentTerm < newTerm) {
+      setTermOfLeaderSCM(newTerm);
+    }
+  }
+
+  @VisibleForTesting
+  public void setTermOfLeaderSCM(long term) {
+    termOfLeaderSCM = OptionalLong.of(term);
+  }
+
+  public OptionalLong getTermOfLeaderSCM() {
+    return termOfLeaderSCM;
   }
 
   /**
@@ -758,13 +794,14 @@ public class StateContext {
         }
 
         updateTermOfLeaderSCM(command);
-        if (command.getTerm() == termOfLeaderSCM.get()) {
+        final long currentTerm = termOfLeaderSCM.getAsLong();
+        if (command.getTerm() == currentTerm) {
           return command;
         }
 
         LOG.warn("Detect and drop a SCMCommand {} from stale leader SCM," +
             " stale term {}, latest term {}.",
-            command, command.getTerm(), termOfLeaderSCM.get());
+            command, command.getTerm(), currentTerm);
       }
     } finally {
       lock.unlock();
@@ -779,6 +816,12 @@ public class StateContext {
   public void addCommand(SCMCommand command) {
     lock.lock();
     try {
+      if (commandQueue.size() >= maxCommandQueueLimit) {
+        LOG.warn("Ignore command as command queue crosses max limit {}.",
+            maxCommandQueueLimit);
+        return;
+      }
+      updateTermOfLeaderSCM(command);
       commandQueue.add(command);
     } finally {
       lock.unlock();
@@ -886,6 +929,9 @@ public class StateContext {
         mp.putIfAbsent(e, new AtomicBoolean(true));
       });
       this.isFullReportReadyToBeSent.putIfAbsent(endpoint, mp);
+      if (getQueueMetrics() != null) {
+        getQueueMetrics().addEndpoint(endpoint);
+      }
     }
   }
 
@@ -919,4 +965,27 @@ public class StateContext {
   public long getReconHeartbeatFrequency() {
     return reconHeartbeatFrequency.get();
   }
+
+  public Map<InetSocketAddress, Integer> getPipelineActionQueueSize() {
+    return pipelineActions.entrySet().stream()
+        .collect(Collectors.toMap(Map.Entry::getKey, e -> e.getValue().size()));
+  }
+
+  public Map<InetSocketAddress, Integer> getContainerActionQueueSize() {
+    return containerActions.entrySet().stream()
+        .collect(Collectors.toMap(Map.Entry::getKey, e -> e.getValue().size()));
+  }
+
+  public Map<InetSocketAddress, Integer> getIncrementalReportQueueSize() {
+    return incrementalReportsQueue.entrySet().stream()
+        .collect(Collectors.toMap(Map.Entry::getKey, e -> e.getValue().size()));
+  }
+
+  public DatanodeQueueMetrics getQueueMetrics() {
+    return parentDatanodeStateMachine.getQueueMetrics();
+  }
+
+  public String getThreadNamePrefix() {
+    return threadNamePrefix;
+  }
 }
