@@ -18,6 +18,7 @@
 
 package org.apache.hadoop.ozone.container.common.volume;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import org.apache.hadoop.fs.StorageType;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
@@ -27,8 +28,9 @@ import org.apache.hadoop.hdfs.server.datanode.checker.Checkable;
 import org.apache.hadoop.hdfs.server.datanode.checker.VolumeCheckResult;
 import org.apache.hadoop.ozone.common.InconsistentStorageStateException;
 import org.apache.hadoop.ozone.container.common.helpers.DatanodeVersionFile;
+import org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration;
+import org.apache.hadoop.ozone.container.common.utils.DiskCheckUtil;
 import org.apache.hadoop.ozone.container.common.utils.StorageVolumeUtil;
-import org.apache.hadoop.util.DiskChecker;
 import org.apache.hadoop.util.Time;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -36,11 +38,20 @@ import org.slf4j.LoggerFactory;
 import javax.annotation.Nullable;
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.util.LinkedList;
 import java.util.Objects;
+import java.util.Optional;
 import java.util.Properties;
+import java.util.Queue;
 import java.util.UUID;
+import java.util.concurrent.atomic.AtomicInteger;
+import java.util.stream.Stream;
 
 import static org.apache.hadoop.ozone.container.common.HDDSVolumeLayoutVersion.getLatestVersion;
+import static org.apache.hadoop.ozone.container.common.volume.VolumeUsage.PrecomputedVolumeSpace;
+
 
 /**
  * StorageVolume represents a generic Volume in datanode, could be
@@ -62,6 +73,12 @@ public abstract class StorageVolume
   private static final Logger LOG =
       LoggerFactory.getLogger(StorageVolume.class);
 
+  // The name of the directory used for temporary files on the volume.
+  public static final String TMP_DIR_NAME = "tmp";
+  // The name of the directory where temporary files used to check disk
+  // health are written to. This will go inside the tmp directory.
+  public static final String TMP_DISK_CHECK_DIR_NAME = "disk-check";
+
   /**
    * Type for StorageVolume.
    */
@@ -92,7 +109,7 @@ public abstract class StorageVolume
     NOT_INITIALIZED
   }
 
-  private VolumeState state;
+  private volatile VolumeState state;
 
   // VERSION file properties
   private String storageID;       // id of the file system
@@ -104,32 +121,55 @@ public abstract class StorageVolume
   private ConfigurationSource conf;
 
   private final File storageDir;
+  private String workingDirName;
+  private File tmpDir;
+  private File diskCheckDir;
 
-  private final VolumeInfo volumeInfo;
+  private final Optional<VolumeInfo> volumeInfo;
 
   private final VolumeSet volumeSet;
 
-  private String workingDir;
+  /*
+  Fields used to implement IO based disk health checks.
+  If more than ioFailureTolerance IO checks fail out of the last ioTestCount
+  tests run, then the volume is considered failed.
+   */
+  private final int ioTestCount;
+  private final int ioFailureTolerance;
+  private AtomicInteger currentIOFailureCount;
+  private Queue<Boolean> ioTestSlidingWindow;
+  private int healthCheckFileSize;
 
   protected StorageVolume(Builder<?> b) throws IOException {
     if (!b.failedVolume) {
       StorageLocation location = StorageLocation.parse(b.volumeRootStr);
       storageDir = new File(location.getUri().getPath(), b.storageDirStr);
-      this.volumeInfo = new VolumeInfo.Builder(b.volumeRootStr, b.conf)
+      this.volumeInfo = Optional.of(
+              new VolumeInfo.Builder(b.volumeRootStr, b.conf)
           .storageType(b.storageType)
           .usageCheckFactory(b.usageCheckFactory)
-          .build();
+          .build());
       this.volumeSet = b.volumeSet;
       this.state = VolumeState.NOT_INITIALIZED;
       this.clusterID = b.clusterID;
       this.datanodeUuid = b.datanodeUuid;
       this.conf = b.conf;
+
+      DatanodeConfiguration dnConf =
+          conf.getObject(DatanodeConfiguration.class);
+      this.ioTestCount = dnConf.getVolumeIOTestCount();
+      this.ioFailureTolerance = dnConf.getVolumeIOFailureTolerance();
+      this.ioTestSlidingWindow = new LinkedList<>();
+      this.currentIOFailureCount = new AtomicInteger(0);
+      this.healthCheckFileSize = dnConf.getVolumeHealthCheckFileSize();
     } else {
       storageDir = new File(b.volumeRootStr);
-      this.volumeInfo = null;
+      this.volumeInfo = Optional.empty();
       this.volumeSet = null;
       this.storageID = UUID.randomUUID().toString();
       this.state = VolumeState.FAILED;
+      this.ioTestCount = 0;
+      this.ioFailureTolerance = 0;
     }
   }
 
@@ -146,7 +186,16 @@ public abstract class StorageVolume
    * otherwise returns with IOException.
    * @throws IOException
    */
-  protected void initialize() throws IOException {
+  protected final void initialize() throws IOException {
+    try {
+      initializeImpl();
+    } catch (Exception e) {
+      shutdown();
+      throw e;
+    }
+  }
+
+  protected void initializeImpl() throws IOException {
     VolumeState intialVolumeState = analyzeVolumeState();
     switch (intialVolumeState) {
     case NON_EXISTENT:
@@ -178,19 +227,45 @@ public abstract class StorageVolume
   }
 
   /**
-   * Create working directory for cluster io loads.
-   * @param workingDirName scmID or clusterID according to SCM HA config
-   * @param dbVolumeSet optional dbVolumes
+   * Create the working directory for the volume at
+   * <volume>/<hdds>/<workingDirName>.
+   * Creates necessary subdirectories of the working directory as well. This
+   * includes the tmp directory at <volume>/<hdds>/<workingDirName>/tmp.
+   * Child classes may override this method to add volume specific
+   * subdirectories, but they should call the parent method first to make
+   * sure initial directories are constructed.
+   *
+   * @param dirName scmID or clusterID according to SCM HA
+   *    layout feature upgrade finalization status.
    * @throws IOException
    */
-  public void createWorkingDir(String workingDirName,
-      MutableVolumeSet dbVolumeSet) throws IOException {
-    File idDir = new File(getStorageDir(), workingDirName);
-    if (!idDir.mkdir()) {
+  public void createWorkingDir(String dirName, MutableVolumeSet dbVolumeSet)
+      throws IOException {
+    File idDir = new File(getStorageDir(), dirName);
+    if (!idDir.exists() && !idDir.mkdir()) {
       throw new IOException("Unable to create ID directory " + idDir +
           " for datanode.");
     }
-    this.workingDir = workingDirName;
+    this.workingDirName = dirName;
+  }
+
+  public void createTmpDirs(String workDirName) throws IOException {
+    this.tmpDir =
+        new File(new File(getStorageDir(), workDirName), TMP_DIR_NAME);
+    Files.createDirectories(tmpDir.toPath());
+    diskCheckDir = createTmpSubdirIfNeeded(TMP_DISK_CHECK_DIR_NAME);
+    cleanTmpDiskCheckDir();
+  }
+
+  /**
+   * Create a subdirectory within this volume's tmp directory.
+   * This subdirectory can be used as a work space for temporary filesystem
+   * operations before they are moved to their final destination.
+   */
+  protected final File createTmpSubdirIfNeeded(String name) throws IOException {
+    File newDir = new File(tmpDir, name);
+    Files.createDirectories(newDir.toPath());
+    return newDir;
   }
 
   private VolumeState analyzeVolumeState() {
@@ -368,44 +443,67 @@ public abstract class StorageVolume
     }
   }
 
+  public String getVolumeRootDir() {
+    return volumeInfo.map(VolumeInfo::getRootDir).orElse(null);
+  }
+
   public long getCapacity() {
-    return volumeInfo != null ? volumeInfo.getCapacity() : 0;
+    return volumeInfo.map(VolumeInfo::getCapacity).orElse(0L);
   }
 
   public long getAvailable() {
-    return volumeInfo != null ? volumeInfo.getAvailable() : 0;
+    return volumeInfo.map(VolumeInfo::getAvailable).orElse(0L);
+
+  }
+
+  public long getAvailable(PrecomputedVolumeSpace precomputedVolumeSpace) {
+    return volumeInfo.map(info -> info.getAvailable(precomputedVolumeSpace))
+        .orElse(0L);
+  }
+
+  public PrecomputedVolumeSpace getPrecomputedVolumeSpace() {
+    return volumeInfo.map(VolumeInfo::getPrecomputedVolumeSpace)
+        .orElse(new PrecomputedVolumeSpace(0L, 0L));
   }
 
   public long getUsedSpace() {
-    return volumeInfo != null ? volumeInfo.getScmUsed() : 0;
+    return volumeInfo.map(VolumeInfo::getScmUsed).orElse(0L);
+
   }
 
   public File getStorageDir() {
     return this.storageDir;
   }
 
-  public String getWorkingDir() {
-    return this.workingDir;
+  public String getWorkingDirName() {
+    return this.workingDirName;
+  }
+
+  public File getTmpDir() {
+    return this.tmpDir;
+  }
+
+  @VisibleForTesting
+  public File getDiskCheckDir() {
+    return this.diskCheckDir;
   }
 
   public void refreshVolumeInfo() {
-    volumeInfo.refreshNow();
+    volumeInfo.ifPresent(VolumeInfo::refreshNow);
   }
 
-  public VolumeInfo getVolumeInfo() {
+  public Optional<VolumeInfo> getVolumeInfo() {
     return this.volumeInfo;
   }
 
   public void incrementUsedSpace(long usedSpace) {
-    if (this.volumeInfo != null) {
-      this.volumeInfo.incrementUsedSpace(usedSpace);
-    }
+    volumeInfo.ifPresent(volInfo -> volInfo
+            .incrementUsedSpace(usedSpace));
   }
 
   public void decrementUsedSpace(long reclaimedSpace) {
-    if (this.volumeInfo != null) {
-      this.volumeInfo.decrementUsedSpace(reclaimedSpace);
-    }
+    volumeInfo.ifPresent(volInfo -> volInfo
+            .decrementUsedSpace(reclaimedSpace));
   }
 
   public VolumeSet getVolumeSet() {
@@ -413,10 +511,8 @@ public abstract class StorageVolume
   }
 
   public StorageType getStorageType() {
-    if (this.volumeInfo != null) {
-      return this.volumeInfo.getStorageType();
-    }
-    return StorageType.DEFAULT;
+    return volumeInfo.map(VolumeInfo::getStorageType)
+            .orElse(StorageType.DEFAULT);
   }
 
   public String getStorageID() {
@@ -457,31 +553,129 @@ public abstract class StorageVolume
 
   public void failVolume() {
     setState(VolumeState.FAILED);
-    if (volumeInfo != null) {
-      volumeInfo.shutdownUsageThread();
-    }
+    volumeInfo.ifPresent(VolumeInfo::shutdownUsageThread);
   }
 
   public void shutdown() {
     setState(VolumeState.NON_EXISTENT);
-    if (volumeInfo != null) {
-      volumeInfo.shutdownUsageThread();
+    volumeInfo.ifPresent(VolumeInfo::shutdownUsageThread);
+    cleanTmpDiskCheckDir();
+  }
+
+  /**
+   * Delete all temporary files in the directory used ot check disk health.
+   */
+  private void cleanTmpDiskCheckDir() {
+    // If the volume was shut down before initialization completed, skip
+    // emptying the directory.
+    if (diskCheckDir == null) {
+      return;
+    }
+
+    if (!diskCheckDir.exists()) {
+      LOG.warn("Unable to clear disk check files from {}. Directory does " +
+          "not exist.", diskCheckDir);
+      return;
+    }
+
+    if (!diskCheckDir.isDirectory()) {
+      LOG.warn("Unable to clear disk check files from {}. Location is not a" +
+          " directory", diskCheckDir);
+      return;
+    }
+
+    try (Stream<Path> files = Files.list(diskCheckDir.toPath())) {
+      files.map(Path::toFile).filter(File::isFile).forEach(file -> {
+        try {
+          Files.delete(file.toPath());
+        } catch (IOException ex) {
+          LOG.warn("Failed to delete temporary volume health check file {}",
+              file);
+        }
+      });
+    } catch (IOException ex) {
+      LOG.warn("Failed to list contents of volume health check directory {} " +
+          "for deleting.", diskCheckDir);
     }
   }
 
   /**
-   * Run a check on the current volume to determine if it is healthy.
+   * Run a check on the current volume to determine if it is healthy. The
+   * check consists of a directory check and an IO check.
+   *
+   * If the directory check fails, the volume check fails immediately.
+   * The IO check is allows to fail up to {@code ioFailureTolerance} times
+   * out of the last {@code ioTestCount} IO checks before this volume check is
+   * failed. Each call to this method runs one IO check.
+   *
    * @param unused context for the check, ignored.
    * @return result of checking the volume.
+   * @throws InterruptedException if there was an error during the volume
+   *    check because the thread was interrupted.
    * @throws Exception if an exception was encountered while running
-   *            the volume check.
+   *    the volume check and the thread was not interrupted.
    */
   @Override
-  public VolumeCheckResult check(@Nullable Boolean unused) throws Exception {
-    if (!storageDir.exists()) {
+  public synchronized VolumeCheckResult check(@Nullable Boolean unused)
+      throws Exception {
+    boolean directoryChecksPassed =
+        DiskCheckUtil.checkExistence(storageDir) &&
+        DiskCheckUtil.checkPermissions(storageDir);
+    // If the directory is not present or has incorrect permissions, fail the
+    // volume immediately. This is not an intermittent error.
+    if (!directoryChecksPassed) {
+      if (Thread.currentThread().isInterrupted()) {
+        throw new InterruptedException("Directory check of volume " + this +
+            " interrupted.");
+      }
       return VolumeCheckResult.FAILED;
     }
-    DiskChecker.checkDir(storageDir);
+
+    // If IO test count is set to 0, IO tests for disk health are disabled.
+    if (ioTestCount == 0) {
+      return VolumeCheckResult.HEALTHY;
+    }
+
+    // Since IO errors may be intermittent, volume remains healthy until the
+    // threshold of failures is crossed.
+    boolean diskChecksPassed = DiskCheckUtil.checkReadWrite(storageDir,
+        diskCheckDir, healthCheckFileSize);
+    if (Thread.currentThread().isInterrupted()) {
+      // Thread interrupt may have caused IO operations to abort. Do not
+      // consider this a failure.
+      throw new InterruptedException("IO check of volume " + this +
+          " interrupted.");
+    }
+
+    // Move the sliding window of IO test results forward 1 by adding the
+    // latest entry and removing the oldest entry from the window.
+    // Update the failure counter for the new window.
+    ioTestSlidingWindow.add(diskChecksPassed);
+    if (!diskChecksPassed) {
+      currentIOFailureCount.incrementAndGet();
+    }
+    if (ioTestSlidingWindow.size() > ioTestCount &&
+        Objects.equals(ioTestSlidingWindow.poll(), Boolean.FALSE)) {
+      currentIOFailureCount.decrementAndGet();
+    }
+
+    // If the failure threshold has been crossed, fail the volume without
+    // further scans.
+    // Once the volume is failed, it will not be checked anymore.
+    // The failure counts can be left as is.
+    if (currentIOFailureCount.get() > ioFailureTolerance) {
+      LOG.info("Failed IO test for volume {}: the last {} runs " +
+              "encountered {} out of {} tolerated failures.", this,
+          ioTestSlidingWindow.size(), currentIOFailureCount,
+          ioFailureTolerance);
+      return VolumeCheckResult.FAILED;
+    } else if (LOG.isDebugEnabled()) {
+      LOG.debug("IO test results for volume {}: the last {} runs encountered " +
+              "{} out of {} tolerated failures", this,
+          ioTestSlidingWindow.size(),
+          currentIOFailureCount, ioFailureTolerance);
+    }
+
     return VolumeCheckResult.HEALTHY;
   }
 
