@@ -20,7 +20,7 @@ package org.apache.hadoop.hdds.datanode.metadata;
 import org.apache.hadoop.fs.FileUtil;
 import org.apache.hadoop.hdds.HddsConfigKeys;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
-import org.apache.hadoop.hdds.security.x509.SecurityConfig;
+import org.apache.hadoop.hdds.security.SecurityConfig;
 import org.apache.hadoop.hdds.security.x509.certificate.authority.CRLApprover;
 import org.apache.hadoop.hdds.security.x509.certificate.authority.DefaultCRLApprover;
 import org.apache.hadoop.hdds.security.x509.certificate.utils.CertificateCodec;
