@@ -17,7 +17,16 @@
  */
 package org.apache.hadoop.ozone.container.common.statemachine;
 
+import org.apache.hadoop.hdds.conf.DatanodeRatisServerConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.scm.pipeline.MockPipeline;
+import org.apache.hadoop.ozone.OzoneConfigKeys;
+import org.apache.hadoop.ozone.container.common.ContainerTestUtils;
+import org.apache.ratis.conf.RaftProperties;
+import org.apache.ratis.server.RaftServerConfigKeys;
+import org.apache.ratis.util.TimeDuration;
+import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Test;
 
 import java.util.concurrent.TimeUnit;
@@ -34,6 +43,8 @@ import static org.apache.hadoop.ozone.container.common.statemachine.DatanodeConf
 import static org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration.FAILED_DATA_VOLUMES_TOLERATED_KEY;
 import static org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration.FAILED_METADATA_VOLUMES_TOLERATED_KEY;
 import static org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration.FAILED_VOLUMES_TOLERATED_DEFAULT;
+import static org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration.BLOCK_DELETE_COMMAND_WORKER_INTERVAL;
+import static org.apache.hadoop.ozone.container.common.statemachine.DatanodeConfiguration.BLOCK_DELETE_COMMAND_WORKER_INTERVAL_DEFAULT;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 
@@ -50,6 +61,7 @@ public class TestDatanodeConfiguration {
     int validFailedVolumesTolerated = 10;
     long validDiskCheckMinGap = 2;
     long validDiskCheckTimeout = 1;
+    long validBlockDeleteCommandWorkerInterval = 1;
     OzoneConfiguration conf = new OzoneConfiguration();
     conf.setInt(CONTAINER_DELETE_THREADS_MAX_KEY, validDeleteThreads);
     conf.setLong(PERIODIC_DISK_CHECK_INTERVAL_MINUTES_KEY,
@@ -64,6 +76,8 @@ public class TestDatanodeConfiguration {
         validDiskCheckMinGap, TimeUnit.MINUTES);
     conf.setTimeDuration(DISK_CHECK_TIMEOUT_KEY,
         validDiskCheckTimeout, TimeUnit.MINUTES);
+    conf.setTimeDuration(BLOCK_DELETE_COMMAND_WORKER_INTERVAL,
+        validBlockDeleteCommandWorkerInterval, TimeUnit.SECONDS);
 
     // WHEN
     DatanodeConfiguration subject = conf.getObject(DatanodeConfiguration.class);
@@ -82,6 +96,8 @@ public class TestDatanodeConfiguration {
         subject.getDiskCheckMinGap().toMinutes());
     assertEquals(validDiskCheckTimeout,
         subject.getDiskCheckTimeout().toMinutes());
+    assertEquals(validBlockDeleteCommandWorkerInterval,
+        subject.getBlockDeleteCommandWorkerInterval().getSeconds());
   }
 
   @Test
@@ -92,6 +108,7 @@ public class TestDatanodeConfiguration {
     int invalidFailedVolumesTolerated = -2;
     long invalidDiskCheckMinGap = -1;
     long invalidDiskCheckTimeout = -1;
+    long invalidBlockDeleteCommandWorkerInterval = -1;
     OzoneConfiguration conf = new OzoneConfiguration();
     conf.setInt(CONTAINER_DELETE_THREADS_MAX_KEY, invalidDeleteThreads);
     conf.setLong(PERIODIC_DISK_CHECK_INTERVAL_MINUTES_KEY,
@@ -106,6 +123,8 @@ public class TestDatanodeConfiguration {
         invalidDiskCheckMinGap, TimeUnit.MINUTES);
     conf.setTimeDuration(DISK_CHECK_TIMEOUT_KEY,
         invalidDiskCheckTimeout, TimeUnit.MINUTES);
+    conf.setTimeDuration(BLOCK_DELETE_COMMAND_WORKER_INTERVAL,
+        invalidBlockDeleteCommandWorkerInterval, TimeUnit.SECONDS);
 
     // WHEN
     DatanodeConfiguration subject = conf.getObject(DatanodeConfiguration.class);
@@ -122,9 +141,11 @@ public class TestDatanodeConfiguration {
     assertEquals(FAILED_VOLUMES_TOLERATED_DEFAULT,
         subject.getFailedDbVolumesTolerated());
     assertEquals(DISK_CHECK_MIN_GAP_DEFAULT,
-        subject.getDiskCheckMinGap().toMillis());
+        subject.getDiskCheckMinGap());
     assertEquals(DISK_CHECK_TIMEOUT_DEFAULT,
-        subject.getDiskCheckTimeout().toMillis());
+        subject.getDiskCheckTimeout());
+    assertEquals(BLOCK_DELETE_COMMAND_WORKER_INTERVAL_DEFAULT,
+        subject.getBlockDeleteCommandWorkerInterval());
   }
 
   @Test
@@ -147,9 +168,37 @@ public class TestDatanodeConfiguration {
     assertEquals(FAILED_VOLUMES_TOLERATED_DEFAULT,
         subject.getFailedDbVolumesTolerated());
     assertEquals(DISK_CHECK_MIN_GAP_DEFAULT,
-        subject.getDiskCheckMinGap().toMillis());
+        subject.getDiskCheckMinGap());
     assertEquals(DISK_CHECK_TIMEOUT_DEFAULT,
-        subject.getDiskCheckTimeout().toMillis());
+        subject.getDiskCheckTimeout());
+    assertEquals(BLOCK_DELETE_COMMAND_WORKER_INTERVAL_DEFAULT,
+        subject.getBlockDeleteCommandWorkerInterval());
   }
 
+  @Test
+  public void testConf() throws Exception {
+    final OzoneConfiguration conf = new OzoneConfiguration();
+    final String dir = "dummy/dir";
+    conf.set(OzoneConfigKeys.DFS_CONTAINER_RATIS_DATANODE_STORAGE_DIR, dir);
+
+    final DatanodeRatisServerConfig ratisConf = conf.getObject(
+        DatanodeRatisServerConfig.class);
+    Assertions.assertEquals(0, ratisConf.getLogAppenderWaitTimeMin(),
+        "getLogAppenderWaitTimeMin");
+
+    assertWaitTimeMin(TimeDuration.ZERO, conf);
+    ratisConf.setLogAppenderWaitTimeMin(1);
+    conf.setFromObject(ratisConf);
+    assertWaitTimeMin(TimeDuration.ONE_MILLISECOND, conf);
+  }
+
+  static void assertWaitTimeMin(TimeDuration expected,
+      OzoneConfiguration conf) throws Exception {
+    final DatanodeDetails dn = MockPipeline.createPipeline(1).getFirstNode();
+    final RaftProperties p = ContainerTestUtils.newXceiverServerRatis(dn, conf)
+        .newRaftProperties();
+    final TimeDuration t = RaftServerConfigKeys.Log.Appender.waitTimeMin(p);
+    Assertions.assertEquals(expected, t,
+        RaftServerConfigKeys.Log.Appender.WAIT_TIME_MIN_KEY);
+  }
 }
