@@ -16,16 +16,16 @@
  */
 package org.apache.hadoop.ozone.container.common.statemachine.commandhandler;
 
+import java.util.concurrent.TimeoutException;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
+import org.apache.hadoop.ozone.container.common.ContainerTestUtils;
 import org.apache.hadoop.ozone.container.common.impl.ContainerLayoutVersion;
 import org.apache.hadoop.ozone.container.common.impl.ContainerSet;
 import org.apache.hadoop.ozone.container.common.interfaces.Container;
 import org.apache.hadoop.ozone.container.common.interfaces.Handler;
-import org.apache.hadoop.ozone.container.common.statemachine
-    .DatanodeStateMachine;
 import org.apache.hadoop.ozone.container.common.statemachine.StateContext;
 import org.apache.hadoop.ozone.container.common.transport.server.XceiverServerSpi;
 import org.apache.hadoop.ozone.container.keyvalue.ContainerLayoutTestInfo;
@@ -70,7 +70,7 @@ public class TestCloseContainerCommandHandler {
   private ContainerController controller;
   private ContainerSet containerSet;
   private CloseContainerCommandHandler subject =
-      new CloseContainerCommandHandler();
+      new CloseContainerCommandHandler(1, 1000, "");
 
   private final ContainerLayoutVersion layout;
 
@@ -85,19 +85,15 @@ public class TestCloseContainerCommandHandler {
 
   @Before
   public void before() throws Exception {
-    context = mock(StateContext.class);
-    DatanodeStateMachine dnStateMachine = mock(DatanodeStateMachine.class);
-    when(dnStateMachine.getDatanodeDetails())
-        .thenReturn(randomDatanodeDetails());
-    when(context.getParent()).thenReturn(dnStateMachine);
-
+    OzoneConfiguration conf = new OzoneConfiguration();
+    context = ContainerTestUtils.getMockContext(randomDatanodeDetails(), conf);
     pipelineID = PipelineID.randomId();
 
     KeyValueContainerData data = new KeyValueContainerData(CONTAINER_ID,
         layout, GB,
         pipelineID.getId().toString(), null);
 
-    container = new KeyValueContainer(data, new OzoneConfiguration());
+    container = new KeyValueContainer(data, conf);
     containerSet = new ContainerSet(1000);
     containerSet.addContainer(container);
 
@@ -120,19 +116,20 @@ public class TestCloseContainerCommandHandler {
   public void closeContainerWithPipeline() throws Exception {
     // close a container that's associated with an existing pipeline
     subject.handle(closeWithKnownPipeline(), ozoneContainer, context, null);
-
+    waitTillFinishExecution(subject);
     verify(containerHandler)
         .markContainerForClose(container);
     verify(writeChannel)
         .submitRequest(any(), eq(pipelineID.getProtobuf()));
     verify(containerHandler, never())
-        .quasiCloseContainer(container);
+        .quasiCloseContainer(eq(container), any());
   }
 
   @Test
-  public void closeContainerWithoutPipeline() throws IOException {
+  public void closeContainerWithoutPipeline() throws Exception {
     // close a container that's NOT associated with an open pipeline
     subject.handle(closeWithUnknownPipeline(), ozoneContainer, context, null);
+    waitTillFinishExecution(subject);
 
     verify(containerHandler)
         .markContainerForClose(container);
@@ -141,13 +138,14 @@ public class TestCloseContainerCommandHandler {
     // Container in CLOSING state is moved to UNHEALTHY if pipeline does not
     // exist. Container should not exist in CLOSING state without a pipeline.
     verify(containerHandler)
-        .quasiCloseContainer(container);
+        .quasiCloseContainer(eq(container), any());
   }
 
   @Test
-  public void closeContainerWithForceFlagSet() throws IOException {
+  public void closeContainerWithForceFlagSet() throws Exception {
     // close a container that's associated with an existing pipeline
     subject.handle(forceCloseWithoutPipeline(), ozoneContainer, context, null);
+    waitTillFinishExecution(subject);
 
     verify(containerHandler)
         .markContainerForClose(container);
@@ -162,6 +160,7 @@ public class TestCloseContainerCommandHandler {
         .setState(ContainerProtos.ContainerDataProto.State.QUASI_CLOSED);
 
     subject.handle(forceCloseWithoutPipeline(), ozoneContainer, context, null);
+    waitTillFinishExecution(subject);
 
     verify(writeChannel, never())
         .submitRequest(any(), any());
@@ -173,6 +172,7 @@ public class TestCloseContainerCommandHandler {
   public void forceCloseOpenContainer() throws Exception {
     // force-close a container that's NOT associated with an open pipeline
     subject.handle(forceCloseWithoutPipeline(), ozoneContainer, context, null);
+    waitTillFinishExecution(subject);
 
     verify(writeChannel, never())
         .submitRequest(any(), any());
@@ -186,13 +186,14 @@ public class TestCloseContainerCommandHandler {
   public void forceCloseOpenContainerWithPipeline() throws Exception {
     // force-close a container that's associated with an existing pipeline
     subject.handle(forceCloseWithPipeline(), ozoneContainer, context, null);
+    waitTillFinishExecution(subject);
 
     verify(containerHandler)
         .markContainerForClose(container);
     verify(writeChannel)
         .submitRequest(any(), any());
     verify(containerHandler, never())
-        .quasiCloseContainer(container);
+        .quasiCloseContainer(eq(container), any());
     verify(containerHandler, never())
         .closeContainer(container);
   }
@@ -205,12 +206,14 @@ public class TestCloseContainerCommandHandler {
     // Since the container is already closed, these commands should do nothing,
     // neither should they fail
     subject.handle(closeWithUnknownPipeline(), ozoneContainer, context, null);
+    waitTillFinishExecution(subject);
     subject.handle(closeWithKnownPipeline(), ozoneContainer, context, null);
+    waitTillFinishExecution(subject);
 
     verify(containerHandler, never())
         .markContainerForClose(container);
     verify(containerHandler, never())
-        .quasiCloseContainer(container);
+        .quasiCloseContainer(eq(container), any());
     verify(containerHandler, never())
         .closeContainer(container);
     verify(writeChannel, never())
@@ -279,4 +282,11 @@ public class TestCloseContainerCommandHandler {
         .addPort(restPort);
     return builder.build();
   }
+
+  private void waitTillFinishExecution(
+      CloseContainerCommandHandler closeHandler)
+      throws InterruptedException, TimeoutException {
+    GenericTestUtils.waitFor(()
+        -> closeHandler.getQueuedCount() <= 0, 10, 3000);
+  }
 }
