@@ -28,6 +28,7 @@ import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.OptionalLong;
 import java.util.UUID;
 
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
@@ -94,7 +95,7 @@ public class TestHeartbeatEndpointTask {
     DatanodeStateMachine datanodeStateMachine =
         Mockito.mock(DatanodeStateMachine.class);
     StateContext context = new StateContext(conf, DatanodeStates.RUNNING,
-        datanodeStateMachine);
+        datanodeStateMachine, "");
 
     // WHEN
     HeartbeatEndpointTask task = getHeartbeatEndpointTask(conf, context, scm);
@@ -107,6 +108,7 @@ public class TestHeartbeatEndpointTask {
 
   @Test
   public void testheartbeatWithoutReports() throws Exception {
+    final long termInSCM = 42;
     StorageContainerDatanodeProtocolClientSideTranslatorPB scm =
         Mockito.mock(
             StorageContainerDatanodeProtocolClientSideTranslatorPB.class);
@@ -118,9 +120,15 @@ public class TestHeartbeatEndpointTask {
                 .setDatanodeUUID(
                     ((SCMHeartbeatRequestProto)invocation.getArgument(0))
                         .getDatanodeDetails().getUuid())
+                .setTerm(termInSCM)
                 .build());
 
-    HeartbeatEndpointTask endpointTask = getHeartbeatEndpointTask(scm);
+    OzoneConfiguration conf = new OzoneConfiguration();
+    StateContext context = new StateContext(conf, DatanodeStates.RUNNING,
+        Mockito.mock(DatanodeStateMachine.class), "");
+    context.setTermOfLeaderSCM(1);
+    HeartbeatEndpointTask endpointTask = getHeartbeatEndpointTask(
+        conf, context, scm);
     endpointTask.call();
     SCMHeartbeatRequestProto heartbeat = argument.getValue();
     Assertions.assertTrue(heartbeat.hasDatanodeDetails());
@@ -128,13 +136,16 @@ public class TestHeartbeatEndpointTask {
     Assertions.assertFalse(heartbeat.hasContainerReport());
     Assertions.assertTrue(heartbeat.getCommandStatusReportsCount() == 0);
     Assertions.assertFalse(heartbeat.hasContainerActions());
+    OptionalLong termInDatanode = context.getTermOfLeaderSCM();
+    Assertions.assertTrue(termInDatanode.isPresent());
+    Assertions.assertEquals(termInSCM, termInDatanode.getAsLong());
   }
 
   @Test
   public void testheartbeatWithNodeReports() throws Exception {
     OzoneConfiguration conf = new OzoneConfiguration();
     StateContext context = new StateContext(conf, DatanodeStates.RUNNING,
-        Mockito.mock(DatanodeStateMachine.class));
+        Mockito.mock(DatanodeStateMachine.class), "");
 
     StorageContainerDatanodeProtocolClientSideTranslatorPB scm =
         Mockito.mock(
@@ -166,7 +177,7 @@ public class TestHeartbeatEndpointTask {
   public void testheartbeatWithContainerReports() throws Exception {
     OzoneConfiguration conf = new OzoneConfiguration();
     StateContext context = new StateContext(conf, DatanodeStates.RUNNING,
-        Mockito.mock(DatanodeStateMachine.class));
+        Mockito.mock(DatanodeStateMachine.class), "");
 
     StorageContainerDatanodeProtocolClientSideTranslatorPB scm =
         Mockito.mock(
@@ -198,7 +209,7 @@ public class TestHeartbeatEndpointTask {
   public void testheartbeatWithCommandStatusReports() throws Exception {
     OzoneConfiguration conf = new OzoneConfiguration();
     StateContext context = new StateContext(conf, DatanodeStates.RUNNING,
-        Mockito.mock(DatanodeStateMachine.class));
+        Mockito.mock(DatanodeStateMachine.class), "");
 
     StorageContainerDatanodeProtocolClientSideTranslatorPB scm =
         Mockito.mock(
@@ -231,7 +242,7 @@ public class TestHeartbeatEndpointTask {
   public void testheartbeatWithContainerActions() throws Exception {
     OzoneConfiguration conf = new OzoneConfiguration();
     StateContext context = new StateContext(conf, DatanodeStates.RUNNING,
-        Mockito.mock(DatanodeStateMachine.class));
+        Mockito.mock(DatanodeStateMachine.class), "");
 
     StorageContainerDatanodeProtocolClientSideTranslatorPB scm =
         Mockito.mock(
@@ -265,7 +276,7 @@ public class TestHeartbeatEndpointTask {
     DatanodeStateMachine datanodeStateMachine =
         Mockito.mock(DatanodeStateMachine.class);
     StateContext context = new StateContext(conf, DatanodeStates.RUNNING,
-        datanodeStateMachine);
+        datanodeStateMachine, "");
 
     // Return a Map of command counts when the heartbeat logic requests it
     final Map<SCMCommandProto.Type, Integer> commands = new HashMap<>();
@@ -314,22 +325,6 @@ public class TestHeartbeatEndpointTask {
     }
   }
 
-  /**
-   * Creates HeartbeatEndpointTask for the given StorageContainerManager proxy.
-   *
-   * @param proxy StorageContainerDatanodeProtocolClientSideTranslatorPB
-   *
-   * @return HeartbeatEndpointTask
-   */
-  private HeartbeatEndpointTask getHeartbeatEndpointTask(
-      StorageContainerDatanodeProtocolClientSideTranslatorPB proxy) {
-    OzoneConfiguration conf = new OzoneConfiguration();
-    StateContext context = new StateContext(conf, DatanodeStates.RUNNING,
-        Mockito.mock(DatanodeStateMachine.class));
-    return getHeartbeatEndpointTask(conf, context, proxy);
-
-  }
-
   /**
    * Creates HeartbeatEndpointTask with the given conf, context and
    * StorageContainerManager client side proxy.
