@@ -20,6 +20,7 @@ package org.apache.hadoop.ozone.container.common.volume;
 
 import java.io.IOException;
 import org.apache.commons.io.FileUtils;
+import org.junit.jupiter.api.Timeout;
 import org.slf4j.LoggerFactory;
 import org.apache.hadoop.fs.FileUtil;
 import org.apache.hadoop.hdds.DFSConfigKeysLegacy;
@@ -34,16 +35,14 @@ import org.apache.ozone.test.GenericTestUtils.LogCapturer;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.HDDS_DATANODE_DIR_KEY;
 import static org.apache.hadoop.ozone.container.common.volume.HddsVolume
     .HDDS_VOLUME_DIR;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertFalse;
-import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.Timeout;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 import java.io.File;
 import java.lang.reflect.Method;
@@ -54,6 +53,7 @@ import java.util.UUID;
 /**
  * Tests {@link MutableVolumeSet} operations.
  */
+@Timeout(300)
 public class TestVolumeSet {
 
   private OzoneConfiguration conf;
@@ -70,10 +70,7 @@ public class TestVolumeSet {
         null, StorageVolume.VolumeType.DATA_VOLUME, null);
   }
 
-  @Rule
-  public Timeout testTimeout = Timeout.seconds(300);
-
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
     conf = new OzoneConfiguration();
     String dataDirKey = volume1 + "," + volume2;
@@ -85,7 +82,7 @@ public class TestVolumeSet {
     initializeVolumeSet();
   }
 
-  @After
+  @AfterEach
   public void shutdown() throws IOException {
     // Delete the volume root dir
     List<StorageVolume> vols = new ArrayList<>();
@@ -116,12 +113,12 @@ public class TestVolumeSet {
     List<StorageVolume> volumesList = volumeSet.getVolumesList();
 
     // VolumeSet initialization should add volume1 and volume2 to VolumeSet
-    assertEquals("VolumeSet intialization is incorrect",
-        volumesList.size(), volumes.size());
-    assertTrue("VolumeSet not initailized correctly",
-        checkVolumeExistsInVolumeSet(volume1));
-    assertTrue("VolumeSet not initailized correctly",
-        checkVolumeExistsInVolumeSet(volume2));
+    assertEquals(volumesList.size(), volumes.size(),
+        "VolumeSet intialization is incorrect");
+    assertTrue(checkVolumeExistsInVolumeSet(volume1),
+        "VolumeSet not initailized correctly");
+    assertTrue(checkVolumeExistsInVolumeSet(volume2),
+        "VolumeSet not initailized correctly");
   }
 
   @Test
@@ -135,8 +132,8 @@ public class TestVolumeSet {
 
     assertTrue(success);
     assertEquals(3, volumeSet.getVolumesList().size());
-    assertTrue("AddVolume did not add requested volume to VolumeSet",
-        checkVolumeExistsInVolumeSet(volume3));
+    assertTrue(checkVolumeExistsInVolumeSet(volume3),
+        "AddVolume did not add requested volume to VolumeSet");
   }
 
   @Test
@@ -149,11 +146,11 @@ public class TestVolumeSet {
     assertEquals(1, volumeSet.getVolumesList().size());
 
     // Failed volume should be added to FailedVolumeList
-    assertEquals("Failed volume not present in FailedVolumeMap",
-        1, volumeSet.getFailedVolumesList().size());
-    assertEquals("Failed Volume list did not match",
-        HddsVolumeUtil.getHddsRoot(volume1),
-        volumeSet.getFailedVolumesList().get(0).getStorageDir().getPath());
+    assertEquals(1, volumeSet.getFailedVolumesList().size(),
+        "Failed volume not present in FailedVolumeMap");
+    assertEquals(HddsVolumeUtil.getHddsRoot(volume1),
+        volumeSet.getFailedVolumesList().get(0).getStorageDir().getPath(),
+        "Failed Volume list did not match");
 
     // Failed volume should not exist in VolumeMap
     assertFalse(volumeSet.getVolumeMap().containsKey(volume1));
@@ -171,13 +168,14 @@ public class TestVolumeSet {
     // Attempting to remove a volume which does not exist in VolumeSet should
     // log a warning.
     LogCapturer logs = LogCapturer.captureLogs(
-            LoggerFactory.getLogger(MutableVolumeSet.class));
+        LoggerFactory.getLogger(MutableVolumeSet.class));
     volumeSet.removeVolume(HddsVolumeUtil.getHddsRoot(volume1));
     assertEquals(1, volumeSet.getVolumesList().size());
     String expectedLogMessage = "Volume : " +
         HddsVolumeUtil.getHddsRoot(volume1) + " does not exist in VolumeSet";
-    assertTrue("Log output does not contain expected log message: "
-        + expectedLogMessage, logs.getOutput().contains(expectedLogMessage));
+    assertTrue(logs.getOutput().contains(expectedLogMessage),
+        "Log output does not contain expected log message: " +
+            expectedLogMessage);
   }
 
   @Test
@@ -191,7 +189,7 @@ public class TestVolumeSet {
     File newVolume = new File(volume3, HDDS_VOLUME_DIR);
     System.out.println("new volume root: " + newVolume);
     newVolume.mkdirs();
-    assertTrue("Failed to create new volume root", newVolume.exists());
+    assertTrue(newVolume.exists(), "Failed to create new volume root");
     File dataDir = new File(newVolume, "chunks");
     dataDir.mkdirs();
     assertTrue(dataDir.exists());
@@ -203,8 +201,8 @@ public class TestVolumeSet {
 
     assertFalse(success);
     assertEquals(2, volumeSet.getVolumesList().size());
-    assertTrue("AddVolume should fail for an inconsistent volume",
-        !checkVolumeExistsInVolumeSet(volume3));
+    assertFalse(checkVolumeExistsInVolumeSet(volume3), "AddVolume should fail" +
+        " for an inconsistent volume");
 
     // Delete volume3
     File volume = new File(volume3);
@@ -219,7 +217,8 @@ public class TestVolumeSet {
 
     // Verify that volume usage can be queried during shutdown.
     for (StorageVolume volume : volumesList) {
-      Assert.assertNotNull(volume.getVolumeInfo().getUsageForTesting());
+      Assertions.assertNotNull(volume.getVolumeInfo().get()
+              .getUsageForTesting());
       volume.getAvailable();
     }
   }
