@@ -19,26 +19,27 @@ package org.apache.hadoop.ozone.container.replication;
 
 import java.time.Instant;
 import java.time.temporal.ChronoUnit;
-import java.util.ArrayList;
 
-import org.apache.hadoop.ozone.container.replication.ReplicationTask.Status;
+import org.apache.hadoop.ozone.container.replication.AbstractReplicationTask.Status;
 
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
 
+import static org.apache.hadoop.ozone.protocol.commands.ReplicateContainerCommand.forTest;
+
 /**
  * Test replicator metric measurement.
  */
 public class TestMeasuredReplicator {
 
   private MeasuredReplicator measuredReplicator;
+  private ContainerReplicator replicator;
 
   @BeforeEach
   public void initReplicator() {
-    measuredReplicator = new MeasuredReplicator(task -> {
-
+    replicator = task -> {
       task.setTransferredBytes(task.getContainerId() * 1024);
 
       //fail if container id is even
@@ -52,7 +53,8 @@ public class TestMeasuredReplicator {
       } catch (InterruptedException e) {
         e.printStackTrace();
       }
-    });
+    };
+    measuredReplicator = new MeasuredReplicator(replicator, "test");
   }
 
   @AfterEach
@@ -63,9 +65,9 @@ public class TestMeasuredReplicator {
   @Test
   public void measureFailureSuccessAndBytes() {
     //WHEN
-    measuredReplicator.replicate(new ReplicationTask(1L, new ArrayList<>()));
-    measuredReplicator.replicate(new ReplicationTask(2L, new ArrayList<>()));
-    measuredReplicator.replicate(new ReplicationTask(3L, new ArrayList<>()));
+    measuredReplicator.replicate(new ReplicationTask(forTest(1), replicator));
+    measuredReplicator.replicate(new ReplicationTask(forTest(2), replicator));
+    measuredReplicator.replicate(new ReplicationTask(forTest(3), replicator));
 
     //THEN
     //even containers should be failed
@@ -83,9 +85,9 @@ public class TestMeasuredReplicator {
   public void testReplicationTime() throws Exception {
     //WHEN
     //will wait at least the 300ms
-    measuredReplicator.replicate(new ReplicationTask(101L, new ArrayList<>()));
-    measuredReplicator.replicate(new ReplicationTask(201L, new ArrayList<>()));
-    measuredReplicator.replicate(new ReplicationTask(300L, new ArrayList<>()));
+    measuredReplicator.replicate(new ReplicationTask(forTest(101), replicator));
+    measuredReplicator.replicate(new ReplicationTask(forTest(201), replicator));
+    measuredReplicator.replicate(new ReplicationTask(forTest(300), replicator));
 
     //THEN
     //even containers should be failed
@@ -101,7 +103,8 @@ public class TestMeasuredReplicator {
   public void testFailureTimeSuccessExcluded() {
     //WHEN
     //will wait at least the 15ms
-    measuredReplicator.replicate(new ReplicationTask(15L, new ArrayList<>()));
+    measuredReplicator.replicate(new ReplicationTask(forTest(15), replicator));
+
 
     //THEN
     //even containers should be failed, supposed to be zero
@@ -112,7 +115,8 @@ public class TestMeasuredReplicator {
   public void testSuccessTimeFailureExcluded() {
     //WHEN
     //will wait at least the 10ms
-    measuredReplicator.replicate(new ReplicationTask(10L, new ArrayList<>()));
+    measuredReplicator.replicate(new ReplicationTask(forTest(10), replicator));
+
 
     //THEN
     //even containers should be failed, supposed to be zero
@@ -122,7 +126,7 @@ public class TestMeasuredReplicator {
   @Test
   public void testReplicationQueueTimeMetrics() {
     final Instant queued = Instant.now().minus(1, ChronoUnit.SECONDS);
-    ReplicationTask task = new ReplicationTask(100L, new ArrayList<>()) {
+    ReplicationTask task = new ReplicationTask(forTest(100), replicator) {
       @Override
       public Instant getQueued() {
         return queued;
@@ -132,4 +136,4 @@ public class TestMeasuredReplicator {
     // There might be some deviation, so we use >= 1000 here.
     Assertions.assertTrue(measuredReplicator.getQueueTime().value() >= 1000);
   }
-}
\ No newline at end of file
+}
