@@ -21,9 +21,8 @@ package org.apache.hadoop.hdds.security.x509.certificate.authority;
 
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.NodeType;
 import org.apache.hadoop.hdds.scm.metadata.SCMMetadataStore;
+import org.apache.hadoop.hdds.security.SecurityConfig;
 import org.apache.hadoop.hdds.security.exception.SCMSecurityException;
-import org.apache.hadoop.hdds.security.x509.SecurityConfig;
-import org.apache.hadoop.hdds.security.x509.certificate.authority.CertificateApprover.ApprovalType;
 import org.apache.hadoop.hdds.security.x509.crl.CRLInfo;
 import org.bouncycastle.asn1.x509.CRLReason;
 import org.bouncycastle.cert.X509CertificateHolder;
@@ -31,6 +30,7 @@ import org.bouncycastle.pkcs.PKCS10CertificationRequest;
 
 import java.io.IOException;
 import java.math.BigInteger;
+import java.security.cert.CertPath;
 import java.security.cert.CertificateException;
 import java.security.cert.X509Certificate;
 import java.util.Date;
@@ -65,6 +65,19 @@ public interface CertificateServer {
   X509CertificateHolder getCACertificate()
       throws CertificateException, IOException;
 
+  /**
+   * Gets the certificate bundle for the CA certificate of this server.
+   * The first element of the list is the CA certificate. The issuer of an
+   * element of this list is always the next element of the list. The root CA
+   * certificate is the final element.
+   *
+   * @return the certificate bundle starting with the CA certificate.
+   * @throws CertificateException
+   * @throws IOException
+   */
+  CertPath getCaCertPath() throws CertificateException,
+      IOException;
+
   /**
    * Returns the Certificate corresponding to given certificate serial id if
    * exist. Return null if it doesn't exist.
@@ -83,28 +96,15 @@ public interface CertificateServer {
    * @param csr  - Certificate Signing Request.
    * @param type - An Enum which says what kind of approval process to follow.
    * @param role : OM/SCM/DN
+   * @param certSerialId - New certificate ID
    * @return A future that will have this certificate when this request is
    * approved.
    * @throws SCMSecurityException - on Error.
    */
-  Future<X509CertificateHolder> requestCertificate(
+  Future<CertPath> requestCertificate(
       PKCS10CertificationRequest csr,
-      CertificateApprover.ApprovalType type, NodeType role)
-      throws SCMSecurityException;
-
-
-  /**
-   * Request a Certificate based on Certificate Signing Request.
-   *
-   * @param csr - Certificate Signing Request as a PEM encoded String.
-   * @param type - An Enum which says what kind of approval process to follow.
-   * @param nodeType: OM/SCM/DN
-   * @return A future that will have this certificate when this request is
-   * approved.
-   * @throws SCMSecurityException - on Error.
-   */
-  Future<X509CertificateHolder> requestCertificate(String csr,
-      ApprovalType type, NodeType nodeType) throws IOException;
+      CertificateApprover.ApprovalType type, NodeType role,
+      String certSerialId) throws SCMSecurityException;
 
   /**
    * Revokes a Certificate issued by this CertificateServer.
@@ -150,12 +150,4 @@ public interface CertificateServer {
    * @return latest CRL id.
    */
   long getLatestCrlId();
-
-  /**
-   * Make it explicit what type of CertificateServer we are creating here.
-   */
-  enum CAType {
-    SELF_SIGNED_CA,
-    INTERMEDIARY_CA
-  }
 }
