@@ -17,6 +17,7 @@
 
 package org.apache.hadoop.hdds.server.http;
 
+import java.util.Map;
 import javax.servlet.http.HttpServlet;
 import java.io.IOException;
 import java.net.InetSocketAddress;
@@ -24,6 +25,7 @@ import java.net.URI;
 import java.util.Optional;
 import java.util.OptionalInt;
 
+import com.google.common.annotations.VisibleForTesting;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.hdds.DFSConfigKeysLegacy;
 import org.apache.hadoop.hdds.HddsConfigKeys;
@@ -43,6 +45,7 @@ import org.apache.commons.lang3.StringUtils;
 import static org.apache.hadoop.hdds.HddsUtils.getHostNameFromConfigKeys;
 import static org.apache.hadoop.hdds.HddsUtils.getPortNumberFromConfigKeys;
 import static org.apache.hadoop.hdds.HddsUtils.createDir;
+import static org.apache.hadoop.hdds.server.ServerUtils.getOzoneMetaDirPath;
 import static org.apache.hadoop.hdds.server.http.HttpConfig.getHttpPolicy;
 import static org.apache.hadoop.ozone.OzoneConfigKeys.OZONE_ADMINISTRATORS;
 import static org.apache.hadoop.ozone.OzoneConfigKeys.OZONE_ADMINISTRATORS_GROUPS;
@@ -70,6 +73,7 @@ public abstract class BaseHttpServer {
   static final String PROMETHEUS_SINK = "PROMETHEUS_SINK";
   private static final String JETTY_BASETMPDIR =
       "org.eclipse.jetty.webapp.basetempdir";
+  public static final String SERVER_DIR = "/webserver";
 
   private HttpServer2 httpServer;
   private final MutableConfigurationSource conf;
@@ -103,8 +107,6 @@ public abstract class BaseHttpServer {
       HttpServer2.Builder builder = newHttpServer2BuilderForOzone(
           conf, httpAddress, httpsAddress, name);
 
-      boolean isSecurityEnabled = UserGroupInformation.isSecurityEnabled() &&
-          OzoneSecurityUtil.isHttpSecurityEnabled(conf);
       LOG.info("Hadoop Security Enabled: {} " +
               "Ozone Security Enabled: {} " +
               "Ozone HTTP Security Enabled: {} ",
@@ -114,7 +116,7 @@ public abstract class BaseHttpServer {
           conf.getBoolean(OZONE_HTTP_SECURITY_ENABLED_KEY,
               OZONE_HTTP_SECURITY_ENABLED_DEFAULT));
 
-      if (isSecurityEnabled) {
+      if (isSecurityEnabled()) {
         String httpAuthType = conf.get(getHttpAuthType(), "simple");
         LOG.info("HttpAuthType: {} = {}", getHttpAuthType(), httpAuthType);
         // Ozone config prefix must be set to avoid AuthenticationFilter
@@ -148,7 +150,7 @@ public abstract class BaseHttpServer {
           conf.getBoolean(HddsConfigKeys.HDDS_PROFILER_ENABLED, false);
 
       if (prometheusSupport) {
-        prometheusMetricsSink = new PrometheusMetricsSink();
+        prometheusMetricsSink = new PrometheusMetricsSink(name);
         httpServer.getWebAppContext().getServletContext()
             .setAttribute(PROMETHEUS_SINK, prometheusMetricsSink);
         HddsPrometheusConfig prometheusConfig =
@@ -178,14 +180,22 @@ public abstract class BaseHttpServer {
       }
 
       String baseDir = conf.get(OzoneConfigKeys.OZONE_HTTP_BASEDIR);
-      if (!StringUtils.isEmpty(baseDir)) {
-        createDir(baseDir);
-        httpServer.getWebAppContext().setAttribute(JETTY_BASETMPDIR, baseDir);
-        LOG.info("HTTP server of {} uses base directory {}", name, baseDir);
+
+      if (StringUtils.isEmpty(baseDir)) {
+        baseDir = getOzoneMetaDirPath(conf) + SERVER_DIR;
       }
+      createDir(baseDir);
+      httpServer.getWebAppContext().setAttribute(JETTY_BASETMPDIR, baseDir);
+      LOG.info("HTTP server of {} uses base directory {}", name, baseDir);
     }
   }
 
+  @VisibleForTesting
+  public String getJettyBaseTmpDir() {
+    return httpServer.getWebAppContext().getAttribute(JETTY_BASETMPDIR)
+        .toString();
+  }
+
   /**
    * Return a HttpServer.Builder that the OzoneManager/SCM/Datanode/S3Gateway/
    * Recon to initialize their HTTP / HTTPS server.
@@ -246,6 +256,17 @@ public abstract class BaseHttpServer {
     httpServer.addInternalServlet(servletName, pathSpec, clazz);
   }
 
+  /**
+   * Add a filter to BaseHttpServer.
+   *
+   * @param filterName The name of the filter
+   * @param classname  The filter class
+   * @param parameters The filter parameters
+   */
+  protected void addFilter(String filterName, String classname,
+      Map<String, String> parameters) {
+    httpServer.addFilter(filterName, classname, parameters);
+  }
 
   /**
    * Returns the WebAppContext associated with this HttpServer.
@@ -425,6 +446,11 @@ public abstract class BaseHttpServer {
     return httpsAddress;
   }
 
+  public boolean isSecurityEnabled() {
+    return UserGroupInformation.isSecurityEnabled() &&
+        OzoneSecurityUtil.isHttpSecurityEnabled(conf);
+  }
+
   protected abstract String getHttpAddressKey();
 
   protected abstract String getHttpsAddressKey();
