@@ -17,16 +17,14 @@
  */
 package org.apache.hadoop.hdds.server.http;
 
-import static org.apache.hadoop.hdds.utils.RocksDBStoreMBean.ROCKSDB_CONTEXT_PREFIX;
-
 import java.io.IOException;
 import java.io.Writer;
 import java.util.Collections;
+import java.util.List;
 import java.util.Map;
 import java.util.TreeMap;
-import java.util.regex.Pattern;
 import org.apache.commons.configuration2.SubsetConfiguration;
-import org.apache.commons.lang3.StringUtils;
+import org.apache.hadoop.hdds.utils.PrometheusMetricsSinkUtil;
 import org.apache.hadoop.metrics2.AbstractMetric;
 import org.apache.hadoop.metrics2.MetricType;
 import org.apache.hadoop.metrics2.MetricsRecord;
@@ -43,96 +41,90 @@ public class PrometheusMetricsSink implements MetricsSink {
   /**
    * Cached output lines for each metrics.
    */
-  private final Map<String, Map<String, String>> metricLines =
+  private Map<String, Map<String, String>> metricLines =
       Collections.synchronizedSortedMap(new TreeMap<>());
+  private Map<String, Map<String, String>> nextMetricLines =
+      Collections.synchronizedSortedMap(new TreeMap<>());
+  private final String servername;
 
-  private static final Pattern SPLIT_PATTERN =
-      Pattern.compile("(?<!(^|[A-Z_]))(?=[A-Z])|(?<!^)(?=[A-Z][a-z])");
-
-  private static final Pattern REPLACE_PATTERN =
-      Pattern.compile("[^a-zA-Z0-9]+");
-
-  public PrometheusMetricsSink() {
+  public PrometheusMetricsSink(String servername) {
+    this.servername = servername;
   }
 
   @Override
   public void putMetrics(MetricsRecord metricsRecord) {
-    for (AbstractMetric metrics : metricsRecord.metrics()) {
-      if (metrics.type() == MetricType.COUNTER
-          || metrics.type() == MetricType.GAUGE) {
+    for (AbstractMetric metric : metricsRecord.metrics()) {
+      if (metric.type() == MetricType.COUNTER
+          || metric.type() == MetricType.GAUGE) {
 
-        String key = prometheusName(
-            metricsRecord.name(), metrics.name());
+        String metricName =
+            PrometheusMetricsSinkUtil.getMetricName(metricsRecord.name(),
+                metric.name());
+
+        // If there is no username this should be null
+        String username =
+            PrometheusMetricsSinkUtil.getUsername(metricsRecord.name(),
+                metric.name());
+
+        String key = PrometheusMetricsSinkUtil.prometheusName(
+            metricsRecord.name(), metricName);
 
         String prometheusMetricKeyAsString =
-            getPrometheusMetricKeyAsString(metricsRecord, key);
+            getPrometheusMetricKeyAsString(metricsRecord, key, username);
 
         String metricKey = "# TYPE "
             + key
             + " "
-            + metrics.type().toString().toLowerCase();
+            + metric.type().toString().toLowerCase();
 
-        metricLines.computeIfAbsent(metricKey,
-            any -> Collections.synchronizedSortedMap(new TreeMap<>()))
-            .put(prometheusMetricKeyAsString, String.valueOf(metrics.value()));
+        synchronized (this) {
+          nextMetricLines.computeIfAbsent(metricKey,
+                  any -> Collections.synchronizedSortedMap(new TreeMap<>()))
+              .put(prometheusMetricKeyAsString, String.valueOf(metric.value()));
+        }
       }
     }
   }
 
   private String getPrometheusMetricKeyAsString(MetricsRecord metricsRecord,
-      String key) {
+      String key, String username) {
     StringBuilder prometheusMetricKey = new StringBuilder();
     prometheusMetricKey.append(key)
         .append("{");
     String sep = "";
 
+    List<MetricsTag> metricsTags =
+        PrometheusMetricsSinkUtil.addTags(key, username, servername,
+            metricsRecord.tags());
+
     //add tags
-    for (MetricsTag tag : metricsRecord.tags()) {
+    for (MetricsTag tag : metricsTags) {
       String tagName = tag.name().toLowerCase();
 
       //ignore specific tag which includes sub-hierarchy
-      if (!tagName.equals("numopenconnectionsperuser")) {
-        prometheusMetricKey.append(sep)
-            .append(tagName)
-            .append("=\"")
-            .append(tag.value())
-            .append("\"");
-        sep = ",";
+      if (tagName.equals("numopenconnectionsperuser")) {
+        continue;
       }
+
+      prometheusMetricKey.append(sep)
+          .append(tagName)
+          .append("=\"")
+          .append(tag.value())
+          .append("\"");
+      sep = ",";
     }
     prometheusMetricKey.append("}");
 
     return prometheusMetricKey.toString();
   }
 
-  /**
-   * Convert CamelCase based names to lower-case names where the separator
-   * is the underscore, to follow prometheus naming conventions.
-   */
-  public String prometheusName(String recordName,
-      String metricName) {
-
-    // RocksDB metric names already have underscores as delimiters,
-    // but record name is from DB file name and '.' (as in 'om.db') is invalid
-    if (StringUtils.isNotEmpty(recordName) &&
-        recordName.startsWith(ROCKSDB_CONTEXT_PREFIX)) {
-      return normalizeName(recordName) + "_" + metricName.toLowerCase();
-    }
-
-    String baseName = StringUtils.capitalize(recordName)
-        + StringUtils.capitalize(metricName);
-    return normalizeName(baseName);
-  }
-
-  public static String normalizeName(String baseName) {
-    String[] parts = SPLIT_PATTERN.split(baseName);
-    String result = String.join("_", parts).toLowerCase();
-    return REPLACE_PATTERN.matcher(result).replaceAll("_");
-  }
-
   @Override
   public void flush() {
-
+    synchronized (this) {
+      metricLines = nextMetricLines;
+      nextMetricLines = Collections
+          .synchronizedSortedMap(new TreeMap<>());
+    }
   }
 
   @Override
@@ -140,7 +132,8 @@ public class PrometheusMetricsSink implements MetricsSink {
 
   }
 
-  public void writeMetrics(Writer writer) throws IOException {
+  public synchronized void writeMetrics(Writer writer)
+      throws IOException {
     for (Map.Entry<String, Map<String, String>> metricsEntry
         : metricLines.entrySet()) {
       writer.write(metricsEntry.getKey() + "\n");
