@@ -28,6 +28,7 @@ import java.util.Map;
 import org.apache.commons.lang3.NotImplementedException;
 import org.apache.hadoop.hdds.annotation.InterfaceStability;
 import org.apache.hadoop.hdds.utils.MetadataKeyFilters;
+import org.apache.hadoop.hdds.utils.TableCacheMetrics;
 import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
 import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
 /**
@@ -147,6 +148,15 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
    */
   void deleteWithBatch(BatchOperation batch, KEY key) throws IOException;
 
+  /**
+   * Deletes a range of keys from the metadata store.
+   *
+   * @param beginKey start metadata key
+   * @param endKey end metadata key
+   * @throws IOException on Failure
+   */
+  void deleteRange(KEY beginKey, KEY endKey) throws IOException;
+
   /**
    * Returns the iterator for this metadata store.
    *
@@ -190,6 +200,17 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
     throw new NotImplementedException("addCacheEntry is not implemented");
   }
 
+  /** Add entry to the table cache with a non-null key and a null value. */
+  default void addCacheEntry(KEY cacheKey, long epoch) {
+    addCacheEntry(new CacheKey<>(cacheKey), CacheValue.get(epoch));
+  }
+
+  /** Add entry to the table cache with a non-null key and a non-null value. */
+  default void addCacheEntry(KEY cacheKey, VALUE value, long epoch) {
+    addCacheEntry(new CacheKey<>(cacheKey),
+        CacheValue.get(epoch, value));
+  }
+
   /**
    * Get the cache value from table cache.
    * @param cacheKey
@@ -215,6 +236,13 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
     throw new NotImplementedException("cacheIterator is not implemented");
   }
 
+  /**
+   * Create the metrics datasource that emits table cache metrics.
+   */
+  default TableCacheMetrics createCacheMetrics() throws IOException {
+    throw new NotImplementedException("getCacheValue is not implemented");
+  }
+
   /**
    * Returns a certain range of key value pairs as a list based on a
    * startKey or count. Further a {@link MetadataKeyFilters.MetadataKeyFilter}
@@ -309,4 +337,28 @@ public interface Table<KEY, VALUE> extends AutoCloseable {
 
     VALUE getValue() throws IOException;
   }
+
+  static <K, V> KeyValue<K, V> newKeyValue(K key, V value) {
+    return new KeyValue<K, V>() {
+      @Override
+      public K getKey() {
+        return key;
+      }
+
+      @Override
+      public V getValue() {
+        return value;
+      }
+
+      @Override
+      public String toString() {
+        return "(key=" + key + ", value=" + value + ")";
+      }
+    };
+  }
+
+  /** A {@link TableIterator} to iterate {@link KeyValue}s. */
+  interface KeyValueIterator<KEY, VALUE>
+      extends TableIterator<KEY, KeyValue<KEY, VALUE>> {
+  }
 }
