@@ -19,25 +19,30 @@ package org.apache.hadoop.hdds.security.token;
 
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerCommandRequestProto;
-import org.apache.hadoop.hdds.security.x509.SecurityConfig;
-import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient;
+import org.apache.hadoop.hdds.security.symmetric.ManagedSecretKey;
+import org.apache.hadoop.hdds.security.symmetric.SecretKeySignerClient;
+import org.apache.hadoop.hdds.security.symmetric.SecretKeyVerifierClient;
+import org.apache.hadoop.hdds.security.SecurityConfig;
 import org.apache.hadoop.security.token.Token;
+import org.apache.hadoop.security.token.TokenIdentifier;
+import org.jetbrains.annotations.NotNull;
 import org.junit.jupiter.api.Test;
+import org.mockito.Mockito;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import javax.crypto.SecretKey;
 import java.io.IOException;
-import java.security.cert.CertificateException;
-import java.security.cert.CertificateExpiredException;
-import java.security.cert.CertificateNotYetValidException;
-import java.security.cert.X509Certificate;
+import java.time.Duration;
 import java.time.Instant;
+import java.util.UUID;
 import java.util.concurrent.TimeUnit;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
+import static org.hamcrest.CoreMatchers.containsString;
+import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.mockito.ArgumentMatchers.any;
-import static org.mockito.Mockito.doThrow;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.never;
 import static org.mockito.Mockito.verify;
@@ -51,13 +56,13 @@ public abstract class TokenVerifierTests<T extends ShortLivedTokenIdentifier> {
   private static final Logger LOG =
       LoggerFactory.getLogger(TokenVerifierTests.class);
 
-  protected static final String CERT_ID = "123";
+  protected static final UUID SECRET_KEY_ID = UUID.randomUUID();
 
   /**
    * Create the specific kind of TokenVerifier.
    */
   protected abstract TokenVerifier newTestSubject(
-      SecurityConfig secConf, CertificateClient caClient);
+      SecurityConfig secConf, SecretKeyVerifierClient secretKeyClient);
 
   /**
    * @return the config key to enable/disable the specific kind of tokens
@@ -82,98 +87,138 @@ public abstract class TokenVerifierTests<T extends ShortLivedTokenIdentifier> {
   @Test
   public void skipsVerificationIfDisabled() throws IOException {
     // GIVEN
-    CertificateClient caClient = mock(CertificateClient.class);
-    TokenVerifier subject = newTestSubject(tokenDisabled(), caClient);
+    SecretKeyVerifierClient secretKeyClient = mock(
+        SecretKeyVerifierClient.class);
+    TokenVerifier subject = newTestSubject(tokenDisabled(), secretKeyClient);
 
     // WHEN
     subject.verify("anyUser", anyToken(), verifiedRequest(newTokenId()));
 
     // THEN
-    verify(caClient, never()).getCertificate(any());
+    verify(secretKeyClient, never()).getSecretKey(any());
   }
 
   @Test
   public void skipsVerificationForMiscCommands() throws IOException {
     // GIVEN
-    CertificateClient caClient = mock(CertificateClient.class);
-    TokenVerifier subject = newTestSubject(tokenEnabled(), caClient);
+    SecretKeyVerifierClient secretKeyClient = mock(
+        SecretKeyVerifierClient.class);
+    TokenVerifier subject = newTestSubject(tokenEnabled(), secretKeyClient);
 
     // WHEN
     subject.verify("anyUser", anyToken(), unverifiedRequest());
 
     // THEN
-    verify(caClient, never()).getCertificate(any());
+    verify(secretKeyClient, never()).getSecretKey(any());
   }
 
   @Test
-  public void rejectsExpiredCertificate() throws Exception {
-    rejectsInvalidCertificate(CertificateExpiredException.class);
-  }
-
-  @Test
-  public void rejectsNotYetValidCertificate() throws Exception {
-    rejectsInvalidCertificate(CertificateNotYetValidException.class);
-  }
-
-  private void rejectsInvalidCertificate(
-      Class<? extends CertificateException> problem) throws Exception {
+  public void rejectsExpiredSecretKey() throws Exception {
     // GIVEN
-    CertificateClient caClient = mock(CertificateClient.class);
-    X509Certificate cert = invalidCertificate(problem);
-    when(caClient.getCertificate(CERT_ID)).thenReturn(cert);
-    ContainerCommandRequestProto cmd = verifiedRequest(newTokenId());
-    TokenVerifier subject = newTestSubject(tokenEnabled(), caClient);
+    SecretKeyVerifierClient secretKeyClient =
+        mock(SecretKeyVerifierClient.class);
+
+    Instant past = Instant.now().minus(Duration.ofHours(1));
+    ManagedSecretKey expiredSecretKey = new ManagedSecretKey(UUID.randomUUID(),
+        past, past, Mockito.mock(SecretKey.class));
+
+    when(secretKeyClient.getSecretKey(SECRET_KEY_ID))
+        .thenReturn(expiredSecretKey);
+    T tokenId = newTokenId();
+    ContainerCommandRequestProto cmd = verifiedRequest(tokenId);
+    TokenVerifier subject = newTestSubject(tokenEnabled(), secretKeyClient);
 
     // WHEN+THEN
-    assertThrows(BlockTokenException.class, () ->
-        subject.verify("anyUser", anyToken(), cmd));
+    ShortLivedTokenSecretManager<T> secretManager = new MockTokenManager();
+    Token<T> token = secretManager.generateToken(tokenId);
+    BlockTokenException ex = assertThrows(BlockTokenException.class, () ->
+        subject.verify("anyUser", token, cmd));
+    assertThat(ex.getMessage(), containsString("expired secret key"));
+  }
+
+  @Test
+  public void rejectsTokenWithInvalidSecretId() throws Exception {
+    // GIVEN
+    SecretKeyVerifierClient secretKeyClient =
+        mock(SecretKeyVerifierClient.class);
+
+    when(secretKeyClient.getSecretKey(SECRET_KEY_ID)).thenReturn(null);
+    T tokenId = newTokenId();
+    ContainerCommandRequestProto cmd = verifiedRequest(tokenId);
+    TokenVerifier subject = newTestSubject(tokenEnabled(), secretKeyClient);
+
+    // WHEN+THEN
+    ShortLivedTokenSecretManager<T> secretManager = new MockTokenManager();
+    Token<T> token = secretManager.generateToken(tokenId);
+    BlockTokenException ex = assertThrows(BlockTokenException.class, () ->
+        subject.verify("anyUser", token, cmd));
+    assertThat(ex.getMessage(),
+        containsString("Can't find the signing secret key"));
   }
 
   @Test
   public void rejectsInvalidSignature() throws Exception {
     // GIVEN
-    CertificateClient caClient = mock(CertificateClient.class);
-    when(caClient.getCertificate(CERT_ID)).thenReturn(validCertificate());
-    Token<?> invalidToken = new Token<>();
-    validSignature(caClient, false);
-    ContainerCommandRequestProto cmd = verifiedRequest(newTokenId());
-    TokenVerifier subject = newTestSubject(tokenEnabled(), caClient);
+    SecretKeyVerifierClient secretKeyClient =
+        mockSecretKeyClient(false);
+
+    ShortLivedTokenSecretManager<T> secretManager = new MockTokenManager();
+    T tokenId = newTokenId();
+    Token<?> invalidToken = secretManager.generateToken(tokenId);
+    ContainerCommandRequestProto cmd = verifiedRequest(tokenId);
+    TokenVerifier subject = newTestSubject(tokenEnabled(), secretKeyClient);
 
     // WHEN+THEN
-    assertThrows(BlockTokenException.class, () ->
-        subject.verify("anyUser", invalidToken, cmd));
+    BlockTokenException ex =
+        assertThrows(BlockTokenException.class, () ->
+            subject.verify("anyUser", invalidToken, cmd));
+    assertThat(ex.getMessage(),
+        containsString("Invalid token for user"));
+  }
+
+  @NotNull
+  private SecretKeyVerifierClient mockSecretKeyClient(boolean validSignature)
+      throws IOException {
+    SecretKeyVerifierClient secretKeyClient =
+        mock(SecretKeyVerifierClient.class);
+    ManagedSecretKey validSecretKey = Mockito.mock(ManagedSecretKey.class);
+    when(secretKeyClient.getSecretKey(SECRET_KEY_ID))
+        .thenReturn(validSecretKey);
+    when(validSecretKey.isValidSignature((TokenIdentifier) any(), any()))
+        .thenReturn(validSignature);
+    return secretKeyClient;
   }
 
   @Test
   public void rejectsExpiredToken() throws Exception {
     // GIVEN
-    SecurityConfig conf = tokenEnabled();
-    CertificateClient caClient = mock(CertificateClient.class);
-    when(caClient.getCertificate(CERT_ID)).thenReturn(validCertificate());
-    validSignature(caClient, true);
-    ShortLivedTokenSecretManager<T> secretManager = new MockTokenManager(conf);
+    SecretKeyVerifierClient secretKeyClient = mockSecretKeyClient(true);
+
+    ShortLivedTokenSecretManager<T> secretManager = new MockTokenManager();
     T tokenId = expired(newTokenId());
     ContainerCommandRequestProto cmd = verifiedRequest(tokenId);
     Token<?> token = secretManager.generateToken(tokenId);
-    TokenVerifier subject = newTestSubject(tokenEnabled(), caClient);
+    TokenVerifier subject = newTestSubject(tokenEnabled(), secretKeyClient);
 
     // WHEN+THEN
-    assertThrows(BlockTokenException.class, () ->
-        subject.verify("anyUser", token, cmd));
+    BlockTokenException ex =
+        assertThrows(BlockTokenException.class, () ->
+            subject.verify("anyUser", token, cmd));
+    assertThat(ex.getMessage(),
+        containsString("Expired token for user"));
   }
 
   @Test
   public void acceptsValidToken() throws Exception {
     // GIVEN
     SecurityConfig conf = tokenEnabled();
-    CertificateClient caClient = mock(CertificateClient.class);
-    when(caClient.getCertificate(CERT_ID)).thenReturn(validCertificate());
-    validSignature(caClient, true);
-    ShortLivedTokenSecretManager<T> secretManager = new MockTokenManager(conf);
+    SecretKeyVerifierClient secretKeyClient = mockSecretKeyClient(true);
+
+    ShortLivedTokenSecretManager<T> secretManager = new MockTokenManager();
     T tokenId = valid(newTokenId());
     ContainerCommandRequestProto cmd = verifiedRequest(tokenId);
     Token<?> token = secretManager.generateToken(tokenId);
-    TokenVerifier subject = newTestSubject(conf, caClient);
+    TokenVerifier subject = newTestSubject(conf, secretKeyClient);
 
     // WHEN+THEN
     subject.verify("anyUser", token, cmd);
@@ -189,24 +234,6 @@ public abstract class TokenVerifierTests<T extends ShortLivedTokenIdentifier> {
     return tokenId;
   }
 
-  private void validSignature(CertificateClient caClient, boolean valid)
-      throws Exception {
-    when(caClient.verifySignature(any(byte[].class), any(), any()))
-        .thenReturn(valid);
-  }
-
-  private static X509Certificate invalidCertificate(
-      Class<? extends CertificateException> problem)
-      throws CertificateExpiredException, CertificateNotYetValidException {
-    X509Certificate cert = mock(X509Certificate.class);
-    doThrow(problem).when(cert).checkValidity();
-    return cert;
-  }
-
-  private static X509Certificate validCertificate() {
-    return mock(X509Certificate.class);
-  }
-
   protected SecurityConfig tokenDisabled() {
     return getSecurityConfig(false);
   }
@@ -230,8 +257,9 @@ public abstract class TokenVerifierTests<T extends ShortLivedTokenIdentifier> {
    */
   private class MockTokenManager extends ShortLivedTokenSecretManager<T> {
 
-    MockTokenManager(SecurityConfig conf) {
-      super(conf, TimeUnit.HOURS.toMillis(1), CERT_ID, LOG);
+    MockTokenManager() {
+      super(TimeUnit.HOURS.toMillis(1),
+          Mockito.mock(SecretKeySignerClient.class));
     }
 
     @Override
