@@ -27,7 +27,7 @@ import java.security.PublicKey;
 import java.security.interfaces.RSAPublicKey;
 import java.security.spec.PKCS8EncodedKeySpec;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
-import org.apache.hadoop.hdds.security.x509.SecurityConfig;
+import org.apache.hadoop.hdds.security.SecurityConfig;
 import org.apache.ozone.test.GenericTestUtils;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.BeforeEach;
@@ -56,7 +56,7 @@ public class TestHDDSKeyGenerator {
   @Test
   public void testGenerateKey()
       throws NoSuchProviderException, NoSuchAlgorithmException {
-    HDDSKeyGenerator keyGen = new HDDSKeyGenerator(config.getConfiguration());
+    HDDSKeyGenerator keyGen = new HDDSKeyGenerator(config);
     KeyPair keyPair = keyGen.generateKey();
     Assertions.assertEquals(config.getKeyAlgo(),
         keyPair.getPrivate().getAlgorithm());
@@ -76,7 +76,7 @@ public class TestHDDSKeyGenerator {
   @Test
   public void testGenerateKeyWithSize() throws NoSuchProviderException,
       NoSuchAlgorithmException {
-    HDDSKeyGenerator keyGen = new HDDSKeyGenerator(config.getConfiguration());
+    HDDSKeyGenerator keyGen = new HDDSKeyGenerator(config);
     KeyPair keyPair = keyGen.generateKey(4096);
     PublicKey publicKey = keyPair.getPublic();
     if (publicKey instanceof RSAPublicKey) {
@@ -84,4 +84,4 @@ public class TestHDDSKeyGenerator {
           ((RSAPublicKey)(publicKey)).getModulus().bitLength());
     }
   }
-}
\ No newline at end of file
+}
