@@ -19,10 +19,11 @@
 
 package org.apache.hadoop.hdds.utils.db;
 
-import javax.management.MBeanServer;
 import java.io.File;
+import java.io.FileInputStream;
+import java.io.FilenameFilter;
 import java.io.IOException;
-import java.lang.management.ManagementFactory;
+import java.io.InputStream;
 import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.util.Arrays;
@@ -37,6 +38,7 @@ import org.apache.hadoop.hdds.StringUtils;
 import org.apache.commons.lang3.RandomStringUtils;
 import org.apache.hadoop.hdds.utils.db.managed.ManagedColumnFamilyOptions;
 import org.apache.hadoop.hdds.utils.db.managed.ManagedDBOptions;
+import org.apache.hadoop.hdds.utils.db.managed.ManagedWriteOptions;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.BeforeEach;
@@ -46,10 +48,22 @@ import org.rocksdb.RocksDB;
 import org.rocksdb.Statistics;
 import org.rocksdb.StatsLevel;
 
+import static org.apache.hadoop.ozone.OzoneConsts.ROCKSDB_SST_SUFFIX;
+
 /**
  * RDBStore Tests.
  */
 public class TestRDBStore {
+  public static RDBStore newRDBStore(File dbFile, ManagedDBOptions options,
+      Set<TableConfig> families,
+      long maxDbUpdatesSizeThreshold)
+      throws IOException {
+    return new RDBStore(dbFile, options, new ManagedWriteOptions(), families,
+        CodecRegistry.newBuilder().build(), false, 1000, null, false,
+        maxDbUpdatesSizeThreshold, true, null, "");
+  }
+
+  public static final int MAX_DB_UPDATES_SIZE_THRESHOLD = 80;
   private final List<String> families =
       Arrays.asList(StringUtils.bytes2String(RocksDB.DEFAULT_COLUMN_FAMILY),
           "First", "Second", "Third",
@@ -61,6 +75,8 @@ public class TestRDBStore {
 
   @BeforeEach
   public void setUp(@TempDir File tempDir) throws Exception {
+    CodecBuffer.enableLeakDetection();
+
     options = new ManagedDBOptions();
     options.setCreateIfMissing(true);
     options.setCreateMissingColumnFamilies(true);
@@ -74,7 +90,8 @@ public class TestRDBStore {
           new ManagedColumnFamilyOptions());
       configSet.add(newConfig);
     }
-    rdbStore = new RDBStore(tempDir, options, configSet);
+    rdbStore = newRDBStore(tempDir, options, configSet,
+        MAX_DB_UPDATES_SIZE_THRESHOLD);
   }
 
   @AfterEach
@@ -82,10 +99,13 @@ public class TestRDBStore {
     if (rdbStore != null) {
       rdbStore.close();
     }
+    CodecBuffer.assertNoLeaks();
   }
-  private void insertRandomData(RDBStore dbStore, int familyIndex)
-      throws Exception {
-    try (Table firstTable = dbStore.getTable(families.get(familyIndex))) {
+
+  public void insertRandomData(RDBStore dbStore, int familyIndex)
+      throws IOException {
+    try (Table<byte[], byte[]> firstTable = dbStore.getTable(families.
+        get(familyIndex))) {
       Assertions.assertNotNull(firstTable, "Table cannot be null");
       for (int x = 0; x < 100; x++) {
         byte[] key =
@@ -94,16 +114,28 @@ public class TestRDBStore {
           RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
         firstTable.put(key, value);
       }
+    } catch (Exception e) {
+      throw new IOException(e);
     }
   }
 
   @Test
   public void compactDB() throws Exception {
     Assertions.assertNotNull(rdbStore, "DB Store cannot be null");
-    insertRandomData(rdbStore, 1);
-    // This test does not assert anything if there is any error this test
-    // will throw and fail.
+    for (int i = 0; i < 2; i++) {
+      for (int j = 0; j <= 10; j++) {
+        insertRandomData(rdbStore, i);
+        rdbStore.flushDB();
+      }
+    }
+
+    int metaSizeBeforeCompact = rdbStore.getDb().getLiveFilesMetaDataSize();
     rdbStore.compactDB();
+    int metaSizeAfterCompact = rdbStore.getDb().getLiveFilesMetaDataSize();
+
+    Assertions.assertTrue(metaSizeAfterCompact < metaSizeBeforeCompact);
+    Assertions.assertEquals(metaSizeAfterCompact, 2);
+
   }
 
   @Test
@@ -185,32 +217,6 @@ public class TestRDBStore {
         || rdbStore.getEstimatedKeyCount() < 220);
   }
 
-  @Test
-  public void getStatMBeanName() throws Exception {
-
-    try (Table firstTable = rdbStore.getTable(families.get(1))) {
-      for (int y = 0; y < 100; y++) {
-        byte[] key =
-            RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-        byte[] value =
-            RandomStringUtils.random(10).getBytes(StandardCharsets.UTF_8);
-        firstTable.put(key, value);
-      }
-    }
-    MBeanServer platformMBeanServer =
-        ManagementFactory.getPlatformMBeanServer();
-    Thread.sleep(2000);
-
-    Object keysWritten = platformMBeanServer
-        .getAttribute(rdbStore.getStatMBeanName(), "NUMBER_KEYS_WRITTEN");
-
-    Assertions.assertTrue(((Long) keysWritten) >= 99L);
-
-    Object dbWriteAverage = platformMBeanServer
-        .getAttribute(rdbStore.getStatMBeanName(), "DB_WRITE_AVERAGE");
-    Assertions.assertTrue((double) dbWriteAverage > 0);
-  }
-
   @Test
   public void getTable() throws Exception {
     for (String tableName : families) {
@@ -251,8 +257,8 @@ public class TestRDBStore {
     Assertions.assertNotNull(checkpoint);
 
     RDBStore restoredStoreFromCheckPoint =
-        new RDBStore(checkpoint.getCheckpointLocation().toFile(),
-            options, configSet);
+        newRDBStore(checkpoint.getCheckpointLocation().toFile(),
+            options, configSet, MAX_DB_UPDATES_SIZE_THRESHOLD);
 
     // Let us make sure that our estimate is not off by 10%
     Assertions.assertTrue(
@@ -308,11 +314,24 @@ public class TestRDBStore {
           org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key2"),
           org.apache.commons.codec.binary.StringUtils
               .getBytesUtf16("Value2"));
+      firstTable.put(
+          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key3"),
+          org.apache.commons.codec.binary.StringUtils
+              .getBytesUtf16("Value3"));
+      firstTable.put(
+          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key4"),
+          org.apache.commons.codec.binary.StringUtils
+              .getBytesUtf16("Value4"));
+      firstTable.put(
+          org.apache.commons.codec.binary.StringUtils.getBytesUtf16("Key5"),
+          org.apache.commons.codec.binary.StringUtils
+              .getBytesUtf16("Value5"));
     }
-    Assertions.assertEquals(2, rdbStore.getDb().getLatestSequenceNumber());
+    Assertions.assertEquals(5, rdbStore.getDb().getLatestSequenceNumber());
 
-    DBUpdatesWrapper dbUpdatesSince = rdbStore.getUpdatesSince(0, 1);
-    Assertions.assertEquals(1, dbUpdatesSince.getData().size());
+    DBUpdatesWrapper dbUpdatesSince = rdbStore.getUpdatesSince(0, 5);
+    Assertions.assertEquals(2, dbUpdatesSince.getData().size());
+    Assertions.assertEquals(2, dbUpdatesSince.getCurrentSequenceNumber());
   }
 
   @Test
@@ -342,7 +361,8 @@ public class TestRDBStore {
           new ManagedColumnFamilyOptions());
       configSet.add(newConfig);
     }
-    rdbStore = new RDBStore(rdbStore.getDbLocation(), options, configSet);
+    rdbStore = newRDBStore(rdbStore.getDbLocation(), options, configSet,
+        MAX_DB_UPDATES_SIZE_THRESHOLD);
     for (String family : familiesMinusOne) {
       try (Table table = rdbStore.getTable(family)) {
         Assertions.assertNotNull(table, family + "is null");
@@ -361,4 +381,60 @@ public class TestRDBStore {
     }
   }
 
-}
\ No newline at end of file
+  @Test
+  public void testSstConsistency() throws IOException {
+    for (int i = 0; i < 10; i++) {
+      insertRandomData(rdbStore, 0);
+      insertRandomData(rdbStore, 1);
+      insertRandomData(rdbStore, 2);
+    }
+    DBCheckpoint dbCheckpoint1 = rdbStore.getCheckpoint(true);
+
+    for (int i = 0; i < 10; i++) {
+      insertRandomData(rdbStore, 0);
+      insertRandomData(rdbStore, 1);
+      insertRandomData(rdbStore, 2);
+    }
+    DBCheckpoint dbCheckpoint2 = rdbStore.getCheckpoint(true);
+    compareSstWithSameName(dbCheckpoint1.getCheckpointLocation().toFile(),
+        dbCheckpoint2.getCheckpointLocation().toFile());
+
+    for (int i = 0; i < 10; i++) {
+      insertRandomData(rdbStore, 0);
+      insertRandomData(rdbStore, 1);
+      insertRandomData(rdbStore, 2);
+    }
+    DBCheckpoint dbCheckpoint3 = rdbStore.getCheckpoint(true);
+    compareSstWithSameName(dbCheckpoint2.getCheckpointLocation().toFile(),
+        dbCheckpoint3.getCheckpointLocation().toFile());
+  }
+
+  private void compareSstWithSameName(File checkpoint1, File checkpoint2)
+      throws IOException {
+    FilenameFilter filter = (dir, name) -> name.endsWith(ROCKSDB_SST_SUFFIX);
+    String[] files1 = checkpoint1.list(filter);
+    String[] files2 = checkpoint1.list(filter);
+    assert files1 != null;
+    assert files2 != null;
+    // Get all file names in the both checkpoints
+    List<String> result = Arrays.asList(files1);
+    result.retainAll(Arrays.asList(files2));
+
+    for (String name: result) {
+      File fileInCk1 = new File(checkpoint1.getAbsoluteFile(), name);
+      File fileInCk2 = new File(checkpoint2.getAbsoluteFile(), name);
+      long length1 = fileInCk1.length();
+      long length2 = fileInCk2.length();
+      Assertions.assertEquals(length1, length2, name);
+
+      try (InputStream fileStream1 = new FileInputStream(fileInCk1);
+           InputStream fileStream2 = new FileInputStream(fileInCk2)) {
+        byte[] content1 = new byte[fileStream1.available()];
+        byte[] content2 = new byte[fileStream2.available()];
+        fileStream1.read(content1);
+        fileStream2.read(content2);
+        Assertions.assertArrayEquals(content1, content2);
+      }
+    }
+  }
+}
