@@ -26,18 +26,21 @@ import org.apache.hadoop.hdds.scm.exceptions.SCMException;
 import org.apache.hadoop.hdds.scm.net.NetworkTopology;
 import org.apache.hadoop.hdds.scm.net.Node;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
+import org.apache.hadoop.hdds.scm.pipeline.PipelineStateManager;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashSet;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
-import java.util.stream.Collectors;
-import java.util.stream.Stream;
+
+import static org.apache.hadoop.hdds.scm.exceptions.SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE;
 
 /**
  * Container placement policy that scatter datanodes on different racks
@@ -77,11 +80,26 @@ public final class SCMContainerPlacementRackScatter
     this.metrics = metrics;
   }
 
-  public Set<DatanodeDetails> chooseNodesFromRacks(List<Node> racks,
-          List<Node> unavailableNodes,
-          List<DatanodeDetails> mutableFavoredNodes,
-          int nodesRequired, final long metadataSizeRequired,
-          final long dataSizeRequired, int maxOuterLoopIterations) {
+  /**
+   * Constructor for Pipeline Provider Pipeline Placement with rack awareness.
+   * @param nodeManager Node Manager
+   * @param stateManager State Manager
+   * @param conf Configuration
+   */
+  public SCMContainerPlacementRackScatter(NodeManager nodeManager,
+      PipelineStateManager stateManager, ConfigurationSource conf) {
+    super(nodeManager, conf);
+    this.networkTopology = nodeManager.getClusterNetworkTopologyMap();
+    this.metrics = null;
+  }
+
+  @SuppressWarnings("checkstyle:parameternumber")
+  private Set<DatanodeDetails> chooseNodesFromRacks(List<Node> racks,
+      List<Node> unavailableNodes,
+      List<DatanodeDetails> mutableFavoredNodes,
+      int nodesRequired, long metadataSizeRequired, long dataSizeRequired,
+      int maxOuterLoopIterations, Map<Node, Integer> rackCntMap,
+      int maxReplicasPerRack) {
     if (nodesRequired <= 0) {
       return Collections.emptySet();
     }
@@ -97,8 +115,12 @@ public final class SCMContainerPlacementRackScatter
       int chosenListSize = chosenNodes.size();
 
       // Refill toChooseRacks, we put skippedRacks in front of toChooseRacks
-      // for a even distribution
-      toChooseRacks.addAll(racks);
+      // for an even distribution
+      for (Node rack : racks) {
+        if (rackCntMap.getOrDefault(rack, 0) < maxReplicasPerRack) {
+          toChooseRacks.add(rack);
+        }
+      }
       if (!skippedRacks.isEmpty()) {
         toChooseRacks.removeAll(skippedRacks);
         toChooseRacks.addAll(0, skippedRacks);
@@ -111,6 +133,7 @@ public final class SCMContainerPlacementRackScatter
           Node curRack = getRackOfDatanodeDetails(favoredNode);
           if (toChooseRacks.contains(curRack)) {
             chosenNodes.add(favoredNode);
+            rackCntMap.merge(curRack, 1, Math::addExact);
             toChooseRacks.remove(curRack);
             chosenFavoredNodesInForLoop.add(favoredNode);
             unavailableNodes.add(favoredNode);
@@ -137,6 +160,7 @@ public final class SCMContainerPlacementRackScatter
                 metadataSizeRequired, dataSizeRequired);
         if (node != null) {
           chosenNodes.add((DatanodeDetails) node);
+          rackCntMap.merge(rack, 1, Math::addExact);
           mutableFavoredNodes.remove(node);
           unavailableNodes.add(node);
           nodesRequired--;
@@ -173,42 +197,48 @@ public final class SCMContainerPlacementRackScatter
    *                     allocator, whether the favored nodes will be used
    *                     depends on whether the nodes meets the allocator's
    *                     requirement.
-   * @param nodesRequiredToChoose - number of datanodes required.
+   * @param nodesRequired - number of datanodes required.
    * @param dataSizeRequired - size required for the container.
    * @param metadataSizeRequired - size required for Ratis metadata.
    * @return List of datanodes.
    * @throws SCMException  SCMException
    */
   @Override
+  @SuppressWarnings("checkstyle:methodlength")
   protected List<DatanodeDetails> chooseDatanodesInternal(
           List<DatanodeDetails> usedNodes,
           final List<DatanodeDetails> excludedNodes,
           final List<DatanodeDetails> favoredNodes,
-          final int nodesRequiredToChoose, final long metadataSizeRequired,
+          final int nodesRequired, final long metadataSizeRequired,
           final long dataSizeRequired) throws SCMException {
-    if (nodesRequiredToChoose <= 0) {
+    if (nodesRequired <= 0) {
       String errorMsg = "num of nodes required to choose should bigger" +
-          "than 0, but the given num is " + nodesRequiredToChoose;
+          "than 0, but the given num is " + nodesRequired;
       throw new SCMException(errorMsg, null);
     }
-    metrics.incrDatanodeRequestCount(nodesRequiredToChoose);
-    int nodesRequired = nodesRequiredToChoose;
+    if (metrics != null) {
+      metrics.incrDatanodeRequestCount(nodesRequired);
+    }
     int excludedNodesCount = excludedNodes == null ? 0 : excludedNodes.size();
+    int usedNodesCount = usedNodes == null ? 0 : usedNodes.size();
     List<Node> availableNodes = networkTopology.getNodes(
         networkTopology.getMaxLevel());
     int totalNodesCount = availableNodes.size();
     if (excludedNodes != null) {
       availableNodes.removeAll(excludedNodes);
     }
+    if (usedNodes != null) {
+      availableNodes.removeAll(usedNodes);
+    }
     if (availableNodes.size() < nodesRequired) {
       throw new SCMException("No enough datanodes to choose. " +
-          "TotalNode = " + totalNodesCount +
-          " AvailableNode = " + availableNodes.size() +
-          " RequiredNode = " + nodesRequired +
-          " ExcludedNode = " + excludedNodesCount,
-          SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE);
+          "TotalNodes = " + totalNodesCount +
+          " AvailableNodes = " + availableNodes.size() +
+          " RequiredNodes = " + nodesRequired +
+          " ExcludedNodes = " + excludedNodesCount +
+          " UsedNodes = " + usedNodesCount,
+          FAILED_TO_FIND_SUITABLE_NODE);
     }
-
     List<DatanodeDetails> mutableFavoredNodes = new ArrayList<>();
     if (favoredNodes != null) {
       // Generate mutableFavoredNodes, only stores valid favoredNodes
@@ -223,41 +253,46 @@ public final class SCMContainerPlacementRackScatter
     if (excludedNodes != null) {
       mutableFavoredNodes.removeAll(excludedNodes);
     }
-
     if (usedNodes == null) {
       usedNodes = Collections.emptyList();
     }
     List<Node> racks = getAllRacks();
-    Set<Node> usedRacks = usedNodes.stream()
-            .map(node -> networkTopology.getAncestor(node, RACK_LEVEL))
-            .filter(node -> node != null)
-            .collect(Collectors.toSet());
-    int requiredReplicationFactor = usedNodes.size() + nodesRequired;
-    int numberOfRacksRequired =
-            getRequiredRackCount(requiredReplicationFactor);
-    int additionalRacksRequired = numberOfRacksRequired - usedRacks.size();
-    if (nodesRequired < additionalRacksRequired) {
-      String reason = "Required nodes size: " + nodesRequired
-              + " is less than required number of racks to choose: "
-              + additionalRacksRequired + ".";
-      LOG.warn("Placement policy cannot choose the enough racks. {}"
-                      + "Total number of Required Racks: {} Used Racks Count:" +
-                      " {}, Required Nodes count: {}",
-              reason, numberOfRacksRequired, usedRacks.size(), nodesRequired);
-      throw new SCMException(reason,
-              SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE);
+    // usedRacksCntMap maps a rack to the number of usedNodes it contains
+    Map<Node, Integer> usedRacksCntMap = new HashMap<>();
+    for (Node node : usedNodes) {
+      Node rack = networkTopology.getAncestor(node, RACK_LEVEL);
+      if (rack != null) {
+        usedRacksCntMap.merge(rack, 1, Math::addExact);
+      }
     }
-    // For excluded nodes, we sort their racks at rear
-    racks = sortRackWithExcludedNodes(racks, excludedNodes, usedRacks);
 
-    List<Node> unavailableNodes = new ArrayList<>();
+    List<Node> unavailableRacks = findRacksWithOnlyExcludedNodes(excludedNodes,
+        usedRacksCntMap);
+    for (Node rack : unavailableRacks) {
+      racks.remove(rack);
+    }
+
+    int requiredReplicationFactor = usedNodes.size() + nodesRequired;
+    int numberOfRacksRequired = getRequiredRackCount(requiredReplicationFactor,
+        unavailableRacks.size());
+    int additionalRacksRequired =
+        Math.min(nodesRequired, numberOfRacksRequired - usedRacksCntMap.size());
+    LOG.debug("Additional nodes required: {}. Additional racks required: {}.",
+        nodesRequired, additionalRacksRequired);
+    int maxReplicasPerRack = getMaxReplicasPerRack(requiredReplicationFactor,
+            numberOfRacksRequired);
+    LOG.debug("According to required replication factor: {}, and total number" +
+            " of racks required: {}, max replicas per rack is {}.",
+        requiredReplicationFactor, numberOfRacksRequired, maxReplicasPerRack);
+    // For excluded nodes, we sort their racks at rear
+    racks = sortRackWithExcludedNodes(racks, excludedNodes, usedRacksCntMap);
+
+    List<Node> unavailableNodes = new ArrayList<>(usedNodes);
     if (excludedNodes != null) {
       unavailableNodes.addAll(excludedNodes);
     }
-    unavailableNodes.addAll(usedNodes);
-
-    Set<DatanodeDetails> chosenNodes = new LinkedHashSet<>();
 
+    LOG.debug("Available racks excluding racks with used nodes: {}.", racks);
     if (racks.size() < additionalRacksRequired) {
       String reason = "Number of existing racks: " + racks.size()
               + "is less than additional required number of racks to choose: "
@@ -265,68 +300,126 @@ public final class SCMContainerPlacementRackScatter
       LOG.warn("Placement policy cannot choose the enough racks. {}"
                       + "Total number of Required Racks: {} Used Racks Count:" +
                       " {}, Required Nodes count: {}",
-              reason, numberOfRacksRequired, usedRacks.size(), nodesRequired);
+              reason, numberOfRacksRequired, usedRacksCntMap.size(),
+              nodesRequired);
       throw new SCMException(reason,
-              SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE);
+              FAILED_TO_FIND_SUITABLE_NODE);
     }
 
-    chosenNodes.addAll(chooseNodesFromRacks(racks, unavailableNodes,
+    Set<DatanodeDetails> chosenNodes = new LinkedHashSet<>(
+        chooseNodesFromRacks(racks, unavailableNodes,
             mutableFavoredNodes, additionalRacksRequired,
-            metadataSizeRequired, dataSizeRequired, 1));
+            metadataSizeRequired, dataSizeRequired, maxReplicasPerRack,
+            usedRacksCntMap, maxReplicasPerRack));
 
     if (chosenNodes.size() < additionalRacksRequired) {
       String reason = "Chosen nodes size from Unique Racks: " + chosenNodes
               .size() + ", but required nodes to choose from Unique Racks: "
               + additionalRacksRequired + " do not match.";
       LOG.warn("Placement policy could not choose the enough nodes from " +
-                      "available racks."
-                      + " {} Available racks count: {}," +
-                      " Excluded nodes count: {}",
-              reason, racks.size(), excludedNodesCount);
+                      "available racks. {} Available racks count: {},"
+                      + " Excluded nodes count: {}, UsedNodes count: {}",
+              reason, racks.size(), excludedNodesCount, usedNodesCount);
       throw new SCMException(reason,
               SCMException.ResultCodes.FAILED_TO_FIND_HEALTHY_NODES);
     }
 
     if (chosenNodes.size() < nodesRequired) {
-      racks.addAll(usedRacks);
-      usedRacks.addAll(chosenNodes.stream()
-              .map(node -> networkTopology.getAncestor(node, RACK_LEVEL))
-              .filter(node -> node != null)
-              .collect(Collectors.toSet()));
-      sortRackWithExcludedNodes(racks, excludedNodes, usedRacks);
-      racks.addAll(usedRacks);
+      racks.addAll(usedRacksCntMap.keySet());
+      racks = sortRackWithExcludedNodes(racks, excludedNodes, usedRacksCntMap);
+      racks.addAll(usedRacksCntMap.keySet());
+      LOG.debug("Available racks considering racks with used and exclude " +
+              "nodes: {}.", racks);
       chosenNodes.addAll(chooseNodesFromRacks(racks, unavailableNodes,
               mutableFavoredNodes, nodesRequired - chosenNodes.size(),
-              metadataSizeRequired, dataSizeRequired, Integer.MAX_VALUE));
+              metadataSizeRequired, dataSizeRequired,
+              Integer.MAX_VALUE, usedRacksCntMap, maxReplicasPerRack));
     }
-
     List<DatanodeDetails> result = new ArrayList<>(chosenNodes);
-
-    if (nodesRequiredToChoose != chosenNodes.size()) {
+    if (nodesRequired != chosenNodes.size()) {
       String reason = "Chosen nodes size: " + chosenNodes
               .size() + ", but required nodes to choose: "
-              + nodesRequiredToChoose + " do not match.";
+              + nodesRequired + " do not match.";
       LOG.warn("Placement policy could not choose the enough nodes."
-               + " {} Available nodes count: {}, Excluded nodes count: {}",
-              reason, totalNodesCount, excludedNodesCount);
+               + " {} Available nodes count: {}, Excluded nodes count: {}, "
+               + " Used nodes count: {}",
+              reason, totalNodesCount, excludedNodesCount, usedNodesCount);
       throw new SCMException(reason,
               SCMException.ResultCodes.FAILED_TO_FIND_HEALTHY_NODES);
     }
-
+    List<DatanodeDetails> newPlacement =
+        new ArrayList<>(usedNodes.size() + result.size());
+    newPlacement.addAll(usedNodes);
+    newPlacement.addAll(chosenNodes);
     ContainerPlacementStatus placementStatus =
-        validateContainerPlacement(
-                Stream.of(usedNodes, result)
-                      .flatMap(List::stream).collect(Collectors.toList()),
-                requiredReplicationFactor);
+        validateContainerPlacement(newPlacement, requiredReplicationFactor);
     if (!placementStatus.isPolicySatisfied()) {
-      String errorMsg = "ContainerPlacementPolicy not met, currentRacks is " +
-          placementStatus.actualPlacementCount() + " desired racks is " +
-          placementStatus.expectedPlacementCount();
-      throw new SCMException(errorMsg, null);
+      ContainerPlacementStatus initialPlacementStatus =
+          validateContainerPlacement(usedNodes, requiredReplicationFactor);
+      if (initialPlacementStatus.misReplicationCount()
+              < placementStatus.misReplicationCount()) {
+        String errorMsg = "ContainerPlacementPolicy not met. Misreplication" +
+                " Reason: " + placementStatus.misReplicatedReason() +
+                " Initial Used nodes mis-replication Count: " +
+                initialPlacementStatus.misReplicationCount() +
+                " Used nodes + Chosen nodes mis-replication Count: " +
+                placementStatus.misReplicationCount();
+        throw new SCMException(errorMsg, FAILED_TO_FIND_SUITABLE_NODE);
+      }
     }
+    LOG.info("Chosen nodes: {}. isPolicySatisfied: {}.", result,
+        placementStatus.isPolicySatisfied());
     return result;
   }
 
+  /**
+   * Given a list of excluded nodes, check if the rack for each excluded node is
+   * empty after removing the excluded nodes. If it is empty, then the rack
+   * contains only excluded nodes, and we return a list of these racks.
+   * @param excludedNodes List of excluded nodes
+   * @param usedRacksCntMap Map of used racks and their used node count
+   * @return List of racks that contain only excluded nodes or an empty list
+   */
+  private List<Node> findRacksWithOnlyExcludedNodes(
+      List<DatanodeDetails> excludedNodes, Map<Node, Integer> usedRacksCntMap) {
+    if (excludedNodes == null || excludedNodes.isEmpty()) {
+      return Collections.emptyList();
+    }
+
+    List<Node> unavailableRacks = new ArrayList<>();
+    Set<Node> excludedNodeRacks = new HashSet<>();
+    for (Node node : excludedNodes) {
+      Node rack = networkTopology.getAncestor(node, RACK_LEVEL);
+      if (rack != null && !usedRacksCntMap.containsKey(rack)) {
+        // Dead nodes are removed from the topology, so the node may have a null
+        // rack, hence the not null check.
+        // Anything that reaches here is the rack for an excluded node and no
+        // used nodes on are on the rack.
+        excludedNodeRacks.add(rack);
+      }
+    }
+    Set<Node> exc = new HashSet<>(excludedNodes);
+    for (Node rack : excludedNodeRacks) {
+      // If a node is removed from the cluster (eg goes dead), but the client
+      // already added it to the exclude list, then the rack may not be in the
+      // topology any longer. See test
+      // testExcludedNodesOverlapsOutOfServiceNodes
+      String rackPath = rack.getNetworkFullPath();
+      if (networkTopology.getNode(rackPath) == null) {
+        continue;
+      }
+
+      Node node = networkTopology.chooseRandom(rack.getNetworkFullPath(), exc);
+      if (node == null) {
+        // This implies we have a rack with all nodes excluded, so it is as if
+        // that rack does not exist. We also know there are no used nodes on
+        // this rack, so it means we need to reduce the rack count by 1
+        unavailableRacks.add(rack);
+      }
+    }
+    return unavailableRacks;
+  }
+
   @Override
   public DatanodeDetails chooseNode(List<DatanodeDetails> healthyNodes) {
     return null;
@@ -348,7 +441,9 @@ public final class SCMContainerPlacementRackScatter
       long metadataSizeRequired, long dataSizeRequired) {
     int maxRetry = INNER_LOOP_MAX_RETRY;
     while (true) {
-      metrics.incrDatanodeChooseAttemptCount();
+      if (metrics != null) {
+        metrics.incrDatanodeChooseAttemptCount();
+      }
       Node node = null;
       try {
         node = networkTopology.chooseRandom(scope, excludedNodes);
@@ -364,7 +459,9 @@ public final class SCMContainerPlacementRackScatter
         DatanodeDetails datanodeDetails = (DatanodeDetails) node;
         if (isValidNode(datanodeDetails, metadataSizeRequired,
             dataSizeRequired)) {
-          metrics.incrDatanodeChooseSuccessCount();
+          if (metrics != null) {
+            metrics.incrDatanodeChooseSuccessCount();
+          }
           return node;
         }
         // exclude the unavailable node for the following retries.
@@ -389,15 +486,19 @@ public final class SCMContainerPlacementRackScatter
    * For EC placement policy, desired rack count would be equal to the num of
    * Replicas.
    * @param numReplicas - num of Replicas.
+   * @param excludedRackCount - The number of racks excluded due to containing
+   *                          only excluded nodes. The total racks on the
+   *                          cluster will be reduced by this number.
    * @return required rack count.
    */
   @Override
-  protected int getRequiredRackCount(int numReplicas) {
+  protected int getRequiredRackCount(int numReplicas, int excludedRackCount) {
     if (networkTopology == null) {
       return 1;
     }
     int maxLevel = networkTopology.getMaxLevel();
-    int numRacks = networkTopology.getNumOfNodes(maxLevel - 1);
+    int numRacks = networkTopology.getNumOfNodes(maxLevel - 1)
+        - excludedRackCount;
     // Return the num of Rack if numRack less than numReplicas
     return Math.min(numRacks, numReplicas);
   }
@@ -413,24 +514,27 @@ public final class SCMContainerPlacementRackScatter
    * @param racks
    * @param excludedNodes
    * @param usedRacks
-   * @return
+   * @return racks that are present in the specified racks, excluding racks
+   * that are present in the specified usedRacks
    */
   private List<Node> sortRackWithExcludedNodes(List<Node> racks,
-          List<DatanodeDetails> excludedNodes, Set<Node> usedRacks) {
+          List<DatanodeDetails> excludedNodes, Map<Node, Integer> usedRacks) {
     if ((excludedNodes == null || excludedNodes.isEmpty())
             && usedRacks.isEmpty()) {
       return racks;
     }
-    Set<Node> lessPreferredRacks = excludedNodes.stream()
-        .map(node -> networkTopology.getAncestor(node, RACK_LEVEL))
-        // Dead Nodes have been removed from the topology and so have a
-        // null rack. We need to exclude those from the rack list.
-        .filter(node -> node != null)
-        .filter(node -> !usedRacks.contains(node))
-        .collect(Collectors.toSet());
+    Set<Node> lessPreferredRacks = new HashSet<>();
+    for (Node node : excludedNodes) {
+      Node rack = networkTopology.getAncestor(node, RACK_LEVEL);
+      // Dead Nodes have been removed from the topology and so have a
+      // null rack. We need to exclude those from the rack list.
+      if (rack != null && !usedRacks.containsKey(rack)) {
+        lessPreferredRacks.add(rack);
+      }
+    }
     List <Node> result = new ArrayList<>();
     for (Node rack : racks) {
-      if (!usedRacks.contains(rack) && !lessPreferredRacks.contains(rack)) {
+      if (!usedRacks.containsKey(rack) && !lessPreferredRacks.contains(rack)) {
         result.add(rack);
       }
     }
