@@ -21,12 +21,16 @@ import org.apache.hadoop.hdds.scm.block.DeletedBlockLog;
 import org.apache.hadoop.hdds.scm.block.DeletedBlockLogImpl;
 import org.apache.hadoop.hdds.scm.metadata.SCMMetadataStore;
 import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
+import org.apache.hadoop.hdds.utils.IOUtils;
 import org.apache.hadoop.hdds.utils.TransactionInfo;
 import org.apache.hadoop.hdds.utils.db.BatchOperation;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.ratis.statemachine.SnapshotInfo;
 
 import java.io.IOException;
+import java.util.concurrent.atomic.AtomicLong;
+import java.util.concurrent.atomic.AtomicReference;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import static org.apache.hadoop.ozone.OzoneConsts.TRANSACTION_INFO_KEY;
 
@@ -41,7 +45,11 @@ public class SCMHADBTransactionBufferImpl implements SCMHADBTransactionBuffer {
   private SCMMetadataStore metadataStore;
   private BatchOperation currentBatchOperation;
   private TransactionInfo latestTrxInfo;
-  private SnapshotInfo latestSnapshot;
+  private final AtomicReference<SnapshotInfo> latestSnapshot
+      = new AtomicReference<>();
+  private final AtomicLong txFlushPending = new AtomicLong(0);
+  private long lastSnapshotTimeMs = 0;
+  private final ReentrantReadWriteLock rwLock = new ReentrantReadWriteLock();
 
   public SCMHADBTransactionBufferImpl(StorageContainerManager scm)
       throws IOException {
@@ -56,13 +64,25 @@ public class SCMHADBTransactionBufferImpl implements SCMHADBTransactionBuffer {
   @Override
   public <KEY, VALUE> void addToBuffer(
       Table<KEY, VALUE> table, KEY key, VALUE value) throws IOException {
-    table.putWithBatch(getCurrentBatchOperation(), key, value);
+    rwLock.readLock().lock();
+    try {
+      txFlushPending.getAndIncrement();
+      table.putWithBatch(getCurrentBatchOperation(), key, value);
+    } finally {
+      rwLock.readLock().unlock();
+    }
   }
 
   @Override
   public <KEY, VALUE> void removeFromBuffer(Table<KEY, VALUE> table, KEY key)
       throws IOException {
-    table.deleteWithBatch(getCurrentBatchOperation(), key);
+    rwLock.readLock().lock();
+    try {
+      txFlushPending.getAndIncrement();
+      table.deleteWithBatch(getCurrentBatchOperation(), key);
+    } finally {
+      rwLock.readLock().unlock();
+    }
   }
 
   @Override
@@ -82,53 +102,86 @@ public class SCMHADBTransactionBufferImpl implements SCMHADBTransactionBuffer {
 
   @Override
   public SnapshotInfo getLatestSnapshot() {
-    return latestSnapshot;
+    return latestSnapshot.get();
   }
 
   @Override
   public void setLatestSnapshot(SnapshotInfo latestSnapshot) {
-    this.latestSnapshot = latestSnapshot;
+    this.latestSnapshot.set(latestSnapshot);
+  }
+
+  @Override
+  public AtomicReference<SnapshotInfo> getLatestSnapshotRef() {
+    return latestSnapshot;
   }
 
   @Override
   public void flush() throws IOException {
-    // write latest trx info into trx table in the same batch
-    Table<String, TransactionInfo> transactionInfoTable
-        = metadataStore.getTransactionInfoTable();
-    transactionInfoTable.putWithBatch(currentBatchOperation,
-        TRANSACTION_INFO_KEY, latestTrxInfo);
+    rwLock.writeLock().lock();
+    try {
+      // write latest trx info into trx table in the same batch
+      Table<String, TransactionInfo> transactionInfoTable
+          = metadataStore.getTransactionInfoTable();
+      transactionInfoTable.putWithBatch(currentBatchOperation,
+          TRANSACTION_INFO_KEY, latestTrxInfo);
 
-    metadataStore.getStore().commitBatchOperation(currentBatchOperation);
-    currentBatchOperation.close();
-    this.latestSnapshot = latestTrxInfo.toSnapshotInfo();
-    // reset batch operation
-    currentBatchOperation = metadataStore.getStore().initBatchOperation();
+      metadataStore.getStore().commitBatchOperation(currentBatchOperation);
+      currentBatchOperation.close();
+      this.latestSnapshot.set(latestTrxInfo.toSnapshotInfo());
+      // reset batch operation
+      currentBatchOperation = metadataStore.getStore().initBatchOperation();
 
-    DeletedBlockLog deletedBlockLog = scm.getScmBlockManager()
-        .getDeletedBlockLog();
-    Preconditions.checkArgument(
-        deletedBlockLog instanceof DeletedBlockLogImpl);
-    ((DeletedBlockLogImpl) deletedBlockLog).onFlush();
+      DeletedBlockLog deletedBlockLog = scm.getScmBlockManager()
+          .getDeletedBlockLog();
+      Preconditions.checkArgument(
+          deletedBlockLog instanceof DeletedBlockLogImpl);
+      ((DeletedBlockLogImpl) deletedBlockLog).onFlush();
+    } finally {
+      txFlushPending.set(0);
+      lastSnapshotTimeMs = scm.getSystemClock().millis();
+      rwLock.writeLock().unlock();
+    }
   }
 
   @Override
   public void init() throws IOException {
     metadataStore = scm.getScmMetadataStore();
 
-    // initialize a batch operation during construction time
-    currentBatchOperation = this.metadataStore.getStore().initBatchOperation();
-    latestTrxInfo = this.metadataStore.getTransactionInfoTable()
-        .get(TRANSACTION_INFO_KEY);
-    if (latestTrxInfo == null) {
-      // transaction table is empty
-      latestTrxInfo =
-          TransactionInfo
-              .builder()
-              .setTransactionIndex(-1)
-              .setCurrentTerm(0)
-              .build();
+    rwLock.writeLock().lock();
+    try {
+      IOUtils.closeQuietly(currentBatchOperation);
+
+      // initialize a batch operation during construction time
+      currentBatchOperation = this.metadataStore.getStore().
+          initBatchOperation();
+      latestTrxInfo = this.metadataStore.getTransactionInfoTable()
+          .get(TRANSACTION_INFO_KEY);
+      if (latestTrxInfo == null) {
+        // transaction table is empty
+        latestTrxInfo =
+            TransactionInfo
+                .builder()
+                .setTransactionIndex(-1)
+                .setCurrentTerm(0)
+                .build();
+      }
+      latestSnapshot.set(latestTrxInfo.toSnapshotInfo());
+    } finally {
+      txFlushPending.set(0);
+      lastSnapshotTimeMs = scm.getSystemClock().millis();
+      rwLock.writeLock().unlock();
+    }
+  }
+  
+  @Override
+  public boolean shouldFlush(long snapshotWaitTime) {
+    rwLock.readLock().lock();
+    try {
+      long timeDiff = scm.getSystemClock().millis() - lastSnapshotTimeMs;
+      return txFlushPending.get() > 0 && timeDiff > snapshotWaitTime;
+    } finally {
+      rwLock.readLock().unlock();
     }
-    latestSnapshot = latestTrxInfo.toSnapshotInfo();
   }
 
   @Override
