@@ -19,7 +19,6 @@ package org.apache.hadoop.hdds.scm.node;
 
 import java.io.IOException;
 import java.util.Set;
-import java.util.concurrent.TimeoutException;
 
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
@@ -91,8 +90,8 @@ public class HealthyReadOnlyNodeHandler
             pipelineID, pipeline.getPipelineState(),
             HddsProtos.NodeState.HEALTHY_READONLY,
             datanodeDetails.getUuidString());
-        pipelineManager.closePipeline(pipeline, true);
-      } catch (IOException | TimeoutException ex) {
+        pipelineManager.closePipeline(pipelineID);
+      } catch (IOException ex) {
         LOG.error("Failed to close pipeline {} which uses HEALTHY READONLY " +
             "datanode {}: ", pipelineID, datanodeDetails, ex);
       }
@@ -105,7 +104,7 @@ public class HealthyReadOnlyNodeHandler
       // make sure after DN is added back into topology, DatanodeDetails
       // instance returned from nodeStateManager has parent correctly set.
       Preconditions.checkNotNull(
-          nodeManager.getNodeByUuid(datanodeDetails.getUuidString())
+          nodeManager.getNodeByUuid(datanodeDetails.getUuid())
               .getParent());
     }
   }
