@@ -62,7 +62,6 @@ import java.util.Map;
 import java.util.NavigableSet;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.locks.ReentrantReadWriteLock;
 import java.util.stream.Collectors;
@@ -184,7 +183,7 @@ public class PipelineManagerImpl implements PipelineManager {
             .setPeriodicalTask(() -> {
               try {
                 pipelineManager.scrubPipelines();
-              } catch (IOException | TimeoutException e) {
+              } catch (IOException e) {
                 LOG.error("Unexpected error during pipeline scrubbing", e);
               }
             }).build();
@@ -195,9 +194,50 @@ public class PipelineManagerImpl implements PipelineManager {
     return pipelineManager;
   }
 
+  /**
+   * Build a new pipeline and return it, but do not add it to the pipeline
+   * manager. This new pipeline will be in ALLOCATED state, but also unavailable
+   * to clients in the system until it is added to the pipeline manager via the
+   * addPipeline method.
+   * @param replicationConfig
+   * @param excludedNodes
+   * @param favoredNodes
+   * @return The created pipeline.
+   * @throws IOException
+   */
+  @Override
+  public Pipeline buildECPipeline(ReplicationConfig replicationConfig,
+      List<DatanodeDetails> excludedNodes, List<DatanodeDetails> favoredNodes)
+      throws IOException {
+    if (replicationConfig.getReplicationType() != ReplicationType.EC) {
+      throw new IllegalArgumentException("Replication type must be EC");
+    }
+    checkIfPipelineCreationIsAllowed(replicationConfig);
+    return pipelineFactory.create(replicationConfig, excludedNodes,
+        favoredNodes);
+  }
+
+  /**
+   * Add a previously built pipeline to the pipeline manager. This will allow
+   * the pipline to be used by clients in the system.
+   * @param pipeline
+   * @throws IOException
+   */
+  @Override
+  public void addEcPipeline(Pipeline pipeline)
+      throws IOException {
+    if (pipeline.getReplicationConfig().getReplicationType()
+        != ReplicationType.EC) {
+      throw new IllegalArgumentException(
+          "Pipeline replication type must be EC");
+    }
+    checkIfPipelineCreationIsAllowed(pipeline.getReplicationConfig());
+    addPipelineToManager(pipeline);
+  }
+
   @Override
   public Pipeline createPipeline(ReplicationConfig replicationConfig)
-      throws IOException, TimeoutException {
+      throws IOException {
     return createPipeline(replicationConfig, Collections.emptyList(),
         Collections.emptyList());
   }
@@ -205,7 +245,28 @@ public class PipelineManagerImpl implements PipelineManager {
   @Override
   public Pipeline createPipeline(ReplicationConfig replicationConfig,
       List<DatanodeDetails> excludedNodes, List<DatanodeDetails> favoredNodes)
-      throws IOException, TimeoutException {
+      throws IOException {
+    checkIfPipelineCreationIsAllowed(replicationConfig);
+
+    acquireWriteLock();
+    final Pipeline pipeline;
+    try {
+      try {
+        pipeline = pipelineFactory.create(replicationConfig,
+            excludedNodes, favoredNodes);
+      } catch (IOException e) {
+        metrics.incNumPipelineCreationFailed();
+        throw e;
+      }
+      addPipelineToManager(pipeline);
+      return pipeline;
+    } finally {
+      releaseWriteLock();
+    }
+  }
+
+  private void checkIfPipelineCreationIsAllowed(
+      ReplicationConfig replicationConfig) throws IOException {
     if (!isPipelineCreationAllowed() && !factorOne(replicationConfig)) {
       LOG.debug("Pipeline creation is not allowed until safe mode prechecks " +
           "complete");
@@ -219,23 +280,23 @@ public class PipelineManagerImpl implements PipelineManager {
       LOG.info(message);
       throw new IOException(message);
     }
+  }
 
+  private void addPipelineToManager(Pipeline pipeline)
+      throws IOException {
+    HddsProtos.Pipeline pipelineProto = pipeline.getProtobufMessage(
+        ClientVersion.CURRENT_VERSION);
     acquireWriteLock();
     try {
-      Pipeline pipeline = pipelineFactory.create(replicationConfig,
-          excludedNodes, favoredNodes);
-      stateManager.addPipeline(pipeline.getProtobufMessage(
-          ClientVersion.CURRENT_VERSION));
-      recordMetricsForPipeline(pipeline);
-      return pipeline;
-    } catch (IOException | TimeoutException ex) {
-      LOG.debug("Failed to create pipeline with replicationConfig {}.",
-          replicationConfig, ex);
+      stateManager.addPipeline(pipelineProto);
+    } catch (IOException ex) {
+      LOG.debug("Failed to add pipeline {}.", pipeline, ex);
       metrics.incNumPipelineCreationFailed();
       throw ex;
     } finally {
       releaseWriteLock();
     }
+    recordMetricsForPipeline(pipeline);
   }
 
   private boolean factorOne(ReplicationConfig replicationConfig) {
@@ -309,14 +370,14 @@ public class PipelineManagerImpl implements PipelineManager {
         .getPipelines(replicationConfig, state, excludeDns, excludePipelines);
   }
 
-  @Override
   /**
    * Returns the count of pipelines meeting the given ReplicationConfig and
    * state.
-   * @param replicationConfig The ReplicationConfig of the pipelines to count
+   * @param config The ReplicationConfig of the pipelines to count
    * @param state The current state of the pipelines to count
    * @return The count of pipelines meeting the above criteria
    */
+  @Override
   public int getPipelineCount(ReplicationConfig config,
                                      Pipeline.PipelineState state) {
     return stateManager.getPipelineCount(config, state);
@@ -333,7 +394,7 @@ public class PipelineManagerImpl implements PipelineManager {
   public void addContainerToPipelineSCMStart(
       PipelineID pipelineID, ContainerID containerID) throws IOException {
     // should not lock here, since no ratis operation happens.
-    stateManager.addContainerToPipelineSCMStart(pipelineID, containerID);
+    stateManager.addContainerToPipelineForce(pipelineID, containerID);
   }
 
   @Override
@@ -356,23 +417,24 @@ public class PipelineManagerImpl implements PipelineManager {
 
   @Override
   public void openPipeline(PipelineID pipelineId)
-      throws IOException, TimeoutException {
+      throws IOException {
+    HddsProtos.PipelineID pipelineIdProtobuf = pipelineId.getProtobuf();
     acquireWriteLock();
+    final Pipeline pipeline;
     try {
-      Pipeline pipeline = stateManager.getPipeline(pipelineId);
+      pipeline = stateManager.getPipeline(pipelineId);
       if (pipeline.isClosed()) {
         throw new IOException("Closed pipeline can not be opened");
       }
       if (pipeline.getPipelineState() == Pipeline.PipelineState.ALLOCATED) {
-        LOG.info("Pipeline {} moved to OPEN state", pipeline);
-        stateManager.updatePipelineState(
-            pipelineId.getProtobuf(), HddsProtos.PipelineState.PIPELINE_OPEN);
+        stateManager.updatePipelineState(pipelineIdProtobuf,
+            HddsProtos.PipelineState.PIPELINE_OPEN);
       }
-      metrics.incNumPipelineCreated();
-      metrics.createPerPipelineMetrics(pipeline);
     } finally {
       releaseWriteLock();
     }
+    metrics.incNumPipelineCreated();
+    metrics.createPerPipelineMetrics(pipeline);
   }
 
   /**
@@ -382,19 +444,20 @@ public class PipelineManagerImpl implements PipelineManager {
    * @throws IOException
    */
   protected void removePipeline(Pipeline pipeline)
-      throws IOException, TimeoutException {
+      throws IOException {
     pipelineFactory.close(pipeline.getType(), pipeline);
-    PipelineID pipelineID = pipeline.getId();
+    HddsProtos.PipelineID pipelineID = pipeline.getId().getProtobuf();
     acquireWriteLock();
     try {
-      stateManager.removePipeline(pipelineID.getProtobuf());
-      metrics.incNumPipelineDestroyed();
+      stateManager.removePipeline(pipelineID);
     } catch (IOException ex) {
       metrics.incNumPipelineDestroyFailed();
       throw ex;
     } finally {
       releaseWriteLock();
     }
+    LOG.info("Pipeline {} removed.", pipeline);
+    metrics.incNumPipelineDestroyed();
   }
 
   /**
@@ -402,8 +465,8 @@ public class PipelineManagerImpl implements PipelineManager {
    * @param pipelineId - ID of the pipeline.
    * @throws IOException
    */
-  protected void closeContainersForPipeline(final PipelineID pipelineId)
-      throws IOException, TimeoutException {
+  private void closeContainersForPipeline(final PipelineID pipelineId)
+      throws IOException {
     Set<ContainerID> containerIDs = stateManager.getContainers(pipelineId);
     ContainerManager containerManager = scmContext.getScm()
         .getContainerManager();
@@ -426,29 +489,46 @@ public class PipelineManagerImpl implements PipelineManager {
    * put pipeline in CLOSED state.
    * @param pipeline - ID of the pipeline.
    * @param onTimeout - whether to remove pipeline after some time.
-   * @throws IOException
+   * @throws IOException throws exception in case of failure
+   * @deprecated Do not use this method, onTimeout is not honored.
    */
-  @Override
+  @Deprecated
   public void closePipeline(Pipeline pipeline, boolean onTimeout)
-      throws IOException, TimeoutException {
-    PipelineID pipelineID = pipeline.getId();
+          throws IOException {
+    closePipeline(pipeline.getId());
+  }
+
+  /**
+   * Move the Pipeline to CLOSED state.
+   * @param pipelineID ID of the Pipeline to be closed
+   * @throws IOException In case of exception while closing the Pipeline
+   */
+  public void closePipeline(PipelineID pipelineID) throws IOException {
+    HddsProtos.PipelineID pipelineIDProtobuf = pipelineID.getProtobuf();
     // close containers.
     closeContainersForPipeline(pipelineID);
-    acquireWriteLock();
-    try {
-      if (!pipeline.isClosed()) {
-        stateManager.updatePipelineState(pipelineID.getProtobuf(),
+    if (!getPipeline(pipelineID).isClosed()) {
+      acquireWriteLock();
+      try {
+        stateManager.updatePipelineState(pipelineIDProtobuf,
             HddsProtos.PipelineState.PIPELINE_CLOSED);
-        LOG.info("Pipeline {} moved to CLOSED state", pipeline);
+      } finally {
+        releaseWriteLock();
       }
-      metrics.removePipelineMetrics(pipelineID);
-    } finally {
-      releaseWriteLock();
-    }
-    if (!onTimeout) {
-      // close pipeline right away.
-      removePipeline(pipeline);
+      LOG.info("Pipeline {} moved to CLOSED state", pipelineID);
     }
+
+    metrics.removePipelineMetrics(pipelineID);
+
+  }
+
+  /**
+   * Deletes the Pipeline for the given PipelineID.
+   * @param pipelineID ID of the Pipeline to be deleted
+   * @throws IOException In case of exception while deleting the Pipeline
+   */
+  public void deletePipeline(PipelineID pipelineID) throws IOException {
+    removePipeline(getPipeline(pipelineID));
   }
 
   /** close the pipelines whose nodes' IPs are stale.
@@ -468,10 +548,11 @@ public class PipelineManagerImpl implements PipelineManager {
             pipelinesWithStaleIpOrHostname.size());
     pipelinesWithStaleIpOrHostname.forEach(p -> {
       try {
-        LOG.info("Closing the stale pipeline: {}", p.getId());
-        closePipeline(p, false);
-        LOG.info("Closed the stale pipeline: {}", p.getId());
-      } catch (IOException | TimeoutException e) {
+        final PipelineID id = p.getId();
+        LOG.info("Closing the stale pipeline: {}", id);
+        closePipeline(id);
+        deletePipeline(id);
+      } catch (IOException e) {
         LOG.error("Closing the stale pipeline failed: {}", p, e);
       }
     });
@@ -495,32 +576,40 @@ public class PipelineManagerImpl implements PipelineManager {
    * Scrub pipelines.
    */
   @Override
-  public void scrubPipelines() throws IOException, TimeoutException {
+  public void scrubPipelines() throws IOException {
     Instant currentTime = clock.instant();
-    Long pipelineScrubTimeoutInMills = conf.getTimeDuration(
+    long pipelineScrubTimeoutInMills = conf.getTimeDuration(
         ScmConfigKeys.OZONE_SCM_PIPELINE_ALLOCATED_TIMEOUT,
         ScmConfigKeys.OZONE_SCM_PIPELINE_ALLOCATED_TIMEOUT_DEFAULT,
         TimeUnit.MILLISECONDS);
+    long pipelineDeleteTimoutInMills = conf.getTimeDuration(
+            ScmConfigKeys.OZONE_SCM_PIPELINE_DESTROY_TIMEOUT,
+            ScmConfigKeys.OZONE_SCM_PIPELINE_DESTROY_TIMEOUT_DEFAULT,
+            TimeUnit.MILLISECONDS);
 
     List<Pipeline> candidates = stateManager.getPipelines();
 
     for (Pipeline p : candidates) {
+      final PipelineID id = p.getId();
       // scrub pipelines who stay ALLOCATED for too long.
       if (p.getPipelineState() == Pipeline.PipelineState.ALLOCATED &&
           (currentTime.toEpochMilli() - p.getCreationTimestamp()
               .toEpochMilli() >= pipelineScrubTimeoutInMills)) {
+
         LOG.info("Scrubbing pipeline: id: {} since it stays at ALLOCATED " +
-            "stage for {} mins.", p.getId(),
+            "stage for {} mins.", id,
             Duration.between(currentTime, p.getCreationTimestamp())
                 .toMinutes());
-        closePipeline(p, false);
+        closePipeline(id);
+        deletePipeline(id);
       }
       // scrub pipelines who stay CLOSED for too long.
-      if (p.getPipelineState() == Pipeline.PipelineState.CLOSED) {
+      if (p.getPipelineState() == Pipeline.PipelineState.CLOSED &&
+          (currentTime.toEpochMilli() - p.getStateEnterTime().toEpochMilli())
+              >= pipelineDeleteTimoutInMills) {
         LOG.info("Scrubbing pipeline: id: {} since it stays at CLOSED stage.",
             p.getId());
-        closeContainersForPipeline(p.getId());
-        removePipeline(p);
+        deletePipeline(id);
       }
       // If a datanode is stopped and then SCM is restarted, a pipeline can get
       // stuck in an open state. For Ratis, provided some other DNs that were
@@ -532,8 +621,7 @@ public class PipelineManagerImpl implements PipelineManager {
       if (isOpenWithUnregisteredNodes(p)) {
         LOG.info("Scrubbing pipeline: id: {} as it has unregistered nodes",
             p.getId());
-        closeContainersForPipeline(p.getId());
-        closePipeline(p, true);
+        closePipeline(id);
       }
     }
   }
@@ -548,7 +636,7 @@ public class PipelineManagerImpl implements PipelineManager {
       return false;
     }
     for (DatanodeDetails dn : pipeline.getNodes()) {
-      if (nodeManager.getNodeByUuid(dn.getUuidString()) == null) {
+      if (nodeManager.getNodeByUuid(dn.getUuid()) == null) {
         return true;
       }
     }
@@ -594,11 +682,12 @@ public class PipelineManagerImpl implements PipelineManager {
    */
   @Override
   public void activatePipeline(PipelineID pipelineID)
-      throws IOException, TimeoutException {
+      throws IOException {
+    HddsProtos.PipelineID pipelineIDProtobuf = pipelineID.getProtobuf();
     acquireWriteLock();
     try {
-      stateManager.updatePipelineState(pipelineID.getProtobuf(),
-              HddsProtos.PipelineState.PIPELINE_OPEN);
+      stateManager.updatePipelineState(pipelineIDProtobuf,
+          HddsProtos.PipelineState.PIPELINE_OPEN);
     } finally {
       releaseWriteLock();
     }
@@ -612,10 +701,11 @@ public class PipelineManagerImpl implements PipelineManager {
    */
   @Override
   public void deactivatePipeline(PipelineID pipelineID)
-      throws IOException, TimeoutException {
+      throws IOException {
+    HddsProtos.PipelineID pipelineIDProtobuf = pipelineID.getProtobuf();
     acquireWriteLock();
     try {
-      stateManager.updatePipelineState(pipelineID.getProtobuf(),
+      stateManager.updatePipelineState(pipelineIDProtobuf,
           HddsProtos.PipelineState.PIPELINE_DORMANT);
     } finally {
       releaseWriteLock();
