@@ -83,7 +83,10 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
     this.heavyNodeCriteria = dnLimit == null ? 0 : Integer.parseInt(dnLimit);
   }
 
-  int currentRatisThreePipelineCount(DatanodeDetails datanodeDetails) {
+  public static int currentRatisThreePipelineCount(
+      NodeManager nodeManager,
+      PipelineStateManager stateManager,
+      DatanodeDetails datanodeDetails) {
     // Safe to cast collection's size to int
     return (int) nodeManager.getPipelines(datanodeDetails).stream()
         .map(id -> {
@@ -95,16 +98,24 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
             return null;
           }
         })
-        .filter(this::isNonClosedRatisThreePipeline)
+        .filter(PipelinePlacementPolicy::isNonClosedRatisThreePipeline)
         .count();
   }
 
-  private boolean isNonClosedRatisThreePipeline(Pipeline p) {
-    return p.getReplicationConfig()
+  private static boolean isNonClosedRatisThreePipeline(Pipeline p) {
+    return p != null && p.getReplicationConfig()
         .equals(RatisReplicationConfig.getInstance(ReplicationFactor.THREE))
         && !p.isClosed();
   }
 
+  @Override
+  protected int getMaxReplicasPerRack(int numReplicas, int numberOfRacks) {
+    if (numberOfRacks == 1) {
+      return numReplicas;
+    }
+    return Math.max(numReplicas - 1, 1);
+  }
+
   /**
    * Filter out viable nodes based on
    * 1. nodes that are healthy
@@ -113,31 +124,46 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
    * The results are sorted based on pipeline count of each node.
    *
    * @param excludedNodes - excluded nodes
+   * @param usedNodes - used nodes
    * @param nodesRequired - number of datanodes required.
    * @return a list of viable nodes
    * @throws SCMException when viable nodes are not enough in numbers
    */
   List<DatanodeDetails> filterViableNodes(
-      List<DatanodeDetails> excludedNodes, int nodesRequired,
+      List<DatanodeDetails> excludedNodes,
+      List<DatanodeDetails> usedNodes, int nodesRequired,
       long metadataSizeRequired, long dataSizeRequired)
       throws SCMException {
     // get nodes in HEALTHY state
     List<DatanodeDetails> healthyNodes =
         nodeManager.getNodes(NodeStatus.inServiceHealthy());
+    String msg;
+    if (healthyNodes.size() == 0) {
+      msg = "No healthy node found to allocate container.";
+      LOG.error(msg);
+      throw new SCMException(msg, SCMException.ResultCodes
+              .FAILED_TO_FIND_HEALTHY_NODES);
+    }
+
     healthyNodes = filterNodesWithSpace(healthyNodes, nodesRequired,
         metadataSizeRequired, dataSizeRequired);
     boolean multipleRacks = multipleRacksAvailable(healthyNodes);
+    int excludedNodesSize = 0;
     if (excludedNodes != null) {
+      excludedNodesSize = excludedNodes.size();
       healthyNodes.removeAll(excludedNodes);
     }
+    if (usedNodes != null) {
+      excludedNodesSize += usedNodes.size();
+      healthyNodes.removeAll(usedNodes);
+    }
     int initialHealthyNodesCount = healthyNodes.size();
-    String msg;
 
     if (initialHealthyNodesCount < nodesRequired) {
       msg = String.format("Pipeline creation failed due to no sufficient" +
-              " healthy datanodes. Required %d. Found %d.",
-          nodesRequired, initialHealthyNodesCount);
-      LOG.warn(msg);
+              " healthy datanodes. Required %d. Found %d. Excluded %d.",
+          nodesRequired, initialHealthyNodesCount, excludedNodesSize);
+      LOG.debug(msg);
       throw new SCMException(msg,
           SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE);
     }
@@ -148,7 +174,8 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
     // TODO check if sorting could cause performance issue: HDDS-3466.
     List<DatanodeDetails> healthyList = healthyNodes.stream()
         .map(d ->
-            new DnWithPipelines(d, currentRatisThreePipelineCount(d)))
+            new DnWithPipelines(d, currentRatisThreePipelineCount(nodeManager,
+                stateManager, d)))
         .filter(d ->
             (d.getPipelines() < nodeManager.pipelineLimit(d.getDn())))
         .sorted(Comparator.comparingInt(DnWithPipelines::getPipelines))
@@ -159,14 +186,16 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
       if (LOG.isDebugEnabled()) {
         LOG.debug("Unable to find enough nodes that meet the criteria that" +
             " cannot engage in more than" + heavyNodeCriteria +
-            " pipelines. Nodes required: " + nodesRequired + " Found:" +
+            " pipelines. Nodes required: " + nodesRequired + " Excluded: " +
+            excludedNodesSize + " Found:" +
             healthyList.size() + " healthy nodes count in NodeManager: " +
             initialHealthyNodesCount);
       }
       msg = String.format("Pipeline creation failed because nodes are engaged" +
               " in other pipelines and every node can only be engaged in" +
-              " max %d pipelines. Required %d. Found %d",
-          heavyNodeCriteria, nodesRequired, healthyList.size());
+              " max %d pipelines. Required %d. Found %d. Excluded: %d.",
+          heavyNodeCriteria, nodesRequired, healthyList.size(),
+          excludedNodesSize);
       throw new SCMException(msg,
           SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE);
     }
@@ -225,7 +254,7 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
     // Get a list of viable nodes based on criteria
     // and make sure excludedNodes are excluded from list.
     List<DatanodeDetails> healthyNodes = filterViableNodes(excludedNodes,
-        nodesRequired, metadataSizeRequired, dataSizeRequired);
+        usedNodes, nodesRequired, metadataSizeRequired, dataSizeRequired);
 
     // Randomly picks nodes when all nodes are equal or factor is ONE.
     // This happens when network topology is absent or
@@ -235,7 +264,8 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
     } else {
       // Since topology and rack awareness are available, picks nodes
       // based on them.
-      return this.getResultSet(nodesRequired, healthyNodes);
+      return this.getResultSetWithTopology(nodesRequired, healthyNodes,
+          usedNodes);
     }
   }
 
@@ -258,71 +288,68 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
    * network topology and rack awareness.
    * @param nodesRequired - Nodes Required
    * @param healthyNodes - List of Nodes in the result set.
+   * @param usedNodes - List of used Nodes.
    * @return a list of datanodes
    * @throws SCMException SCMException
    */
-  @Override
-  public List<DatanodeDetails> getResultSet(
-      int nodesRequired, List<DatanodeDetails> healthyNodes)
+  private List<DatanodeDetails> getResultSetWithTopology(
+      int nodesRequired, List<DatanodeDetails> healthyNodes,
+      List<DatanodeDetails> usedNodes)
       throws SCMException {
-    if (nodesRequired != HddsProtos.ReplicationFactor.THREE.getNumber()) {
+    Preconditions.checkNotNull(usedNodes);
+    Preconditions.checkNotNull(healthyNodes);
+    Preconditions.checkState(nodesRequired >= 1);
+
+    if (nodesRequired + usedNodes.size() !=
+        HddsProtos.ReplicationFactor.THREE.getNumber()) {
       throw new SCMException("Nodes required number is not supported: " +
           nodesRequired, SCMException.ResultCodes.INVALID_CAPACITY);
     }
 
-    // Assume rack awareness is not enabled.
-    boolean rackAwareness = false;
     List <DatanodeDetails> results = new ArrayList<>(nodesRequired);
-    // Since nodes are widely distributed, the results should be selected
-    // base on distance in topology, rack awareness and load balancing.
-    List<DatanodeDetails> exclude = new ArrayList<>();
-    // First choose an anchor node.
-    DatanodeDetails anchor = chooseFirstNode(healthyNodes);
-    if (anchor != null) {
-      results.add(anchor);
-      removePeers(anchor, healthyNodes);
-      exclude.add(anchor);
-    } else {
+    List <DatanodeDetails> mutableLstNodes = new ArrayList<>();
+    List<DatanodeDetails> mutableExclude = new ArrayList<>();
+    boolean rackAwareness = getAnchorAndNextNode(healthyNodes,
+        usedNodes, results, mutableLstNodes, mutableExclude);
+    if (mutableLstNodes.size() == 0) {
       LOG.warn("Unable to find healthy node for anchor(first) node.");
       throw new SCMException("Unable to find anchor node.",
           SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE);
     }
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("First node chosen: {}", anchor);
-    }
-
-
-    // Choose the second node on different racks from anchor.
-    DatanodeDetails nextNode = chooseNodeBasedOnRackAwareness(
-        healthyNodes, exclude,
-        nodeManager.getClusterNetworkTopologyMap(), anchor);
-    if (nextNode != null) {
-      // Rack awareness is detected.
-      rackAwareness = true;
-      results.add(nextNode);
-      removePeers(nextNode, healthyNodes);
-      exclude.add(nextNode);
-      if (LOG.isDebugEnabled()) {
-        LOG.debug("Second node chosen: {}", nextNode);
-      }
-    } else {
-      LOG.debug("Pipeline Placement: Unable to find 2nd node on different " +
-          "rack based on rack awareness. anchor: {}", anchor);
+    // First node is anchor node
+    DatanodeDetails anchor = mutableLstNodes.get(0);
+    DatanodeDetails nextNode = null;
+    if (mutableLstNodes.size() == 2) {
+      // Second node is next node
+      nextNode = mutableLstNodes.get(1);
     }
 
     // Then choose nodes close to anchor based on network topology
     int nodesToFind = nodesRequired - results.size();
+    boolean bCheckNodeInAnchorRack = true;
     for (int x = 0; x < nodesToFind; x++) {
       // Pick remaining nodes based on the existence of rack awareness.
       DatanodeDetails pick = null;
-      if (rackAwareness) {
+      if (rackAwareness && bCheckNodeInAnchorRack) {
         pick = chooseNodeBasedOnSameRack(
-            healthyNodes, exclude,
+            healthyNodes, mutableExclude,
             nodeManager.getClusterNetworkTopologyMap(), anchor);
+        if (pick == null) {
+          // No available node to pick from first anchor node
+          // Make nextNode as anchor node and pick remaining node
+          anchor = nextNode;
+          pick = chooseNodeBasedOnSameRack(
+              healthyNodes, mutableExclude,
+              nodeManager.getClusterNetworkTopologyMap(), anchor);
+        }
       }
       // fall back protection
       if (pick == null) {
-        pick = fallBackPickNodes(healthyNodes, exclude);
+        // Make bNodeFoundInAnchorRack to false so that from next node search
+        // it will just search in fallback nodes and avoid searching in
+        // anchor node rack.
+        bCheckNodeInAnchorRack = false;
+        pick = fallBackPickNodes(healthyNodes, mutableExclude);
         if (rackAwareness) {
           LOG.debug("Failed to choose node based on topology. Fallback " +
               "picks node as: {}", pick);
@@ -332,20 +359,21 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
       if (pick != null) {
         results.add(pick);
         removePeers(pick, healthyNodes);
-        exclude.add(pick);
+        mutableExclude.add(pick);
         LOG.debug("Remaining node chosen: {}", pick);
       } else {
         String msg = String.format("Unable to find suitable node in " +
             "pipeline allocation. healthyNodes size: %d, " +
-            "excludeNodes size: %d", healthyNodes.size(), exclude.size());
-        LOG.warn(msg);
+            "excludeNodes size: %d", healthyNodes.size(),
+            mutableExclude.size());
+        LOG.debug(msg);
         throw new SCMException(msg,
             SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE);
       }
     }
 
     if (results.size() < nodesRequired) {
-      LOG.warn("Unable to find the required number of " +
+      LOG.debug("Unable to find the required number of " +
               "healthy nodes that  meet the criteria. Required nodes: {}, " +
               "Found nodes: {}", nodesRequired, results.size());
       throw new SCMException("Unable to find required number of nodes.",
@@ -354,6 +382,92 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
     return results;
   }
 
+  /**
+   * Get Anchor and Next node based on healthyNodes and usedNodes.
+   * @param healthyNodes - List of healthy nodes.
+   * @param usedNodes - List of used nodes.
+   * @param results - List of Nodes in the result set.
+   * @param mutableLstNodes - Anchor and Next node.
+   * @param mutableExclude - List of excluded nodes.
+   * @return whether rack aware is satisfied
+   * @throws SCMException SCMException
+   */
+  private boolean getAnchorAndNextNode(List<DatanodeDetails> healthyNodes,
+                                       List<DatanodeDetails> usedNodes,
+                                       List<DatanodeDetails> results,
+                                       List<DatanodeDetails> mutableLstNodes,
+                                       List<DatanodeDetails> mutableExclude)
+      throws SCMException {
+    // Assume rack awareness is not enabled.
+    boolean rackAwareness = false;
+    // Since nodes are widely distributed, the results should be selected
+    // base on distance in topology, rack awareness and load balancing.
+    DatanodeDetails anchor;
+    DatanodeDetails nextNode = null;
+    // First choose an anchor node.
+    if (usedNodes.size() == 0) {
+      // No usedNode, choose anchor based on healthyNodes
+      anchor = chooseFirstNode(healthyNodes);
+      if (anchor != null) {
+        results.add(anchor);
+        removePeers(anchor, healthyNodes);
+        mutableExclude.add(anchor);
+      } else {
+        LOG.debug("Unable to find healthy node for anchor(first) node.");
+        throw new SCMException("Unable to find anchor node.",
+            SCMException.ResultCodes.FAILED_TO_FIND_SUITABLE_NODE);
+      }
+      LOG.debug("First node chosen: {}", anchor);
+    } else if (usedNodes.size() == 1) {
+      // Only 1 usedNode, consider it as anchor node.
+      anchor = usedNodes.get(0);
+      removePeers(anchor, healthyNodes);
+      mutableExclude.add(anchor);
+    } else if (usedNodes.size() == 2) {
+      // 2 usedNodes, consider 1st as anchor node
+      anchor = usedNodes.get(0);
+      removePeers(anchor, healthyNodes);
+      mutableExclude.add(anchor);
+      if (usedNodes.get(0).getParent() != usedNodes.get(1).getParent()) {
+        // Rack awareness is detected.
+        // If 2 usedNodes are in different rack then
+        // consider second node as next node
+        nextNode = usedNodes.get(1);
+        rackAwareness = true;
+      }
+      mutableExclude.add(usedNodes.get(1));
+      removePeers(nextNode, healthyNodes);
+    } else {
+      // Maximum 2 used nodes can exist.
+      LOG.warn("More than 2 used nodes, unable to choose anchor node.");
+      throw new SCMException("Used Nodes required number is not supported: " +
+          usedNodes.size(), SCMException.ResultCodes.INVALID_CAPACITY);
+    }
+
+    if (nextNode == null) {
+      // Choose the second node on different racks from anchor.
+      nextNode = chooseNodeBasedOnRackAwareness(
+          healthyNodes, mutableExclude,
+          nodeManager.getClusterNetworkTopologyMap(), anchor);
+      if (nextNode != null) {
+        // Rack awareness is detected.
+        rackAwareness = true;
+        results.add(nextNode);
+        removePeers(nextNode, healthyNodes);
+        mutableExclude.add(nextNode);
+        LOG.debug("Second node chosen: {}", nextNode);
+      } else {
+        LOG.debug("Pipeline Placement: Unable to find 2nd node on different " +
+            "rack based on rack awareness. anchor: {}", anchor);
+      }
+    }
+    mutableLstNodes.add(anchor);
+    if (nextNode != null) {
+      mutableLstNodes.add(nextNode);
+    }
+    return rackAwareness;
+  }
+
   /**
    * Find a node from the healthy list and return it after removing it from the
    * list that we are operating on.
@@ -459,11 +573,15 @@ public final class PipelinePlacementPolicy extends SCMCommonPlacementPolicy {
   }
 
   @Override
-  protected int getRequiredRackCount(int numReplicas) {
+  protected int getRequiredRackCount(int numReplicas, int excludedRackCount) {
     return REQUIRED_RACKS;
   }
 
-  private static class DnWithPipelines {
+  /**
+   * static inner utility class for datanodes with pipeline, used for
+   * pipeline engagement checking.
+   */
+  public static class DnWithPipelines {
     private DatanodeDetails dn;
     private int pipelines;
 
