@@ -25,6 +25,7 @@ import org.apache.hadoop.hdds.protocol.proto
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.IncrementalContainerReportProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.LayoutVersionProto;
+import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto;
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.PipelineReportsProto;
 import org.apache.hadoop.hdds.protocol.proto
@@ -41,14 +42,17 @@ import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.SCMHeartbeatRequestProto;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
+import org.apache.hadoop.hdds.server.events.IEventInfo;
 import org.apache.hadoop.ozone.protocol.commands.ReregisterCommand;
 import org.apache.hadoop.ozone.protocol.commands.SCMCommand;
 
 import com.google.protobuf.Message;
+import org.apache.hadoop.util.Time;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.util.List;
+import java.util.Map;
 import java.util.UUID;
 
 import static org.apache.hadoop.hdds.scm.events.SCMEvents.CONTAINER_ACTIONS;
@@ -59,7 +63,6 @@ import static org.apache.hadoop.hdds.scm.events.SCMEvents.NODE_REPORT;
 import static org.apache.hadoop.hdds.scm.events.SCMEvents.CMD_STATUS_REPORT;
 import static org.apache.hadoop.hdds.scm.events.SCMEvents.PIPELINE_ACTIONS;
 import static org.apache.hadoop.hdds.scm.events.SCMEvents.PIPELINE_REPORT;
-import static org.apache.hadoop.hdds.scm.events.SCMEvents.COMMAND_QUEUE_REPORT;
 import static org.apache.hadoop.hdds.upgrade.HDDSLayoutFeature.INITIAL_VERSION;
 import static org.apache.hadoop.ozone.container.upgrade.UpgradeUtils.toLayoutVersionProto;
 
@@ -69,7 +72,7 @@ import static org.apache.hadoop.ozone.container.upgrade.UpgradeUtils.toLayoutVer
  */
 public final class SCMDatanodeHeartbeatDispatcher {
 
-  private static final Logger LOG =
+  public static final Logger LOG =
       LoggerFactory.getLogger(SCMDatanodeHeartbeatDispatcher.class);
 
   private final NodeManager nodeManager;
@@ -122,9 +125,13 @@ public final class SCMDatanodeHeartbeatDispatcher {
       LOG.debug("Processing DataNode Layout Report.");
       nodeManager.processLayoutVersionReport(datanodeDetails, layoutVersion);
 
+      CommandQueueReportProto commandQueueReport = null;
+      if (heartbeat.hasCommandQueueReport()) {
+        commandQueueReport = heartbeat.getCommandQueueReport();
+      }
       // should we dispatch heartbeat through eventPublisher?
       commands = nodeManager.processHeartbeat(datanodeDetails,
-          layoutVersion);
+          layoutVersion, commandQueueReport);
       if (heartbeat.hasNodeReport()) {
         LOG.debug("Dispatching Node Report.");
         eventPublisher.fireEvent(
@@ -134,13 +141,6 @@ public final class SCMDatanodeHeartbeatDispatcher {
                 heartbeat.getNodeReport()));
       }
 
-      if (heartbeat.hasCommandQueueReport()) {
-        LOG.debug("Dispatching Queued Command Report");
-        eventPublisher.fireEvent(COMMAND_QUEUE_REPORT,
-            new CommandQueueReportFromDatanode(datanodeDetails,
-                heartbeat.getCommandQueueReport()));
-      }
-
       if (heartbeat.hasContainerReport()) {
         LOG.debug("Dispatching Container Report.");
         eventPublisher.fireEvent(
@@ -247,9 +247,17 @@ public final class SCMDatanodeHeartbeatDispatcher {
    */
   public static class CommandQueueReportFromDatanode
       extends ReportFromDatanode<CommandQueueReportProto> {
+
+    private final Map<SCMCommandProto.Type, Integer> commandsToBeSent;
     public CommandQueueReportFromDatanode(DatanodeDetails datanodeDetails,
-                                          CommandQueueReportProto report) {
+        CommandQueueReportProto report,
+        Map<SCMCommandProto.Type, Integer> commandsToBeSent) {
       super(datanodeDetails, report);
+      this.commandsToBeSent = commandsToBeSent;
+    }
+
+    public Map<SCMCommandProto.Type, Integer> getCommandsToBeSent() {
+      return commandsToBeSent;
     }
   }
 
@@ -265,11 +273,37 @@ public final class SCMDatanodeHeartbeatDispatcher {
     }
   }
 
+  /**
+   * Container report payload base reference.
+   */
+  public interface ContainerReport {
+    DatanodeDetails getDatanodeDetails();
+    ContainerReportType getType();
+  }
+
+  /**
+   * Container Report Type.
+   */
+  public enum ContainerReportType {
+    /**
+     * Incremental container report type
+     * {@liks IncrementalContainerReportFromDatanode}.
+     */
+    ICR,
+    /**
+     * Full container report type
+     * {@liks ContainerReportFromDatanode}.
+     */
+    FCR
+  }
+
   /**
    * Container report event payload with origin.
    */
   public static class ContainerReportFromDatanode
-      extends ReportFromDatanode<ContainerReportsProto> {
+      extends ReportFromDatanode<ContainerReportsProto>
+      implements ContainerReport, IEventInfo {
+    private long createTime = Time.monotonicNow();
 
     public ContainerReportFromDatanode(DatanodeDetails datanodeDetails,
         ContainerReportsProto report) {
@@ -285,14 +319,31 @@ public final class SCMDatanodeHeartbeatDispatcher {
     public int hashCode() {
       return this.getDatanodeDetails().getUuid().hashCode();
     }
+    
+    public ContainerReportType getType() {
+      return ContainerReportType.FCR;
+    }
+    
+    @Override
+    public long getCreateTime() {
+      return createTime;
+    }
+
+    @Override
+    public String getEventId() {
+      return getDatanodeDetails().toString() + ", {type: " + getType()
+          + ", size: " + getReport().getReportsList().size() + "}";
+    }
   }
 
   /**
    * Incremental Container report event payload with origin.
    */
   public static class IncrementalContainerReportFromDatanode
-      extends ReportFromDatanode<IncrementalContainerReportProto> {
-
+      extends ReportFromDatanode<IncrementalContainerReportProto>
+      implements ContainerReport, IEventInfo {
+    private long createTime = Time.monotonicNow();
+    
     public IncrementalContainerReportFromDatanode(
         DatanodeDetails datanodeDetails,
         IncrementalContainerReportProto report) {
@@ -308,6 +359,21 @@ public final class SCMDatanodeHeartbeatDispatcher {
     public int hashCode() {
       return this.getDatanodeDetails().getUuid().hashCode();
     }
+
+    public ContainerReportType getType() {
+      return ContainerReportType.ICR;
+    }
+
+    @Override
+    public long getCreateTime() {
+      return createTime;
+    }
+    
+    @Override
+    public String getEventId() {
+      return getDatanodeDetails().toString() + ", {type: " + getType()
+          + ", size: " + getReport().getReportList().size() + "}";
+    }
   }
 
   /**
