@@ -309,6 +309,10 @@ public class MockNodeManager implements NodeManager {
       return deadNodes;
     }
 
+    if (nodestate == null) {
+      return new ArrayList<>(nodeMetricMap.keySet());
+    }
+
     return null;
   }
 
@@ -590,18 +594,6 @@ public class MockNodeManager implements NodeManager {
     // do nothing
   }
 
-  /**
-   * Process the Command Queue Report sent from datanodes as part of the
-   * heartbeat message.
-   * @param datanodeDetails
-   * @param commandReport
-   */
-  @Override
-  public void processNodeCommandQueueReport(DatanodeDetails datanodeDetails,
-      CommandQueueReportProto commandReport) {
-    // do nothing.
-  }
-
   /**
    * Get the number of commands of the given type queued on the datanode at the
    * last heartbeat. If the Datanode has not reported information for the given
@@ -615,6 +607,42 @@ public class MockNodeManager implements NodeManager {
     return -1;
   }
 
+  /**
+   * Get the number of commands of the given type queued in the SCM CommandQueue
+   * for the given datanode.
+   * @param dnID The UUID of the datanode.
+   * @param cmdType The Type of command to query the current count for.
+   * @return The count of commands queued, or zero if none.
+   */
+  @Override
+  public int getCommandQueueCount(UUID dnID, SCMCommandProto.Type cmdType) {
+    return 0;
+  }
+
+  /**
+   * Get the total number of pending commands of the given type on the given
+   * datanode. This includes both the number of commands queued in SCM which
+   * will be sent to the datanode on the next heartbeat, and the number of
+   * commands reported by the datanode in the last heartbeat.
+   * If the datanode has not reported any information for the given command,
+   * zero is assumed.
+   * @param datanodeDetails The datanode to query.
+   * @param cmdType The command Type To query.
+   * @return The number of commands of the given type pending on the datanode.
+   * @throws NodeNotFoundException
+   */
+  @Override
+  public int getTotalDatanodeCommandCount(DatanodeDetails datanodeDetails,
+      SCMCommandProto.Type cmdType) throws NodeNotFoundException {
+    return 0;
+  }
+
+  @Override
+  public Map<SCMCommandProto.Type, Integer> getTotalDatanodeCommandCounts(
+      DatanodeDetails datanodeDetails, SCMCommandProto.Type... cmdType) {
+    return Collections.emptyMap();
+  }
+
   /**
    * Update set of containers available on a datanode.
    * @param uuid - DatanodeID
@@ -745,11 +773,13 @@ public class MockNodeManager implements NodeManager {
    *
    * @param datanodeDetails - Datanode ID.
    * @param layoutInfo - DataNode Layout info
+   * @param commandQueueReportProto - Command Queue Report Proto
    * @return SCMheartbeat response list
    */
   @Override
   public List<SCMCommand> processHeartbeat(DatanodeDetails datanodeDetails,
-                                           LayoutVersionProto layoutInfo) {
+      LayoutVersionProto layoutInfo,
+      CommandQueueReportProto commandQueueReportProto) {
     return null;
   }
 
@@ -788,6 +818,11 @@ public class MockNodeManager implements NodeManager {
     return nodeInfo;
   }
 
+  @Override
+  public Map<String, Map<String, String>> getNodeStatusInfo() {
+    return null;
+  }
+
   /**
    * Makes it easy to add a container.
    *
@@ -870,6 +905,11 @@ public class MockNodeManager implements NodeManager {
     return numHealthyDisksPerDatanode;
   }
 
+  @Override
+  public int totalHealthyVolumeCount() {
+    return healthyNodes.size() * numHealthyDisksPerDatanode;
+  }
+
   @Override
   public int pipelineLimit(DatanodeDetails dn) {
     // by default 1 single node pipeline and 1 three node pipeline
@@ -882,6 +922,11 @@ public class MockNodeManager implements NodeManager {
     return numPipelinePerDatanode;
   }
 
+  @Override
+  public long getLastHeartbeat(DatanodeDetails datanodeDetails) {
+    return -1;
+  }
+
   public void setNumPipelinePerDatanode(int value) {
     numPipelinePerDatanode = value;
   }
