@@ -19,6 +19,8 @@
 package org.apache.hadoop.hdds.scm.container.balancer;
 
 import com.google.protobuf.ByteString;
+import org.apache.hadoop.conf.StorageUnit;
+import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
@@ -37,7 +39,6 @@ import org.apache.hadoop.hdds.scm.container.MockNodeManager;
 import org.apache.hadoop.hdds.scm.container.placement.algorithms.ContainerPlacementPolicyFactory;
 import org.apache.hadoop.hdds.scm.container.placement.algorithms.SCMContainerPlacementMetrics;
 import org.apache.hadoop.hdds.scm.container.placement.metrics.SCMNodeStat;
-import org.apache.hadoop.hdds.scm.container.replication.LegacyReplicationManager.MoveResult;
 import org.apache.hadoop.hdds.scm.container.replication.ReplicationManager;
 import org.apache.hadoop.hdds.scm.ha.SCMContext;
 import org.apache.hadoop.hdds.scm.ha.SCMService;
@@ -52,6 +53,7 @@ import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.ozone.test.GenericTestUtils;
+import org.apache.ozone.test.tag.Unhealthy;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
@@ -61,7 +63,9 @@ import org.slf4j.LoggerFactory;
 import org.slf4j.event.Level;
 
 import java.io.IOException;
+import java.time.Clock;
 import java.time.Duration;
+import java.time.ZoneId;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -70,10 +74,13 @@ import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ThreadLocalRandom;
+import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
 import java.util.stream.Collectors;
 
+import static org.apache.hadoop.hdds.scm.container.replication.ReplicationManager.ReplicationManagerConfiguration;
 import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.Mockito.atLeastOnce;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.when;
@@ -87,11 +94,13 @@ public class TestContainerBalancerTask {
       LoggerFactory.getLogger(TestContainerBalancerTask.class);
 
   private ReplicationManager replicationManager;
+  private MoveManager moveManager;
   private ContainerManager containerManager;
   private ContainerBalancerTask containerBalancerTask;
   private MockNodeManager mockNodeManager;
   private StorageContainerManager scm;
   private OzoneConfiguration conf;
+  private ReplicationManagerConfiguration rmConf;
   private PlacementPolicy placementPolicy;
   private PlacementPolicy ecPlacementPolicy;
   private PlacementPolicyValidateProxy placementPolicyValidateProxy;
@@ -118,12 +127,24 @@ public class TestContainerBalancerTask {
   public void setup() throws IOException, NodeNotFoundException,
       TimeoutException {
     conf = new OzoneConfiguration();
+    rmConf = new ReplicationManagerConfiguration();
     scm = Mockito.mock(StorageContainerManager.class);
     containerManager = Mockito.mock(ContainerManager.class);
     replicationManager = Mockito.mock(ReplicationManager.class);
     serviceStateManager = Mockito.mock(StatefulServiceStateManagerImpl.class);
     SCMServiceManager scmServiceManager = Mockito.mock(SCMServiceManager.class);
+    moveManager = Mockito.mock(MoveManager.class);
+    Mockito.when(moveManager.move(any(ContainerID.class),
+            any(DatanodeDetails.class), any(DatanodeDetails.class)))
+        .thenReturn(CompletableFuture.completedFuture(
+            MoveManager.MoveResult.COMPLETED));
 
+    /*
+    Disable LegacyReplicationManager. This means balancer should select RATIS
+     as well as EC containers for balancing. Also, MoveManager will be used.
+     */
+    Mockito.when(replicationManager.getConfig()).thenReturn(rmConf);
+    rmConf.setEnableLegacy(false);
     // these configs will usually be specified in each test
     balancerConfiguration =
         conf.getObject(ContainerBalancerConfiguration.class);
@@ -133,7 +154,7 @@ public class TestContainerBalancerTask {
     balancerConfiguration.setMaxSizeToMovePerIteration(50 * STORAGE_UNIT);
     balancerConfiguration.setMaxSizeEnteringTarget(50 * STORAGE_UNIT);
     conf.setFromObject(balancerConfiguration);
-    GenericTestUtils.setLogLevel(ContainerBalancer.LOG, Level.DEBUG);
+    GenericTestUtils.setLogLevel(ContainerBalancerTask.LOG, Level.DEBUG);
 
     averageUtilization = createCluster();
     mockNodeManager = new MockNodeManager(datanodeToContainersMap);
@@ -156,7 +177,11 @@ public class TestContainerBalancerTask {
     Mockito.when(replicationManager.move(Mockito.any(ContainerID.class),
         Mockito.any(DatanodeDetails.class),
         Mockito.any(DatanodeDetails.class)))
-        .thenReturn(CompletableFuture.completedFuture(MoveResult.COMPLETED));
+        .thenReturn(CompletableFuture.
+            completedFuture(MoveManager.MoveResult.COMPLETED));
+
+    Mockito.when(replicationManager.getClock())
+        .thenReturn(Clock.system(ZoneId.systemDefault()));
 
     when(containerManager.getContainerReplicas(Mockito.any(ContainerID.class)))
         .thenAnswer(invocationOnMock -> {
@@ -185,6 +210,7 @@ public class TestContainerBalancerTask {
     when(scm.getSCMServiceManager()).thenReturn(scmServiceManager);
     when(scm.getPlacementPolicyValidateProxy())
         .thenReturn(placementPolicyValidateProxy);
+    when(scm.getMoveManager()).thenReturn(moveManager);
 
     /*
     When StatefulServiceStateManager#saveConfiguration is called, save to
@@ -209,7 +235,7 @@ public class TestContainerBalancerTask {
         .register(Mockito.any(SCMService.class));
     ContainerBalancer sb = new ContainerBalancer(scm);
     containerBalancerTask = new ContainerBalancerTask(scm, 0, sb,
-        sb.getMetrics(), null);
+        sb.getMetrics(), balancerConfiguration, false);
   }
 
   @Test
@@ -245,7 +271,7 @@ public class TestContainerBalancerTask {
 
       balancerConfiguration.setThreshold(randomThreshold);
       containerBalancerTask = new ContainerBalancerTask(scm, 0, sb,
-          sb.getMetrics(), balancerConfiguration);
+          sb.getMetrics(), balancerConfiguration, false);
       containerBalancerTask.run();
 
       unBalancedNodesAccordingToBalancer =
@@ -263,6 +289,23 @@ public class TestContainerBalancerTask {
     }
   }
 
+  @Test
+  public void testBalancerWithMoveManager()
+      throws IllegalContainerBalancerStateException, IOException,
+      InvalidContainerBalancerConfigurationException, TimeoutException,
+      NodeNotFoundException {
+    rmConf.setEnableLegacy(false);
+    startBalancer(balancerConfiguration);
+    Mockito.verify(moveManager, atLeastOnce())
+        .move(Mockito.any(ContainerID.class),
+            Mockito.any(DatanodeDetails.class),
+            Mockito.any(DatanodeDetails.class));
+
+    Mockito.verify(replicationManager, times(0))
+        .move(Mockito.any(ContainerID.class), Mockito.any(
+            DatanodeDetails.class), Mockito.any(DatanodeDetails.class));
+  }
+
   /**
    * Checks whether the list of unBalanced nodes is empty when the cluster is
    * balanced.
@@ -290,7 +333,7 @@ public class TestContainerBalancerTask {
   public void containerBalancerShouldObeyMaxDatanodesToInvolveLimit()
       throws IllegalContainerBalancerStateException, IOException,
       InvalidContainerBalancerConfigurationException, TimeoutException {
-    int percent = 20;
+    int percent = 40;
     balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(
         percent);
     balancerConfiguration.setMaxSizeToMovePerIteration(100 * STORAGE_UNIT);
@@ -298,7 +341,6 @@ public class TestContainerBalancerTask {
     balancerConfiguration.setIterations(1);
     startBalancer(balancerConfiguration);
 
-
     int number = percent * numberOfNodes / 100;
     ContainerBalancerMetrics metrics = containerBalancerTask.getMetrics();
     Assertions.assertFalse(
@@ -351,6 +393,47 @@ public class TestContainerBalancerTask {
     }
   }
 
+  /**
+   * Container Balancer should not select a non-CLOSED replica for moving.
+   */
+  @Test
+  public void balancerShouldNotSelectNonClosedContainerReplicas()
+      throws IOException, IllegalContainerBalancerStateException,
+      InvalidContainerBalancerConfigurationException, TimeoutException {
+
+    // let's mock such that all replicas have CLOSING state
+    when(containerManager.getContainerReplicas(Mockito.any(ContainerID.class)))
+        .thenAnswer(invocationOnMock -> {
+          ContainerID cid = (ContainerID) invocationOnMock.getArguments()[0];
+          Set<ContainerReplica> replicas = cidToReplicasMap.get(cid);
+          Set<ContainerReplica> replicasToReturn =
+              new HashSet<>(replicas.size());
+          for (ContainerReplica replica : replicas) {
+            ContainerReplica newReplica =
+                replica.toBuilder().setContainerState(
+                    ContainerReplicaProto.State.CLOSING).build();
+            replicasToReturn.add(newReplica);
+          }
+
+          return replicasToReturn;
+        });
+
+    balancerConfiguration.setThreshold(10);
+    balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(100);
+    balancerConfiguration.setMaxSizeToMovePerIteration(50 * STORAGE_UNIT);
+    balancerConfiguration.setMaxSizeEnteringTarget(50 * STORAGE_UNIT);
+
+    startBalancer(balancerConfiguration);
+    stopBalancer();
+
+    // balancer should have identified unbalanced nodes
+    Assertions.assertFalse(containerBalancerTask.getUnBalancedNodes()
+        .isEmpty());
+    // no container should have moved because all replicas are CLOSING
+    Assertions.assertTrue(
+        containerBalancerTask.getContainerToSourceMap().isEmpty());
+  }
+
   @Test
   public void containerBalancerShouldObeyMaxSizeToMoveLimit()
       throws IllegalContainerBalancerStateException, IOException,
@@ -426,9 +509,16 @@ public class TestContainerBalancerTask {
       replicas.add(containerToTargetMap.get(container));
 
       ContainerInfo containerInfo = cidToInfoMap.get(container);
-      ContainerPlacementStatus placementStatus =
-          placementPolicy.validateContainerPlacement(replicas,
-              containerInfo.getReplicationConfig().getRequiredNodes());
+      ContainerPlacementStatus placementStatus;
+      if (containerInfo.getReplicationType() ==
+          HddsProtos.ReplicationType.RATIS) {
+        placementStatus = placementPolicy.validateContainerPlacement(replicas,
+            containerInfo.getReplicationConfig().getRequiredNodes());
+      } else {
+        placementStatus =
+            ecPlacementPolicy.validateContainerPlacement(replicas,
+                containerInfo.getReplicationConfig().getRequiredNodes());
+      }
       Assertions.assertTrue(placementStatus.isPolicySatisfied());
     }
   }
@@ -464,6 +554,8 @@ public class TestContainerBalancerTask {
     balancerConfiguration.setMaxSizeToMovePerIteration(50 * STORAGE_UNIT);
     balancerConfiguration.setMaxSizeEnteringTarget(50 * STORAGE_UNIT);
     balancerConfiguration.setIterations(1);
+    rmConf.setEnableLegacy(true);
+
     startBalancer(balancerConfiguration);
 
     stopBalancer();
@@ -481,6 +573,18 @@ public class TestContainerBalancerTask {
     Mockito.verify(replicationManager, times(numContainers))
         .move(any(ContainerID.class), any(DatanodeDetails.class),
             any(DatanodeDetails.class));
+
+    /*
+     Try the same test by disabling LegacyReplicationManager so that
+     MoveManager is used.
+     */
+    rmConf.setEnableLegacy(false);
+    startBalancer(balancerConfiguration);
+    stopBalancer();
+    numContainers = containerBalancerTask.getContainerToTargetMap().size();
+    Mockito.verify(moveManager, times(numContainers))
+        .move(any(ContainerID.class), any(DatanodeDetails.class),
+            any(DatanodeDetails.class));
   }
 
   @Test
@@ -532,7 +636,7 @@ public class TestContainerBalancerTask {
     cbc.setBalancingInterval(1);
     ContainerBalancer sb = new ContainerBalancer(scm);
     containerBalancerTask = new ContainerBalancerTask(scm, 0, sb,
-        sb.getMetrics(), cbc);
+        sb.getMetrics(), cbc, false);
     containerBalancerTask.run();
 
     stopBalancer();
@@ -544,9 +648,51 @@ public class TestContainerBalancerTask {
   }
 
   @Test
-  public void testMetrics()
+  public void balancerShouldObeyMaxSizeLeavingSourceLimit()
       throws IllegalContainerBalancerStateException, IOException,
       InvalidContainerBalancerConfigurationException, TimeoutException {
+    conf.set("ozone.scm.container.size", "1MB");
+    balancerConfiguration =
+        conf.getObject(ContainerBalancerConfiguration.class);
+    balancerConfiguration.setThreshold(10);
+    balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(100);
+    balancerConfiguration.setMaxSizeToMovePerIteration(50 * STORAGE_UNIT);
+
+    // no source containers should be selected when the limit is just 2 MB
+    balancerConfiguration.setMaxSizeLeavingSource(2 * OzoneConsts.MB);
+    startBalancer(balancerConfiguration);
+
+    Assertions.assertFalse(containerBalancerTask.getUnBalancedNodes()
+        .isEmpty());
+    Assertions.assertTrue(containerBalancerTask.getContainerToSourceMap()
+        .isEmpty());
+    stopBalancer();
+
+    // some containers should be selected when using default values
+    OzoneConfiguration ozoneConfiguration = new OzoneConfiguration();
+    ContainerBalancerConfiguration cbc = ozoneConfiguration.
+        getObject(ContainerBalancerConfiguration.class);
+    cbc.setBalancingInterval(1);
+    ContainerBalancer sb = new ContainerBalancer(scm);
+    containerBalancerTask = new ContainerBalancerTask(scm, 0, sb,
+        sb.getMetrics(), cbc, false);
+    containerBalancerTask.run();
+
+    stopBalancer();
+    // balancer should have identified unbalanced nodes
+    Assertions.assertFalse(containerBalancerTask.getUnBalancedNodes()
+        .isEmpty());
+    Assertions.assertFalse(containerBalancerTask.getContainerToSourceMap()
+        .isEmpty());
+    Assertions.assertTrue(0 !=
+        containerBalancerTask.getSizeScheduledForMoveInLatestIteration());
+  }
+
+  @Test
+  public void testMetrics()
+      throws IllegalContainerBalancerStateException, IOException,
+      InvalidContainerBalancerConfigurationException, TimeoutException,
+      NodeNotFoundException {
     conf.set("hdds.datanode.du.refresh.period", "1ms");
     balancerConfiguration.setBalancingInterval(Duration.ofMillis(2));
     balancerConfiguration.setThreshold(10);
@@ -555,6 +701,11 @@ public class TestContainerBalancerTask {
     // deliberately set max size per iteration to a low value, 6 GB
     balancerConfiguration.setMaxSizeToMovePerIteration(6 * STORAGE_UNIT);
     balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(100);
+    Mockito.when(moveManager.move(any(), any(), any()))
+           .thenReturn(CompletableFuture.completedFuture(
+               MoveManager.MoveResult.REPLICATION_FAIL_NODE_UNHEALTHY))
+           .thenReturn(CompletableFuture.completedFuture(
+               MoveManager.MoveResult.COMPLETED));
 
     startBalancer(balancerConfiguration);
     stopBalancer();
@@ -564,7 +715,18 @@ public class TestContainerBalancerTask {
             balancerConfiguration.getThreshold()).size(),
         metrics.getNumDatanodesUnbalanced());
     Assertions.assertTrue(metrics.getDataSizeMovedGBInLatestIteration() <= 6);
+    Assertions.assertTrue(metrics.getDataSizeMovedGB() > 0);
     Assertions.assertEquals(1, metrics.getNumIterations());
+    Assertions.assertTrue(
+        metrics.getNumContainerMovesScheduledInLatestIteration() > 0);
+    Assertions.assertEquals(metrics.getNumContainerMovesScheduled(),
+        metrics.getNumContainerMovesScheduledInLatestIteration());
+    Assertions.assertEquals(metrics.getNumContainerMovesScheduled(),
+        metrics.getNumContainerMovesCompleted() +
+            metrics.getNumContainerMovesFailed() +
+            metrics.getNumContainerMovesTimeout());
+    Assertions.assertEquals(0, metrics.getNumContainerMovesTimeout());
+    Assertions.assertEquals(1, metrics.getNumContainerMovesFailed());
   }
 
   /**
@@ -638,16 +800,28 @@ public class TestContainerBalancerTask {
     ozoneConfiguration.set("ozone.scm.container.size", "5GB");
     ozoneConfiguration.setDouble(
         "hdds.container.balancer.utilization.threshold", 1);
+    long maxSizeLeavingSource = 26;
+    ozoneConfiguration.setStorageSize(
+        "hdds.container.balancer.size.leaving.source.max", maxSizeLeavingSource,
+        StorageUnit.GB);
+    long moveTimeout = 90;
+    ozoneConfiguration.setTimeDuration("hdds.container.balancer.move.timeout",
+        moveTimeout, TimeUnit.MINUTES);
+    long replicationTimeout = 60;
+    ozoneConfiguration.setTimeDuration(
+        "hdds.container.balancer.move.replication.timeout",
+        replicationTimeout, TimeUnit.MINUTES);
 
     ContainerBalancerConfiguration cbConf =
         ozoneConfiguration.getObject(ContainerBalancerConfiguration.class);
     Assertions.assertEquals(1, cbConf.getThreshold(), 0.001);
 
-    Assertions.assertEquals(26 * 1024 * 1024 * 1024L,
+    // Expected is 26 GB
+    Assertions.assertEquals(maxSizeLeavingSource * 1024 * 1024 * 1024,
         cbConf.getMaxSizeLeavingSource());
-
-    Assertions.assertEquals(30 * 60 * 1000,
-        cbConf.getMoveTimeout().toMillis());
+    Assertions.assertEquals(moveTimeout, cbConf.getMoveTimeout().toMinutes());
+    Assertions.assertEquals(replicationTimeout,
+        cbConf.getMoveReplicationTimeout().toMinutes());
   }
 
   @Test
@@ -661,6 +835,7 @@ public class TestContainerBalancerTask {
     balancerConfiguration.setMaxSizeEnteringTarget(10 * STORAGE_UNIT);
     balancerConfiguration.setMaxSizeToMovePerIteration(100 * STORAGE_UNIT);
     balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(100);
+    rmConf.setEnableLegacy(true);
 
     startBalancer(balancerConfiguration);
 
@@ -681,7 +856,7 @@ public class TestContainerBalancerTask {
             Mockito.any(DatanodeDetails.class),
             Mockito.any(DatanodeDetails.class)))
         .thenReturn(CompletableFuture.completedFuture(
-            MoveResult.REPLICATION_FAIL_NODE_UNHEALTHY));
+            MoveManager.MoveResult.REPLICATION_FAIL_NODE_UNHEALTHY));
     balancerConfiguration.setMaxSizeToMovePerIteration(10 * STORAGE_UNIT);
 
     startBalancer(balancerConfiguration);
@@ -690,8 +865,22 @@ public class TestContainerBalancerTask {
         ContainerBalancerTask.IterationResult.ITERATION_COMPLETED,
         containerBalancerTask.getIterationResult());
     stopBalancer();
+
+    /*
+    Try the same but use MoveManager for container move instead of legacy RM.
+     */
+    rmConf.setEnableLegacy(false);
+    startBalancer(balancerConfiguration);
+    Assertions.assertEquals(
+        ContainerBalancerTask.IterationResult.ITERATION_COMPLETED,
+        containerBalancerTask.getIterationResult());
+    stopBalancer();
   }
 
+  /**
+   * Tests the situation where some container moves time out because they
+   * take longer than "move.timeout".
+   */
   @Test
   public void checkIterationResultTimeout()
       throws NodeNotFoundException, IOException,
@@ -699,10 +888,13 @@ public class TestContainerBalancerTask {
       InvalidContainerBalancerConfigurationException,
       TimeoutException {
 
+    CompletableFuture<MoveManager.MoveResult> completedFuture =
+        CompletableFuture.completedFuture(MoveManager.MoveResult.COMPLETED);
     Mockito.when(replicationManager.move(Mockito.any(ContainerID.class),
             Mockito.any(DatanodeDetails.class),
             Mockito.any(DatanodeDetails.class)))
-        .thenReturn(genCompletableFuture(200), genCompletableFuture(2000));
+        .thenReturn(completedFuture)
+        .thenAnswer(invocation -> genCompletableFuture(2000));
 
     balancerConfiguration.setThreshold(10);
     balancerConfiguration.setIterations(1);
@@ -710,7 +902,7 @@ public class TestContainerBalancerTask {
     balancerConfiguration.setMaxSizeToMovePerIteration(100 * STORAGE_UNIT);
     balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(100);
     balancerConfiguration.setMoveTimeout(Duration.ofMillis(500));
-
+    rmConf.setEnableLegacy(true);
     startBalancer(balancerConfiguration);
 
     /*
@@ -727,6 +919,28 @@ public class TestContainerBalancerTask {
             .getNumContainerMovesTimeoutInLatestIteration() > 1);
     stopBalancer();
 
+    /*
+    Test the same but use MoveManager instead of LegacyReplicationManager.
+    The first move being 10ms falls within the timeout duration of 500ms. It
+    should be successful. The rest should fail.
+     */
+    rmConf.setEnableLegacy(false);
+    Mockito.when(moveManager.move(Mockito.any(ContainerID.class),
+            Mockito.any(DatanodeDetails.class),
+            Mockito.any(DatanodeDetails.class)))
+        .thenReturn(completedFuture)
+        .thenAnswer(invocation -> genCompletableFuture(2000));
+
+    startBalancer(balancerConfiguration);
+    Assertions.assertEquals(
+        ContainerBalancerTask.IterationResult.ITERATION_COMPLETED,
+        containerBalancerTask.getIterationResult());
+    Assertions.assertEquals(1,
+        containerBalancerTask.getMetrics()
+            .getNumContainerMovesCompletedInLatestIteration());
+    Assertions.assertTrue(containerBalancerTask.getMetrics()
+        .getNumContainerMovesTimeoutInLatestIteration() > 1);
+    stopBalancer();
   }
 
   @Test
@@ -734,12 +948,12 @@ public class TestContainerBalancerTask {
       throws NodeNotFoundException, IOException,
       IllegalContainerBalancerStateException,
       InvalidContainerBalancerConfigurationException, TimeoutException {
-    CompletableFuture<MoveResult> future
+    CompletableFuture<MoveManager.MoveResult> future
         = CompletableFuture.supplyAsync(() ->
-        MoveResult.REPLICATION_FAIL_TIME_OUT);
-    CompletableFuture<MoveResult> future2
+        MoveManager.MoveResult.REPLICATION_FAIL_TIME_OUT);
+    CompletableFuture<MoveManager.MoveResult> future2
         = CompletableFuture.supplyAsync(() ->
-        MoveResult.DELETION_FAIL_TIME_OUT);
+        MoveManager.MoveResult.DELETION_FAIL_TIME_OUT);
     Mockito.when(replicationManager.move(Mockito.any(ContainerID.class),
             Mockito.any(DatanodeDetails.class),
             Mockito.any(DatanodeDetails.class)))
@@ -751,11 +965,29 @@ public class TestContainerBalancerTask {
     balancerConfiguration.setMaxSizeToMovePerIteration(100 * STORAGE_UNIT);
     balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(100);
     balancerConfiguration.setMoveTimeout(Duration.ofMillis(500));
-
+    rmConf.setEnableLegacy(true);
     startBalancer(balancerConfiguration);
 
     Assertions.assertTrue(containerBalancerTask.getMetrics()
         .getNumContainerMovesTimeoutInLatestIteration() > 0);
+    Assertions.assertEquals(0, containerBalancerTask.getMetrics()
+        .getNumContainerMovesCompletedInLatestIteration());
+    stopBalancer();
+
+    /*
+    Try the same test with MoveManager instead of LegacyReplicationManager.
+     */
+    Mockito.when(moveManager.move(Mockito.any(ContainerID.class),
+            Mockito.any(DatanodeDetails.class),
+            Mockito.any(DatanodeDetails.class)))
+        .thenReturn(future).thenAnswer(invocation -> future2);
+
+    rmConf.setEnableLegacy(false);
+    startBalancer(balancerConfiguration);
+    Assertions.assertTrue(containerBalancerTask.getMetrics()
+        .getNumContainerMovesTimeoutInLatestIteration() > 0);
+    Assertions.assertEquals(0, containerBalancerTask.getMetrics()
+        .getNumContainerMovesCompletedInLatestIteration());
     stopBalancer();
   }
 
@@ -766,20 +998,21 @@ public class TestContainerBalancerTask {
       InvalidContainerBalancerConfigurationException,
       TimeoutException {
 
-    CompletableFuture<MoveResult> f = new CompletableFuture();
-    f.completeExceptionally(new RuntimeException("Runtime Exception"));
+    CompletableFuture<MoveManager.MoveResult> future =
+        new CompletableFuture<>();
+    future.completeExceptionally(new RuntimeException("Runtime Exception"));
     Mockito.when(replicationManager.move(Mockito.any(ContainerID.class),
             Mockito.any(DatanodeDetails.class),
             Mockito.any(DatanodeDetails.class)))
-        .thenThrow(new ContainerNotFoundException("Test Container not found"),
-            new NodeNotFoundException("Test Node not found"))
-        .thenReturn(f).thenReturn(CompletableFuture.supplyAsync(() -> {
+        .thenReturn(CompletableFuture.supplyAsync(() -> {
           try {
-            Thread.sleep(200);
-          } catch (Exception ex) {
+            Thread.sleep(1);
+          } catch (Exception ignored) {
           }
-          throw new RuntimeException("Throw");
-        }));
+          throw new RuntimeException("Runtime Exception after doing work");
+        }))
+        .thenThrow(new ContainerNotFoundException("Test Container not found"))
+        .thenReturn(future);
 
     balancerConfiguration.setThreshold(10);
     balancerConfiguration.setIterations(1);
@@ -787,6 +1020,7 @@ public class TestContainerBalancerTask {
     balancerConfiguration.setMaxSizeToMovePerIteration(100 * STORAGE_UNIT);
     balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(100);
     balancerConfiguration.setMoveTimeout(Duration.ofMillis(500));
+    rmConf.setEnableLegacy(true);
 
     startBalancer(balancerConfiguration);
 
@@ -798,6 +1032,101 @@ public class TestContainerBalancerTask {
             .getNumContainerMovesFailed() >= 3);
     stopBalancer();
 
+    /*
+    Try the same test but with MoveManager instead of ReplicationManager.
+     */
+    Mockito.when(moveManager.move(Mockito.any(ContainerID.class),
+            Mockito.any(DatanodeDetails.class),
+            Mockito.any(DatanodeDetails.class)))
+        .thenReturn(CompletableFuture.supplyAsync(() -> {
+          try {
+            Thread.sleep(1);
+          } catch (Exception ignored) {
+          }
+          throw new RuntimeException("Runtime Exception after doing work");
+        }))
+        .thenThrow(new ContainerNotFoundException("Test Container not found"))
+        .thenReturn(future);
+
+    rmConf.setEnableLegacy(false);
+    startBalancer(balancerConfiguration);
+    Assertions.assertEquals(
+        ContainerBalancerTask.IterationResult.ITERATION_COMPLETED,
+        containerBalancerTask.getIterationResult());
+    Assertions.assertTrue(
+        containerBalancerTask.getMetrics()
+            .getNumContainerMovesFailed() >= 3);
+    stopBalancer();
+  }
+
+  @Unhealthy("HDDS-8941")
+  @Test
+  public void testDelayedStart() throws InterruptedException, TimeoutException {
+    conf.setTimeDuration("hdds.scm.wait.time.after.safemode.exit", 10,
+        TimeUnit.SECONDS);
+    ContainerBalancer balancer = new ContainerBalancer(scm);
+    containerBalancerTask = new ContainerBalancerTask(scm, 2, balancer,
+        balancer.getMetrics(), balancerConfiguration, true);
+    Thread balancingThread = new Thread(containerBalancerTask);
+    // start the thread and assert that balancer is RUNNING
+    balancingThread.start();
+    Assertions.assertEquals(ContainerBalancerTask.Status.RUNNING,
+        containerBalancerTask.getBalancerStatus());
+
+    /*
+     Wait for the thread to start sleeping and assert that it's sleeping.
+     This is the delay before it starts balancing.
+     */
+    GenericTestUtils.waitFor(
+        () -> balancingThread.getState() == Thread.State.TIMED_WAITING, 1, 20);
+    Assertions.assertEquals(Thread.State.TIMED_WAITING,
+        balancingThread.getState());
+
+    // interrupt the thread from its sleep, wait and assert that balancer has
+    // STOPPED
+    balancingThread.interrupt();
+    GenericTestUtils.waitFor(() -> containerBalancerTask.getBalancerStatus() ==
+        ContainerBalancerTask.Status.STOPPED, 1, 20);
+    Assertions.assertEquals(ContainerBalancerTask.Status.STOPPED,
+        containerBalancerTask.getBalancerStatus());
+
+    // ensure the thread dies
+    GenericTestUtils.waitFor(() -> !balancingThread.isAlive(), 1, 20);
+    Assertions.assertFalse(balancingThread.isAlive());
+  }
+
+  /**
+   * The expectation is that only RATIS containers should be selected for
+   * balancing when LegacyReplicationManager is enabled. This is because
+   * LegacyReplicationManager does not support moving EC containers.
+   */
+  @Test
+  public void balancerShouldExcludeECContainersWhenLegacyRmIsEnabled()
+      throws IllegalContainerBalancerStateException, IOException,
+      InvalidContainerBalancerConfigurationException, TimeoutException {
+    // Enable LegacyReplicationManager
+    rmConf.setEnableLegacy(true);
+    balancerConfiguration.setThreshold(10);
+    balancerConfiguration.setIterations(1);
+    balancerConfiguration.setMaxSizeEnteringTarget(10 * STORAGE_UNIT);
+    balancerConfiguration.setMaxSizeToMovePerIteration(100 * STORAGE_UNIT);
+    balancerConfiguration.setMaxDatanodesPercentageToInvolvePerIteration(100);
+
+    startBalancer(balancerConfiguration);
+
+    /*
+     Get all containers that were selected by balancer and assert none of
+     them is an EC container.
+     */
+    Map<ContainerID, DatanodeDetails> containerToSource =
+        containerBalancerTask.getContainerToSourceMap();
+    Assertions.assertFalse(containerToSource.isEmpty());
+    for (Map.Entry<ContainerID, DatanodeDetails> entry :
+        containerToSource.entrySet()) {
+      ContainerInfo containerInfo = cidToInfoMap.get(entry.getKey());
+      Assertions.assertNotSame(HddsProtos.ReplicationType.EC,
+          containerInfo.getReplicationType());
+    }
   }
 
   /**
@@ -924,14 +1253,23 @@ public class TestContainerBalancerTask {
   }
 
   private ContainerInfo createContainer(long id, int multiple) {
-    return new ContainerInfo.Builder()
+    ContainerInfo.Builder builder = new ContainerInfo.Builder()
         .setContainerID(id)
-        .setReplicationConfig(RatisReplicationConfig
-                .getInstance(HddsProtos.ReplicationFactor.THREE))
         .setState(HddsProtos.LifeCycleState.CLOSED)
         .setOwner("TestContainerBalancer")
-        .setUsedBytes(STORAGE_UNIT * multiple)
-        .build();
+        .setUsedBytes(STORAGE_UNIT * multiple);
+
+    /*
+    Make it a RATIS container if id is even, else make it an EC container
+     */
+    if (id % 2 == 0) {
+      builder.setReplicationConfig(RatisReplicationConfig
+          .getInstance(HddsProtos.ReplicationFactor.THREE));
+    } else {
+      builder.setReplicationConfig(new ECReplicationConfig(3, 2));
+    }
+
+    return builder.build();
   }
 
   /**
@@ -948,7 +1286,8 @@ public class TestContainerBalancerTask {
 
         // randomly pick a datanode for this replica
         int datanodeIndex = RANDOM.nextInt(0, numberOfNodes);
-        if (nodeUtilizations.get(i) != 0.0d) {
+        // don't put replicas in DNs that are supposed to have 0 utilization
+        if (Math.abs(nodeUtilizations.get(datanodeIndex) - 0.0d) > 0.00001) {
           DatanodeDetails node =
               nodesInCluster.get(datanodeIndex).getDatanodeDetails();
           Set<ContainerReplica> replicas =
@@ -956,6 +1295,8 @@ public class TestContainerBalancerTask {
           replicas.add(createReplica(container.containerID(), node,
               container.getUsedBytes()));
           cidToReplicasMap.put(container.containerID(), replicas);
+          datanodeToContainersMap.get(nodesInCluster.get(datanodeIndex))
+              .add(container.containerID());
         }
       }
     }
@@ -978,6 +1319,7 @@ public class TestContainerBalancerTask {
       throws IllegalContainerBalancerStateException, IOException,
       InvalidContainerBalancerConfigurationException, TimeoutException {
     containerBalancerTask.setConfig(config);
+    containerBalancerTask.setTaskStatus(ContainerBalancerTask.Status.RUNNING);
     containerBalancerTask.run();
   }
 
@@ -985,7 +1327,7 @@ public class TestContainerBalancerTask {
     // do nothing as testcase is not threaded
   }
 
-  private CompletableFuture<MoveResult>
+  private CompletableFuture<MoveManager.MoveResult>
       genCompletableFuture(int sleepMilSec) {
     return CompletableFuture.supplyAsync(() -> {
       try {
@@ -993,7 +1335,7 @@ public class TestContainerBalancerTask {
       } catch (InterruptedException e) {
         e.printStackTrace();
       }
-      return MoveResult.COMPLETED;
+      return MoveManager.MoveResult.COMPLETED;
     });
   }
 }
