@@ -16,15 +16,22 @@
  */
 package org.apache.hadoop.hdds.scm.ha;
 
+import com.google.common.base.Preconditions;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.hdds.scm.exceptions.SCMException;
 import org.apache.hadoop.hdds.scm.metadata.SCMMetadataStore;
 import org.apache.hadoop.hdds.scm.metadata.SCMMetadataStoreImpl;
 import org.apache.hadoop.hdds.scm.metadata.SCMDBTransactionBufferImpl;
+import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.container.common.SCMTestUtils;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Test;
 
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_SEQUENCE_ID_BATCH_SIZE;
+import static org.mockito.ArgumentMatchers.anyLong;
+import static org.mockito.ArgumentMatchers.anyString;
+import static org.mockito.Mockito.spy;
+import static org.mockito.Mockito.when;
 
 /**
  * Tests for {@link SequenceIdGenerator}.
@@ -116,4 +123,53 @@ public class TestSequenceIDGenerator {
     Assertions.assertEquals(202, sequenceIdGen.getNextId("otherKey"));
     Assertions.assertEquals(203, sequenceIdGen.getNextId("otherKey"));
   }
+
+  @Test
+  public void testSequenceIDGenUponRatisWhenCurrentScmIsNotALeader()
+      throws Exception {
+    int batchSize = 100;
+    OzoneConfiguration conf = SCMTestUtils.getConf();
+    conf.setInt(OZONE_SCM_SEQUENCE_ID_BATCH_SIZE, batchSize);
+    SCMMetadataStore scmMetadataStore = new SCMMetadataStoreImpl(conf);
+    scmMetadataStore.start(conf);
+    SCMHAManager scmHAManager = SCMHAManagerStub
+        .getInstance(true, new SCMDBTransactionBufferImpl());
+
+    SequenceIdGenerator.StateManager stateManager =
+        spy(new SequenceIdGenerator.StateManagerImpl.Builder()
+            .setRatisServer(scmHAManager.getRatisServer())
+            .setDBTransactionBuffer(scmHAManager.getDBTransactionBuffer())
+            .setSequenceIdTable(scmMetadataStore.getSequenceIdTable())
+            .build());
+    SequenceIdGenerator sequenceIdGen = new SequenceIdGenerator(
+        conf, scmHAManager, scmMetadataStore.getSequenceIdTable()) {
+      @Override
+      public StateManager createStateManager(
+          SCMHAManager scmhaManager, Table<String, Long> sequenceIdTable) {
+        Preconditions.checkNotNull(scmhaManager);
+        return stateManager;
+      }
+    };
+
+    Assertions.assertEquals(1L, sequenceIdGen.getNextId("someKey"));
+
+    // Simulation currently this SCM is not a leader node,
+    // So this SCM can only allocate IDs within the current batch
+    // ([1, batchSize]), does not allow the allocation of IDs for the next batch
+    // ([batchSize + 1, batchSize * 2])
+    when(stateManager.allocateBatch(anyString(), anyLong(), anyLong()))
+        .thenThrow(new SCMException(SCMException.ResultCodes.SCM_NOT_LEADER));
+
+    for (int i = 0; i < batchSize * 3; i++) {
+      try {
+        long nextID = sequenceIdGen.getNextId("someKey");
+        if (nextID > batchSize) {
+          Assertions.fail("Should not allocate a blockID: " + nextID +
+              " that exceeds the current Batch: " + batchSize);
+        }
+      } catch (Exception e) {
+        // ignore
+      }
+    }
+  }
 }
