@@ -17,6 +17,7 @@
 
 package org.apache.hadoop.hdds.scm.pipeline;
 
+import com.google.common.collect.ImmutableList;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.MockDatanodeDetails;
@@ -39,7 +40,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.NavigableSet;
 import java.util.Set;
-import java.util.concurrent.TimeoutException;
 import java.util.stream.Collectors;
 import java.util.stream.Stream;
 
@@ -48,7 +48,7 @@ import java.util.stream.Stream;
  */
 public class MockPipelineManager implements PipelineManager {
 
-  private PipelineStateManager stateManager;
+  private final PipelineStateManager stateManager;
 
   public MockPipelineManager(DBStore dbStore, SCMHAManager scmhaManager,
                              NodeManager nodeManager) throws IOException {
@@ -62,7 +62,7 @@ public class MockPipelineManager implements PipelineManager {
 
   @Override
   public Pipeline createPipeline(ReplicationConfig replicationConfig)
-      throws IOException, TimeoutException {
+      throws IOException {
     return createPipeline(replicationConfig, Collections.emptyList(),
         Collections.emptyList());
   }
@@ -70,9 +70,30 @@ public class MockPipelineManager implements PipelineManager {
   @Override
   public Pipeline createPipeline(ReplicationConfig replicationConfig,
       List<DatanodeDetails> excludedNodes, List<DatanodeDetails> favoredNodes)
-      throws IOException, TimeoutException {
+      throws IOException {
+    Pipeline pipeline;
+    if (replicationConfig.getReplicationType()
+        == HddsProtos.ReplicationType.EC) {
+      pipeline = buildECPipeline(
+          replicationConfig, excludedNodes, favoredNodes);
+    } else {
+      pipeline = createPipeline(replicationConfig,
+          ImmutableList.of(MockDatanodeDetails.randomDatanodeDetails(),
+              MockDatanodeDetails.randomDatanodeDetails(),
+              MockDatanodeDetails.randomDatanodeDetails()));
+    }
+
+    stateManager.addPipeline(pipeline.getProtobufMessage(
+        ClientVersion.CURRENT_VERSION));
+    return pipeline;
+  }
+
+  @Override
+  public Pipeline buildECPipeline(ReplicationConfig replicationConfig,
+      List<DatanodeDetails> excludedNodes, List<DatanodeDetails> favoredNodes)
+      throws IOException {
     final List<DatanodeDetails> nodes = Stream.generate(
-        MockDatanodeDetails::randomDatanodeDetails)
+            MockDatanodeDetails::randomDatanodeDetails)
         .limit(replicationConfig.getRequiredNodes())
         .collect(Collectors.toList());
     final Pipeline pipeline = Pipeline.newBuilder()
@@ -81,10 +102,14 @@ public class MockPipelineManager implements PipelineManager {
         .setNodes(nodes)
         .setState(Pipeline.PipelineState.OPEN)
         .build();
+    return pipeline;
+  }
 
+  @Override
+  public void addEcPipeline(Pipeline pipeline)
+      throws IOException {
     stateManager.addPipeline(pipeline.getProtobufMessage(
         ClientVersion.CURRENT_VERSION));
-    return pipeline;
   }
 
   @Override
@@ -182,7 +207,7 @@ public class MockPipelineManager implements PipelineManager {
   @Override
   public void addContainerToPipelineSCMStart(
           PipelineID pipelineID, ContainerID containerID) throws IOException {
-    stateManager.addContainerToPipelineSCMStart(pipelineID, containerID);
+    stateManager.addContainerToPipelineForce(pipelineID, containerID);
   }
 
   @Override
@@ -206,18 +231,28 @@ public class MockPipelineManager implements PipelineManager {
 
   @Override
   public void openPipeline(final PipelineID pipelineId)
-      throws IOException, TimeoutException {
+      throws IOException {
     stateManager.updatePipelineState(
         pipelineId.getProtobuf(), HddsProtos.PipelineState.PIPELINE_OPEN);
   }
 
   @Override
   public void closePipeline(final Pipeline pipeline, final boolean onTimeout)
-      throws IOException, TimeoutException {
+      throws IOException {
     stateManager.updatePipelineState(pipeline.getId().getProtobuf(),
         HddsProtos.PipelineState.PIPELINE_CLOSED);
   }
 
+  @Override
+  public void closePipeline(PipelineID pipelineID) throws IOException {
+
+  }
+
+  @Override
+  public void deletePipeline(PipelineID pipelineID) throws IOException {
+
+  }
+
   @Override
   public void closeStalePipelines(DatanodeDetails datanodeDetails) {
 
@@ -260,7 +295,7 @@ public class MockPipelineManager implements PipelineManager {
 
   @Override
   public void deactivatePipeline(final PipelineID pipelineID)
-      throws IOException, TimeoutException {
+      throws IOException {
     stateManager.updatePipelineState(pipelineID.getProtobuf(),
         HddsProtos.PipelineState.PIPELINE_DORMANT);
   }
@@ -320,4 +355,4 @@ public class MockPipelineManager implements PipelineManager {
   public boolean isPipelineCreationFrozen() {
     return false;
   }
-}
\ No newline at end of file
+}
