@@ -32,7 +32,7 @@ import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerReplica;
 import org.apache.hadoop.hdds.scm.container.MockNodeManager;
-import org.apache.hadoop.hdds.scm.container.TestContainerManagerImpl;
+import org.apache.hadoop.hdds.scm.container.placement.algorithms.SCMContainerPlacementRackScatter;
 import org.apache.hadoop.hdds.scm.exceptions.SCMException;
 import org.apache.hadoop.hdds.scm.ha.SCMHAManagerStub;
 import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
@@ -42,8 +42,9 @@ import org.apache.hadoop.hdds.utils.db.DBStore;
 import org.apache.hadoop.hdds.utils.db.DBStoreBuilder;
 import org.apache.hadoop.ozone.ClientVersion;
 import org.apache.ozone.test.GenericTestUtils;
+import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.Assertions;
-import org.junit.Assume;
+import org.junit.jupiter.api.Assumptions;
 import org.junit.jupiter.api.Test;
 
 import java.io.File;
@@ -58,8 +59,10 @@ import java.util.concurrent.TimeoutException;
 import java.util.stream.Collectors;
 
 import static org.apache.commons.collections.CollectionUtils.intersection;
-import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_DATANODE_RATIS_VOLUME_FREE_SPACE_MIN;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_CONTAINER_SIZE;
+import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_DATANODE_PIPELINE_LIMIT;
+import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_DATANODE_RATIS_VOLUME_FREE_SPACE_MIN;
+import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_PIPELINE_PLACEMENT_IMPL_KEY;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
@@ -86,7 +89,7 @@ public class TestRatisPipelineProvider {
   public void init(int maxPipelinePerNode, OzoneConfiguration conf)
       throws Exception {
     testDir = GenericTestUtils.getTestDir(
-        TestContainerManagerImpl.class.getSimpleName() + UUID.randomUUID());
+        TestRatisPipelineProvider.class.getSimpleName() + UUID.randomUUID());
     conf.set(HddsConfigKeys.OZONE_METADATA_DIRS, testDir.getAbsolutePath());
     dbStore = DBStoreBuilder.createDBStore(
         conf, new SCMDBDefinition());
@@ -105,7 +108,8 @@ public class TestRatisPipelineProvider {
         stateManager, conf);
   }
 
-  private void cleanup() throws Exception {
+  @AfterEach
+  void cleanup() throws Exception {
     if (dbStore != null) {
       dbStore.close();
     }
@@ -157,14 +161,12 @@ public class TestRatisPipelineProvider {
   public void testCreatePipelineWithFactorThree() throws Exception {
     init(1);
     createPipelineAndAssertions(HddsProtos.ReplicationFactor.THREE);
-    cleanup();
   }
 
   @Test
   public void testCreatePipelineWithFactorOne() throws Exception {
     init(1);
     createPipelineAndAssertions(HddsProtos.ReplicationFactor.ONE);
-    cleanup();
   }
 
   private List<DatanodeDetails> createListOfNodes(int nodeCount) {
@@ -198,7 +200,6 @@ public class TestRatisPipelineProvider {
     // With enough pipeline quote on datanodes, they should not share
     // the same set of datanodes.
     assertNotEquals(pipeline.getNodeSet(), pipeline1.getNodeSet());
-    cleanup();
   }
 
   @Test
@@ -216,7 +217,6 @@ public class TestRatisPipelineProvider {
         createListOfNodes(factor.getNumber()));
     assertPipelineProperties(pipeline, factor, REPLICATION_TYPE,
         Pipeline.PipelineState.OPEN);
-    cleanup();
   }
 
   @Test
@@ -240,7 +240,6 @@ public class TestRatisPipelineProvider {
 
     Assertions.assertEquals(pipeline1.getNodeSet(), pipeline2.getNodeSet());
     Assertions.assertEquals(pipeline2.getNodeSet(), pipeline3.getNodeSet());
-    cleanup();
   }
 
   @Test
@@ -251,7 +250,7 @@ public class TestRatisPipelineProvider {
     List<DatanodeDetails> healthyNodes =
         nodeManager.getNodes(NodeStatus.inServiceHealthy());
 
-    Assume.assumeTrue(healthyNodes.size() == 8);
+    Assumptions.assumeTrue(healthyNodes.size() == 8);
 
     HddsProtos.ReplicationFactor factor = HddsProtos.ReplicationFactor.THREE;
 
@@ -289,7 +288,6 @@ public class TestRatisPipelineProvider {
     assertTrue(
         nodes.stream().anyMatch(membersOfClosedPipelines::contains),
         "at least 1 node should have been from members of closed pipelines");
-    cleanup();
   }
 
   @Test
@@ -316,6 +314,23 @@ public class TestRatisPipelineProvider {
     }
   }
 
+  @Test
+  // Test pipeline provider with RackScatter policy cannot create
+  // pipeline due to nodes with full pipeline engagement.
+  public void testFactorTHREEPipelineRackScatterEngagement()
+      throws Exception {
+    OzoneConfiguration conf = new OzoneConfiguration();
+    conf.set(OZONE_SCM_PIPELINE_PLACEMENT_IMPL_KEY,
+        SCMContainerPlacementRackScatter.class.getCanonicalName());
+    conf.set(OZONE_DATANODE_PIPELINE_LIMIT, "0");
+    init(0, conf);
+    List<DatanodeDetails> excludedNodes = new ArrayList<>();
+
+    Assertions.assertThrows(SCMException.class, () ->
+        provider.create(RatisReplicationConfig
+                .getInstance(ReplicationFactor.THREE),
+            excludedNodes, Collections.EMPTY_LIST));
+  }
 
   @Test
   public void testCreatePipelinesWhenNotEnoughSpace() throws Exception {
@@ -355,7 +370,6 @@ public class TestRatisPipelineProvider {
         Assertions.assertTrue(ex.getMessage().contains(expectedErrorSubstring));
       }
     }
-    cleanup();
   }
 
   private void addPipeline(
