@@ -22,7 +22,6 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.UUID;
 
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
-import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.CommandQueueReportProto;
 import org.apache.hadoop.hdds.protocol.proto.
     StorageContainerDatanodeProtocolProtos.CommandStatusReportsProto;
 import org.apache.hadoop.hdds.scm.server.
@@ -38,7 +37,6 @@ import org.apache.hadoop.hdds.scm.server.SCMDatanodeHeartbeatDispatcher
     .ContainerReportFromDatanode;
 import org.apache.hadoop.hdds.scm.server.SCMDatanodeHeartbeatDispatcher
     .NodeReportFromDatanode;
-import org.apache.hadoop.hdds.scm.server.SCMDatanodeHeartbeatDispatcher.CommandQueueReportFromDatanode;
 import org.apache.hadoop.hdds.server.events.Event;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
 import org.apache.hadoop.ozone.protocol.commands.ReregisterCommand;
@@ -51,7 +49,6 @@ import static org.apache.hadoop.hdds.protocol.MockDatanodeDetails.randomDatanode
 import static org.apache.hadoop.hdds.scm.events.SCMEvents.CONTAINER_REPORT;
 import static org.apache.hadoop.hdds.scm.events.SCMEvents.NODE_REPORT;
 import static org.apache.hadoop.hdds.scm.events.SCMEvents.CMD_STATUS_REPORT;
-import static org.apache.hadoop.hdds.scm.events.SCMEvents.COMMAND_QUEUE_REPORT;
 
 /**
  * This class tests the behavior of SCMDatanodeHeartbeatDispatcher.
@@ -149,46 +146,6 @@ public class TestSCMDatanodeHeartbeatDispatcher {
 
   }
 
-  @Test
-  public void testCommandQueueReportDispatched() {
-    AtomicInteger eventReceived = new AtomicInteger();
-
-    CommandQueueReportProto commandQueueReport =
-        CommandQueueReportProto.getDefaultInstance();
-
-    NodeManager mockNodeManager = Mockito.mock(NodeManager.class);
-    Mockito.when(mockNodeManager.isNodeRegistered(Mockito.any()))
-        .thenReturn(true);
-
-    SCMDatanodeHeartbeatDispatcher dispatcher =
-        new SCMDatanodeHeartbeatDispatcher(
-            mockNodeManager,
-            new EventPublisher() {
-              @Override
-              public <PAYLOAD, EVENT extends Event<PAYLOAD>> void fireEvent(
-                  EVENT event, PAYLOAD payload) {
-                Assertions.assertTrue(event.equals(COMMAND_QUEUE_REPORT));
-
-                if (payload instanceof CommandQueueReportFromDatanode) {
-                  Assertions.assertEquals(commandQueueReport,
-                      ((CommandQueueReportFromDatanode) payload).getReport());
-                }
-                eventReceived.incrementAndGet();
-              }
-            });
-
-    DatanodeDetails datanodeDetails = randomDatanodeDetails();
-
-    SCMHeartbeatRequestProto heartbeat =
-        SCMHeartbeatRequestProto.newBuilder()
-            .setDatanodeDetails(datanodeDetails.getProtoBufMessage())
-            .setCommandQueueReport(commandQueueReport)
-            .setCommandQueueReport(commandQueueReport)
-            .build();
-    dispatcher.dispatch(heartbeat);
-    Assertions.assertEquals(1, eventReceived.get());
-  }
-
   /**
    * Asserts scm informs datanodes to re-register on a restart.
    *
