@@ -25,24 +25,28 @@ import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.io.UnsupportedEncodingException;
+import java.util.List;
+import java.util.Map;
 import java.util.concurrent.TimeoutException;
 
 import com.google.common.base.Preconditions;
-import com.google.common.base.Supplier;
 import org.apache.commons.io.IOUtils;
-import org.apache.commons.lang3.RandomStringUtils;
 import org.apache.commons.lang3.StringUtils;
-import org.apache.log4j.Appender;
+import org.apache.commons.lang3.tuple.Pair;
 import org.apache.log4j.Layout;
 import org.apache.log4j.Level;
 import org.apache.log4j.LogManager;
 import org.apache.log4j.Logger;
-import org.apache.log4j.PatternLayout;
-import org.apache.log4j.WriterAppender;
-import org.junit.Assert;
+import org.junit.jupiter.api.Assertions;
+import org.mockito.Mockito;
+import java.lang.reflect.Field;
+import java.lang.reflect.Modifier;
+import java.util.function.BooleanSupplier;
+import java.util.stream.Collectors;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
-import static org.junit.Assert.assertTrue;
+import static org.apache.commons.lang3.RandomStringUtils.randomAlphanumeric;
+import static org.apache.logging.log4j.util.StackLocatorUtil.getCallerClass;
 
 /**
  * Provides some very generic helpers which might be used across the tests.
@@ -53,7 +57,8 @@ public abstract class GenericTestUtils {
   public static final String DEFAULT_TEST_DATA_DIR;
   public static final String DEFAULT_TEST_DATA_PATH = "target/test/data/";
   /**
-   * Error string used in {@link GenericTestUtils#waitFor(Supplier, int, int)}.
+   * Error string used in
+   * {@link GenericTestUtils#waitFor(BooleanSupplier, int, int)}.
    */
   public static final String ERROR_MISSING_ARGUMENT =
       "Input supplier interface should be initialized";
@@ -137,22 +142,23 @@ public abstract class GenericTestUtils {
    */
   @SuppressWarnings("java:S2245") // no need for secure random
   public static String getRandomizedTempPath() {
-    return getTempPath(RandomStringUtils.randomAlphanumeric(10));
+    return getTempPath(getCallerClass(GenericTestUtils.class).getSimpleName()
+        + "-" + randomAlphanumeric(10));
   }
 
   /**
    * Assert that a given file exists.
    */
   public static void assertExists(File f) {
-    assertTrue("File " + f + " should exist", f.exists());
+    Assertions.assertTrue(f.exists(), "File " + f + " should exist");
   }
 
   /**
    * Assert that a given dir can be created or it already exists.
    */
   public static void assertDirCreation(File f) {
-    assertTrue("Could not create dir " + f + ", nor does it exist",
-        f.mkdirs() || f.exists());
+    Assertions.assertTrue(f.mkdirs() || f.exists(),
+        "Could not create dir " + f + ", nor does it exist");
   }
 
   public static void assertExceptionContains(String expectedText, Throwable t) {
@@ -161,7 +167,7 @@ public abstract class GenericTestUtils {
 
   public static void assertExceptionContains(String expectedText, Throwable t,
       String message) {
-    Assert.assertNotNull("Null Throwable", t);
+    Assertions.assertNotNull(t, "Null Throwable");
     String msg = t.toString();
     if (msg == null) {
       throw new AssertionError("Null Throwable.toString() value", t);
@@ -204,18 +210,18 @@ public abstract class GenericTestUtils {
    *                              time
    * @throws InterruptedException if the method is interrupted while waiting
    */
-  public static void waitFor(Supplier<Boolean> check, int checkEveryMillis,
+  public static void waitFor(BooleanSupplier check, int checkEveryMillis,
       int waitForMillis) throws TimeoutException, InterruptedException {
     Preconditions.checkNotNull(check, ERROR_MISSING_ARGUMENT);
     Preconditions.checkArgument(waitForMillis >= checkEveryMillis,
         ERROR_INVALID_ARGUMENT);
 
     long st = monotonicNow();
-    boolean result = check.get();
+    boolean result = check.getAsBoolean();
 
     while (!result && (monotonicNow() - st < waitForMillis)) {
       Thread.sleep(checkEveryMillis);
-      result = check.get();
+      result = check.getAsBoolean();
     }
 
     if (!result) {
@@ -242,51 +248,102 @@ public abstract class GenericTestUtils {
     setLogLevel(LogManager.getRootLogger(), Level.toLevel(level.toString()));
   }
 
+  public static <T> T mockFieldReflection(Object object, String fieldName)
+          throws NoSuchFieldException, IllegalAccessException {
+    Field field = object.getClass().getDeclaredField(fieldName);
+    boolean isAccessible = field.isAccessible();
+
+    field.setAccessible(true);
+    Field modifiersField = Field.class.getDeclaredField("modifiers");
+    boolean modifierFieldAccessible = modifiersField.isAccessible();
+    modifiersField.setAccessible(true);
+    int modifierVal = modifiersField.getInt(field);
+    modifiersField.setInt(field, field.getModifiers() & ~Modifier.FINAL);
+    T value = (T) field.get(object);
+    value = Mockito.spy(value);
+    field.set(object, value);
+    modifiersField.setInt(field, modifierVal);
+    modifiersField.setAccessible(modifierFieldAccessible);
+    field.setAccessible(isAccessible);
+    return value;
+  }
+
+  public static <T> T getFieldReflection(Object object, String fieldName)
+          throws NoSuchFieldException, IllegalAccessException {
+    Field field = object.getClass().getDeclaredField(fieldName);
+    boolean isAccessible = field.isAccessible();
+
+    field.setAccessible(true);
+    Field modifiersField = Field.class.getDeclaredField("modifiers");
+    boolean modifierFieldAccessible = modifiersField.isAccessible();
+    modifiersField.setAccessible(true);
+    int modifierVal = modifiersField.getInt(field);
+    modifiersField.setInt(field, field.getModifiers() & ~Modifier.FINAL);
+    T value = (T) field.get(object);
+    modifiersField.setInt(field, modifierVal);
+    modifiersField.setAccessible(modifierFieldAccessible);
+    field.setAccessible(isAccessible);
+    return value;
+  }
+
+  public static <K, V> Map<V, K> getReverseMap(Map<K, List<V>> map) {
+    return map.entrySet().stream().flatMap(entry -> entry.getValue().stream()
+            .map(v -> Pair.of(v, entry.getKey())))
+            .collect(Collectors.toMap(Pair::getKey, Pair::getValue));
+  }
+
+  /***
+   * Removed all files and dirs in the given dir recursively.
+   */
+  public static boolean deleteDirectory(File dir) {
+    File[] allContents = dir.listFiles();
+    if (allContents != null) {
+      for (File content : allContents) {
+        if (!deleteDirectory(content)) {
+          return false;
+        }
+      }
+    }
+    return dir.delete();
+  }
+
   /**
    * Class to capture logs for doing assertions.
    */
-  public static final class LogCapturer {
-    private StringWriter sw = new StringWriter();
-    private WriterAppender appender;
-    private Logger logger;
+  public abstract static class LogCapturer {
+    private final StringWriter sw = new StringWriter();
+
+    public static LogCapturer captureLogs(Logger logger) {
+      return new Log4j1Capturer(logger);
+    }
+
+    public static LogCapturer captureLogs(Logger logger, Layout layout) {
+      return new Log4j1Capturer(logger, layout);
+    }
 
     public static LogCapturer captureLogs(org.slf4j.Logger logger) {
-      return new LogCapturer(toLog4j(logger), getDefaultLayout());
+      return new Log4j1Capturer(toLog4j(logger));
     }
 
-    public static LogCapturer captureLogs(org.slf4j.Logger logger,
-        Layout layout) {
-      return new LogCapturer(toLog4j(logger), layout);
-    }
-
-    private static Layout getDefaultLayout() {
-      Appender defaultAppender = Logger.getRootLogger().getAppender("stdout");
-      if (defaultAppender == null) {
-        defaultAppender = Logger.getRootLogger().getAppender("console");
-      }
-      return (defaultAppender == null) ? new PatternLayout() :
-          defaultAppender.getLayout();
-    }
-
-    private LogCapturer(Logger logger, Layout layout) {
-      this.logger = logger;
-      this.appender = new WriterAppender(layout, sw);
-      logger.addAppender(this.appender);
+    // TODO: let Log4j2Capturer capture only specific logger's logs
+    public static LogCapturer log4j2(String ignoredLoggerName) {
+      return Log4j2Capturer.getInstance();
     }
 
     public String getOutput() {
-      return sw.toString();
+      return writer().toString();
     }
 
-    public void stopCapturing() {
-      logger.removeAppender(appender);
+    public abstract void stopCapturing();
+
+    protected StringWriter writer() {
+      return sw;
     }
 
     public void clearOutput() {
-      sw.getBuffer().setLength(0);
+      writer().getBuffer().setLength(0);
     }
   }
-
   @Deprecated
   public static Logger toLog4j(org.slf4j.Logger logger) {
     return LogManager.getLogger(logger.getName());
