@@ -17,21 +17,26 @@
  */
 package org.apache.hadoop.hdds.scm.cli;
 
-import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.hadoop.conf.StorageUnit;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.SecretKeyProtocolScm;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ContainerDataProto;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.ReadContainerResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
+import org.apache.hadoop.hdds.protocol.proto.HddsProtos.DeletedBlocksTransactionInfo;
+import org.apache.hadoop.hdds.protocol.proto.StorageContainerLocationProtocolProtos.DecommissionScmResponseProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerLocationProtocolProtos.StartContainerBalancerResponseProto;
 import org.apache.hadoop.hdds.scm.DatanodeAdminError;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.XceiverClientManager;
 import org.apache.hadoop.hdds.scm.XceiverClientSpi;
+import org.apache.hadoop.hdds.scm.client.ClientTrustManager;
+import org.apache.hadoop.hdds.security.x509.certificate.client.CACertificateProvider;
 import org.apache.hadoop.hdds.scm.client.ScmClient;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
@@ -42,6 +47,7 @@ import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.protocol.StorageContainerLocationProtocol;
 import org.apache.hadoop.hdds.scm.storage.ContainerProtocolCalls;
 import org.apache.hadoop.hdds.utils.HAUtils;
+import org.apache.hadoop.hdds.utils.HddsServerUtil;
 import org.apache.hadoop.ozone.ClientVersion;
 import org.apache.hadoop.ozone.OzoneSecurityUtil;
 import org.apache.hadoop.ozone.upgrade.UpgradeFinalizer.StatusAndMessages;
@@ -49,7 +55,6 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
-import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
@@ -72,6 +77,7 @@ public class ContainerOperationClient implements ScmClient {
   private final HddsProtos.ReplicationType replicationType;
   private final StorageContainerLocationProtocol
       storageContainerLocationClient;
+  private final SecretKeyProtocolScm secretKeyClient;
   private final boolean containerTokenEnabled;
   private final OzoneConfiguration configuration;
   private XceiverClientManager xceiverClientManager;
@@ -84,9 +90,10 @@ public class ContainerOperationClient implements ScmClient {
     return xceiverClientManager;
   }
 
-  public ContainerOperationClient(OzoneConfiguration conf) {
+  public ContainerOperationClient(OzoneConfiguration conf) throws IOException {
     this.configuration = conf;
     storageContainerLocationClient = newContainerRpcClient(conf);
+    secretKeyClient = newSecretKeyClient(conf);
     containerSizeB = (int) conf.getStorageSize(OZONE_SCM_CONTAINER_SIZE,
         OZONE_SCM_CONTAINER_SIZE_DEFAULT, StorageUnit.BYTES);
     boolean useRatis = conf.getBoolean(
@@ -103,21 +110,14 @@ public class ContainerOperationClient implements ScmClient {
         HDDS_CONTAINER_TOKEN_ENABLED_DEFAULT);
   }
 
-  @VisibleForTesting
-  public StorageContainerLocationProtocol
-      getStorageContainerLocationProtocol() {
-    return storageContainerLocationClient;
-  }
-
   private XceiverClientManager newXCeiverClientManager(ConfigurationSource conf)
       throws IOException {
     XceiverClientManager manager;
     if (OzoneSecurityUtil.isSecurityEnabled(conf)) {
-      List<X509Certificate> caCertificates =
-          HAUtils.buildCAX509List(null, conf);
+      CACertificateProvider caCerts = () -> HAUtils.buildCAX509List(null, conf);
       manager = new XceiverClientManager(conf,
           conf.getObject(XceiverClientManager.ScmClientConfig.class),
-          caCertificates);
+          new ClientTrustManager(caCerts, null));
     } else {
       manager = new XceiverClientManager(conf);
     }
@@ -129,6 +129,11 @@ public class ContainerOperationClient implements ScmClient {
     return HAUtils.getScmContainerClient(configSource);
   }
 
+  public static SecretKeyProtocolScm newSecretKeyClient(
+      ConfigurationSource configSource) throws IOException {
+    return HddsServerUtil.getSecretKeyClientForSCM(configSource);
+  }
+
   @Override
   public ContainerWithPipeline createContainer(String owner)
       throws IOException {
@@ -173,8 +178,8 @@ public class ContainerOperationClient implements ScmClient {
     // Let us log this info after we let SCM know that we have completed the
     // creation state.
     if (LOG.isDebugEnabled()) {
-      LOG.debug("Created container " + containerId
-          + " machines:" + client.getPipeline().getNodes());
+      LOG.debug("Created container {} machines {}", containerId,
+              client.getPipeline().getNodes());
     }
   }
 
@@ -187,55 +192,6 @@ public class ContainerOperationClient implements ScmClient {
         .encodeToUrlString();
   }
 
-  /**
-   * Creates a pipeline over the machines chosen by the SCM.
-   *
-   * @param client   - Client
-   * @param pipeline - pipeline to be createdon Datanodes.
-   * @throws IOException
-   */
-  private void createPipeline(XceiverClientSpi client, Pipeline pipeline)
-      throws IOException {
-
-    Preconditions.checkNotNull(pipeline.getId(), "Pipeline " +
-        "name cannot be null when client create flag is set.");
-
-    // Pipeline creation is a three step process.
-    //
-    // 1. Notify SCM that this client is doing a create pipeline on
-    // datanodes.
-    //
-    // 2. Talk to Datanodes to create the pipeline.
-    //
-    // 3. update SCM that pipeline creation was successful.
-
-    // TODO: this has not been fully implemented on server side
-    // SCMClientProtocolServer#notifyObjectStageChange
-    // TODO: when implement the pipeline state machine, change
-    // the pipeline name (string) to pipeline id (long)
-    //storageContainerLocationClient.notifyObjectStageChange(
-    //    ObjectStageChangeRequestProto.Type.pipeline,
-    //    pipeline.getPipelineName(),
-    //    ObjectStageChangeRequestProto.Op.create,
-    //    ObjectStageChangeRequestProto.Stage.begin);
-
-    // client.createPipeline();
-    // TODO: Use PipelineManager to createPipeline
-
-    //storageContainerLocationClient.notifyObjectStageChange(
-    //    ObjectStageChangeRequestProto.Type.pipeline,
-    //    pipeline.getPipelineName(),
-    //    ObjectStageChangeRequestProto.Op.create,
-    //    ObjectStageChangeRequestProto.Stage.complete);
-
-    // TODO : Should we change the state on the client side ??
-    // That makes sense, but it is not needed for the client to work.
-    if (LOG.isDebugEnabled()) {
-      LOG.debug("Pipeline creation successful. Pipeline: {}",
-          pipeline);
-    }
-  }
-
   @Override
   public ContainerWithPipeline createContainer(HddsProtos.ReplicationType type,
       HddsProtos.ReplicationFactor factor, String owner) throws IOException {
@@ -259,18 +215,6 @@ public class ContainerOperationClient implements ScmClient {
     }
   }
 
-  /**
-   * Returns a set of Nodes that meet a query criteria.
-   *
-   * @param opState - The operational state we want the node to have
-   *                eg IN_SERVICE, DECOMMISSIONED, etc
-   * @param nodeState - The health we want the node to have, eg HEALTHY, STALE,
-   *                  etc
-   * @param queryScope - Query scope - Cluster or pool.
-   * @param poolName - if it is pool, a pool name is required.
-   * @return A set of nodes that meet the requested criteria.
-   * @throws IOException
-   */
   @Override
   public List<HddsProtos.Node> queryNode(
       HddsProtos.NodeOperationalState opState,
@@ -300,9 +244,6 @@ public class ContainerOperationClient implements ScmClient {
         hosts, endHours);
   }
 
-  /**
-   * Creates a specified replication pipeline.
-   */
   @Override
   public Pipeline createReplicationPipeline(HddsProtos.ReplicationType type,
       HddsProtos.ReplicationFactor factor, HddsProtos.NodePool nodePool)
@@ -354,14 +295,6 @@ public class ContainerOperationClient implements ScmClient {
     }
   }
 
-  /**
-   * Deletes an existing container.
-   *
-   * @param containerId - ID of the container.
-   * @param pipeline    - Pipeline that represents the container.
-   * @param force       - true to forcibly delete the container.
-   * @throws IOException
-   */
   @Override
   public void deleteContainer(long containerId, Pipeline pipeline,
       boolean force) throws IOException {
@@ -386,13 +319,6 @@ public class ContainerOperationClient implements ScmClient {
     }
   }
 
-  /**
-   * Delete the container, this will release any resource it uses.
-   *
-   * @param containerID - containerID.
-   * @param force       - True to forcibly delete the container.
-   * @throws IOException
-   */
   @Override
   public void deleteContainer(long containerID, boolean force)
       throws IOException {
@@ -416,14 +342,6 @@ public class ContainerOperationClient implements ScmClient {
         startContainerID, count, state, repType, replicationConfig);
   }
 
-  /**
-   * Get meta data from an existing container.
-   *
-   * @param containerID - ID of the container.
-   * @param pipeline    - Pipeline where the container is located.
-   * @return ContainerInfo
-   * @throws IOException
-   */
   @Override
   public ContainerDataProto readContainer(long containerID,
       Pipeline pipeline) throws IOException {
@@ -446,53 +364,43 @@ public class ContainerOperationClient implements ScmClient {
     }
   }
 
-  /**
-   * Get meta data from an existing container.
-   *
-   * @param containerID - ID of the container.
-   * @return ContainerInfo - a message of protobuf which has basic info
-   * of a container.
-   * @throws IOException
-   */
+  public Map<DatanodeDetails, ReadContainerResponseProto>
+      readContainerFromAllNodes(long containerID, Pipeline pipeline)
+      throws IOException, InterruptedException {
+    XceiverClientManager clientManager = getXceiverClientManager();
+    String encodedToken = getEncodedContainerToken(containerID);
+    XceiverClientSpi client = null;
+    try {
+      client = clientManager.acquireClientForReadData(pipeline);
+      Map<DatanodeDetails, ReadContainerResponseProto> responses =
+          ContainerProtocolCalls.readContainerFromAllNodes(client, containerID,
+              encodedToken);
+      return responses;
+    } finally {
+      if (client != null) {
+        clientManager.releaseClient(client, false);
+      }
+    }
+  }
+
   @Override
   public ContainerDataProto readContainer(long containerID) throws IOException {
     ContainerWithPipeline info = getContainerWithPipeline(containerID);
     return readContainer(containerID, info.getPipeline());
   }
 
-  /**
-   * Given an id, return the pipeline associated with the container.
-   *
-   * @param containerId - String Container ID
-   * @return Pipeline of the existing container, corresponding to the given id.
-   * @throws IOException
-   */
   @Override
   public ContainerInfo getContainer(long containerId) throws
       IOException {
     return storageContainerLocationClient.getContainer(containerId);
   }
 
-  /**
-   * Gets a container by Name -- Throws if the container does not exist.
-   *
-   * @param containerId - Container ID
-   * @return ContainerWithPipeline
-   * @throws IOException
-   */
   @Override
   public ContainerWithPipeline getContainerWithPipeline(long containerId)
       throws IOException {
     return storageContainerLocationClient.getContainerWithPipeline(containerId);
   }
 
-  /**
-   * Gets the list of ReplicaInfo known by SCM for a given container.
-   * @param containerId - The Container ID
-   * @return List of ContainerReplicaInfo for the container or an empty list
-   *         if none.
-   * @throws IOException
-   */
   @Override
   public List<ContainerReplicaInfo>
       getContainerReplicas(long containerId) throws IOException {
@@ -506,11 +414,6 @@ public class ContainerOperationClient implements ScmClient {
     return replicas;
   }
 
-  /**
-   * Close a container.
-   *
-   * @throws IOException
-   */
   @Override
   public void closeContainer(long containerId)
       throws IOException {
@@ -520,13 +423,6 @@ public class ContainerOperationClient implements ScmClient {
     storageContainerLocationClient.closeContainer(containerId);
   }
 
-  /**
-   * Get the the current usage information.
-   *
-   * @param containerID - ID of the container.
-   * @return the size of the given container.
-   * @throws IOException
-   */
   @Override
   public long getContainerSize(long containerID) throws IOException {
     // TODO : Fix this, it currently returns the capacity
@@ -534,12 +430,6 @@ public class ContainerOperationClient implements ScmClient {
     return containerSizeB;
   }
 
-  /**
-   * Check if SCM is in safe mode.
-   *
-   * @return Returns true if SCM is in safe mode else returns false.
-   * @throws IOException
-   */
   @Override
   public boolean inSafeMode() throws IOException {
     return storageContainerLocationClient.inSafeMode();
@@ -551,12 +441,6 @@ public class ContainerOperationClient implements ScmClient {
     return storageContainerLocationClient.getSafeModeRuleStatuses();
   }
 
-  /**
-   * Force SCM out of safe mode.
-   *
-   * @return returns true if operation is successful.
-   * @throws IOException
-   */
   public boolean forceExitSafeMode() throws IOException {
     return storageContainerLocationClient.forceExitSafeMode();
   }
@@ -611,36 +495,35 @@ public class ContainerOperationClient implements ScmClient {
     return storageContainerLocationClient.getScmInfo().getRatisPeerRoles();
   }
 
+  @Override
+  public boolean rotateSecretKeys(boolean force) throws IOException {
+    return secretKeyClient.checkAndRotate(force);
+  }
+
+  @Override
+  public void transferLeadership(String newLeaderId) throws IOException {
+    storageContainerLocationClient.transferLeadership(newLeaderId);
+  }
+
+  @Override
+  public List<DeletedBlocksTransactionInfo> getFailedDeletedBlockTxn(int count,
+      long startTxId) throws IOException {
+    return storageContainerLocationClient.getFailedDeletedBlockTxn(count,
+        startTxId);
+  }
+
   @Override
   public int resetDeletedBlockRetryCount(List<Long> txIDs) throws IOException {
     return storageContainerLocationClient.resetDeletedBlockRetryCount(txIDs);
   }
 
-  /**
-   * Get Datanode Usage information by ipaddress or uuid.
-   *
-   * @param ipaddress datanode ipaddress String
-   * @param uuid datanode uuid String
-   * @return List of DatanodeUsageInfoProto. Each element contains info such as
-   * capacity, SCMused, and remaining space.
-   * @throws IOException
-   */
   @Override
   public List<HddsProtos.DatanodeUsageInfoProto> getDatanodeUsageInfo(
-      String ipaddress, String uuid) throws IOException {
-    return storageContainerLocationClient.getDatanodeUsageInfo(ipaddress,
+      String address, String uuid) throws IOException {
+    return storageContainerLocationClient.getDatanodeUsageInfo(address,
         uuid, ClientVersion.CURRENT_VERSION);
   }
 
-  /**
-   * Get usage information of most or least used datanodes.
-   *
-   * @param mostUsed true if most used, false if least used
-   * @param count Integer number of nodes to get info for
-   * @return List of DatanodeUsageInfoProto. Each element contains info such as
-   * capacity, SCMUsed, and remaining space.
-   * @throws IOException
-   */
   @Override
   public List<HddsProtos.DatanodeUsageInfoProto> getDatanodeUsageInfo(
       boolean mostUsed, int count) throws IOException {
@@ -661,4 +544,12 @@ public class ContainerOperationClient implements ScmClient {
     return storageContainerLocationClient.queryUpgradeFinalizationProgress(
         upgradeClientID, force, readonly);
   }
+
+  @Override
+  public DecommissionScmResponseProto decommissionScm(
+      String scmId)
+      throws IOException {
+    return storageContainerLocationClient.decommissionScm(scmId);
+  }
+
 }
