@@ -18,6 +18,7 @@
 
 package org.apache.hadoop.ozone.client;
 
+import org.apache.hadoop.hdds.client.DefaultReplicationConfig;
 import org.apache.hadoop.hdds.protocol.StorageType;
 import org.apache.hadoop.ozone.OzoneAcl;
 import org.apache.hadoop.ozone.OzoneConsts;
@@ -56,14 +57,17 @@ public final class BucketArgs {
    * Bucket encryption key name.
    */
   private String bucketEncryptionKey;
+  private DefaultReplicationConfig defaultReplicationConfig;
   private final String sourceVolume;
   private final String sourceBucket;
 
   private long quotaInBytes;
   private long quotaInNamespace;
 
+  private String owner;
+
   /**
-   * Bucket Type.
+   * Bucket Layout.
    */
   private BucketLayout bucketLayout = BucketLayout.DEFAULT;
 
@@ -78,13 +82,16 @@ public final class BucketArgs {
    * @param sourceBucket
    * @param quotaInBytes Bucket quota in bytes.
    * @param quotaInNamespace Bucket quota in counts.
-   * @param bucketLayout Bucket Layouts.
+   * @param bucketLayout bucket layout.
+   * @param owner owner of the bucket.
+   * @param defaultReplicationConfig default replication config.
    */
   @SuppressWarnings("parameternumber")
   private BucketArgs(Boolean versioning, StorageType storageType,
       List<OzoneAcl> acls, Map<String, String> metadata,
       String bucketEncryptionKey, String sourceVolume, String sourceBucket,
-      long quotaInBytes, long quotaInNamespace, BucketLayout bucketLayout) {
+      long quotaInBytes, long quotaInNamespace, BucketLayout bucketLayout,
+      String owner, DefaultReplicationConfig defaultReplicationConfig) {
     this.acls = acls;
     this.versioning = versioning;
     this.storageType = storageType;
@@ -95,6 +102,8 @@ public final class BucketArgs {
     this.quotaInBytes = quotaInBytes;
     this.quotaInNamespace = quotaInNamespace;
     this.bucketLayout = bucketLayout;
+    this.owner = owner;
+    this.defaultReplicationConfig = defaultReplicationConfig;
   }
 
   /**
@@ -138,6 +147,14 @@ public final class BucketArgs {
     return bucketEncryptionKey;
   }
 
+  /**
+   * Returns the bucket default replication config.
+   * @return bucket's default Replication Config.
+   */
+  public DefaultReplicationConfig getDefaultReplicationConfig() {
+    return this.defaultReplicationConfig;
+  }
+
   /**
    * Returns new builder class that builds a OmBucketInfo.
    *
@@ -172,12 +189,19 @@ public final class BucketArgs {
   }
 
   /**
-   * Returns the Bucket Type.
+   * Returns the Bucket Layout.
    */
   public BucketLayout getBucketLayout() {
     return bucketLayout;
   }
 
+  /**
+   * Returns the Owner Name.
+   */
+  public String getOwner() {
+    return owner;
+  }
+
   /**
    * Builder for OmBucketInfo.
    */
@@ -192,6 +216,8 @@ public final class BucketArgs {
     private long quotaInBytes;
     private long quotaInNamespace;
     private BucketLayout bucketLayout;
+    private String owner;
+    private DefaultReplicationConfig defaultReplicationConfig;
 
     public Builder() {
       metadata = new HashMap<>();
@@ -244,11 +270,21 @@ public final class BucketArgs {
       return this;
     }
 
-    public BucketArgs.Builder setBucketLayout(BucketLayout type) {
-      bucketLayout = type;
+    public BucketArgs.Builder setBucketLayout(BucketLayout buckLayout) {
+      bucketLayout = buckLayout;
       return this;
     }
 
+    public BucketArgs.Builder setOwner(String ownerName) {
+      owner = ownerName;
+      return this;
+    }
+
+    public BucketArgs.Builder setDefaultReplicationConfig(
+        DefaultReplicationConfig defaultReplConfig) {
+      defaultReplicationConfig = defaultReplConfig;
+      return this;
+    }
 
     /**
      * Constructs the BucketArgs.
@@ -257,7 +293,7 @@ public final class BucketArgs {
     public BucketArgs build() {
       return new BucketArgs(versioning, storageType, acls, metadata,
           bucketEncryptionKey, sourceVolume, sourceBucket, quotaInBytes,
-          quotaInNamespace, bucketLayout);
+          quotaInNamespace, bucketLayout, owner, defaultReplicationConfig);
     }
   }
 }
