@@ -22,7 +22,16 @@ import java.nio.Buffer;
 import java.nio.ByteBuffer;
 import java.util.Arrays;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
+import java.util.concurrent.ArrayBlockingQueue;
+import java.util.concurrent.BlockingQueue;
+import java.util.concurrent.ExecutionException;
+import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executors;
+import java.util.concurrent.Future;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicLong;
 import java.util.stream.Collectors;
 import java.util.stream.IntStream;
 
@@ -43,8 +52,10 @@ import org.apache.hadoop.ozone.om.helpers.OmMultipartCommitUploadPartInfo;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
+import org.apache.hadoop.ozone.om.protocol.S3Auth;
 import org.apache.ozone.erasurecode.rawcoder.RawErasureEncoder;
 import org.apache.ozone.erasurecode.rawcoder.util.CodecUtil;
+import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -52,14 +63,28 @@ import org.slf4j.LoggerFactory;
  * ECKeyOutputStream handles the EC writes by writing the data into underlying
  * block output streams chunk by chunk.
  */
-public final class ECKeyOutputStream extends KeyOutputStream {
+public final class ECKeyOutputStream extends KeyOutputStream
+    implements KeyMetadataAware {
   private OzoneClientConfig config;
   private ECChunkBuffers ecChunkBufferCache;
+  private final BlockingQueue<ECChunkBuffers> ecStripeQueue;
+  private int chunkIndex;
   private int ecChunkSize;
   private final int numDataBlks;
   private final int numParityBlks;
   private final ByteBufferPool bufferPool;
   private final RawErasureEncoder encoder;
+  private final ExecutorService flushExecutor;
+  private final Future<Boolean> flushFuture;
+  private final AtomicLong flushCheckpoint;
+
+  /**
+   * Indicates if an atomic write is required. When set to true,
+   * the amount of data written must match the declared size during the commit.
+   * A mismatch will prevent the commit from succeeding.
+   * This is essential for operations like S3 put to ensure atomicity.
+   */
+  private boolean atomicKeyCreation;
 
   private enum StripeWriteStatus {
     SUCCESS,
@@ -69,7 +94,8 @@ public final class ECKeyOutputStream extends KeyOutputStream {
   public static final Logger LOG =
       LoggerFactory.getLogger(KeyOutputStream.class);
 
-  private boolean closed;
+  private volatile boolean closed;
+  private volatile boolean closing;
   // how much of data is actually written yet to underlying stream
   private long offset;
   // how much data has been ingested into the stream
@@ -91,8 +117,18 @@ public final class ECKeyOutputStream extends KeyOutputStream {
     return blockOutputStreamEntryPool.getLocationInfoList();
   }
 
+  @VisibleForTesting
+  public void insertFlushCheckpoint(long version) throws IOException {
+    addStripeToQueue(new CheckpointDummyStripe(version));
+  }
+
+  @VisibleForTesting
+  public long getFlushCheckpoint() {
+    return flushCheckpoint.get();
+  }
+
   private ECKeyOutputStream(Builder builder) {
-    super(builder.getClientMetrics());
+    super(builder.getReplicationConfig(), builder.getClientMetrics());
     this.config = builder.getClientConfig();
     this.bufferPool = builder.getByteBufferPool();
     // For EC, cell/chunk size and buffer size can be same for now.
@@ -104,6 +140,8 @@ public final class ECKeyOutputStream extends KeyOutputStream {
     this.numParityBlks = builder.getReplicationConfig().getParity();
     ecChunkBufferCache = new ECChunkBuffers(
         ecChunkSize, numDataBlks, numParityBlks, bufferPool);
+    chunkIndex = 0;
+    ecStripeQueue = new ArrayBlockingQueue<>(config.getEcStripeQueueSize());
     OmKeyInfo info = builder.getOpenHandler().getKeyInfo();
     blockOutputStreamEntryPool =
         new ECBlockOutputStreamEntryPool(config,
@@ -118,6 +156,14 @@ public final class ECKeyOutputStream extends KeyOutputStream {
     this.writeOffset = 0;
     this.encoder = CodecUtil.createRawEncoderWithFallback(
         builder.getReplicationConfig());
+    this.flushExecutor = Executors.newSingleThreadExecutor();
+    S3Auth s3Auth = builder.getS3CredentialsProvider().get();
+    ThreadLocal<S3Auth> s3CredentialsProvider =
+        builder.getS3CredentialsProvider();
+    flushExecutor.submit(() -> s3CredentialsProvider.set(s3Auth));
+    this.flushFuture = this.flushExecutor.submit(this::flushStripeFromQueue);
+    this.flushCheckpoint = new AtomicLong(0);
+    this.atomicKeyCreation = builder.getAtomicKeyCreation();
   }
 
   /**
@@ -164,14 +210,14 @@ public final class ECKeyOutputStream extends KeyOutputStream {
       }
     } catch (Exception e) {
       markStreamClosed();
-      throw new IOException(e.getMessage());
+      throw e;
     }
     writeOffset += len;
   }
 
-  private StripeWriteStatus rewriteStripeToNewBlockGroup() throws IOException {
+  private void rollbackAndReset(ECChunkBuffers stripe) throws IOException {
     // Rollback the length/offset updated as part of this failed stripe write.
-    final ByteBuffer[] dataBuffers = ecChunkBufferCache.getDataBuffers();
+    final ByteBuffer[] dataBuffers = stripe.getDataBuffers();
     offset -= Arrays.stream(dataBuffers).mapToInt(Buffer::limit).sum();
 
     final ECBlockOutputStreamEntry failedStreamEntry =
@@ -184,32 +230,14 @@ public final class ECKeyOutputStream extends KeyOutputStream {
         failedStreamEntry.getPipeline().getId());
     // Let's close the current entry.
     failedStreamEntry.close();
-
-    // Let's rewrite the last stripe, so that it will be written to new block
-    // group.
-    // TODO: we can improve to write partial stripe failures. In that case,
-    //  we just need to write only available buffers.
-    blockOutputStreamEntryPool.allocateBlockIfNeeded();
-    final ECBlockOutputStreamEntry currentStreamEntry =
-        blockOutputStreamEntryPool.getCurrentStreamEntry();
-    for (int i = 0; i < numDataBlks; i++) {
-      if (dataBuffers[i].limit() > 0) {
-        handleOutputStreamWrite(i, dataBuffers[i].limit(), false);
-      }
-      currentStreamEntry.useNextBlockStream();
-    }
-    return handleParityWrites();
-  }
-
-  private void encodeAndWriteParityCells() throws IOException {
-    generateParityCells();
-    if (handleParityWrites() == StripeWriteStatus.FAILED) {
-      retryStripeWrite(config.getMaxECStripeWriteRetries());
-    }
   }
 
   private void logStreamError(List<ECBlockOutputStream> failedStreams,
                               String operation) {
+    if (!LOG.isWarnEnabled()) {
+      return;
+    }
+
     Set<Integer> failedStreamIndexSet =
             failedStreams.stream().map(ECBlockOutputStream::getReplicationIndex)
                     .collect(Collectors.toSet());
@@ -227,39 +255,36 @@ public final class ECKeyOutputStream extends KeyOutputStream {
     }
   }
 
-  private StripeWriteStatus handleParityWrites() throws IOException {
-    writeParityCells();
+  private StripeWriteStatus commitStripeWrite(ECChunkBuffers stripe)
+      throws IOException {
 
     ECBlockOutputStreamEntry streamEntry =
         blockOutputStreamEntryPool.getCurrentStreamEntry();
     List<ECBlockOutputStream> failedStreams =
         streamEntry.streamsWithWriteFailure();
     if (!failedStreams.isEmpty()) {
-      if (LOG.isDebugEnabled()) {
-        logStreamError(failedStreams, "EC stripe write");
-      }
+      logStreamError(failedStreams, "EC stripe write");
       excludePipelineAndFailedDN(streamEntry.getPipeline(), failedStreams);
       return StripeWriteStatus.FAILED;
     }
 
     // By this time, we should have finished full stripe. So, lets call
     // executePutBlock for all.
-    // TODO: we should alter the put block calls to share CRC to each stream.
     final boolean isLastStripe = streamEntry.getRemaining() <= 0 ||
-        ecChunkBufferCache.getLastDataCell().limit() < ecChunkSize;
-    streamEntry.executePutBlock(isLastStripe, streamEntry.getCurrentPosition());
+        stripe.getLastDataCell().limit() < ecChunkSize;
+    ByteString checksum = streamEntry.calculateChecksum();
+    streamEntry.executePutBlock(isLastStripe,
+        streamEntry.getCurrentPosition(), checksum);
 
     failedStreams = streamEntry.streamsWithPutBlockFailure();
     if (!failedStreams.isEmpty()) {
-      if (LOG.isDebugEnabled()) {
-        logStreamError(failedStreams, "Put block");
-      }
+      logStreamError(failedStreams, "Put block");
       excludePipelineAndFailedDN(streamEntry.getPipeline(), failedStreams);
       return StripeWriteStatus.FAILED;
     }
     streamEntry.updateBlockGroupToAckedPosition(
         streamEntry.getCurrentPosition());
-    ecChunkBufferCache.clear();
+    stripe.clear();
 
     if (streamEntry.getRemaining() <= 0) {
       streamEntry.close();
@@ -338,65 +363,71 @@ public final class ECKeyOutputStream extends KeyOutputStream {
     }
   }
 
-  private void writeParityCells() {
+  private void writeDataCells(ECChunkBuffers stripe) throws IOException {
+    blockOutputStreamEntryPool.allocateBlockIfNeeded();
+    ByteBuffer[] dataCells = stripe.getDataBuffers();
+    for (int i = 0; i < numDataBlks; i++) {
+      if (dataCells[i].limit() > 0) {
+        handleOutputStreamWrite(dataCells[i], false);
+      }
+      blockOutputStreamEntryPool.getCurrentStreamEntry().useNextBlockStream();
+    }
+  }
+
+  private void writeParityCells(ECChunkBuffers stripe) {
     // Move the stream entry cursor to parity block index
     blockOutputStreamEntryPool
         .getCurrentStreamEntry().forceToFirstParityBlock();
-    ByteBuffer[] parityCells = ecChunkBufferCache.getParityBuffers();
+    ByteBuffer[] parityCells = stripe.getParityBuffers();
     for (int i = 0; i < numParityBlks; i++) {
-      handleOutputStreamWrite(numDataBlks + i, parityCells[i].limit(), true);
+      handleOutputStreamWrite(parityCells[i], true);
       blockOutputStreamEntryPool.getCurrentStreamEntry().useNextBlockStream();
     }
   }
 
   private int handleWrite(byte[] b, int off, int len) throws IOException {
-
-    blockOutputStreamEntryPool.allocateBlockIfNeeded();
-
-    int currIdx = blockOutputStreamEntryPool
-        .getCurrentStreamEntry().getCurrentStreamIdx();
-    int bufferRem = ecChunkBufferCache.dataBuffers[currIdx].remaining();
+    int bufferRem = ecChunkBufferCache.dataBuffers[chunkIndex].remaining();
     final int writeLen = Math.min(len, Math.min(bufferRem, ecChunkSize));
-    int pos = ecChunkBufferCache.addToDataBuffer(currIdx, b, off, writeLen);
+    int pos = ecChunkBufferCache.addToDataBuffer(chunkIndex, b, off, writeLen);
 
-    // if this cell is full, send data to the OutputStream
+    // if this cell is full, use next buffer
     if (pos == ecChunkSize) {
-      handleOutputStreamWrite(currIdx, pos, false);
-      blockOutputStreamEntryPool.getCurrentStreamEntry().useNextBlockStream();
+      chunkIndex++;
 
       // if this is last data cell in the stripe,
-      // compute and write the parity cells
-      if (currIdx == numDataBlks - 1) {
-        encodeAndWriteParityCells();
+      // compute parity cells and write data
+      if (chunkIndex == numDataBlks) {
+        generateParityCells();
+        addStripeToQueue(ecChunkBufferCache);
+        ecChunkBufferCache = new ECChunkBuffers(ecChunkSize,
+            numDataBlks, numParityBlks, bufferPool);
+        chunkIndex = 0;
       }
     }
     return writeLen;
   }
 
-  private void handleOutputStreamWrite(int currIdx, int len, boolean isParity) {
-    ByteBuffer bytesToWrite = isParity ?
-        ecChunkBufferCache.getParityBuffers()[currIdx - numDataBlks] :
-        ecChunkBufferCache.getDataBuffers()[currIdx];
+  private void handleOutputStreamWrite(ByteBuffer buffer, boolean isParity) {
     try {
       // Since it's a full cell, let's write all content from buffer.
       // At a time we write max cell size in EC. So, it should safe to cast
       // the len to int to use the super class defined write API.
       // The len cannot be bigger than cell buffer size.
-      assert len <= ecChunkSize : " The len: " + len + ". EC chunk size: "
-          + ecChunkSize;
-      assert len <= bytesToWrite
-          .limit() : " The len: " + len + ". Chunk buffer limit: "
-          + bytesToWrite.limit();
+      assert buffer.limit() <= ecChunkSize : "The buffer size: " +
+          buffer.limit() + " should not exceed EC chunk size: " + ecChunkSize;
       writeToOutputStream(blockOutputStreamEntryPool.getCurrentStreamEntry(),
-          bytesToWrite.array(), len, 0, isParity);
+          buffer.array(), buffer.limit(), 0, isParity);
     } catch (Exception e) {
       markStreamAsFailed(e);
     }
   }
 
-  private long writeToOutputStream(ECBlockOutputStreamEntry current,
+  private void writeToOutputStream(ECBlockOutputStreamEntry current,
       byte[] b, int writeLen, int off, boolean isParity)
       throws IOException {
+    if (closing) {
+      throw new IOException("Stream is closing, avoid re-opening streams");
+    }
     try {
       if (!isParity) {
         // In case if exception while writing, this length will be updated back
@@ -412,7 +443,6 @@ public final class ECKeyOutputStream extends KeyOutputStream {
               .getCurrentStreamIdx(), ioe);
       handleException(current, ioe);
     }
-    return writeLen;
   }
 
   private void handleException(BlockOutputStreamEntry streamEntry,
@@ -428,8 +458,7 @@ public final class ECKeyOutputStream extends KeyOutputStream {
   }
 
   private void markStreamClosed() {
-    blockOutputStreamEntryPool.cleanup();
-    closed = true;
+    closing = true;
   }
 
   private void markStreamAsFailed(Exception e) {
@@ -477,39 +506,105 @@ public final class ECKeyOutputStream extends KeyOutputStream {
     }
     closed = true;
     try {
-      // If stripe buffer is not empty, encode and flush the stripe.
-      if (ecChunkBufferCache.getFirstDataCell().position() > 0) {
-        final int index = blockOutputStreamEntryPool.getCurrentStreamEntry()
-            .getCurrentStreamIdx();
-        ByteBuffer lastCell = ecChunkBufferCache.getDataBuffers()[index];
-
-        // Finish writing the current partial cached chunk
-        if (lastCell.position() % ecChunkSize != 0) {
-          handleOutputStreamWrite(index, lastCell.position(), false);
+      if (!closing) {
+        // If stripe buffer is not empty, encode and flush the stripe.
+        if (ecChunkBufferCache.getFirstDataCell().position() > 0) {
+          generateParityCells();
+          addStripeToQueue(ecChunkBufferCache);
         }
+        // Send EOF mark to flush thread.
+        addStripeToQueue(new EOFDummyStripe());
 
-        encodeAndWriteParityCells();
+        // Wait for all the stripes to be written.
+        flushFuture.get();
+
+        Preconditions.checkArgument(writeOffset == offset,
+            "Expected writeOffset= " + writeOffset
+                + " Expected offset=" + offset);
+        if (atomicKeyCreation) {
+          long expectedSize = blockOutputStreamEntryPool.getDataSize();
+          Preconditions.checkState(expectedSize == offset, String.format(
+              "Expected: %d and actual %d write sizes do not match",
+                  expectedSize, offset));
+        }
+        blockOutputStreamEntryPool.commitKey(offset);
       }
-
-      closeCurrentStreamEntry();
-      Preconditions.checkArgument(writeOffset == offset,
-          "Expected writeOffset= " + writeOffset
-              + " Expected offset=" + offset);
-      blockOutputStreamEntryPool.commitKey(offset);
+    } catch (ExecutionException e) {
+      Throwable cause = e.getCause();
+      if (cause instanceof IOException) {
+        throw (IOException) cause;
+      } else if (cause instanceof RuntimeException) {
+        throw (RuntimeException) cause;
+      } else {
+        throw new IOException(cause);
+      }
+    } catch (InterruptedException e) {
+      throw new IOException("Flushing thread was interrupted", e);
     } finally {
+      flushExecutor.shutdownNow();
+      closeCurrentStreamEntry();
       blockOutputStreamEntryPool.cleanup();
     }
-    ecChunkBufferCache.release();
   }
 
-  private void retryStripeWrite(int times) throws IOException {
-    for (int i = 0; i < times; i++) {
-      if (rewriteStripeToNewBlockGroup() == StripeWriteStatus.SUCCESS) {
-        return;
+  private void addStripeToQueue(ECChunkBuffers stripe) throws IOException {
+    try {
+      do {
+        // If flushFuture is done, it means that the flush thread has
+        // encountered an exception. Call get() to throw that exception here.
+        if (flushFuture.isDone()) {
+          flushFuture.get();
+          // We should never reach here.
+          throw new IOException("Flush thread has ended before stream close");
+        }
+      } while (!ecStripeQueue.offer(stripe, 1, TimeUnit.SECONDS));
+    } catch (InterruptedException e) {
+      Thread.currentThread().interrupt();
+      throw new IOException("Interrupted while adding stripe to queue", e);
+    } catch (ExecutionException e) {
+      if (e.getCause() instanceof IOException) {
+        throw (IOException) e.getCause();
+      } else if (e.getCause() instanceof RuntimeException) {
+        throw (RuntimeException) e.getCause();
+      } else {
+        throw new IOException(e.getCause());
       }
     }
+  }
+
+  private boolean flushStripeFromQueue() throws IOException {
+    try {
+      ECChunkBuffers stripe = ecStripeQueue.take();
+      while (!closing && !(stripe instanceof EOFDummyStripe)) {
+        if (stripe instanceof CheckpointDummyStripe) {
+          flushCheckpoint.set(((CheckpointDummyStripe) stripe).version);
+        } else {
+          flushStripeToDatanodes(stripe);
+          stripe.release();
+        }
+        stripe = ecStripeQueue.take();
+      }
+    } catch (InterruptedException e) {
+      Thread.currentThread().interrupt();
+      throw new IOException("Interrupted while polling stripe from queue", e);
+    }
+    return true;
+  }
+
+  private void flushStripeToDatanodes(ECChunkBuffers stripe)
+      throws IOException {
+    int maxRetry = config.getMaxECStripeWriteRetries();
+    for (int i = 0; i <= maxRetry; i++) {
+      writeDataCells(stripe);
+      writeParityCells(stripe);
+      if (commitStripeWrite(stripe) == StripeWriteStatus.SUCCESS) {
+        return;
+      }
+      // In case of failure, cleanup before retry
+      rollbackAndReset(stripe);
+    }
     throw new IOException("Completed max allowed retries " +
-        times + " on stripe failures.");
+        maxRetry + " on stripe failures.");
   }
 
   public static void padBufferToLimit(ByteBuffer buf, int limit) {
@@ -530,6 +625,11 @@ public final class ECKeyOutputStream extends KeyOutputStream {
     return blockOutputStreamEntryPool.getExcludeList();
   }
 
+  @Override
+  public Map<String, String> getMetadata() {
+    return this.blockOutputStreamEntryPool.getMetadata();
+  }
+
   /**
    * Builder class of ECKeyOutputStream.
    */
@@ -537,6 +637,8 @@ public final class ECKeyOutputStream extends KeyOutputStream {
     private ECReplicationConfig replicationConfig;
     private ByteBufferPool byteBufferPool;
 
+    private ThreadLocal<S3Auth> s3CredentialsProvider;
+
     @Override
     public ECReplicationConfig getReplicationConfig() {
       return replicationConfig;
@@ -558,6 +660,16 @@ public final class ECKeyOutputStream extends KeyOutputStream {
       return this;
     }
 
+    public ECKeyOutputStream.Builder setS3CredentialsProvider(
+        ThreadLocal<S3Auth> s3CredentialsThreadLocal) {
+      this.s3CredentialsProvider = s3CredentialsThreadLocal;
+      return this;
+    }
+
+    public ThreadLocal<S3Auth> getS3CredentialsProvider() {
+      return s3CredentialsProvider;
+    }
+
     @Override
     public ECKeyOutputStream build() {
       return new ECKeyOutputStream(this);
@@ -571,19 +683,37 @@ public final class ECKeyOutputStream extends KeyOutputStream {
    * @throws IOException if the connection is closed.
    */
   private void checkNotClosed() throws IOException {
-    if (closed) {
+    if (closing || closed) {
       throw new IOException(
           ": " + FSExceptionMessages.STREAM_IS_CLOSED + " Key: "
               + blockOutputStreamEntryPool.getKeyName());
     }
   }
 
+  private static class EOFDummyStripe extends ECChunkBuffers {
+    EOFDummyStripe() {
+    }
+  }
+
+  private static class CheckpointDummyStripe extends ECChunkBuffers {
+    private final long version;
+    CheckpointDummyStripe(long version) {
+      super();
+      this.version = version;
+    }
+  }
+
   private static class ECChunkBuffers {
     private final ByteBuffer[] dataBuffers;
     private final ByteBuffer[] parityBuffers;
     private int cellSize;
     private ByteBufferPool byteBufferPool;
 
+    ECChunkBuffers() {
+      dataBuffers = null;
+      parityBuffers = null;
+    }
+
     ECChunkBuffers(int cellSize, int numData, int numParity,
         ByteBufferPool byteBufferPool) {
       this.cellSize = cellSize;
