@@ -20,6 +20,7 @@ package org.apache.hadoop.ozone.client;
 
 import org.apache.hadoop.conf.StorageUnit;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
+import org.apache.hadoop.hdds.client.ReplicationConfigValidator;
 import org.apache.hadoop.hdds.client.ReplicationType;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
@@ -31,19 +32,18 @@ import org.apache.hadoop.ozone.client.io.OzoneOutputStream;
 import org.apache.hadoop.ozone.client.rpc.RpcClient;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes;
+import org.apache.hadoop.ozone.om.helpers.ServiceInfoEx;
 import org.apache.hadoop.ozone.om.protocolPB.OmTransport;
 import org.apache.ozone.test.LambdaTestUtils.VoidCallable;
 import org.jetbrains.annotations.NotNull;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 import java.io.IOException;
-import java.security.cert.X509Certificate;
 import java.time.Instant;
 import java.util.HashMap;
-import java.util.List;
 import java.util.UUID;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
@@ -65,13 +65,13 @@ public class TestOzoneClient {
       throws Exception {
     try {
       eval.call();
-      Assert.fail("OMException is expected");
+      Assertions.fail("OMException is expected");
     } catch (OMException ex) {
-      Assert.assertEquals(code, ex.getResult());
+      Assertions.assertEquals(code, ex.getResult());
     }
   }
 
-  @Before
+  @BeforeEach
   public void init() throws IOException {
     OzoneConfiguration config = new OzoneConfiguration();
     createNewClient(config, new SinglePipelineBlockAllocator(config));
@@ -82,15 +82,14 @@ public class TestOzoneClient {
     client = new OzoneClient(config, new RpcClient(config, null) {
 
       @Override
-      protected OmTransport createOmTransport(String omServiceId)
-          throws IOException {
+      protected OmTransport createOmTransport(String omServiceId) {
         return new MockOmTransport(blkAllocator);
       }
 
       @NotNull
       @Override
       protected XceiverClientFactory createXceiverClientFactory(
-          List<X509Certificate> x509Certificates) throws IOException {
+          ServiceInfoEx serviceInfo) {
         return new MockXceiverClientFactory();
       }
     });
@@ -98,7 +97,7 @@ public class TestOzoneClient {
     store = client.getObjectStore();
   }
 
-  @After
+  @AfterEach
   public void close() throws IOException {
     client.close();
   }
@@ -109,7 +108,7 @@ public class TestOzoneClient {
     String volumeName = UUID.randomUUID().toString();
     store.createVolume(volumeName);
     OzoneVolume volume = store.getVolume(volumeName);
-    Assert.assertNotNull(volume);
+    Assertions.assertNotNull(volume);
     store.deleteVolume(volumeName);
     expectOmException(ResultCodes.VOLUME_NOT_FOUND,
         () -> store.getVolume(volumeName));
@@ -125,10 +124,11 @@ public class TestOzoneClient {
         .build();
     store.createVolume(volumeName, volumeArgs);
     OzoneVolume volume = store.getVolume(volumeName);
-    Assert.assertEquals(OzoneConsts.QUOTA_RESET, volume.getQuotaInNamespace());
-    Assert.assertEquals(OzoneConsts.QUOTA_RESET, volume.getQuotaInBytes());
-    Assert.assertEquals("val1", volume.getMetadata().get("key1"));
-    Assert.assertEquals(volumeName, volume.getName());
+    Assertions.assertEquals(OzoneConsts.QUOTA_RESET,
+        volume.getQuotaInNamespace());
+    Assertions.assertEquals(OzoneConsts.QUOTA_RESET, volume.getQuotaInBytes());
+    Assertions.assertEquals("val1", volume.getMetadata().get("key1"));
+    Assertions.assertEquals(volumeName, volume.getName());
   }
 
   @Test
@@ -141,9 +141,9 @@ public class TestOzoneClient {
     OzoneVolume volume = store.getVolume(volumeName);
     volume.createBucket(bucketName);
     OzoneBucket bucket = volume.getBucket(bucketName);
-    Assert.assertEquals(bucketName, bucket.getName());
-    Assert.assertFalse(bucket.getCreationTime().isBefore(testStartTime));
-    Assert.assertFalse(volume.getCreationTime().isBefore(testStartTime));
+    Assertions.assertEquals(bucketName, bucket.getName());
+    Assertions.assertFalse(bucket.getCreationTime().isBefore(testStartTime));
+    Assertions.assertFalse(volume.getCreationTime().isBefore(testStartTime));
   }
 
   @Test
@@ -161,14 +161,14 @@ public class TestOzoneClient {
       out.write(value.getBytes(UTF_8));
       out.close();
       OzoneKey key = bucket.getKey(keyName);
-      Assert.assertEquals(keyName, key.getName());
+      Assertions.assertEquals(keyName, key.getName());
       OzoneInputStream is = bucket.readKey(keyName);
       byte[] fileContent = new byte[value.getBytes(UTF_8).length];
-      Assert.assertEquals(value.length(), is.read(fileContent));
+      Assertions.assertEquals(value.length(), is.read(fileContent));
       is.close();
-      Assert.assertEquals(value, new String(fileContent, UTF_8));
-      Assert.assertFalse(key.getCreationTime().isBefore(testStartTime));
-      Assert.assertFalse(key.getModificationTime().isBefore(testStartTime));
+      Assertions.assertEquals(value, new String(fileContent, UTF_8));
+      Assertions.assertFalse(key.getCreationTime().isBefore(testStartTime));
+      Assertions.assertFalse(key.getModificationTime().isBefore(testStartTime));
     }
   }
 
@@ -193,6 +193,10 @@ public class TestOzoneClient {
   public void testPutKeyWithECReplicationConfig() throws IOException {
     close();
     OzoneConfiguration config = new OzoneConfiguration();
+    ReplicationConfigValidator validator =
+        config.getObject(ReplicationConfigValidator.class);
+    validator.disableValidation();
+    config.setFromObject(validator);
     config.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 2,
         StorageUnit.KB);
     int data = 3;
@@ -214,7 +218,41 @@ public class TestOzoneClient {
         out.write(value.getBytes(UTF_8));
       }
       OzoneKey key = bucket.getKey(keyName);
-      Assert.assertEquals(keyName, key.getName());
+      Assertions.assertEquals(keyName, key.getName());
+    }
+  }
+
+  /**
+   * This test validates that for S3G,
+   * the key upload process needs to be atomic.
+   * It simulates two mismatch scenarios where the actual write data size does
+   * not match the expected size.
+   */
+  @Test
+  public void testPutKeySizeMismatch() throws IOException {
+    String value = new String(new byte[1024], UTF_8);
+    OzoneBucket bucket = getOzoneBucket();
+    String keyName = UUID.randomUUID().toString();
+    try {
+      // Simulating first mismatch: Write less data than expected
+      client.getProxy().setIsS3Request(true);
+      OzoneOutputStream out1 = bucket.createKey(keyName,
+          value.getBytes(UTF_8).length, ReplicationType.RATIS, ONE,
+          new HashMap<>());
+      out1.write(value.substring(0, value.length() - 1).getBytes(UTF_8));
+      Assertions.assertThrows(IllegalStateException.class, out1::close,
+          "Expected IllegalArgumentException due to size mismatch.");
+
+      // Simulating second mismatch: Write more data than expected
+      OzoneOutputStream out2 = bucket.createKey(keyName,
+          value.getBytes(UTF_8).length, ReplicationType.RATIS, ONE,
+          new HashMap<>());
+      value += "1";
+      out2.write(value.getBytes(UTF_8));
+      Assertions.assertThrows(IllegalStateException.class, out2::close,
+          "Expected IllegalArgumentException due to size mismatch.");
+    } finally {
+      client.getProxy().setIsS3Request(false);
     }
   }
 
@@ -226,4 +264,4 @@ public class TestOzoneClient {
     volume.createBucket(bucketName);
     return volume.getBucket(bucketName);
   }
-}
\ No newline at end of file
+}
