@@ -22,7 +22,7 @@ import org.apache.hadoop.conf.StorageUnit;
 import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.client.DefaultReplicationConfig;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
-import org.apache.hadoop.hdds.client.ReplicationType;
+import org.apache.hadoop.hdds.client.ReplicationConfigValidator;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
@@ -41,19 +41,20 @@ import org.apache.hadoop.ozone.client.io.OzoneOutputStream;
 import org.apache.hadoop.ozone.client.rpc.RpcClient;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfoGroup;
+import org.apache.hadoop.ozone.om.helpers.ServiceInfoEx;
 import org.apache.hadoop.ozone.om.protocolPB.OmTransport;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.ozone.erasurecode.rawcoder.RSRawErasureCoderFactory;
 import org.apache.ozone.erasurecode.rawcoder.RawErasureEncoder;
+import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ratis.thirdparty.com.google.protobuf.ByteString;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 import java.io.IOException;
 import java.nio.ByteBuffer;
-import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
@@ -84,7 +85,7 @@ public class TestOzoneECClient {
   private byte[][] inputChunks = new byte[dataBlocks][chunkSize];
   private final XceiverClientFactory factoryStub =
       new MockXceiverClientFactory();
-  private OzoneConfiguration conf = new OzoneConfiguration();
+  private OzoneConfiguration conf = createConfiguration();
   private MultiNodePipelineBlockAllocator allocator =
       new MultiNodePipelineBlockAllocator(conf, dataBlocks + parityBlocks, 15);
   private final MockOmTransport transportStub = new MockOmTransport(allocator);
@@ -92,10 +93,8 @@ public class TestOzoneECClient {
       new RSRawErasureCoderFactory().createEncoder(
           new ECReplicationConfig(dataBlocks, parityBlocks));
 
-  @Before
+  @BeforeEach
   public void init() throws IOException {
-    conf.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 2,
-        StorageUnit.KB);
     createNewClient(conf, transportStub);
   }
 
@@ -109,14 +108,13 @@ public class TestOzoneECClient {
     client = new OzoneClient(config, new RpcClient(config, null) {
 
       @Override
-      protected OmTransport createOmTransport(String omServiceId)
-          throws IOException {
+      protected OmTransport createOmTransport(String omServiceId) {
         return transport;
       }
 
       @Override
       protected XceiverClientFactory createXceiverClientFactory(
-          List<X509Certificate> x509Certificates) throws IOException {
+          ServiceInfoEx serviceInfo) {
         return factoryStub;
       }
     });
@@ -139,7 +137,7 @@ public class TestOzoneECClient {
     return builder.toString().getBytes(UTF_8);
   }
 
-  @After
+  @AfterEach
   public void close() throws IOException {
     client.close();
   }
@@ -148,7 +146,7 @@ public class TestOzoneECClient {
   public void testPutECKeyAndCheckDNStoredData() throws IOException {
     OzoneBucket bucket = writeIntoECKey(inputChunks, keyName, null);
     OzoneKey key = bucket.getKey(keyName);
-    Assert.assertEquals(keyName, key.getName());
+    Assertions.assertEquals(keyName, key.getName());
     Map<DatanodeDetails, MockDatanodeStorage> storages =
         ((MockXceiverClientFactory) factoryStub).getStorages();
     DatanodeDetails[] dnDetails =
@@ -156,10 +154,10 @@ public class TestOzoneECClient {
     Arrays.sort(dnDetails);
     for (int i = 0; i < inputChunks.length; i++) {
       MockDatanodeStorage datanodeStorage = storages.get(dnDetails[i]);
-      Assert.assertEquals(1, datanodeStorage.getAllBlockData().size());
+      Assertions.assertEquals(1, datanodeStorage.getAllBlockData().size());
       ByteString content =
           datanodeStorage.getAllBlockData().values().iterator().next();
-      Assert.assertEquals(new String(inputChunks[i], UTF_8),
+      Assertions.assertEquals(new String(inputChunks[i], UTF_8),
           content.toStringUtf8());
     }
   }
@@ -177,7 +175,7 @@ public class TestOzoneECClient {
     }
     encoder.encode(dataBuffers, parityBuffers);
     OzoneKey key = bucket.getKey(keyName);
-    Assert.assertEquals(keyName, key.getName());
+    Assertions.assertEquals(keyName, key.getName());
     Map<DatanodeDetails, MockDatanodeStorage> storages =
         ((MockXceiverClientFactory) factoryStub).getStorages();
     DatanodeDetails[] dnDetails =
@@ -186,10 +184,10 @@ public class TestOzoneECClient {
 
     for (int i = dataBlocks; i < parityBlocks + dataBlocks; i++) {
       MockDatanodeStorage datanodeStorage = storages.get(dnDetails[i]);
-      Assert.assertEquals(1, datanodeStorage.getAllBlockData().size());
+      Assertions.assertEquals(1, datanodeStorage.getAllBlockData().size());
       ByteString content =
           datanodeStorage.getAllBlockData().values().iterator().next();
-      Assert.assertEquals(
+      Assertions.assertEquals(
           new String(parityBuffers[i - dataBlocks].array(), UTF_8),
           content.toStringUtf8());
     }
@@ -200,15 +198,15 @@ public class TestOzoneECClient {
   public void testPutECKeyAndReadContent() throws IOException {
     OzoneBucket bucket = writeIntoECKey(inputChunks, keyName, null);
     OzoneKey key = bucket.getKey(keyName);
-    Assert.assertEquals(keyName, key.getName());
+    Assertions.assertEquals(keyName, key.getName());
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
       for (int i = 0; i < dataBlocks; i++) {
-        Assert.assertEquals(inputChunks[i].length, is.read(fileContent));
-        Assert.assertTrue(Arrays.equals(inputChunks[i], fileContent));
+        Assertions.assertEquals(inputChunks[i].length, is.read(fileContent));
+        Assertions.assertTrue(Arrays.equals(inputChunks[i], fileContent));
       }
       // A further read should give EOF
-      Assert.assertEquals(-1, is.read(fileContent));
+      Assertions.assertEquals(-1, is.read(fileContent));
     }
   }
 
@@ -216,14 +214,14 @@ public class TestOzoneECClient {
   public void testCreateBucketWithDefaultReplicationConfig()
       throws IOException {
     final OzoneBucket bucket = writeIntoECKey(inputChunks, keyName,
-        new DefaultReplicationConfig(ReplicationType.EC,
+        new DefaultReplicationConfig(
             new ECReplicationConfig(dataBlocks, parityBlocks,
                 ECReplicationConfig.EcCodec.RS, chunkSize)));
 
     // create key without mentioning replication config. Since we set EC
     // replication in bucket, key should be EC key.
     try (OzoneOutputStream out = bucket.createKey("mykey", inputSize)) {
-      Assert.assertTrue(out.getOutputStream() instanceof ECKeyOutputStream);
+      Assertions.assertTrue(out.getOutputStream() instanceof ECKeyOutputStream);
       for (int i = 0; i < inputChunks.length; i++) {
         out.write(inputChunks[i]);
       }
@@ -285,7 +283,7 @@ public class TestOzoneECClient {
           String.valueOf(i % 9).getBytes(UTF_8)[0]);
     }
     final OzoneBucket bucket = writeIntoECKey(offset, numChunks * chunkSize,
-            inputData, keyName, new DefaultReplicationConfig(ReplicationType.EC,
+            inputData, keyName, new DefaultReplicationConfig(
                     new ECReplicationConfig(dataBlocks, parityBlocks,
                             ECReplicationConfig.EcCodec.RS, chunkSize)));
     OzoneKey key = bucket.getKey(keyName);
@@ -301,11 +299,11 @@ public class TestOzoneECClient {
   private void validateContent(int offset, int length, byte[] inputData,
                                OzoneBucket bucket,
       OzoneKey key) throws IOException {
-    Assert.assertEquals(keyName, key.getName());
+    Assertions.assertEquals(keyName, key.getName());
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[length];
-      Assert.assertEquals(length, is.read(fileContent));
-      Assert.assertEquals(new String(Arrays.copyOfRange(inputData, offset,
+      Assertions.assertEquals(length, is.read(fileContent));
+      Assertions.assertEquals(new String(Arrays.copyOfRange(inputData, offset,
                       offset + length),
                       UTF_8),
           new String(fileContent, UTF_8));
@@ -319,7 +317,7 @@ public class TestOzoneECClient {
         Byte.parseByte("1"));
 
     writeIntoECKey(firstSmallChunk, keyName,
-        new DefaultReplicationConfig(ReplicationType.EC,
+        new DefaultReplicationConfig(
             new ECReplicationConfig(dataBlocks, parityBlocks,
                 ECReplicationConfig.EcCodec.RS, chunkSize)));
     OzoneManagerProtocolProtos.KeyLocationList blockList =
@@ -343,7 +341,8 @@ public class TestOzoneECClient {
         keyLocations.getBlockID().getContainerBlockID().getContainerID(),
         keyLocations.getBlockID().getContainerBlockID().getLocalID()));
 
-    Assert.assertArrayEquals(firstSmallChunk, firstBlockData.getBytes(UTF_8));
+    Assertions.assertArrayEquals(
+        firstSmallChunk, firstBlockData.getBytes(UTF_8));
 
     final ByteBuffer[] dataBuffers = new ByteBuffer[dataBlocks];
     dataBuffers[0] = ByteBuffer.wrap(firstSmallChunk);
@@ -366,8 +365,8 @@ public class TestOzoneECClient {
           keyLocations.getBlockID().getContainerBlockID().getLocalID()));
       String expected =
           new String(parityBuffers[i - dataBlocks].array(), UTF_8);
-      Assert.assertEquals(expected, parityBlockData);
-      Assert.assertEquals(expected.length(), parityBlockData.length());
+      Assertions.assertEquals(expected, parityBlockData);
+      Assertions.assertEquals(expected.length(), parityBlockData.length());
 
     }
   }
@@ -376,12 +375,12 @@ public class TestOzoneECClient {
   public void testPutBlockHasBlockGroupLen() throws IOException {
     OzoneBucket bucket = writeIntoECKey(inputChunks, keyName, null);
     OzoneKey key = bucket.getKey(keyName);
-    Assert.assertEquals(keyName, key.getName());
+    Assertions.assertEquals(keyName, key.getName());
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
       for (int i = 0; i < dataBlocks; i++) {
-        Assert.assertEquals(inputChunks[i].length, is.read(fileContent));
-        Assert.assertTrue(Arrays.equals(inputChunks[i], fileContent));
+        Assertions.assertEquals(inputChunks[i].length, is.read(fileContent));
+        Assertions.assertTrue(Arrays.equals(inputChunks[i], fileContent));
       }
 
       Map<DatanodeDetails, MockDatanodeStorage> storages =
@@ -412,7 +411,7 @@ public class TestOzoneECClient {
                 .equals(OzoneConsts.BLOCK_GROUP_LEN_KEY_IN_PUT_BLOCK))
                 .collect(Collectors.toList());
 
-        Assert.assertEquals(3L * chunkSize,
+        Assertions.assertEquals(3L * chunkSize,
             Long.parseLong(metadataList.get(0).getValue()));
       }
     }
@@ -446,7 +445,7 @@ public class TestOzoneECClient {
     Arrays.fill(inputData, (numFullChunks * chunkSize),
         ((numFullChunks * chunkSize)) + partialChunkLen - 1, (byte) 1);
     final OzoneBucket bucket = writeIntoECKey(inputData, keyName,
-        new DefaultReplicationConfig(ReplicationType.EC,
+        new DefaultReplicationConfig(
             new ECReplicationConfig(dataBlocks, parityBlocks,
                 ECReplicationConfig.EcCodec.RS, chunkSize)));
     OzoneKey key = bucket.getKey(keyName);
@@ -457,14 +456,14 @@ public class TestOzoneECClient {
   public void testCommitKeyInfo()
       throws IOException {
     final OzoneBucket bucket = writeIntoECKey(inputChunks, keyName,
-        new DefaultReplicationConfig(ReplicationType.EC,
+        new DefaultReplicationConfig(
             new ECReplicationConfig(dataBlocks, parityBlocks,
                 ECReplicationConfig.EcCodec.RS, chunkSize)));
 
     // create key without mentioning replication config. Since we set EC
     // replication in bucket, key should be EC key.
     try (OzoneOutputStream out = bucket.createKey("mykey", 6 * inputSize)) {
-      Assert.assertTrue(out.getOutputStream() instanceof ECKeyOutputStream);
+      Assertions.assertTrue(out.getOutputStream() instanceof ECKeyOutputStream);
       // Block Size is 2kb, so to create 3 blocks we need 6 iterations here
       for (int j = 0; j < 6; j++) {
         for (int i = 0; i < inputChunks.length; i++) {
@@ -476,20 +475,20 @@ public class TestOzoneECClient {
         transportStub.getKeys().get(volumeName).get(bucketName).get("mykey")
             .getKeyLocationListList().get(0);
 
-    Assert.assertEquals(3, blockList.getKeyLocationsCount());
+    Assertions.assertEquals(3, blockList.getKeyLocationsCount());
     // As the mock allocator allocates block with id's increasing sequentially
     // from 1. Therefore the block should be in the order with id starting 1, 2
     // and then 3.
     for (int i = 0; i < 3; i++) {
       long localId = blockList.getKeyLocationsList().get(i).getBlockID()
           .getContainerBlockID().getLocalID();
-      Assert.assertEquals(i + 1, localId);
+      Assertions.assertEquals(i + 1, localId);
     }
 
-    Assert.assertEquals(1,
+    Assertions.assertEquals(1,
         transportStub.getKeys().get(volumeName).get(bucketName).get("mykey")
             .getKeyLocationListCount());
-    Assert.assertEquals(inputChunks[0].length * 3 * 6,
+    Assertions.assertEquals(inputChunks[0].length * 3 * 6,
         transportStub.getKeys().get(volumeName).get(bucketName).get("mykey")
             .getDataSize());
   }
@@ -561,14 +560,14 @@ public class TestOzoneECClient {
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
       for (int i = 0; i < 2; i++) {
-        Assert.assertEquals(inputChunks[i].length, is.read(fileContent));
-        Assert.assertTrue(Arrays.equals(inputChunks[i], fileContent));
+        Assertions.assertEquals(inputChunks[i].length, is.read(fileContent));
+        Assertions.assertTrue(Arrays.equals(inputChunks[i], fileContent));
       }
-      Assert.assertEquals(lastChunk.length, is.read(fileContent));
-      Assert.assertTrue(Arrays.equals(lastChunk,
+      Assertions.assertEquals(lastChunk.length, is.read(fileContent));
+      Assertions.assertTrue(Arrays.equals(lastChunk,
           Arrays.copyOf(fileContent, lastChunk.length)));
       // A further read should give EOF
-      Assert.assertEquals(-1, is.read(fileContent));
+      Assertions.assertEquals(-1, is.read(fileContent));
     }
   }
 
@@ -602,39 +601,38 @@ public class TestOzoneECClient {
 
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
-      Assert.assertEquals(inSize, is.read(fileContent));
-      Assert.assertTrue(Arrays.equals(partialChunk,
+      Assertions.assertEquals(inSize, is.read(fileContent));
+      Assertions.assertTrue(Arrays.equals(partialChunk,
           Arrays.copyOf(fileContent, inSize)));
     }
   }
 
   @Test
   public void testWriteShouldFailIfMoreThanParityNodesFail()
-      throws IOException {
+      throws Exception {
     testNodeFailuresWhileWriting(new int[] {0, 1, 2}, 3, 2);
   }
 
   @Test
   public void testWriteShouldSuccessIfLessThanParityNodesFail()
-      throws IOException {
+      throws Exception {
     testNodeFailuresWhileWriting(new int[] {0}, 2, 2);
   }
 
   @Test
-  public void testWriteShouldSuccessIf4NodesFailed() throws IOException {
+  public void testWriteShouldSuccessIf4NodesFailed() throws Exception {
     testNodeFailuresWhileWriting(new int[] {0, 1, 2, 3}, 1, 2);
   }
 
   @Test
   public void testWriteShouldSuccessWithAdditional1BlockGroupAfterFailure()
-      throws IOException {
+      throws Exception {
     testNodeFailuresWhileWriting(new int[] {0, 1, 2, 3}, 10, 3);
   }
 
   @Test
-  public void testStripeWriteRetriesOn2Failures() throws IOException {
-    OzoneConfiguration con = new OzoneConfiguration();
-    con.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 2, StorageUnit.KB);
+  public void testStripeWriteRetriesOn2Failures() throws Exception {
+    OzoneConfiguration con = createConfiguration();
     // Cluster has 15 nodes. So, first we will create 3 block groups with
     // distinct nodes in each. Block Group 1:  0-4, Block Group 2: 5-9, Block
     // Group 3: 10-14
@@ -650,14 +648,13 @@ public class TestOzoneECClient {
         nodesIndexesToMarkFailure);
     // It should have used 3rd block group also. So, total initialized nodes
     // count should be clusterSize.
-    Assert.assertTrue(((MockXceiverClientFactory) factoryStub).getStorages()
-        .size() == clusterSize);
+    Assertions.assertTrue(((MockXceiverClientFactory) factoryStub)
+        .getStorages().size() == clusterSize);
   }
 
   @Test
-  public void testStripeWriteRetriesOn3Failures() throws IOException {
-    OzoneConfiguration con = new OzoneConfiguration();
-    con.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 2, StorageUnit.KB);
+  public void testStripeWriteRetriesOn3Failures() throws Exception {
+    OzoneConfiguration con = createConfiguration();
 
     int[] nodesIndexesToMarkFailure = new int[3];
     nodesIndexesToMarkFailure[0] = 0;
@@ -672,16 +669,15 @@ public class TestOzoneECClient {
         nodesIndexesToMarkFailure);
     // It should have used 3rd block group also. So, total initialized nodes
     // count should be clusterSize.
-    Assert.assertTrue(((MockXceiverClientFactory) factoryStub).getStorages()
-        .size() == clusterSize);
+    Assertions.assertTrue(((MockXceiverClientFactory) factoryStub)
+        .getStorages().size() == clusterSize);
   }
 
   // The mocked impl throws IllegalStateException when there are not enough
-  // nodes in allocateBlock request. But write() converts it to IOException.
-  @Test(expected = IOException.class)
-  public void testStripeWriteRetriesOnAllNodeFailures() throws IOException {
-    OzoneConfiguration con = new OzoneConfiguration();
-    con.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 2, StorageUnit.KB);
+  // nodes in allocateBlock request.
+  @Test
+  public void testStripeWriteRetriesOnAllNodeFailures() throws Exception {
+    OzoneConfiguration con = createConfiguration();
 
     // After writing first stripe, we will mark all nodes as bad in the cluster.
     int clusterSize = 5;
@@ -691,15 +687,15 @@ public class TestOzoneECClient {
     }
     // Mocked MultiNodePipelineBlockAllocator#allocateBlock implementation can
     // not pick new block group as all nodes in cluster marked as bad.
-    testStripeWriteRetriesOnFailures(con, clusterSize,
-        nodesIndexesToMarkFailure);
+    Assertions.assertThrows(IllegalStateException.class, () ->
+        testStripeWriteRetriesOnFailures(con, clusterSize,
+            nodesIndexesToMarkFailure));
   }
 
   @Test
   public void testStripeWriteRetriesOn4FailuresWith3RetriesAllowed()
-      throws IOException {
-    OzoneConfiguration con = new OzoneConfiguration();
-    con.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 2, StorageUnit.KB);
+      throws Exception {
+    OzoneConfiguration con = createConfiguration();
     con.setInt(OzoneConfigKeys.OZONE_CLIENT_MAX_EC_STRIPE_WRITE_RETRIES, 3);
 
     int[] nodesIndexesToMarkFailure = new int[4];
@@ -716,17 +712,18 @@ public class TestOzoneECClient {
       // OZONE_CLIENT_MAX_EC_STRIPE_WRITE_RETRIES_ON_FAILURE(here it was
       // configured as 3). So, it should fail as we have marked 3 nodes as bad.
       testStripeWriteRetriesOnFailures(con, 20, nodesIndexesToMarkFailure);
-      Assert.fail(
+      Assertions.fail(
           "Expecting it to fail as retries should exceed the max allowed times:"
               + " " + 3);
     } catch (IOException e) {
-      Assert.assertEquals("Completed max allowed retries 3 on stripe failures.",
+      Assertions.assertEquals(
+          "Completed max allowed retries 3 on stripe failures.",
           e.getMessage());
     }
   }
 
   public void testStripeWriteRetriesOnFailures(OzoneConfiguration con,
-      int clusterSize, int[] nodesIndexesToMarkFailure) throws IOException {
+      int clusterSize, int[] nodesIndexesToMarkFailure) throws Exception {
     close();
     MultiNodePipelineBlockAllocator blkAllocator =
         new MultiNodePipelineBlockAllocator(con, dataBlocks + parityBlocks,
@@ -744,7 +741,8 @@ public class TestOzoneECClient {
       for (int i = 0; i < dataBlocks; i++) {
         out.write(inputChunks[i]);
       }
-      Assert.assertTrue(
+      waitForFlushingThreadToFinish((ECKeyOutputStream) out.getOutputStream());
+      Assertions.assertTrue(
           ((MockXceiverClientFactory) factoryStub).getStorages().size() == 5);
       List<DatanodeDetails> failedDNs = new ArrayList<>();
       List<HddsProtos.DatanodeDetailsProto> dns = blkAllocator.getClusterDns();
@@ -767,27 +765,27 @@ public class TestOzoneECClient {
     // failures after first stripe, the second stripe data should have been
     // written into new blockgroup. So, we should have 2 block groups. That
     // means two keyLocations.
-    Assert.assertEquals(2, key.getOzoneKeyLocations().size());
+    Assertions.assertEquals(2, key.getOzoneKeyLocations().size());
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
       for (int i = 0; i < dataBlocks; i++) {
-        Assert.assertEquals(inputChunks[i].length, is.read(fileContent));
-        Assert.assertTrue("Expected: " + new String(inputChunks[i],
-                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8),
-            Arrays.equals(inputChunks[i], fileContent));
+        Assertions.assertEquals(inputChunks[i].length, is.read(fileContent));
+        Assertions.assertTrue(Arrays.equals(inputChunks[i], fileContent),
+            "Expected: " + new String(inputChunks[i],
+                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8));
       }
       for (int i = 0; i < numChunksToWriteAfterFailure; i++) {
-        Assert.assertEquals(inputChunks[i].length, is.read(fileContent));
-        Assert.assertTrue("Expected: " + new String(inputChunks[i],
-                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8),
-            Arrays.equals(inputChunks[i], fileContent));
+        Assertions.assertEquals(inputChunks[i].length, is.read(fileContent));
+        Assertions.assertTrue(Arrays.equals(inputChunks[i], fileContent),
+            "Expected: " + new String(inputChunks[i],
+                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8));
       }
     }
   }
 
   public void testNodeFailuresWhileWriting(int[] nodesIndexesToMarkFailure,
       int numChunksToWriteAfterFailure, int numExpectedBlockGrps)
-      throws IOException {
+      throws Exception {
     store.createVolume(volumeName);
     OzoneVolume volume = store.getVolume(volumeName);
     volume.createBucket(bucketName);
@@ -800,6 +798,7 @@ public class TestOzoneECClient {
       for (int i = 0; i < dataBlocks; i++) {
         out.write(inputChunks[i]);
       }
+      waitForFlushingThreadToFinish((ECKeyOutputStream) out.getOutputStream());
 
       List<DatanodeDetails> failedDNs = new ArrayList<>();
       List<HddsProtos.DatanodeDetailsProto> dns = allocator.getClusterDns();
@@ -820,45 +819,46 @@ public class TestOzoneECClient {
     // failures after first stripe, the second stripe data should have been
     // written into new block group. So, we should have numExpectedBlockGrps.
     // That means two keyLocations.
-    Assert
+    Assertions
         .assertEquals(numExpectedBlockGrps, key.getOzoneKeyLocations().size());
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
       for (int i = 0; i < dataBlocks; i++) {
-        Assert.assertEquals(inputChunks[i].length, is.read(fileContent));
-        Assert.assertTrue("Expected: " + new String(inputChunks[i],
-                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8),
-            Arrays.equals(inputChunks[i], fileContent));
+        Assertions.assertEquals(inputChunks[i].length, is.read(fileContent));
+        Assertions.assertTrue(Arrays.equals(inputChunks[i], fileContent),
+            "Expected: " + new String(inputChunks[i],
+                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8));
       }
       for (int i = 0; i < numChunksToWriteAfterFailure; i++) {
-        Assert.assertEquals(inputChunks[i % dataBlocks].length,
+        Assertions.assertEquals(inputChunks[i % dataBlocks].length,
             is.read(fileContent));
-        Assert.assertTrue("Expected: " + new String(inputChunks[i % dataBlocks],
-                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8),
-            Arrays.equals(inputChunks[i % dataBlocks], fileContent));
+        Assertions.assertTrue(
+            Arrays.equals(inputChunks[i % dataBlocks], fileContent),
+            "Expected: " + new String(inputChunks[i % dataBlocks],
+                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8));
       }
     }
   }
 
   @Test
-  public void testExcludeOnDNFailure() throws IOException {
+  public void testExcludeOnDNFailure() throws Exception {
     testExcludeFailedDN(IntStream.range(0, 5), IntStream.empty());
   }
 
   @Test
-  public void testExcludeOnDNClosed() throws IOException {
+  public void testExcludeOnDNClosed() throws Exception {
     testExcludeFailedDN(IntStream.empty(), IntStream.range(0, 5));
   }
 
   @Test
-  public void testExcludeOnDNMixed() throws IOException {
+  public void testExcludeOnDNMixed() throws Exception {
     testExcludeFailedDN(IntStream.range(0, 3), IntStream.range(3, 5));
   }
 
   private void testExcludeFailedDN(IntStream failedDNIndex,
-      IntStream closedDNIndex) throws IOException {
+      IntStream closedDNIndex) throws Exception {
     close();
-    OzoneConfiguration con = new OzoneConfiguration();
+    OzoneConfiguration con = createConfiguration();
     MultiNodePipelineBlockAllocator blkAllocator =
         new MultiNodePipelineBlockAllocator(con, dataBlocks + parityBlocks, 10);
     createNewClient(con, blkAllocator);
@@ -874,7 +874,7 @@ public class TestOzoneECClient {
     try (OzoneOutputStream out = bucket.createKey(keyName,
         2L * dataBlocks * chunkSize, repConfig, new HashMap<>())) {
 
-      Assert.assertTrue(out.getOutputStream() instanceof ECKeyOutputStream);
+      Assertions.assertTrue(out.getOutputStream() instanceof ECKeyOutputStream);
       ECKeyOutputStream ecKeyOut = (ECKeyOutputStream) out.getOutputStream();
 
       List<HddsProtos.DatanodeDetailsProto> dns = blkAllocator.getClusterDns();
@@ -882,6 +882,7 @@ public class TestOzoneECClient {
       for (int i = 0; i < dataBlocks; i++) {
         out.write(inputChunks[i % dataBlocks]);
       }
+      waitForFlushingThreadToFinish((ECKeyOutputStream) out.getOutputStream());
 
       // Then let's mark datanodes with closed container
       List<DatanodeDetails> closedDNs = closedDNIndex
@@ -899,9 +900,10 @@ public class TestOzoneECClient {
       for (int i = 0; i < dataBlocks; i++) {
         out.write(inputChunks[i % dataBlocks]);
       }
+      waitForFlushingThreadToFinish((ECKeyOutputStream) out.getOutputStream());
 
       // Assert excludeList only includes failedDNs
-      Assert.assertArrayEquals(failedDNs.toArray(new DatanodeDetails[0]),
+      Assertions.assertArrayEquals(failedDNs.toArray(new DatanodeDetails[0]),
           ecKeyOut.getExcludeList().getDatanodes()
               .toArray(new DatanodeDetails[0]));
     }
@@ -909,9 +911,9 @@ public class TestOzoneECClient {
 
   @Test
   public void testLargeWriteOfMultipleStripesWithStripeFailure()
-      throws IOException {
+      throws Exception {
     close();
-    OzoneConfiguration con = new OzoneConfiguration();
+    OzoneConfiguration con = createConfiguration();
     // block size of 3KB could hold 3 full stripes
     con.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 3, StorageUnit.KB);
     con.setInt(OzoneConfigKeys.OZONE_CLIENT_MAX_EC_STRIPE_WRITE_RETRIES, 3);
@@ -943,6 +945,7 @@ public class TestOzoneECClient {
           out.write(inputChunks[i]);
         }
       }
+      waitForFlushingThreadToFinish((ECKeyOutputStream) out.getOutputStream());
 
       List<DatanodeDetails> failedDNs = new ArrayList<>();
       List<HddsProtos.DatanodeDetailsProto> dns = allocator.getClusterDns();
@@ -964,25 +967,25 @@ public class TestOzoneECClient {
     // failures after first stripe, the second stripe data should have been
     // written into new block group. So, we should have numExpectedBlockGrps.
     // That means two keyLocations.
-    Assert
+    Assertions
         .assertEquals(numExpectedBlockGrps, key.getOzoneKeyLocations().size());
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
       for (int i = 0; i < dataBlocks * numFullStripesBeforeFailure; i++) {
-        Assert.assertEquals(inputChunks[i % dataBlocks].length,
+        Assertions.assertEquals(inputChunks[i % dataBlocks].length,
             is.read(fileContent));
-        Assert.assertTrue(
+        Assertions.assertTrue(
+            Arrays.equals(inputChunks[i % dataBlocks], fileContent),
             "Expected: " + new String(inputChunks[i % dataBlocks], UTF_8)
-                + " \n " + "Actual: " + new String(fileContent, UTF_8),
-            Arrays.equals(inputChunks[i % dataBlocks], fileContent));
+                + " \n " + "Actual: " + new String(fileContent, UTF_8));
       }
       for (int i = 0; i < numChunksToWriteAfterFailure; i++) {
-        Assert.assertEquals(inputChunks[i % dataBlocks].length,
+        Assertions.assertEquals(inputChunks[i % dataBlocks].length,
             is.read(fileContent));
-        Assert.assertTrue(
+        Assertions.assertTrue(
+            Arrays.equals(inputChunks[i % dataBlocks], fileContent),
             "Expected: " + new String(inputChunks[i % dataBlocks],
-                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8),
-            Arrays.equals(inputChunks[i % dataBlocks], fileContent));
+                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8));
       }
     }
   }
@@ -991,7 +994,7 @@ public class TestOzoneECClient {
   public void testPartialStripeWithPartialChunkRetry()
       throws IOException {
     close();
-    OzoneConfiguration con = new OzoneConfiguration();
+    OzoneConfiguration con = createConfiguration();
     // block size of 3KB could hold 3 full stripes
     con.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 3, StorageUnit.KB);
     con.setInt(OzoneConfigKeys.OZONE_CLIENT_MAX_EC_STRIPE_WRITE_RETRIES, 3);
@@ -1037,30 +1040,29 @@ public class TestOzoneECClient {
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
       for (int i = 0; i < numFullChunks; i++) {
-        Assert.assertEquals(inputChunks[i % dataBlocks].length,
+        Assertions.assertEquals(inputChunks[i % dataBlocks].length,
             is.read(fileContent));
-        Assert.assertTrue("Expected: " + new String(inputChunks[i % dataBlocks],
-                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8),
-            Arrays.equals(inputChunks[i % dataBlocks], fileContent));
+        Assertions.assertTrue(
+            Arrays.equals(inputChunks[i % dataBlocks], fileContent),
+            "Expected: " + new String(inputChunks[i % dataBlocks],
+                UTF_8) + " \n " + "Actual: " + new String(fileContent, UTF_8));
       }
 
       byte[] partialChunkToRead = new byte[partialChunkSize];
-      Assert
+      Assertions
           .assertEquals(partialChunkToRead.length, is.read(partialChunkToRead));
-      Assert.assertTrue(Arrays.equals(partialChunk, partialChunkToRead));
+      Assertions.assertTrue(Arrays.equals(partialChunk, partialChunkToRead));
 
-      Assert.assertEquals(-1, is.read(partialChunkToRead));
+      Assertions.assertEquals(-1, is.read(partialChunkToRead));
     }
   }
 
   @Test
   public void testDiscardPreAllocatedBlocksPreventRetryExceeds()
-      throws IOException {
+      throws Exception {
     close();
-    OzoneConfiguration con = new OzoneConfiguration();
+    OzoneConfiguration con = createConfiguration();
     int maxRetries = 3;
-    con.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE,
-        2, StorageUnit.KB);
     con.setInt(OzoneConfigKeys.OZONE_CLIENT_MAX_EC_STRIPE_WRITE_RETRIES,
         maxRetries);
     MultiNodePipelineBlockAllocator blkAllocator =
@@ -1085,10 +1087,10 @@ public class TestOzoneECClient {
         new ECReplicationConfig(dataBlocks, parityBlocks,
             ECReplicationConfig.EcCodec.RS,
             chunkSize), new HashMap<>())) {
-      Assert.assertTrue(out.getOutputStream() instanceof ECKeyOutputStream);
+      Assertions.assertTrue(out.getOutputStream() instanceof ECKeyOutputStream);
       ECKeyOutputStream kos = (ECKeyOutputStream) out.getOutputStream();
       List<OmKeyLocationInfo> blockInfos = getAllLocationInfoList(kos);
-      Assert.assertEquals(1, blockInfos.size());
+      Assertions.assertEquals(1, blockInfos.size());
 
       // Mock some pre-allocated blocks to the key,
       // should be > maxRetries
@@ -1114,6 +1116,7 @@ public class TestOzoneECClient {
           out.write(inputChunks[i]);
         }
       }
+      waitForFlushingThreadToFinish((ECKeyOutputStream) out.getOutputStream());
 
       // Make the writes fail to trigger retry
       List<DatanodeDetails> failedDNs = new ArrayList<>();
@@ -1135,23 +1138,22 @@ public class TestOzoneECClient {
       } catch (IOException e) {
         // If we don't discard pre-allocated blocks,
         // retries should exceed the maxRetries and write will fail.
-        Assert.fail("Max retries exceeded");
+        Assertions.fail("Max retries exceeded");
       }
     }
 
     final OzoneKeyDetails key = bucket.getKey(keyName);
-    Assert.assertEquals(numExpectedBlockGrps,
+    Assertions.assertEquals(numExpectedBlockGrps,
         key.getOzoneKeyLocations().size());
 
     try (OzoneInputStream is = bucket.readKey(keyName)) {
       byte[] fileContent = new byte[chunkSize];
       for (int i = 0; i < dataBlocks * numStripesTotal; i++) {
-        Assert.assertEquals(inputChunks[i % dataBlocks].length,
+        Assertions.assertEquals(inputChunks[i % dataBlocks].length,
             is.read(fileContent));
-        Assert.assertArrayEquals(
+        Assertions.assertArrayEquals(inputChunks[i % dataBlocks], fileContent,
             "Expected: " + new String(inputChunks[i % dataBlocks], UTF_8)
-                + " \n " + "Actual: " + new String(fileContent, UTF_8),
-            inputChunks[i % dataBlocks], fileContent);
+                + " \n " + "Actual: " + new String(fileContent, UTF_8));
       }
     }
   }
@@ -1221,4 +1223,26 @@ public class TestOzoneECClient {
     }
     return locationInfoList;
   }
+
+  private static void waitForFlushingThreadToFinish(
+      ECKeyOutputStream ecOut) throws Exception {
+    final long checkpoint = System.currentTimeMillis();
+    ecOut.insertFlushCheckpoint(checkpoint);
+    GenericTestUtils.waitFor(() -> ecOut.getFlushCheckpoint() == checkpoint,
+        100, 10000);
+  }
+
+  private static OzoneConfiguration createConfiguration() {
+    OzoneConfiguration conf = new OzoneConfiguration();
+
+    ReplicationConfigValidator validator =
+        conf.getObject(ReplicationConfigValidator.class);
+    validator.disableValidation();
+    conf.setFromObject(validator);
+
+    conf.setStorageSize(OzoneConfigKeys.OZONE_SCM_BLOCK_SIZE, 2,
+        StorageUnit.KB);
+
+    return conf;
+  }
 }
