@@ -17,7 +17,10 @@
  */
 package org.apache.hadoop.ozone.om.helpers;
 
-import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
+import org.apache.hadoop.hdds.utils.db.Codec;
+import org.apache.hadoop.hdds.utils.db.DelegatedCodec;
+import org.apache.hadoop.hdds.utils.db.Proto2Codec;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.S3Secret;
 
 import java.util.Objects;
 
@@ -25,13 +28,31 @@ import java.util.Objects;
  * S3Secret to be saved in database.
  */
 public class S3SecretValue {
+  private static final Codec<S3SecretValue> CODEC = new DelegatedCodec<>(
+      Proto2Codec.get(S3Secret.getDefaultInstance()),
+      S3SecretValue::fromProtobuf,
+      S3SecretValue::getProtobuf);
+
+  public static Codec<S3SecretValue> getCodec() {
+    return CODEC;
+  }
+
   // TODO: This field should be renamed to accessId for generalization.
   private String kerberosID;
   private String awsSecret;
+  private boolean isDeleted;
+  private long transactionLogIndex;
 
   public S3SecretValue(String kerberosID, String awsSecret) {
+    this(kerberosID, awsSecret, false, 0L);
+  }
+
+  public S3SecretValue(String kerberosID, String awsSecret, boolean isDeleted,
+                       long transactionLogIndex) {
     this.kerberosID = kerberosID;
     this.awsSecret = awsSecret;
+    this.isDeleted = isDeleted;
+    this.transactionLogIndex = transactionLogIndex;
   }
 
   public String getKerberosID() {
@@ -50,17 +71,32 @@ public class S3SecretValue {
     this.awsSecret = awsSecret;
   }
 
+  public boolean isDeleted() {
+    return isDeleted;
+  }
+
+  public void setDeleted(boolean status) {
+    this.isDeleted = status;
+  }
+
   public String getAwsAccessKey() {
     return kerberosID;
   }
 
-  public static S3SecretValue fromProtobuf(
-      OzoneManagerProtocolProtos.S3Secret s3Secret) {
+  public long getTransactionLogIndex() {
+    return transactionLogIndex;
+  }
+
+  public void setTransactionLogIndex(long transactionLogIndex) {
+    this.transactionLogIndex = transactionLogIndex;
+  }
+
+  public static S3SecretValue fromProtobuf(S3Secret s3Secret) {
     return new S3SecretValue(s3Secret.getKerberosID(), s3Secret.getAwsSecret());
   }
 
-  public OzoneManagerProtocolProtos.S3Secret getProtobuf() {
-    return OzoneManagerProtocolProtos.S3Secret.newBuilder()
+  public S3Secret getProtobuf() {
+    return S3Secret.newBuilder()
         .setAwsSecret(this.awsSecret)
         .setKerberosID(this.kerberosID)
         .build();
@@ -68,7 +104,9 @@ public class S3SecretValue {
 
   @Override
   public String toString() {
-    return "awsAccessKey=" + kerberosID + "\nawsSecret=" + awsSecret;
+    return "awsAccessKey=" + kerberosID + "\nawsSecret=" + awsSecret +
+        "\nisDeleted=" + isDeleted + "\ntransactionLogIndex=" +
+        transactionLogIndex;
   }
 
   @Override
@@ -81,11 +119,12 @@ public class S3SecretValue {
     }
     S3SecretValue that = (S3SecretValue) o;
     return kerberosID.equals(that.kerberosID) &&
-        awsSecret.equals(that.awsSecret);
+        awsSecret.equals(that.awsSecret) && isDeleted == that.isDeleted &&
+        transactionLogIndex == that.transactionLogIndex;
   }
 
   @Override
   public int hashCode() {
-    return Objects.hash(kerberosID, awsSecret);
+    return Objects.hash(kerberosID, awsSecret, isDeleted, transactionLogIndex);
   }
 }
