@@ -16,8 +16,8 @@
  */
 package org.apache.hadoop.ozone.om.helpers;
 
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Test;
 
 import java.util.ArrayList;
 import java.util.Collection;
@@ -33,15 +33,16 @@ public class TestOmKeyLocationInfoGroup {
     OmKeyLocationInfoGroup testInstance = createTestInstance();
     List<OmKeyLocationInfo> latestList =
         testInstance.getBlocksLatestVersionOnly();
-    Assert.assertEquals(1, latestList.size());
-    Assert.assertEquals(2, latestList.get(0).getCreateVersion());
+    Assertions.assertEquals(1, latestList.size());
+    Assertions.assertEquals(2, latestList.get(0).getCreateVersion());
   }
 
   @Test
   public void testGettingPreviousVersions() {
     OmKeyLocationInfoGroup testInstance = createTestInstance();
-    Collection<OmKeyLocationInfo> list = testInstance.getLocationList(1L);
-    Assert.assertEquals(2, list.size());
+    Collection<OmKeyLocationInfo> list = testInstance.getLocationList(
+        1L);
+    Assertions.assertEquals(2, list.size());
   }
 
   @Test
@@ -50,9 +51,9 @@ public class TestOmKeyLocationInfoGroup {
     List<OmKeyLocationInfo> locationInfoList = createLocationList();
     OmKeyLocationInfoGroup newInstance =
         testInstance.generateNextVersion(locationInfoList);
-    Assert.assertEquals(1, newInstance.getLocationList().size());
+    Assertions.assertEquals(1, newInstance.getLocationList().size());
     // createTestInstance is of version 2, nextVersion should be 3
-    Assert.assertEquals(3, newInstance.getVersion());
+    Assertions.assertEquals(3, newInstance.getVersion());
 
   }
 
