@@ -17,8 +17,8 @@
  */
 package org.apache.hadoop.ozone.om.helpers;
 
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Test;
 
 /**
  * Test utilities inside OmMutipartUpload.
@@ -31,9 +31,9 @@ public class TestOmMultipartUpload {
         OmMultipartUpload.getDbKey("vol1", "bucket1", "dir1/key1", "uploadId");
     OmMultipartUpload info = OmMultipartUpload.from(key1);
 
-    Assert.assertEquals("vol1", info.getVolumeName());
-    Assert.assertEquals("bucket1", info.getBucketName());
-    Assert.assertEquals("dir1/key1", info.getKeyName());
-    Assert.assertEquals("uploadId", info.getUploadId());
+    Assertions.assertEquals("vol1", info.getVolumeName());
+    Assertions.assertEquals("bucket1", info.getBucketName());
+    Assertions.assertEquals("dir1/key1", info.getKeyName());
+    Assertions.assertEquals("uploadId", info.getUploadId());
   }
-}
\ No newline at end of file
+}
