@@ -34,7 +34,6 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.hdds.client.DefaultReplicationConfig;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
-import org.apache.hadoop.hdds.client.ReplicationType;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.StorageType;
 import org.apache.hadoop.io.IOUtils;
@@ -46,28 +45,25 @@ import org.apache.hadoop.ozone.client.BucketArgs;
 import org.apache.hadoop.ozone.client.OzoneBucket;
 
 import org.apache.commons.lang3.RandomStringUtils;
+import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.om.OMConfigKeys;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
-import org.junit.AfterClass;
-import org.junit.Assert;
-import org.junit.BeforeClass;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.Timeout;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
+import org.junit.jupiter.api.Assertions;
 
 import static org.apache.hadoop.hdds.StringUtils.string2Bytes;
 
 /**
  * Test OzoneFSInputStream by reading through multiple interfaces.
  */
+@Timeout(300)
 public class TestOzoneFSInputStream {
 
-  /**
-    * Set a timeout for each test.
-    */
-  @Rule
-  public Timeout timeout = Timeout.seconds(300);
   private static MiniOzoneCluster cluster = null;
+  private static OzoneClient client;
   private static FileSystem fs;
   private static FileSystem ecFs;
   private static Path filePath = null;
@@ -81,7 +77,7 @@ public class TestOzoneFSInputStream {
    *
    * @throws IOException
    */
-  @BeforeClass
+  @BeforeAll
   public static void init() throws Exception {
     conf = new OzoneConfiguration();
     conf.set(OMConfigKeys.OZONE_DEFAULT_BUCKET_LAYOUT,
@@ -94,9 +90,10 @@ public class TestOzoneFSInputStream {
         .setStreamBufferMaxSize(4) // MB
         .build();
     cluster.waitForClusterToBeReady();
+    client = cluster.newClient();
 
     // create a volume and a bucket to be used by OzoneFileSystem
-    OzoneBucket bucket = TestDataUtil.createVolumeAndBucket(cluster);
+    OzoneBucket bucket = TestDataUtil.createVolumeAndBucket(client);
 
     // Set the fs.defaultFS and start the filesystem
     String uri = String.format("%s://%s.%s/",
@@ -115,12 +112,12 @@ public class TestOzoneFSInputStream {
     builder.setStorageType(StorageType.DISK);
     builder.setBucketLayout(BucketLayout.FILE_SYSTEM_OPTIMIZED);
     builder.setDefaultReplicationConfig(
-        new DefaultReplicationConfig(ReplicationType.EC,
+        new DefaultReplicationConfig(
             new ECReplicationConfig(3, 2, ECReplicationConfig.EcCodec.RS,
-                1024)));
+                (int) OzoneConsts.MB)));
     BucketArgs omBucketArgs = builder.build();
     String ecBucket = UUID.randomUUID().toString();
-    TestDataUtil.createBucket(cluster, bucket.getVolumeName(), omBucketArgs,
+    TestDataUtil.createBucket(client, bucket.getVolumeName(), omBucketArgs,
         ecBucket);
     String ecUri = String.format("%s://%s.%s/",
         OzoneConsts.OZONE_URI_SCHEME, ecBucket, bucket.getVolumeName());
@@ -131,8 +128,9 @@ public class TestOzoneFSInputStream {
   /**
    * Shutdown MiniDFSCluster.
    */
-  @AfterClass
+  @AfterAll
   public static void shutdown() throws IOException {
+    IOUtils.cleanupWithLogger(null, client);
     fs.close();
     ecFs.close();
     cluster.shutdown();
@@ -149,11 +147,11 @@ public class TestOzoneFSInputStream {
           break;
         }
         value[i] = (byte) val;
-        Assert.assertEquals("value mismatch at:" + i, value[i], data[i]);
+        Assertions.assertEquals(value[i], data[i], "value mismatch at:" + i);
         i++;
       }
-      Assert.assertEquals(i, data.length);
-      Assert.assertArrayEquals(value, data);
+      Assertions.assertEquals(i, data.length);
+      Assertions.assertArrayEquals(value, data);
     }
   }
 
@@ -171,8 +169,8 @@ public class TestOzoneFSInputStream {
         System.arraycopy(tmp, 0, value, i * tmp.length, tmp.length);
         i++;
       }
-      Assert.assertEquals((long) i * tmp.length, data.length);
-      Assert.assertArrayEquals(value, data);
+      Assertions.assertEquals((long) i * tmp.length, data.length);
+      Assertions.assertArrayEquals(value, data);
     }
   }
 
@@ -183,12 +181,12 @@ public class TestOzoneFSInputStream {
       ByteBuffer buffer = ByteBuffer.allocate(1024 * 1024);
       int byteRead = inputStream.read(buffer);
 
-      Assert.assertEquals(byteRead, 1024 * 1024);
+      Assertions.assertEquals(byteRead, 1024 * 1024);
 
       byte[] value = new byte[1024 * 1024];
       System.arraycopy(data, 0, value, 0, value.length);
 
-      Assert.assertArrayEquals(value, buffer.array());
+      Assertions.assertArrayEquals(value, buffer.array());
     }
   }
 
@@ -210,7 +208,7 @@ public class TestOzoneFSInputStream {
     in.sync(0);
     blockStart = in.getPosition();
     // The behavior should be consistent with HDFS
-    Assert.assertEquals(srcfile.length(), blockStart);
+    Assertions.assertEquals(srcfile.length(), blockStart);
     in.close();
   }
 
@@ -232,7 +230,7 @@ public class TestOzoneFSInputStream {
     in.sync(0);
     blockStart = in.getPosition();
     // The behavior should be consistent with HDFS
-    Assert.assertEquals(srcfile.length(), blockStart);
+    Assertions.assertEquals(srcfile.length(), blockStart);
     in.close();
   }
 }
