@@ -29,7 +29,6 @@ import java.util.UUID;
 
 import org.apache.hadoop.conf.StorageUnit;
 import org.apache.hadoop.fs.BlockLocation;
-import org.apache.hadoop.fs.CommonConfigurationKeysPublic;
 import org.apache.hadoop.fs.FSDataInputStream;
 import org.apache.hadoop.fs.FSDataOutputStream;
 import org.apache.hadoop.fs.FileStatus;
@@ -58,22 +57,28 @@ import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.hadoop.util.Time;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
-import org.apache.commons.io.IOUtils;
+import org.apache.hadoop.hdds.utils.IOUtils;
 import org.apache.commons.lang3.RandomStringUtils;
+
+import static org.apache.hadoop.fs.CommonConfigurationKeysPublic.FS_DEFAULT_NAME_KEY;
 import static org.apache.hadoop.fs.ozone.Constants.OZONE_DEFAULT_USER;
 
 import org.junit.After;
 import org.junit.AfterClass;
-import org.junit.Assert;
 
+import static org.apache.hadoop.ozone.om.helpers.BucketLayout.FILE_SYSTEM_OPTIMIZED;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
+import static org.junit.Assume.assumeFalse;
+
 import org.junit.Before;
-import org.junit.Ignore;
 import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.TestRule;
 import org.junit.rules.Timeout;
+import org.apache.ozone.test.JUnit5AwareTimeout;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 import org.junit.runners.Parameterized.Parameters;
@@ -91,7 +96,7 @@ public class TestOzoneFileInterfaces {
     * Set a timeout for each test.
     */
   @Rule
-  public Timeout timeout = Timeout.seconds(300);
+  public TestRule timeout = new JUnit5AwareTimeout(Timeout.seconds(300));
 
   private String rootPath;
 
@@ -176,15 +181,17 @@ public class TestOzoneFileInterfaces {
     OzoneConfiguration conf = cluster.getConf();
 
     // create a volume and a bucket to be used by OzoneFileSystem
-    TestDataUtil.createVolumeAndBucket(cluster, volumeName, bucketName,
-        getBucketLayout());
+    try (OzoneClient client = cluster.newClient()) {
+      TestDataUtil.createVolumeAndBucket(client, volumeName, bucketName,
+          getBucketLayout());
+    }
 
     rootPath = String
         .format("%s://%s.%s/", OzoneConsts.OZONE_URI_SCHEME, bucketName,
             volumeName);
     if (setDefaultFs) {
       // Set the fs.defaultFS and start the filesystem
-      conf.set(CommonConfigurationKeysPublic.FS_DEFAULT_NAME_KEY, rootPath);
+      conf.set(FS_DEFAULT_NAME_KEY, rootPath);
       fs = FileSystem.get(conf);
     } else {
       fs = FileSystem.get(new URI(rootPath + "/test.txt"), conf);
@@ -215,6 +222,8 @@ public class TestOzoneFileInterfaces {
 
   @Test
   public void testFileSystemInit() throws IOException {
+    assumeFalse(FILE_SYSTEM_OPTIMIZED.equals(getBucketLayout()));
+
     if (setDefaultFs) {
       assertTrue(
           "The initialized file system is not OzoneFileSystem but " +
@@ -227,6 +236,8 @@ public class TestOzoneFileInterfaces {
 
   @Test
   public void testOzFsReadWrite() throws IOException {
+    assumeFalse(FILE_SYSTEM_OPTIMIZED.equals(getBucketLayout()));
+
     long currentTime = Time.now();
     int stringLen = 20;
     OMMetadataManager metadataManager = cluster.getOzoneManager()
@@ -313,6 +324,8 @@ public class TestOzoneFileInterfaces {
 
   @Test
   public void testReplication() throws IOException {
+    assumeFalse(FILE_SYSTEM_OPTIMIZED.equals(getBucketLayout()));
+
     int stringLen = 20;
     String data = RandomStringUtils.randomAlphanumeric(stringLen);
     String filePath = RandomStringUtils.randomAlphanumeric(5);
@@ -339,6 +352,8 @@ public class TestOzoneFileInterfaces {
 
   @Test
   public void testDirectory() throws IOException {
+    assumeFalse(FILE_SYSTEM_OPTIMIZED.equals(getBucketLayout()));
+
     String leafName = RandomStringUtils.randomAlphanumeric(5);
     OMMetadataManager metadataManager = cluster.getOzoneManager()
         .getMetadataManager();
@@ -451,7 +466,7 @@ public class TestOzoneFileInterfaces {
         cluster.getOzoneManager().getMetrics().getNumGetFileStatus();
     FileStatus status = fs.getFileStatus(path);
 
-    Assert.assertEquals(numFileStatus + 1,
+    assertEquals(numFileStatus + 1,
         cluster.getOzoneManager().getMetrics().getNumGetFileStatus());
     assertTrue(status.isDirectory());
     assertEquals(FsPermission.getDirDefault(), status.getPermission());
@@ -466,7 +481,7 @@ public class TestOzoneFileInterfaces {
     OzoneFileStatus omStatus =
         cluster.getOzoneManager().getFileStatus(keyArgs);
     //Another get file status here, incremented the counter.
-    Assert.assertEquals(numFileStatus + 2,
+    assertEquals(numFileStatus + 2,
         cluster.getOzoneManager().getMetrics().getNumGetFileStatus());
 
     assertTrue("The created path is not directory.", omStatus.isDirectory());
@@ -500,7 +515,6 @@ public class TestOzoneFileInterfaces {
   }
 
   @Test
-  @Ignore("HDDS-3506")
   public void testOzoneManagerLocatedFileStatusBlockOffsetsWithMultiBlockFile()
       throws Exception {
     // naive assumption: MiniOzoneCluster will not have larger than ~1GB
@@ -531,6 +545,7 @@ public class TestOzoneFileInterfaces {
 
   @Test
   public void testPathToKey() throws Exception {
+    assumeFalse(FILE_SYSTEM_OPTIMIZED.equals(getBucketLayout()));
 
     assertEquals("a/b/1", o3fs.pathToKey(new Path("/a/b/1")));
 
@@ -551,31 +566,32 @@ public class TestOzoneFileInterfaces {
   public void testFileSystemWithObjectStoreLayout() throws IOException {
     String obsVolume = UUID.randomUUID().toString();
 
-    OzoneClient client = cluster.getClient();
-    ObjectStore store = client.getObjectStore();
+    try (OzoneClient client = cluster.newClient()) {
+      ObjectStore store = client.getObjectStore();
 
-    // Create volume and bucket
-    store.createVolume(obsVolume);
-    OzoneVolume volume = store.getVolume(obsVolume);
-    String obsBucket = UUID.randomUUID().toString();
-    // create bucket with OBJECT_STORE bucket layout (incompatible with fs)
-    volume.createBucket(obsBucket,
-        BucketArgs.newBuilder().setBucketLayout(BucketLayout.OBJECT_STORE)
-            .build());
+      // Create volume and bucket
+      store.createVolume(obsVolume);
+      OzoneVolume volume = store.getVolume(obsVolume);
+      String obsBucket = UUID.randomUUID().toString();
+      // create bucket with OBJECT_STORE bucket layout (incompatible with fs)
+      volume.createBucket(obsBucket,
+          BucketArgs.newBuilder().setBucketLayout(BucketLayout.OBJECT_STORE)
+              .build());
 
-    String obsRootPath = String.format("%s://%s.%s/",
-        OzoneConsts.OZONE_URI_SCHEME, obsBucket, obsVolume);
+      String obsRootPath = String.format("%s://%s.%s/",
+          OzoneConsts.OZONE_URI_SCHEME, obsBucket, obsVolume);
 
-    OzoneConfiguration config = (OzoneConfiguration) fs.getConf();
-    config.set(CommonConfigurationKeysPublic.FS_DEFAULT_NAME_KEY, obsRootPath);
+      OzoneConfiguration config = (OzoneConfiguration) fs.getConf();
+      config.set(FS_DEFAULT_NAME_KEY, obsRootPath);
 
-    try {
-      fs = FileSystem.get(fs.getConf());
-      Assert.fail("Should throw Exception due incompatible bucket layout");
-    } catch (IllegalArgumentException iae) {
-      // Expected exception
-      Assert.assertTrue(iae.getMessage().contains(
-          "OBJECT_STORE, which does not support file system semantics"));
+      try {
+        fs = FileSystem.get(fs.getConf());
+        fail("Should throw Exception due incompatible bucket layout");
+      } catch (IllegalArgumentException iae) {
+        // Expected exception
+        assertTrue(iae.getMessage().contains(
+            "OBJECT_STORE, which does not support file system semantics"));
+      }
     }
   }
 
