@@ -20,21 +20,21 @@ package org.apache.hadoop.fs.ozone;
 
 import org.apache.hadoop.fs.FSDataOutputStream;
 import org.apache.hadoop.fs.FileStatus;
+import org.apache.hadoop.fs.LeaseRecoverable;
 import org.apache.hadoop.fs.LocatedFileStatus;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.fs.RemoteIterator;
 import org.apache.hadoop.fs.contract.ContractTestUtils;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
+import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmDirectoryInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.ozone.test.GenericTestUtils;
-import org.junit.Assert;
 import org.junit.After;
 import org.junit.BeforeClass;
-import org.junit.Ignore;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
@@ -48,7 +48,10 @@ import java.util.Arrays;
 import java.util.Collection;
 
 import static org.junit.Assert.assertEquals;
+import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNotNull;
+import static org.junit.Assert.assertThrows;
+import static org.junit.Assert.assertTrue;
 import static org.junit.Assert.fail;
 
 /**
@@ -80,38 +83,11 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
   @Override
   public void cleanup() {
     super.cleanup();
-    try {
-      deleteRootDir();
-    } catch (IOException e) {
-      LOG.info("Failed to cleanup DB tables.", e);
-      fail("Failed to cleanup DB tables." + e.getMessage());
-    }
   }
 
   private static final Logger LOG =
       LoggerFactory.getLogger(TestOzoneFileSystemWithFSO.class);
 
-  @Override
-  @Test
-  @Ignore("HDDS-2939")
-  public void testGetDirectoryModificationTime() {
-    // ignore as this is not relevant to PREFIX layout changes
-  }
-
-  @Override
-  @Test
-  @Ignore("HDDS-2939")
-  public void testOzoneFsServiceLoader() {
-    // ignore as this is not relevant to PREFIX layout changes
-  }
-
-  @Override
-  @Test
-  @Ignore("HDDS-2939")
-  public void testCreateWithInvalidPaths() {
-    // ignore as this is not relevant to PREFIX layout changes
-  }
-
   @Test
   public void testListStatusWithoutRecursiveSearch() throws Exception {
     /*
@@ -319,7 +295,7 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
     final Path sourceRoot = new Path(getFs().getUri().toString() + root);
     LOG.info("Rename op-> source:{} to destin:{}", sourceRoot, subDir1);
     //  rename should fail and return false
-    Assert.assertFalse(getFs().rename(sourceRoot, subDir1));
+    assertFalse(getFs().rename(sourceRoot, subDir1));
   }
 
   /**
@@ -339,25 +315,115 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
             + root + "/b/c");
 
     // rename should fail and return false
-    Assert.assertFalse(getFs().rename(dir2SourcePath, destinPath));
+    assertFalse(getFs().rename(dir2SourcePath, destinPath));
     // (b) parent of dst is a file. /root_dir/file1/c
     Path filePath = new Path(getFs().getUri().toString() + root + "/file1");
     ContractTestUtils.touch(getFs(), filePath);
     Path newDestinPath = new Path(filePath, "c");
     // rename should fail and return false
-    Assert.assertFalse(getFs().rename(dir2SourcePath, newDestinPath));
+    assertFalse(getFs().rename(dir2SourcePath, newDestinPath));
   }
 
-  @Override
   @Test
-  @Ignore("TODO:HDDS-2939")
-  public void testListStatusWithIntermediateDir() throws Exception {
+  public void testRenameParentDirModificationTime() throws IOException {
+    Path dir1 = new Path(getFs().getUri().toString(), "/dir1");
+    Path file1 = new Path(dir1, "file1");
+    Path dir2 = new Path(getFs().getUri().toString(), "/dir2");
+    // mv "/dir1/file1" to "/dir2/file1"
+    Path renamedFile1 = new Path(dir2, "file1");
+    getFs().mkdirs(dir1);
+    getFs().create(file1, false).close();
+    getFs().mkdirs(dir2);
+
+    long dir1BeforeMTime = getFs().getFileStatus(dir1).getModificationTime();
+    long dir2BeforeMTime = getFs().getFileStatus(dir2).getModificationTime();
+    long file1BeforeMTime = getFs().getFileStatus(file1).getModificationTime();
+    getFs().rename(file1, renamedFile1);
+    long dir1AfterMTime = getFs().getFileStatus(dir1).getModificationTime();
+    long dir2AfterMTime = getFs().getFileStatus(dir2).getModificationTime();
+    long file1AfterMTime = getFs().getFileStatus(renamedFile1)
+        .getModificationTime();
+    // rename should change the parent directory of source and object files
+    // modification time but not change modification time of the renamed file
+    assertTrue(dir1BeforeMTime < dir1AfterMTime);
+    assertTrue(dir2BeforeMTime < dir2AfterMTime);
+    assertEquals(file1BeforeMTime, file1AfterMTime);
+
+    // mv "/dir1/subdir1/" to "/dir2/subdir1/"
+    Path subdir1 = new Path(dir1, "subdir1");
+    Path renamedSubdir1 = new Path(dir2, "subdir1");
+    getFs().mkdirs(subdir1);
+
+    dir1BeforeMTime = getFs().getFileStatus(dir1).getModificationTime();
+    dir2BeforeMTime = getFs().getFileStatus(dir2).getModificationTime();
+    long subdir1BeforeMTime = getFs().getFileStatus(subdir1)
+        .getModificationTime();
+    getFs().rename(subdir1, renamedSubdir1);
+    dir1AfterMTime = getFs().getFileStatus(dir1).getModificationTime();
+    dir2AfterMTime = getFs().getFileStatus(dir2).getModificationTime();
+    long subdir1AfterMTime = getFs().getFileStatus(renamedSubdir1)
+        .getModificationTime();
+    assertTrue(dir1BeforeMTime < dir1AfterMTime);
+    assertTrue(dir2BeforeMTime < dir2AfterMTime);
+    assertEquals(subdir1BeforeMTime, subdir1AfterMTime);
   }
 
-  @Override
   @Test
-  @Ignore("TODO:HDDS-5012")
-  public void testListStatusOnLargeDirectory() throws Exception {
+  public void testRenameParentBucketModificationTime() throws IOException {
+    OMMetadataManager omMgr =
+        getCluster().getOzoneManager().getMetadataManager();
+
+    // mv /file1 -> /renamedFile1, the bucket mtime should be changed
+    Path file1 = new Path("/file1");
+    Path renamedFile1 = new Path("/renamedFile1");
+    getFs().create(file1, false).close();
+    renameAndAssert(omMgr, file1, renamedFile1, true);
+
+    // mv /dir1/subFile2 -> /dir2/renamedSubFile2,
+    // the bucket mtime should not be changed
+    Path dir1 = new Path(getFs().getUri().toString(), "/dir1");
+    Path subFile2 = new Path(dir1, "subFile2");
+    Path dir2 = new Path(getFs().getUri().toString(), "/dir2");
+    Path renamedSubFile2 = new Path(dir2, "renamedSubFile2");
+    getFs().mkdirs(dir1);
+    getFs().mkdirs(dir2);
+    getFs().create(subFile2, false).close();
+    renameAndAssert(omMgr, subFile2, renamedSubFile2, false);
+
+    // mv /dir3/subFile3 -> "/renamedFile3"  the bucket mtime should be changed
+    Path dir3 = new Path(getFs().getUri().toString(), "/dir3");
+    Path subFile3 = new Path(dir3, "subFile3");
+    Path renamedFile3 = new Path("/renamedFile3");
+    getFs().mkdirs(dir3);
+    getFs().create(subFile3, false).close();
+    renameAndAssert(omMgr, subFile3, renamedFile3, true);
+
+    // mv /file4 -> "/dir4/renamedFile4"  the bucket mtime should be changed
+    Path file4 = new Path("/file4");
+    Path dir4 = new Path(getFs().getUri().toString(), "/dir4");
+    Path renamedSubFile4 = new Path(dir4, "subFile3");
+    getFs().mkdirs(dir4);
+    getFs().create(file4, false).close();
+    renameAndAssert(omMgr, file4, renamedSubFile4, true);
+  }
+
+  private void renameAndAssert(OMMetadataManager omMgr,
+      Path from, Path to, boolean exceptChangeMtime) throws IOException {
+    OmBucketInfo omBucketInfo = omMgr.getBucketTable()
+        .get(omMgr.getBucketKey(getVolumeName(), getBucketName()));
+    long bucketBeforeMTime = omBucketInfo.getModificationTime();
+    long fileBeforeMTime = getFs().getFileStatus(from).getModificationTime();
+    getFs().rename(from, to);
+    omBucketInfo = omMgr.getBucketTable()
+        .get(omMgr.getBucketKey(getVolumeName(), getBucketName()));
+    long bucketAfterMTime = omBucketInfo.getModificationTime();
+    long fileAfterMTime = getFs().getFileStatus(to).getModificationTime();
+    if (exceptChangeMtime) {
+      assertTrue(bucketBeforeMTime < bucketAfterMTime);
+    } else {
+      assertEquals(bucketBeforeMTime, bucketAfterMTime);
+    }
+    assertEquals(fileBeforeMTime, fileAfterMTime);
   }
 
   @Test
@@ -365,7 +431,7 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
     // reset metrics
     long numKeys = getCluster().getOzoneManager().getMetrics().getNumKeys();
     getCluster().getOzoneManager().getMetrics().decNumKeys(numKeys);
-    Assert.assertEquals(0,
+    assertEquals(0,
         getCluster().getOzoneManager().getMetrics().getNumKeys());
 
     // Op 1. create dir -> /d1/d2/d3/d4/
@@ -378,7 +444,7 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
         getCluster().getOzoneManager().getMetadataManager();
     OmBucketInfo omBucketInfo = omMgr.getBucketTable()
         .get(omMgr.getBucketKey(getVolumeName(), getBucketName()));
-    Assert.assertNotNull("Failed to find bucketInfo", omBucketInfo);
+    assertNotNull("Failed to find bucketInfo", omBucketInfo);
 
     final long volumeId = omMgr.getVolumeId(getVolumeName());
     final long bucketId = omMgr.getBucketId(getVolumeName(), getBucketName());
@@ -396,7 +462,7 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
         verifyDirKey(volumeId, bucketId, d3ObjectID,
                 "d4", "/d1/d2/d3/d4", dirKeys, omMgr);
 
-    Assert.assertEquals("Wrong OM numKeys metrics", 4,
+    assertEquals("Wrong OM numKeys metrics", 4,
         getCluster().getOzoneManager().getMetrics().getNumKeys());
 
     // create sub-dirs under same parent
@@ -410,11 +476,11 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
     long d6ObjectID =
         verifyDirKey(volumeId, bucketId, d4ObjectID,
                 "d6", "/d1/d2/d3/d4/d6", dirKeys, omMgr);
-    Assert.assertTrue(
+    assertTrue(
         "Wrong objectIds for sub-dirs[" + d5ObjectID + "/d5, " + d6ObjectID
             + "/d6] of same parent!", d5ObjectID != d6ObjectID);
 
-    Assert.assertEquals("Wrong OM numKeys metrics", 6,
+    assertEquals("Wrong OM numKeys metrics", 6,
         getCluster().getOzoneManager().getMetrics().getNumKeys());
   }
 
@@ -430,7 +496,7 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
         getCluster().getOzoneManager().getMetadataManager();
     OmBucketInfo omBucketInfo = omMgr.getBucketTable()
         .get(omMgr.getBucketKey(getVolumeName(), getBucketName()));
-    Assert.assertNotNull("Failed to find bucketInfo", omBucketInfo);
+    assertNotNull("Failed to find bucketInfo", omBucketInfo);
 
     ArrayList<String> dirKeys = new ArrayList<>();
 
@@ -450,7 +516,7 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
     outputStream.close();
 
     OmKeyInfo omKeyInfo = omMgr.getKeyTable(getBucketLayout()).get(openFileKey);
-    Assert.assertNotNull("Invalid Key!", omKeyInfo);
+    assertNotNull("Invalid Key!", omKeyInfo);
     verifyOMFileInfoFormat(omKeyInfo, file.getName(), d2ObjectID);
 
     // wait for DB updates
@@ -459,19 +525,57 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
         return omMgr.getOpenKeyTable(getBucketLayout()).isEmpty();
       } catch (IOException e) {
         LOG.error("DB failure!", e);
-        Assert.fail("DB failure!");
+        fail("DB failure!");
         return false;
       }
     }, 1000, 120000);
   }
 
+  /**
+   * Verify recoverLease() and isFileClosed() APIs.
+   * @throws Exception
+   */
+  @Test
+  public void testLeaseRecoverable() throws Exception {
+    // Create a file
+    Path parent = new Path("/d1/d2/");
+    Path source = new Path(parent, "file1");
+
+    LeaseRecoverable fs = (LeaseRecoverable)getFs();
+    FSDataOutputStream stream = getFs().create(source);
+    try {
+      // file not visible yet
+      assertThrows(OMException.class, () -> fs.isFileClosed(source));
+      stream.write(1);
+      stream.hsync();
+      // file is visible and open
+      assertFalse(fs.isFileClosed(source));
+      assertTrue(fs.recoverLease(source));
+      // file is closed after lease recovery
+      assertTrue(fs.isFileClosed(source));
+    } finally {
+      TestLeaseRecovery.closeIgnoringKeyNotFound(stream);
+    }
+  }
+
+  @Test
+  public void testFSDeleteLogWarnNoExist() throws Exception {
+    GenericTestUtils.LogCapturer logCapture = GenericTestUtils.LogCapturer
+        .captureLogs(BasicOzoneClientAdapterImpl.LOG);
+    getFs().delete(new Path("/d1/d3/noexist/"), true);
+    assertTrue(logCapture.getOutput().contains(
+        "delete key failed Unable to get file status"));
+    assertTrue(logCapture.getOutput().contains(
+        "WARN  ozone.BasicOzoneClientAdapterImpl"));
+  }
+
   private void verifyOMFileInfoFormat(OmKeyInfo omKeyInfo, String fileName,
       long parentID) {
-    Assert.assertEquals("Wrong keyName", fileName, omKeyInfo.getKeyName());
-    Assert.assertEquals("Wrong parentID", parentID,
+    assertEquals("Wrong keyName", fileName, omKeyInfo.getKeyName());
+    assertEquals("Wrong parentID", parentID,
         omKeyInfo.getParentObjectID());
     String dbKey = parentID + OzoneConsts.OM_KEY_PREFIX + fileName;
-    Assert.assertEquals("Wrong path format", dbKey, omKeyInfo.getPath());
+    assertEquals("Wrong path format", dbKey, omKeyInfo.getPath());
   }
 
   long verifyDirKey(long volumeId, long bucketId, long parentId,
@@ -482,15 +586,15 @@ public class TestOzoneFileSystemWithFSO extends TestOzoneFileSystem {
             parentId + "/" + dirKey;
     dirKeys.add(dbKey);
     OmDirectoryInfo dirInfo = omMgr.getDirectoryTable().get(dbKey);
-    Assert.assertNotNull("Failed to find " + absolutePath +
+    assertNotNull("Failed to find " + absolutePath +
         " using dbKey: " + dbKey, dirInfo);
-    Assert.assertEquals("Parent Id mismatches", parentId,
+    assertEquals("Parent Id mismatches", parentId,
         dirInfo.getParentObjectID());
-    Assert.assertEquals("Mismatches directory name", dirKey,
+    assertEquals("Mismatches directory name", dirKey,
         dirInfo.getName());
-    Assert.assertTrue("Mismatches directory creation time param",
+    assertTrue("Mismatches directory creation time param",
         dirInfo.getCreationTime() > 0);
-    Assert.assertEquals("Mismatches directory modification time param",
+    assertEquals("Mismatches directory modification time param",
         dirInfo.getCreationTime(), dirInfo.getModificationTime());
     return dirInfo.getObjectID();
   }
