@@ -16,9 +16,6 @@
  */
 package org.apache.hadoop.hdds.scm.container;
 
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -38,23 +35,26 @@ import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.ozone.common.statemachine.InvalidStateTransitionException;
 import org.apache.hadoop.ozone.container.common.SCMTestUtils;
 import org.apache.hadoop.security.authentication.client.AuthenticationException;
-import org.junit.After;
+import org.junit.jupiter.api.AfterEach;
 import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Ignore;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeEach;
+import org.apache.ozone.test.tag.Flaky;
+import org.junit.jupiter.api.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.Executor;
+import java.util.concurrent.Executors;
 import java.util.concurrent.TimeoutException;
 
+import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_DATANODE_PIPELINE_LIMIT;
+
 /**
  * Tests for ContainerStateManager.
  */
-@Ignore
 public class TestContainerStateManagerIntegration {
 
   private static final Logger LOG =
@@ -64,14 +64,14 @@ public class TestContainerStateManagerIntegration {
   private MiniOzoneCluster cluster;
   private StorageContainerManager scm;
   private ContainerManager containerManager;
-  private ContainerStateManagerV2 containerStateManager;
+  private ContainerStateManager containerStateManager;
   private int numContainerPerOwnerInPipeline;
-  private  Set<ContainerID> excludedContainerIDS;
 
 
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
     conf = new OzoneConfiguration();
+    conf.setInt(OZONE_DATANODE_PIPELINE_LIMIT, 1);
     numContainerPerOwnerInPipeline =
         conf.getInt(ScmConfigKeys.OZONE_SCM_PIPELINE_OWNER_CONTAINER_COUNT,
             ScmConfigKeys.OZONE_SCM_PIPELINE_OWNER_CONTAINER_COUNT_DEFAULT);
@@ -82,10 +82,9 @@ public class TestContainerStateManagerIntegration {
     containerManager = scm.getContainerManager();
     containerStateManager = containerManager
         .getContainerStateManager();
-    excludedContainerIDS = new HashSet<>();
   }
 
-  @After
+  @AfterEach
   public void cleanUp() {
     if (cluster != null) {
       cluster.shutdown();
@@ -100,7 +99,7 @@ public class TestContainerStateManagerIntegration {
             SCMTestUtils.getReplicationFactor(conf), OzoneConsts.OZONE);
     ContainerInfo info = containerManager
         .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-            container1.getPipeline(), excludedContainerIDS);
+            container1.getPipeline());
     Assert.assertNotEquals(container1.getContainerInfo().getContainerID(),
         info.getContainerID());
     Assert.assertEquals(OzoneConsts.OZONE, info.getOwner());
@@ -128,7 +127,7 @@ public class TestContainerStateManagerIntegration {
             SCMTestUtils.getReplicationFactor(conf), OzoneConsts.OZONE);
     ContainerInfo info = containerManager
         .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-            container1.getPipeline(), excludedContainerIDS);
+            container1.getPipeline());
     Assert.assertNotNull(info);
 
     String newContainerOwner = "OZONE_NEW";
@@ -137,7 +136,7 @@ public class TestContainerStateManagerIntegration {
             SCMTestUtils.getReplicationFactor(conf), newContainerOwner);
     ContainerInfo info2 = containerManager
         .getMatchingContainer(OzoneConsts.GB * 3, newContainerOwner,
-            container1.getPipeline(), excludedContainerIDS);
+            container1.getPipeline());
     Assert.assertNotNull(info2);
 
     Assert.assertNotEquals(info.containerID(), info2.containerID());
@@ -167,7 +166,7 @@ public class TestContainerStateManagerIntegration {
     cluster.restartStorageContainerManager(false);
 
     List<ContainerInfo> result = cluster.getStorageContainerManager()
-        .getContainerManager().getContainers(null, 100);
+        .getContainerManager().getContainers();
 
     long matchCount = result.stream()
         .filter(info ->
@@ -209,7 +208,7 @@ public class TestContainerStateManagerIntegration {
     for (int i = 1; i < numContainerPerOwnerInPipeline; i++) {
       ContainerInfo info = containerManager
           .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-              container1.getPipeline(), excludedContainerIDS);
+              container1.getPipeline());
       Assert.assertTrue(info.getContainerID() > cid);
       cid = info.getContainerID();
     }
@@ -218,67 +217,13 @@ public class TestContainerStateManagerIntegration {
     // next container should be the same as first container
     ContainerInfo info = containerManager
         .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-            container1.getPipeline(), excludedContainerIDS);
+            container1.getPipeline());
     Assert.assertEquals(container1.getContainerInfo().getContainerID(),
         info.getContainerID());
   }
 
   @Test
-  public void testGetMatchingContainerWithExcludedList() throws IOException {
-    long cid;
-    ContainerWithPipeline container1 = scm.getClientProtocolServer().
-        allocateContainer(SCMTestUtils.getReplicationType(conf),
-            SCMTestUtils.getReplicationFactor(conf), OzoneConsts.OZONE);
-    cid = container1.getContainerInfo().getContainerID();
-
-    // each getMatchingContainer call allocates a container in the
-    // pipeline till the pipeline has numContainerPerOwnerInPipeline number of
-    // containers.
-    for (int i = 1; i < numContainerPerOwnerInPipeline; i++) {
-      ContainerInfo info = containerManager
-          .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-              container1.getPipeline(), excludedContainerIDS);
-      Assert.assertTrue(info.getContainerID() > cid);
-      cid = info.getContainerID();
-    }
-
-    // At this point there are already three containers in the pipeline.
-    // next container should be the same as first container
-    ContainerInfo info = containerManager
-        .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-            container1.getPipeline(),
-            new HashSet<>(Collections.singletonList(ContainerID.valueOf(1))));
-    Assert.assertNotEquals(container1.getContainerInfo().getContainerID(),
-        info.getContainerID());
-  }
-
-
-  @Test
-  public void testCreateContainerLogicWithExcludedList() throws IOException {
-    long cid;
-    ContainerWithPipeline container1 = scm.getClientProtocolServer().
-        allocateContainer(SCMTestUtils.getReplicationType(conf),
-            SCMTestUtils.getReplicationFactor(conf), OzoneConsts.OZONE);
-    cid = container1.getContainerInfo().getContainerID();
-
-    for (int i = 1; i < numContainerPerOwnerInPipeline; i++) {
-      ContainerInfo info = containerManager
-          .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-              container1.getPipeline(), excludedContainerIDS);
-      Assert.assertTrue(info.getContainerID() > cid);
-      cid = info.getContainerID();
-    }
-
-    ContainerInfo info = containerManager
-        .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-            container1.getPipeline(),
-            new HashSet<>(Arrays.asList(ContainerID.valueOf(1),
-                ContainerID.valueOf(2), ContainerID.valueOf(3))));
-    Assert.assertEquals(info.getContainerID(), 4);
-  }
-
-  @Test
-  @Ignore("TODO:HDDS-1159")
+  @Flaky("HDDS-1159")
   public void testGetMatchingContainerMultipleThreads()
       throws IOException, InterruptedException {
     ContainerWithPipeline container1 = scm.getClientProtocolServer().
@@ -286,17 +231,19 @@ public class TestContainerStateManagerIntegration {
             SCMTestUtils.getReplicationFactor(conf), OzoneConsts.OZONE);
     Map<Long, Long> container2MatchedCount = new ConcurrentHashMap<>();
 
+    Executor executor = Executors.newFixedThreadPool(4);
+
     // allocate blocks using multiple threads
     int numBlockAllocates = 100000;
     for (int i = 0; i < numBlockAllocates; i++) {
       CompletableFuture.supplyAsync(() -> {
         ContainerInfo info = containerManager
             .getMatchingContainer(OzoneConsts.GB * 3, OzoneConsts.OZONE,
-                container1.getPipeline(), excludedContainerIDS);
+                container1.getPipeline());
         container2MatchedCount
             .compute(info.getContainerID(), (k, v) -> v == null ? 1L : v + 1);
         return null;
-      });
+      }, executor);
     }
 
     // make sure pipeline has has numContainerPerOwnerInPipeline number of
@@ -321,7 +268,7 @@ public class TestContainerStateManagerIntegration {
 
   @Test
   public void testUpdateContainerState() throws IOException,
-      InvalidStateTransitionException {
+      InvalidStateTransitionException, TimeoutException {
     Set<ContainerID> containerList = containerStateManager
         .getContainerIDs(HddsProtos.LifeCycleState.OPEN);
     int containers = containerList == null ? 0 : containerList.size();
@@ -397,9 +344,9 @@ public class TestContainerStateManagerIntegration {
 
     // Test 1: no replica's exist
     ContainerID containerID = ContainerID.valueOf(RandomUtils.nextLong());
-    Set<ContainerReplica> replicaSet;
-    containerStateManager.getContainerReplicas(containerID.getProtobuf());
-    Assert.fail();
+    Set<ContainerReplica> replicaSet =
+        containerStateManager.getContainerReplicas(containerID);
+    Assert.assertNull(replicaSet);
 
     ContainerWithPipeline container = scm.getClientProtocolServer()
         .allocateContainer(
@@ -419,44 +366,44 @@ public class TestContainerStateManagerIntegration {
         .setContainerState(ContainerReplicaProto.State.OPEN)
         .setDatanodeDetails(dn2)
         .build();
-    containerStateManager.updateContainerReplica(id.getProtobuf(), replicaOne);
-    containerStateManager.updateContainerReplica(id.getProtobuf(), replicaTwo);
-    replicaSet = containerStateManager.getContainerReplicas(id.getProtobuf());
+    containerStateManager.updateContainerReplica(id, replicaOne);
+    containerStateManager.updateContainerReplica(id, replicaTwo);
+    replicaSet = containerStateManager.getContainerReplicas(id);
     Assert.assertEquals(2, replicaSet.size());
     Assert.assertTrue(replicaSet.contains(replicaOne));
     Assert.assertTrue(replicaSet.contains(replicaTwo));
 
     // Test 3: Remove one replica node and then test
-    containerStateManager.removeContainerReplica(id.getProtobuf(), replicaOne);
-    replicaSet = containerStateManager.getContainerReplicas(id.getProtobuf());
+    containerStateManager.removeContainerReplica(id, replicaOne);
+    replicaSet = containerStateManager.getContainerReplicas(id);
     Assert.assertEquals(1, replicaSet.size());
     Assert.assertFalse(replicaSet.contains(replicaOne));
     Assert.assertTrue(replicaSet.contains(replicaTwo));
 
     // Test 3: Remove second replica node and then test
-    containerStateManager.removeContainerReplica(id.getProtobuf(), replicaTwo);
-    replicaSet = containerStateManager.getContainerReplicas(id.getProtobuf());
+    containerStateManager.removeContainerReplica(id, replicaTwo);
+    replicaSet = containerStateManager.getContainerReplicas(id);
     Assert.assertEquals(0, replicaSet.size());
     Assert.assertFalse(replicaSet.contains(replicaOne));
     Assert.assertFalse(replicaSet.contains(replicaTwo));
 
     // Test 4: Re-insert dn1
-    containerStateManager.updateContainerReplica(id.getProtobuf(), replicaOne);
-    replicaSet = containerStateManager.getContainerReplicas(id.getProtobuf());
+    containerStateManager.updateContainerReplica(id, replicaOne);
+    replicaSet = containerStateManager.getContainerReplicas(id);
     Assert.assertEquals(1, replicaSet.size());
     Assert.assertTrue(replicaSet.contains(replicaOne));
     Assert.assertFalse(replicaSet.contains(replicaTwo));
 
     // Re-insert dn2
-    containerStateManager.updateContainerReplica(id.getProtobuf(), replicaTwo);
-    replicaSet = containerStateManager.getContainerReplicas(id.getProtobuf());
+    containerStateManager.updateContainerReplica(id, replicaTwo);
+    replicaSet = containerStateManager.getContainerReplicas(id);
     Assert.assertEquals(2, replicaSet.size());
     Assert.assertTrue(replicaSet.contains(replicaOne));
     Assert.assertTrue(replicaSet.contains(replicaTwo));
 
     // Re-insert dn1
-    containerStateManager.updateContainerReplica(id.getProtobuf(), replicaOne);
-    replicaSet = containerStateManager.getContainerReplicas(id.getProtobuf());
+    containerStateManager.updateContainerReplica(id, replicaOne);
+    replicaSet = containerStateManager.getContainerReplicas(id);
     Assert.assertEquals(2, replicaSet.size());
     Assert.assertTrue(replicaSet.contains(replicaOne));
     Assert.assertTrue(replicaSet.contains(replicaTwo));
