@@ -20,6 +20,7 @@ package org.apache.hadoop.ozone.client.rpc;
 import java.io.File;
 import java.io.IOException;
 import java.net.URI;
+import java.nio.ByteBuffer;
 import java.nio.charset.StandardCharsets;
 import java.security.NoSuchAlgorithmException;
 import java.time.Instant;
@@ -31,7 +32,6 @@ import java.util.Map;
 import java.util.Random;
 import java.util.TreeMap;
 import java.util.UUID;
-import java.util.Collection;
 
 import com.google.common.cache.Cache;
 import org.apache.hadoop.conf.StorageUnit;
@@ -47,17 +47,20 @@ import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.protocolPB.StorageContainerLocationProtocolClientSideTranslatorPB;
+import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClientTestImpl;
+import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.MiniOzoneCluster;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.client.BucketArgs;
-import org.apache.hadoop.ozone.client.CertificateClientTestImpl;
 import org.apache.hadoop.ozone.client.ObjectStore;
 import org.apache.hadoop.ozone.client.OzoneBucket;
 import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientFactory;
 import org.apache.hadoop.ozone.client.OzoneKeyDetails;
 import org.apache.hadoop.ozone.client.OzoneVolume;
-import org.apache.hadoop.ozone.client.io.MultipartCryptoKeyInputStream;
+import org.apache.hadoop.hdds.scm.storage.MultipartInputStream;
+import org.apache.hadoop.ozone.client.io.OzoneDataStreamOutput;
+import org.apache.hadoop.ozone.client.SecretKeyTestClient;
 import org.apache.hadoop.ozone.client.io.OzoneInputStream;
 import org.apache.hadoop.ozone.client.io.OzoneOutputStream;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
@@ -75,27 +78,23 @@ import org.apache.ozone.test.GenericTestUtils;
 import static org.apache.hadoop.hdds.HddsConfigKeys.OZONE_METADATA_DIRS;
 import static org.apache.hadoop.hdds.client.ReplicationFactor.ONE;
 import static org.apache.hadoop.hdds.client.ReplicationType.RATIS;
+import static org.junit.jupiter.api.Assertions.assertArrayEquals;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
-import org.junit.AfterClass;
-import org.junit.Assert;
-import org.junit.BeforeClass;
-import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
+import org.apache.ozone.test.tag.Flaky;
+import org.junit.jupiter.api.AfterAll;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.EnumSource;
 import org.mockito.Mockito;
 
-/**
- * This class is to test all the public facing APIs of Ozone Client.
- */
-@RunWith(Parameterized.class)
-public class TestOzoneAtRestEncryption {
-
-  @Parameterized.Parameters
-  public static Collection<BucketLayout> data() {
-    return Arrays.asList(
-        BucketLayout.FILE_SYSTEM_OPTIMIZED,
-        BucketLayout.OBJECT_STORE);
-  }
+class TestOzoneAtRestEncryption {
 
   private static MiniOzoneCluster cluster = null;
   private static MiniKMS miniKMS;
@@ -118,19 +117,14 @@ public class TestOzoneAtRestEncryption {
   private static final int DEFAULT_CRYPTO_BUFFER_SIZE = 8 * 1024; // 8KB
   // (this is the default Crypto Buffer size as determined by the config
   // hadoop.security.crypto.buffer.size)
-  private final BucketLayout bucketLayout;
 
-  public TestOzoneAtRestEncryption(BucketLayout layout) {
-    bucketLayout = layout;
-  }
-
-  @BeforeClass
-  public static void init() throws Exception {
+  @BeforeAll
+  static void init() throws Exception {
     testDir = GenericTestUtils.getTestDir(
         TestSecureOzoneRpcClient.class.getSimpleName());
 
     File kmsDir = new File(testDir, UUID.randomUUID().toString());
-    Assert.assertTrue(kmsDir.mkdirs());
+    assertTrue(kmsDir.mkdirs());
     MiniKMS.Builder miniKMSBuilder = new MiniKMS.Builder();
     miniKMS = miniKMSBuilder.setKmsConfDir(kmsDir).build();
     miniKMS.start();
@@ -152,6 +146,7 @@ public class TestOzoneAtRestEncryption {
         .setChunkSize(CHUNK_SIZE)
         .setStreamBufferSizeUnit(StorageUnit.BYTES)
         .setCertificateClient(certificateClientTest)
+        .setSecretKeyClient(new SecretKeyTestClient())
         .build();
     cluster.getOzoneManager().startSecretManager();
     cluster.waitForClusterToBeReady();
@@ -173,11 +168,8 @@ public class TestOzoneAtRestEncryption {
     createKey(TEST_KEY, cluster.getOzoneManager().getKmsProvider(), conf);
   }
 
-  /**
-   * Close OzoneClient and shutdown MiniOzoneCluster.
-   */
-  @AfterClass
-  public static void shutdown() throws IOException {
+  @AfterAll
+  static void shutdown() throws IOException {
     if (ozClient != null) {
       ozClient.close();
     }
@@ -195,8 +187,9 @@ public class TestOzoneAtRestEncryption {
     }
   }
 
-  @Test
-  public void testPutKeyWithEncryption() throws Exception {
+  @ParameterizedTest
+  @EnumSource
+  void testPutKeyWithEncryption(BucketLayout bucketLayout) throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
 
@@ -209,15 +202,17 @@ public class TestOzoneAtRestEncryption {
     OzoneBucket bucket = volume.getBucket(bucketName);
 
     createAndVerifyKeyData(bucket);
+    createAndVerifyStreamKeyData(bucket);
   }
 
-  @Test
-  public void testLinkEncryptedBuckets() throws Exception {
+  @ParameterizedTest
+  @EnumSource
+  void testLinkEncryptedBuckets(BucketLayout bucketLayout) throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
 
     // Create source volume/bucket.
-    createVolumeAndBucket(volumeName, bucketName);
+    createVolumeAndBucket(volumeName, bucketName, bucketLayout);
 
     // Create link volume/bucket.
     String linkVolumeName = UUID.randomUUID().toString();
@@ -226,27 +221,45 @@ public class TestOzoneAtRestEncryption {
         linkVolumeName, linkBucketName);
 
     createAndVerifyKeyData(linkBucket);
+    createAndVerifyStreamKeyData(linkBucket);
   }
 
+  static void createAndVerifyStreamKeyData(OzoneBucket bucket)
+      throws Exception {
+    Instant testStartTime = Instant.now();
+    String keyName = UUID.randomUUID().toString();
+    String value = "sample value";
+    try (OzoneDataStreamOutput out = bucket.createStreamKey(keyName,
+        value.getBytes(StandardCharsets.UTF_8).length,
+        ReplicationConfig.fromTypeAndFactor(RATIS, ONE),
+        new HashMap<>())) {
+      out.write(value.getBytes(StandardCharsets.UTF_8));
+    }
+    verifyKeyData(bucket, keyName, value, testStartTime);
+  }
 
-  private void createAndVerifyKeyData(OzoneBucket bucket) throws Exception {
+  static void createAndVerifyKeyData(OzoneBucket bucket) throws Exception {
     Instant testStartTime = Instant.now();
     String keyName = UUID.randomUUID().toString();
     String value = "sample value";
     try (OzoneOutputStream out = bucket.createKey(keyName,
         value.getBytes(StandardCharsets.UTF_8).length,
-        ReplicationType.RATIS,
-        ReplicationFactor.ONE, new HashMap<>())) {
+        ReplicationConfig.fromTypeAndFactor(RATIS, ONE),
+        new HashMap<>())) {
       out.write(value.getBytes(StandardCharsets.UTF_8));
     }
+    verifyKeyData(bucket, keyName, value, testStartTime);
+  }
 
+  static void verifyKeyData(OzoneBucket bucket, String keyName, String value,
+      Instant testStartTime) throws Exception {
     // Verify content.
     OzoneKeyDetails key = bucket.getKey(keyName);
-    Assert.assertEquals(keyName, key.getName());
+    assertEquals(keyName, key.getName());
 
     // Check file encryption info is set,
     // if set key will use this encryption info and encrypt data.
-    Assert.assertTrue(key.getFileEncryptionInfo() != null);
+    assertNotNull(key.getFileEncryptionInfo());
 
     byte[] fileContent;
     int len = 0;
@@ -257,17 +270,17 @@ public class TestOzoneAtRestEncryption {
     }
 
 
-    Assert.assertEquals(len, value.length());
-    Assert.assertTrue(verifyRatisReplication(bucket.getVolumeName(),
-        bucket.getName(), keyName, ReplicationType.RATIS,
-        ReplicationFactor.ONE));
-    Assert.assertEquals(value, new String(fileContent, StandardCharsets.UTF_8));
-    Assert.assertFalse(key.getCreationTime().isBefore(testStartTime));
-    Assert.assertFalse(key.getModificationTime().isBefore(testStartTime));
+    assertEquals(len, value.length());
+    assertTrue(verifyRatisReplication(bucket.getVolumeName(),
+        bucket.getName(), keyName, RATIS,
+        ONE));
+    assertEquals(value, new String(fileContent, StandardCharsets.UTF_8));
+    assertFalse(key.getCreationTime().isBefore(testStartTime));
+    assertFalse(key.getModificationTime().isBefore(testStartTime));
   }
 
   private OzoneBucket createVolumeAndBucket(String volumeName,
-      String bucketName) throws Exception {
+      String bucketName, BucketLayout bucketLayout) throws Exception {
     store.createVolume(volumeName);
     OzoneVolume volume = store.getVolume(volumeName);
     BucketArgs bucketArgs = BucketArgs.newBuilder()
@@ -282,8 +295,9 @@ public class TestOzoneAtRestEncryption {
     store.createVolume(linkVol);
     OzoneVolume linkVolume = store.getVolume(linkVol);
     BucketArgs linkBucketArgs = BucketArgs.newBuilder()
-        .setSourceVolume(sourceVol).setSourceBucket(sourceBucket)
-        .setBucketLayout(bucketLayout).build();
+        .setSourceVolume(sourceVol)
+        .setSourceBucket(sourceBucket)
+        .build();
     linkVolume.createBucket(linkBucket, linkBucketArgs);
     return linkVolume.getBucket(linkBucket);
   }
@@ -293,10 +307,11 @@ public class TestOzoneAtRestEncryption {
    * 1. Create a GDPR enforced bucket
    * 2. PutKey with Encryption in above bucket and verify.
    * 3. DeleteKey and confirm the metadata does not have encryption key.
-   * @throws Exception
    */
-  @Test
-  public void testKeyWithEncryptionAndGdpr() throws Exception {
+  @ParameterizedTest
+  @EnumSource
+  void testKeyWithEncryptionAndGdpr(BucketLayout bucketLayout)
+      throws Exception {
     //Step 1
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
@@ -312,10 +327,9 @@ public class TestOzoneAtRestEncryption {
         .setBucketLayout(bucketLayout).build();
     volume.createBucket(bucketName, bucketArgs);
     OzoneBucket bucket = volume.getBucket(bucketName);
-    Assert.assertEquals(bucketName, bucket.getName());
-    Assert.assertNotNull(bucket.getMetadata());
-    Assert.assertEquals("true",
-        bucket.getMetadata().get(OzoneConsts.GDPR_FLAG));
+    assertEquals(bucketName, bucket.getName());
+    assertNotNull(bucket.getMetadata());
+    assertEquals("true", bucket.getMetadata().get(OzoneConsts.GDPR_FLAG));
 
     //Step 2
     String keyName = UUID.randomUUID().toString();
@@ -323,13 +337,13 @@ public class TestOzoneAtRestEncryption {
     keyMetadata.put(OzoneConsts.GDPR_FLAG, "true");
     try (OzoneOutputStream out = bucket.createKey(keyName,
         value.getBytes(StandardCharsets.UTF_8).length,
-        ReplicationType.RATIS,
-        ReplicationFactor.ONE, keyMetadata)) {
+        ReplicationConfig.fromTypeAndFactor(RATIS, ONE),
+        keyMetadata)) {
       out.write(value.getBytes(StandardCharsets.UTF_8));
     }
 
     OzoneKeyDetails key = bucket.getKey(keyName);
-    Assert.assertEquals(keyName, key.getName());
+    assertEquals(keyName, key.getName());
     byte[] fileContent;
     int len = 0;
 
@@ -338,44 +352,41 @@ public class TestOzoneAtRestEncryption {
       len = is.read(fileContent);
     }
 
-    Assert.assertEquals(len, value.length());
-    Assert.assertTrue(verifyRatisReplication(volumeName, bucketName,
-        keyName, ReplicationType.RATIS,
-        ReplicationFactor.ONE));
-    Assert.assertEquals(value, new String(fileContent, StandardCharsets.UTF_8));
-    Assert.assertFalse(key.getCreationTime().isBefore(testStartTime));
-    Assert.assertFalse(key.getModificationTime().isBefore(testStartTime));
-    Assert.assertEquals("true", key.getMetadata().get(OzoneConsts.GDPR_FLAG));
+    assertEquals(len, value.length());
+    assertTrue(verifyRatisReplication(volumeName, bucketName,
+        keyName, RATIS,
+        ONE));
+    assertEquals(value, new String(fileContent, StandardCharsets.UTF_8));
+    assertFalse(key.getCreationTime().isBefore(testStartTime));
+    assertFalse(key.getModificationTime().isBefore(testStartTime));
+    assertEquals("true", key.getMetadata().get(OzoneConsts.GDPR_FLAG));
     //As TDE is enabled, the TDE encryption details should not be null.
-    Assert.assertNotNull(key.getFileEncryptionInfo());
+    assertNotNull(key.getFileEncryptionInfo());
 
     //Step 3
     bucket.deleteKey(key.getName());
 
     OMMetadataManager omMetadataManager = ozoneManager.getMetadataManager();
-    String objectKey = omMetadataManager.getOzoneKey(volumeName, bucketName,
-        keyName);
 
     GenericTestUtils.waitFor(() -> {
       try {
-        return omMetadataManager.getDeletedTable().isExist(objectKey);
+        return getMatchedKeyInfo(keyName, omMetadataManager) != null;
       } catch (IOException e) {
         return false;
       }
     }, 500, 100000);
     RepeatedOmKeyInfo deletedKeys =
-        omMetadataManager.getDeletedTable().get(objectKey);
+        getMatchedKeyInfo(keyName, omMetadataManager);
+    assertNotNull(deletedKeys);
     Map<String, String> deletedKeyMetadata =
         deletedKeys.getOmKeyInfoList().get(0).getMetadata();
-    Assert.assertFalse(deletedKeyMetadata.containsKey(OzoneConsts.GDPR_FLAG));
-    Assert.assertFalse(deletedKeyMetadata.containsKey(OzoneConsts.GDPR_SECRET));
-    Assert.assertFalse(
-        deletedKeyMetadata.containsKey(OzoneConsts.GDPR_ALGORITHM));
-    Assert.assertNull(
-        deletedKeys.getOmKeyInfoList().get(0).getFileEncryptionInfo());
+    assertFalse(deletedKeyMetadata.containsKey(OzoneConsts.GDPR_FLAG));
+    assertFalse(deletedKeyMetadata.containsKey(OzoneConsts.GDPR_SECRET));
+    assertFalse(deletedKeyMetadata.containsKey(OzoneConsts.GDPR_ALGORITHM));
+    assertNull(deletedKeys.getOmKeyInfoList().get(0).getFileEncryptionInfo());
   }
 
-  private boolean verifyRatisReplication(String volumeName, String bucketName,
+  static boolean verifyRatisReplication(String volumeName, String bucketName,
       String keyName, ReplicationType type, ReplicationFactor factor)
       throws IOException {
     OmKeyArgs keyArgs = new OmKeyArgs.Builder()
@@ -416,39 +427,77 @@ public class TestOzoneAtRestEncryption {
     provider.flush();
   }
 
-  @Test
-  public void testMPUwithOnePart() throws Exception {
+  @ParameterizedTest
+  @EnumSource
+  void mpuOnePart(BucketLayout bucketLayout) throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
-    OzoneBucket bucket = createVolumeAndBucket(volumeName, bucketName);
-    testMultipartUploadWithEncryption(bucket, 1);
+    testMultipartUploadWithEncryption(
+        createVolumeAndBucket(volumeName, bucketName, bucketLayout), 1);
   }
 
-  @Test
-  public void testMPUwithTwoParts() throws Exception {
+  @ParameterizedTest
+  @EnumSource
+  void mpuTwoParts(BucketLayout bucketLayout) throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
-    OzoneBucket bucket = createVolumeAndBucket(volumeName, bucketName);
-    testMultipartUploadWithEncryption(bucket, 2);
+    testMultipartUploadWithEncryption(
+        createVolumeAndBucket(volumeName, bucketName, bucketLayout), 2);
   }
 
-  @Test
-  public void testMPUwithThreePartsOverride() throws Exception {
+  @ParameterizedTest
+  @EnumSource
+  void mpuThreePartsOverride(BucketLayout bucketLayout) throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
-    OzoneBucket bucket = createVolumeAndBucket(volumeName, bucketName);
+    OzoneBucket bucket = createVolumeAndBucket(volumeName, bucketName,
+        bucketLayout);
     testMultipartUploadWithEncryption(bucket, 3);
 
     // override the key and check content
     testMultipartUploadWithEncryption(bucket, 3);
   }
 
-
-  @Test
-  public void testMPUwithLinkBucket() throws Exception {
+  @ParameterizedTest
+  @EnumSource
+  @Flaky("HDDS-8947")
+  void mpuStreamOnePart(BucketLayout bucketLayout) throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
-    createVolumeAndBucket(volumeName, bucketName);
+    testMultipartUploadWithEncryption(
+        createVolumeAndBucket(volumeName, bucketName, bucketLayout), 1, true);
+  }
+
+  @ParameterizedTest
+  @EnumSource
+  @Flaky("HDDS-8947")
+  void mpuStreamTwoParts(BucketLayout bucketLayout) throws Exception {
+    String volumeName = UUID.randomUUID().toString();
+    String bucketName = UUID.randomUUID().toString();
+    testMultipartUploadWithEncryption(
+        createVolumeAndBucket(volumeName, bucketName, bucketLayout), 2, true);
+  }
+
+  @ParameterizedTest
+  @EnumSource
+  @Flaky("HDDS-8947")
+  void mpuStreamThreePartsOverride(BucketLayout bucketLayout) throws Exception {
+    String volumeName = UUID.randomUUID().toString();
+    String bucketName = UUID.randomUUID().toString();
+    OzoneBucket bucket = createVolumeAndBucket(volumeName, bucketName,
+        bucketLayout);
+    testMultipartUploadWithEncryption(bucket, 3);
+
+    // override the key and check content
+    testMultipartUploadWithEncryption(bucket, 3, true);
+  }
+
+  @ParameterizedTest
+  @EnumSource
+  void mpuWithLink(BucketLayout bucketLayout) throws Exception {
+    String volumeName = UUID.randomUUID().toString();
+    String bucketName = UUID.randomUUID().toString();
+    createVolumeAndBucket(volumeName, bucketName, bucketLayout);
 
     String linkVolumeName = UUID.randomUUID().toString();
     String linkBucketName = UUID.randomUUID().toString();
@@ -457,13 +506,18 @@ public class TestOzoneAtRestEncryption {
     testMultipartUploadWithEncryption(linkBucket, 2);
   }
 
-  public void testMultipartUploadWithEncryption(OzoneBucket bucket,
+  private void testMultipartUploadWithEncryption(OzoneBucket bucket,
       int numParts) throws Exception {
+    testMultipartUploadWithEncryption(bucket, numParts, false);
+  }
+
+  private void testMultipartUploadWithEncryption(OzoneBucket bucket,
+      int numParts, boolean isStream) throws Exception {
     String keyName = "mpu_test_key_" + numParts;
 
     // Initiate multipart upload
-    String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
-        ONE);
+    String uploadID = initiateMultipartUpload(bucket, keyName,
+        ReplicationConfig.fromTypeAndFactor(RATIS, ONE));
 
     // Upload Parts
     Map<Integer, String> partsMap = new TreeMap<>();
@@ -477,7 +531,14 @@ public class TestOzoneAtRestEncryption {
       int partSize = (MPU_PART_MIN_SIZE * i) +
           RANDOM.nextInt(DEFAULT_CRYPTO_BUFFER_SIZE - 1) + 1;
       byte[] data = generateRandomData(partSize);
-      String partName = uploadPart(bucket, keyName, uploadID, i, data);
+
+      String partName;
+      if (isStream) {
+        partName = uploadStreamPart(bucket, keyName, uploadID, i, data);
+      } else {
+        partName = uploadPart(bucket, keyName, uploadID, i, data);
+      }
+
       partsMap.put(i, partName);
       partsData.add(data);
       keySize += data.length;
@@ -496,45 +557,47 @@ public class TestOzoneAtRestEncryption {
     completeMultipartUpload(bucket, keyName, uploadID, partsMap);
 
     // Create an input stream to read the data
-    OzoneInputStream inputStream = bucket.readKey(keyName);
-    Assert.assertTrue(inputStream instanceof MultipartCryptoKeyInputStream);
+    try (OzoneInputStream inputStream = bucket.readKey(keyName)) {
 
-    // Test complete read
-    byte[] completeRead = new byte[keySize];
-    int bytesRead = inputStream.read(completeRead, 0, keySize);
-    Assert.assertEquals(bytesRead, keySize);
-    Assert.assertArrayEquals(inputData, completeRead);
+      assertTrue(inputStream.getInputStream()
+          instanceof MultipartInputStream);
 
-    // Read different data lengths and starting from different offsets and
-    // verify the data matches.
-    Random random = new Random();
-    int randomSize = random.nextInt(keySize / 2);
-    int randomOffset = random.nextInt(keySize - randomSize);
+      // Test complete read
+      byte[] completeRead = new byte[keySize];
+      int bytesRead = inputStream.read(completeRead, 0, keySize);
+      assertEquals(bytesRead, keySize);
+      assertArrayEquals(inputData, completeRead);
 
-    int[] readDataSizes = {keySize, keySize / 3 + 1, BLOCK_SIZE,
-        BLOCK_SIZE * 2 + 1, CHUNK_SIZE, CHUNK_SIZE / 4 - 1,
-        DEFAULT_CRYPTO_BUFFER_SIZE, DEFAULT_CRYPTO_BUFFER_SIZE / 2, 1,
-        randomSize};
+      // Read different data lengths and starting from different offsets and
+      // verify the data matches.
+      Random random = new Random();
+      int randomSize = random.nextInt(keySize / 2);
+      int randomOffset = random.nextInt(keySize - randomSize);
 
-    int[] readFromPositions = {0, DEFAULT_CRYPTO_BUFFER_SIZE + 10, CHUNK_SIZE,
-        BLOCK_SIZE - DEFAULT_CRYPTO_BUFFER_SIZE + 1, BLOCK_SIZE, keySize / 3,
-        keySize - 1, randomOffset};
+      int[] readDataSizes = {keySize, keySize / 3 + 1, BLOCK_SIZE,
+          BLOCK_SIZE * 2 + 1, CHUNK_SIZE, CHUNK_SIZE / 4 - 1,
+          DEFAULT_CRYPTO_BUFFER_SIZE, DEFAULT_CRYPTO_BUFFER_SIZE / 2, 1,
+          randomSize};
 
-    for (int readDataLen : readDataSizes) {
-      for (int readFromPosition : readFromPositions) {
-        // Check that offset + buffer size does not exceed the key size
-        if (readFromPosition + readDataLen > keySize) {
-          continue;
+      int[] readFromPositions = {0, DEFAULT_CRYPTO_BUFFER_SIZE + 10, CHUNK_SIZE,
+          BLOCK_SIZE - DEFAULT_CRYPTO_BUFFER_SIZE + 1, BLOCK_SIZE, keySize / 3,
+          keySize - 1, randomOffset};
+
+      for (int readDataLen : readDataSizes) {
+        for (int readFromPosition : readFromPositions) {
+          // Check that offset + buffer size does not exceed the key size
+          if (readFromPosition + readDataLen > keySize) {
+            continue;
+          }
+
+          byte[] readData = new byte[readDataLen];
+          inputStream.seek(readFromPosition);
+          int actualReadLen = inputStream.read(readData, 0, readDataLen);
+
+          assertReadContent(inputData, readData, readFromPosition);
+          assertEquals(readFromPosition + readDataLen, inputStream.getPos());
+          assertEquals(readDataLen, actualReadLen);
         }
-
-        byte[] readData = new byte[readDataLen];
-        inputStream.seek(readFromPosition);
-        int actualReadLen = inputStream.read(readData, 0, readDataLen);
-
-        assertReadContent(inputData, readData, readFromPosition);
-        Assert.assertEquals(readFromPosition + readDataLen,
-            inputStream.getPos());
-        Assert.assertEquals(readDataLen, actualReadLen);
       }
     }
   }
@@ -546,16 +609,36 @@ public class TestOzoneAtRestEncryption {
   }
 
   private String initiateMultipartUpload(OzoneBucket bucket, String keyName,
-      ReplicationType replicationType, ReplicationFactor replicationFactor)
+      ReplicationConfig replicationConfig)
       throws Exception {
     OmMultipartInfo multipartInfo = bucket.initiateMultipartUpload(keyName,
-        replicationType, replicationFactor);
+        replicationConfig);
 
     String uploadID = multipartInfo.getUploadID();
-    Assert.assertNotNull(uploadID);
+    assertNotNull(uploadID);
     return uploadID;
   }
 
+  private String uploadStreamPart(OzoneBucket bucket, String keyName,
+      String uploadID, int partNumber, byte[] data) throws Exception {
+    final int length = data.length;
+
+    OzoneDataStreamOutput multipartStreamKey =
+        bucket.createMultipartStreamKey(keyName,
+            length, partNumber, uploadID);
+
+    ByteBuffer dataBuffer = ByteBuffer.wrap(data);
+    multipartStreamKey.write(dataBuffer, 0, length);
+    multipartStreamKey.close();
+
+    OmMultipartCommitUploadPartInfo omMultipartCommitUploadPartInfo =
+        multipartStreamKey.getCommitUploadPartInfo();
+
+    assertNotNull(omMultipartCommitUploadPartInfo);
+    assertNotNull(omMultipartCommitUploadPartInfo.getPartName());
+    return omMultipartCommitUploadPartInfo.getPartName();
+  }
+
   private String uploadPart(OzoneBucket bucket, String keyName,
       String uploadID, int partNumber, byte[] data) throws Exception {
     OzoneOutputStream ozoneOutputStream = bucket.createMultipartKey(keyName,
@@ -566,8 +649,8 @@ public class TestOzoneAtRestEncryption {
     OmMultipartCommitUploadPartInfo omMultipartCommitUploadPartInfo =
         ozoneOutputStream.getCommitUploadPartInfo();
 
-    Assert.assertNotNull(omMultipartCommitUploadPartInfo);
-    Assert.assertNotNull(omMultipartCommitUploadPartInfo.getPartName());
+    assertNotNull(omMultipartCommitUploadPartInfo);
+    assertNotNull(omMultipartCommitUploadPartInfo.getPartName());
     return omMultipartCommitUploadPartInfo.getPartName();
   }
 
@@ -576,43 +659,56 @@ public class TestOzoneAtRestEncryption {
     OmMultipartUploadCompleteInfo omMultipartUploadCompleteInfo = bucket
         .completeMultipartUpload(keyName, uploadID, partsMap);
 
-    Assert.assertNotNull(omMultipartUploadCompleteInfo);
-    Assert.assertEquals(omMultipartUploadCompleteInfo.getBucket(), bucket
+    assertNotNull(omMultipartUploadCompleteInfo);
+    assertEquals(omMultipartUploadCompleteInfo.getBucket(), bucket
         .getName());
-    Assert.assertEquals(omMultipartUploadCompleteInfo.getVolume(), bucket
+    assertEquals(omMultipartUploadCompleteInfo.getVolume(), bucket
         .getVolumeName());
-    Assert.assertEquals(omMultipartUploadCompleteInfo.getKey(), keyName);
-    Assert.assertNotNull(omMultipartUploadCompleteInfo.getHash());
+    assertEquals(omMultipartUploadCompleteInfo.getKey(), keyName);
+    assertNotNull(omMultipartUploadCompleteInfo.getHash());
   }
 
   private static void assertReadContent(byte[] inputData, byte[] readData,
       int offset) {
     byte[] inputDataForComparison = Arrays.copyOfRange(inputData, offset,
         offset + readData.length);
-    Assert.assertArrayEquals("Read data does not match input data at offset " +
-        offset + " and length " + readData.length,
-        inputDataForComparison, readData);
+    assertArrayEquals(inputDataForComparison, readData,
+        "Read data does not match input data at offset " +
+            offset + " and length " + readData.length);
   }
 
   @Test
-  public void testGetKeyProvider() throws Exception {
+  void testGetKeyProvider() throws Exception {
     KeyProvider kp1 = store.getKeyProvider();
     KeyProvider kpSpy = Mockito.spy(kp1);
-    Assert.assertNotEquals(kpSpy, kp1);
+    assertNotEquals(kpSpy, kp1);
     Cache<URI, KeyProvider> cacheSpy =
         ((RpcClient)store.getClientProxy()).getKeyProviderCache();
     cacheSpy.put(store.getKeyProviderUri(), kpSpy);
     KeyProvider kp2 = store.getKeyProvider();
-    Assert.assertEquals(kpSpy, kp2);
+    assertEquals(kpSpy, kp2);
 
     // Verify the spied key provider is closed upon ozone client close
     ozClient.close();
     Mockito.verify(kpSpy).close();
 
     KeyProvider kp3 = ozClient.getObjectStore().getKeyProvider();
-    Assert.assertNotEquals(kp3, kpSpy);
+    assertNotEquals(kp3, kpSpy);
     // Restore ozClient and store
     TestOzoneRpcClient.setOzClient(OzoneClientFactory.getRpcClient(conf));
     TestOzoneRpcClient.setStore(ozClient.getObjectStore());
   }
+
+  private static RepeatedOmKeyInfo getMatchedKeyInfo(
+      String keyName, OMMetadataManager omMetadataManager) throws IOException {
+    List<? extends Table.KeyValue<String, RepeatedOmKeyInfo>> rangeKVs
+        = omMetadataManager.getDeletedTable().getRangeKVs(
+        null, 100, "/");
+    for (int i = 0; i < rangeKVs.size(); ++i) {
+      if (rangeKVs.get(i).getKey().contains(keyName)) {
+        return rangeKVs.get(i).getValue();
+      }
+    }
+    return null;
+  }
 }
