@@ -17,7 +17,10 @@
 
 package org.apache.hadoop.ozone.client.rpc;
 
+import java.util.HashMap;
 import org.apache.commons.lang3.RandomUtils;
+import org.apache.hadoop.hdds.client.DefaultReplicationConfig;
+import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationFactor;
 import org.apache.hadoop.hdds.client.ReplicationType;
@@ -26,6 +29,7 @@ import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.ozone.MiniOzoneCluster;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.OzoneTestUtils;
+import org.apache.hadoop.ozone.client.BucketArgs;
 import org.apache.hadoop.ozone.client.ObjectStore;
 import org.apache.hadoop.ozone.client.OzoneBucket;
 import org.apache.hadoop.ozone.client.OzoneClient;
@@ -52,16 +56,20 @@ import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.hadoop.hdds.StringUtils.string2Bytes;
 import static org.apache.hadoop.hdds.client.ReplicationFactor.THREE;
 
+import org.apache.hadoop.ozone.om.helpers.QuotaUtil;
 import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.om.request.file.OMFileRequest;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.ozone.test.GenericTestUtils;
 import org.junit.AfterClass;
 import org.junit.Assert;
+import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Rule;
 import org.junit.Test;
+import org.junit.rules.TestRule;
 import org.junit.rules.Timeout;
+import org.apache.ozone.test.JUnit5AwareTimeout;
 
 import java.io.IOException;
 import java.nio.file.Path;
@@ -97,7 +105,12 @@ public class TestOzoneClientMultipartUploadWithFSO {
    * Set a timeout for each test.
    */
   @Rule
-  public Timeout timeout = new Timeout(300000);
+  public TestRule timeout = new JUnit5AwareTimeout(new Timeout(300000));
+  private String volumeName;
+  private String bucketName;
+  private String keyName;
+  private OzoneVolume volume;
+  private OzoneBucket bucket;
 
   /**
    * Create a MiniOzoneCluster for testing.
@@ -129,7 +142,7 @@ public class TestOzoneClientMultipartUploadWithFSO {
    */
   static void startCluster(OzoneConfiguration conf) throws Exception {
     cluster = MiniOzoneCluster.newBuilder(conf)
-            .setNumDatanodes(3)
+            .setNumDatanodes(5)
             .setTotalPipelineNumLimit(10)
             .setScmId(scmId)
             .build();
@@ -150,52 +163,35 @@ public class TestOzoneClientMultipartUploadWithFSO {
       cluster.shutdown();
     }
   }
+  
+  @Before
+  public void preTest() throws Exception {
+    volumeName = UUID.randomUUID().toString();
+    bucketName = UUID.randomUUID().toString();
+    keyName = UUID.randomUUID().toString();
+
+    store.createVolume(volumeName);
+    volume = store.getVolume(volumeName);
+    volume.createBucket(bucketName);
+    bucket = volume.getBucket(bucketName);
+  }
 
   @Test
   public void testInitiateMultipartUploadWithReplicationInformationSet() throws
           IOException {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-    OmMultipartInfo multipartInfo = bucket.initiateMultipartUpload(keyName,
-        RATIS, ONE);
-
-    Assert.assertNotNull(multipartInfo);
-    String uploadID = multipartInfo.getUploadID();
-    Assert.assertEquals(volumeName, multipartInfo.getVolumeName());
-    Assert.assertEquals(bucketName, multipartInfo.getBucketName());
-    Assert.assertEquals(keyName, multipartInfo.getKeyName());
-    Assert.assertNotNull(multipartInfo.getUploadID());
+    String uploadID = initiateMultipartUpload(bucket, keyName,
+        ReplicationType.RATIS, ONE);
 
     // Call initiate multipart upload for the same key again, this should
     // generate a new uploadID.
-    multipartInfo = bucket.initiateMultipartUpload(keyName,
-        RATIS, ONE);
-
-    Assert.assertNotNull(multipartInfo);
-    Assert.assertEquals(volumeName, multipartInfo.getVolumeName());
-    Assert.assertEquals(bucketName, multipartInfo.getBucketName());
-    Assert.assertEquals(keyName, multipartInfo.getKeyName());
-    Assert.assertNotEquals(multipartInfo.getUploadID(), uploadID);
-    Assert.assertNotNull(multipartInfo.getUploadID());
+    String uploadIDNew = initiateMultipartUpload(bucket, keyName,
+        ReplicationType.RATIS, ONE);
+    Assert.assertNotEquals(uploadIDNew, uploadID);
   }
 
   @Test
   public void testInitiateMultipartUploadWithDefaultReplication() throws
           IOException {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
     OmMultipartInfo multipartInfo = bucket.initiateMultipartUpload(keyName);
 
     Assert.assertNotNull(multipartInfo);
@@ -219,24 +215,9 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
   @Test
   public void testUploadPartWithNoOverride() throws IOException {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
     String sampleData = "sample Value";
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-    OmMultipartInfo multipartInfo = bucket.initiateMultipartUpload(keyName,
-        RATIS, ONE);
-
-    Assert.assertNotNull(multipartInfo);
-    String uploadID = multipartInfo.getUploadID();
-    Assert.assertEquals(volumeName, multipartInfo.getVolumeName());
-    Assert.assertEquals(bucketName, multipartInfo.getBucketName());
-    Assert.assertEquals(keyName, multipartInfo.getKeyName());
-    Assert.assertNotNull(multipartInfo.getUploadID());
+    String uploadID = initiateMultipartUpload(bucket, keyName,
+        ReplicationType.RATIS, ONE);
 
     OzoneOutputStream ozoneOutputStream = bucket.createMultipartKey(keyName,
             sampleData.length(), 1, uploadID);
@@ -251,53 +232,18 @@ public class TestOzoneClientMultipartUploadWithFSO {
   }
 
   @Test
-  public void testUploadPartOverrideWithRatis() throws IOException {
-
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
+  public void testUploadPartOverrideWithRatis() throws Exception {
     String sampleData = "sample Value";
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-    OmMultipartInfo multipartInfo = bucket.initiateMultipartUpload(keyName,
-            ReplicationType.RATIS, THREE);
-
-    Assert.assertNotNull(multipartInfo);
-    String uploadID = multipartInfo.getUploadID();
-    Assert.assertEquals(volumeName, multipartInfo.getVolumeName());
-    Assert.assertEquals(bucketName, multipartInfo.getBucketName());
-    Assert.assertEquals(keyName, multipartInfo.getKeyName());
-    Assert.assertNotNull(multipartInfo.getUploadID());
+    String uploadID = initiateMultipartUpload(bucket, keyName,
+        ReplicationType.RATIS, THREE);
 
     int partNumber = 1;
-
-    OzoneOutputStream ozoneOutputStream = bucket.createMultipartKey(keyName,
-            sampleData.length(), partNumber, uploadID);
-    ozoneOutputStream.write(string2Bytes(sampleData), 0, sampleData.length());
-    ozoneOutputStream.close();
-
-    OmMultipartCommitUploadPartInfo commitUploadPartInfo = ozoneOutputStream
-            .getCommitUploadPartInfo();
-
-    Assert.assertNotNull(commitUploadPartInfo);
-    String partName = commitUploadPartInfo.getPartName();
-    Assert.assertNotNull(commitUploadPartInfo.getPartName());
+    String partName = uploadPart(bucket, keyName, uploadID, partNumber,
+        sampleData.getBytes(UTF_8));
 
     //Overwrite the part by creating part key with same part number.
-    sampleData = "sample Data Changed";
-    ozoneOutputStream = bucket.createMultipartKey(keyName,
-            sampleData.length(), partNumber, uploadID);
-    ozoneOutputStream.write(string2Bytes(sampleData), 0, "name".length());
-    ozoneOutputStream.close();
-
-    commitUploadPartInfo = ozoneOutputStream
-            .getCommitUploadPartInfo();
-
-    Assert.assertNotNull(commitUploadPartInfo);
-    Assert.assertNotNull(commitUploadPartInfo.getPartName());
+    String partNameNew = uploadPart(bucket, keyName, uploadID, partNumber,
+        "name".getBytes(UTF_8));
 
     // PartName should be same from old part Name.
     // AWS S3 for same content generates same partName during upload part.
@@ -308,20 +254,94 @@ public class TestOzoneClientMultipartUploadWithFSO {
     // of content in ozone s3. This will make S3 Mpu completeMPU pass when
     // comparing part names and large file uploads work using aws cp.
     Assert.assertEquals("Part names should be same", partName,
-        commitUploadPartInfo.getPartName());
+        partNameNew);
+
+    // old part bytes written needs discard and have only
+    // new part bytes in quota for this bucket
+    long byteWritten = "name".length() * 3; // data written with replication
+    Assert.assertEquals(volume.getBucket(bucketName).getUsedBytes(),
+        byteWritten);
   }
 
+  @Test
+  public void testUploadTwiceWithEC() throws IOException {
+    bucketName = UUID.randomUUID().toString();
+    bucket = getOzoneECBucket(bucketName);
+
+    byte[] data = generateData(81920, (byte) 97);
+    // perform upload and complete
+    OmMultipartInfo multipartInfo = bucket.initiateMultipartUpload(keyName);
+
+    String uploadID = multipartInfo.getUploadID();
+    int partNumber = 1;
+
+    String partName = uploadPart(bucket, keyName, uploadID, partNumber,
+        data);
+    
+    Map<Integer, String> partsMap = new HashMap<>();
+    partsMap.put(partNumber, partName);
+    bucket.completeMultipartUpload(keyName, uploadID, partsMap);
+
+    long replicatedSize = QuotaUtil.getReplicatedSize(data.length,
+        bucket.getReplicationConfig());
+    Assert.assertEquals(volume.getBucket(bucketName).getUsedBytes(),
+        replicatedSize);
+
+    //upload same key again
+    multipartInfo = bucket.initiateMultipartUpload(keyName);
+    uploadID = multipartInfo.getUploadID();
+
+    partName = uploadPart(bucket, keyName, uploadID, partNumber,
+        data);
+
+    partsMap = new HashMap<>();
+    partsMap.put(partNumber, partName);
+    bucket.completeMultipartUpload(keyName, uploadID, partsMap);
+
+    // used sized should remain same, overwrite previous upload
+    Assert.assertEquals(volume.getBucket(bucketName).getUsedBytes(),
+        replicatedSize);
+  }
+
+  @Test
+  public void testUploadAbortWithEC() throws IOException {
+    byte[] data = generateData(81920, (byte) 97);
+
+    bucketName = UUID.randomUUID().toString();
+    bucket = getOzoneECBucket(bucketName);
+
+    // perform upload and abort
+    OmMultipartInfo multipartInfo = bucket.initiateMultipartUpload(keyName);
+
+    String uploadID = multipartInfo.getUploadID();
+    int partNumber = 1;
+    uploadPart(bucket, keyName, uploadID, partNumber, data);
+    
+    long replicatedSize = QuotaUtil.getReplicatedSize(data.length,
+        bucket.getReplicationConfig());
+    Assert.assertEquals(volume.getBucket(bucketName).getUsedBytes(),
+        replicatedSize);
+
+    bucket.abortMultipartUpload(keyName, uploadID);
+
+    // used size should become zero after aport upload
+    Assert.assertEquals(volume.getBucket(bucketName).getUsedBytes(), 0);
+  }
+
+  private OzoneBucket getOzoneECBucket(String myBucket)
+      throws IOException {
+    final BucketArgs.Builder bucketArgs = BucketArgs.newBuilder();
+    bucketArgs.setDefaultReplicationConfig(
+        new DefaultReplicationConfig(
+            new ECReplicationConfig(3, 2, ECReplicationConfig.EcCodec.RS,
+                (int) OzoneConsts.MB)));
+
+    volume.createBucket(myBucket, bucketArgs.build());
+    return volume.getBucket(myBucket);
+  }
+  
   @Test
   public void testMultipartUploadWithPartsLessThanMinSize() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
     // Initiate multipart upload
     String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
             ONE);
@@ -342,18 +362,39 @@ public class TestOzoneClientMultipartUploadWithFSO {
         () -> completeMultipartUpload(bucket, keyName, uploadID, partsMap));
   }
 
+  @Test
+  public void testMultipartUploadWithDiscardedUnusedPartSize()
+      throws Exception {
+    // Initiate multipart upload
+    String uploadID = initiateMultipartUpload(bucket, keyName, RATIS, ONE);
+    byte[] data = generateData(10000000, (byte) 97);
+
+    // Upload Parts
+    Map<Integer, String> partsMap = new TreeMap<>();
+
+    // Upload part 1 and add it to the partsMap for completing the upload.
+    String partName1 = uploadPart(bucket, keyName, uploadID, 1, data);
+    partsMap.put(1, partName1);
+
+    // Upload part 2 and add it to the partsMap for completing the upload.
+    String partName2 = uploadPart(bucket, keyName, uploadID, 2, data);
+    partsMap.put(2, partName2);
+
+    // Upload part 3 but do not add it to the partsMap.
+    uploadPart(bucket, keyName, uploadID, 3, data);
+
+    completeMultipartUpload(bucket, keyName, uploadID, partsMap);
+
+    // Check the bucket size. Since part number 3 was not added to the partsMap,
+    // the unused part size should be discarded from the bucket size,
+    // 30000000 - 10000000 = 20000000
+    long bucketSize = volume.getBucket(bucketName).getUsedBytes();
+    Assert.assertEquals(bucketSize, data.length * 2);
+  }
+
   @Test
   public void testMultipartUploadWithPartsMisMatchWithListSizeDifferent()
           throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
     String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
             ONE);
 
@@ -369,15 +410,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
   @Test
   public void testMultipartUploadWithPartsMisMatchWithIncorrectPartName()
           throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
     String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
             ONE);
 
@@ -393,15 +425,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
   @Test
   public void testMultipartUploadWithMissingParts() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
     String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
             ONE);
 
@@ -416,23 +439,32 @@ public class TestOzoneClientMultipartUploadWithFSO {
   }
 
   @Test
-  public void testCommitPartAfterCompleteUpload() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String parentDir = "a/b/c/d/";
-    String keyName = parentDir + UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
-    OmMultipartInfo omMultipartInfo = bucket.initiateMultipartUpload(keyName,
+  public void testMultipartPartNumberExceedingAllowedRange() throws Exception {
+    String uploadID = initiateMultipartUpload(bucket, keyName,
         RATIS, ONE);
+    byte[] data = "data".getBytes(UTF_8);
 
-    Assert.assertNotNull(omMultipartInfo.getUploadID());
+    // Multipart part number must be an integer between 1 and 10000. So the
+    // part number 1, 5000, 10000 will succeed,
+    // the part number 0, 10001 will fail.
+    bucket.createMultipartKey(keyName, data.length, 1, uploadID);
+    bucket.createMultipartKey(keyName, data.length, 5000, uploadID);
+    bucket.createMultipartKey(keyName, data.length, 10000, uploadID);
+    OzoneTestUtils.expectOmException(OMException.ResultCodes.INVALID_PART,
+        () -> bucket.createMultipartKey(
+            keyName, data.length, 0, uploadID));
+    OzoneTestUtils.expectOmException(OMException.ResultCodes.INVALID_PART,
+        () -> bucket.createMultipartKey(
+            keyName, data.length, 10001, uploadID));
+  }
 
-    String uploadID = omMultipartInfo.getUploadID();
+  @Test
+  public void testCommitPartAfterCompleteUpload() throws Exception {
+    String parentDir = "a/b/c/d/";
+    keyName = parentDir + UUID.randomUUID().toString();
+    String uploadID = initiateMultipartUpload(bucket, keyName, RATIS, ONE);
+
+    Assert.assertEquals(volume.getBucket(bucketName).getUsedNamespace(), 4);
 
     // upload part 1.
     byte[] data = generateData(5 * 1024 * 1024,
@@ -447,7 +479,7 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
     // Do not close output stream for part 2.
     ozoneOutputStream = bucket.createMultipartKey(keyName,
-            data.length, 2, omMultipartInfo.getUploadID());
+            data.length, 2, uploadID);
     ozoneOutputStream.write(data, 0, data.length);
 
     Map<Integer, String> partsMap = new LinkedHashMap<>();
@@ -460,8 +492,9 @@ public class TestOzoneClientMultipartUploadWithFSO {
     Assert.assertNotNull(omMultipartCommitUploadPartInfo);
 
     byte[] fileContent = new byte[data.length];
-    OzoneInputStream inputStream = bucket.readKey(keyName);
-    inputStream.read(fileContent);
+    try (OzoneInputStream inputStream = bucket.readKey(keyName)) {
+      inputStream.read(fileContent);
+    }
     StringBuilder sb = new StringBuilder(data.length);
 
     // Combine all parts data, and check is it matching with get key data.
@@ -481,44 +514,26 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
   @Test
   public void testAbortUploadFail() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
     OzoneTestUtils.expectOmException(NO_SUCH_MULTIPART_UPLOAD_ERROR,
         () -> bucket.abortMultipartUpload(keyName, "random"));
   }
 
   @Test
   public void testAbortUploadFailWithInProgressPartUpload() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
     String parentDir = "a/b/c/d/";
-    String keyName = parentDir + UUID.randomUUID().toString();
+    keyName = parentDir + UUID.randomUUID().toString();
 
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
-    OmMultipartInfo omMultipartInfo = bucket.initiateMultipartUpload(keyName,
+    String uploadID = initiateMultipartUpload(bucket, keyName,
         RATIS, ONE);
 
-    Assert.assertNotNull(omMultipartInfo.getUploadID());
-
     // Do not close output stream.
     byte[] data = "data".getBytes(UTF_8);
     OzoneOutputStream ozoneOutputStream = bucket.createMultipartKey(keyName,
-        data.length, 1, omMultipartInfo.getUploadID());
+        data.length, 1, uploadID);
     ozoneOutputStream.write(data, 0, data.length);
 
     // Abort before completing part upload.
-    bucket.abortMultipartUpload(keyName, omMultipartInfo.getUploadID());
+    bucket.abortMultipartUpload(keyName, uploadID);
 
     try {
       ozoneOutputStream.close();
@@ -532,15 +547,8 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
   @Test
   public void testAbortUploadSuccessWithOutAnyParts() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
     String parentDir = "a/b/c/d/";
-    String keyName = parentDir + UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
+    keyName = parentDir + UUID.randomUUID().toString();
 
     String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
         ONE);
@@ -549,15 +557,9 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
   @Test
   public void testAbortUploadSuccessWithParts() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
     String parentDir = "a/b/c/d/";
-    String keyName = parentDir + UUID.randomUUID().toString();
+    keyName = parentDir + UUID.randomUUID().toString();
 
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
     OzoneManager ozoneManager = cluster.getOzoneManager();
     String buckKey = ozoneManager.getMetadataManager()
         .getBucketKey(volume.getName(), bucket.getName());
@@ -572,8 +574,7 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
     OMMetadataManager metadataMgr =
         cluster.getOzoneManager().getMetadataManager();
-    String multipartKey = verifyUploadedPart(volumeName, bucketName, keyName,
-        uploadID, partName, metadataMgr);
+    String multipartKey = verifyUploadedPart(uploadID, partName, metadataMgr);
 
     bucket.abortMultipartUpload(keyName, uploadID);
 
@@ -594,15 +595,8 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
   @Test
   public void testListMultipartUploadParts() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
     String parentDir = "a/b/c/d/e/f/";
-    String keyName = parentDir + "file-ABC";
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
+    keyName = parentDir + "file-ABC";
 
     Map<Integer, String> partsMap = new TreeMap<>();
     String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
@@ -629,14 +623,13 @@ public class TestOzoneClientMultipartUploadWithFSO {
     Assert.assertEquals(3,
         ozoneMultipartUploadPartListParts.getPartInfoList().size());
 
-    verifyPartNamesInDB(volumeName, bucketName, keyName, partsMap,
+    verifyPartNamesInDB(partsMap,
         ozoneMultipartUploadPartListParts, uploadID);
 
     Assert.assertFalse(ozoneMultipartUploadPartListParts.isTruncated());
   }
 
-  private void verifyPartNamesInDB(String volumeName, String bucketName,
-      String keyName, Map<Integer, String> partsMap,
+  private void verifyPartNamesInDB(Map<Integer, String> partsMap,
       OzoneMultipartUploadPartListParts ozoneMultipartUploadPartListParts,
       String uploadID) throws IOException {
 
@@ -661,11 +654,8 @@ public class TestOzoneClientMultipartUploadWithFSO {
         metadataMgr.getMultipartInfoTable().get(multipartKey);
     Assert.assertNotNull(omMultipartKeyInfo);
 
-    TreeMap<Integer, OzoneManagerProtocolProtos.PartKeyInfo> partKeyInfoMap =
-        omMultipartKeyInfo.getPartKeyInfoMap();
-    for (Map.Entry<Integer, OzoneManagerProtocolProtos.PartKeyInfo> entry :
-        partKeyInfoMap.entrySet()) {
-      OzoneManagerProtocolProtos.PartKeyInfo partKeyInfo = entry.getValue();
+    for (OzoneManagerProtocolProtos.PartKeyInfo partKeyInfo :
+        omMultipartKeyInfo.getPartKeyInfoMap()) {
       String partKeyName = partKeyInfo.getPartName();
 
       // reconstruct full part name with volume, bucket, partKeyName
@@ -699,15 +689,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
   @Test
   public void testListMultipartUploadPartsWithContinuation()
       throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
     Map<Integer, String> partsMap = new TreeMap<>();
     String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
         ONE);
@@ -763,15 +744,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
   @Test
   public void testListPartsInvalidPartMarker() throws Exception {
     try {
-      String volumeName = UUID.randomUUID().toString();
-      String bucketName = UUID.randomUUID().toString();
-      String keyName = UUID.randomUUID().toString();
-
-      store.createVolume(volumeName);
-      OzoneVolume volume = store.getVolume(volumeName);
-      volume.createBucket(bucketName);
-      OzoneBucket bucket = volume.getBucket(bucketName);
-
       bucket.listParts(keyName, "random", -1, 2);
       Assert.fail("Should throw exception as partNumber is an invalid number!");
     } catch (IllegalArgumentException ex) {
@@ -783,15 +755,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
   @Test
   public void testListPartsInvalidMaxParts() throws Exception {
     try {
-      String volumeName = UUID.randomUUID().toString();
-      String bucketName = UUID.randomUUID().toString();
-      String keyName = UUID.randomUUID().toString();
-
-      store.createVolume(volumeName);
-      OzoneVolume volume = store.getVolume(volumeName);
-      volume.createBucket(bucketName);
-      OzoneBucket bucket = volume.getBucket(bucketName);
-
       bucket.listParts(keyName, "random", 1, -1);
       Assert.fail("Should throw exception as max parts is an invalid number!");
     } catch (IllegalArgumentException ex) {
@@ -803,16 +766,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
   @Test
   public void testListPartsWithPartMarkerGreaterThanPartCount()
       throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String keyName = UUID.randomUUID().toString();
-
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
-
     String uploadID = initiateMultipartUpload(bucket, keyName, RATIS,
         ONE);
     uploadPart(bucket, keyName, uploadID, 1,
@@ -840,14 +793,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
   public void testListPartsWithInvalidUploadID() throws Exception {
     OzoneTestUtils
         .expectOmException(NO_SUCH_MULTIPART_UPLOAD_ERROR, () -> {
-          String volumeName = UUID.randomUUID().toString();
-          String bucketName = UUID.randomUUID().toString();
-          String keyName = UUID.randomUUID().toString();
-
-          store.createVolume(volumeName);
-          OzoneVolume volume = store.getVolume(volumeName);
-          volume.createBucket(bucketName);
-          OzoneBucket bucket = volume.getBucket(bucketName);
           OzoneMultipartUploadPartListParts ozoneMultipartUploadPartListParts =
               bucket.listParts(keyName, "random", 100, 2);
         });
@@ -855,8 +800,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
 
   @Test
   public void testListMultipartUpload() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
     String dirName = "dir1/dir2/dir3";
     String key1 = "dir1" + "/key1";
     String key2 = "dir1/dir2" + "/key2";
@@ -866,11 +809,6 @@ public class TestOzoneClientMultipartUploadWithFSO {
     keys.add(key2);
     keys.add(key3);
 
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
-
     // Initiate multipart upload
     String uploadID1 = initiateMultipartUpload(bucket, key1, RATIS,
         ONE);
@@ -931,8 +869,7 @@ public class TestOzoneClientMultipartUploadWithFSO {
     Assert.assertEquals(0, expectedList.size());
   }
 
-  private String verifyUploadedPart(String volumeName, String bucketName,
-      String keyName, String uploadID, String partName,
+  private String verifyUploadedPart(String uploadID, String partName,
       OMMetadataManager metadataMgr) throws IOException {
     OzoneManager ozoneManager = cluster.getOzoneManager();
     String buckKey = ozoneManager.getMetadataManager()
@@ -957,11 +894,8 @@ public class TestOzoneClientMultipartUploadWithFSO {
         omKeyInfo.getKeyName());
     Assert.assertEquals(uploadID, omMultipartKeyInfo.getUploadID());
 
-    TreeMap<Integer, OzoneManagerProtocolProtos.PartKeyInfo> partKeyInfoMap =
-        omMultipartKeyInfo.getPartKeyInfoMap();
-    for (Map.Entry<Integer, OzoneManagerProtocolProtos.PartKeyInfo> entry :
-        partKeyInfoMap.entrySet()) {
-      OzoneManagerProtocolProtos.PartKeyInfo partKeyInfo = entry.getValue();
+    for (OzoneManagerProtocolProtos.PartKeyInfo partKeyInfo :
+        omMultipartKeyInfo.getPartKeyInfoMap()) {
       OmKeyInfo currentKeyPartInfo =
           OmKeyInfo.getFromProtobuf(partKeyInfo.getPartKeyInfo());
 
@@ -974,14 +908,14 @@ public class TestOzoneClientMultipartUploadWithFSO {
   }
 
   private String getMultipartOpenKey(String multipartUploadID,
-      String volumeName, String bucketName, String keyName,
+      String volName, String buckName, String kName,
       OMMetadataManager omMetadataManager) throws IOException {
 
-    String fileName = OzoneFSUtils.getFileName(keyName);
-    final long volumeId = omMetadataManager.getVolumeId(volumeName);
-    final long bucketId = omMetadataManager.getBucketId(volumeName,
-            bucketName);
-    long parentID = getParentID(volumeName, bucketName, keyName,
+    String fileName = OzoneFSUtils.getFileName(kName);
+    final long volumeId = omMetadataManager.getVolumeId(volName);
+    final long bucketId = omMetadataManager.getBucketId(volName,
+        buckName);
+    long parentID = getParentID(volName, buckName, kName,
         omMetadataManager);
 
     String multipartKey = omMetadataManager.getMultipartKey(volumeId, bucketId,
@@ -990,32 +924,36 @@ public class TestOzoneClientMultipartUploadWithFSO {
     return multipartKey;
   }
 
-  private long getParentID(String volumeName, String bucketName,
-      String keyName, OMMetadataManager omMetadataManager) throws IOException {
-    Iterator<Path> pathComponents = Paths.get(keyName).iterator();
-    final long volumeId = omMetadataManager.getVolumeId(volumeName);
-    final long bucketId = omMetadataManager.getBucketId(volumeName,
-            bucketName);
+  private long getParentID(String volName, String buckName,
+      String kName, OMMetadataManager omMetadataManager) throws IOException {
+    Iterator<Path> pathComponents = Paths.get(kName).iterator();
+    final long volumeId = omMetadataManager.getVolumeId(volName);
+    final long bucketId = omMetadataManager.getBucketId(volName,
+        buckName);
     return OMFileRequest.getParentID(volumeId, bucketId, pathComponents,
-        keyName, omMetadataManager);
+        kName, omMetadataManager);
   }
 
-  private String initiateMultipartUpload(OzoneBucket bucket, String keyName,
+  private String initiateMultipartUpload(OzoneBucket oBucket, String kName,
       ReplicationType replicationType, ReplicationFactor replicationFactor)
-          throws Exception {
-    OmMultipartInfo multipartInfo = bucket.initiateMultipartUpload(keyName,
+          throws IOException {
+    OmMultipartInfo multipartInfo = oBucket.initiateMultipartUpload(kName,
             replicationType, replicationFactor);
 
+    Assert.assertNotNull(multipartInfo);
     String uploadID = multipartInfo.getUploadID();
-    Assert.assertNotNull(uploadID);
+    Assert.assertEquals(volumeName, multipartInfo.getVolumeName());
+    Assert.assertEquals(bucketName, multipartInfo.getBucketName());
+    Assert.assertEquals(kName, multipartInfo.getKeyName());
+    Assert.assertNotNull(multipartInfo.getUploadID());
 
     return uploadID;
   }
 
-  private String uploadPart(OzoneBucket bucket, String keyName, String
-      uploadID, int partNumber, byte[] data) throws Exception {
+  private String uploadPart(OzoneBucket oBucket, String kName, String
+      uploadID, int partNumber, byte[] data) throws IOException {
 
-    OzoneOutputStream ozoneOutputStream = bucket.createMultipartKey(keyName,
+    OzoneOutputStream ozoneOutputStream = oBucket.createMultipartKey(kName,
             data.length, partNumber, uploadID);
     ozoneOutputStream.write(data, 0,
             data.length);
@@ -1030,17 +968,17 @@ public class TestOzoneClientMultipartUploadWithFSO {
     return omMultipartCommitUploadPartInfo.getPartName();
   }
 
-  private void completeMultipartUpload(OzoneBucket bucket, String keyName,
+  private void completeMultipartUpload(OzoneBucket oBucket, String kName,
       String uploadID, Map<Integer, String> partsMap) throws Exception {
-    OmMultipartUploadCompleteInfo omMultipartUploadCompleteInfo = bucket
-            .completeMultipartUpload(keyName, uploadID, partsMap);
+    OmMultipartUploadCompleteInfo omMultipartUploadCompleteInfo = oBucket
+            .completeMultipartUpload(kName, uploadID, partsMap);
 
     Assert.assertNotNull(omMultipartUploadCompleteInfo);
-    Assert.assertEquals(omMultipartUploadCompleteInfo.getBucket(), bucket
+    Assert.assertEquals(omMultipartUploadCompleteInfo.getBucket(), oBucket
             .getName());
-    Assert.assertEquals(omMultipartUploadCompleteInfo.getVolume(), bucket
+    Assert.assertEquals(omMultipartUploadCompleteInfo.getVolume(), oBucket
             .getVolumeName());
-    Assert.assertEquals(omMultipartUploadCompleteInfo.getKey(), keyName);
+    Assert.assertEquals(omMultipartUploadCompleteInfo.getKey(), kName);
     Assert.assertNotNull(omMultipartUploadCompleteInfo.getHash());
   }
 
