@@ -27,24 +27,33 @@ import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.protocolPB.StorageContainerLocationProtocolClientSideTranslatorPB;
-import org.apache.hadoop.hdds.security.x509.SecurityConfig;
+import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClientTestImpl;
+import org.apache.hadoop.hdds.utils.db.Table;
+import org.apache.hadoop.hdds.utils.db.TableIterator;
+import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
+import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
 import org.apache.hadoop.ozone.ClientVersion;
 import org.apache.hadoop.ozone.MiniOzoneCluster;
 import org.apache.hadoop.ozone.OzoneConfigKeys;
-import org.apache.hadoop.ozone.client.CertificateClientTestImpl;
 import org.apache.hadoop.ozone.client.ObjectStore;
 import org.apache.hadoop.ozone.client.OzoneBucket;
 import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientFactory;
 import org.apache.hadoop.ozone.client.OzoneKey;
 import org.apache.hadoop.ozone.client.OzoneVolume;
+import org.apache.hadoop.ozone.client.BucketArgs;
+import org.apache.hadoop.ozone.client.SecretKeyTestClient;
 import org.apache.hadoop.ozone.client.io.OzoneInputStream;
 import org.apache.hadoop.ozone.client.io.OzoneOutputStream;
+import org.apache.hadoop.ozone.om.OMConfigKeys;
+import org.apache.hadoop.ozone.om.OMMetadataManager;
 import org.apache.hadoop.ozone.om.OzoneManager;
-import org.apache.hadoop.ozone.om.helpers.OmKeyArgs;
+import org.apache.hadoop.ozone.om.S3SecretManager;
+import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
-import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
 import org.apache.hadoop.ozone.om.helpers.S3SecretValue;
+import org.apache.hadoop.ozone.om.helpers.OmKeyArgs;
+import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.CreateVolumeRequest;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.InfoVolumeRequest;
@@ -53,14 +62,10 @@ import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMRespo
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.S3Authentication;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.VolumeInfo;
-import org.apache.hadoop.hdds.security.token.OzoneBlockTokenSecretManager;
 import org.apache.hadoop.security.UserGroupInformation;
 import org.apache.ozone.test.GenericTestUtils;
-import org.apache.ozone.test.LambdaTestUtils;
 import org.junit.jupiter.api.AfterAll;
-import org.junit.Assert;
 import org.junit.jupiter.api.BeforeAll;
-import org.junit.jupiter.api.Disabled;
 import org.junit.jupiter.api.Test;
 
 import java.io.File;
@@ -71,6 +76,11 @@ import java.util.UUID;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.hadoop.hdds.HddsConfigKeys.OZONE_METADATA_DIRS;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 
 /**
  * This class is to test all the public facing APIs of Ozone Client.
@@ -88,7 +98,6 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
   private static final String CLUSTER_ID = UUID.randomUUID().toString();
   private static File testDir;
   private static OzoneConfiguration conf;
-  private static OzoneBlockTokenSecretManager secretManager;
 
   /**
    * Create a MiniOzoneCluster for testing.
@@ -112,16 +121,18 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
         OzoneConfigKeys.OZONE_ACL_AUTHORIZER_CLASS_NATIVE);
     CertificateClientTestImpl certificateClientTest =
         new CertificateClientTestImpl(conf);
+    // These tests manually insert keys into RocksDB. This is easier to do
+    // with object store layout so keys with path IDs do not need to be
+    // constructed.
+    conf.set(OMConfigKeys.OZONE_DEFAULT_BUCKET_LAYOUT,
+        OMConfigKeys.OZONE_BUCKET_LAYOUT_OBJECT_STORE);
     cluster = MiniOzoneCluster.newBuilder(conf)
         .setNumDatanodes(14)
         .setScmId(SCM_ID)
         .setClusterId(CLUSTER_ID)
         .setCertificateClient(certificateClientTest)
+        .setSecretKeyClient(new SecretKeyTestClient())
         .build();
-    secretManager = new OzoneBlockTokenSecretManager(new SecurityConfig(conf),
-        60 * 60, certificateClientTest.getCertificate().
-        getSerialNumber().toString());
-    secretManager.start(certificateClientTest);
     cluster.getOzoneManager().startSecretManager();
     cluster.waitForClusterToBeReady();
     ozClient = OzoneClientFactory.getRpcClient(conf);
@@ -146,14 +157,23 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
    * */
   @Test
   public void testPutKeySuccessWithBlockToken() throws Exception {
+    testPutKeySuccessWithBlockTokenWithBucketLayout(BucketLayout.OBJECT_STORE);
+    testPutKeySuccessWithBlockTokenWithBucketLayout(
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
+  }
+
+  private void testPutKeySuccessWithBlockTokenWithBucketLayout(
+      BucketLayout bucketLayout) throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
     Instant testStartTime = Instant.now();
+    OMMetadataManager omMetadataManager = ozoneManager.getMetadataManager();
 
     String value = "sample value";
     store.createVolume(volumeName);
     OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
+    volume.createBucket(bucketName,
+        new BucketArgs.Builder().setBucketLayout(bucketLayout).build());
     OzoneBucket bucket = volume.getBucket(bucketName);
 
     for (int i = 0; i < 10; i++) {
@@ -166,62 +186,58 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
         out.write(value.getBytes(UTF_8));
       }
 
-      Assert.assertEquals(committedBytes + value.getBytes(UTF_8).length,
-              ozoneManager.getMetrics().getDataCommittedBytes());
+      assertEquals(committedBytes + value.getBytes(UTF_8).length,
+          ozoneManager.getMetrics().getDataCommittedBytes());
 
       OzoneKey key = bucket.getKey(keyName);
-      Assert.assertEquals(keyName, key.getName());
+      assertEquals(keyName, key.getName());
       byte[] fileContent;
       try (OzoneInputStream is = bucket.readKey(keyName)) {
         fileContent = new byte[value.getBytes(UTF_8).length];
         is.read(fileContent);
       }
 
-      Assert.assertTrue(verifyRatisReplication(volumeName, bucketName,
-          keyName, ReplicationType.RATIS,
-          ReplicationFactor.ONE));
-      Assert.assertEquals(value, new String(fileContent, UTF_8));
-      Assert.assertFalse(key.getCreationTime().isBefore(testStartTime));
-      Assert.assertFalse(key.getModificationTime().isBefore(testStartTime));
-    }
-  }
+      String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
+      String bucketId = String.valueOf(
+          omMetadataManager.getBucketTable().get(bucketKey).getObjectID());
+      String keyPrefix =
+          bucketLayout.isFileSystemOptimized() ? bucketId : bucketKey;
+      Table table = omMetadataManager.getKeyTable(bucketLayout);
 
-  /**
-   * Tests failure in following operations when grpc block token is
-   * not present.
-   * 1. getKey
-   * 2. writeChunk
-   * */
-  @Test
-  @Disabled("Needs to be moved out of this class as  client setup is static")
-  public void testKeyOpFailureWithoutBlockToken() throws Exception {
-    String volumeName = UUID.randomUUID().toString();
-    String bucketName = UUID.randomUUID().toString();
-    String value = "sample value";
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    volume.createBucket(bucketName);
-    OzoneBucket bucket = volume.getBucket(bucketName);
+      // Check table entry.
+      try (
+          TableIterator<String, ? extends Table.KeyValue<String, OmKeyInfo>>
+              keyIterator = table.iterator()) {
+        Table.KeyValue<String, OmKeyInfo> kv =
+            keyIterator.seek(keyPrefix + "/" + keyName);
 
-    for (int i = 0; i < 10; i++) {
-      String keyName = UUID.randomUUID().toString();
-
-      try (OzoneOutputStream out = bucket.createKey(keyName,
-          value.getBytes(UTF_8).length, ReplicationType.RATIS,
-          ReplicationFactor.ONE, new HashMap<>())) {
-        LambdaTestUtils.intercept(IOException.class, "UNAUTHENTICATED: Fail " +
-                "to find any token ",
-            () -> out.write(value.getBytes(UTF_8)));
+        CacheValue<OmKeyInfo> cacheValue =
+            table.getCacheValue(new CacheKey(kv.getKey()));
+        if (cacheValue != null) {
+          assertTokenIsNull(cacheValue.getCacheValue());
+        } else {
+          assertTokenIsNull(kv.getValue());
+        }
       }
 
-      OzoneKey key = bucket.getKey(keyName);
-      Assert.assertEquals(keyName, key.getName());
-      LambdaTestUtils.intercept(IOException.class, "Failed to authenticate" +
-              " with GRPC XceiverServer with Ozone block token.",
-          () -> bucket.readKey(keyName));
+
+      assertTrue(verifyRatisReplication(volumeName, bucketName,
+          keyName, ReplicationType.RATIS,
+          ReplicationFactor.ONE));
+      assertEquals(value, new String(fileContent, UTF_8));
+      assertFalse(key.getCreationTime().isBefore(testStartTime));
+      assertFalse(key.getModificationTime().isBefore(testStartTime));
     }
   }
 
+  private void assertTokenIsNull(OmKeyInfo value) {
+    value.getKeyLocationVersions()
+        .forEach(
+            keyLocationInfoGroup -> keyLocationInfoGroup.getLocationList()
+                .forEach(
+                    keyLocationInfo -> assertNull(keyLocationInfo
+                        .getToken())));
+  }
 
   @Test
   public void testS3Auth() throws Exception {
@@ -242,9 +258,12 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
 
     String accessKey = UserGroupInformation.getCurrentUser().getUserName();
 
+    S3SecretManager s3SecretManager = cluster.getOzoneManager()
+        .getS3SecretManager();
+
     // Add secret to S3Secret table.
-    cluster.getOzoneManager().getMetadataManager().getS3SecretTable().put(
-        accessKey, new S3SecretValue(accessKey, secret));
+    s3SecretManager.storeSecret(accessKey,
+        new S3SecretValue(accessKey, secret));
 
     OMRequest writeRequest = OMRequest.newBuilder()
         .setCmdType(OzoneManagerProtocolProtos.Type.CreateVolume)
@@ -265,7 +284,7 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
         .submitRequest(null, writeRequest);
 
     // Verify response.
-    Assert.assertTrue(omResponse.getStatus() == Status.OK);
+    assertEquals(Status.OK, omResponse.getStatus());
 
     // Read Request
     OMRequest readRequest = OMRequest.newBuilder()
@@ -283,29 +302,27 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
         .submitRequest(null, readRequest);
 
     // Verify response.
-    Assert.assertTrue(omResponse.getStatus() == Status.OK);
+    assertEquals(Status.OK, omResponse.getStatus());
 
     VolumeInfo volumeInfo = omResponse.getInfoVolumeResponse().getVolumeInfo();
-    Assert.assertNotNull(volumeInfo);
-    Assert.assertEquals(volumeName, volumeInfo.getVolume());
-    Assert.assertEquals(accessKey, volumeInfo.getAdminName());
-    Assert.assertEquals(accessKey, volumeInfo.getOwnerName());
+    assertNotNull(volumeInfo);
+    assertEquals(volumeName, volumeInfo.getVolume());
+    assertEquals(accessKey, volumeInfo.getAdminName());
+    assertEquals(accessKey, volumeInfo.getOwnerName());
 
-    // Override secret to S3Secret table with some dummy value
-    cluster.getOzoneManager().getMetadataManager().getS3SecretTable().put(
-        accessKey, new S3SecretValue(accessKey, "dummy"));
+    // Override secret to S3Secret store with some dummy value
+    s3SecretManager
+        .storeSecret(accessKey, new S3SecretValue(accessKey, "dummy"));
 
     // Write request with invalid credentials.
     omResponse = cluster.getOzoneManager().getOmServerProtocol()
         .submitRequest(null, writeRequest);
-    Assert.assertTrue(omResponse.getStatus() == Status.INVALID_TOKEN);
+    assertEquals(Status.INVALID_TOKEN, omResponse.getStatus());
 
-  // Read request with invalid credentials.
+    // Read request with invalid credentials.
     omResponse = cluster.getOzoneManager().getOmServerProtocol()
         .submitRequest(null, readRequest);
-    Assert.assertTrue(omResponse.getStatus() == Status.INVALID_TOKEN);
-
-
+    assertEquals(Status.INVALID_TOKEN, omResponse.getStatus());
   }
 
   private boolean verifyRatisReplication(String volumeName, String bucketName,
@@ -357,5 +374,4 @@ public class TestSecureOzoneRpcClient extends TestOzoneRpcClient {
       cluster.shutdown();
     }
   }
-
 }
