@@ -47,8 +47,10 @@ import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.protocolPB.StorageContainerLocationProtocolClientSideTranslatorPB;
 import org.apache.hadoop.hdds.scm.storage.BlockOutputStream;
 import org.apache.hadoop.hdds.scm.storage.RatisBlockOutputStream;
+import org.apache.hadoop.hdds.utils.IOUtils;
 import org.apache.hadoop.ozone.HddsDatanodeService;
 import org.apache.hadoop.ozone.MiniOzoneCluster;
+import org.apache.hadoop.ozone.OzoneConfigKeys;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.RatisTestHelper;
 import org.apache.hadoop.ozone.client.ObjectStore;
@@ -62,7 +64,6 @@ import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ozone.test.tag.Flaky;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
-import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_PIPELINE_DESTROY_TIMEOUT;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_STALENODE_INTERVAL;
 import org.apache.ratis.protocol.exceptions.GroupMismatchException;
 import org.junit.Assert;
@@ -108,9 +109,10 @@ public class TestWatchForCommit {
     OzoneClientConfig clientConfig = conf.getObject(OzoneClientConfig.class);
     clientConfig.setStreamBufferFlushDelay(false);
     conf.setFromObject(clientConfig);
-    
-    conf.setTimeDuration(OZONE_SCM_PIPELINE_DESTROY_TIMEOUT, 10,
-            TimeUnit.SECONDS);
+
+    conf.set(OzoneConfigKeys.OZONE_SCM_CLOSE_CONTAINER_WAIT_DURATION, "2s");
+    conf.set(ScmConfigKeys.OZONE_SCM_PIPELINE_SCRUB_INTERVAL, "2s");
+    conf.set(ScmConfigKeys.OZONE_SCM_PIPELINE_DESTROY_TIMEOUT, "5s");
     conf.setQuietMode(false);
     conf.setInt(ScmConfigKeys.OZONE_DATANODE_PIPELINE_LIMIT, 1);
 
@@ -161,6 +163,7 @@ public class TestWatchForCommit {
    */
   @AfterEach
   public void shutdown() {
+    IOUtils.closeQuietly(client);
     if (cluster != null) {
       cluster.shutdown();
     }
@@ -234,143 +237,153 @@ public class TestWatchForCommit {
     Assert.assertTrue(keyOutputStream.getRetryCount() == 0);
     // now close the stream, It will update the ack length after watchForCommit
     key.close();
-    Assert
-        .assertEquals(0, blockOutputStream.getBufferPool().computeBufferData());
-    Assert
-        .assertEquals(dataLength, blockOutputStream.getTotalAckDataLength());
-    Assert.assertNull(blockOutputStream.getCommitIndex2flushedDataMap());
     Assert
         .assertEquals(dataLength, blockOutputStream.getTotalAckDataLength());
     // make sure the bufferPool is empty
     Assert
         .assertEquals(0, blockOutputStream.getBufferPool().computeBufferData());
-    Assert.assertNull(blockOutputStream.getCommitIndex2flushedDataMap());
+    Assert.assertTrue(
+        blockOutputStream.getCommitIndex2flushedDataMap().isEmpty());
     validateData(keyName, data1);
   }
 
   @Test
   public void testWatchForCommitForRetryfailure() throws Exception {
-    XceiverClientManager clientManager = new XceiverClientManager(conf);
-    ContainerWithPipeline container1 = storageContainerLocationClient
-        .allocateContainer(HddsProtos.ReplicationType.RATIS,
-            HddsProtos.ReplicationFactor.THREE, OzoneConsts.OZONE);
-    XceiverClientSpi xceiverClient = clientManager
-        .acquireClient(container1.getPipeline());
-    Assert.assertEquals(1, xceiverClient.getRefcount());
-    Assert.assertEquals(container1.getPipeline(),
-        xceiverClient.getPipeline());
-    Pipeline pipeline = xceiverClient.getPipeline();
-    TestHelper.createPipelineOnDatanode(pipeline, cluster);
-    XceiverClientReply reply = xceiverClient.sendCommandAsync(
-        ContainerTestHelper.getCreateContainerRequest(
-            container1.getContainerInfo().getContainerID(),
-            xceiverClient.getPipeline()));
-    reply.getResponse().get();
-    long index = reply.getLogIndex();
-    cluster.shutdownHddsDatanode(pipeline.getNodes().get(0));
-    cluster.shutdownHddsDatanode(pipeline.getNodes().get(1));
-    // again write data with more than max buffer limit. This wi
-    try {
-      // just watch for a log index which in not updated in the commitInfo Map
-      // as well as there is no logIndex generate in Ratis.
-      // The basic idea here is just to test if its throws an exception.
-      xceiverClient
-          .watchForCommit(index + new Random().nextInt(100) + 10);
-      Assert.fail("expected exception not thrown");
-    } catch (Exception e) {
-      Assert.assertTrue(e instanceof ExecutionException);
-      // since the timeout value is quite long, the watch request will either
-      // fail with NotReplicated exceptio, RetryFailureException or
-      // RuntimeException
-      Assert.assertFalse(HddsClientUtils
-          .checkForException(e) instanceof TimeoutException);
+    GenericTestUtils.LogCapturer logCapturer =
+        GenericTestUtils.LogCapturer.captureLogs(XceiverClientRatis.LOG);
+    try (XceiverClientManager clientManager = new XceiverClientManager(conf)) {
+      ContainerWithPipeline container1 = storageContainerLocationClient
+          .allocateContainer(HddsProtos.ReplicationType.RATIS,
+              HddsProtos.ReplicationFactor.THREE, OzoneConsts.OZONE);
+      XceiverClientSpi xceiverClient = clientManager
+          .acquireClient(container1.getPipeline());
+      Assert.assertEquals(1, xceiverClient.getRefcount());
+      Assert.assertEquals(container1.getPipeline(),
+          xceiverClient.getPipeline());
+      Pipeline pipeline = xceiverClient.getPipeline();
+      TestHelper.createPipelineOnDatanode(pipeline, cluster);
+      XceiverClientReply reply = xceiverClient.sendCommandAsync(
+          ContainerTestHelper.getCreateContainerRequest(
+              container1.getContainerInfo().getContainerID(),
+              xceiverClient.getPipeline()));
+      reply.getResponse().get();
+      long index = reply.getLogIndex();
+      cluster.shutdownHddsDatanode(pipeline.getNodes().get(0));
+      cluster.shutdownHddsDatanode(pipeline.getNodes().get(1));
+      // emulate closing pipeline when SCM detects DEAD datanodes
+      cluster.getStorageContainerManager()
+          .getPipelineManager().closePipeline(pipeline, false);
+      // again write data with more than max buffer limit. This wi
+      try {
+        // just watch for a log index which in not updated in the commitInfo Map
+        // as well as there is no logIndex generate in Ratis.
+        // The basic idea here is just to test if its throws an exception.
+        xceiverClient
+            .watchForCommit(index + new Random().nextInt(100) + 10);
+        Assert.fail("expected exception not thrown");
+      } catch (Exception e) {
+        Assert.assertTrue(e instanceof ExecutionException);
+        // since the timeout value is quite long, the watch request will either
+        // fail with NotReplicated exceptio, RetryFailureException or
+        // RuntimeException
+        Assert.assertFalse(HddsClientUtils
+            .checkForException(e) instanceof TimeoutException);
+        // client should not attempt to watch with
+        // MAJORITY_COMMITTED replication level, except the grpc IO issue
+        if (!logCapturer.getOutput().contains("Connection refused")) {
+          Assert.assertFalse(
+              e.getMessage().contains("Watch-MAJORITY_COMMITTED"));
+        }
+      }
+      clientManager.releaseClient(xceiverClient, false);
     }
-    clientManager.releaseClient(xceiverClient, false);
   }
 
   @Test
   public void test2WayCommitForTimeoutException() throws Exception {
     GenericTestUtils.LogCapturer logCapturer =
         GenericTestUtils.LogCapturer.captureLogs(XceiverClientRatis.LOG);
-    XceiverClientManager clientManager = new XceiverClientManager(conf);
+    try (XceiverClientManager clientManager = new XceiverClientManager(conf)) {
 
-    ContainerWithPipeline container1 = storageContainerLocationClient
-        .allocateContainer(HddsProtos.ReplicationType.RATIS,
-            HddsProtos.ReplicationFactor.THREE, OzoneConsts.OZONE);
-    XceiverClientSpi xceiverClient = clientManager
-        .acquireClient(container1.getPipeline());
-    Assert.assertEquals(1, xceiverClient.getRefcount());
-    Assert.assertEquals(container1.getPipeline(),
-        xceiverClient.getPipeline());
-    Pipeline pipeline = xceiverClient.getPipeline();
-    TestHelper.createPipelineOnDatanode(pipeline, cluster);
-    XceiverClientRatis ratisClient = (XceiverClientRatis) xceiverClient;
-    XceiverClientReply reply = xceiverClient.sendCommandAsync(
-        ContainerTestHelper.getCreateContainerRequest(
-            container1.getContainerInfo().getContainerID(),
-            xceiverClient.getPipeline()));
-    reply.getResponse().get();
-    Assert.assertEquals(3, ratisClient.getCommitInfoMap().size());
-    List<DatanodeDetails> nodesInPipeline = pipeline.getNodes();
-    for (HddsDatanodeService dn : cluster.getHddsDatanodes()) {
-      // shutdown the ratis follower
-      if (nodesInPipeline.contains(dn.getDatanodeDetails())
-          && RatisTestHelper.isRatisFollower(dn, pipeline)) {
-        cluster.shutdownHddsDatanode(dn.getDatanodeDetails());
-        break;
+      ContainerWithPipeline container1 = storageContainerLocationClient
+          .allocateContainer(HddsProtos.ReplicationType.RATIS,
+              HddsProtos.ReplicationFactor.THREE, OzoneConsts.OZONE);
+      XceiverClientSpi xceiverClient = clientManager
+          .acquireClient(container1.getPipeline());
+      Assert.assertEquals(1, xceiverClient.getRefcount());
+      Assert.assertEquals(container1.getPipeline(),
+          xceiverClient.getPipeline());
+      Pipeline pipeline = xceiverClient.getPipeline();
+      TestHelper.createPipelineOnDatanode(pipeline, cluster);
+      XceiverClientRatis ratisClient = (XceiverClientRatis) xceiverClient;
+      XceiverClientReply reply = xceiverClient.sendCommandAsync(
+          ContainerTestHelper.getCreateContainerRequest(
+              container1.getContainerInfo().getContainerID(),
+              xceiverClient.getPipeline()));
+      reply.getResponse().get();
+      Assert.assertEquals(3, ratisClient.getCommitInfoMap().size());
+      List<DatanodeDetails> nodesInPipeline = pipeline.getNodes();
+      for (HddsDatanodeService dn : cluster.getHddsDatanodes()) {
+        // shutdown the ratis follower
+        if (nodesInPipeline.contains(dn.getDatanodeDetails())
+            && RatisTestHelper.isRatisFollower(dn, pipeline)) {
+          cluster.shutdownHddsDatanode(dn.getDatanodeDetails());
+          break;
+        }
       }
-    }
-    reply = xceiverClient.sendCommandAsync(ContainerTestHelper
-        .getCloseContainer(pipeline,
-            container1.getContainerInfo().getContainerID()));
-    reply.getResponse().get();
-    xceiverClient.watchForCommit(reply.getLogIndex());
+      reply = xceiverClient.sendCommandAsync(ContainerTestHelper
+          .getCloseContainer(pipeline,
+              container1.getContainerInfo().getContainerID()));
+      reply.getResponse().get();
+      xceiverClient.watchForCommit(reply.getLogIndex());
 
-    // commitInfo Map will be reduced to 2 here
-    Assert.assertEquals(2, ratisClient.getCommitInfoMap().size());
-    clientManager.releaseClient(xceiverClient, false);
-    Assert.assertTrue(logCapturer.getOutput().contains("3 way commit failed"));
-    Assert.assertTrue(logCapturer.getOutput().contains("TimeoutException"));
-    Assert
-        .assertTrue(logCapturer.getOutput().contains("Committed by majority"));
+      // commitInfo Map will be reduced to 2 here
+      Assert.assertEquals(2, ratisClient.getCommitInfoMap().size());
+      clientManager.releaseClient(xceiverClient, false);
+      String output = logCapturer.getOutput();
+      Assert.assertTrue(output.contains("3 way commit failed"));
+      Assert.assertTrue(output.contains("TimeoutException"));
+      Assert.assertTrue(output.contains("Committed by majority"));
+    }
     logCapturer.stopCapturing();
   }
 
   @Test
   public void testWatchForCommitForGroupMismatchException() throws Exception {
-    XceiverClientManager clientManager = new XceiverClientManager(conf);
-    ContainerWithPipeline container1 = storageContainerLocationClient
-        .allocateContainer(HddsProtos.ReplicationType.RATIS,
-            HddsProtos.ReplicationFactor.THREE, OzoneConsts.OZONE);
-    XceiverClientSpi xceiverClient = clientManager
-        .acquireClient(container1.getPipeline());
-    Assert.assertEquals(1, xceiverClient.getRefcount());
-    Assert.assertEquals(container1.getPipeline(),
-        xceiverClient.getPipeline());
-    Pipeline pipeline = xceiverClient.getPipeline();
-    XceiverClientRatis ratisClient = (XceiverClientRatis) xceiverClient;
-    long containerId = container1.getContainerInfo().getContainerID();
-    XceiverClientReply reply = xceiverClient.sendCommandAsync(
-        ContainerTestHelper.getCreateContainerRequest(containerId,
-            xceiverClient.getPipeline()));
-    reply.getResponse().get();
-    Assert.assertEquals(3, ratisClient.getCommitInfoMap().size());
-    List<Pipeline> pipelineList = new ArrayList<>();
-    pipelineList.add(pipeline);
-    TestHelper.waitForPipelineClose(pipelineList, cluster);
-    try {
-      // just watch for a log index which in not updated in the commitInfo Map
-      // as well as there is no logIndex generate in Ratis.
-      // The basic idea here is just to test if its throws an exception.
-      xceiverClient
-          .watchForCommit(reply.getLogIndex() +
-                  new Random().nextInt(100) + 10);
-      Assert.fail("Expected exception not thrown");
-    } catch (Exception e) {
-      Assert.assertTrue(HddsClientUtils
-          .checkForException(e) instanceof GroupMismatchException);
+    try (XceiverClientManager clientManager = new XceiverClientManager(conf)) {
+      ContainerWithPipeline container1 = storageContainerLocationClient
+          .allocateContainer(HddsProtos.ReplicationType.RATIS,
+              HddsProtos.ReplicationFactor.THREE, OzoneConsts.OZONE);
+      XceiverClientSpi xceiverClient = clientManager
+          .acquireClient(container1.getPipeline());
+      Assert.assertEquals(1, xceiverClient.getRefcount());
+      Assert.assertEquals(container1.getPipeline(),
+          xceiverClient.getPipeline());
+      Pipeline pipeline = xceiverClient.getPipeline();
+      XceiverClientRatis ratisClient = (XceiverClientRatis) xceiverClient;
+      long containerId = container1.getContainerInfo().getContainerID();
+      XceiverClientReply reply = xceiverClient.sendCommandAsync(
+          ContainerTestHelper.getCreateContainerRequest(containerId,
+              xceiverClient.getPipeline()));
+      reply.getResponse().get();
+      Assert.assertEquals(3, ratisClient.getCommitInfoMap().size());
+      List<Pipeline> pipelineList = new ArrayList<>();
+      pipelineList.add(pipeline);
+      TestHelper.waitForPipelineClose(pipelineList, cluster);
+      try {
+        // just watch for a log index which in not updated in the commitInfo Map
+        // as well as there is no logIndex generate in Ratis.
+        // The basic idea here is just to test if its throws an exception.
+        xceiverClient
+            .watchForCommit(reply.getLogIndex() +
+                new Random().nextInt(100) + 10);
+        Assert.fail("Expected exception not thrown");
+      } catch (Exception e) {
+        Assert.assertTrue(HddsClientUtils
+            .checkForException(e) instanceof GroupMismatchException);
+      }
+      clientManager.releaseClient(xceiverClient, false);
     }
-    clientManager.releaseClient(xceiverClient, false);
   }
 
   private OzoneOutputStream createKey(String keyName, ReplicationType type,
