@@ -22,8 +22,12 @@ import static org.apache.hadoop.ozone.security.acl.OzoneObj.ResourceType.VOLUME;
 import static org.apache.hadoop.ozone.security.acl.OzoneObj.StoreType.OZONE;
 import static org.apache.hadoop.test.MetricsAsserts.assertCounter;
 import static org.apache.hadoop.test.MetricsAsserts.getMetrics;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyInt;
+import static org.mockito.ArgumentMatchers.eq;
 
 import java.io.IOException;
 import java.util.Collections;
@@ -31,9 +35,11 @@ import java.util.List;
 import java.util.UUID;
 import java.util.concurrent.TimeUnit;
 
+import org.apache.hadoop.hdds.utils.IOUtils;
 import org.apache.hadoop.hdds.HddsConfigKeys;
 import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.client.ContainerBlockID;
+import org.apache.hadoop.hdds.client.DefaultReplicationConfig;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
@@ -47,6 +53,7 @@ import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.ozone.OzoneAcl;
 import org.apache.hadoop.ozone.TestDataUtil;
 import org.apache.hadoop.ozone.client.ObjectStore;
+import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.helpers.OmBucketArgs;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
@@ -58,25 +65,19 @@ import org.apache.hadoop.ozone.om.protocol.OzoneManagerProtocol;
 import org.apache.hadoop.ozone.security.acl.IAccessAuthorizer;
 import org.apache.hadoop.ozone.security.acl.OzoneObj;
 import org.apache.hadoop.ozone.security.acl.OzoneObjInfo;
+import org.apache.hadoop.ozone.snapshot.SnapshotDiffResponse;
 import org.assertj.core.util.Lists;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.Timeout;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
 import org.mockito.Mockito;
 
 /**
  * Test for OM metrics.
  */
+@Timeout(300)
 public class TestOmMetrics {
-
-  /**
-    * Set a timeout for each test.
-    */
-  @Rule
-  public Timeout timeout = Timeout.seconds(300);
   private MiniOzoneCluster cluster;
   private MiniOzoneCluster.Builder clusterBuilder;
   private OzoneConfiguration conf;
@@ -87,11 +88,12 @@ public class TestOmMetrics {
    */
   private final OMException exception =
       new OMException("dummyException", OMException.ResultCodes.TIMEOUT);
+  private OzoneClient client;
 
   /**
    * Create a MiniDFSCluster for testing.
    */
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
     conf = new OzoneConfiguration();
     conf.setTimeDuration(OMConfigKeys.OZONE_OM_METRICS_SAVE_INTERVAL,
@@ -103,22 +105,22 @@ public class TestOmMetrics {
     cluster = clusterBuilder.build();
     cluster.waitForClusterToBeReady();
     ozoneManager = cluster.getOzoneManager();
-    writeClient = cluster.getRpcClient().getObjectStore()
+    client = cluster.newClient();
+    writeClient = client.getObjectStore()
         .getClientProxy().getOzoneManagerClient();
   }
 
   /**
    * Shutdown MiniDFSCluster.
    */
-  @After
+  @AfterEach
   public void shutdown() {
+    IOUtils.closeQuietly(client);
     if (cluster != null) {
       cluster.shutdown();
     }
   }
 
-
-
   @Test
   public void testVolumeOps() throws Exception {
     startCluster();
@@ -196,7 +198,7 @@ public class TestOmMetrics {
             ozoneManager, "bucketManager");
     BucketManager mockBm = Mockito.spy(bucketManager);
 
-    OmBucketInfo bucketInfo = createBucketInfo();
+    OmBucketInfo bucketInfo = createBucketInfo(false);
     doBucketOps(bucketInfo);
 
     MetricsRecordBuilder omMetrics = getMetrics("OMMetrics");
@@ -208,11 +210,18 @@ public class TestOmMetrics {
     assertCounter("NumBucketLists", 1L, omMetrics);
     assertCounter("NumBuckets", 0L, omMetrics);
 
-    bucketInfo = createBucketInfo();
+    OmBucketInfo ecBucketInfo = createBucketInfo(true);
+    writeClient.createBucket(ecBucketInfo);
+    writeClient.deleteBucket(ecBucketInfo.getVolumeName(),
+        ecBucketInfo.getBucketName());
+    omMetrics = getMetrics("OMMetrics");
+    assertCounter("EcBucketCreateTotal", 1L, omMetrics);
+
+    bucketInfo = createBucketInfo(false);
     writeClient.createBucket(bucketInfo);
-    bucketInfo = createBucketInfo();
+    bucketInfo = createBucketInfo(false);
     writeClient.createBucket(bucketInfo);
-    bucketInfo = createBucketInfo();
+    bucketInfo = createBucketInfo(false);
     writeClient.createBucket(bucketInfo);
     writeClient.deleteBucket(bucketInfo.getVolumeName(),
         bucketInfo.getBucketName());
@@ -223,7 +232,7 @@ public class TestOmMetrics {
     // inject exception to test for Failure Metrics on the read path
     Mockito.doThrow(exception).when(mockBm).getBucketInfo(any(), any());
     Mockito.doThrow(exception).when(mockBm).listBuckets(any(), any(),
-        any(), anyInt());
+        any(), anyInt(), eq(false));
 
     HddsWhiteboxTestUtils.setInternalState(
         ozoneManager, "bucketManager", mockBm);
@@ -232,15 +241,24 @@ public class TestOmMetrics {
     mockWritePathExceptions(OmBucketInfo.class);
     doBucketOps(bucketInfo);
 
+    ecBucketInfo = createBucketInfo(true);
+    try {
+      writeClient.createBucket(ecBucketInfo);
+    } catch (Exception e) {
+      //Expected failure
+    }
     omMetrics = getMetrics("OMMetrics");
-    assertCounter("NumBucketOps", 14L, omMetrics);
-    assertCounter("NumBucketCreates", 5L, omMetrics);
+    assertCounter("EcBucketCreateFailsTotal", 1L, omMetrics);
+
+    omMetrics = getMetrics("OMMetrics");
+    assertCounter("NumBucketOps", 17L, omMetrics);
+    assertCounter("NumBucketCreates", 7L, omMetrics);
     assertCounter("NumBucketUpdates", 2L, omMetrics);
     assertCounter("NumBucketInfos", 2L, omMetrics);
-    assertCounter("NumBucketDeletes", 3L, omMetrics);
+    assertCounter("NumBucketDeletes", 4L, omMetrics);
     assertCounter("NumBucketLists", 2L, omMetrics);
 
-    assertCounter("NumBucketCreateFails", 1L, omMetrics);
+    assertCounter("NumBucketCreateFails", 2L, omMetrics);
     assertCounter("NumBucketUpdateFails", 1L, omMetrics);
     assertCounter("NumBucketInfoFails", 1L, omMetrics);
     assertCounter("NumBucketDeleteFails", 1L, omMetrics);
@@ -263,7 +281,7 @@ public class TestOmMetrics {
     KeyManager keyManager = (KeyManager) HddsWhiteboxTestUtils
         .getInternalState(ozoneManager, "keyManager");
     KeyManager mockKm = Mockito.spy(keyManager);
-    TestDataUtil.createVolumeAndBucket(cluster, volumeName, bucketName);
+    TestDataUtil.createVolumeAndBucket(client, volumeName, bucketName);
     OmKeyArgs keyArgs = createKeyArgs(volumeName, bucketName,
         RatisReplicationConfig.getInstance(HddsProtos.ReplicationFactor.THREE));
     doKeyOps(keyArgs);
@@ -283,6 +301,7 @@ public class TestOmMetrics {
     doKeyOps(keyArgs);
     omMetrics = getMetrics("OMMetrics");
     assertCounter("NumKeyOps", 14L, omMetrics);
+    assertCounter("EcKeyCreateTotal", 1L, omMetrics);
 
     keyArgs = createKeyArgs(volumeName, bucketName,
         RatisReplicationConfig.getInstance(HddsProtos.ReplicationFactor.THREE));
@@ -298,18 +317,35 @@ public class TestOmMetrics {
     writeClient.commitKey(keyArgs, keySession.getId());
     writeClient.deleteKey(keyArgs);
 
+    keyArgs = createKeyArgs(volumeName, bucketName,
+        new ECReplicationConfig("rs-6-3-1024K"));
+    try {
+      keySession = writeClient.openKey(keyArgs);
+      writeClient.commitKey(keyArgs, keySession.getId());
+    } catch (Exception e) {
+      //Expected Failure in preExecute due to not enough datanode
+      assertTrue(e.getMessage().contains("No enough datanodes to choose"),
+          e::getMessage);
+    }
+
     omMetrics = getMetrics("OMMetrics");
     assertCounter("NumKeys", 2L, omMetrics);
+    assertCounter("NumBlockAllocationFails", 1L, omMetrics);
 
     // inject exception to test for Failure Metrics on the read path
-    Mockito.doThrow(exception).when(mockKm).lookupKey(any(), any());
+    Mockito.doThrow(exception).when(mockKm).lookupKey(any(), any(), any());
     Mockito.doThrow(exception).when(mockKm).listKeys(
         any(), any(), any(), any(), anyInt());
     Mockito.doThrow(exception).when(mockKm).listTrash(
         any(), any(), any(), any(), anyInt());
+    OmMetadataReader omMetadataReader =
+        (OmMetadataReader) ozoneManager.getOmMetadataReader().get();
     HddsWhiteboxTestUtils.setInternalState(
         ozoneManager, "keyManager", mockKm);
 
+    HddsWhiteboxTestUtils.setInternalState(
+        omMetadataReader, "keyManager", mockKm);
+
     // inject exception to test for Failure Metrics on the write path
     mockWritePathExceptions(OmBucketInfo.class);
     keyArgs = createKeyArgs(volumeName, bucketName,
@@ -331,15 +367,108 @@ public class TestOmMetrics {
     assertCounter("NumKeyListFails", 1L, omMetrics);
     assertCounter("NumTrashKeyListFails", 1L, omMetrics);
     assertCounter("NumInitiateMultipartUploadFails", 1L, omMetrics);
-
-
     assertCounter("NumKeys", 2L, omMetrics);
 
+    keyArgs = createKeyArgs(volumeName, bucketName,
+        new ECReplicationConfig("rs-3-2-1024K"));
+    try {
+      keySession = writeClient.openKey(keyArgs);
+      writeClient.commitKey(keyArgs, keySession.getId());
+    } catch (Exception e) {
+      //Expected Failure
+    }
+    omMetrics = getMetrics("OMMetrics");
+    assertCounter("EcKeyCreateFailsTotal", 1L, omMetrics);
+
     cluster.restartOzoneManager();
     assertCounter("NumKeys", 2L, omMetrics);
 
   }
 
+  @Test
+  public void testSnapshotOps() throws Exception {
+    // This tests needs enough dataNodes to allocate the blocks for the keys.
+    clusterBuilder.setNumDatanodes(3);
+    startCluster();
+
+    OmBucketInfo omBucketInfo = createBucketInfo(false);
+
+    String volumeName = omBucketInfo.getVolumeName();
+    String bucketName = omBucketInfo.getBucketName();
+    String snapshot1 = "snap1";
+    String snapshot2 = "snap2";
+
+    writeClient.createBucket(omBucketInfo);
+
+    // Create first key
+    OmKeyArgs keyArgs1 = createKeyArgs(volumeName, bucketName,
+        RatisReplicationConfig.getInstance(HddsProtos.ReplicationFactor.THREE));
+    OpenKeySession keySession = writeClient.openKey(keyArgs1);
+    writeClient.commitKey(keyArgs1, keySession.getId());
+
+    // Create first snapshot
+    writeClient.createSnapshot(volumeName, bucketName, snapshot1);
+
+    MetricsRecordBuilder omMetrics = getMetrics("OMMetrics");
+
+    assertCounter("NumSnapshotCreateFails", 0L, omMetrics);
+    assertCounter("NumSnapshotCreates", 1L, omMetrics);
+    assertCounter("NumSnapshotListFails", 0L, omMetrics);
+    assertCounter("NumSnapshotLists", 0L, omMetrics);
+    assertCounter("NumSnapshotActive", 1L, omMetrics);
+    assertCounter("NumSnapshotDeleted", 0L, omMetrics);
+    assertCounter("NumSnapshotDiffJobs", 0L, omMetrics);
+    assertCounter("NumSnapshotDiffJobFails", 0L, omMetrics);
+
+    // Create second key
+    OmKeyArgs keyArgs2 = createKeyArgs(volumeName, bucketName,
+        RatisReplicationConfig.getInstance(HddsProtos.ReplicationFactor.THREE));
+    OpenKeySession keySession2 = writeClient.openKey(keyArgs2);
+    writeClient.commitKey(keyArgs2, keySession2.getId());
+
+    // Create second snapshot
+    writeClient.createSnapshot(volumeName, bucketName, snapshot2);
+
+    // Snapshot diff
+    while (true) {
+      SnapshotDiffResponse response =
+          writeClient.snapshotDiff(volumeName, bucketName, snapshot1, snapshot2,
+              null, 100, false, false);
+      if (response.getJobStatus() == SnapshotDiffResponse.JobStatus.DONE) {
+        break;
+      } else {
+        Thread.sleep(response.getWaitTimeInMs());
+      }
+    }
+    omMetrics = getMetrics("OMMetrics");
+    assertCounter("NumSnapshotDiffJobs", 1L, omMetrics);
+    assertCounter("NumSnapshotDiffJobFails", 0L, omMetrics);
+
+    // List snapshots
+    writeClient.listSnapshot(
+        volumeName, bucketName, null, null, Integer.MAX_VALUE);
+
+    omMetrics = getMetrics("OMMetrics");
+    assertCounter("NumSnapshotActive", 2L, omMetrics);
+    assertCounter("NumSnapshotCreates", 2L, omMetrics);
+    assertCounter("NumSnapshotLists", 1L, omMetrics);
+    assertCounter("NumSnapshotListFails", 0L, omMetrics);
+
+    // List snapshot: invalid bucket case.
+    assertThrows(OMException.class, () -> writeClient.listSnapshot(volumeName,
+        "invalidBucket", null, null, Integer.MAX_VALUE));
+    omMetrics = getMetrics("OMMetrics");
+    assertCounter("NumSnapshotLists", 2L, omMetrics);
+    assertCounter("NumSnapshotListFails", 1L, omMetrics);
+
+    // restart OM
+    cluster.restartOzoneManager();
+
+    // Check number of active snapshots in the snapshot table
+    // is the same after OM restart
+    assertCounter("NumSnapshotActive", 2L, omMetrics);
+  }
+
   private <T> void mockWritePathExceptions(Class<T>klass) throws Exception {
     String tableName;
     if (klass == OmBucketInfo.class) {
@@ -369,7 +498,7 @@ public class TestOmMetrics {
     startCluster();
     try {
       // Create a volume.
-      cluster.getClient().getObjectStore().createVolume("volumeacl");
+      client.getObjectStore().createVolume("volumeacl");
 
       OzoneObj volObj = new OzoneObjInfo.Builder().setVolumeName("volumeacl")
           .setResType(VOLUME).setStoreType(OZONE).build();
@@ -397,7 +526,7 @@ public class TestOmMetrics {
       assertCounter("NumRemoveAcl", 1L, omMetrics);
 
     } finally {
-      cluster.getClient().getObjectStore().deleteVolume("volumeacl");
+      client.getObjectStore().deleteVolume("volumeacl");
     }
   }
 
@@ -408,7 +537,7 @@ public class TestOmMetrics {
     conf.setBoolean(HddsConfigKeys.HDDS_SCM_SAFEMODE_ENABLED, true);
     startCluster();
 
-    ObjectStore objectStore = cluster.getClient().getObjectStore();
+    ObjectStore objectStore = client.getObjectStore();
     // Create a volume.
     objectStore.createVolume("volumeacl");
     // Create a bucket.
@@ -450,18 +579,18 @@ public class TestOmMetrics {
         new OzoneAcl(IAccessAuthorizer.ACLIdentityType.USER, "ozoneuser",
             IAccessAuthorizer.ACLType.ALL, ACCESS));
 
-    Assert.assertEquals(initialValue + 1, metrics.getNumAddAcl());
+    assertEquals(initialValue + 1, metrics.getNumAddAcl());
 
     // Test setAcl
     initialValue = metrics.getNumSetAcl();
 
     objectStore.setAcl(volObj, acls);
-    Assert.assertEquals(initialValue + 1, metrics.getNumSetAcl());
+    assertEquals(initialValue + 1, metrics.getNumSetAcl());
 
     // Test removeAcl
     initialValue = metrics.getNumRemoveAcl();
     objectStore.removeAcl(volObj, acls.get(0));
-    Assert.assertEquals(initialValue + 1, metrics.getNumRemoveAcl());
+    assertEquals(initialValue + 1, metrics.getNumRemoveAcl());
   }
 
   /**
@@ -514,7 +643,7 @@ public class TestOmMetrics {
     }
 
     try {
-      ozoneManager.listBuckets(info.getVolumeName(), null, null, 0);
+      ozoneManager.listBuckets(info.getVolumeName(), null, null, 0, false);
     } catch (IOException ignored) {
     }
 
@@ -601,13 +730,19 @@ public class TestOmMetrics {
         .setBucketName(info.getBucketName())
         .build();
   }
-  private OmBucketInfo createBucketInfo() throws IOException {
+  private OmBucketInfo createBucketInfo(boolean isEcBucket) throws IOException {
     OmVolumeArgs volumeArgs = createVolumeArgs();
     writeClient.createVolume(volumeArgs);
+    DefaultReplicationConfig repConf = new DefaultReplicationConfig(
+        new ECReplicationConfig("rs-3-2-1024k"));
     String bucketName = UUID.randomUUID().toString();
-    return new OmBucketInfo.Builder()
+
+    OmBucketInfo.Builder builder = new OmBucketInfo.Builder()
         .setVolumeName(volumeArgs.getVolume())
-        .setBucketName(bucketName)
-        .build();
+        .setBucketName(bucketName);
+    if (isEcBucket) {
+      builder.setDefaultReplicationConfig(repConf);
+    }
+    return builder.build();
   }
 }
