@@ -35,14 +35,13 @@ import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.security.acl.OzoneObj;
 import org.apache.hadoop.ozone.security.acl.OzoneObjInfo;
 import org.apache.hadoop.security.UserGroupInformation;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.Timeout;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
 
 import java.io.IOException;
+import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
@@ -52,14 +51,16 @@ import static org.apache.hadoop.ozone.OzoneConfigKeys.OZONE_ACL_AUTHORIZER_CLASS
 import static org.apache.hadoop.ozone.OzoneConfigKeys.OZONE_ACL_AUTHORIZER_CLASS_NATIVE;
 import static org.apache.hadoop.ozone.OzoneConfigKeys.OZONE_ACL_ENABLED;
 import static org.apache.hadoop.ozone.security.acl.OzoneObj.StoreType.OZONE;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.fail;
 
 /**
  * Test recursive acl checks for delete and rename for FSO Buckets.
  */
+@Timeout(120)
 public class TestRecursiveAclWithFSO {
 
-  @Rule public Timeout timeout = Timeout.seconds(120);
-
   private MiniOzoneCluster cluster;
 
   private final UserGroupInformation adminUser =
@@ -69,7 +70,7 @@ public class TestRecursiveAclWithFSO {
   private final UserGroupInformation user2 = UserGroupInformation
       .createUserForTesting("user2", new String[] {"test2"});
 
-  @Before
+  @BeforeEach
   public void init() throws Exception {
     // loginUser is the user running this test.
     // Implication: loginUser is automatically added to the OM admin list.
@@ -81,150 +82,154 @@ public class TestRecursiveAclWithFSO {
 
   @Test
   public void testKeyDeleteAndRenameWithoutPermission() throws Exception {
-
-    List<String> keys = new ArrayList<>();
-    // Create volumes with user1
-
-    OzoneClient client = cluster.getClient();
-    ObjectStore objectStore = client.getObjectStore();
-
     /* r = READ, w = WRITE, c = CREATE, d = DELETE
        l = LIST, a = ALL, n = NONE, x = READ_ACL, y = WRITE_ACL */
     String aclWorldAll = "world::a";
-    createVolumeWithOwnerAndAcl(objectStore, "volume1", "user1", aclWorldAll);
+    List<String> keys = new ArrayList<>();
+    // Create volumes with user1
+    try (OzoneClient client = cluster.newClient()) {
+      ObjectStore objectStore = client.getObjectStore();
+      createVolumeWithOwnerAndAcl(objectStore, "volume1", "user1", aclWorldAll);
+    }
 
     // Login as user1, create directories and keys
     UserGroupInformation.setLoginUser(user1);
-    client = cluster.getClient();
-    objectStore = client.getObjectStore();
+    try (OzoneClient client = cluster.newClient()) {
+      ObjectStore objectStore = client.getObjectStore();
+      OzoneVolume volume = objectStore.getVolume("volume1");
+      BucketArgs omBucketArgs =
+          BucketArgs.newBuilder().setStorageType(StorageType.DISK).build();
 
-    OzoneVolume volume = objectStore.getVolume("volume1");
+      // create bucket with user1
+      volume.createBucket("bucket1", omBucketArgs);
+      setBucketAcl(objectStore, volume.getName(), "bucket1", aclWorldAll);
+      OzoneBucket ozoneBucket = volume.getBucket("bucket1");
 
-    BucketArgs omBucketArgs =
-        BucketArgs.newBuilder().setStorageType(StorageType.DISK).build();
+      /*
+       *                       buck-1
+       *                        |
+       *                        a
+       *                        |
+       *          ------------------------------------
+       *         |           |              |        |
+       *         b1          b2             b3      file1
+       *       -----       ------           -----
+       *       |    |      |    |          |    |
+       *      c1   c2     d1   d2          e1   e2
+       *       |    |      |    |           |    |
+       *       f1   f2     f3  --------     f5   f6
+       *                      |        |
+       *                    d21        file2
+       *                     |
+       *                     f4
+       *
+       *     Test Case 1 :
+       *     Remove delete acl from file File2
+       *     Try deleting b2
+       *
+       *     Test case 2:
+       *     Remove delete acl from dir c2
+       *     Try deleting b1
+       *
+       *     Test case 3
+       *     try deleting b3
+       */
 
-    // create bucket with user1
-    volume.createBucket("bucket1", omBucketArgs);
-    setBucketAcl(objectStore, volume.getName(), "bucket1", aclWorldAll);
-    OzoneBucket ozoneBucket = volume.getBucket("bucket1");
+      String keyf1 = "a/b1/c1/f1";
+      String keyf2 = "a/b1/c2/f2";
+      String keyf3 = "a/b2/d1/f3";
+      String keyf4 = "a/b2/d2/d21/f4";
+      String keyf5 = "/a/b3/e1/f5";
+      String keyf6 = "/a/b3/e2/f6";
+      String file1 = "a/" + "file" + RandomStringUtils.randomNumeric(5);
+      String file2 = "a/b2/d2/" + "file" + RandomStringUtils.randomNumeric(5);
 
-    /**
-     *                       buck-1
-     *                        |
-     *                        a
-     *                        |
-     *          ------------------------------------
-     *         |           |              |        |
-     *         b1          b2             b3      file1
-     *       -----       ------           -----
-     *       |    |      |    |          |    |
-     *      c1   c2     d1   d2          e1   e2
-     *       |    |      |    |           |    |
-     *       f1   f2     f3  --------     f5   f6
-     *                      |        |
-     *                    d21        file2
-     *                     |
-     *                     f4
-     *
-     *     Test Case 1 :
-     *     Remove delete acl from file File2
-     *     Try deleting b2
-     *
-     *     Test case 2:
-     *     Remove delete acl fro dir c2
-     *     Try deleting b1
-     *
-     *     Test case 3
-     *     try deleting b3
-     */
+      keys.add(keyf1);
+      keys.add(keyf2);
+      keys.add(keyf3);
+      keys.add(keyf4);
+      keys.add(keyf5);
+      keys.add(keyf6);
+      keys.add(file1);
+      keys.add(file2);
+      createKeys(objectStore, ozoneBucket, keys);
 
-    String keyf1 = "a/b1/c1/f1";
-    String keyf2 = "a/b1/c2/f2";
-    String keyf3 = "a/b2/d1/f3";
-    String keyf4 = "a/b2/d2/d21/f4";
-    String keyf5 = "/a/b3/e1/f5";
-    String keyf6 = "/a/b3/e2/f6";
-
-    String file1 = "a/" + "file" + RandomStringUtils.randomNumeric(5);
-    String file2 = "a/b2/d2/" + "file" + RandomStringUtils.randomNumeric(5);
-
-    keys.add(keyf1);
-    keys.add(keyf2);
-    keys.add(keyf3);
-    keys.add(keyf4);
-    keys.add(keyf5);
-    keys.add(keyf6);
-    keys.add(file1);
-    keys.add(file2);
-
-    createKeys(objectStore, ozoneBucket, keys);
-
-    // Test case 1
-    // Remove acls from file2
-    // Delete/Rename on directory a/b2 should throw permission denied
-    // (since file2 is a child)
-    removeAclsFromKey(objectStore, ozoneBucket, file2);
-    OzoneObj ozoneObj;
-    List<OzoneAcl> aclList1;
+      // Test case 1
+      // Remove acls from file2
+      // Delete/Rename on directory a/b2 should throw permission denied
+      // (since file2 is a child)
+      removeAclsFromKey(objectStore, ozoneBucket, file2);
+    }
 
     UserGroupInformation.setLoginUser(user2);
-    client = cluster.getClient();
-    objectStore = client.getObjectStore();
-    volume = objectStore.getVolume("volume1");
-    ozoneBucket = volume.getBucket("bucket1");
+    try (OzoneClient client = cluster.newClient()) {
+      ObjectStore objectStore = client.getObjectStore();
+      OzoneVolume volume = objectStore.getVolume("volume1");
+      OzoneBucket ozoneBucket = volume.getBucket("bucket1");
 
-    // perform  delete
-    try {
-      ozoneBucket.deleteDirectory("a/b2", true);
-      Assert.fail("Should throw permission denied !");
-    } catch (OMException ome) {
-      // expect permission error
-      Assert.assertEquals("Permission check failed",
-          OMException.ResultCodes.PERMISSION_DENIED, ome.getResult());
+      // perform  delete
+      try {
+        ozoneBucket.deleteDirectory("a/b2", true);
+        fail("Should throw permission denied !");
+      } catch (OMException ome) {
+        // expect permission error
+        assertEquals(OMException.ResultCodes.PERMISSION_DENIED,
+            ome.getResult(), "Permission check failed");
+      }
+      // perform rename
+      try {
+        ozoneBucket.renameKey("a/b2", "a/b2_renamed");
+        fail("Should throw permission denied !");
+      } catch (OMException ome) {
+        // expect permission error
+        assertEquals(OMException.ResultCodes.PERMISSION_DENIED,
+            ome.getResult(), "Permission check failed");
+      }
+
+      // Test case 2
+      // Remove acl from directory c2, delete/rename a/b1 should throw
+      // permission denied since c2 is a subdirectory
+      user1.doAs((PrivilegedExceptionAction<Void>) () -> {
+        try (OzoneClient c = cluster.newClient()) {
+          ObjectStore o = c.getObjectStore();
+          OzoneBucket b = o.getVolume("volume1").getBucket("bucket1");
+          removeAclsFromKey(o, b, "a/b1/c2");
+        }
+        return null;
+      });
+
+      UserGroupInformation.setLoginUser(user2);
+      // perform  delete
+      try {
+        ozoneBucket.deleteDirectory("a/b1", true);
+        fail("Should throw permission denied !");
+      } catch (OMException ome) {
+        // expect permission error
+        assertEquals(OMException.ResultCodes.PERMISSION_DENIED,
+            ome.getResult(), "Permission check failed");
+      }
+
+      // perform rename
+      try {
+        ozoneBucket.renameKey("a/b1", "a/b1_renamed");
+        fail("Should throw permission denied !");
+      } catch (OMException ome) {
+        // expect permission error
+        assertEquals(OMException.ResultCodes.PERMISSION_DENIED,
+            ome.getResult(), "Permission check failed");
+      }
+
+      // Test case 3
+      // delete b3 and this should throw exception because user2 has no acls
+      try {
+        ozoneBucket.deleteDirectory("a/b3", true);
+        fail("Should throw permission denied !");
+      } catch (OMException ome) {
+        // expect permission error
+        assertEquals(OMException.ResultCodes.PERMISSION_DENIED,
+            ome.getResult(), "Permission check failed");
+      }
     }
-
-    // perform rename
-    try {
-      ozoneBucket.renameKey("a/b2", "a/b2_renamed");
-      Assert.fail("Should throw permission denied !");
-    } catch (OMException ome) {
-      // expect permission error
-      Assert.assertEquals("Permission check failed",
-          OMException.ResultCodes.PERMISSION_DENIED, ome.getResult());
-    }
-
-    // Test case 2
-    // Remove acl from directory c2, delete/rename a/b1 should throw
-    // permission denied since c2 is a subdirectory
-
-    UserGroupInformation.setLoginUser(user1);
-    removeAclsFromKey(objectStore, ozoneBucket, "a/b1/c2");
-
-    UserGroupInformation.setLoginUser(user2);
-    // perform  delete
-    try {
-      ozoneBucket.deleteDirectory("a/b1", true);
-      Assert.fail("Should throw permission denied !");
-    } catch (OMException ome) {
-      // expect permission error
-      Assert.assertEquals("Permission check failed",
-          OMException.ResultCodes.PERMISSION_DENIED, ome.getResult());
-    }
-
-    // perform rename
-    try {
-      ozoneBucket.renameKey("a/b1", "a/b1_renamed");
-      Assert.fail("Should throw permission denied !");
-    } catch (OMException ome) {
-      // expect permission error
-      Assert.assertEquals("Permission check failed",
-          OMException.ResultCodes.PERMISSION_DENIED, ome.getResult());
-    }
-
-    // Test case 3
-    // delete b3 and this shouldn't throw exception because acls have not
-    // been removed from subpaths.
-    ozoneBucket.deleteDirectory("a/b3", true);
   }
 
   private void removeAclsFromKey(ObjectStore objectStore,
@@ -264,7 +269,7 @@ public class TestRecursiveAclWithFSO {
 
   }
 
-  @After
+  @AfterEach
   public void stopCluster() {
     if (cluster != null) {
       cluster.shutdown();
@@ -287,7 +292,7 @@ public class TestRecursiveAclWithFSO {
       String aclString) throws IOException {
     OzoneObj obj = OzoneObjInfo.Builder.newBuilder().setVolumeName(volumeName)
         .setResType(OzoneObj.ResourceType.VOLUME).setStoreType(OZONE).build();
-    Assert.assertTrue(objectStore.setAcl(obj, OzoneAcl.parseAcls(aclString)));
+    assertTrue(objectStore.setAcl(obj, OzoneAcl.parseAcls(aclString)));
   }
 
   /**
@@ -298,7 +303,7 @@ public class TestRecursiveAclWithFSO {
     OzoneObj obj = OzoneObjInfo.Builder.newBuilder().setVolumeName(volumeName)
         .setBucketName(bucket).setResType(OzoneObj.ResourceType.BUCKET)
         .setStoreType(OZONE).build();
-    Assert.assertTrue(objectStore.setAcl(obj, OzoneAcl.parseAcls(aclString)));
+    assertTrue(objectStore.setAcl(obj, OzoneAcl.parseAcls(aclString)));
   }
 
   /**
@@ -309,7 +314,7 @@ public class TestRecursiveAclWithFSO {
     OzoneObj obj = OzoneObjInfo.Builder.newBuilder().setVolumeName(volumeName)
         .setBucketName(bucket).setKeyName(key)
         .setResType(OzoneObj.ResourceType.KEY).setStoreType(OZONE).build();
-    Assert.assertTrue(objectStore.setAcl(obj, OzoneAcl.parseAcls(aclString)));
+    assertTrue(objectStore.setAcl(obj, OzoneAcl.parseAcls(aclString)));
   }
 
   private void createKeys(ObjectStore objectStore, OzoneBucket ozoneBucket,
