@@ -23,8 +23,8 @@ import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.ozone.OzoneAcl;
 import org.apache.hadoop.ozone.storage.proto.OzoneManagerStorageProtos;
 import org.apache.hadoop.ozone.security.acl.IAccessAuthorizer;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Test;
 
 import java.util.Collections;
 import java.util.HashMap;
@@ -91,7 +91,7 @@ public class TestOmPrefixInfo {
         ACCESS);
     OmPrefixInfo clonePrefixInfo = omPrefixInfo.copyObject();
 
-    Assert.assertEquals(omPrefixInfo, clonePrefixInfo);
+    Assertions.assertEquals(omPrefixInfo, clonePrefixInfo);
 
 
     // Change acls and check.
@@ -99,7 +99,7 @@ public class TestOmPrefixInfo {
         IAccessAuthorizer.ACLIdentityType.USER, username,
         IAccessAuthorizer.ACLType.READ, ACCESS));
 
-    Assert.assertNotEquals(omPrefixInfo, clonePrefixInfo);
+    Assertions.assertNotEquals(omPrefixInfo, clonePrefixInfo);
 
   }
 
@@ -116,10 +116,10 @@ public class TestOmPrefixInfo {
 
     OmPrefixInfo ompri = OmPrefixInfo.getFromProtobuf(prefixInfo);
 
-    Assert.assertEquals(prefixInfoPath, ompri.getName());
-    Assert.assertEquals(1, ompri.getMetadata().size());
-    Assert.assertEquals(metaval, ompri.getMetadata().get(metakey));
-    Assert.assertEquals(1, ompri.getAcls().size());
+    Assertions.assertEquals(prefixInfoPath, ompri.getName());
+    Assertions.assertEquals(1, ompri.getMetadata().size());
+    Assertions.assertEquals(metaval, ompri.getMetadata().get(metakey));
+    Assertions.assertEquals(1, ompri.getAcls().size());
   }
 
   @Test
@@ -133,8 +133,8 @@ public class TestOmPrefixInfo {
     omPrefixInfo.getMetadata().put("key", "value");
     OzoneManagerStorageProtos.PersistedPrefixInfo pi =
         omPrefixInfo.getProtobuf();
-    Assert.assertEquals(testPath, pi.getName());
-    Assert.assertEquals(1, pi.getAclsCount());
-    Assert.assertEquals(1, pi.getMetadataCount());
+    Assertions.assertEquals(testPath, pi.getName());
+    Assertions.assertEquals(1, pi.getAclsCount());
+    Assertions.assertEquals(1, pi.getMetadataCount());
   }
 }
