@@ -938,9 +938,8 @@ public class OMMultiTenantManagerImpl implements OMMultiTenantManager {
 
     final UserGroupInformation ugi = ProtobufRpcEngine.Server.getRemoteUser();
     if (!ozoneManager.isAdmin(ugi)) {
-      throw new OMException("User '" + ugi.getUserName() + "' or '" +
-          ugi.getShortUserName() + "' is not an Ozone admin",
-          OMException.ResultCodes.PERMISSION_DENIED);
+      throw new OMException("User '" + ugi.getShortUserName() +
+          "' is not an Ozone admin", OMException.ResultCodes.PERMISSION_DENIED);
     }
   }
 
