@@ -26,12 +26,17 @@ import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.EnumSet;
+import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
+import java.util.stream.Collectors;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Optional;
 import com.google.common.base.Preconditions;
 import org.apache.commons.lang3.tuple.Pair;
+import org.apache.hadoop.hdds.client.ContainerBlockID;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
@@ -39,6 +44,7 @@ import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
 import org.apache.hadoop.ozone.OmUtils;
 import org.apache.hadoop.ozone.OzoneAcl;
 import org.apache.hadoop.ozone.OzoneConsts;
+import org.apache.hadoop.ozone.om.OMMetrics;
 import org.apache.hadoop.ozone.om.PrefixManager;
 import org.apache.hadoop.ozone.om.ResolvedBucket;
 import org.apache.hadoop.ozone.om.helpers.BucketEncryptionKeyInfo;
@@ -56,6 +62,7 @@ import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
 import org.apache.hadoop.ozone.om.lock.OzoneLockStrategy;
 import org.apache.hadoop.ozone.om.request.OMClientRequestUtils;
 import org.apache.hadoop.ozone.om.request.file.OMFileRequest;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.UserInfo;
 import org.apache.hadoop.ozone.protocolPB.OMPBHelper;
 import org.apache.hadoop.ozone.security.acl.IAccessAuthorizer;
 import org.apache.hadoop.ozone.security.acl.OzoneObj;
@@ -85,6 +92,7 @@ import org.apache.hadoop.security.UserGroupInformation;
 
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.BlockTokenSecretProto.AccessModeProto.READ;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.BlockTokenSecretProto.AccessModeProto.WRITE;
+import static org.apache.hadoop.ozone.OzoneConsts.OBJECT_ID_RECLAIM_BLOCKS;
 import static org.apache.hadoop.ozone.OzoneConsts.OZONE_URI_DELIMITER;
 import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes
     .BUCKET_NOT_FOUND;
@@ -98,7 +106,8 @@ import static org.apache.hadoop.util.Time.monotonicNow;
  */
 public abstract class OMKeyRequest extends OMClientRequest {
 
-  private static final Logger LOG = LoggerFactory.getLogger(OMKeyRequest.class);
+  @VisibleForTesting
+  public static final Logger LOG = LoggerFactory.getLogger(OMKeyRequest.class);
 
   private BucketLayout bucketLayout = BucketLayout.DEFAULT;
 
@@ -134,20 +143,28 @@ public abstract class OMKeyRequest extends OMClientRequest {
       OzoneBlockTokenSecretManager secretManager,
       ReplicationConfig replicationConfig, ExcludeList excludeList,
       long requestedSize, long scmBlockSize, int preallocateBlocksMax,
-      boolean grpcBlockTokenEnabled, String omID) throws IOException {
+      boolean grpcBlockTokenEnabled, String serviceID, OMMetrics omMetrics,
+      boolean shouldSortDatanodes, UserInfo userInfo)
+      throws IOException {
     int dataGroupSize = replicationConfig instanceof ECReplicationConfig
         ? ((ECReplicationConfig) replicationConfig).getData() : 1;
     int numBlocks = (int) Math.min(preallocateBlocksMax,
         (requestedSize - 1) / (scmBlockSize * dataGroupSize) + 1);
 
+    String clientMachine = "";
+    if (shouldSortDatanodes) {
+      clientMachine = userInfo.getRemoteAddress();
+    }
+
     List<OmKeyLocationInfo> locationInfos = new ArrayList<>(numBlocks);
     String remoteUser = getRemoteUser().getShortUserName();
     List<AllocatedBlock> allocatedBlocks;
     try {
       allocatedBlocks = scmClient.getBlockClient()
-          .allocateBlock(scmBlockSize, numBlocks, replicationConfig, omID,
-              excludeList);
+          .allocateBlock(scmBlockSize, numBlocks, replicationConfig, serviceID,
+              excludeList, clientMachine);
     } catch (SCMException ex) {
+      omMetrics.incNumBlockAllocateCallFails();
       if (ex.getResult()
           .equals(SCMException.ResultCodes.SAFE_MODE_EXCEPTION)) {
         throw new OMException(ex.getMessage(),
@@ -266,9 +283,10 @@ public abstract class OMKeyRequest extends OMClientRequest {
   }
 
   protected List< OzoneAcl > getAclsForKey(KeyArgs keyArgs,
-      OmBucketInfo bucketInfo, PrefixManager prefixManager) {
-    List<OzoneAcl> acls = new ArrayList<>();
+      OmBucketInfo bucketInfo, OMFileRequest.OMPathInfo omPathInfo,
+      PrefixManager prefixManager) {
 
+    List<OzoneAcl> acls = new ArrayList<>();
     if (keyArgs.getAclsList() != null) {
       acls.addAll(OzoneAclUtil.fromProtobuf(keyArgs.getAclsList()));
     }
@@ -292,8 +310,16 @@ public abstract class OMKeyRequest extends OMClientRequest {
       }
     }
 
+    // Inherit DEFAULT acls from parent-dir only if DEFAULT acls for
+    // prefix are not set
+    if (omPathInfo != null) {
+      if (OzoneAclUtil.inheritDefaultAcls(acls, omPathInfo.getAcls())) {
+        return acls;
+      }
+    }
+
     // Inherit DEFAULT acls from bucket only if DEFAULT acls for
-    // prefix are not set.
+    // parent-dir are not set.
     if (bucketInfo != null) {
       if (OzoneAclUtil.inheritDefaultAcls(acls, bucketInfo.getAcls())) {
         return acls;
@@ -303,6 +329,39 @@ public abstract class OMKeyRequest extends OMClientRequest {
     return acls;
   }
 
+  /**
+   * Inherit parent DEFAULT acls and generate its own ACCESS acls.
+   * @param keyArgs
+   * @param bucketInfo
+   * @param omPathInfo
+   * @return Acls which inherited parent DEFAULT and keyArgs ACCESS acls.
+   */
+  protected static List<OzoneAcl> getAclsForDir(KeyArgs keyArgs,
+      OmBucketInfo bucketInfo, OMFileRequest.OMPathInfo omPathInfo) {
+    // Acls inherited from parent or bucket will convert to DEFAULT scope
+    List<OzoneAcl> acls = new ArrayList<>();
+
+    // Inherit DEFAULT acls from parent-dir
+    if (omPathInfo != null) {
+      if (OzoneAclUtil.inheritDefaultAcls(acls, omPathInfo.getAcls())) {
+        OzoneAclUtil.toDefaultScope(acls);
+      }
+    }
+
+    // Inherit DEFAULT acls from bucket only if DEFAULT acls for
+    // parent-dir are not set.
+    if (acls.isEmpty() && bucketInfo != null) {
+      if (OzoneAclUtil.inheritDefaultAcls(acls, bucketInfo.getAcls())) {
+        OzoneAclUtil.toDefaultScope(acls);
+      }
+    }
+
+    // add itself acls
+    acls.addAll(OzoneAclUtil.fromProtobuf(keyArgs.getAclsList()));
+
+    return acls;
+  }
+
   /**
    * Check Acls for the ozone bucket.
    * @param ozoneManager
@@ -377,11 +436,12 @@ public abstract class OMKeyRequest extends OMClientRequest {
   protected void checkKeyAclsInOpenKeyTable(OzoneManager ozoneManager,
       String volume, String bucket, String key,
       IAccessAuthorizer.ACLType aclType, long clientId) throws IOException {
+
     String keyNameForAclCheck = key;
     // Native authorizer requires client id as part of key name to check
     // write ACL on key. Add client id to key name if ozone native
     // authorizer is configured.
-    if (ozoneManager.isNativeAuthorizerEnabled()) {
+    if (ozoneManager.getAccessAuthorizer().isNative()) {
       keyNameForAclCheck = key + "/" + clientId;
     }
 
@@ -416,8 +476,9 @@ public abstract class OMKeyRequest extends OMClientRequest {
     OmBucketInfo bucketInfo = null;
     if (ozoneManager.getKmsProvider() != null) {
       try {
-        acquireLock = omMetadataManager.getLock().acquireReadLock(
-            BUCKET_LOCK, volumeName, bucketName);
+        mergeOmLockDetails(omMetadataManager.getLock().acquireReadLock(
+            BUCKET_LOCK, volumeName, bucketName));
+        acquireLock = getOmLockDetails().isLockAcquired();
 
         bucketInfo = omMetadataManager.getBucketTable().get(
             omMetadataManager.getBucketKey(volumeName,
@@ -436,8 +497,8 @@ public abstract class OMKeyRequest extends OMClientRequest {
 
       } finally {
         if (acquireLock) {
-          omMetadataManager.getLock().releaseReadLock(
-              BUCKET_LOCK, volumeName, bucketName);
+          mergeOmLockDetails(omMetadataManager.getLock().releaseReadLock(
+              BUCKET_LOCK, volumeName, bucketName));
         }
       }
 
@@ -472,8 +533,9 @@ public abstract class OMKeyRequest extends OMClientRequest {
     OMMetadataManager omMetadataManager = ozoneManager.getMetadataManager();
 
     if (ozoneManager.getKmsProvider() != null) {
-      acquireLock = omMetadataManager.getLock().acquireReadLock(
-          BUCKET_LOCK, volumeName, bucketName);
+      mergeOmLockDetails(omMetadataManager.getLock().acquireReadLock(
+          BUCKET_LOCK, volumeName, bucketName));
+      acquireLock = getOmLockDetails().isLockAcquired();
       try {
         ResolvedBucket resolvedBucket = ozoneManager.resolveBucketLink(
             Pair.of(keyArgs.getVolumeName(), keyArgs.getBucketName()));
@@ -495,8 +557,8 @@ public abstract class OMKeyRequest extends OMClientRequest {
         }
       } finally {
         if (acquireLock) {
-          omMetadataManager.getLock()
-              .releaseReadLock(BUCKET_LOCK, volumeName, bucketName);
+          mergeOmLockDetails(omMetadataManager.getLock()
+              .releaseReadLock(BUCKET_LOCK, volumeName, bucketName));
         }
       }
     }
@@ -517,18 +579,20 @@ public abstract class OMKeyRequest extends OMClientRequest {
 
   /**
    * Check bucket quota in bytes.
+   * @paran metadataManager
    * @param omBucketInfo
    * @param allocateSize
    * @throws IOException
    */
-  protected void checkBucketQuotaInBytes(OmBucketInfo omBucketInfo,
+  protected void checkBucketQuotaInBytes(
+      OMMetadataManager metadataManager, OmBucketInfo omBucketInfo,
       long allocateSize) throws IOException {
     if (omBucketInfo.getQuotaInBytes() > OzoneConsts.QUOTA_RESET) {
       long usedBytes = omBucketInfo.getUsedBytes();
       long quotaInBytes = omBucketInfo.getQuotaInBytes();
       if (quotaInBytes - usedBytes < allocateSize) {
         throw new OMException("The DiskSpace quota of bucket:"
-            + omBucketInfo.getBucketName() + "exceeded: quotaInBytes: "
+            + omBucketInfo.getBucketName() + " exceeded quotaInBytes: "
             + quotaInBytes + " Bytes but diskspace consumed: " + (usedBytes
             + allocateSize) + " Bytes.",
             OMException.ResultCodes.QUOTA_EXCEEDED);
@@ -616,12 +680,13 @@ public abstract class OMKeyRequest extends OMClientRequest {
           @Nullable FileEncryptionInfo encInfo,
           @Nonnull PrefixManager prefixManager,
           @Nullable OmBucketInfo omBucketInfo,
+          OMFileRequest.OMPathInfo omPathInfo,
           long transactionLogIndex, long objectID, boolean isRatisEnabled,
           ReplicationConfig replicationConfig)
           throws IOException {
 
     return prepareFileInfo(omMetadataManager, keyArgs, dbKeyInfo, size,
-            locations, encInfo, prefixManager, omBucketInfo, null,
+            locations, encInfo, prefixManager, omBucketInfo, omPathInfo,
             transactionLogIndex, objectID, isRatisEnabled, replicationConfig);
   }
 
@@ -638,7 +703,7 @@ public abstract class OMKeyRequest extends OMClientRequest {
           @Nullable FileEncryptionInfo encInfo,
           @Nonnull PrefixManager prefixManager,
           @Nullable OmBucketInfo omBucketInfo,
-          OMFileRequest.OMPathInfoWithFSO omPathInfo,
+          OMFileRequest.OMPathInfo omPathInfo,
           long transactionLogIndex, long objectID,
           boolean isRatisEnabled, ReplicationConfig replicationConfig)
           throws IOException {
@@ -679,14 +744,15 @@ public abstract class OMKeyRequest extends OMClientRequest {
    * @return OmKeyInfo
    */
   @SuppressWarnings("parameterNumber")
-  protected OmKeyInfo createFileInfo(@Nonnull KeyArgs keyArgs,
+  protected OmKeyInfo createFileInfo(
+      @Nonnull KeyArgs keyArgs,
       @Nonnull List<OmKeyLocationInfo> locations,
       @Nonnull ReplicationConfig replicationConfig,
       long size,
       @Nullable FileEncryptionInfo encInfo,
       @Nonnull PrefixManager prefixManager,
       @Nullable OmBucketInfo omBucketInfo,
-      OMFileRequest.OMPathInfoWithFSO omPathInfo,
+      OMFileRequest.OMPathInfo omPathInfo,
       long transactionLogIndex, long objectID) {
     OmKeyInfo.Builder builder = new OmKeyInfo.Builder();
     builder.setVolumeName(keyArgs.getVolumeName())
@@ -699,15 +765,19 @@ public abstract class OMKeyRequest extends OMClientRequest {
             .setDataSize(size)
             .setReplicationConfig(replicationConfig)
             .setFileEncryptionInfo(encInfo)
-            .setAcls(getAclsForKey(keyArgs, omBucketInfo, prefixManager))
+            .setAcls(getAclsForKey(
+                keyArgs, omBucketInfo, omPathInfo, prefixManager))
             .addAllMetadata(KeyValueUtil.getFromProtobuf(
                     keyArgs.getMetadataList()))
-            .setUpdateID(transactionLogIndex);
-    if (omPathInfo != null) {
+            .setUpdateID(transactionLogIndex)
+            .setFile(true);
+    if (omPathInfo instanceof OMFileRequest.OMPathInfoWithFSO) {
       // FileTable metadata format
-      objectID = omPathInfo.getLeafNodeObjectId();
-      builder.setParentObjectID(omPathInfo.getLastKnownParentId());
-      builder.setFileName(omPathInfo.getLeafNodeName());
+      OMFileRequest.OMPathInfoWithFSO omPathInfoFSO
+          = (OMFileRequest.OMPathInfoWithFSO) omPathInfo;
+      objectID = omPathInfoFSO.getLeafNodeObjectId();
+      builder.setParentObjectID(omPathInfoFSO.getLastKnownParentId());
+      builder.setFileName(omPathInfoFSO.getLeafNodeName());
     }
     builder.setObjectID(objectID);
     return builder.build();
@@ -726,7 +796,7 @@ public abstract class OMKeyRequest extends OMClientRequest {
           @Nonnull List<OmKeyLocationInfo> locations,
           FileEncryptionInfo encInfo,  @Nonnull PrefixManager prefixManager,
           @Nullable OmBucketInfo omBucketInfo,
-          OMFileRequest.OMPathInfoWithFSO omPathInfo,
+          OMFileRequest.OMPathInfo omPathInfo,
           @Nonnull long transactionLogIndex, long objectID)
           throws IOException {
 
@@ -739,15 +809,17 @@ public abstract class OMKeyRequest extends OMClientRequest {
     String uploadID = args.getMultipartUploadID();
     Preconditions.checkNotNull(uploadID);
     String multipartKey = "";
-    if (omPathInfo != null) {
+    if (omPathInfo instanceof OMFileRequest.OMPathInfoWithFSO) {
+      OMFileRequest.OMPathInfoWithFSO omPathInfoFSO
+          = (OMFileRequest.OMPathInfoWithFSO) omPathInfo;
       final long volumeId = omMetadataManager.getVolumeId(
               args.getVolumeName());
       final long bucketId = omMetadataManager.getBucketId(
               args.getVolumeName(), args.getBucketName());
       // FileTable metadata format
       multipartKey = omMetadataManager.getMultipartKey(volumeId, bucketId,
-              omPathInfo.getLastKnownParentId(),
-              omPathInfo.getLeafNodeName(), uploadID);
+          omPathInfoFSO.getLastKnownParentId(),
+          omPathInfoFSO.getLeafNodeName(), uploadID);
     } else {
       multipartKey = omMetadataManager
               .getMultipartKey(args.getVolumeName(), args.getBucketName(),
@@ -790,25 +862,16 @@ public abstract class OMKeyRequest extends OMClientRequest {
   /**
    * Prepare key for deletion service on overwrite.
    *
-   * @param dbOzoneKey key to point to an object in RocksDB
    * @param keyToDelete OmKeyInfo of a key to be in deleteTable
-   * @param omMetadataManager
    * @param trxnLogIndex
    * @param isRatisEnabled
    * @return Old keys eligible for deletion.
    * @throws IOException
    */
   protected RepeatedOmKeyInfo getOldVersionsToCleanUp(
-      @Nonnull String dbOzoneKey, @Nonnull OmKeyInfo keyToDelete,
-      OMMetadataManager omMetadataManager, long trxnLogIndex,
+      @Nonnull OmKeyInfo keyToDelete, long trxnLogIndex,
       boolean isRatisEnabled) throws IOException {
-
-    // Past keys that was deleted but still in deleted table,
-    // waiting for deletion service.
-    RepeatedOmKeyInfo keysToDelete =
-        omMetadataManager.getDeletedTable().get(dbOzoneKey);
-
-    return OmUtils.prepareKeyForDelete(keyToDelete, keysToDelete,
+    return OmUtils.prepareKeyForDelete(keyToDelete,
           trxnLogIndex, isRatisEnabled);
   }
 
@@ -829,9 +892,12 @@ public abstract class OMKeyRequest extends OMClientRequest {
     if (uncommitted.isEmpty()) {
       return null;
     }
-    LOG.info("Detect allocated but uncommitted blocks {} in key {}.",
+    LOG.debug("Detect allocated but uncommitted blocks {} in key {}.",
         uncommitted, omKeyInfo.getKeyName());
     OmKeyInfo pseudoKeyInfo = omKeyInfo.copyObject();
+    // This is a special marker to indicate that SnapshotDeletingService
+    // can reclaim this key's blocks unconditionally.
+    pseudoKeyInfo.setObjectID(OBJECT_ID_RECLAIM_BLOCKS);
     // TODO dataSize of pseudoKey is not real here
     List<OmKeyLocationInfoGroup> uncommittedGroups = new ArrayList<>();
     // version not matters in the current logic of keyDeletingService,
@@ -840,4 +906,109 @@ public abstract class OMKeyRequest extends OMClientRequest {
     pseudoKeyInfo.setKeyLocationVersions(uncommittedGroups);
     return pseudoKeyInfo;
   }
+
+  /**
+   * Remove blocks in-place from keysToBeFiltered that exist in referenceKey.
+   * <p>
+   * keysToBeFiltered.getOmKeyInfoList() becomes an empty list when all blocks
+   * are filtered out.
+   *
+   * @param referenceKey OmKeyInfo
+   * @param keysToBeFiltered RepeatedOmKeyInfo
+   */
+  protected void filterOutBlocksStillInUse(OmKeyInfo referenceKey,
+                                           RepeatedOmKeyInfo keysToBeFiltered) {
+
+    LOG.debug("Before block filtering, keysToBeFiltered = {}",
+        keysToBeFiltered);
+
+    // A HashSet for fast lookup. Gathers all ContainerBlockID entries inside
+    // the referenceKey.
+    HashSet<ContainerBlockID> cbIdSet = referenceKey.getKeyLocationVersions()
+        .stream()
+        .flatMap(e -> e.getLocationList().stream())
+        .map(omKeyLocationInfo ->
+            omKeyLocationInfo.getBlockID().getContainerBlockID())
+        .collect(Collectors.toCollection(HashSet::new));
+
+    // Pardon the nested loops. ContainerBlockID is 9-layer deep from:
+    // keysToBeFiltered               // Layer 0. RepeatedOmKeyInfo
+    //     .getOmKeyInfoList()        // 1. List<OmKeyInfo>
+    //     .get(0)                    // 2. OmKeyInfo
+    //     .getKeyLocationVersions()  // 3. List<OmKeyLocationInfoGroup>
+    //     .get(0)                    // 4. OmKeyLocationInfoGroup
+    //     .getLocationVersionMap()   // 5. Map<Long, List<OmKeyLocationInfo>>
+    //     .get(version)              // 6. List<OmKeyLocationInfo>
+    //     .get(0)                    // 7. OmKeyLocationInfo
+    //     .getBlockID()              // 8. BlockID
+    //     .getContainerBlockID();    // 9. ContainerBlockID
+
+    // Using iterator instead of `for` or `forEach` for in-place entry removal
+
+    // Layer 1: List<OmKeyInfo>
+    Iterator<OmKeyInfo> iterOmKeyInfo = keysToBeFiltered
+        .getOmKeyInfoList().iterator();
+
+    while (iterOmKeyInfo.hasNext()) {
+      // Note with HDDS-8462, each RepeatedOmKeyInfo should have only one entry,
+      // so this outer most loop should never be entered twice in each call.
+
+      // But for completeness sake I shall put it here.
+      // Remove only when RepeatedOmKeyInfo is no longer used.
+
+      // Layer 2: OmKeyInfo
+      OmKeyInfo oldOmKeyInfo = iterOmKeyInfo.next();
+      // Layer 3: List<OmKeyLocationInfoGroup>
+      Iterator<OmKeyLocationInfoGroup> iterKeyLocInfoGroup = oldOmKeyInfo
+          .getKeyLocationVersions().iterator();
+      while (iterKeyLocInfoGroup.hasNext()) {
+        // Layer 4: OmKeyLocationInfoGroup
+        OmKeyLocationInfoGroup keyLocInfoGroup = iterKeyLocInfoGroup.next();
+        // Layer 5: Map<Long, List<OmKeyLocationInfo>>
+        Iterator<Map.Entry<Long, List<OmKeyLocationInfo>>> iterVerMap =
+            keyLocInfoGroup.getLocationVersionMap().entrySet().iterator();
+
+        while (iterVerMap.hasNext()) {
+          Map.Entry<Long, List<OmKeyLocationInfo>> mapEntry = iterVerMap.next();
+          // Layer 6: List<OmKeyLocationInfo>
+          List<OmKeyLocationInfo> omKeyLocationInfoList = mapEntry.getValue();
+
+          Iterator<OmKeyLocationInfo> iterKeyLocInfo =
+              omKeyLocationInfoList.iterator();
+          while (iterKeyLocInfo.hasNext()) {
+            // Layer 7: OmKeyLocationInfo
+            OmKeyLocationInfo keyLocationInfo = iterKeyLocInfo.next();
+            // Layer 8: BlockID. Then Layer 9: ContainerBlockID
+            ContainerBlockID cbId = keyLocationInfo
+                .getBlockID().getContainerBlockID();
+
+            if (cbIdSet.contains(cbId)) {
+              // Remove this block from oldVerKeyInfo because it is referenced.
+              iterKeyLocInfo.remove();
+              LOG.debug("Filtered out block: {}", cbId);
+            }
+          }
+
+          // Cleanup when Layer 6 is an empty list
+          if (omKeyLocationInfoList.isEmpty()) {
+            iterVerMap.remove();
+          }
+        }
+
+        // Cleanup when Layer 5 is an empty map
+        if (keyLocInfoGroup.getLocationVersionMap().isEmpty()) {
+          iterKeyLocInfoGroup.remove();
+        }
+      }
+
+      // Cleanup when Layer 3 is an empty list
+      if (oldOmKeyInfo.getKeyLocationVersions().isEmpty()) {
+        iterOmKeyInfo.remove();
+      }
+    }
+
+    // Intentional extra space for alignment
+    LOG.debug("After block filtering,  keysToBeFiltered = {}",
+        keysToBeFiltered);
+  }
 }
