@@ -22,6 +22,7 @@ import org.apache.hadoop.hdds.utils.db.BatchOperation;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
 import org.apache.hadoop.ozone.om.helpers.OmRenameKeys;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
+import org.apache.hadoop.ozone.om.request.OMClientRequestUtils;
 import org.apache.hadoop.ozone.om.response.CleanupTableInfo;
 import org.apache.hadoop.ozone.om.response.OMClientResponse;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMResponse;
@@ -31,13 +32,14 @@ import java.io.IOException;
 import java.util.Map;
 
 import static org.apache.hadoop.ozone.om.OmMetadataManagerImpl.KEY_TABLE;
+import static org.apache.hadoop.ozone.om.OmMetadataManagerImpl.SNAPSHOT_RENAMED_TABLE;
 import static org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status.OK;
 import static org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status.PARTIAL_RENAME;
 
 /**
  * Response for RenameKeys request.
  */
-@CleanupTableInfo(cleanupTables = {KEY_TABLE})
+@CleanupTableInfo(cleanupTables = {KEY_TABLE, SNAPSHOT_RENAMED_TABLE})
 public class OMKeysRenameResponse extends OMClientResponse {
 
   private OmRenameKeys omRenameKeys;
@@ -77,14 +79,29 @@ public class OMKeysRenameResponse extends OMClientResponse {
       String fromKeyName = entry.getKey();
       OmKeyInfo newKeyInfo = entry.getValue();
       String toKeyName = newKeyInfo.getKeyName();
+      String fromDbKey = omMetadataManager
+          .getOzoneKey(volumeName, bucketName, fromKeyName);
 
       omMetadataManager.getKeyTable(getBucketLayout())
-          .deleteWithBatch(batchOperation, omMetadataManager
-              .getOzoneKey(volumeName, bucketName, fromKeyName));
+          .deleteWithBatch(batchOperation, fromDbKey);
       omMetadataManager.getKeyTable(getBucketLayout())
           .putWithBatch(batchOperation,
               omMetadataManager.getOzoneKey(volumeName, bucketName, toKeyName),
               newKeyInfo);
+
+
+      boolean isSnapshotBucket = OMClientRequestUtils.
+          isSnapshotBucket(omMetadataManager, newKeyInfo);
+      String renameDbKey = omMetadataManager.getRenameKey(
+          newKeyInfo.getVolumeName(), newKeyInfo.getBucketName(),
+          newKeyInfo.getObjectID());
+
+      String renamedKey = omMetadataManager.getSnapshotRenamedTable()
+          .get(renameDbKey);
+      if (isSnapshotBucket && renamedKey == null) {
+        omMetadataManager.getSnapshotRenamedTable().putWithBatch(
+            batchOperation, renameDbKey, fromDbKey);
+      }
     }
   }
 
