@@ -32,6 +32,7 @@ import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos
     .OMResponse;
 import org.apache.hadoop.hdds.utils.db.BatchOperation;
 
+import javax.annotation.CheckForNull;
 import javax.annotation.Nonnull;
 
 import static org.apache.hadoop.ozone.om.OmMetadataManagerImpl.BUCKET_TABLE;
@@ -54,9 +55,8 @@ public class S3MultipartUploadCompleteResponse extends OmKeyResponse {
   private String multipartKey;
   private String multipartOpenKey;
   private OmKeyInfo omKeyInfo;
-  private List<OmKeyInfo> partsUnusedList;
+  private List<OmKeyInfo> allKeyInfoToRemove;
   private OmBucketInfo omBucketInfo;
-  private RepeatedOmKeyInfo keyVersionsToDelete;
 
   @SuppressWarnings("checkstyle:ParameterNumber")
   public S3MultipartUploadCompleteResponse(
@@ -64,17 +64,15 @@ public class S3MultipartUploadCompleteResponse extends OmKeyResponse {
       @Nonnull String multipartKey,
       @Nonnull String multipartOpenKey,
       @Nonnull OmKeyInfo omKeyInfo,
-      @Nonnull List<OmKeyInfo> unUsedParts,
+      @Nonnull List<OmKeyInfo> allKeyInfoToRemove,
       @Nonnull BucketLayout bucketLayout,
-      @Nonnull OmBucketInfo omBucketInfo,
-      RepeatedOmKeyInfo keyVersionsToDelete) {
+      @CheckForNull OmBucketInfo omBucketInfo) {
     super(omResponse, bucketLayout);
-    this.partsUnusedList = unUsedParts;
+    this.allKeyInfoToRemove = allKeyInfoToRemove;
     this.multipartKey = multipartKey;
     this.multipartOpenKey = multipartOpenKey;
     this.omKeyInfo = omKeyInfo;
     this.omBucketInfo = omBucketInfo;
-    this.keyVersionsToDelete = keyVersionsToDelete;
   }
 
   /**
@@ -98,23 +96,18 @@ public class S3MultipartUploadCompleteResponse extends OmKeyResponse {
         multipartKey);
 
     // 2. Add key to KeyTable
-    String ozoneKey = addToKeyTable(omMetadataManager, batchOperation);
+    addToKeyTable(omMetadataManager, batchOperation);
 
     // 3. Delete unused parts
-    if (!partsUnusedList.isEmpty()) {
+    if (!allKeyInfoToRemove.isEmpty()) {
       // Add unused parts to deleted key table.
-      if (keyVersionsToDelete == null) {
-        keyVersionsToDelete = new RepeatedOmKeyInfo(partsUnusedList);
-      } else {
-        for (OmKeyInfo unusedParts : partsUnusedList) {
-          keyVersionsToDelete.addOmKeyInfo(unusedParts);
-        }
+      for (OmKeyInfo keyInfoToRemove : allKeyInfoToRemove) {
+        String deleteKey = omMetadataManager.getOzoneDeletePathKey(
+            keyInfoToRemove.getObjectID(), multipartKey);
+        omMetadataManager.getDeletedTable().putWithBatch(batchOperation,
+            deleteKey, new RepeatedOmKeyInfo(keyInfoToRemove));
       }
     }
-    if (keyVersionsToDelete != null) {
-      omMetadataManager.getDeletedTable().putWithBatch(batchOperation,
-          ozoneKey, keyVersionsToDelete);
-    }
 
     // update bucket usedBytes, only when total bucket size has changed
     // due to unused parts cleanup or an overwritten version.
@@ -135,19 +128,7 @@ public class S3MultipartUploadCompleteResponse extends OmKeyResponse {
     return ozoneKey;
   }
 
-  protected String getMultipartKey() {
-    return multipartKey;
-  }
-
   protected OmKeyInfo getOmKeyInfo() {
     return omKeyInfo;
   }
-
-  protected List<OmKeyInfo> getPartsUnusedList() {
-    return partsUnusedList;
-  }
-
-  public OmBucketInfo getOmBucketInfo() {
-    return omBucketInfo;
-  }
 }
