@@ -16,42 +16,64 @@
  */
 
 package org.apache.hadoop.ozone.om;
-import com.google.common.base.Optional;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.StorageType;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.utils.db.cache.CacheKey;
 import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
+import org.apache.hadoop.ozone.om.exceptions.OMException;
+import org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
+import org.apache.hadoop.ozone.om.helpers.OmMultipartKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
 import org.apache.hadoop.hdds.utils.TransactionInfo;
 import org.apache.hadoop.ozone.om.helpers.OzoneFSUtils;
+import org.apache.hadoop.ozone.om.helpers.SnapshotInfo;
 import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
+import org.apache.hadoop.ozone.om.request.util.OMMultipartUploadUtils;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.ExpiredMultipartUploadInfo;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.ExpiredMultipartUploadsBucket;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OpenKey;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OpenKeyBucket;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.TemporaryFolder;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.PartKeyInfo;
+import org.apache.hadoop.util.Time;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.io.TempDir;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.Arguments;
+import org.junit.jupiter.params.provider.MethodSource;
 
+import java.io.File;
 import java.time.Duration;
+import java.time.Instant;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
+import java.util.TreeMap;
 import java.util.TreeSet;
-import java.time.Instant;
 import java.util.UUID;
 import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
+import java.util.stream.Stream;
 
+import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_MPU_EXPIRE_THRESHOLD;
+import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_MPU_EXPIRE_THRESHOLD_DEFAULT;
 import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_OPEN_KEY_EXPIRE_THRESHOLD;
 import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_OPEN_KEY_EXPIRE_THRESHOLD_DEFAULT;
 import static org.apache.hadoop.ozone.OzoneConsts.TRANSACTION_INFO_KEY;
 import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_DB_DIRS;
+import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.BUCKET_NOT_FOUND;
+import static org.apache.hadoop.ozone.om.exceptions.OMException.ResultCodes.VOLUME_NOT_FOUND;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.params.provider.Arguments.arguments;
 
 /**
  * Tests OzoneManager MetadataManager.
@@ -60,17 +82,16 @@ public class TestOmMetadataManager {
 
   private OMMetadataManager omMetadataManager;
   private OzoneConfiguration ozoneConfiguration;
-
-  @Rule
-  public TemporaryFolder folder = new TemporaryFolder();
+  @TempDir
+  private File folder;
 
 
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
     ozoneConfiguration = new OzoneConfiguration();
     ozoneConfiguration.set(OZONE_OM_DB_DIRS,
-        folder.getRoot().getAbsolutePath());
-    omMetadataManager = new OmMetadataManagerImpl(ozoneConfiguration);
+        folder.getAbsolutePath());
+    omMetadataManager = new OmMetadataManagerImpl(ozoneConfiguration, null);
   }
 
   @Test
@@ -90,8 +111,8 @@ public class TestOmMetadataManager {
     TransactionInfo transactionInfo =
         omMetadataManager.getTransactionInfoTable().get(TRANSACTION_INFO_KEY);
 
-    Assert.assertEquals(3, transactionInfo.getTerm());
-    Assert.assertEquals(250, transactionInfo.getTransactionIndex());
+    assertEquals(3, transactionInfo.getTerm());
+    assertEquals(250, transactionInfo.getTransactionIndex());
 
 
   }
@@ -125,7 +146,7 @@ public class TestOmMetadataManager {
     String startVolume = "vol" + startOrder;
     List<OmVolumeArgs> volumeList = omMetadataManager.listVolumes(ownerName,
         prefix, startVolume, 100);
-    Assert.assertEquals(volumeList.size(), totalVol - startOrder - 1);
+    assertEquals(volumeList.size(), totalVol - startOrder - 1);
   }
 
   @Test
@@ -157,13 +178,13 @@ public class TestOmMetadataManager {
     // Test list all volumes
     List<OmVolumeArgs> volListA = omMetadataManager.listVolumes(null,
         prefix, startKey, 1000);
-    Assert.assertEquals(volListA.size(), 100);
+    assertEquals(volListA.size(), 100);
 
     // Test list all volumes with prefix
     prefix = "volb";
     List<OmVolumeArgs> volListB = omMetadataManager.listVolumes(null,
         prefix, startKey, 1000);
-    Assert.assertEquals(volListB.size(), 50);
+    assertEquals(volListB.size(), 50);
 
     // Test list all volumes with setting startVolume
     // that was not part of result.
@@ -173,7 +194,7 @@ public class TestOmMetadataManager {
     startKey = "volb" + startOrder;
     List<OmVolumeArgs> volListC = omMetadataManager.listVolumes(null,
         prefix, startKey, 1000);
-    Assert.assertEquals(volListC.size(), totalVol - startOrder - 1);
+    assertEquals(volListC.size(), totalVol - startOrder - 1);
   }
 
   @Test
@@ -227,15 +248,15 @@ public class TestOmMetadataManager {
     // List all buckets which have prefix ozoneBucket
     List<OmBucketInfo> omBucketInfoList =
         omMetadataManager.listBuckets(volumeName1,
-            null, prefixBucketNameWithOzoneOwner, 100);
+            null, prefixBucketNameWithOzoneOwner, 100, false);
 
     // Cause adding a exact name in prefixBucketNameWithOzoneOwner
     // and another 49 buckets, so if we list buckets with --prefix
     // prefixBucketNameWithOzoneOwner, we should get 50 buckets.
-    Assert.assertEquals(omBucketInfoList.size(), 50);
+    assertEquals(omBucketInfoList.size(), 50);
 
     for (OmBucketInfo omBucketInfo : omBucketInfoList) {
-      Assert.assertTrue(omBucketInfo.getBucketName().startsWith(
+      assertTrue(omBucketInfo.getBucketName().startsWith(
           prefixBucketNameWithOzoneOwner));
     }
 
@@ -244,39 +265,39 @@ public class TestOmMetadataManager {
     omBucketInfoList =
         omMetadataManager.listBuckets(volumeName1,
             startBucket, prefixBucketNameWithOzoneOwner,
-            100);
+            100, false);
 
-    Assert.assertEquals(volumeABucketsPrefixWithOzoneOwner.tailSet(
+    assertEquals(volumeABucketsPrefixWithOzoneOwner.tailSet(
         startBucket).size() - 1, omBucketInfoList.size());
 
     startBucket = prefixBucketNameWithOzoneOwner + 38;
     omBucketInfoList =
         omMetadataManager.listBuckets(volumeName1,
             startBucket, prefixBucketNameWithOzoneOwner,
-            100);
+            100, false);
 
-    Assert.assertEquals(volumeABucketsPrefixWithOzoneOwner.tailSet(
+    assertEquals(volumeABucketsPrefixWithOzoneOwner.tailSet(
         startBucket).size() - 1, omBucketInfoList.size());
 
     for (OmBucketInfo omBucketInfo : omBucketInfoList) {
-      Assert.assertTrue(omBucketInfo.getBucketName().startsWith(
+      assertTrue(omBucketInfo.getBucketName().startsWith(
           prefixBucketNameWithOzoneOwner));
-      Assert.assertFalse(omBucketInfo.getBucketName().equals(
+      assertFalse(omBucketInfo.getBucketName().equals(
           prefixBucketNameWithOzoneOwner + 10));
     }
 
 
 
     omBucketInfoList = omMetadataManager.listBuckets(volumeName2,
-        null, prefixBucketNameWithHadoopOwner, 100);
+        null, prefixBucketNameWithHadoopOwner, 100, false);
 
     // Cause adding a exact name in prefixBucketNameWithOzoneOwner
     // and another 49 buckets, so if we list buckets with --prefix
     // prefixBucketNameWithOzoneOwner, we should get 50 buckets.
-    Assert.assertEquals(omBucketInfoList.size(), 50);
+    assertEquals(omBucketInfoList.size(), 50);
 
     for (OmBucketInfo omBucketInfo : omBucketInfoList) {
-      Assert.assertTrue(omBucketInfo.getBucketName().startsWith(
+      assertTrue(omBucketInfo.getBucketName().startsWith(
           prefixBucketNameWithHadoopOwner));
     }
 
@@ -287,30 +308,29 @@ public class TestOmMetadataManager {
     for (int i = 0; i < 5; i++) {
 
       omBucketInfoList = omMetadataManager.listBuckets(volumeName2,
-          startBucket, prefixBucketNameWithHadoopOwner, 10);
+          startBucket, prefixBucketNameWithHadoopOwner, 10, false);
 
-      Assert.assertEquals(omBucketInfoList.size(), 10);
+      assertEquals(omBucketInfoList.size(), 10);
 
       for (OmBucketInfo omBucketInfo : omBucketInfoList) {
         expectedBuckets.add(omBucketInfo.getBucketName());
-        Assert.assertTrue(omBucketInfo.getBucketName().startsWith(
+        assertTrue(omBucketInfo.getBucketName().startsWith(
             prefixBucketNameWithHadoopOwner));
         startBucket =  omBucketInfo.getBucketName();
       }
     }
 
 
-    Assert.assertEquals(volumeBBucketsPrefixWithHadoopOwner, expectedBuckets);
+    assertEquals(volumeBBucketsPrefixWithHadoopOwner, expectedBuckets);
     // As now we have iterated all 50 buckets, calling next time should
     // return empty list.
     omBucketInfoList = omMetadataManager.listBuckets(volumeName2,
-        startBucket, prefixBucketNameWithHadoopOwner, 10);
+        startBucket, prefixBucketNameWithHadoopOwner, 10, false);
 
-    Assert.assertEquals(omBucketInfoList.size(), 0);
+    assertEquals(omBucketInfoList.size(), 0);
 
   }
 
-
   private void addBucketsToCache(String volumeName, String bucketName) {
 
     OmBucketInfo omBucketInfo = OmBucketInfo.newBuilder()
@@ -322,7 +342,7 @@ public class TestOmMetadataManager {
 
     omMetadataManager.getBucketTable().addCacheEntry(
         new CacheKey<>(omMetadataManager.getBucketKey(volumeName, bucketName)),
-        new CacheValue<>(Optional.of(omBucketInfo), 1));
+        CacheValue.get(1, omBucketInfo));
   }
 
   @Test
@@ -377,12 +397,12 @@ public class TestOmMetadataManager {
     // List all keys which have prefix "key-a"
     List<OmKeyInfo> omKeyInfoList =
         omMetadataManager.listKeys(volumeNameA, ozoneBucket,
-            null, prefixKeyA, 100);
+            null, prefixKeyA, 100).getKeys();
 
-    Assert.assertEquals(omKeyInfoList.size(),  50);
+    assertEquals(omKeyInfoList.size(),  50);
 
     for (OmKeyInfo omKeyInfo : omKeyInfoList) {
-      Assert.assertTrue(omKeyInfo.getKeyName().startsWith(
+      assertTrue(omKeyInfo.getKeyName().startsWith(
           prefixKeyA));
     }
 
@@ -390,35 +410,35 @@ public class TestOmMetadataManager {
     String startKey = prefixKeyA + 10;
     omKeyInfoList =
         omMetadataManager.listKeys(volumeNameA, ozoneBucket,
-            startKey, prefixKeyA, 100);
+            startKey, prefixKeyA, 100).getKeys();
 
-    Assert.assertEquals(keysASet.tailSet(
+    assertEquals(keysASet.tailSet(
         startKey).size() - 1, omKeyInfoList.size());
 
     startKey = prefixKeyA + 38;
     omKeyInfoList =
         omMetadataManager.listKeys(volumeNameA, ozoneBucket,
-            startKey, prefixKeyA, 100);
+            startKey, prefixKeyA, 100).getKeys();
 
-    Assert.assertEquals(keysASet.tailSet(
+    assertEquals(keysASet.tailSet(
         startKey).size() - 1, omKeyInfoList.size());
 
     for (OmKeyInfo omKeyInfo : omKeyInfoList) {
-      Assert.assertTrue(omKeyInfo.getKeyName().startsWith(
+      assertTrue(omKeyInfo.getKeyName().startsWith(
           prefixKeyA));
-      Assert.assertFalse(omKeyInfo.getBucketName().equals(
+      assertFalse(omKeyInfo.getBucketName().equals(
           prefixKeyA + 38));
     }
 
 
 
     omKeyInfoList = omMetadataManager.listKeys(volumeNameB, hadoopBucket,
-        null, prefixKeyB, 100);
+        null, prefixKeyB, 100).getKeys();
 
-    Assert.assertEquals(omKeyInfoList.size(),  50);
+    assertEquals(omKeyInfoList.size(),  50);
 
     for (OmKeyInfo omKeyInfo : omKeyInfoList) {
-      Assert.assertTrue(omKeyInfo.getKeyName().startsWith(
+      assertTrue(omKeyInfo.getKeyName().startsWith(
           prefixKeyB));
     }
 
@@ -429,34 +449,34 @@ public class TestOmMetadataManager {
     for (int i = 0; i < 5; i++) {
 
       omKeyInfoList = omMetadataManager.listKeys(volumeNameB, hadoopBucket,
-          startKey, prefixKeyB, 10);
+          startKey, prefixKeyB, 10).getKeys();
 
-      Assert.assertEquals(10, omKeyInfoList.size());
+      assertEquals(10, omKeyInfoList.size());
 
       for (OmKeyInfo omKeyInfo : omKeyInfoList) {
         expectedKeys.add(omKeyInfo.getKeyName());
-        Assert.assertTrue(omKeyInfo.getKeyName().startsWith(
+        assertTrue(omKeyInfo.getKeyName().startsWith(
             prefixKeyB));
         startKey =  omKeyInfo.getKeyName();
       }
     }
 
-    Assert.assertEquals(expectedKeys, keysBVolumeBSet);
+    assertEquals(expectedKeys, keysBVolumeBSet);
 
 
     // As now we have iterated all 50 buckets, calling next time should
     // return empty list.
     omKeyInfoList = omMetadataManager.listKeys(volumeNameB, hadoopBucket,
-        startKey, prefixKeyB, 10);
+        startKey, prefixKeyB, 10).getKeys();
 
-    Assert.assertEquals(omKeyInfoList.size(), 0);
+    assertEquals(omKeyInfoList.size(), 0);
 
     // List all keys with empty prefix
     omKeyInfoList = omMetadataManager.listKeys(volumeNameA, ozoneBucket,
-        null, null, 100);
-    Assert.assertEquals(50, omKeyInfoList.size());
+        null, null, 100).getKeys();
+    assertEquals(50, omKeyInfoList.size());
     for (OmKeyInfo omKeyInfo : omKeyInfoList) {
-      Assert.assertTrue(omKeyInfo.getKeyName().startsWith(
+      assertTrue(omKeyInfo.getKeyName().startsWith(
           prefixKeyA));
     }
   }
@@ -488,7 +508,7 @@ public class TestOmMetadataManager {
         // Mark as deleted in cache.
         omMetadataManager.getKeyTable(getDefaultBucketLayout()).addCacheEntry(
             new CacheKey<>(key),
-            new CacheValue<>(Optional.absent(), 100L));
+            CacheValue.get(100L));
         deleteKeySet.add(key);
       }
     }
@@ -496,19 +516,19 @@ public class TestOmMetadataManager {
     // Now list keys which match with prefixKeyA.
     List<OmKeyInfo> omKeyInfoList =
         omMetadataManager.listKeys(volumeNameA, ozoneBucket,
-            null, prefixKeyA, 100);
+            null, prefixKeyA, 100).getKeys();
 
     // As in total 100, 50 are marked for delete. It should list only 50 keys.
-    Assert.assertEquals(50, omKeyInfoList.size());
+    assertEquals(50, omKeyInfoList.size());
 
     TreeSet<String> expectedKeys = new TreeSet<>();
 
     for (OmKeyInfo omKeyInfo : omKeyInfoList) {
       expectedKeys.add(omKeyInfo.getKeyName());
-      Assert.assertTrue(omKeyInfo.getKeyName().startsWith(prefixKeyA));
+      assertTrue(omKeyInfo.getKeyName().startsWith(prefixKeyA));
     }
 
-    Assert.assertEquals(expectedKeys, keysASet);
+    assertEquals(expectedKeys, keysASet);
 
 
     // Now get key count by 10.
@@ -517,28 +537,28 @@ public class TestOmMetadataManager {
     for (int i = 0; i < 5; i++) {
 
       omKeyInfoList = omMetadataManager.listKeys(volumeNameA, ozoneBucket,
-          startKey, prefixKeyA, 10);
+          startKey, prefixKeyA, 10).getKeys();
 
       System.out.println(i);
-      Assert.assertEquals(10, omKeyInfoList.size());
+      assertEquals(10, omKeyInfoList.size());
 
       for (OmKeyInfo omKeyInfo : omKeyInfoList) {
         expectedKeys.add(omKeyInfo.getKeyName());
-        Assert.assertTrue(omKeyInfo.getKeyName().startsWith(
+        assertTrue(omKeyInfo.getKeyName().startsWith(
             prefixKeyA));
         startKey =  omKeyInfo.getKeyName();
       }
     }
 
-    Assert.assertEquals(keysASet, expectedKeys);
+    assertEquals(keysASet, expectedKeys);
 
 
     // As now we have iterated all 50 buckets, calling next time should
     // return empty list.
     omKeyInfoList = omMetadataManager.listKeys(volumeNameA, ozoneBucket,
-        startKey, prefixKeyA, 10);
+        startKey, prefixKeyA, 10).getKeys();
 
-    Assert.assertEquals(omKeyInfoList.size(), 0);
+    assertEquals(omKeyInfoList.size(), 0);
 
 
 
@@ -553,11 +573,26 @@ public class TestOmMetadataManager {
     testGetExpiredOpenKeys(BucketLayout.DEFAULT);
   }
 
+  @Test
+  public void testGetExpiredOpenKeysExcludeMPUs() throws Exception {
+    testGetExpiredOpenKeysExcludeMPUKeys(BucketLayout.DEFAULT);
+  }
+
   @Test
   public void testGetExpiredOpenKeysFSO() throws Exception {
     testGetExpiredOpenKeys(BucketLayout.FILE_SYSTEM_OPTIMIZED);
   }
 
+  @Test
+  public void testGetExpiredOpenKeysExcludeMPUsFSO() throws Exception {
+    testGetExpiredOpenKeysExcludeMPUKeys(BucketLayout.FILE_SYSTEM_OPTIMIZED);
+  }
+
+  @Test
+  public void testGetExpiredMultipartUploads() throws Exception {
+    testGetExpiredMPUs();
+  }
+
   private void testGetExpiredOpenKeys(BucketLayout bucketLayout)
       throws Exception {
     final String bucketName = UUID.randomUUID().toString();
@@ -581,7 +616,7 @@ public class TestOmMetadataManager {
     final Duration expireThreshold = Duration.ofMillis(expireThresholdMillis);
 
     final long expiredOpenKeyCreationTime =
-        Instant.now().minus(expireThreshold).toEpochMilli();
+        expireThreshold.negated().plusMillis(Time.now()).toMillis();
 
     // Add expired keys to open key table.
     // The method under test does not check for expired open keys in the
@@ -589,7 +624,7 @@ public class TestOmMetadataManager {
     Set<String> expiredKeys = new HashSet<>();
     for (int i = 0; i < numExpiredOpenKeys + numUnexpiredOpenKeys; i++) {
       final long creationTime = i < numExpiredOpenKeys ?
-          expiredOpenKeyCreationTime : Instant.now().toEpochMilli();
+          expiredOpenKeyCreationTime : Time.now();
       final OmKeyInfo keyInfo = OMRequestTestUtils.createOmKeyInfo(volumeName,
           bucketName, "expired" + i, HddsProtos.ReplicationType.RATIS,
           HddsProtos.ReplicationFactor.ONE, 0L, creationTime);
@@ -612,38 +647,221 @@ public class TestOmMetadataManager {
     }
 
     // Test retrieving fewer expired keys than actually exist.
-    List<OpenKeyBucket> someExpiredKeys =
+    final Collection<OpenKeyBucket.Builder> someExpiredKeys =
         omMetadataManager.getExpiredOpenKeys(expireThreshold,
-            numExpiredOpenKeys - 1, bucketLayout);
+            numExpiredOpenKeys - 1, bucketLayout).getOpenKeyBuckets();
     List<String> names = getOpenKeyNames(someExpiredKeys);
-    Assert.assertEquals(numExpiredOpenKeys - 1, names.size());
-    Assert.assertTrue(expiredKeys.containsAll(names));
+    assertEquals(numExpiredOpenKeys - 1, names.size());
+    assertTrue(expiredKeys.containsAll(names));
 
     // Test attempting to retrieving more expired keys than actually exist.
-    List<OpenKeyBucket> allExpiredKeys =
+    Collection<OpenKeyBucket.Builder> allExpiredKeys =
         omMetadataManager.getExpiredOpenKeys(expireThreshold,
-            numExpiredOpenKeys + 1, bucketLayout);
+            numExpiredOpenKeys + 1, bucketLayout).getOpenKeyBuckets();
     names = getOpenKeyNames(allExpiredKeys);
-    Assert.assertEquals(numExpiredOpenKeys, names.size());
-    Assert.assertTrue(expiredKeys.containsAll(names));
+    assertEquals(numExpiredOpenKeys, names.size());
+    assertTrue(expiredKeys.containsAll(names));
 
     // Test retrieving exact amount of expired keys that exist.
     allExpiredKeys =
         omMetadataManager.getExpiredOpenKeys(expireThreshold,
-            numExpiredOpenKeys, bucketLayout);
+            numExpiredOpenKeys, bucketLayout).getOpenKeyBuckets();
     names = getOpenKeyNames(allExpiredKeys);
-    Assert.assertEquals(numExpiredOpenKeys, names.size());
-    Assert.assertTrue(expiredKeys.containsAll(names));
+    assertEquals(numExpiredOpenKeys, names.size());
+    assertTrue(expiredKeys.containsAll(names));
   }
 
-  private List<String> getOpenKeyNames(List<OpenKeyBucket> openKeyBuckets) {
+  private void testGetExpiredOpenKeysExcludeMPUKeys(
+      BucketLayout bucketLayout) throws Exception {
+    final String bucketName = UUID.randomUUID().toString();
+    final String volumeName = UUID.randomUUID().toString();
+    // Add volume, bucket, key entries to DB.
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, bucketName,
+        omMetadataManager, BucketLayout.FILE_SYSTEM_OPTIMIZED);
+    final int numExpiredMPUOpenKeys = 4;
+    final long clientID = 1000L;
+    // To create expired keys, they will be assigned a creation time as
+    // old as the minimum expiration time.
+    final long expireThresholdMillis = ozoneConfiguration.getTimeDuration(
+        OZONE_OM_OPEN_KEY_EXPIRE_THRESHOLD,
+        OZONE_OM_OPEN_KEY_EXPIRE_THRESHOLD_DEFAULT,
+        TimeUnit.MILLISECONDS);
+
+    final Duration expireThreshold = Duration.ofMillis(expireThresholdMillis);
+
+    final long expiredOpenKeyCreationTime =
+        expireThreshold.negated().plusMillis(Time.now()).toMillis();
+
+    // Ensure that "expired" MPU-related open keys are not fetched.
+    // MPU-related open keys, identified by isMultipartKey = false
+    for (int i = 0; i < numExpiredMPUOpenKeys; i++) {
+      final OmKeyInfo keyInfo = OMRequestTestUtils.createOmKeyInfo(volumeName,
+          bucketName, "expired" + i,
+          HddsProtos.ReplicationType.RATIS, HddsProtos.ReplicationFactor.ONE,
+          0L, expiredOpenKeyCreationTime, true);
+
+      final String uploadId = OMMultipartUploadUtils.getMultipartUploadId();
+      final OmMultipartKeyInfo multipartKeyInfo = OMRequestTestUtils.
+          createOmMultipartKeyInfo(uploadId, expiredOpenKeyCreationTime,
+              HddsProtos.ReplicationType.RATIS,
+              HddsProtos.ReplicationFactor.ONE, 0L);
+
+      if (bucketLayout.isFileSystemOptimized()) {
+        keyInfo.setParentObjectID(i);
+        keyInfo.setFileName(OzoneFSUtils.getFileName(keyInfo.getKeyName()));
+        OMRequestTestUtils.addMultipartKeyToOpenFileTable(false,
+            keyInfo.getFileName(), keyInfo, uploadId, 0L, omMetadataManager);
+      } else {
+        OMRequestTestUtils.addMultipartKeyToOpenKeyTable(false,
+            keyInfo, uploadId, 0L, omMetadataManager);
+      }
+      OMRequestTestUtils.addMultipartInfoToTable(false, keyInfo,
+          multipartKeyInfo, 0L, omMetadataManager);
+    }
+
+    // Return empty since only MPU-related open keys exist.
+    assertTrue(omMetadataManager.getExpiredOpenKeys(expireThreshold,
+        numExpiredMPUOpenKeys, bucketLayout).getOpenKeyBuckets().isEmpty());
+
+
+    // This is for MPU-related open keys prior to isMultipartKey fix in
+    // HDDS-9017. Although these open keys are MPU-related,
+    // the isMultipartKey flags are set to false
+    for (int i = numExpiredMPUOpenKeys; i < 2 * numExpiredMPUOpenKeys; i++) {
+      final OmKeyInfo keyInfo = OMRequestTestUtils.createOmKeyInfo(volumeName,
+          bucketName, "expired" + i,
+          HddsProtos.ReplicationType.RATIS, HddsProtos.ReplicationFactor.ONE,
+          0L, expiredOpenKeyCreationTime, false);
+
+      final String uploadId = OMMultipartUploadUtils.getMultipartUploadId();
+      final OmMultipartKeyInfo multipartKeyInfo = OMRequestTestUtils.
+          createOmMultipartKeyInfo(uploadId, expiredOpenKeyCreationTime,
+              HddsProtos.ReplicationType.RATIS,
+              HddsProtos.ReplicationFactor.ONE, 0L);
+
+      if (bucketLayout.isFileSystemOptimized()) {
+        keyInfo.setParentObjectID(i);
+        keyInfo.setFileName(OzoneFSUtils.getFileName(keyInfo.getKeyName()));
+        OMRequestTestUtils.addMultipartKeyToOpenFileTable(false,
+            keyInfo.getFileName(), keyInfo, uploadId, 0L, omMetadataManager);
+      } else {
+        OMRequestTestUtils.addMultipartKeyToOpenKeyTable(false,
+            keyInfo, uploadId, 0L, omMetadataManager);
+      }
+      OMRequestTestUtils.addMultipartInfoToTable(false, keyInfo,
+          multipartKeyInfo, 0L, omMetadataManager);
+    }
+
+    // MPU-related open keys should not be fetched regardless of isMultipartKey
+    // flag if has the multipart upload characteristics
+    assertTrue(omMetadataManager.getExpiredOpenKeys(expireThreshold,
+            numExpiredMPUOpenKeys, bucketLayout).getOpenKeyBuckets()
+        .isEmpty());
+  }
+
+  private void testGetExpiredMPUs() throws Exception {
+    final String bucketName = UUID.randomUUID().toString();
+    final String volumeName = UUID.randomUUID().toString();
+    final int numExpiredMPUs = 4;
+    final int numUnexpiredMPUs = 1;
+    final int numPartsPerMPU = 5;
+    // To create expired keys, they will be assigned a creation time as
+    // old as the minimum expiration time.
+    final long expireThresholdMillis = ozoneConfiguration.getTimeDuration(
+        OZONE_OM_MPU_EXPIRE_THRESHOLD,
+        OZONE_OM_MPU_EXPIRE_THRESHOLD_DEFAULT,
+        TimeUnit.MILLISECONDS);
+
+    final Duration expireThreshold = Duration.ofMillis(expireThresholdMillis);
+
+    final long expiredMPUCreationTime =
+        Instant.now().minus(expireThreshold).toEpochMilli();
+
+    // Add expired MPUs to multipartInfoTable.
+    // The method under test does not check for expired open keys in the
+    // cache, since they will be picked up once the cache is flushed.
+    Set<String> expiredMPUs = new HashSet<>();
+    for (int i = 0; i < numExpiredMPUs + numUnexpiredMPUs; i++) {
+      final long creationTime = i < numExpiredMPUs ?
+          expiredMPUCreationTime : Instant.now().toEpochMilli();
+
+      String uploadId = OMMultipartUploadUtils.getMultipartUploadId();
+      final OmMultipartKeyInfo mpuKeyInfo = OMRequestTestUtils
+          .createOmMultipartKeyInfo(uploadId, creationTime,
+              HddsProtos.ReplicationType.RATIS,
+              HddsProtos.ReplicationFactor.ONE, 0L);
+
+      String keyName = "expired" + i;
+      // Key info to construct the MPU DB key
+      final OmKeyInfo keyInfo = OMRequestTestUtils.createOmKeyInfo(volumeName,
+          bucketName, keyName, HddsProtos.ReplicationType.RATIS,
+          HddsProtos.ReplicationFactor.ONE, 0L, creationTime);
+
+
+      for (int j = 1; j <= numPartsPerMPU; j++) {
+        PartKeyInfo partKeyInfo = OMRequestTestUtils
+            .createPartKeyInfo(volumeName, bucketName, keyName, uploadId, j);
+        OMRequestTestUtils.addPart(partKeyInfo, mpuKeyInfo);
+      }
+
+      final String mpuDbKey = OMRequestTestUtils.addMultipartInfoToTable(
+          false, keyInfo, mpuKeyInfo, 0L, omMetadataManager);
+
+      expiredMPUs.add(mpuDbKey);
+    }
+
+    // Test retrieving fewer expire MPU parts than actually exist (exact).
+    List<ExpiredMultipartUploadsBucket>
+        someExpiredMPUs = omMetadataManager.getExpiredMultipartUploads(
+        expireThreshold,
+        (numExpiredMPUs * numPartsPerMPU) - (numPartsPerMPU));
+    List<String> names = getMultipartKeyNames(someExpiredMPUs);
+    assertEquals(numExpiredMPUs - 1, names.size());
+    assertTrue(expiredMPUs.containsAll(names));
+
+    // Test retrieving fewer expire MPU parts than actually exist (round up).
+    someExpiredMPUs = omMetadataManager.getExpiredMultipartUploads(
+        expireThreshold,
+        (numExpiredMPUs * numPartsPerMPU) - (numPartsPerMPU + 1));
+    names = getMultipartKeyNames(someExpiredMPUs);
+    assertEquals(numExpiredMPUs - 1, names.size());
+    assertTrue(expiredMPUs.containsAll(names));
+
+    // Test attempting to retrieving more expire MPU parts than actually exist.
+    List<ExpiredMultipartUploadsBucket> allExpiredMPUs =
+        omMetadataManager.getExpiredMultipartUploads(expireThreshold,
+            (numExpiredMPUs * numPartsPerMPU) + numPartsPerMPU);
+    names = getMultipartKeyNames(allExpiredMPUs);
+    assertEquals(numExpiredMPUs, names.size());
+    assertTrue(expiredMPUs.containsAll(names));
+
+    // Test retrieving exact amount of MPU parts than actually exist.
+    allExpiredMPUs =
+        omMetadataManager.getExpiredMultipartUploads(expireThreshold,
+            (numExpiredMPUs * numPartsPerMPU));
+    names = getMultipartKeyNames(allExpiredMPUs);
+    assertEquals(numExpiredMPUs, names.size());
+    assertTrue(expiredMPUs.containsAll(names));
+  }
+
+  private List<String> getOpenKeyNames(
+      Collection<OpenKeyBucket.Builder> openKeyBuckets) {
     return openKeyBuckets.stream()
-        .map(OpenKeyBucket::getKeysList)
+        .map(OpenKeyBucket.Builder::getKeysList)
         .flatMap(List::stream)
         .map(OpenKey::getName)
         .collect(Collectors.toList());
   }
 
+  private List<String> getMultipartKeyNames(
+      List<ExpiredMultipartUploadsBucket> expiredMultipartUploadsBuckets) {
+    return expiredMultipartUploadsBuckets.stream()
+        .map(ExpiredMultipartUploadsBucket::getMultipartUploadsList)
+        .flatMap(List::stream)
+        .map(ExpiredMultipartUploadInfo::getName)
+        .collect(Collectors.toList());
+  }
+
   private void addKeysToOM(String volumeName, String bucketName,
       String keyName, int i) throws Exception {
 
@@ -664,6 +882,146 @@ public class TestOmMetadataManager {
         new HashSet<>(Arrays.asList(OmMetadataManagerImpl.ALL_TABLES));
     Set<String> tablesInManager = omMetadataManager.listTableNames();
 
-    Assert.assertEquals(tablesByDefinition, tablesInManager);
+    assertEquals(tablesByDefinition, tablesInManager);
   }
-}
\ No newline at end of file
+
+  @Test
+  public void testListSnapshot() throws Exception {
+    String vol1 = "vol1";
+    String bucket1 = "bucket1";
+
+    OMRequestTestUtils.addVolumeToDB(vol1, omMetadataManager);
+    addBucketsToCache(vol1, bucket1);
+    String prefixA = "snapshotA";
+    String prefixB = "snapshotB";
+    TreeMap<String, SnapshotInfo> snapshotsASnapshotIDMap = new TreeMap<>();
+
+    for (int i = 1; i <= 100; i++) {
+      if (i % 2 == 0) {
+        snapshotsASnapshotIDMap.put(prefixA + i,
+            OMRequestTestUtils.addSnapshotToTable(vol1, bucket1,
+            prefixA + i, omMetadataManager));
+        if (i % 4 == 0) {
+          snapshotsASnapshotIDMap.put(prefixA + i,
+              OMRequestTestUtils.addSnapshotToTableCache(vol1, bucket1,
+              prefixA + i, omMetadataManager));
+        }
+      } else {
+        OMRequestTestUtils.addSnapshotToTableCache(vol1, bucket1,
+            prefixB + i, omMetadataManager);
+      }
+    }
+
+    //Test listing all snapshots.
+    List<SnapshotInfo> snapshotInfos = omMetadataManager.listSnapshot(vol1,
+        bucket1, null, null, 100);
+    assertEquals(100, snapshotInfos.size());
+
+    snapshotInfos = omMetadataManager.listSnapshot(vol1,
+        bucket1, prefixA, null, 50);
+    assertEquals(50, snapshotInfos.size());
+    for (SnapshotInfo snapshotInfo : snapshotInfos) {
+      assertTrue(snapshotInfo.getName().startsWith(prefixA));
+    }
+
+    String startSnapshot = prefixA + 38;
+    snapshotInfos = omMetadataManager.listSnapshot(vol1,
+        bucket1, prefixA, startSnapshot, 50);
+    assertEquals(snapshotsASnapshotIDMap.tailMap(startSnapshot).size() - 1,
+        snapshotInfos.size());
+    for (SnapshotInfo snapshotInfo : snapshotInfos) {
+      assertTrue(snapshotInfo.getName().startsWith(prefixA));
+      assertEquals(snapshotInfo, snapshotsASnapshotIDMap.get(
+          snapshotInfo.getName()));
+      assertTrue(snapshotInfo.getName().compareTo(startSnapshot) > 0);
+    }
+
+    startSnapshot = null;
+    TreeSet<String> expectedSnapshot = new TreeSet<>();
+    for (int i = 1; i <= 5; i++) {
+      snapshotInfos = omMetadataManager.listSnapshot(
+          vol1, bucket1, prefixA, startSnapshot, 10);
+      assertEquals(10, snapshotInfos.size());
+
+      for (SnapshotInfo snapshotInfo : snapshotInfos) {
+        expectedSnapshot.add(snapshotInfo.getName());
+        assertEquals(snapshotInfo, snapshotsASnapshotIDMap.get(
+            snapshotInfo.getName()));
+        assertTrue(snapshotInfo.getName().startsWith(prefixA));
+        startSnapshot = snapshotInfo.getName();
+      }
+    }
+    assertEquals(snapshotsASnapshotIDMap.keySet(), expectedSnapshot);
+
+    // As now we have iterated all 50 snapshots, calling next time should
+    // return empty list.
+    snapshotInfos = omMetadataManager.listSnapshot(vol1, bucket1,
+        prefixA, startSnapshot, 10);
+
+    assertEquals(snapshotInfos.size(), 0);
+  }
+
+  @ParameterizedTest
+  @MethodSource("listSnapshotWithInvalidPathCases")
+  public void testListSnapshotWithInvalidPath(String volume,
+                                              String bucket,
+                                              ResultCodes expectedResultCode)
+      throws Exception {
+    String vol1 = "vol1";
+    String bucket1 = "bucket1";
+
+    OMRequestTestUtils.addVolumeToDB(vol1, omMetadataManager);
+    addBucketsToCache(vol1, bucket1);
+
+    OMException oe = assertThrows(OMException.class,
+        () -> omMetadataManager.listSnapshot(volume, bucket, null, null, 100));
+    assertEquals(expectedResultCode, oe.getResult());
+  }
+
+  private static Stream<Arguments> listSnapshotWithInvalidPathCases() {
+    return Stream.of(
+        arguments(null, null, VOLUME_NOT_FOUND),
+        arguments("vol1", null, BUCKET_NOT_FOUND),
+        arguments("vol1", "nonexistentBucket", BUCKET_NOT_FOUND)
+    );
+  }
+
+  @Test
+  public void testListSnapshotDoesNotListOtherBucketSnapshots()
+          throws Exception {
+    String vol1 = "vol1";
+    String bucket1 = "bucket1";
+    String bucket2 = "bucket2";
+
+    OMRequestTestUtils.addVolumeToDB(vol1, omMetadataManager);
+    addBucketsToCache(vol1, bucket1);
+    addBucketsToCache(vol1, bucket2);
+    String snapshotName1 = "snapshot1-";
+    String snapshotName2 = "snapshot2-";
+
+    for (int i = 1; i <= 2; i++) {
+      OMRequestTestUtils.addSnapshotToTable(vol1, bucket1,
+              snapshotName1 + i, omMetadataManager);
+    }
+
+    for (int i = 1; i <= 5; i++) {
+      OMRequestTestUtils.addSnapshotToTable(vol1, bucket2,
+              snapshotName2 + i, omMetadataManager);
+    }
+
+    //Test listing snapshots only lists snapshots of specified bucket
+    List<SnapshotInfo> snapshotInfos1 = omMetadataManager.listSnapshot(vol1,
+            bucket1, null, null, Integer.MAX_VALUE);
+    assertEquals(2, snapshotInfos1.size());
+    for (SnapshotInfo snapshotInfo : snapshotInfos1) {
+      assertTrue(snapshotInfo.getName().startsWith(snapshotName1));
+    }
+
+    List<SnapshotInfo> snapshotInfos2 = omMetadataManager.listSnapshot(vol1,
+            bucket2, null, null, Integer.MAX_VALUE);
+    assertEquals(5, snapshotInfos2.size());
+    for (SnapshotInfo snapshotInfo : snapshotInfos2) {
+      assertTrue(snapshotInfo.getName().startsWith(snapshotName2));
+    }
+  }
+}
