@@ -19,6 +19,7 @@
 package org.apache.hadoop.ozone.om.ratis;
 
 import java.io.IOException;
+import java.nio.file.Path;
 import java.util.Queue;
 import java.util.UUID;
 import java.util.concurrent.ConcurrentLinkedQueue;
@@ -32,19 +33,18 @@ import org.apache.hadoop.ozone.audit.AuditMessage;
 import org.apache.hadoop.ozone.om.OMConfigKeys;
 import org.apache.hadoop.ozone.om.OMMetrics;
 import org.apache.hadoop.ozone.om.OzoneManager;
-import org.apache.hadoop.ozone.om.ratis.utils.OzoneManagerDoubleBufferHelper;
 import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.om.request.bucket.OMBucketCreateRequest;
 import org.apache.hadoop.ozone.om.request.bucket.OMBucketDeleteRequest;
 import org.apache.hadoop.ozone.om.request.volume.OMVolumeCreateRequest;
 import org.apache.hadoop.ozone.om.response.OMClientResponse;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.TemporaryFolder;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.BucketInfo;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
@@ -63,8 +63,9 @@ import org.apache.hadoop.util.Daemon;
 import org.mockito.Mockito;
 
 import static org.apache.hadoop.ozone.OzoneConsts.TRANSACTION_INFO_KEY;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.fail;
+import static org.apache.hadoop.ozone.om.request.OMRequestTestUtils.newBucketInfoBuilder;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.fail;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.Mockito.when;
 
@@ -78,7 +79,6 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
   private OzoneManager ozoneManager;
   private OMMetrics omMetrics;
   private AuditLogger auditLogger;
-  private OzoneManagerDoubleBufferHelper ozoneManagerDoubleBufferHelper;
   private OMMetadataManager omMetadataManager;
   private OzoneManagerDoubleBuffer doubleBuffer;
   private final AtomicLong trxId = new AtomicLong(0);
@@ -86,18 +86,19 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
   private volatile long lastAppliedIndex;
   private long term = 1L;
 
-  @Rule
-  public TemporaryFolder folder = new TemporaryFolder();
+  @TempDir
+  private Path folder;
 
-  @Before
+  @BeforeEach
   public void setup() throws IOException {
     ozoneManager = Mockito.mock(OzoneManager.class,
         Mockito.withSettings().stubOnly());
     omMetrics = OMMetrics.create();
     OzoneConfiguration ozoneConfiguration = new OzoneConfiguration();
     ozoneConfiguration.set(OMConfigKeys.OZONE_OM_DB_DIRS,
-        folder.newFolder().getAbsolutePath());
-    omMetadataManager = new OmMetadataManagerImpl(ozoneConfiguration);
+        folder.toAbsolutePath().toString());
+    omMetadataManager = new OmMetadataManagerImpl(ozoneConfiguration,
+        ozoneManager);
     when(ozoneManager.getMetrics()).thenReturn(omMetrics);
     when(ozoneManager.getMetadataManager()).thenReturn(omMetadataManager);
     when(ozoneManager.getMaxUserVolumeCount()).thenReturn(10L);
@@ -114,10 +115,9 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
         .enableRatis(true)
         .setIndexToTerm((i) -> term)
         .build();
-    ozoneManagerDoubleBufferHelper = doubleBuffer::add;
   }
 
-  @After
+  @AfterEach
   public void stop() {
     doubleBuffer.stop();
   }
@@ -180,10 +180,10 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
         doubleBuffer.getFlushedTransactionCount() ==
             (bucketCount + deleteCount + 1), 100, 120000);
 
-    Assert.assertEquals(1, omMetadataManager.countRowsInTable(
+    Assertions.assertEquals(1, omMetadataManager.countRowsInTable(
         omMetadataManager.getVolumeTable()));
 
-    Assert.assertEquals(5, omMetadataManager.countRowsInTable(
+    Assertions.assertEquals(5, omMetadataManager.countRowsInTable(
         omMetadataManager.getBucketTable()));
 
     // Now after this in our DB we should have 5 buckets and one volume
@@ -203,9 +203,9 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
         omMetadataManager.getTransactionInfoTable().get(TRANSACTION_INFO_KEY);
     assertNotNull(transactionInfo);
 
-    Assert.assertEquals(lastAppliedIndex,
+    Assertions.assertEquals(lastAppliedIndex,
         transactionInfo.getTransactionIndex());
-    Assert.assertEquals(term, transactionInfo.getTerm());
+    Assertions.assertEquals(term, transactionInfo.getTerm());
   }
 
   /**
@@ -253,10 +253,10 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
     GenericTestUtils.waitFor(() -> doubleBuffer.getFlushedTransactionCount()
             == (bucketCount + deleteCount + 2), 100, 120000);
 
-    Assert.assertEquals(2, omMetadataManager.countRowsInTable(
+    Assertions.assertEquals(2, omMetadataManager.countRowsInTable(
         omMetadataManager.getVolumeTable()));
 
-    Assert.assertEquals(10, omMetadataManager.countRowsInTable(
+    Assertions.assertEquals(10, omMetadataManager.countRowsInTable(
         omMetadataManager.getBucketTable()));
 
     // Now after this in our DB we should have 5 buckets and one volume
@@ -273,7 +273,7 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
     // running in parallel, so lastAppliedIndex cannot be always
     // total transaction count. So, just checking here whether it is less
     // than total transaction count.
-    Assert.assertTrue(lastAppliedIndex <= bucketCount + deleteCount + 2);
+    Assertions.assertTrue(lastAppliedIndex <= bucketCount + deleteCount + 2);
 
 
   }
@@ -310,8 +310,10 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
     OMBucketDeleteRequest omBucketDeleteRequest =
         new OMBucketDeleteRequest(omRequest);
 
-    return omBucketDeleteRequest.validateAndUpdateCache(ozoneManager,
-        transactionID, ozoneManagerDoubleBufferHelper);
+    OMClientResponse omClientResponse =
+        omBucketDeleteRequest.validateAndUpdateCache(ozoneManager, transactionID);
+    doubleBuffer.add(omClientResponse, transactionID);
+    return omClientResponse;
   }
 
   /**
@@ -322,16 +324,17 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
       OMVolumeCreateResponse omVolumeCreateResponse) throws Exception {
     OmVolumeArgs tableVolumeArgs = omMetadataManager.getVolumeTable().get(
         omMetadataManager.getVolumeKey(volumeName));
-    Assert.assertNotNull(tableVolumeArgs);
+    Assertions.assertNotNull(tableVolumeArgs);
 
     OmVolumeArgs omVolumeArgs = omVolumeCreateResponse.getOmVolumeArgs();
 
-    Assert.assertEquals(omVolumeArgs.getVolume(), tableVolumeArgs.getVolume());
-    Assert.assertEquals(omVolumeArgs.getAdminName(),
+    Assertions.assertEquals(omVolumeArgs.getVolume(),
+        tableVolumeArgs.getVolume());
+    Assertions.assertEquals(omVolumeArgs.getAdminName(),
         tableVolumeArgs.getAdminName());
-    Assert.assertEquals(omVolumeArgs.getOwnerName(),
+    Assertions.assertEquals(omVolumeArgs.getOwnerName(),
         tableVolumeArgs.getOwnerName());
-    Assert.assertEquals(omVolumeArgs.getCreationTime(),
+    Assertions.assertEquals(omVolumeArgs.getCreationTime(),
         tableVolumeArgs.getCreationTime());
   }
 
@@ -352,13 +355,13 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
       } catch (IOException ex) {
         fail("testDoubleBufferWithMixOfTransactions failed");
       }
-      Assert.assertNotNull(tableBucketInfo);
+      Assertions.assertNotNull(tableBucketInfo);
 
-      Assert.assertEquals(omBucketInfo.getVolumeName(),
+      Assertions.assertEquals(omBucketInfo.getVolumeName(),
           tableBucketInfo.getVolumeName());
-      Assert.assertEquals(omBucketInfo.getBucketName(),
+      Assertions.assertEquals(omBucketInfo.getBucketName(),
           tableBucketInfo.getBucketName());
-      Assert.assertEquals(omBucketInfo.getCreationTime(),
+      Assertions.assertEquals(omBucketInfo.getCreationTime(),
           tableBucketInfo.getCreationTime());
     });
   }
@@ -371,7 +374,7 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
       deleteBucketQueue) {
     deleteBucketQueue.forEach((omBucketDeleteResponse -> {
       try {
-        Assert.assertNull(omMetadataManager.getBucketTable().get(
+        Assertions.assertNull(omMetadataManager.getBucketTable().get(
             omMetadataManager.getBucketKey(
                 omBucketDeleteResponse.getVolumeName(),
                 omBucketDeleteResponse.getBucketName())));
@@ -415,7 +418,7 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
         assertRowCount(expectedBuckets, omMetadataManager.getBucketTable()),
         300, volumeCount * 300);
 
-    Assert.assertTrue(doubleBuffer.getFlushIterations() > 0);
+    Assertions.assertTrue(doubleBuffer.getFlushIterations() > 0);
   }
 
   private boolean assertRowCount(int expected, Table<String, ?> table) {
@@ -456,8 +459,10 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
     OMVolumeCreateRequest omVolumeCreateRequest =
         new OMVolumeCreateRequest(omRequest);
 
-    return omVolumeCreateRequest.validateAndUpdateCache(ozoneManager,
-        transactionId, ozoneManagerDoubleBufferHelper);
+    OMClientResponse omClientResponse =
+        omVolumeCreateRequest.validateAndUpdateCache(ozoneManager, transactionId);
+    doubleBuffer.add(omClientResponse, transactionId);
+    return omClientResponse;
   }
 
   /**
@@ -467,17 +472,19 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
   private OMBucketCreateResponse createBucket(String volumeName,
       String bucketName, long transactionID)  {
 
+    BucketInfo.Builder bucketInfo =
+        newBucketInfoBuilder(bucketName, volumeName)
+            .setStorageType(OzoneManagerProtocolProtos.StorageTypeProto.DISK);
     OzoneManagerProtocolProtos.OMRequest omRequest =
-        OMRequestTestUtils.createBucketRequest(bucketName, volumeName, false,
-            OzoneManagerProtocolProtos.StorageTypeProto.DISK);
+        OMRequestTestUtils.newCreateBucketRequest(bucketInfo).build();
 
     OMBucketCreateRequest omBucketCreateRequest =
         new OMBucketCreateRequest(omRequest);
 
-    return (OMBucketCreateResponse) omBucketCreateRequest
-        .validateAndUpdateCache(ozoneManager, transactionID,
-            ozoneManagerDoubleBufferHelper);
-
+    OMClientResponse omClientResponse =
+        omBucketCreateRequest.validateAndUpdateCache(ozoneManager, transactionID);
+    doubleBuffer.add(omClientResponse, transactionID);
+    return (OMBucketCreateResponse) omClientResponse;
   }
 
   /**
@@ -492,7 +499,5 @@ public class TestOzoneManagerDoubleBufferWithOMResponse {
         .setDeleteBucketResponse(DeleteBucketResponse.newBuilder().build())
         .build(), volumeName, bucketName);
   }
-
-
 }
 
