@@ -19,17 +19,22 @@
 
 package org.apache.hadoop.ozone.om.request.bucket;
 
+import org.apache.hadoop.ozone.om.OMConfigKeys;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.hadoop.ozone.om.response.OMClientResponse;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMRequest;
-import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.StorageTypeProto;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 import java.util.UUID;
 
+import static org.apache.hadoop.ozone.om.request.OMRequestTestUtils.newBucketInfoBuilder;
+import static org.apache.hadoop.ozone.om.request.OMRequestTestUtils.newCreateBucketRequest;
+import static org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.BucketLayoutProto.FILE_SYSTEM_OPTIMIZED;
 import static org.mockito.Mockito.when;
 
 /**
@@ -38,34 +43,86 @@ import static org.mockito.Mockito.when;
 public class TestOMBucketCreateRequestWithFSO
     extends TestOMBucketCreateRequest {
 
+  @BeforeEach
+  public void setupWithFSO() {
+    ozoneManager.getConfiguration()
+        .set(OMConfigKeys.OZONE_DEFAULT_BUCKET_LAYOUT,
+            OMConfigKeys.OZONE_BUCKET_LAYOUT_FILE_SYSTEM_OPTIMIZED);
+  }
+
   @Test
   public void testValidateAndUpdateCacheWithFSO() throws Exception {
     when(ozoneManager.getOMDefaultBucketLayout()).thenReturn(
-        BucketLayout.FILE_SYSTEM_OPTIMIZED.name());
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
     String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
 
-    Assert.assertEquals(0, omMetrics.getNumFSOBucketCreates());
+    Assertions.assertEquals(0, omMetrics.getNumFSOBucketCreates());
 
     OMBucketCreateRequest omBucketCreateRequest = doPreExecute(volumeName,
-        bucketName);
+        bucketName, true);
 
     doValidateAndUpdateCache(volumeName, bucketName,
         omBucketCreateRequest.getOmRequest());
 
-    Assert.assertEquals(1, omMetrics.getNumFSOBucketCreates());
-    Assert.assertEquals(BucketLayout.FILE_SYSTEM_OPTIMIZED,
+    Assertions.assertEquals(1, omMetrics.getNumFSOBucketCreates());
+    Assertions.assertEquals(BucketLayout.FILE_SYSTEM_OPTIMIZED,
+        omMetadataManager.getBucketTable().get(bucketKey).getBucketLayout());
+  }
+
+  /**
+   * Gets the bucket layout from the ozone configuration and
+   * creates a bucket request with the latest client version.
+   * Checking that the configuration layout is the same with
+   * the one used for the request.
+   * @throws Exception
+   */
+  @Test
+  public void testValidateAndUpdateCacheVerifyBucketLayoutWithFSO()
+      throws Exception {
+    String volumeName = UUID.randomUUID().toString();
+    String bucketName = UUID.randomUUID().toString();
+    String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
+
+    // Checking bucket layout from configuration
+    Assertions.assertEquals(
+        OMConfigKeys.OZONE_BUCKET_LAYOUT_FILE_SYSTEM_OPTIMIZED,
+        ozoneManager.getConfiguration()
+            .get(OMConfigKeys.OZONE_DEFAULT_BUCKET_LAYOUT));
+
+    Assertions.assertEquals(0, omMetrics.getNumFSOBucketCreates());
+
+    // OzoneManager is mocked, the bucket layout will return null
+    when(ozoneManager.getOMDefaultBucketLayout()).thenReturn(
+        BucketLayout.FILE_SYSTEM_OPTIMIZED);
+
+    OMBucketCreateRequest omBucketCreateRequest =
+        doPreExecute(volumeName, bucketName, false);
+
+    doValidateAndUpdateCache(volumeName, bucketName,
+        omBucketCreateRequest.getOmRequest());
+
+    Assertions.assertEquals(1, omMetrics.getNumFSOBucketCreates());
+    Assertions.assertEquals(BucketLayout.FILE_SYSTEM_OPTIMIZED,
         omMetadataManager.getBucketTable().get(bucketKey).getBucketLayout());
   }
 
   private OMBucketCreateRequest doPreExecute(String volumeName,
-      String bucketName) throws Exception {
+      String bucketName, boolean layoutFSOFromCli)
+      throws Exception {
     addCreateVolumeToTable(volumeName, omMetadataManager);
-    OMRequest originalRequest =
-        OMRequestTestUtils.createBucketReqFSO(bucketName, volumeName,
-                false, StorageTypeProto.SSD);
 
+    OzoneManagerProtocolProtos.BucketInfo.Builder bucketInfo =
+        newBucketInfoBuilder(bucketName, volumeName);
+
+    if (layoutFSOFromCli) {
+      bucketInfo
+          .setBucketLayout(FILE_SYSTEM_OPTIMIZED)
+          .addMetadata(OMRequestTestUtils.fsoMetadata());
+    }
+
+    OMRequest originalRequest = newCreateBucketRequest(bucketInfo).build();
     OMBucketCreateRequest omBucketCreateRequest =
         new OMBucketCreateRequest(originalRequest);
 
@@ -74,47 +131,50 @@ public class TestOMBucketCreateRequestWithFSO
     return new OMBucketCreateRequest(modifiedRequest);
   }
 
-  private void doValidateAndUpdateCache(String volumeName, String bucketName,
+  protected void doValidateAndUpdateCache(String volumeName, String bucketName,
       OMRequest modifiedRequest) throws Exception {
     String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
 
     // As we have not still called validateAndUpdateCache, get() should
     // return null.
 
-    Assert.assertNull(omMetadataManager.getBucketTable().get(bucketKey));
+    Assertions.assertNull(omMetadataManager.getBucketTable().get(bucketKey));
     OMBucketCreateRequest omBucketCreateRequest =
         new OMBucketCreateRequest(modifiedRequest);
 
 
     OMClientResponse omClientResponse =
-        omBucketCreateRequest.validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        omBucketCreateRequest.validateAndUpdateCache(ozoneManager, 1);
 
     // As now after validateAndUpdateCache it should add entry to cache, get
     // should return non null value.
     OmBucketInfo dbBucketInfo =
         omMetadataManager.getBucketTable().get(bucketKey);
-    Assert.assertNotNull(omMetadataManager.getBucketTable().get(bucketKey));
+    Assertions.assertNotNull(omMetadataManager.getBucketTable().get(bucketKey));
+
+    BucketLayout bucketLayout = BucketLayout
+        .fromString(ozoneManager.getConfiguration()
+            .get(OMConfigKeys.OZONE_DEFAULT_BUCKET_LAYOUT));
 
     // verify table data with actual request data.
     OmBucketInfo bucketInfoFromProto = OmBucketInfo.getFromProtobuf(
-        modifiedRequest.getCreateBucketRequest().getBucketInfo());
+        modifiedRequest.getCreateBucketRequest().getBucketInfo(), bucketLayout);
 
-    Assert.assertEquals(bucketInfoFromProto.getCreationTime(),
+    Assertions.assertEquals(bucketInfoFromProto.getCreationTime(),
         dbBucketInfo.getCreationTime());
-    Assert.assertEquals(bucketInfoFromProto.getModificationTime(),
+    Assertions.assertEquals(bucketInfoFromProto.getModificationTime(),
         dbBucketInfo.getModificationTime());
-    Assert.assertEquals(bucketInfoFromProto.getAcls(),
+    Assertions.assertEquals(bucketInfoFromProto.getAcls(),
         dbBucketInfo.getAcls());
-    Assert.assertEquals(bucketInfoFromProto.getIsVersionEnabled(),
+    Assertions.assertEquals(bucketInfoFromProto.getIsVersionEnabled(),
         dbBucketInfo.getIsVersionEnabled());
-    Assert.assertEquals(bucketInfoFromProto.getStorageType(),
+    Assertions.assertEquals(bucketInfoFromProto.getStorageType(),
         dbBucketInfo.getStorageType());
-    Assert.assertEquals(bucketInfoFromProto.getMetadata(),
+    Assertions.assertEquals(bucketInfoFromProto.getMetadata(),
         dbBucketInfo.getMetadata());
-    Assert.assertEquals(bucketInfoFromProto.getEncryptionKeyInfo(),
+    Assertions.assertEquals(bucketInfoFromProto.getEncryptionKeyInfo(),
         dbBucketInfo.getEncryptionKeyInfo());
-    Assert.assertEquals(bucketInfoFromProto.getBucketLayout(),
+    Assertions.assertEquals(bucketInfoFromProto.getBucketLayout(),
         dbBucketInfo.getBucketLayout());
 
     // verify OMResponse.
