@@ -32,8 +32,8 @@ import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
 import org.apache.ozone.test.GenericTestUtils;
 
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Test;
 import org.slf4j.LoggerFactory;
 
 import org.apache.hadoop.ozone.om.response.OMClientResponse;
@@ -75,10 +75,10 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
         .getModificationTime();
     long newModTime = preExecuteRequest.getSetBucketPropertyRequest()
         .getModificationTime();
-    Assert.assertTrue(newModTime > originModTime);
+    Assertions.assertTrue(newModTime > originModTime);
 
     // As user info gets added.
-    Assert.assertNotEquals(omRequest,
+    Assertions.assertNotEquals(omRequest,
         omBucketSetPropertyRequest.preExecute(ozoneManager));
   }
 
@@ -99,15 +99,13 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
         new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse =
-        omBucketSetPropertyRequest.validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        omBucketSetPropertyRequest.validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertEquals(true,
-        omMetadataManager.getBucketTable().get(
+    Assertions.assertTrue(omMetadataManager.getBucketTable().get(
             omMetadataManager.getBucketKey(volumeName, bucketName))
-            .getIsVersionEnabled());
+        .getIsVersionEnabled());
 
-    Assert.assertEquals(OzoneManagerProtocolProtos.Status.OK,
+    Assertions.assertEquals(OzoneManagerProtocolProtos.Status.OK,
         omClientResponse.getOMResponse().getStatus());
   }
 
@@ -128,15 +126,14 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
         new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse =
-        omBucketSetPropertyRequest.validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        omBucketSetPropertyRequest.validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertEquals(BucketLayout.FILE_SYSTEM_OPTIMIZED,
+    Assertions.assertEquals(BucketLayout.FILE_SYSTEM_OPTIMIZED,
         omMetadataManager.getBucketTable().get(
             omMetadataManager.getBucketKey(volumeName, bucketName))
             .getBucketLayout());
 
-    Assert.assertEquals(OzoneManagerProtocolProtos.Status.OK,
+    Assertions.assertEquals(OzoneManagerProtocolProtos.Status.OK,
         omClientResponse.getOMResponse().getStatus());
   }
 
@@ -153,13 +150,12 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
         new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse =
-        omBucketSetPropertyRequest.validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        omBucketSetPropertyRequest.validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertEquals(OzoneManagerProtocolProtos.Status.BUCKET_NOT_FOUND,
+    Assertions.assertEquals(OzoneManagerProtocolProtos.Status.BUCKET_NOT_FOUND,
         omClientResponse.getOMResponse().getStatus());
 
-    Assert.assertNull(omMetadataManager.getBucketTable().get(
+    Assertions.assertNull(omMetadataManager.getBucketTable().get(
         omMetadataManager.getBucketKey(volumeName, bucketName)));
 
   }
@@ -199,17 +195,16 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
             );
 
     OMClientResponse omClientResponse = omBucketSetPropertyRequest
-        .validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
     //capture the error log
-    Assert.assertTrue(logs.getOutput().contains(
+    Assertions.assertTrue(logs.getOutput().contains(
         "Setting bucket property failed for bucket"));
 
-    Assert.assertFalse(omClientResponse.getOMResponse().getSuccess());
-    Assert.assertEquals(omClientResponse.getOMResponse().getStatus(),
+    Assertions.assertFalse(omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertEquals(omClientResponse.getOMResponse().getStatus(),
         OzoneManagerProtocolProtos.Status.QUOTA_EXCEEDED);
-    Assert.assertTrue(omClientResponse.getOMResponse().getMessage().
+    Assertions.assertTrue(omClientResponse.getOMResponse().getMessage().
         contains("Total buckets quota in this volume " +
             "should not be greater than volume quota"));
   }
@@ -237,16 +232,16 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
 
     // WHEN
     OMClientResponse omClientResponse = omBucketSetPropertyRequest
-        .validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
     // THEN
-    Assert.assertFalse(omClientResponse.getOMResponse().getSuccess());
-    Assert.assertEquals(
+    Assertions.assertFalse(omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertEquals(
         OzoneManagerProtocolProtos.Status.NOT_SUPPORTED_OPERATION,
         omClientResponse.getOMResponse().getStatus());
     String message = omClientResponse.getOMResponse().getMessage();
-    Assert.assertTrue(message, message.contains("Cannot set property on link"));
+    Assertions.assertTrue(message.contains("Cannot set property on link"),
+        message);
   }
 
   @Test
@@ -261,7 +256,7 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
 
     BucketArgs bucketArgs = OmBucketArgs.newBuilder()
         .setDefaultReplicationConfig(new DefaultReplicationConfig(
-            EC, new ECReplicationConfig(3, 2)))
+            new ECReplicationConfig(3, 2)))
         .setBucketName(bucketName)
         .setVolumeName(volumeName)
         .setIsVersionEnabled(true)
@@ -277,17 +272,16 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
         new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse = omBucketSetPropertyRequest
-        .validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertEquals(true, omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertTrue(omClientResponse.getOMResponse().getSuccess());
 
     String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
     OmBucketInfo dbBucketInfo =
         omMetadataManager.getBucketTable().get(bucketKey);
 
-    Assert.assertEquals(8 * GB, dbBucketInfo.getQuotaInBytes());
-    Assert.assertEquals(EC,
+    Assertions.assertEquals(8 * GB, dbBucketInfo.getQuotaInBytes());
+    Assertions.assertEquals(EC,
         dbBucketInfo.getDefaultReplicationConfig().getType());
   }
 
@@ -309,7 +303,7 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
 
     BucketArgs bucketArgs = OmBucketArgs.newBuilder()
             .setDefaultReplicationConfig(new DefaultReplicationConfig(
-                    EC, new ECReplicationConfig(3, 2)))
+                    new ECReplicationConfig(3, 2)))
             .setBucketName(bucketName)
             .setVolumeName(volumeName)
             .setIsVersionEnabled(true)
@@ -325,31 +319,31 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
             new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse = omBucketSetPropertyRequest
-            .validateAndUpdateCache(ozoneManager, 1,
-                    ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertEquals(true, omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertTrue(omClientResponse.getOMResponse().getSuccess());
 
     String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
     OmBucketInfo dbBucketInfo =
             omMetadataManager.getBucketTable().get(bucketKey);
 
-    Assert.assertEquals(TEST_KEY,
+    Assertions.assertEquals(TEST_KEY,
             dbBucketInfo.getEncryptionKeyInfo().getKeyName());
-    Assert.assertEquals(EC,
+    Assertions.assertEquals(EC,
             dbBucketInfo.getDefaultReplicationConfig().getType());
   }
 
   @Test
-  public void testSettingQuotaWithEncryption() throws Exception {
+  public void testSettingQuotaWithEncryptionAndOwner() throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
 
     OmBucketInfo.Builder bucketInfo = new OmBucketInfo.Builder()
-            .setVolumeName(volumeName)
-            .setBucketName(bucketName)
-            .setBucketEncryptionKey(new BucketEncryptionKeyInfo.Builder()
-                    .setKeyName(TEST_KEY).build());
+        .setVolumeName(volumeName)
+        .setBucketName(bucketName)
+        .setOwner("testUser")
+        .setBucketEncryptionKey(new BucketEncryptionKeyInfo.Builder()
+            .setKeyName(TEST_KEY).build());
 
     OMRequestTestUtils.addVolumeToDB(
             volumeName, omMetadataManager);
@@ -363,21 +357,22 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
             new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse = omBucketSetPropertyRequest
-            .validateAndUpdateCache(ozoneManager, 1,
-                    ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertEquals(true, omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertTrue(omClientResponse.getOMResponse().getSuccess());
 
     String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
     OmBucketInfo dbBucketInfo =
             omMetadataManager.getBucketTable().get(bucketKey);
 
-    Assert.assertEquals(TEST_KEY,
+    Assertions.assertEquals(TEST_KEY,
             dbBucketInfo.getEncryptionKeyInfo().getKeyName());
-    Assert.assertEquals(20 * GB,
+    Assertions.assertEquals(20 * GB,
             dbBucketInfo.getQuotaInBytes());
-    Assert.assertEquals(1000L,
+    Assertions.assertEquals(1000L,
             dbBucketInfo.getQuotaInNamespace());
+    Assertions.assertEquals("testUser",
+        dbBucketInfo.getOwner());
   }
 
   @Test
@@ -395,19 +390,18 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
     cacheValue.getCacheValue().incrUsedBytes(5 * GB);
     cacheValue.getCacheValue().incrUsedNamespace(10);
     OMRequest omRequest = createSetBucketPropertyRequest(volumeName,
-        bucketName, true, 1 * GB);
+        bucketName, true, GB);
 
     OMBucketSetPropertyRequest omBucketSetPropertyRequest =
         new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse = omBucketSetPropertyRequest
-        .validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertFalse(omClientResponse.getOMResponse().getSuccess());
-    Assert.assertEquals(omClientResponse.getOMResponse().getStatus(),
+    Assertions.assertFalse(omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertEquals(omClientResponse.getOMResponse().getStatus(),
         OzoneManagerProtocolProtos.Status.QUOTA_ERROR);
-    Assert.assertTrue(omClientResponse.getOMResponse().getMessage().
+    Assertions.assertTrue(omClientResponse.getOMResponse().getMessage().
         contains("Cannot update bucket quota. Requested spaceQuota less than " +
             "used spaceQuota"));
   }
@@ -434,13 +428,12 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
         new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse = omBucketSetPropertyRequest
-        .validateAndUpdateCache(ozoneManager, 1,
-            ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertFalse(omClientResponse.getOMResponse().getSuccess());
-    Assert.assertEquals(omClientResponse.getOMResponse().getStatus(),
+    Assertions.assertFalse(omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertEquals(omClientResponse.getOMResponse().getStatus(),
         OzoneManagerProtocolProtos.Status.QUOTA_ERROR);
-    Assert.assertTrue(omClientResponse.getOMResponse().getMessage().
+    Assertions.assertTrue(omClientResponse.getOMResponse().getMessage().
         contains("Cannot update bucket quota. NamespaceQuota requested " +
             "is less than used namespaceQuota"));
   }
@@ -470,22 +463,18 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
             new OMBucketSetPropertyRequest(omRequest);
 
     OMClientResponse omClientResponse = omBucketSetPropertyRequest
-            .validateAndUpdateCache(ozoneManager, 1,
-                    ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertEquals(true, omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertTrue(omClientResponse.getOMResponse().getSuccess());
 
     OmBucketInfo dbBucketInfoAfter =
             omMetadataManager.getBucketTable().get(bucketKey);
 
-    Assert.assertEquals(null,
-            dbBucketInfoAfter.getDefaultReplicationConfig());
-    Assert.assertEquals(
-            dbBucketInfoBefore.getDefaultReplicationConfig(),
-            dbBucketInfoAfter.getDefaultReplicationConfig());
-    Assert.assertEquals(20 * GB,
+    Assertions.assertNull(dbBucketInfoAfter.getDefaultReplicationConfig());
+    Assertions.assertNull(dbBucketInfoBefore.getDefaultReplicationConfig());
+    Assertions.assertEquals(20 * GB,
             dbBucketInfoAfter.getQuotaInBytes());
-    Assert.assertEquals(1000L,
+    Assertions.assertEquals(1000L,
             dbBucketInfoAfter.getQuotaInNamespace());
 
     /* Bucket with EC replication */
@@ -493,7 +482,7 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
             .setVolumeName(volumeName2)
             .setBucketName(bucketName2)
             .setDefaultReplicationConfig(new DefaultReplicationConfig(
-                    EC, new ECReplicationConfig(3, 2)));
+                    new ECReplicationConfig(3, 2)));
 
     OMRequestTestUtils.addVolumeToDB(volumeName2, omMetadataManager);
     OMRequestTestUtils.addBucketToDB(omMetadataManager, bucketInfo);
@@ -511,25 +500,21 @@ public class TestOMBucketSetPropertyRequest extends TestBucketRequest {
             new OMBucketSetPropertyRequest(omRequest);
 
     omClientResponse = omBucketSetPropertyRequest
-            .validateAndUpdateCache(ozoneManager, 1,
-                    ozoneManagerDoubleBufferHelper);
+        .validateAndUpdateCache(ozoneManager, 1);
 
-    Assert.assertEquals(true, omClientResponse.getOMResponse().getSuccess());
+    Assertions.assertTrue(omClientResponse.getOMResponse().getSuccess());
 
     dbBucketInfoAfter =
             omMetadataManager.getBucketTable().get(bucketKey);
 
-    Assert.assertEquals(EC,
+    Assertions.assertEquals(EC,
             dbBucketInfoAfter.getDefaultReplicationConfig().getType());
-    Assert.assertEquals(
-            dbBucketInfoBefore.getDefaultReplicationConfig().getType(),
-            dbBucketInfoAfter.getDefaultReplicationConfig().getType());
-    Assert.assertEquals(
-            dbBucketInfoBefore.getDefaultReplicationConfig().getFactor(),
-            dbBucketInfoAfter.getDefaultReplicationConfig().getFactor());
-    Assert.assertEquals(20 * GB,
+    Assertions.assertEquals(
+            dbBucketInfoBefore.getDefaultReplicationConfig(),
+            dbBucketInfoAfter.getDefaultReplicationConfig());
+    Assertions.assertEquals(20 * GB,
             dbBucketInfoAfter.getQuotaInBytes());
-    Assert.assertEquals(1000L,
+    Assertions.assertEquals(1000L,
             dbBucketInfoAfter.getQuotaInNamespace());
   }
 }
