@@ -20,17 +20,21 @@ package org.apache.hadoop.ozone.om.request.file;
 
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
+import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmDirectoryInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
+import org.apache.hadoop.ozone.om.response.OMClientResponse;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMRequest;
 import org.apache.hadoop.util.StringUtils;
 import org.apache.hadoop.util.Time;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Test;
 
 import java.util.UUID;
 
+import static org.apache.hadoop.ozone.OzoneConsts.OM_SNAPSHOT_INDICATOR;
 /**
  * Tests OMFileCreateRequest - prefix layout.
  */
@@ -40,7 +44,8 @@ public class TestOMFileCreateRequestWithFSO extends TestOMFileCreateRequest {
   public void testValidateAndUpdateCacheWithNonRecursive() throws Exception {
     testNonRecursivePath(UUID.randomUUID().toString(), false, false, false);
     testNonRecursivePath("a/b", false, false, true);
-    Assert.assertEquals("Invalid metrics value", 0, omMetrics.getNumKeys());
+    Assertions.assertEquals(0, omMetrics.getNumKeys(),
+        "Invalid metrics value");
 
     // Create parent dirs for the path
     OMRequestTestUtils.addParentsToDirTable(volumeName, bucketName,
@@ -55,7 +60,7 @@ public class TestOMFileCreateRequestWithFSO extends TestOMFileCreateRequest {
 
     // Delete child key but retain path "a/b/ in the key table
     OmDirectoryInfo dirPathC = getDirInfo("a/b/c");
-    Assert.assertNotNull("Failed to find dir path: a/b/c", dirPathC);
+    Assertions.assertNotNull(dirPathC, "Failed to find dir path: a/b/c");
     final long volumeId = omMetadataManager.getVolumeId(volumeName);
     final long bucketId = omMetadataManager.getBucketId(volumeName,
             bucketName);
@@ -68,13 +73,41 @@ public class TestOMFileCreateRequestWithFSO extends TestOMFileCreateRequest {
     testNonRecursivePath("a/b/e", false, false, false);
   }
 
+  @Test
+  public void testValidateAndUpdateCacheWithNamespaceQuotaExceeded()
+      throws Exception {
+    OMRequest omRequest = createFileRequest(volumeName, bucketName,
+        "/test/a1/a2", HddsProtos.ReplicationFactor.ONE,
+        HddsProtos.ReplicationType.RATIS, false, true);
+
+    // create bucket with quota limit 1
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, omMetadataManager,
+        OmBucketInfo.newBuilder().setVolumeName(volumeName)
+            .setBucketName(bucketName)
+            .setBucketLayout(getBucketLayout())
+            .setQuotaInNamespace(1));
+    
+    OMFileCreateRequest omFileCreateRequest = getOMFileCreateRequest(omRequest);
+    OMRequest modifiedOmRequest = omFileCreateRequest.preExecute(ozoneManager);
+
+    omFileCreateRequest = getOMFileCreateRequest(modifiedOmRequest);
+    OMClientResponse omFileCreateResponse =
+        omFileCreateRequest.validateAndUpdateCache(ozoneManager, 100L);
+    Assertions.assertSame(omFileCreateResponse.getOMResponse().getStatus(),
+        OzoneManagerProtocolProtos.Status.QUOTA_EXCEEDED);
+  }
+
   @Test
   public void testValidateAndUpdateCacheWithRecursiveAndOverWrite()
           throws Exception {
     String key = "c/d/e/f";
     // Should be able to create file even if parent directories does not exist
     testNonRecursivePath(key, false, true, false);
-    Assert.assertEquals("Invalid metrics value", 3, omMetrics.getNumKeys());
+    Assertions.assertEquals(3, omMetrics.getNumKeys(),
+        "Invalid metrics value");
+    Assertions.assertEquals(omMetadataManager.getBucketTable().get(
+        omMetadataManager.getBucketKey(volumeName, bucketName))
+        .getUsedNamespace(), omMetrics.getNumKeys());
 
     // Add the key to key table
     OmDirectoryInfo omDirInfo = getDirInfo("c/d/e");
@@ -125,6 +158,24 @@ public class TestOMFileCreateRequestWithFSO extends TestOMFileCreateRequest {
     testNonRecursivePath(key, false, false, true);
   }
 
+  @Test
+  public void testCreateFileInheritParentDefaultAcls()
+      throws Exception {
+    super.testCreateFileInheritParentDefaultAcls();
+  }
+
+  @Test
+  public void testValidateAndUpdateCacheWithSnapshotReservedWord()
+      throws Exception {
+    String[] validPaths = {
+        "a/b/" + OM_SNAPSHOT_INDICATOR + "c/d",
+        "a/b/c/" + OM_SNAPSHOT_INDICATOR + "/d"
+    };
+    for (String validPath : validPaths) {
+      testNonRecursivePath(validPath, false, true, false);
+    }
+  }
+
   @Override
   protected OmKeyInfo verifyPathInOpenKeyTable(String key, long id,
                                              boolean doAssert)
@@ -144,7 +195,7 @@ public class TestOMFileCreateRequestWithFSO extends TestOMFileCreateRequest {
             omMetadataManager.getOpenKeyTable(getBucketLayout())
                 .get(dbOpenFileName);
         if (doAssert) {
-          Assert.assertNotNull("Invalid key!", omKeyInfo);
+          Assertions.assertNotNull(omKeyInfo, "Invalid key!");
         }
         return omKeyInfo;
       } else {
@@ -157,7 +208,7 @@ public class TestOMFileCreateRequestWithFSO extends TestOMFileCreateRequest {
       }
     }
     if (doAssert) {
-      Assert.fail("Invalid key!");
+      Assertions.fail("Invalid key!");
     }
     return null;
   }
