@@ -21,21 +21,31 @@ package org.apache.hadoop.ozone.om.request.key;
 import java.io.IOException;
 import java.nio.file.Path;
 import java.nio.file.Paths;
+import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.List;
 import java.util.UUID;
+import java.util.stream.Collectors;
+import java.util.Map;
+import java.util.HashMap;
 
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.ozone.OzoneAcl;
 import org.apache.hadoop.ozone.OzoneConsts;
+import org.apache.hadoop.ozone.om.PrefixManager;
+import org.apache.hadoop.ozone.om.PrefixManagerImpl;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
+import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.lock.OzoneLockProvider;
 import org.apache.hadoop.ozone.om.request.OMRequestTestUtils;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.MethodSource;
 
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
@@ -47,26 +57,26 @@ import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos
     .KeyArgs;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos
     .OMRequest;
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
+import org.junit.jupiter.params.provider.ValueSource;
 
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.THREE;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType.EC;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType.RATIS;
+import static org.apache.hadoop.ozone.OzoneConsts.OM_KEY_PREFIX;
+import static org.apache.hadoop.ozone.OzoneConsts.OM_SNAPSHOT_INDICATOR;
 import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_ENABLE_FILESYSTEM_PATHS;
 import static org.apache.hadoop.ozone.om.request.OMRequestTestUtils.addVolumeAndBucketToDB;
 import static org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status.NOT_A_FILE;
 import static org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status.OK;
-import static org.junit.Assert.fail;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.fail;
 import static org.mockito.Mockito.when;
 
 /**
- * Tests OMCreateKeyRequest class.
+ * This class tests the OM Key Create Request.
  */
-@RunWith(Parameterized.class)
 public class TestOMKeyCreateRequest extends TestOMKeyRequest {
 
-  @Parameterized.Parameters
   public static Collection<Object[]> data() {
     return Arrays.asList(
         new Object[]{true, true},
@@ -75,22 +85,6 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
         new Object[]{false, false});
   }
 
-  public TestOMKeyCreateRequest(boolean setKeyPathLock,
-                                boolean setFileSystemPaths) {
-    // Ignored. Actual init done in initParam().
-    // This empty constructor is still required to avoid argument exception.
-  }
-
-  @Parameterized.BeforeParam
-  public static void initParam(boolean setKeyPathLock,
-                               boolean setFileSystemPaths) {
-    keyPathLockEnabled = setKeyPathLock;
-    enableFileSystemPaths = setFileSystemPaths;
-  }
-
-  private static boolean keyPathLockEnabled;
-  private static boolean enableFileSystemPaths;
-
   @Test
   public void testPreExecuteWithNormalKey() throws Exception {
     ReplicationConfig ratis3Config =
@@ -112,7 +106,7 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
   }
 
   private void preExecuteTest(boolean isMultipartKey, int partNumber,
-      ReplicationConfig repConfig) throws Exception {
+                              ReplicationConfig repConfig) throws Exception {
     long scmBlockSize = ozoneManager.getScmBlockSize();
     for (int i = 0; i <= repConfig.getRequiredNodes(); i++) {
       doPreExecute(createKeyRequest(isMultipartKey, partNumber,
@@ -122,16 +116,28 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     }
   }
 
-
   @Test
-  public void testValidateAndUpdateCache() throws Exception {
+  public void preExecuteRejectsInvalidReplication() {
+    ECReplicationConfig invalidReplication = new ECReplicationConfig(1, 2);
+    OMException e = assertThrows(OMException.class,
+        () -> preExecuteTest(false, 0, invalidReplication));
+
+    Assertions.assertEquals(OMException.ResultCodes.INVALID_REQUEST,
+        e.getResult());
+  }
+
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testValidateAndUpdateCache(
+      boolean setKeyPathLock, boolean setFileSystemPaths) throws Exception {
     when(ozoneManager.getOzoneLockProvider()).thenReturn(
-        new OzoneLockProvider(keyPathLockEnabled, enableFileSystemPaths));
+        new OzoneLockProvider(setKeyPathLock, setFileSystemPaths));
+
     OMRequest modifiedOmRequest =
         doPreExecute(createKeyRequest(false, 0));
 
     OMKeyCreateRequest omKeyCreateRequest =
-            getOMKeyCreateRequest(modifiedOmRequest);
+        getOMKeyCreateRequest(modifiedOmRequest);
 
     // Add volume and bucket entries to DB.
     addVolumeAndBucketToDB(volumeName, bucketName,
@@ -143,25 +149,26 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
 
     // Before calling
     OmKeyInfo omKeyInfo =
-        omMetadataManager.getOpenKeyTable(omKeyCreateRequest.getBucketLayout())
+        omMetadataManager.getOpenKeyTable(
+                omKeyCreateRequest.getBucketLayout())
             .get(openKey);
 
-    Assert.assertNull(omKeyInfo);
+    Assertions.assertNull(omKeyInfo);
 
     OMClientResponse omKeyCreateResponse =
-        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L,
-            ozoneManagerDoubleBufferHelper);
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L);
 
     checkResponse(modifiedOmRequest, omKeyCreateResponse, id, false,
         omKeyCreateRequest.getBucketLayout());
 
     // Network returns only latest version.
-    Assert.assertEquals(1, omKeyCreateResponse.getOMResponse()
+    Assertions.assertEquals(1, omKeyCreateResponse.getOMResponse()
         .getCreateKeyResponse().getKeyInfo().getKeyLocationListCount());
 
     // Disk should have 1 version, as it is fresh key create.
-    Assert.assertEquals(1,
-        omMetadataManager.getOpenKeyTable(omKeyCreateRequest.getBucketLayout())
+    Assertions.assertEquals(1,
+        omMetadataManager.getOpenKeyTable(
+                omKeyCreateRequest.getBucketLayout())
             .get(openKey).getKeyLocationVersions().size());
 
     // Write to DB like key commit.
@@ -179,36 +186,63 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
 
     // Before calling
     omKeyInfo =
-        omMetadataManager.getOpenKeyTable(omKeyCreateRequest.getBucketLayout())
+        omMetadataManager.getOpenKeyTable(
+                omKeyCreateRequest.getBucketLayout())
             .get(openKey);
-    Assert.assertNull(omKeyInfo);
+    Assertions.assertNull(omKeyInfo);
 
     omKeyCreateRequest =
         getOMKeyCreateRequest(modifiedOmRequest);
 
     omKeyCreateResponse =
-        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 101L,
-            ozoneManagerDoubleBufferHelper);
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 101L);
 
     checkResponse(modifiedOmRequest, omKeyCreateResponse, id, true,
         omKeyCreateRequest.getBucketLayout());
 
     // Network returns only latest version
-    Assert.assertEquals(1, omKeyCreateResponse.getOMResponse()
+    Assertions.assertEquals(1, omKeyCreateResponse.getOMResponse()
         .getCreateKeyResponse().getKeyInfo().getKeyLocationListCount());
 
     // Disk should have 1 versions when bucket versioning is off.
-    Assert.assertEquals(1,
-        omMetadataManager.getOpenKeyTable(omKeyCreateRequest.getBucketLayout())
+    Assertions.assertEquals(1,
+        omMetadataManager.getOpenKeyTable(
+                omKeyCreateRequest.getBucketLayout())
             .get(openKey).getKeyLocationVersions().size());
 
   }
 
-  private void checkResponse(OMRequest modifiedOmRequest,
-      OMClientResponse omKeyCreateResponse, long id, boolean override,
-      BucketLayout bucketLayout) throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testValidateAndUpdateCacheWithNamespaceQuotaExceeded(
+      boolean setKeyPathLock, boolean setFileSystemPaths)throws Exception {
+    when(ozoneManager.getOzoneLockProvider()).thenReturn(
+        new OzoneLockProvider(setKeyPathLock, setFileSystemPaths));
+    OMRequest modifiedOmRequest =
+        doPreExecute(createKeyRequest(false, 0, "test/" + keyName));
 
-    Assert.assertEquals(OK,
+    // test with FSO type
+    OMKeyCreateRequest omKeyCreateRequest = getOMKeyCreateRequest(
+        modifiedOmRequest, BucketLayout.FILE_SYSTEM_OPTIMIZED);
+
+    // create bucket with quota limit 1
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, omMetadataManager,
+        OmBucketInfo.newBuilder().setVolumeName(volumeName)
+            .setBucketName(bucketName)
+            .setBucketLayout(BucketLayout.FILE_SYSTEM_OPTIMIZED)
+            .setQuotaInNamespace(1));
+
+    OMClientResponse omKeyCreateResponse =
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L);
+    Assertions.assertSame(omKeyCreateResponse.getOMResponse().getStatus(),
+        OzoneManagerProtocolProtos.Status.QUOTA_EXCEEDED);
+  }
+
+  private void checkResponse(
+      OMRequest modifiedOmRequest, OMClientResponse omKeyCreateResponse,
+      long id, boolean override, BucketLayout bucketLayout) throws Exception {
+
+    Assertions.assertEquals(OK,
         omKeyCreateResponse.getOMResponse().getStatus());
 
     String openKey = getOpenKey(id);
@@ -218,26 +252,26 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     OmKeyInfo omKeyInfo =
         omMetadataManager.getOpenKeyTable(bucketLayout).get(openKey);
 
-    Assert.assertNotNull(omKeyInfo);
-    Assert.assertNotNull(omKeyInfo.getLatestVersionLocations());
+    Assertions.assertNotNull(omKeyInfo);
+    Assertions.assertNotNull(omKeyInfo.getLatestVersionLocations());
 
     // As our data size is 100, and scmBlockSize is default to 1000, so we
     // shall have only one block.
     List<OmKeyLocationInfo> omKeyLocationInfoList =
         omKeyInfo.getLatestVersionLocations().getLocationList();
-    Assert.assertEquals(1, omKeyLocationInfoList.size());
+    Assertions.assertEquals(1, omKeyLocationInfoList.size());
 
     OmKeyLocationInfo omKeyLocationInfo = omKeyLocationInfoList.get(0);
 
     // Check modification time
-    Assert.assertEquals(modifiedOmRequest.getCreateKeyRequest()
+    Assertions.assertEquals(modifiedOmRequest.getCreateKeyRequest()
         .getKeyArgs().getModificationTime(), omKeyInfo.getModificationTime());
 
     if (!override) {
-      Assert.assertEquals(omKeyInfo.getModificationTime(),
+      Assertions.assertEquals(omKeyInfo.getModificationTime(),
           omKeyInfo.getCreationTime());
     } else {
-      Assert.assertNotEquals(omKeyInfo.getModificationTime(),
+      Assertions.assertNotEquals(omKeyInfo.getModificationTime(),
           omKeyInfo.getCreationTime());
     }
 
@@ -245,23 +279,24 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     OzoneManagerProtocolProtos.KeyLocation keyLocation =
         modifiedOmRequest.getCreateKeyRequest().getKeyArgs().getKeyLocations(0);
 
-    Assert.assertEquals(keyLocation.getBlockID().getContainerBlockID()
+    Assertions.assertEquals(keyLocation.getBlockID().getContainerBlockID()
         .getContainerID(), omKeyLocationInfo.getContainerID());
-    Assert.assertEquals(keyLocation.getBlockID().getContainerBlockID()
+    Assertions.assertEquals(keyLocation.getBlockID().getContainerBlockID()
         .getLocalID(), omKeyLocationInfo.getLocalID());
   }
 
-  @Test
-  public void testValidateAndUpdateCacheWithNoSuchMultipartUploadError()
-      throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testValidateAndUpdateCacheWithNoSuchMultipartUploadError(
+      boolean setKeyPathLock, boolean setFileSystemPaths) throws Exception {
     when(ozoneManager.getOzoneLockProvider()).thenReturn(
-        new OzoneLockProvider(keyPathLockEnabled, enableFileSystemPaths));
+        new OzoneLockProvider(setKeyPathLock, setFileSystemPaths));
     int partNumber = 1;
     OMRequest modifiedOmRequest =
         doPreExecute(createKeyRequest(true, partNumber));
 
     OMKeyCreateRequest omKeyCreateRequest =
-            getOMKeyCreateRequest(modifiedOmRequest);
+        getOMKeyCreateRequest(modifiedOmRequest);
 
     // Add volume and bucket entries to DB.
     addVolumeAndBucketToDB(volumeName, bucketName,
@@ -276,13 +311,12 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     OmKeyInfo omKeyInfo =
         omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
 
-    Assert.assertNull(omKeyInfo);
+    Assertions.assertNull(omKeyInfo);
 
     OMClientResponse omKeyCreateResponse =
-        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L,
-            ozoneManagerDoubleBufferHelper);
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L);
 
-    Assert.assertEquals(
+    Assertions.assertEquals(
         OzoneManagerProtocolProtos.Status.NO_SUCH_MULTIPART_UPLOAD_ERROR,
         omKeyCreateResponse.getOMResponse().getStatus());
 
@@ -291,20 +325,22 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     omKeyInfo =
         omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
 
-    Assert.assertNull(omKeyInfo);
+    Assertions.assertNull(omKeyInfo);
   }
 
 
 
-  @Test
-  public void testValidateAndUpdateCacheWithVolumeNotFound() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testValidateAndUpdateCacheWithVolumeNotFound(
+      boolean setKeyPathLock, boolean setFileSystemPaths) throws Exception {
     when(ozoneManager.getOzoneLockProvider()).thenReturn(
-        new OzoneLockProvider(keyPathLockEnabled, enableFileSystemPaths));
+        new OzoneLockProvider(setKeyPathLock, setFileSystemPaths));
     OMRequest modifiedOmRequest =
         doPreExecute(createKeyRequest(false, 0));
 
     OMKeyCreateRequest omKeyCreateRequest =
-            getOMKeyCreateRequest(modifiedOmRequest);
+        getOMKeyCreateRequest(modifiedOmRequest);
 
 
     long id = modifiedOmRequest.getCreateKeyRequest().getClientID();
@@ -317,13 +353,12 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     OmKeyInfo omKeyInfo =
         omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
 
-    Assert.assertNull(omKeyInfo);
+    Assertions.assertNull(omKeyInfo);
 
     OMClientResponse omKeyCreateResponse =
-        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L,
-            ozoneManagerDoubleBufferHelper);
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L);
 
-    Assert.assertEquals(OzoneManagerProtocolProtos.Status.VOLUME_NOT_FOUND,
+    Assertions.assertEquals(OzoneManagerProtocolProtos.Status.VOLUME_NOT_FOUND,
         omKeyCreateResponse.getOMResponse().getStatus());
 
 
@@ -331,21 +366,23 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     omKeyInfo =
         omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
 
-    Assert.assertNull(omKeyInfo);
+    Assertions.assertNull(omKeyInfo);
 
   }
 
 
-  @Test
-  public void testValidateAndUpdateCacheWithBucketNotFound() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testValidateAndUpdateCacheWithBucketNotFound(
+      boolean setKeyPathLock, boolean setFileSystemPaths) throws Exception {
     when(ozoneManager.getOzoneLockProvider()).thenReturn(
-        new OzoneLockProvider(keyPathLockEnabled, enableFileSystemPaths));
+        new OzoneLockProvider(setKeyPathLock, setFileSystemPaths));
     OMRequest modifiedOmRequest =
         doPreExecute(createKeyRequest(
             false, 0));
 
     OMKeyCreateRequest omKeyCreateRequest =
-            getOMKeyCreateRequest(modifiedOmRequest);
+        getOMKeyCreateRequest(modifiedOmRequest);
 
 
     long id = modifiedOmRequest.getCreateKeyRequest().getClientID();
@@ -359,13 +396,12 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     OmKeyInfo omKeyInfo =
         omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
 
-    Assert.assertNull(omKeyInfo);
+    Assertions.assertNull(omKeyInfo);
 
     OMClientResponse omKeyCreateResponse =
-        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L,
-            ozoneManagerDoubleBufferHelper);
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L);
 
-    Assert.assertEquals(OzoneManagerProtocolProtos.Status.BUCKET_NOT_FOUND,
+    Assertions.assertEquals(OzoneManagerProtocolProtos.Status.BUCKET_NOT_FOUND,
         omKeyCreateResponse.getOMResponse().getStatus());
 
 
@@ -373,10 +409,57 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     omKeyInfo =
         omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
 
-    Assert.assertNull(omKeyInfo);
+    Assertions.assertNull(omKeyInfo);
 
   }
 
+
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testValidateAndUpdateCacheWithInvalidPath(
+      boolean setKeyPathLock, boolean setFileSystemPaths) throws Exception {
+    PrefixManager prefixManager = new PrefixManagerImpl(
+        ozoneManager.getMetadataManager(), true);
+    when(ozoneManager.getPrefixManager()).thenReturn(prefixManager);
+    when(ozoneManager.getOzoneLockProvider()).thenReturn(
+        new OzoneLockProvider(setKeyPathLock, setFileSystemPaths));
+    OMRequest modifiedOmRequest =
+        doPreExecute(createKeyRequest(
+            false, 0, String.valueOf('\u0000')));
+
+    OMKeyCreateRequest omKeyCreateRequest =
+        getOMKeyCreateRequest(modifiedOmRequest);
+
+
+    long id = modifiedOmRequest.getCreateKeyRequest().getClientID();
+
+    String openKey = getOpenKey(id);
+
+    // Add volume and bucket entries to DB.
+    addVolumeAndBucketToDB(volumeName, bucketName,
+        omMetadataManager, getBucketLayout());
+
+    // Before calling
+    OmKeyInfo omKeyInfo =
+        omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
+
+    Assertions.assertNull(omKeyInfo);
+
+    OMClientResponse omKeyCreateResponse =
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L);
+
+    Assertions.assertEquals(OzoneManagerProtocolProtos.Status.INVALID_PATH,
+        omKeyCreateResponse.getOMResponse().getStatus());
+
+
+
+    // As We got an error, openKey Table should not have entry.
+    omKeyInfo =
+        omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
+
+    Assertions.assertNull(omKeyInfo);
+
+  }
   /**
    * This method calls preExecute and verify the modified request.
    * @param originalOMRequest
@@ -386,17 +469,17 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
   private OMRequest doPreExecute(OMRequest originalOMRequest) throws Exception {
 
     OMKeyCreateRequest omKeyCreateRequest =
-            getOMKeyCreateRequest(originalOMRequest);
+        getOMKeyCreateRequest(originalOMRequest);
 
     OMRequest modifiedOmRequest =
         omKeyCreateRequest.preExecute(ozoneManager);
 
-    Assert.assertEquals(originalOMRequest.getCmdType(),
+    Assertions.assertEquals(originalOMRequest.getCmdType(),
         modifiedOmRequest.getCmdType());
-    Assert.assertEquals(originalOMRequest.getClientId(),
+    Assertions.assertEquals(originalOMRequest.getClientId(),
         modifiedOmRequest.getClientId());
 
-    Assert.assertTrue(modifiedOmRequest.hasCreateKeyRequest());
+    Assertions.assertTrue(modifiedOmRequest.hasCreateKeyRequest());
 
     CreateKeyRequest createKeyRequest =
         modifiedOmRequest.getCreateKeyRequest();
@@ -409,12 +492,12 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
         (keyArgs.getDataSize() - 1) / (blockSize * dataGroupSize) + 1);
 
     // Time should be set
-    Assert.assertTrue(keyArgs.getModificationTime() > 0);
+    Assertions.assertTrue(keyArgs.getModificationTime() > 0);
 
 
     // Client ID should be set.
-    Assert.assertTrue(createKeyRequest.hasClientID());
-    Assert.assertTrue(createKeyRequest.getClientID() > 0);
+    Assertions.assertTrue(createKeyRequest.hasClientID());
+    Assertions.assertTrue(createKeyRequest.getClientID() > 0);
 
 
     if (!originalOMRequest.getCreateKeyRequest().getKeyArgs()
@@ -423,21 +506,21 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
       List<OzoneManagerProtocolProtos.KeyLocation> keyLocations =
           keyArgs.getKeyLocationsList();
       // KeyLocation should be set.
-      Assert.assertEquals(preAllocatedBlocks, keyLocations.size());
-      Assert.assertEquals(CONTAINER_ID,
+      Assertions.assertEquals(preAllocatedBlocks, keyLocations.size());
+      Assertions.assertEquals(CONTAINER_ID,
           keyLocations.get(0).getBlockID().getContainerBlockID()
               .getContainerID());
-      Assert.assertEquals(LOCAL_ID,
+      Assertions.assertEquals(LOCAL_ID,
           keyLocations.get(0).getBlockID().getContainerBlockID()
               .getLocalID());
-      Assert.assertTrue(keyLocations.get(0).hasPipeline());
+      Assertions.assertTrue(keyLocations.get(0).hasPipeline());
 
-      Assert.assertEquals(0, keyLocations.get(0).getOffset());
+      Assertions.assertEquals(0, keyLocations.get(0).getOffset());
 
-      Assert.assertEquals(scmBlockSize, keyLocations.get(0).getLength());
+      Assertions.assertEquals(scmBlockSize, keyLocations.get(0).getLength());
     } else {
       // We don't create blocks for multipart key in createKey preExecute.
-      Assert.assertEquals(0, keyArgs.getKeyLocationsList().size());
+      Assertions.assertEquals(0, keyArgs.getKeyLocationsList().size());
     }
 
     return modifiedOmRequest;
@@ -452,12 +535,12 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
    */
 
   @SuppressWarnings("parameterNumber")
-  private OMRequest createKeyRequest(boolean isMultipartKey, int partNumber) {
+  protected OMRequest createKeyRequest(boolean isMultipartKey, int partNumber) {
     return createKeyRequest(isMultipartKey, partNumber, keyName);
   }
 
   private OMRequest createKeyRequest(boolean isMultipartKey, int partNumber,
-      String keyName) {
+                                     String keyName) {
 
     KeyArgs.Builder keyArgs = KeyArgs.newBuilder()
         .setVolumeName(volumeName).setBucketName(bucketName)
@@ -478,8 +561,9 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
         .setCreateKeyRequest(createKeyRequest).build();
   }
 
-  private OMRequest createKeyRequest(boolean isMultipartKey, int partNumber,
-      long keyLength, ReplicationConfig repConfig) {
+  private OMRequest createKeyRequest(
+      boolean isMultipartKey, int partNumber, long keyLength,
+      ReplicationConfig repConfig) {
 
     KeyArgs.Builder keyArgs = KeyArgs.newBuilder()
         .setVolumeName(volumeName).setBucketName(bucketName)
@@ -508,15 +592,16 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
         .setCreateKeyRequest(createKeyRequest).build();
   }
 
-  @Test
-  public void testKeyCreateWithFileSystemPathsEnabled() throws Exception {
-
+  @ParameterizedTest
+  @ValueSource(booleans =  {true, false})
+  public void testKeyCreateWithFileSystemPathsEnabled(
+      boolean setKeyPathLock) throws Exception {
     OzoneConfiguration configuration = getOzoneConfiguration();
     configuration.setBoolean(OZONE_OM_ENABLE_FILESYSTEM_PATHS, true);
     when(ozoneManager.getConfiguration()).thenReturn(configuration);
     when(ozoneManager.getEnableFileSystemPaths()).thenReturn(true);
     when(ozoneManager.getOzoneLockProvider()).thenReturn(
-        new OzoneLockProvider(keyPathLockEnabled, true));
+        new OzoneLockProvider(setKeyPathLock, true));
 
     // Add volume and bucket entries to DB.
     addVolumeAndBucketToDB(volumeName, bucketName,
@@ -592,6 +677,119 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
 
   }
 
+  @Test
+  public void testPreExecuteWithInvalidKeyPrefix() throws Exception {
+    Map<String, String> invalidKeyScenarios = new HashMap<String, String>() {
+      {
+        put(OM_SNAPSHOT_INDICATOR + "/" + keyName,
+            "Cannot create key under path reserved for snapshot: "
+                + OM_SNAPSHOT_INDICATOR + OM_KEY_PREFIX);
+        put(OM_SNAPSHOT_INDICATOR + "/a/" + keyName,
+            "Cannot create key under path reserved for snapshot: "
+                + OM_SNAPSHOT_INDICATOR + OM_KEY_PREFIX);
+        put(OM_SNAPSHOT_INDICATOR + "/a/b" + keyName,
+            "Cannot create key under path reserved for snapshot: "
+                + OM_SNAPSHOT_INDICATOR + OM_KEY_PREFIX);
+        put(OM_SNAPSHOT_INDICATOR,
+            "Cannot create key with reserved name: " + OM_SNAPSHOT_INDICATOR);
+      }
+    };
+
+    for (Map.Entry<String, String> entry : invalidKeyScenarios.entrySet()) {
+      String invalidKeyName = entry.getKey();
+      String expectedErrorMessage = entry.getValue();
+
+      KeyArgs.Builder keyArgs = KeyArgs.newBuilder()
+          .setVolumeName(volumeName).setBucketName(bucketName)
+          .setKeyName(invalidKeyName);
+
+      OzoneManagerProtocolProtos.CreateKeyRequest createKeyRequest =
+          CreateKeyRequest.newBuilder().setKeyArgs(keyArgs).build();
+
+      OMRequest omRequest = OMRequest.newBuilder()
+          .setCmdType(OzoneManagerProtocolProtos.Type.CreateKey)
+          .setClientId(UUID.randomUUID().toString())
+          .setCreateKeyRequest(createKeyRequest).build();
+
+      OMException ex = Assertions.assertThrows(OMException.class,
+          () -> getOMKeyCreateRequest(omRequest).preExecute(ozoneManager)
+      );
+      Assertions.assertTrue(ex.getMessage().contains(expectedErrorMessage));
+    }
+  }
+
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testKeyCreateInheritParentDefaultAcls(
+      boolean setKeyPathLock, boolean setFileSystemPaths) throws Exception {
+    when(ozoneManager.getOzoneLockProvider()).thenReturn(
+        new OzoneLockProvider(setKeyPathLock, setFileSystemPaths));
+
+    List<OzoneAcl> acls = new ArrayList<>();
+    acls.add(OzoneAcl.parseAcl("user:newUser:rw[DEFAULT]"));
+    acls.add(OzoneAcl.parseAcl("user:noInherit:rw"));
+    acls.add(OzoneAcl.parseAcl("group:newGroup:rwl[DEFAULT]"));
+
+    // create bucket with DEFAULT acls
+    OMRequestTestUtils.addVolumeAndBucketToDB(volumeName, omMetadataManager,
+        OmBucketInfo.newBuilder().setVolumeName(volumeName)
+            .setBucketName(bucketName)
+            .setBucketLayout(getBucketLayout())
+            .setAcls(acls));
+
+    // Verify bucket has DEFAULT acls.
+    String bucketKey = omMetadataManager.getBucketKey(volumeName, bucketName);
+    List<OzoneAcl> bucketAcls = omMetadataManager.getBucketTable()
+        .get(bucketKey).getAcls();
+    Assertions.assertEquals(acls, bucketAcls);
+
+    // create file inherit bucket DEFAULT acls
+    OMRequest modifiedOmRequest =
+        doPreExecute(createKeyRequest(false, 0));
+
+    OMKeyCreateRequest omKeyCreateRequest =
+        getOMKeyCreateRequest(modifiedOmRequest);
+
+    long id = modifiedOmRequest.getCreateKeyRequest().getClientID();
+    String openKey = getOpenKey(id);
+
+    OMClientResponse omKeyCreateResponse =
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 100L);
+    checkResponse(modifiedOmRequest, omKeyCreateResponse, id, false,
+        omKeyCreateRequest.getBucketLayout());
+
+    OmKeyInfo omKeyInfo =
+        omMetadataManager.getOpenKeyTable(getBucketLayout()).get(openKey);
+
+    verifyKeyInheritAcls(omKeyInfo.getAcls(), bucketAcls);
+
+  }
+
+  /**
+   * Leaf file has ACCESS scope acls which inherited
+   * from parent DEFAULT acls.
+   */
+  private void verifyKeyInheritAcls(List<OzoneAcl> keyAcls,
+                                    List<OzoneAcl> bucketAcls) {
+
+    List<OzoneAcl> parentDefaultAcl = bucketAcls.stream()
+        .filter(acl -> acl.getAclScope() == OzoneAcl.AclScope.DEFAULT)
+        .collect(Collectors.toList());
+
+    OzoneAcl parentAccessAcl = bucketAcls.stream()
+        .filter(acl -> acl.getAclScope() == OzoneAcl.AclScope.ACCESS)
+        .findAny().orElse(null);
+
+    // Should inherit parent DEFAULT Acls
+    Assertions.assertEquals(parentDefaultAcl.stream()
+            .map(acl -> acl.setAclScope(OzoneAcl.AclScope.ACCESS))
+            .collect(Collectors.toList()), keyAcls,
+        "Failed to inherit parent DEFAULT acls!,");
+
+    // Should not inherit parent ACCESS Acls
+    Assertions.assertFalse(keyAcls.contains(parentAccessAcl));
+  }
+
   protected void addToKeyTable(String keyName) throws Exception {
     OMRequestTestUtils.addKeyToTable(false, volumeName, bucketName,
         keyName.substring(1), 0L, RATIS, THREE, omMetadataManager);
@@ -606,9 +804,9 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
       omKeyCreateRequest.preExecute(ozoneManager);
       fail("checkNotAValidPath failed for path" + keyName);
     } catch (IOException ex) {
-      Assert.assertTrue(ex instanceof OMException);
+      Assertions.assertTrue(ex instanceof OMException);
       OMException omException = (OMException) ex;
-      Assert.assertEquals(OMException.ResultCodes.INVALID_KEY_NAME,
+      Assertions.assertEquals(OMException.ResultCodes.INVALID_KEY_NAME,
           omException.getResult());
     }
 
@@ -624,10 +822,9 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     omKeyCreateRequest = getOMKeyCreateRequest(omRequest);
 
     OMClientResponse omClientResponse =
-        omKeyCreateRequest.validateAndUpdateCache(ozoneManager,
-            101L, ozoneManagerDoubleBufferHelper);
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 101L);
 
-    Assert.assertEquals(NOT_A_FILE,
+    Assertions.assertEquals(NOT_A_FILE,
         omClientResponse.getOMResponse().getStatus());
   }
 
@@ -642,16 +839,16 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     omKeyCreateRequest = getOMKeyCreateRequest(omRequest);
 
     OMClientResponse omClientResponse =
-        omKeyCreateRequest.validateAndUpdateCache(ozoneManager,
-            101L, ozoneManagerDoubleBufferHelper);
+        omKeyCreateRequest.validateAndUpdateCache(ozoneManager, 101L);
 
-    Assert.assertEquals(OK, omClientResponse.getOMResponse().getStatus());
+    Assertions.assertEquals(OK, omClientResponse.getOMResponse().getStatus());
 
     checkCreatedPaths(omKeyCreateRequest, omRequest, keyName);
   }
 
-  protected void checkCreatedPaths(OMKeyCreateRequest omKeyCreateRequest,
-      OMRequest omRequest, String keyName) throws Exception {
+  protected void checkCreatedPaths(
+      OMKeyCreateRequest omKeyCreateRequest, OMRequest omRequest,
+      String keyName) throws Exception {
     keyName = omKeyCreateRequest.validateAndNormalizeKey(true, keyName);
     // Check intermediate directories created or not.
     Path keyPath = Paths.get(keyName);
@@ -663,16 +860,16 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
     OmKeyInfo omKeyInfo =
         omMetadataManager.getOpenKeyTable(omKeyCreateRequest.getBucketLayout())
             .get(openKey);
-    Assert.assertNotNull(omKeyInfo);
+    Assertions.assertNotNull(omKeyInfo);
   }
 
   protected long checkIntermediatePaths(Path keyPath) throws Exception {
     // Check intermediate paths are created
     keyPath = keyPath.getParent();
     while (keyPath != null) {
-      Assert.assertNotNull(omMetadataManager.getKeyTable(getBucketLayout()).get(
-          omMetadataManager
-              .getOzoneDirKey(volumeName, bucketName, keyPath.toString())));
+      Assertions.assertNotNull(omMetadataManager.getKeyTable(getBucketLayout())
+          .get(omMetadataManager.getOzoneDirKey(
+              volumeName, bucketName, keyPath.toString())));
       keyPath = keyPath.getParent();
     }
     return -1;
@@ -680,7 +877,7 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
 
   protected String getOpenKey(long id) throws IOException {
     return omMetadataManager.getOpenKey(volumeName, bucketName,
-            keyName, id);
+        keyName, id);
   }
 
   protected String getOzoneKey() throws IOException {
@@ -690,4 +887,10 @@ public class TestOMKeyCreateRequest extends TestOMKeyRequest {
   protected OMKeyCreateRequest getOMKeyCreateRequest(OMRequest omRequest) {
     return new OMKeyCreateRequest(omRequest, BucketLayout.DEFAULT);
   }
+
+  protected OMKeyCreateRequest getOMKeyCreateRequest(
+      OMRequest omRequest, BucketLayout layout) {
+    return new OMKeyCreateRequest(omRequest, layout);
+  }
+
 }
