@@ -20,24 +20,30 @@ package org.apache.hadoop.ozone.security;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Path;
 import java.security.KeyPair;
 import java.security.PrivateKey;
 import java.security.PublicKey;
 import java.security.Signature;
+import java.security.cert.CertPath;
 import java.security.cert.X509Certificate;
 import java.util.HashMap;
 import java.util.Map;
+import java.util.UUID;
 
+import com.google.common.collect.ImmutableList;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
-import org.apache.hadoop.hdds.security.x509.SecurityConfig;
-import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient;
-import org.apache.hadoop.hdds.security.x509.certificate.client.OMCertificateClient;
+import org.apache.hadoop.hdds.security.SecurityConfig;
+import org.apache.hadoop.hdds.security.x509.certificate.utils.CertificateCodec;
 import org.apache.hadoop.hdds.server.ServerUtils;
 import org.apache.hadoop.io.Text;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
+import org.apache.hadoop.ozone.om.OMStorage;
 import org.apache.hadoop.ozone.om.OmMetadataManagerImpl;
 import org.apache.hadoop.ozone.om.OzoneManager;
+import org.apache.hadoop.ozone.om.S3SecretCache;
+import org.apache.hadoop.ozone.om.S3SecretLockedManager;
 import org.apache.hadoop.ozone.om.S3SecretManager;
 import org.apache.hadoop.ozone.om.S3SecretManagerImpl;
 import org.apache.hadoop.ozone.om.exceptions.OMLeaderNotReadyException;
@@ -47,17 +53,23 @@ import org.apache.hadoop.security.AccessControlException;
 import org.apache.hadoop.security.ssl.KeyStoreTestUtil;
 import org.apache.hadoop.security.token.SecretManager;
 import org.apache.hadoop.security.token.Token;
-import org.apache.ozone.test.LambdaTestUtils;
 import org.apache.hadoop.util.Time;
 
 import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_RATIS_ENABLE_KEY;
 import static org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMTokenProto.Type.S3AUTHINFO;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.TemporaryFolder;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.when;
+
+import org.apache.ratis.protocol.RaftPeerId;
+import org.bouncycastle.jcajce.provider.asymmetric.x509.CertificateFactory;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 import org.mockito.Mockito;
 
 /**
@@ -65,23 +77,23 @@ import org.mockito.Mockito;
  */
 public class TestOzoneDelegationTokenSecretManager {
 
+  private static final long TOKEN_MAX_LIFETIME = 1000 * 20;
+  private static final long TOKEN_REMOVER_SCAN_INTERVAL = 1000 * 20;
+
   private OzoneManager om;
   private OzoneDelegationTokenSecretManager secretManager;
   private SecurityConfig securityConfig;
-  private CertificateClient certificateClient;
+  private OMCertificateClient certificateClient;
   private long expiryTime;
   private Text serviceRpcAdd;
   private OzoneConfiguration conf;
   private static final Text TEST_USER = new Text("testUser");
-  private long tokenMaxLifetime = 1000 * 20;
-  private long tokenRemoverScanInterval = 1000 * 20;
   private S3SecretManager s3SecretManager;
-  private String s3Secret = "dbaksbzljandlkandlsd";
 
-  @Rule
-  public TemporaryFolder folder = new TemporaryFolder();
+  @TempDir
+  private Path folder;
 
-  @Before
+  @BeforeEach
   public void setUp() throws Exception {
     conf = createNewTestPath();
     securityConfig = new SecurityConfig(conf);
@@ -89,28 +101,19 @@ public class TestOzoneDelegationTokenSecretManager {
     certificateClient.init();
     expiryTime = Time.monotonicNow() + 60 * 60 * 24;
     serviceRpcAdd = new Text("localhost");
-    final Map<String, String> s3Secrets = new HashMap<>();
-    s3Secrets.put("testuser1", s3Secret);
-    s3Secrets.put("abc", "djakjahkd");
+    final Map<String, S3SecretValue> s3Secrets = new HashMap<>();
+    s3Secrets.put("testuser1",
+        new S3SecretValue("testuser1", "dbaksbzljandlkandlsd"));
+    s3Secrets.put("abc",
+        new S3SecretValue("abc", "djakjahkd"));
     om = Mockito.mock(OzoneManager.class);
-    OMMetadataManager metadataManager = new OmMetadataManagerImpl(conf);
-    s3SecretManager = new S3SecretManagerImpl(conf, metadataManager) {
-      @Override
-      public S3SecretValue getS3Secret(String kerberosID) {
-        if (s3Secrets.containsKey(kerberosID)) {
-          return new S3SecretValue(kerberosID, s3Secrets.get(kerberosID));
-        }
-        return null;
-      }
-
-      @Override
-      public String getS3UserSecretString(String awsAccessKey) {
-        if (s3Secrets.containsKey(awsAccessKey)) {
-          return s3Secrets.get(awsAccessKey);
-        }
-        return null;
-      }
-    };
+    OMMetadataManager metadataManager = new OmMetadataManagerImpl(conf, om);
+    Mockito.when(om.getMetadataManager()).thenReturn(metadataManager);
+    s3SecretManager = new S3SecretLockedManager(
+        new S3SecretManagerImpl(new S3SecretStoreMap(s3Secrets),
+            Mockito.mock(S3SecretCache.class)),
+        metadataManager.getLock()
+    );
   }
 
   private OzoneConfiguration createNewTestPath() throws IOException {
@@ -123,9 +126,9 @@ public class TestOzoneDelegationTokenSecretManager {
     //  OzoneDelegationTokenSecretManager, this test should be updated to
     //  test both ratis enabled and disabled case.
     config.setBoolean(OZONE_OM_RATIS_ENABLE_KEY, false);
-    File newFolder = folder.newFolder();
+    File newFolder = folder.toFile();
     if (!newFolder.exists()) {
-      Assert.assertTrue(newFolder.mkdirs());
+      Assertions.assertTrue(newFolder.mkdirs());
     }
     ServerUtils.setOzoneMetaDirPath(config, newFolder.toString());
     return config;
@@ -134,15 +137,23 @@ public class TestOzoneDelegationTokenSecretManager {
   /**
    * Helper function to create certificate client.
    * */
-  private CertificateClient setupCertificateClient() throws Exception {
+  private OMCertificateClient setupCertificateClient() throws Exception {
     KeyPair keyPair = KeyStoreTestUtil.generateKeyPair("RSA");
-    X509Certificate cert = KeyStoreTestUtil
+    CertificateFactory fact = CertificateCodec.getCertFactory();
+    X509Certificate singleCert = KeyStoreTestUtil
         .generateCertificate("CN=OzoneMaster", keyPair, 30, "SHA256withRSA");
+    CertPath certPath = fact.engineGenerateCertPath(
+        ImmutableList.of(singleCert));
 
-    return new OMCertificateClient(securityConfig) {
+    OMStorage omStorage = mock(OMStorage.class);
+    when(omStorage.getOmCertSerialId()).thenReturn(null);
+    when(omStorage.getClusterID()).thenReturn("test");
+    when(omStorage.getOmId()).thenReturn(UUID.randomUUID().toString());
+    return new OMCertificateClient(
+        securityConfig, null, omStorage, null, "", null, null, null) {
       @Override
-      public X509Certificate getCertificate() {
-        return cert;
+      public CertPath getCertPath() {
+        return certPath;
       }
 
       @Override
@@ -157,28 +168,30 @@ public class TestOzoneDelegationTokenSecretManager {
 
       @Override
       public X509Certificate getCertificate(String serialId) {
-        return cert;
+        return CertificateCodec.firstCertificateFrom(certPath);
       }
     };
   }
 
-  @After
+  @AfterEach
   public void tearDown() throws IOException {
     secretManager.stop();
+    certificateClient.close();
   }
 
   @Test
   public void testLeadershipCheckinRetrievePassword() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
-    Mockito.doThrow(new OMNotLeaderException("Not leader"))
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
+    Mockito.doThrow(new OMNotLeaderException(RaftPeerId.valueOf("om")))
         .when(om).checkLeaderStatus();
     OzoneTokenIdentifier identifier = new OzoneTokenIdentifier();
     try {
       secretManager.retrievePassword(identifier);
     } catch (Exception e) {
-      Assert.assertEquals(SecretManager.InvalidToken.class, e.getClass());
-      Assert.assertEquals(OMNotLeaderException.class, e.getCause().getClass());
+      Assertions.assertEquals(SecretManager.InvalidToken.class, e.getClass());
+      Assertions.assertEquals(OMNotLeaderException.class,
+          e.getCause().getClass());
     }
 
     Mockito.doThrow(new OMLeaderNotReadyException("Leader not ready"))
@@ -186,8 +199,8 @@ public class TestOzoneDelegationTokenSecretManager {
     try {
       secretManager.retrievePassword(identifier);
     } catch (Exception e) {
-      Assert.assertEquals(SecretManager.InvalidToken.class, e.getClass());
-      Assert.assertEquals(OMLeaderNotReadyException.class,
+      Assertions.assertEquals(SecretManager.InvalidToken.class, e.getClass());
+      Assertions.assertEquals(OMLeaderNotReadyException.class,
           e.getCause().getClass());
     }
 
@@ -195,24 +208,24 @@ public class TestOzoneDelegationTokenSecretManager {
     try {
       secretManager.retrievePassword(identifier);
     } catch (Exception e) {
-      Assert.assertEquals(SecretManager.InvalidToken.class, e.getClass());
-      Assert.assertNull(e.getCause());
+      Assertions.assertEquals(SecretManager.InvalidToken.class, e.getClass());
+      Assertions.assertNull(e.getCause());
     }
   }
 
   @Test
   public void testCreateToken() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     Token<OzoneTokenIdentifier> token = secretManager.createToken(TEST_USER,
         TEST_USER, TEST_USER);
     OzoneTokenIdentifier identifier =
         OzoneTokenIdentifier.readProtoBuf(token.getIdentifier());
     // Check basic details.
-    Assert.assertTrue(identifier.getRealUser().equals(TEST_USER));
-    Assert.assertTrue(identifier.getRenewer().equals(TEST_USER));
-    Assert.assertTrue(identifier.getOwner().equals(TEST_USER));
+    assertEquals(identifier.getRealUser(), TEST_USER);
+    assertEquals(identifier.getRenewer(), TEST_USER);
+    assertEquals(identifier.getOwner(), TEST_USER);
 
     validateHash(token.getPassword(), token.getIdentifier());
   }
@@ -220,14 +233,14 @@ public class TestOzoneDelegationTokenSecretManager {
   private void restartSecretManager() throws IOException {
     secretManager.stop();
     secretManager = null;
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
   }
 
   private void testRenewTokenSuccessHelper(boolean restartSecretManager)
       throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     Token<OzoneTokenIdentifier> token = secretManager.createToken(TEST_USER,
         TEST_USER,
@@ -239,7 +252,7 @@ public class TestOzoneDelegationTokenSecretManager {
     }
 
     long renewalTime = secretManager.renewToken(token, TEST_USER.toString());
-    Assert.assertTrue(renewalTime > 0);
+    Assertions.assertTrue(renewalTime > 0);
   }
 
   @Test
@@ -257,15 +270,16 @@ public class TestOzoneDelegationTokenSecretManager {
    */
   @Test
   public void testRenewTokenFailure() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     Token<OzoneTokenIdentifier> token = secretManager.createToken(TEST_USER,
         TEST_USER, TEST_USER);
-    LambdaTestUtils.intercept(AccessControlException.class,
-        "rougeUser tries to renew a token", () -> {
-          secretManager.renewToken(token, "rougeUser");
-        });
+    AccessControlException exception =
+        assertThrows(AccessControlException.class,
+            () -> secretManager.renewToken(token, "rougeUser"));
+    assertTrue(exception.getMessage()
+        .startsWith("rougeUser tries to renew a token"));
   }
 
   /**
@@ -274,16 +288,18 @@ public class TestOzoneDelegationTokenSecretManager {
   @Test
   public void testRenewTokenFailureMaxTime() throws Exception {
     secretManager = createSecretManager(conf, 100,
-        100, tokenRemoverScanInterval);
+        100, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     Token<OzoneTokenIdentifier> token = secretManager.createToken(TEST_USER,
         TEST_USER,
         TEST_USER);
     Thread.sleep(101);
-    LambdaTestUtils.intercept(IOException.class,
-        "testUser tried to renew an expired token", () -> {
-          secretManager.renewToken(token, TEST_USER.toString());
-        });
+
+    IOException ioException =
+        assertThrows(IOException.class,
+            () -> secretManager.renewToken(token, TEST_USER.toString()));
+    assertTrue(ioException.getMessage()
+        .startsWith("testUser tried to renew an expired token"));
   }
 
   /**
@@ -292,33 +308,35 @@ public class TestOzoneDelegationTokenSecretManager {
   @Test
   public void testRenewTokenFailureRenewalTime() throws Exception {
     secretManager = createSecretManager(conf, 1000 * 10,
-        10, tokenRemoverScanInterval);
+        10, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     Token<OzoneTokenIdentifier> token = secretManager.createToken(TEST_USER,
         TEST_USER,
         TEST_USER);
     Thread.sleep(15);
-    LambdaTestUtils.intercept(IOException.class, "is expired", () -> {
-      secretManager.renewToken(token, TEST_USER.toString());
-    });
+
+    IOException ioException =
+        assertThrows(IOException.class,
+            () -> secretManager.renewToken(token, TEST_USER.toString()));
+    assertTrue(ioException.getMessage().contains("is expired"));
   }
 
   @Test
   public void testCreateIdentifier() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     OzoneTokenIdentifier identifier = secretManager.createIdentifier();
     // Check basic details.
-    Assert.assertTrue(identifier.getOwner().equals(new Text("")));
-    Assert.assertTrue(identifier.getRealUser().equals(new Text("")));
-    Assert.assertTrue(identifier.getRenewer().equals(new Text("")));
+    assertEquals(identifier.getOwner(), new Text(""));
+    assertEquals(identifier.getRealUser(), new Text(""));
+    assertEquals(identifier.getRenewer(), new Text(""));
   }
 
   @Test
   public void testCancelTokenSuccess() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     Token<OzoneTokenIdentifier> token = secretManager.createToken(TEST_USER,
         TEST_USER, TEST_USER);
@@ -327,49 +345,51 @@ public class TestOzoneDelegationTokenSecretManager {
 
   @Test
   public void testCancelTokenFailure() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     Token<OzoneTokenIdentifier> token = secretManager.createToken(TEST_USER,
         TEST_USER,
         TEST_USER);
-    LambdaTestUtils.intercept(AccessControlException.class,
-        "rougeUser is not authorized to cancel the token", () -> {
-          secretManager.cancelToken(token, "rougeUser");
-        });
+
+    AccessControlException exception =
+        assertThrows(AccessControlException.class,
+            () -> secretManager.cancelToken(token, "rougeUser"));
+    assertTrue(exception.getMessage()
+        .startsWith("rougeUser is not authorized to cancel the token"));
   }
 
   @Test
   public void testVerifySignatureSuccess() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     OzoneTokenIdentifier id = new OzoneTokenIdentifier();
     id.setOmCertSerialId(certificateClient.getCertificate()
         .getSerialNumber().toString());
     id.setMaxDate(Time.now() + 60 * 60 * 24);
     id.setOwner(new Text("test"));
-    Assert.assertTrue(secretManager.verifySignature(id,
+    Assertions.assertTrue(secretManager.verifySignature(id,
         certificateClient.signData(id.getBytes())));
   }
 
   @Test
   public void testVerifySignatureFailure() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
     OzoneTokenIdentifier id = new OzoneTokenIdentifier();
     // set invalid om cert serial id
     id.setOmCertSerialId("1927393");
     id.setMaxDate(Time.now() + 60 * 60 * 24);
     id.setOwner(new Text("test"));
-    Assert.assertFalse(secretManager.verifySignature(id, id.getBytes()));
+    Assertions.assertFalse(secretManager.verifySignature(id, id.getBytes()));
   }
 
   @Test
   public void testValidateS3AUTHINFOSuccess() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
 
     OzoneTokenIdentifier identifier = new OzoneTokenIdentifier();
@@ -387,8 +407,8 @@ public class TestOzoneDelegationTokenSecretManager {
 
   @Test
   public void testValidateS3AUTHINFOFailure() throws Exception {
-    secretManager = createSecretManager(conf, tokenMaxLifetime,
-        expiryTime, tokenRemoverScanInterval);
+    secretManager = createSecretManager(conf, TOKEN_MAX_LIFETIME,
+        expiryTime, TOKEN_REMOVER_SCAN_INTERVAL);
     secretManager.start(certificateClient);
 
     OzoneTokenIdentifier identifier = new OzoneTokenIdentifier();
@@ -402,9 +422,11 @@ public class TestOzoneDelegationTokenSecretManager {
     identifier.setAwsAccessId("testuser2");
     identifier.setOwner(new Text("testuser2"));
     // Case 1: User don't have aws secret set.
-    LambdaTestUtils.intercept(SecretManager.InvalidToken.class, " No S3 " +
-            "secret found for S3 identifier",
-        () -> secretManager.retrievePassword(identifier));
+    SecretManager.InvalidToken invalidToken =
+        assertThrows(SecretManager.InvalidToken.class,
+            () -> secretManager.retrievePassword(identifier));
+    assertTrue(invalidToken.getMessage()
+        .startsWith("No S3 secret found for S3 identifier"));
 
     // Case 2: Invalid hash in string to sign.
     identifier.setStrToSign("AWS4-HMAC-SHA256\n" +
@@ -412,9 +434,11 @@ public class TestOzoneDelegationTokenSecretManager {
         "20190221/us-west-1/s3/aws4_request\n" +
         "c297c080cce4e0927779823d3fd1f5cae71481a8f7dfc7e18d91851294efc47d" +
         "+invalidhash");
-    LambdaTestUtils.intercept(SecretManager.InvalidToken.class, " No S3 " +
-            "secret found for S3 identifier",
+
+    invalidToken = assertThrows(SecretManager.InvalidToken.class,
         () -> secretManager.retrievePassword(identifier));
+    assertTrue(invalidToken.getMessage()
+        .startsWith("No S3 secret found for S3 identifier"));
 
     // Case 3: Invalid hash in authorization hmac.
     identifier.setSignature("56ec73ba1974f8feda8365c3caef89c5d4a688d" +
@@ -423,9 +447,11 @@ public class TestOzoneDelegationTokenSecretManager {
         "20190221T002037Z\n" +
         "20190221/us-west-1/s3/aws4_request\n" +
         "c297c080cce4e0927779823d3fd1f5cae71481a8f7dfc7e18d91851294efc47d");
-    LambdaTestUtils.intercept(SecretManager.InvalidToken.class, " No S3 " +
-            "secret found for S3 identifier",
+
+    invalidToken = assertThrows(SecretManager.InvalidToken.class,
         () -> secretManager.retrievePassword(identifier));
+    assertTrue(invalidToken.getMessage()
+        .startsWith("No S3 secret found for S3 identifier"));
   }
 
   /**
@@ -437,7 +463,7 @@ public class TestOzoneDelegationTokenSecretManager {
             securityConfig.getProvider());
     rsaSignature.initVerify(certificateClient.getPublicKey());
     rsaSignature.update(identifier);
-    Assert.assertTrue(rsaSignature.verify(hash));
+    Assertions.assertTrue(rsaSignature.verify(hash));
   }
 
   /**
@@ -458,4 +484,4 @@ public class TestOzoneDelegationTokenSecretManager {
         .setOmServiceId(OzoneConsts.OM_SERVICE_ID_DEFAULT)
         .build();
   }
-}
\ No newline at end of file
+}
