@@ -17,8 +17,8 @@
 package org.apache.hadoop.ozone.security.acl;
 
 import org.apache.hadoop.security.UserGroupInformation;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Test;
 
 import java.io.IOException;
 
@@ -32,50 +32,50 @@ public class TestRequestContext {
     RequestContext context = getUserRequestContext("om",
             IAccessAuthorizer.ACLType.CREATE, false, "volume1",
             true);
-    Assert.assertTrue("Wrongly sets recursiveAccessCheck flag value",
-            context.isRecursiveAccessCheck());
+    Assertions.assertTrue(context.isRecursiveAccessCheck(),
+        "Wrongly sets recursiveAccessCheck flag value");
 
     context = getUserRequestContext("om",
             IAccessAuthorizer.ACLType.CREATE, false, "volume1",
             false);
-    Assert.assertFalse("Wrongly sets recursiveAccessCheck flag value",
-            context.isRecursiveAccessCheck());
+    Assertions.assertFalse(context.isRecursiveAccessCheck(),
+        "Wrongly sets recursiveAccessCheck flag value");
 
     context = getUserRequestContext(
             "user1", IAccessAuthorizer.ACLType.CREATE,
             true, "volume1");
-    Assert.assertFalse("Wrongly sets recursiveAccessCheck flag value",
-            context.isRecursiveAccessCheck());
+    Assertions.assertFalse(context.isRecursiveAccessCheck(),
+        "Wrongly sets recursiveAccessCheck flag value");
 
     RequestContext.Builder builder = new RequestContext.Builder();
 
-    Assert.assertFalse("Wrongly sets recursive flag value",
-            builder.build().isRecursiveAccessCheck());
+    Assertions.assertFalse(builder.build().isRecursiveAccessCheck(),
+        "Wrongly sets recursive flag value");
 
     builder.setRecursiveAccessCheck(true);
-    Assert.assertTrue("Wrongly sets recursive flag value",
-            builder.build().isRecursiveAccessCheck());
+    Assertions.assertTrue(builder.build().isRecursiveAccessCheck(),
+        "Wrongly sets recursive flag value");
 
     context = new RequestContext("host", null,
             null, "serviceId",
             IAccessAuthorizer.ACLIdentityType.GROUP,
             IAccessAuthorizer.ACLType.CREATE, "owner");
-    Assert.assertFalse("Wrongly sets recursive flag value",
-            context.isRecursiveAccessCheck());
+    Assertions.assertFalse(context.isRecursiveAccessCheck(),
+        "Wrongly sets recursive flag value");
 
     context = new RequestContext("host", null,
             null, "serviceId",
             IAccessAuthorizer.ACLIdentityType.GROUP,
             IAccessAuthorizer.ACLType.CREATE, "owner", false);
-    Assert.assertFalse("Wrongly sets recursive flag value",
-            context.isRecursiveAccessCheck());
+    Assertions.assertFalse(context.isRecursiveAccessCheck(),
+        "Wrongly sets recursive flag value");
 
     context = new RequestContext("host", null,
             null, "serviceId",
             IAccessAuthorizer.ACLIdentityType.GROUP,
             IAccessAuthorizer.ACLType.CREATE, "owner", true);
-    Assert.assertTrue("Wrongly sets recursive flag value",
-            context.isRecursiveAccessCheck());
+    Assertions.assertTrue(context.isRecursiveAccessCheck(),
+        "Wrongly sets recursive flag value");
   }
 
   private RequestContext getUserRequestContext(String username,
