@@ -17,6 +17,7 @@
  */
 package org.apache.hadoop.fs.ozone;
 
+import com.google.common.annotations.VisibleForTesting;
 import org.apache.hadoop.conf.Configuration;
 import org.apache.hadoop.fs.FsShell;
 import org.apache.hadoop.fs.shell.CommandFactory;
@@ -58,6 +59,8 @@ public class OzoneFsShell extends FsShell {
     // commands, and then this method can be abstract
     if (this.getClass().equals(OzoneFsShell.class)) {
       factory.registerCommands(FsCommand.class);
+      // ozone delete rm command registration supersedes fs delete
+      factory.registerCommands(OzoneFsDelete.class);
     }
   }
 
@@ -78,17 +81,29 @@ public class OzoneFsShell extends FsShell {
     TracingUtil.initTracing("FsShell", conf);
     conf.setQuietMode(false);
     shell.setConf(conf);
-    int res;
-    try {
-      res = ToolRunner.run(shell, argv);
-    } finally {
-      shell.close();
-    }
+    String spanName = "ozone fs " + String.join(" ", argv);
+    int res = TracingUtil.executeInNewSpan(spanName,
+        () -> shell.execute(argv));
     System.exit(res);
   }
 
+  private int execute(String[] argv) throws Exception {
+    try {
+      return ToolRunner.run(this, argv);
+    } finally {
+      close();
+    }
+  }
+
   // TODO: this should be abstract in a base class
   protected static OzoneFsShell newShellInstance() {
     return new OzoneFsShell();
   }
+
+  // for testing purposes, ensure that ozone specific
+  // added fs commands are visible
+  @VisibleForTesting
+  public CommandFactory getCommandFactory() {
+    return commandFactory;
+  }
 }
