@@ -143,6 +143,31 @@ public final class  ReconServerConfigKeys {
       "ozone.recon.scm.connection.request.timeout";
   public static final String
       OZONE_RECON_SCM_CONNECTION_REQUEST_TIMEOUT_DEFAULT = "5s";
+
+  public static final String OZONE_RECON_NSSUMMARY_FLUSH_TO_DB_MAX_THRESHOLD =
+      "ozone.recon.nssummary.flush.db.max.threshold";
+
+  public static final long
+      OZONE_RECON_NSSUMMARY_FLUSH_TO_DB_MAX_THRESHOLD_DEFAULT = 150 * 1000L;
+
+  public static final String
+      OZONE_RECON_CONTAINER_KEY_FLUSH_TO_DB_MAX_THRESHOLD =
+      "ozone.recon.containerkey.flush.db.max.threshold";
+
+  public static final long
+      OZONE_RECON_CONTAINER_KEY_FLUSH_TO_DB_MAX_THRESHOLD_DEFAULT = 150 * 1000L;
+
+  public static final String OZONE_RECON_SCM_SNAPSHOT_TASK_INTERVAL_DELAY =
+      "ozone.recon.scm.snapshot.task.interval.delay";
+
+  public static final String OZONE_RECON_SCM_SNAPSHOT_TASK_INTERVAL_DEFAULT
+      = "24h";
+
+  public static final String OZONE_RECON_SCM_SNAPSHOT_TASK_INITIAL_DELAY =
+      "ozone.recon.scm.snapshot.task.initial.delay";
+
+  public static final String
+      OZONE_RECON_SCM_SNAPSHOT_TASK_INITIAL_DELAY_DEFAULT = "1m";
   /**
    * Private constructor for utility class.
    */
