@@ -20,10 +20,14 @@ package org.apache.hadoop.ozone.recon.fsck;
 
 import java.io.IOException;
 import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Map;
 import java.util.Set;
-import java.util.concurrent.TimeoutException;
+import java.util.concurrent.locks.ReadWriteLock;
+import java.util.concurrent.locks.ReentrantReadWriteLock;
 
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.scm.PlacementPolicy;
@@ -36,6 +40,7 @@ import org.apache.hadoop.hdds.scm.container.common.helpers.ContainerWithPipeline
 import org.apache.hadoop.ozone.common.statemachine.InvalidStateTransitionException;
 import org.apache.hadoop.ozone.recon.persistence.ContainerHealthSchemaManager;
 import org.apache.hadoop.ozone.recon.scm.ReconScmTask;
+import org.apache.hadoop.ozone.recon.spi.ReconContainerMetadataManager;
 import org.apache.hadoop.ozone.recon.spi.StorageContainerServiceProvider;
 import org.apache.hadoop.ozone.recon.tasks.ReconTaskConfig;
 import org.apache.hadoop.util.Time;
@@ -47,6 +52,11 @@ import org.jooq.Cursor;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.hadoop.ozone.recon.ReconConstants.CONTAINER_COUNT;
+import static org.apache.hadoop.ozone.recon.ReconConstants.TOTAL_KEYS;
+import static org.apache.hadoop.ozone.recon.ReconConstants.TOTAL_USED_BYTES;
+
+
 /**
  * Class that scans the list of containers and keeps track of containers with
  * no replicas in a SQL table.
@@ -56,11 +66,15 @@ public class ContainerHealthTask extends ReconScmTask {
   private static final Logger LOG =
       LoggerFactory.getLogger(ContainerHealthTask.class);
 
+  private ReadWriteLock lock = new ReentrantReadWriteLock(true);
+
   private StorageContainerServiceProvider scmClient;
   private ContainerManager containerManager;
   private ContainerHealthSchemaManager containerHealthSchemaManager;
+  private ReconContainerMetadataManager reconContainerMetadataManager;
   private PlacementPolicy placementPolicy;
   private final long interval;
+
   private Set<ContainerInfo> processedContainers = new HashSet<>();
 
   public ContainerHealthTask(
@@ -69,36 +83,23 @@ public class ContainerHealthTask extends ReconScmTask {
       ReconTaskStatusDao reconTaskStatusDao,
       ContainerHealthSchemaManager containerHealthSchemaManager,
       PlacementPolicy placementPolicy,
-      ReconTaskConfig reconTaskConfig) {
+      ReconTaskConfig reconTaskConfig,
+      ReconContainerMetadataManager reconContainerMetadataManager) {
     super(reconTaskStatusDao);
     this.scmClient = scmClient;
     this.containerHealthSchemaManager = containerHealthSchemaManager;
+    this.reconContainerMetadataManager = reconContainerMetadataManager;
     this.placementPolicy = placementPolicy;
     this.containerManager = containerManager;
     interval = reconTaskConfig.getMissingContainerTaskInterval().toMillis();
   }
 
   @Override
-  public synchronized void run() {
+  public void run() {
     try {
       while (canRun()) {
-        wait(interval);
-        long start = Time.monotonicNow();
-        long currentTime = System.currentTimeMillis();
-        long existingCount = processExistingDBRecords(currentTime);
-        LOG.info("Container Health task thread took {} milliseconds to" +
-                " process {} existing database records.",
-            Time.monotonicNow() - start, existingCount);
-        start = Time.monotonicNow();
-        final List<ContainerInfo> containers = containerManager.getContainers();
-        containers.stream()
-            .filter(c -> !processedContainers.contains(c))
-            .forEach(c -> processContainer(c, currentTime));
-        recordSingleRunCompletion();
-        LOG.info("Container Health task thread took {} milliseconds for" +
-                " processing {} containers.", Time.monotonicNow() - start,
-            containers.size());
-        processedContainers.clear();
+        triggerContainerHealthCheck();
+        Thread.sleep(interval);
       }
     } catch (Throwable t) {
       LOG.error("Exception in Missing Container task Thread.", t);
@@ -108,20 +109,103 @@ public class ContainerHealthTask extends ReconScmTask {
     }
   }
 
+  public void triggerContainerHealthCheck() {
+    lock.writeLock().lock();
+    // Map contains all UNHEALTHY STATES as keys and value is another map
+    // with 3 keys (CONTAINER_COUNT, TOTAL_KEYS, TOTAL_USED_BYTES) and value
+    // is count for each of these 3 stats.
+    // E.g. <MISSING, <CONTAINER_COUNT, 1>>, <MISSING, <TOTAL_KEYS, 10>>,
+    // <MISSING, <TOTAL_USED_BYTES, 2048>>,
+    // <EMPTY_MISSING, <CONTAINER_COUNT, 10>>, <EMPTY_MISSING, <TOTAL_KEYS, 2>>,
+    // <EMPTY_MISSING, <TOTAL_USED_BYTES, 2048>>
+    Map<UnHealthyContainerStates, Map<String, Long>>
+        unhealthyContainerStateStatsMap;
+    try {
+      unhealthyContainerStateStatsMap = new HashMap<>(Collections.emptyMap());
+      initializeUnhealthyContainerStateStatsMap(
+          unhealthyContainerStateStatsMap);
+      long start = Time.monotonicNow();
+      long currentTime = System.currentTimeMillis();
+      long existingCount = processExistingDBRecords(currentTime,
+          unhealthyContainerStateStatsMap);
+      LOG.info("Container Health task thread took {} milliseconds to" +
+              " process {} existing database records.",
+          Time.monotonicNow() - start, existingCount);
+      start = Time.monotonicNow();
+      final List<ContainerInfo> containers = containerManager.getContainers();
+      containers.stream()
+          .filter(c -> !processedContainers.contains(c))
+          .forEach(c -> processContainer(c, currentTime,
+              unhealthyContainerStateStatsMap));
+      recordSingleRunCompletion();
+      LOG.info("Container Health task thread took {} milliseconds for" +
+              " processing {} containers.", Time.monotonicNow() - start,
+          containers.size());
+      logUnhealthyContainerStats(unhealthyContainerStateStatsMap);
+      processedContainers.clear();
+    } finally {
+      lock.writeLock().unlock();
+    }
+  }
+
+  private void logUnhealthyContainerStats(
+      Map<UnHealthyContainerStates, Map<String, Long>>
+          unhealthyContainerStateStatsMap) {
+    // If any EMPTY_MISSING containers, then it is possible that such
+    // containers got stuck in the closing state which never got
+    // any replicas created on the datanodes. In this case, we log it as
+    // EMPTY, and insert as EMPTY_MISSING in UNHEALTHY_CONTAINERS table.
+    unhealthyContainerStateStatsMap.entrySet().forEach(stateEntry -> {
+      UnHealthyContainerStates unhealthyContainerState = stateEntry.getKey();
+      Map<String, Long> containerStateStatsMap = stateEntry.getValue();
+      StringBuilder logMsgBuilder =
+          new StringBuilder(unhealthyContainerState.toString());
+      logMsgBuilder.append(" **Container State Stats:** \n\t");
+      containerStateStatsMap.entrySet().forEach(statsEntry -> {
+        logMsgBuilder.append(statsEntry.getKey());
+        logMsgBuilder.append(" -> ");
+        logMsgBuilder.append(statsEntry.getValue());
+        logMsgBuilder.append(" , ");
+      });
+      LOG.info(logMsgBuilder.toString());
+    });
+  }
+
+  private void initializeUnhealthyContainerStateStatsMap(
+      Map<UnHealthyContainerStates, Map<String, Long>>
+          unhealthyContainerStateStatsMap) {
+    unhealthyContainerStateStatsMap.put(
+        UnHealthyContainerStates.MISSING, new HashMap<>());
+    unhealthyContainerStateStatsMap.put(
+        UnHealthyContainerStates.EMPTY_MISSING, new HashMap<>());
+    unhealthyContainerStateStatsMap.put(
+        UnHealthyContainerStates.UNDER_REPLICATED, new HashMap<>());
+    unhealthyContainerStateStatsMap.put(
+        UnHealthyContainerStates.OVER_REPLICATED, new HashMap<>());
+    unhealthyContainerStateStatsMap.put(
+        UnHealthyContainerStates.MIS_REPLICATED, new HashMap<>());
+  }
+
   private ContainerHealthStatus setCurrentContainer(long recordId)
       throws ContainerNotFoundException {
     ContainerInfo container =
         containerManager.getContainer(ContainerID.valueOf(recordId));
     Set<ContainerReplica> replicas =
         containerManager.getContainerReplicas(container.containerID());
-    return new ContainerHealthStatus(container, replicas, placementPolicy);
+    return new ContainerHealthStatus(container, replicas, placementPolicy,
+        reconContainerMetadataManager);
   }
 
-  private void completeProcessingContainer(ContainerHealthStatus container,
-      Set<String> existingRecords, long currentTime) {
+  private void completeProcessingContainer(
+      ContainerHealthStatus container,
+      Set<String> existingRecords,
+      long currentTime,
+      Map<UnHealthyContainerStates, Map<String, Long>>
+          unhealthyContainerStateCountMap) {
     containerHealthSchemaManager.insertUnhealthyContainerRecords(
         ContainerHealthRecords.generateUnhealthyRecords(
-            container, existingRecords, currentTime));
+            container, existingRecords, currentTime,
+            unhealthyContainerStateCountMap));
     processedContainers.add(container.getContainer());
   }
 
@@ -137,9 +221,13 @@ public class ContainerHealthTask extends ReconScmTask {
    * additional records need to be added to the database.
    *
    * @param currentTime Timestamp to place on all records generated by this run
+   * @param unhealthyContainerStateCountMap
    * @return Count of records processed
    */
-  private long processExistingDBRecords(long currentTime) {
+  private long processExistingDBRecords(long currentTime,
+                                        Map<UnHealthyContainerStates,
+                                            Map<String, Long>>
+                                            unhealthyContainerStateCountMap) {
     long recordCount = 0;
     try (Cursor<UnhealthyContainersRecord> cursor =
              containerHealthSchemaManager.getAllUnhealthyRecordsCursor()) {
@@ -154,12 +242,14 @@ public class ContainerHealthTask extends ReconScmTask {
           }
           if (currentContainer.getContainerID() != rec.getContainerId()) {
             completeProcessingContainer(
-                currentContainer, existingRecords, currentTime);
+                currentContainer, existingRecords, currentTime,
+                unhealthyContainerStateCountMap);
             existingRecords.clear();
             currentContainer = setCurrentContainer(rec.getContainerId());
           }
           if (ContainerHealthRecords
-              .retainOrUpdateRecord(currentContainer, rec)) {
+              .retainOrUpdateRecord(currentContainer, rec
+              )) {
             // Check if the missing container is deleted in SCM
             if (currentContainer.isMissing() &&
                 containerDeletedInSCM(currentContainer.getContainer())) {
@@ -180,18 +270,22 @@ public class ContainerHealthTask extends ReconScmTask {
       // Remember to finish processing the last container
       if (currentContainer != null) {
         completeProcessingContainer(
-            currentContainer, existingRecords, currentTime);
+            currentContainer, existingRecords, currentTime,
+            unhealthyContainerStateCountMap);
       }
     }
     return recordCount;
   }
 
-  private void processContainer(ContainerInfo container, long currentTime) {
+  private void processContainer(ContainerInfo container, long currentTime,
+                                Map<UnHealthyContainerStates,
+                                    Map<String, Long>>
+                                      unhealthyContainerStateStatsMap) {
     try {
       Set<ContainerReplica> containerReplicas =
           containerManager.getContainerReplicas(container.containerID());
-      ContainerHealthStatus h = new ContainerHealthStatus(
-          container, containerReplicas, placementPolicy);
+      ContainerHealthStatus h = new ContainerHealthStatus(container,
+          containerReplicas, placementPolicy, reconContainerMetadataManager);
       if (h.isHealthy() || h.isDeleted()) {
         return;
       }
@@ -200,7 +294,8 @@ public class ContainerHealthTask extends ReconScmTask {
         return;
       }
       containerHealthSchemaManager.insertUnhealthyContainerRecords(
-          ContainerHealthRecords.generateUnhealthyRecords(h, currentTime));
+          ContainerHealthRecords.generateUnhealthyRecords(h, currentTime,
+              unhealthyContainerStateStatsMap));
     } catch (ContainerNotFoundException e) {
       LOG.error("Container not found while processing container in Container " +
           "Health task", e);
@@ -229,7 +324,7 @@ public class ContainerHealthTask extends ReconScmTask {
     } catch (InvalidStateTransitionException e) {
       LOG.error("Failed to transition Container state while processing " +
           "container in Container Health task", e);
-    } catch (IOException | TimeoutException e) {
+    } catch (IOException e) {
       LOG.error("Got exception while processing container in" +
           " Container Health task", e);
     }
@@ -252,10 +347,13 @@ public class ContainerHealthTask extends ReconScmTask {
      * If the record is to be retained, the fields in the record for actual
      * replica count, delta and reason will be updated if their counts have
      * changed.
-     * @param container ContainerHealthStatus representing the health state of
-     *                  the container.
-     * @param rec Existing database record from the UnhealthyContainers table.
-     * @return
+     *
+     * @param container ContainerHealthStatus representing the
+     *                  health state of the container.
+     * @param rec       Existing database record from the
+     *                  UnhealthyContainers table.
+     * @return returns true or false if need to retain or update the unhealthy
+     * container record
      */
     public static boolean retainOrUpdateRecord(
         ContainerHealthStatus container, UnhealthyContainersRecord rec) {
@@ -280,8 +378,11 @@ public class ContainerHealthTask extends ReconScmTask {
     }
 
     public static List<UnhealthyContainers> generateUnhealthyRecords(
-        ContainerHealthStatus container, long time) {
-      return generateUnhealthyRecords(container, new HashSet<>(), time);
+        ContainerHealthStatus container, long time,
+        Map<UnHealthyContainerStates, Map<String, Long>>
+            unhealthyContainerStateStatsMap) {
+      return generateUnhealthyRecords(container, new HashSet<>(), time,
+          unhealthyContainerStateStatsMap);
     }
 
     /**
@@ -294,7 +395,9 @@ public class ContainerHealthTask extends ReconScmTask {
      */
     public static List<UnhealthyContainers> generateUnhealthyRecords(
         ContainerHealthStatus container, Set<String> recordForStateExists,
-        long time) {
+        long time,
+        Map<UnHealthyContainerStates, Map<String, Long>>
+            unhealthyContainerStateStatsMap) {
       List<UnhealthyContainers> records = new ArrayList<>();
       if (container.isHealthy() || container.isDeleted()) {
         return records;
@@ -302,9 +405,32 @@ public class ContainerHealthTask extends ReconScmTask {
 
       if (container.isMissing()
           && !recordForStateExists.contains(
-              UnHealthyContainerStates.MISSING.toString())) {
-        records.add(
-            recordForState(container, UnHealthyContainerStates.MISSING, time));
+          UnHealthyContainerStates.MISSING.toString())) {
+        if (!container.isEmpty()) {
+          LOG.info("Non-empty container {} is missing. It has {} " +
+                  "keys and {} bytes used according to SCM metadata. " +
+                  "Please visit Recon's missing container page for a list of " +
+                  "keys (and their metadata) mapped to this container.",
+              container.getContainerID(), container.getNumKeys(),
+              container.getContainer().getUsedBytes());
+          records.add(
+              recordForState(container, UnHealthyContainerStates.MISSING,
+                  time));
+          populateContainerStats(container, UnHealthyContainerStates.MISSING,
+              unhealthyContainerStateStatsMap);
+        } else {
+          if (LOG.isDebugEnabled()) {
+            LOG.debug("Empty container {} is missing. Kindly check the " +
+                "consolidated container stats per UNHEALTHY state logged as " +
+                "starting with **Container State Stats:**");
+          }
+          records.add(
+              recordForState(container, UnHealthyContainerStates.EMPTY_MISSING,
+                  time));
+          populateContainerStats(container,
+              UnHealthyContainerStates.EMPTY_MISSING,
+              unhealthyContainerStateStatsMap);
+        }
         // A container cannot have any other records if it is missing so return
         return records;
       }
@@ -314,6 +440,9 @@ public class ContainerHealthTask extends ReconScmTask {
               UnHealthyContainerStates.UNDER_REPLICATED.toString())) {
         records.add(recordForState(
             container, UnHealthyContainerStates.UNDER_REPLICATED, time));
+        populateContainerStats(container,
+            UnHealthyContainerStates.UNDER_REPLICATED,
+            unhealthyContainerStateStatsMap);
       }
 
       if (container.isOverReplicated()
@@ -321,6 +450,9 @@ public class ContainerHealthTask extends ReconScmTask {
               UnHealthyContainerStates.OVER_REPLICATED.toString())) {
         records.add(recordForState(
             container, UnHealthyContainerStates.OVER_REPLICATED, time));
+        populateContainerStats(container,
+            UnHealthyContainerStates.OVER_REPLICATED,
+            unhealthyContainerStateStatsMap);
       }
 
       if (container.isMisReplicated()
@@ -328,7 +460,11 @@ public class ContainerHealthTask extends ReconScmTask {
               UnHealthyContainerStates.MIS_REPLICATED.toString())) {
         records.add(recordForState(
             container, UnHealthyContainerStates.MIS_REPLICATED, time));
+        populateContainerStats(container,
+            UnHealthyContainerStates.MIS_REPLICATED,
+            unhealthyContainerStateStatsMap);
       }
+
       return records;
     }
 
@@ -422,4 +558,23 @@ public class ContainerHealthTask extends ReconScmTask {
     }
   }
 
+  private static void populateContainerStats(
+      ContainerHealthStatus container,
+      UnHealthyContainerStates unhealthyState,
+      Map<UnHealthyContainerStates, Map<String, Long>>
+          unhealthyContainerStateStatsMap) {
+    if (unhealthyContainerStateStatsMap.containsKey(unhealthyState)) {
+      Map<String, Long> containerStatsMap =
+          unhealthyContainerStateStatsMap.get(unhealthyState);
+      containerStatsMap.compute(CONTAINER_COUNT,
+          (containerCount, value) -> (value == null) ? 1 : (value + 1));
+      containerStatsMap.compute(TOTAL_KEYS,
+          (totalKeyCount, value) -> (value == null) ? container.getNumKeys() :
+              (value + container.getNumKeys()));
+      containerStatsMap.compute(TOTAL_USED_BYTES,
+          (totalUsedBytes, value) -> (value == null) ?
+              container.getContainer().getUsedBytes() :
+              (value + container.getContainer().getUsedBytes()));
+    }
+  }
 }
