@@ -25,6 +25,7 @@ import java.util.UUID;
 import org.apache.hadoop.hdds.annotation.InterfaceStability;
 import org.apache.hadoop.hdds.utils.db.BatchOperation;
 import org.apache.hadoop.hdds.utils.db.RDBBatchOperation;
+import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.recon.api.types.ContainerKeyPrefix;
 import org.apache.hadoop.ozone.recon.api.types.ContainerMetadata;
 import org.apache.hadoop.hdds.utils.db.TableIterator;
@@ -243,6 +244,12 @@ public interface ReconContainerMetadataManager {
    */
   TableIterator getKeyContainerTableIterator() throws IOException;
 
+  /**
+   * Get the entire keyContainerTable.
+   * @return keyContainerTable
+   */
+  Table<KeyPrefixContainer, Integer> getKeyContainerTable();
+
   /**
    * Get the stored key prefixes for the given containerId starting
    * after the given keyPrefix.
