@@ -19,6 +19,8 @@
 package org.apache.hadoop.ozone.recon.spi;
 
 import org.apache.hadoop.hdds.annotation.InterfaceStability;
+import org.apache.hadoop.hdds.utils.db.BatchOperation;
+import org.apache.hadoop.hdds.utils.db.RDBBatchOperation;
 import org.apache.hadoop.ozone.recon.api.types.NSSummary;
 
 import java.io.IOException;
@@ -31,9 +33,16 @@ public interface ReconNamespaceSummaryManager {
 
   void clearNSSummaryTable() throws IOException;
 
+  @Deprecated
   void storeNSSummary(long objectId, NSSummary nsSummary) throws IOException;
 
+  void batchStoreNSSummaries(BatchOperation batch, long objectId,
+                             NSSummary nsSummary) throws IOException;
+
   void deleteNSSummary(long objectId) throws IOException;
 
   NSSummary getNSSummary(long objectId) throws IOException;
+
+  void commitBatchOperation(RDBBatchOperation rdbBatchOperation)
+      throws IOException;
 }
