@@ -32,12 +32,24 @@ import com.google.common.base.Preconditions;
 import com.google.common.primitives.Longs;
 
 /**
- * Codec to encode ContainerKeyPrefix as byte array.
+ * Codec to serialize/deserialize {@link ContainerKeyPrefix}.
  */
-public class ContainerKeyPrefixCodec implements Codec<ContainerKeyPrefix> {
+public final class ContainerKeyPrefixCodec
+    implements Codec<ContainerKeyPrefix> {
 
   private static final String KEY_DELIMITER = "_";
 
+  private static final Codec<ContainerKeyPrefix> INSTANCE =
+      new ContainerKeyPrefixCodec();
+
+  public static Codec<ContainerKeyPrefix> get() {
+    return INSTANCE;
+  }
+
+  private ContainerKeyPrefixCodec() {
+    // singleton
+  }
+
   @Override
   public byte[] toPersistedFormat(ContainerKeyPrefix containerKeyPrefix)
       throws IOException {
@@ -82,7 +94,7 @@ public class ContainerKeyPrefixCodec implements Codec<ContainerKeyPrefix> {
         rawData.length - Long.BYTES,
         rawData.length);
     long version = ByteBuffer.wrap(versionBytes).getLong();
-    return new ContainerKeyPrefix(containerIdFromDB, keyPrefix, version);
+    return ContainerKeyPrefix.get(containerIdFromDB, keyPrefix, version);
   }
 
   @Override
