@@ -17,6 +17,7 @@
  */
 
 package org.apache.hadoop.ozone.recon.tasks;
+import com.google.common.util.concurrent.ThreadFactoryBuilder;
 import org.apache.commons.lang3.tuple.ImmutablePair;
 import org.apache.commons.lang3.tuple.Pair;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
@@ -36,6 +37,7 @@ import java.util.concurrent.Executors;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Callable;
 import java.util.concurrent.Future;
+import java.util.concurrent.ThreadFactory;
 
 /**
  * Task to query data from OMDB and write into Recon RocksDB.
@@ -79,7 +81,8 @@ public class NSSummaryTask implements ReconOmTask {
     this.reconOMMetadataManager = reconOMMetadataManager;
     this.ozoneConfiguration = ozoneConfiguration;
     this.nsSummaryTaskWithFSO = new NSSummaryTaskWithFSO(
-        reconNamespaceSummaryManager, reconOMMetadataManager);
+        reconNamespaceSummaryManager,
+        reconOMMetadataManager, ozoneConfiguration);
     this.nsSummaryTaskWithLegacy = new NSSummaryTaskWithLegacy(
         reconNamespaceSummaryManager,
         reconOMMetadataManager, ozoneConfiguration);
@@ -121,8 +124,11 @@ public class NSSummaryTask implements ReconOmTask {
         .reprocessWithLegacy(reconOMMetadataManager));
 
     List<Future<Boolean>> results;
-    ExecutorService executorService = Executors
-        .newFixedThreadPool(2);
+    ThreadFactory threadFactory = new ThreadFactoryBuilder()
+        .setNameFormat("Recon-NSSummaryTask-%d")
+        .build();
+    ExecutorService executorService = Executors.newFixedThreadPool(2,
+        threadFactory);
     try {
       results = executorService.invokeAll(tasks);
       for (int i = 0; i < results.size(); i++) {
