@@ -23,7 +23,12 @@ import {NotFound} from './views/notFound/notFound';
 import {IRoute} from './types/routes.types';
 import {MissingContainers} from './views/missingContainers/missingContainers';
 import {Insights} from './views/insights/insights';
+import {Om} from './views/insights/om/om';
+
 import {DiskUsage} from './views/diskUsage/diskUsage';
+import {Heatmap} from './views/heatMap/heatmap';
+import {Volumes} from './views/volumes/volumes';
+import {Buckets} from './views/buckets/buckets';
 
 export const routes: IRoute[] = [
   {
@@ -34,6 +39,14 @@ export const routes: IRoute[] = [
     path: '/Datanodes',
     component: Datanodes
   },
+  {
+    path: '/Volumes',
+    component: Volumes
+  },
+  {
+    path: '/Buckets',
+    component: Buckets
+  },
   {
     path: '/Pipelines',
     component: Pipelines
@@ -42,6 +55,10 @@ export const routes: IRoute[] = [
     path: '/Insights',
     component: Insights
   },
+  {
+    path: '/Om',
+    component: Om
+  },
   {
     path: '/MissingContainers',
     component: MissingContainers
@@ -58,6 +75,10 @@ export const routes: IRoute[] = [
     path: '/Containers',
     component: MissingContainers,
   },
+  {
+    path: '/Heatmap',
+    component: Heatmap
+  },
   {
     path: '/:NotFound',
     component: NotFound
