@@ -35,5 +35,37 @@ const showErrorNotification = (title: string, description: string) => {
 
 export const showDataFetchError = (error: string) => {
   const title = 'Error while fetching data';
+  if (error.includes("CanceledError")){
+    error = "Previous request cancelled because context changed"
+  }
   showErrorNotification(title, error);
 };
+
+export const byteToSize = (bytes: number, decimals: number) => {
+  if (bytes === 0) {
+    return '0 Bytes';
+  }
+  const k = 1024;
+  const dm = decimals < 0 ? 0 : decimals;
+  const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB'];
+  const i = Math.floor(Math.log(bytes) / Math.log(k));
+  return  isNaN(i) ? `Not Defined`:`${Number.parseFloat((bytes / (k ** i)).toFixed(dm))} ${sizes[i]}`;
+};
+
+export const nullAwareLocaleCompare = (a: string, b: string) => {
+  if (!a && !b) {
+    return 0;
+  }
+
+  if (!a) {
+    return +1;
+  }
+
+  if (!b) {
+    return -1;
+  }
+
+  return a.localeCompare(b);
+};
+
+
