@@ -21,17 +21,18 @@ import {Row, Col, Icon, Tooltip} from 'antd';
 import OverviewCard from 'components/overviewCard/overviewCard';
 import axios from 'axios';
 import {IStorageReport} from 'types/datanode.types';
-import {IMissingContainersResponse} from '../missingContainers/missingContainers';
 import moment from 'moment';
 import AutoReloadPanel from 'components/autoReloadPanel/autoReloadPanel';
-import {showDataFetchError} from 'utils/common';
+import {showDataFetchError,byteToSize} from 'utils/common';
 import {AutoReloadHelper} from 'utils/autoReloadHelper';
 import filesize from 'filesize';
 import './overview.less';
+import { AxiosAllGetHelper, AxiosGetHelper, cancelRequests } from 'utils/axiosRequestHelper';
 
 const size = filesize.partial({round: 1});
 
 interface IClusterStateResponse {
+  missingContainers: number;
   totalDatanodes: number;
   healthyDatanodes: number;
   pipelines: number;
@@ -40,6 +41,9 @@ interface IClusterStateResponse {
   volumes: number;
   buckets: number;
   keys: number;
+  openContainers: number;
+  deletedContainers: number;
+  keysPendingDeletion: number;
 }
 
 interface IOverviewState {
@@ -55,8 +59,20 @@ interface IOverviewState {
   lastRefreshed: number;
   lastUpdatedOMDBDelta: number;
   lastUpdatedOMDBFull: number;
+  omStatus: string;
+  openContainers: number;
+  deletedContainers: number;
+  openSummarytotalUnrepSize: number,
+  openSummarytotalRepSize: number,
+  openSummarytotalOpenKeys: number,
+  deletePendingSummarytotalUnrepSize: number,
+  deletePendingSummarytotalRepSize: number,
+  deletePendingSummarytotalDeletedKeys: number,
 }
 
+let cancelOverviewSignal: AbortController;
+let cancelOMDBSyncSignal: AbortController;
+
 export class Overview extends React.Component<Record<string, object>, IOverviewState> {
   interval = 0;
   autoReload: AutoReloadHelper;
@@ -79,7 +95,16 @@ export class Overview extends React.Component<Record<string, object>, IOverviewS
       missingContainersCount: 0,
       lastRefreshed: 0,
       lastUpdatedOMDBDelta: 0,
-      lastUpdatedOMDBFull: 0
+      lastUpdatedOMDBFull: 0,
+      omStatus: '',
+      openContainers: 0,
+      deletedContainers: 0,
+      openSummarytotalUnrepSize: 0,
+      openSummarytotalRepSize: 0,
+      openSummarytotalOpenKeys: 0,
+      deletePendingSummarytotalUnrepSize: 0,
+      deletePendingSummarytotalRepSize: 0,
+      deletePendingSummarytotalDeletedKeys: 0
     };
     this.autoReload = new AutoReloadHelper(this._loadData);
   }
@@ -88,19 +113,29 @@ export class Overview extends React.Component<Record<string, object>, IOverviewS
     this.setState({
       loading: true
     });
-    axios.all([
-      axios.get('/api/v1/clusterState'),
-      axios.get('/api/v1/containers/missing'),
-      axios.get('/api/v1/task/status')
-    ]).then(axios.spread((clusterStateResponse, missingContainersResponse,taskstatusResponse) => {
+
+    //cancel any previous pending requests
+    cancelRequests([
+      cancelOMDBSyncSignal,
+      cancelOverviewSignal
+    ]);
+
+    const { requests, controller } = AxiosAllGetHelper([
+      '/api/v1/clusterState',
+      '/api/v1/task/status',
+      '/api/v1/keys/open/summary',
+      '/api/v1/keys/deletePending/summary'
+    ], cancelOverviewSignal);
+    cancelOverviewSignal = controller;
+
+    requests.then(axios.spread((clusterStateResponse, taskstatusResponse, openResponse, deletePendingResponse) => {
       
       const clusterState: IClusterStateResponse = clusterStateResponse.data;
-      const missingContainers: IMissingContainersResponse = missingContainersResponse.data;     
       const taskStatus = taskstatusResponse.data;
-      const missingContainersCount = missingContainers.totalCount;
+      const missingContainersCount = clusterState.missingContainers;
       const omDBDeltaObject = taskStatus && taskStatus.find((item:any) => item.taskName === 'OmDeltaRequest');
       const omDBFullObject = taskStatus && taskStatus.find((item:any) => item.taskName === 'OmSnapshotRequest');
-    
+
       this.setState({
         loading: false,
         datanodes: `${clusterState.healthyDatanodes}/${clusterState.totalDatanodes}`,
@@ -111,9 +146,17 @@ export class Overview extends React.Component<Record<string, object>, IOverviewS
         buckets: clusterState.buckets,
         keys: clusterState.keys,
         missingContainersCount,
+        openContainers: clusterState.openContainers,
+        deletedContainers: clusterState.deletedContainers,
         lastRefreshed: Number(moment()),
         lastUpdatedOMDBDelta: omDBDeltaObject && omDBDeltaObject.lastUpdatedTimestamp,
-        lastUpdatedOMDBFull: omDBFullObject && omDBFullObject.lastUpdatedTimestamp
+        lastUpdatedOMDBFull: omDBFullObject && omDBFullObject.lastUpdatedTimestamp,
+        openSummarytotalUnrepSize: openResponse.data  && openResponse.data.totalUnreplicatedDataSize,
+        openSummarytotalRepSize: openResponse.data && openResponse.data.totalReplicatedDataSize,
+        openSummarytotalOpenKeys: openResponse.data && openResponse.data.totalOpenKeys,
+        deletePendingSummarytotalUnrepSize: deletePendingResponse.data && deletePendingResponse.data.totalUnreplicatedDataSize,
+        deletePendingSummarytotalRepSize: deletePendingResponse.data && deletePendingResponse.data.totalReplicatedDataSize,
+        deletePendingSummarytotalDeletedKeys: deletePendingResponse.data && deletePendingResponse.data.totalDeletedKeys
       });
     })).catch(error => {
       this.setState({
@@ -123,6 +166,33 @@ export class Overview extends React.Component<Record<string, object>, IOverviewS
     });
   };
 
+
+  omSyncData = () => {
+    this.setState({
+      loading: true
+    });
+
+    const { request, controller } = AxiosGetHelper(
+      '/api/v1/triggerdbsync/om',
+      cancelOMDBSyncSignal,
+      "OM-DB Sync request cancelled because data was updated"
+    );
+    cancelOMDBSyncSignal = controller;
+
+    request.then( omstatusResponse => {
+      const omStatus = omstatusResponse.data;
+      this.setState({
+        loading: false,
+        omStatus: omStatus
+      });
+    }).catch(error => {
+      this.setState({
+        loading: false
+      });
+      showDataFetchError(error.toString());
+    });
+  };
+
   componentDidMount(): void {
     this._loadData();
     this.autoReload.startPolling();
@@ -130,29 +200,54 @@ export class Overview extends React.Component<Record<string, object>, IOverviewS
 
   componentWillUnmount(): void {
     this.autoReload.stopPolling();
+    cancelRequests([
+      cancelOMDBSyncSignal,
+      cancelOverviewSignal
+    ]);
   }
 
   render() {
-    const {loading, datanodes, pipelines, storageReport, containers, volumes, buckets,
-      keys, missingContainersCount, lastRefreshed, lastUpdatedOMDBDelta, lastUpdatedOMDBFull} = this.state;
+    const {loading, datanodes, pipelines, storageReport, containers, volumes, buckets, openSummarytotalUnrepSize, openSummarytotalRepSize, openSummarytotalOpenKeys,
+      deletePendingSummarytotalUnrepSize,deletePendingSummarytotalRepSize,deletePendingSummarytotalDeletedKeys,
+      keys, missingContainersCount, lastRefreshed, lastUpdatedOMDBDelta, lastUpdatedOMDBFull, omStatus, openContainers, deletedContainers } = this.state;
       
     const datanodesElement = (
       <span>
         <Icon type='check-circle' theme='filled' className='icon-success icon-small'/> {datanodes} <span className='ant-card-meta-description meta'>HEALTHY</span>
       </span>
     );
+    const openSummaryData = (
+        <div>
+          {openSummarytotalRepSize!== undefined ? byteToSize(openSummarytotalRepSize, 1): '0'}   <span className='ant-card-meta-description meta'>Total Replicated Data Size</span><br />
+          {openSummarytotalUnrepSize!== undefined ? byteToSize(openSummarytotalUnrepSize, 1): '0'}  <span className='ant-card-meta-description meta'>Total UnReplicated Data Size</span><br />
+          {openSummarytotalOpenKeys !== undefined ? openSummarytotalOpenKeys: '0'}  <span className='ant-card-meta-description meta'>Total Open Keys</span>
+        </div>
+    );
+    const deletePendingSummaryData = (
+      <div>
+        {deletePendingSummarytotalRepSize!== undefined ? byteToSize(deletePendingSummarytotalRepSize, 1): '0'}  <span className='ant-card-meta-description meta'>Total Replicated Data Size</span><br />
+        {deletePendingSummarytotalUnrepSize!== undefined ? byteToSize(deletePendingSummarytotalUnrepSize,1): '0'}  <span className='ant-card-meta-description meta'>Total UnReplicated Data Size</span><br />
+        {deletePendingSummarytotalDeletedKeys !== undefined ? deletePendingSummarytotalDeletedKeys: '0'}  <span className='ant-card-meta-description meta'>Total Pending Delete Keys</span>
+      </div>
+  );
     const containersTooltip = missingContainersCount === 1 ? 'container is missing' : 'containers are missing';
     const containersLink = missingContainersCount > 0 ? '/MissingContainers' : '/Containers';
-    const duLink = '/DiskUsage';
+    const volumesLink = '/Volumes';
+    const bucketsLink = '/Buckets';
     const containersElement = missingContainersCount > 0 ? (
       <span>
-        <Tooltip placement='bottom' title={`${missingContainersCount} ${containersTooltip}`}>
+        <Tooltip placement='bottom' title={missingContainersCount > 1000 ? `1000+ Containers are missing. For more information, go to the Containers page.` : `${missingContainersCount} ${containersTooltip}`}>
           <Icon type='exclamation-circle' theme='filled' className='icon-failure icon-small'/>
         </Tooltip>
         <span className='padded-text'>{containers - missingContainersCount}/{containers}</span>
       </span>
     ) :
-      containers.toString();
+      <div>
+          <span>{containers.toString()}   </span>
+        <Tooltip placement='bottom' title='Number of open containers'>
+          <span>({openContainers})</span>
+        </Tooltip>
+      </div>
     const clusterCapacity = `${size(storageReport.capacity - storageReport.remaining)}/${size(storageReport.capacity)}`;
     return (
       <div className='overview-content'>
@@ -160,7 +255,7 @@ export class Overview extends React.Component<Record<string, object>, IOverviewS
           Overview
           <AutoReloadPanel isLoading={loading} lastRefreshed={lastRefreshed}
           lastUpdatedOMDBDelta={lastUpdatedOMDBDelta} lastUpdatedOMDBFull={lastUpdatedOMDBFull}
-          togglePolling={this.autoReload.handleAutoReloadToggle} onReload={this._loadData}/>
+          togglePolling={this.autoReload.handleAutoReloadToggle} onReload={this._loadData} omSyncLoad={this.omSyncData} omStatus={omStatus}/>
         </div>
         <Row gutter={[25, 25]}>
           <Col xs={24} sm={18} md={12} lg={12} xl={6}>
@@ -190,14 +285,23 @@ export class Overview extends React.Component<Record<string, object>, IOverviewS
         </Row>
         <Row gutter={[25, 25]}>
           <Col xs={24} sm={18} md={12} lg={12} xl={6}>
-            <OverviewCard loading={loading} title='Volumes' data={volumes.toString()} icon='inbox' linkToUrl={duLink}/>
+            <OverviewCard loading={loading} title='Volumes' data={volumes.toString()} icon='inbox' linkToUrl={volumesLink}/>
           </Col>
           <Col xs={24} sm={18} md={12} lg={12} xl={6}>
-            <OverviewCard loading={loading} title='Buckets' data={buckets.toString()} icon='folder-open' linkToUrl='/Buckets'/>
+            <OverviewCard loading={loading} title='Buckets' data={buckets.toString()} icon='folder-open' linkToUrl={bucketsLink}/>
           </Col>
           <Col xs={24} sm={18} md={12} lg={12} xl={6}>
             <OverviewCard loading={loading} title='Keys' data={keys.toString()} icon='file-text'/>
           </Col>
+          <Col xs={24} sm={18} md={12} lg={12} xl={6}>
+            <OverviewCard loading={loading} title='Deleted Containers' data={deletedContainers.toString()} icon='delete' />
+          </Col>
+          <Col xs={24} sm={18} md={12} lg={12} xl={6} className='summary-font'>
+            <OverviewCard loading={loading} title='Open Keys Summary' data={openSummaryData} icon='file-text' linkToUrl='/Om' />
+          </Col>
+          <Col xs={24} sm={18} md={12} lg={12} xl={6} className='summary-font'>
+            <OverviewCard loading={loading} title='Pending Deleted Keys Summary' data={deletePendingSummaryData} icon='delete' linkToUrl='/Om'/>
+          </Col>
         </Row>
       </div>
     );
