@@ -20,22 +20,25 @@ package org.apache.hadoop.ozone.recon;
 
 import static org.apache.hadoop.hdds.protocol.MockDatanodeDetails.randomDatanodeDetails;
 import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.ONE;
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.THREE;
 import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_DB_DIRS;
 import static org.apache.hadoop.ozone.om.OmMetadataManagerImpl.BUCKET_TABLE;
 import static org.apache.hadoop.ozone.om.OmMetadataManagerImpl.DIRECTORY_TABLE;
 import static org.apache.hadoop.ozone.om.OmMetadataManagerImpl.FILE_TABLE;
 import static org.apache.hadoop.ozone.om.OmMetadataManagerImpl.VOLUME_TABLE;
 import static org.apache.hadoop.ozone.recon.ReconServerConfigKeys.OZONE_RECON_OM_SNAPSHOT_DB_DIR;
-import static org.junit.Assert.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.mockito.Mockito.mock;
 import static org.mockito.Mockito.when;
 
 import java.io.File;
 import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
 
 import org.apache.hadoop.hdds.client.BlockID;
+import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.client.StandaloneReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
@@ -44,6 +47,7 @@ import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
 import org.apache.hadoop.ozone.om.OmMetadataManagerImpl;
+import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyLocationInfoGroup;
 import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
@@ -75,7 +79,7 @@ public final class OMMetadataManagerTestUtils {
     omConfiguration.set(OZONE_OM_DB_DIRS,
         omDbDir.getAbsolutePath());
     OMMetadataManager omMetadataManager = new OmMetadataManagerImpl(
-        omConfiguration);
+        omConfiguration, null);
 
     String volumeKey = omMetadataManager.getVolumeKey("sampleVol");
     OmVolumeArgs args =
@@ -109,7 +113,7 @@ public final class OMMetadataManagerTestUtils {
     OzoneConfiguration omConfiguration = new OzoneConfiguration();
     omConfiguration.set(OZONE_OM_DB_DIRS,
         omDbDir.getAbsolutePath());
-    return new OmMetadataManagerImpl(omConfiguration);
+    return new OmMetadataManagerImpl(omConfiguration, null);
   }
 
   /**
@@ -233,7 +237,8 @@ public final class OMMetadataManagerTestUtils {
                                   long bucketObjectId,
                                   long volumeObjectId,
                                   List<OmKeyLocationInfoGroup> locationVersions,
-                                  BucketLayout bucketLayout)
+                                  BucketLayout bucketLayout,
+                                  long dataSize)
           throws IOException {
 
     String omKey;
@@ -248,6 +253,7 @@ public final class OMMetadataManagerTestUtils {
                     .setBucketName(bucketName)
                     .setVolumeName(volName)
                     .setKeyName(keyName)
+                    .setDataSize(dataSize)
                     .setOmKeyLocationInfos(locationVersions)
                     .setReplicationConfig(
                         StandaloneReplicationConfig.getInstance(ONE))
@@ -256,6 +262,102 @@ public final class OMMetadataManagerTestUtils {
                     .build());
   }
 
+  /**
+   * Write an open key to OM instance optimized for File System.
+   *
+   * @throws IOException while writing.
+   */
+  @SuppressWarnings("checkstyle:parameternumber")
+  public static void writeOpenFileToOm(OMMetadataManager omMetadataManager,
+                                 String keyName,
+                                 String bucketName,
+                                 String volName,
+                                 String fileName,
+                                 long objectId,
+                                 long parentObjectId,
+                                 long bucketObjectId,
+                                 long volumeObjectId,
+                                 List<OmKeyLocationInfoGroup> locationVersions,
+                                 long dataSize)
+      throws IOException {
+
+    String openKey = omMetadataManager.getOzonePathKey(volumeObjectId,
+        bucketObjectId, parentObjectId, fileName);
+
+    OmKeyInfo omKeyInfo = new OmKeyInfo.Builder()
+        .setBucketName(bucketName)
+        .setVolumeName(volName)
+        .setKeyName(keyName)
+        .setDataSize(dataSize)
+        .setOmKeyLocationInfos(locationVersions)
+        .setReplicationConfig(RatisReplicationConfig.getInstance(THREE))
+        .setObjectID(objectId)
+        .setParentObjectID(parentObjectId)
+        .build();
+
+    omMetadataManager.getOpenKeyTable(BucketLayout.FILE_SYSTEM_OPTIMIZED)
+        .put(openKey, omKeyInfo);
+  }
+
+  /**
+   * Write an open key to OM instance with any other BucketLayout.
+   *
+   * @throws IOException while writing.
+   */
+  public static void writeOpenKeyToOm(OMMetadataManager omMetadataManager,
+                                  String keyName,
+                                  String bucketName,
+                                  String volName,
+                                  List<OmKeyLocationInfoGroup> locationVersions,
+                                  long dataSize)
+      throws IOException {
+
+    String openKey =
+        omMetadataManager.getOzoneKey(volName, bucketName, keyName);
+    OmKeyInfo omKeyInfo = new OmKeyInfo.Builder()
+        .setBucketName(bucketName)
+        .setVolumeName(volName)
+        .setKeyName(keyName)
+        .setDataSize(dataSize)
+        .setOmKeyLocationInfos(locationVersions)
+        .setReplicationConfig(RatisReplicationConfig.getInstance(THREE))
+        .build();
+
+    omMetadataManager.getOpenKeyTable(BucketLayout.LEGACY)
+        .put(openKey, omKeyInfo);
+  }
+
+
+  /**
+   * Writes deleted key information to the Ozone Manager metadata table.
+   * @param omMetadataManager the Ozone Manager metadata manager
+   * @param keyNames the names of the deleted keys
+   * @param bucketName name of the bucket that used to contain the deleted keys
+   * @param volName name of the volume that used to contain the deleted keys
+   * @throws IOException if there is an error accessing the metadata table
+   */
+  public static void writeDeletedKeysToOm(OMMetadataManager omMetadataManager,
+                                         List<String> keyNames,
+                                         String bucketName,
+                                         String volName) throws IOException {
+    List<OmKeyInfo> infos = new ArrayList<>();
+    for (int i = 0; i < keyNames.size(); i++) {
+      infos.add(new OmKeyInfo.Builder()
+          .setBucketName(bucketName)
+          .setVolumeName(volName)
+          .setKeyName(keyNames.get(i))
+          .setDataSize(100L)
+          .setReplicationConfig(StandaloneReplicationConfig.getInstance(ONE))
+          .build());
+    }
+    // Get the Ozone key for the first deleted key
+    String omKey = omMetadataManager.getOzoneKey(volName,
+        bucketName, keyNames.get(0));
+    RepeatedOmKeyInfo repeatedKeyInfo = new RepeatedOmKeyInfo(infos);
+    // Put the deleted key information into the deleted table
+    omMetadataManager.getDeletedTable().put(omKey, repeatedKeyInfo);
+  }
+
   /**
    * Write a directory as key on OM instance.
    * We don't need to set size.
@@ -295,6 +397,27 @@ public final class OMMetadataManagerTestUtils {
                     .build());
   }
 
+  public static void writeDeletedDirToOm(OMMetadataManager omMetadataManager,
+                                         String bucketName,
+                                         String volumeName,
+                                         String dirName,
+                                         long parentObjectId,
+                                         long bucketObjectId,
+                                         long volumeObjectId)
+      throws IOException {
+    // DB key in DeletedDirectoryTable => "volumeID/bucketID/parentId/dirName"
+    String omKey = omMetadataManager.getOzonePathKey(volumeObjectId,
+            bucketObjectId, parentObjectId, dirName);
+
+    omMetadataManager.getDeletedDirTable().put(omKey,
+        new OmKeyInfo.Builder()
+            .setBucketName(bucketName)
+            .setVolumeName(volumeName)
+            .setKeyName(dirName)
+            .setReplicationConfig(StandaloneReplicationConfig.getInstance(ONE))
+            .build());
+  }
+
   public static OzoneManagerServiceProviderImpl
       getMockOzoneManagerServiceProvider() throws IOException {
     OzoneManagerServiceProviderImpl omServiceProviderMock =
