@@ -23,6 +23,7 @@ import org.apache.commons.lang3.tuple.Pair;
 import org.apache.hadoop.hdds.client.RatisReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
+import org.apache.hadoop.hdds.protocol.StorageType;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos
     .ExtendedDatanodeDetailsProto;
@@ -42,6 +43,7 @@ import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolPro
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.NodeReportProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReportsProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.PipelineReportsProto;
+import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
 import org.apache.hadoop.hdds.scm.container.common.helpers.ContainerWithPipeline;
 import org.apache.hadoop.hdds.scm.node.NodeManager;
@@ -52,6 +54,8 @@ import org.apache.hadoop.hdds.scm.server.OzoneStorageContainerManager;
 import org.apache.hadoop.hdds.upgrade.HDDSLayoutVersionManager;
 import org.apache.hadoop.hdds.utils.db.TypedTable;
 import org.apache.hadoop.hdfs.web.URLConnectionFactory;
+import org.apache.hadoop.ozone.OzoneAcl;
+import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.om.OMMetadataManager;
 import org.apache.hadoop.ozone.om.OmMetadataManagerImpl;
 import org.apache.hadoop.ozone.om.helpers.BucketLayout;
@@ -61,11 +65,17 @@ import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
 import org.apache.hadoop.ozone.recon.MetricsServiceProviderFactory;
 import org.apache.hadoop.ozone.recon.ReconTestInjector;
 import org.apache.hadoop.ozone.recon.ReconUtils;
+import org.apache.hadoop.ozone.recon.api.types.AclMetadata;
+import org.apache.hadoop.ozone.recon.api.types.BucketObjectDBInfo;
+import org.apache.hadoop.ozone.recon.api.types.BucketsResponse;
 import org.apache.hadoop.ozone.recon.api.types.ClusterStateResponse;
 import org.apache.hadoop.ozone.recon.api.types.DatanodeMetadata;
 import org.apache.hadoop.ozone.recon.api.types.DatanodesResponse;
 import org.apache.hadoop.ozone.recon.api.types.PipelineMetadata;
 import org.apache.hadoop.ozone.recon.api.types.PipelinesResponse;
+import org.apache.hadoop.ozone.recon.api.types.VolumeObjectDBInfo;
+import org.apache.hadoop.ozone.recon.api.types.VolumesResponse;
+import org.apache.hadoop.ozone.recon.common.CommonUtils;
 import org.apache.hadoop.ozone.recon.persistence.AbstractReconSqlDBTest;
 import org.apache.hadoop.ozone.recon.persistence.ContainerHealthSchemaManager;
 import org.apache.hadoop.ozone.recon.recovery.ReconOMMetadataManager;
@@ -73,30 +83,39 @@ import org.apache.hadoop.ozone.recon.scm.ReconStorageContainerManagerFacade;
 import org.apache.hadoop.ozone.recon.spi.StorageContainerServiceProvider;
 import org.apache.hadoop.ozone.recon.spi.impl.OzoneManagerServiceProviderImpl;
 import org.apache.hadoop.ozone.recon.spi.impl.StorageContainerServiceProviderImpl;
+import org.apache.hadoop.ozone.recon.tasks.ContainerSizeCountTask;
 import org.apache.hadoop.ozone.recon.tasks.FileSizeCountTask;
-import org.apache.hadoop.ozone.recon.tasks.TableCountTask;
+import org.apache.hadoop.ozone.recon.tasks.OmTableInsightTask;
 import org.apache.ozone.test.GenericTestUtils;
+import org.apache.hadoop.ozone.security.acl.IAccessAuthorizer;
 import org.apache.ozone.test.LambdaTestUtils;
 import org.hadoop.ozone.recon.schema.UtilizationSchemaDefinition;
+import org.hadoop.ozone.recon.schema.tables.daos.ContainerCountBySizeDao;
 import org.hadoop.ozone.recon.schema.tables.daos.FileCountBySizeDao;
 import org.hadoop.ozone.recon.schema.tables.daos.GlobalStatsDao;
+import org.hadoop.ozone.recon.schema.tables.pojos.ContainerCountBySize;
 import org.hadoop.ozone.recon.schema.tables.pojos.FileCountBySize;
 import org.jooq.Configuration;
 import org.jooq.DSLContext;
 import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 
 import static org.apache.hadoop.hdds.protocol.MockDatanodeDetails.randomDatanodeDetails;
 import static org.apache.hadoop.ozone.container.upgrade.UpgradeUtils.defaultLayoutVersionProto;
+import static org.apache.hadoop.ozone.recon.OMMetadataManagerTestUtils.writeDeletedDirToOm;
 import static org.apache.hadoop.ozone.recon.OMMetadataManagerTestUtils.getRandomPipeline;
 import static org.apache.hadoop.ozone.recon.OMMetadataManagerTestUtils.getTestReconOmMetadataManager;
 import static org.apache.hadoop.ozone.recon.OMMetadataManagerTestUtils.initializeNewOmMetadataManager;
 import static org.apache.hadoop.ozone.recon.OMMetadataManagerTestUtils.writeDataToOm;
+import static org.apache.hadoop.ozone.recon.OMMetadataManagerTestUtils.writeDeletedKeysToOm;
 import static org.apache.hadoop.ozone.recon.spi.impl.PrometheusServiceProviderImpl.PROMETHEUS_INSTANT_QUERY_API;
 import static org.hadoop.ozone.recon.schema.tables.GlobalStatsTable.GLOBAL_STATS;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.mockito.ArgumentMatchers.any;
 import static org.mockito.ArgumentMatchers.anyBoolean;
 import static org.mockito.ArgumentMatchers.anyString;
@@ -115,6 +134,10 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.net.HttpURLConnection;
 import java.net.URI;
+import java.nio.file.Files;
+import java.nio.file.Path;
+import java.util.Arrays;
+import java.util.ArrayList;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.UUID;
@@ -125,14 +148,19 @@ import java.util.concurrent.atomic.AtomicInteger;
  * Test for Recon API endpoints.
  */
 public class TestEndpoints extends AbstractReconSqlDBTest {
+  @TempDir
+  private Path temporaryFolder;
   private NodeEndpoint nodeEndpoint;
   private PipelineEndpoint pipelineEndpoint;
   private ClusterStateEndpoint clusterStateEndpoint;
   private UtilizationEndpoint utilizationEndpoint;
   private MetricsProxyEndpoint metricsProxyEndpoint;
+  private VolumeEndpoint volumeEndpoint;
+  private BucketEndpoint bucketEndpoint;
   private ReconOMMetadataManager reconOMMetadataManager;
   private FileSizeCountTask fileSizeCountTask;
-  private TableCountTask tableCountTask;
+  private ContainerSizeCountTask containerSizeCountTask;
+  private OmTableInsightTask omTableInsightTask;
   private ReconStorageContainerManagerFacade reconScm;
   private boolean isSetupDone = false;
   private String pipelineId;
@@ -143,6 +171,7 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
   private ExtendedDatanodeDetailsProto extendedDatanodeDetailsProto;
   private Pipeline pipeline;
   private FileCountBySizeDao fileCountBySizeDao;
+  private ContainerCountBySizeDao containerCountBySizeDao;
   private DSLContext dslContext;
   private static final String HOST1 = "host1.datanode";
   private static final String HOST2 = "host2.datanode";
@@ -152,10 +181,14 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
       "prometheus-test-response.txt";
   private ReconUtils reconUtilsMock;
 
+  private ContainerHealthSchemaManager containerHealthSchemaManager;
+  private CommonUtils commonUtils;
+
   private void initializeInjector() throws Exception {
     reconOMMetadataManager = getTestReconOmMetadataManager(
-        initializeNewOmMetadataManager(temporaryFolder.newFolder()),
-        temporaryFolder.newFolder());
+        initializeNewOmMetadataManager(Files.createDirectory(
+            temporaryFolder.resolve("JunitOmDBDir")).toFile()),
+        Files.createDirectory(temporaryFolder.resolve("NewDir")).toFile());
     datanodeDetails = randomDatanodeDetails();
     datanodeDetails2 = randomDatanodeDetails();
     datanodeDetails.setHostName(HOST1);
@@ -192,7 +225,6 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
     when(mockScmServiceProvider
         .getExistContainerWithPipelinesInBatch(containerIDs))
         .thenReturn(cpw);
-
     InputStream inputStream =
         Thread.currentThread().getContextClassLoader().getResourceAsStream(
             PROMETHEUS_TEST_RESPONSE_FILE);
@@ -205,9 +237,11 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
         anyString(), anyBoolean())).thenReturn(urlConnectionMock);
     when(reconUtilsMock.getReconDbDir(any(OzoneConfiguration.class),
         anyString())).thenReturn(GenericTestUtils.getRandomizedTestDir());
-
+    when(reconUtilsMock.getReconNodeDetails(
+        any(OzoneConfiguration.class))).thenReturn(
+        commonUtils.getReconNodeDetails());
     ReconTestInjector reconTestInjector =
-        new ReconTestInjector.Builder(temporaryFolder)
+        new ReconTestInjector.Builder(temporaryFolder.toFile())
             .withReconSqlDb()
             .withReconOm(reconOMMetadataManager)
             .withOmServiceProvider(mock(OzoneManagerServiceProviderImpl.class))
@@ -218,6 +252,8 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
             .withContainerDB()
             .addBinding(ClusterStateEndpoint.class)
             .addBinding(NodeEndpoint.class)
+            .addBinding(VolumeEndpoint.class)
+            .addBinding(BucketEndpoint.class)
             .addBinding(MetricsServiceProviderFactory.class)
             .addBinding(ContainerHealthSchemaManager.class)
             .addBinding(UtilizationEndpoint.class)
@@ -225,24 +261,33 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
             .addBinding(StorageContainerLocationProtocol.class, mockScmClient)
             .build();
 
+    reconScm = (ReconStorageContainerManagerFacade)
+        reconTestInjector.getInstance(OzoneStorageContainerManager.class);
     nodeEndpoint = reconTestInjector.getInstance(NodeEndpoint.class);
     pipelineEndpoint = reconTestInjector.getInstance(PipelineEndpoint.class);
+    volumeEndpoint = reconTestInjector.getInstance(VolumeEndpoint.class);
+    bucketEndpoint = reconTestInjector.getInstance(BucketEndpoint.class);
     fileCountBySizeDao = getDao(FileCountBySizeDao.class);
+    containerCountBySizeDao = reconScm.getContainerCountBySizeDao();
     GlobalStatsDao globalStatsDao = getDao(GlobalStatsDao.class);
     UtilizationSchemaDefinition utilizationSchemaDefinition =
         getSchemaDefinition(UtilizationSchemaDefinition.class);
     Configuration sqlConfiguration =
         reconTestInjector.getInstance(Configuration.class);
     utilizationEndpoint = new UtilizationEndpoint(
-        fileCountBySizeDao, utilizationSchemaDefinition);
+        fileCountBySizeDao,
+        containerCountBySizeDao,
+        utilizationSchemaDefinition);
     fileSizeCountTask =
         new FileSizeCountTask(fileCountBySizeDao, utilizationSchemaDefinition);
-    tableCountTask = new TableCountTask(
+    omTableInsightTask = new OmTableInsightTask(
         globalStatsDao, sqlConfiguration, reconOMMetadataManager);
-    reconScm = (ReconStorageContainerManagerFacade)
-        reconTestInjector.getInstance(OzoneStorageContainerManager.class);
+    containerHealthSchemaManager =
+        reconTestInjector.getInstance(ContainerHealthSchemaManager.class);
     clusterStateEndpoint =
-        new ClusterStateEndpoint(reconScm, globalStatsDao);
+        new ClusterStateEndpoint(reconScm, globalStatsDao,
+            containerHealthSchemaManager);
+    containerSizeCountTask = reconScm.getContainerSizeCountTask();
     MetricsServiceProviderFactory metricsServiceProviderFactory =
         reconTestInjector.getInstance(MetricsServiceProviderFactory.class);
     metricsProxyEndpoint =
@@ -250,10 +295,12 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
     dslContext = getDslContext();
   }
 
+  @SuppressWarnings("checkstyle:MethodLength")
   @BeforeEach
   public void setUp() throws Exception {
     // The following setup runs only once
     if (!isSetupDone) {
+      commonUtils = new CommonUtils();
       initializeInjector();
       isSetupDone = true;
     }
@@ -361,9 +408,7 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
     } catch (Exception ex) {
       Assertions.fail(ex.getMessage());
     }
-
     // Write Data to OM
-
     // A sample volume (sampleVol) and a bucket (bucketOne) is already created
     // in AbstractOMMetadataManagerTest.
     // Create a new volume and bucket and then write keys to the bucket.
@@ -371,14 +416,43 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
     OmVolumeArgs args =
         OmVolumeArgs.newBuilder()
             .setVolume("sampleVol2")
-            .setAdminName("TestUser")
-            .setOwnerName("TestUser")
+            .setAdminName("TestUser2")
+            .setOwnerName("TestUser2")
+            .setQuotaInBytes(OzoneConsts.GB)
+            .setQuotaInNamespace(1000)
+            .setUsedNamespace(500)
+            .addOzoneAcls(new OzoneAcl(
+                IAccessAuthorizer.ACLIdentityType.USER,
+                "TestUser2",
+                IAccessAuthorizer.ACLType.WRITE,
+                OzoneAcl.AclScope.ACCESS
+            ))
+            .addOzoneAcls(new OzoneAcl(
+                IAccessAuthorizer.ACLIdentityType.USER,
+                "TestUser2",
+                IAccessAuthorizer.ACLType.READ,
+                OzoneAcl.AclScope.ACCESS
+            ))
             .build();
     reconOMMetadataManager.getVolumeTable().put(volumeKey, args);
 
     OmBucketInfo bucketInfo = OmBucketInfo.newBuilder()
         .setVolumeName("sampleVol2")
         .setBucketName("bucketOne")
+        .addAcl(new OzoneAcl(
+            IAccessAuthorizer.ACLIdentityType.GROUP,
+            "TestGroup2",
+            IAccessAuthorizer.ACLType.WRITE,
+            OzoneAcl.AclScope.ACCESS
+        ))
+        .setQuotaInBytes(OzoneConsts.GB)
+        .setUsedBytes(OzoneConsts.MB)
+        .setQuotaInNamespace(5)
+        .setStorageType(StorageType.DISK)
+        .setUsedNamespace(3)
+        .setBucketLayout(BucketLayout.LEGACY)
+        .setOwner("TestUser2")
+        .setIsVersionEnabled(false)
         .build();
 
     String bucketKey = reconOMMetadataManager.getBucketKey(
@@ -386,15 +460,56 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
 
     reconOMMetadataManager.getBucketTable().put(bucketKey, bucketInfo);
 
+    OmBucketInfo bucketInfo2 = OmBucketInfo.newBuilder()
+        .setVolumeName("sampleVol2")
+        .setBucketName("bucketTwo")
+        .addAcl(new OzoneAcl(
+            IAccessAuthorizer.ACLIdentityType.GROUP,
+            "TestGroup2",
+            IAccessAuthorizer.ACLType.READ,
+            OzoneAcl.AclScope.ACCESS
+        ))
+        .setQuotaInBytes(OzoneConsts.GB)
+        .setUsedBytes(100 * OzoneConsts.MB)
+        .setQuotaInNamespace(5)
+        .setStorageType(StorageType.SSD)
+        .setUsedNamespace(3)
+        .setBucketLayout(BucketLayout.OBJECT_STORE)
+        .setOwner("TestUser2")
+        .setIsVersionEnabled(false)
+        .build();
+
+    String bucketKey2 = reconOMMetadataManager.getBucketKey(
+        bucketInfo2.getVolumeName(), bucketInfo2.getBucketName());
+
+    reconOMMetadataManager.getBucketTable().put(bucketKey2, bucketInfo2);
+
     // key = key_one
     writeDataToOm(reconOMMetadataManager, "key_one");
-
     // key = key_two
     writeDataToOm(reconOMMetadataManager, "key_two");
-
     // key = key_three
     writeDataToOm(reconOMMetadataManager, "key_three");
 
+    // Populate the deletedKeys table in OM DB
+    List<String> deletedKeysList1 = Arrays.asList("key1");
+    writeDeletedKeysToOm(reconOMMetadataManager,
+        deletedKeysList1, "Bucket1", "Volume1");
+    List<String> deletedKeysList2 = Arrays.asList("key2", "key2");
+    writeDeletedKeysToOm(reconOMMetadataManager,
+        deletedKeysList2, "Bucket2", "Volume2");
+    List<String> deletedKeysList3 = Arrays.asList("key3", "key3", "key3");
+    writeDeletedKeysToOm(reconOMMetadataManager,
+        deletedKeysList3, "Bucket3", "Volume3");
+
+    // Populate the deletedDirectories table in OM DB
+    writeDeletedDirToOm(reconOMMetadataManager, "Bucket1", "Volume1", "dir1",
+        3L, 2L, 1L);
+    writeDeletedDirToOm(reconOMMetadataManager, "Bucket2", "Volume2", "dir2",
+        6L, 5L, 4L);
+    writeDeletedDirToOm(reconOMMetadataManager, "Bucket3", "Volume3", "dir3",
+        9L, 8L, 7L);
+
     // Truncate global stats table before running each test
     dslContext.truncate(GLOBAL_STATS);
   }
@@ -589,6 +704,7 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
     Assertions.assertEquals(0, clusterStateResponse.getKeys());
     Assertions.assertEquals(2, clusterStateResponse.getTotalDatanodes());
     Assertions.assertEquals(2, clusterStateResponse.getHealthyDatanodes());
+    Assertions.assertEquals(0, clusterStateResponse.getMissingContainers());
 
     waitAndCheckConditionAfterHeartbeat(() -> {
       Response response1 = clusterStateEndpoint.getClusterState();
@@ -599,13 +715,19 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
 
     // check volume, bucket and key count after running table count task
     Pair<String, Boolean> result =
-        tableCountTask.reprocess(reconOMMetadataManager);
+        omTableInsightTask.reprocess(reconOMMetadataManager);
     assertTrue(result.getRight());
     response = clusterStateEndpoint.getClusterState();
     clusterStateResponse = (ClusterStateResponse) response.getEntity();
     Assertions.assertEquals(2, clusterStateResponse.getVolumes());
-    Assertions.assertEquals(2, clusterStateResponse.getBuckets());
+    Assertions.assertEquals(3, clusterStateResponse.getBuckets());
     Assertions.assertEquals(3, clusterStateResponse.getKeys());
+    // Since a single RepeatedOmKeyInfo can contain multiple deleted keys with
+    // the same name, the total count of pending deletion keys is determined by
+    // summing the count of the keyInfoList. Each keyInfoList comprises
+    // OmKeyInfo objects that represent the deleted keys.
+    Assertions.assertEquals(6, clusterStateResponse.getKeysPendingDeletion());
+    Assertions.assertEquals(3, clusterStateResponse.getDeletedDirs());
   }
 
   @Test
@@ -629,22 +751,44 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
     given(omKeyInfo3.getDataSize()).willReturn(1000L);
 
     OMMetadataManager omMetadataManager = mock(OmMetadataManagerImpl.class);
-    TypedTable<String, OmKeyInfo> keyTable = mock(TypedTable.class);
+    TypedTable<String, OmKeyInfo> keyTableLegacy = mock(TypedTable.class);
+    TypedTable<String, OmKeyInfo> keyTableFso = mock(TypedTable.class);
 
-    TypedTable.TypedTableIterator mockKeyIter = mock(TypedTable
+    TypedTable.TypedTableIterator mockKeyIterLegacy = mock(TypedTable
         .TypedTableIterator.class);
-    TypedTable.TypedKeyValue mockKeyValue = mock(
+    TypedTable.TypedTableIterator mockKeyIterFso = mock(TypedTable
+        .TypedTableIterator.class);
+    TypedTable.TypedKeyValue mockKeyValueLegacy = mock(
+        TypedTable.TypedKeyValue.class);
+    TypedTable.TypedKeyValue mockKeyValueFso = mock(
         TypedTable.TypedKeyValue.class);
 
-    when(keyTable.iterator()).thenReturn(mockKeyIter);
-    when(omMetadataManager.getKeyTable(getBucketLayout())).thenReturn(keyTable);
-    when(mockKeyIter.hasNext())
+    when(keyTableLegacy.iterator()).thenReturn(mockKeyIterLegacy);
+    when(keyTableFso.iterator()).thenReturn(mockKeyIterFso);
+
+    when(omMetadataManager.getKeyTable(BucketLayout.LEGACY)).thenReturn(
+        keyTableLegacy);
+    when(omMetadataManager.getKeyTable(
+        BucketLayout.FILE_SYSTEM_OPTIMIZED)).thenReturn(keyTableFso);
+
+    when(mockKeyIterLegacy.hasNext())
         .thenReturn(true)
         .thenReturn(true)
         .thenReturn(true)
         .thenReturn(false);
-    when(mockKeyIter.next()).thenReturn(mockKeyValue);
-    when(mockKeyValue.getValue())
+    when(mockKeyIterFso.hasNext())
+        .thenReturn(true)
+        .thenReturn(true)
+        .thenReturn(true)
+        .thenReturn(false);
+    when(mockKeyIterLegacy.next()).thenReturn(mockKeyValueLegacy);
+    when(mockKeyIterFso.next()).thenReturn(mockKeyValueFso);
+
+    when(mockKeyValueLegacy.getValue())
+        .thenReturn(omKeyInfo1)
+        .thenReturn(omKeyInfo2)
+        .thenReturn(omKeyInfo3);
+    when(mockKeyValueFso.getValue())
         .thenReturn(omKeyInfo1)
         .thenReturn(omKeyInfo2)
         .thenReturn(omKeyInfo3);
@@ -660,13 +804,13 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
     assertEquals(3, resultSet.size());
     assertTrue(resultSet.stream().anyMatch(o -> o.getVolume().equals("vol1") &&
         o.getBucket().equals("bucket1") && o.getFileSize() == 1024L &&
-        o.getCount() == 1L));
+        o.getCount() == 2L));
     assertTrue(resultSet.stream().anyMatch(o -> o.getVolume().equals("vol1") &&
         o.getBucket().equals("bucket1") && o.getFileSize() == 131072 &&
-        o.getCount() == 1L));
+        o.getCount() == 2L));
     assertTrue(resultSet.stream().anyMatch(o -> o.getVolume().equals("vol2") &&
         o.getBucket().equals("bucket1") && o.getFileSize() == 1024L &&
-        o.getCount() == 1L));
+        o.getCount() == 2L));
 
     // Test for "volume" query param
     response = utilizationEndpoint.getFileCounts("vol1", null, 0);
@@ -705,6 +849,307 @@ public class TestEndpoints extends AbstractReconSqlDBTest {
     assertEquals(0, resultSet.size());
   }
 
+  @Test
+  public void testGetContainerCounts() throws Exception {
+    // Mock container info objects with different sizes
+    ContainerInfo omContainerInfo1 = mock(ContainerInfo.class);
+    given(omContainerInfo1.containerID()).willReturn(new ContainerID(1));
+    given(omContainerInfo1.getUsedBytes()).willReturn(1500000000L); // 1.5GB
+
+    ContainerInfo omContainerInfo2 = mock(ContainerInfo.class);
+    given(omContainerInfo2.containerID()).willReturn(new ContainerID(2));
+    given(omContainerInfo2.getUsedBytes()).willReturn(2500000000L); // 2.5GB
+
+    // Create a list of container info objects
+    List<ContainerInfo> containers = new ArrayList<>();
+    containers.add(omContainerInfo1);
+    containers.add(omContainerInfo2);
+
+    // Process the list of containers through the container size count task
+    containerSizeCountTask.process(containers);
+
+    // Test fetching all container counts
+    Response response = utilizationEndpoint.getContainerCounts(0L);
+    List<ContainerCountBySize> resultSet =
+        (List<ContainerCountBySize>) response.getEntity();
+
+    assertEquals(2, resultSet.size());
+    // The UpperBound Stored for 1.5GB in the table is 2147483648L
+    assertTrue(resultSet.stream().anyMatch(o -> o.getContainerSize() ==
+        2147483648L && o.getCount() == 1L));
+    // The UpperBound Stored for 2.5GB in the table is 4294967296L
+    assertTrue(resultSet.stream().anyMatch(o -> o.getContainerSize() ==
+        4294967296L && o.getCount() == 1L));
+
+    // Test fetching a specific container size (1.5GB)
+    response = utilizationEndpoint.getContainerCounts(1500000000L);
+    resultSet = (List<ContainerCountBySize>) response.getEntity();
+    assertEquals(1, resultSet.size());
+    // The UpperBound Stored for 1.5GB is 2147483648L (2GB)
+    assertEquals(2147483648L, resultSet.get(0).getContainerSize());
+    assertEquals(1, resultSet.get(0).getCount());
+
+    // Test fetching non-existent container size
+    response = utilizationEndpoint.getContainerCounts(8192L);
+    resultSet = (List<ContainerCountBySize>) response.getEntity();
+    assertTrue(resultSet.isEmpty());
+
+  }
+
+  private void testVolumeResponse(VolumeObjectDBInfo volumeMetadata)
+      throws IOException {
+    String volumeName = volumeMetadata.getVolume();
+    switch (volumeName) {
+    case "sampleVol":
+      assertEquals("TestUser", volumeMetadata.getOwner());
+      assertEquals("TestUser", volumeMetadata.getAdmin());
+      assertEquals(-1, volumeMetadata.getQuotaInBytes());
+      assertEquals(-1, volumeMetadata.getQuotaInNamespace());
+      assertEquals(0, volumeMetadata.getUsedNamespace());
+      assertEquals(0, volumeMetadata.getAcls().size());
+      break;
+    case "sampleVol2":
+      assertEquals("TestUser2", volumeMetadata.getOwner());
+      assertEquals("TestUser2", volumeMetadata.getAdmin());
+      assertEquals(OzoneConsts.GB, volumeMetadata.getQuotaInBytes());
+      assertEquals(1000, volumeMetadata.getQuotaInNamespace());
+      assertEquals(500, volumeMetadata.getUsedNamespace());
+
+      assertEquals(1, volumeMetadata.getAcls().size());
+      AclMetadata acl = volumeMetadata.getAcls().get(0);
+      assertEquals("TestUser2", acl.getName());
+      assertEquals("USER", acl.getType());
+      assertEquals("ACCESS", acl.getScope());
+
+      assertEquals(2, acl.getAclList().size());
+
+      if (acl.getAclList().get(0).equals("WRITE")) {
+        assertEquals("READ", acl.getAclList().get(1));
+      } else if (acl.getAclList().get(0).equals("READ")) {
+        assertEquals("WRITE", acl.getAclList().get(1));
+      } else {
+        Assertions.fail(String.format("ACL %s is not recognized",
+            acl.getAclList().get(0)));
+      }
+      break;
+    default:
+      Assertions.fail(String.format("Volume %s not registered",
+          volumeName));
+    }
+
+  }
+
+  @Test
+  public void testGetVolumes() throws Exception {
+    Response response = volumeEndpoint.getVolumes(10, "");
+    Assertions.assertEquals(200, response.getStatus());
+    VolumesResponse volumesResponse =
+        (VolumesResponse) response.getEntity();
+    Assertions.assertEquals(2, volumesResponse.getTotalCount());
+    Assertions.assertEquals(2, volumesResponse.getVolumes().size());
+
+    volumesResponse.getVolumes().forEach(volumeMetadata -> {
+      try {
+        testVolumeResponse(volumeMetadata);
+      } catch (IOException e) {
+        Assertions.fail(e.getMessage());
+      }
+    });
+  }
+
+  @Test
+  public void testGetVolumesWithPrevKeyAndLimit() throws IOException {
+    // Test limit
+    Response responseWithLimit = volumeEndpoint.getVolumes(1, null);
+    Assertions.assertEquals(200, responseWithLimit.getStatus());
+    VolumesResponse volumesResponseWithLimit =
+        (VolumesResponse) responseWithLimit.getEntity();
+    Assertions.assertEquals(1, volumesResponseWithLimit.getTotalCount());
+    Assertions.assertEquals(1, volumesResponseWithLimit.getVolumes().size());
+
+    // Test prevKey
+    Response responseWithPrevKey = volumeEndpoint.getVolumes(1, "sampleVol");
+    Assertions.assertEquals(200, responseWithPrevKey.getStatus());
+    VolumesResponse volumesResponseWithPrevKey =
+        (VolumesResponse) responseWithPrevKey.getEntity();
+
+    Assertions.assertEquals(1, volumesResponseWithPrevKey.getTotalCount());
+    Assertions.assertEquals(1, volumesResponseWithPrevKey.getVolumes().size());
+    Assertions.assertEquals("sampleVol2",
+        volumesResponseWithPrevKey.getVolumes().stream()
+            .findFirst().get().getVolume());
+
+  }
+
+  private void testBucketResponse(BucketObjectDBInfo bucketMetadata)
+      throws IOException {
+    String bucketName = bucketMetadata.getName();
+    switch (bucketName) {
+    case "bucketOne":
+      assertEquals("sampleVol2", bucketMetadata.getVolumeName());
+      assertEquals(StorageType.DISK, bucketMetadata.getStorageType());
+      assertNull(bucketMetadata.getSourceVolume());
+      assertNull(bucketMetadata.getSourceBucket());
+      assertEquals(OzoneConsts.GB, bucketMetadata.getQuotaInBytes());
+      assertEquals(OzoneConsts.MB, bucketMetadata.getUsedBytes());
+      assertEquals(5, bucketMetadata.getQuotaInNamespace());
+      assertEquals(3, bucketMetadata.getUsedNamespace());
+      assertFalse(bucketMetadata.isVersioningEnabled());
+      assertEquals(BucketLayout.LEGACY, bucketMetadata.getBucketLayout());
+      assertEquals("TestUser2", bucketMetadata.getOwner());
+
+
+      assertEquals(1, bucketMetadata.getAcls().size());
+      AclMetadata acl = bucketMetadata.getAcls().get(0);
+      assertEquals("TestGroup2", acl.getName());
+      assertEquals("GROUP", acl.getType());
+      assertEquals("ACCESS", acl.getScope());
+      assertEquals(1, acl.getAclList().size());
+      assertEquals("WRITE", acl.getAclList().get(0));
+      break;
+    case "bucketTwo":
+      assertEquals("sampleVol2", bucketMetadata.getVolumeName());
+      assertEquals(StorageType.SSD, bucketMetadata.getStorageType());
+      assertNull(bucketMetadata.getSourceVolume());
+      assertNull(bucketMetadata.getSourceBucket());
+      assertEquals(OzoneConsts.GB, bucketMetadata.getQuotaInBytes());
+      assertEquals(100 * OzoneConsts.MB, bucketMetadata.getUsedBytes());
+      assertEquals(5, bucketMetadata.getQuotaInNamespace());
+      assertEquals(3, bucketMetadata.getUsedNamespace());
+      assertFalse(bucketMetadata.isVersioningEnabled());
+      assertEquals(BucketLayout.OBJECT_STORE, bucketMetadata.getBucketLayout());
+      assertEquals("TestUser2", bucketMetadata.getOwner());
+
+      assertEquals(1, bucketMetadata.getAcls().size());
+      AclMetadata acl2 = bucketMetadata.getAcls().get(0);
+      assertEquals("TestGroup2", acl2.getName());
+      assertEquals("GROUP", acl2.getType());
+      assertEquals("ACCESS", acl2.getScope());
+      assertEquals(1, acl2.getAclList().size());
+      assertEquals("READ", acl2.getAclList().get(0));
+      break;
+    default:
+      Assertions.fail(String.format("Bucket %s not registered",
+          bucketName));
+    }
+
+  }
+
+  @Test
+  public void testGetBuckets() throws Exception {
+    // Normal bucket under a volume
+    Response response = bucketEndpoint.getBuckets("sampleVol2", 10, null);
+    Assertions.assertEquals(200, response.getStatus());
+    BucketsResponse bucketUnderVolumeResponse =
+        (BucketsResponse) response.getEntity();
+    Assertions.assertEquals(2, bucketUnderVolumeResponse.getTotalCount());
+    Assertions.assertEquals(2,
+        bucketUnderVolumeResponse.getBuckets().size());
+
+    bucketUnderVolumeResponse.getBuckets().forEach(bucketMetadata -> {
+      try {
+        testBucketResponse(bucketMetadata);
+      } catch (IOException e) {
+        Assertions.fail(e.getMessage());
+      }
+    });
+
+    // Listing all buckets
+    Response response3 = bucketEndpoint.getBuckets(null, 10, null);
+    Assertions.assertEquals(200, response3.getStatus());
+    BucketsResponse allBucketsResponse =
+        (BucketsResponse) response3.getEntity();
+
+    // Include bucketOne under sampleVol in
+    // A sample volume (sampleVol) and a bucket (bucketOne) is already created
+    // in initializeNewOmMetadataManager.
+    Assertions.assertEquals(3, allBucketsResponse.getTotalCount());
+    Assertions.assertEquals(3, allBucketsResponse.getBuckets().size());
+  }
+
+  @Test
+  public void testGetBucketsWithPrevKeyAndLimit() throws IOException {
+    // Case 1: Volume is not specified (prevKey is ignored)
+    // Test limit
+    Response responseLimitWithoutVolume = bucketEndpoint.getBuckets(null,
+        1, null);
+    Assertions.assertEquals(200, responseLimitWithoutVolume.getStatus());
+
+    BucketsResponse bucketsResponseLimitWithoutVolume =
+        (BucketsResponse) responseLimitWithoutVolume.getEntity();
+    Assertions.assertEquals(1,
+        bucketsResponseLimitWithoutVolume.getTotalCount());
+    Assertions.assertEquals(1,
+        bucketsResponseLimitWithoutVolume.getBuckets().size());
+
+    // Test prevKey (it should be ignored)
+    Response responsePrevKeyWithoutVolume = bucketEndpoint.getBuckets(
+        null, 3, "bucketOne");
+
+    Assertions.assertEquals(200,
+        responsePrevKeyWithoutVolume.getStatus());
+
+    BucketsResponse bucketResponsePrevKeyWithoutVolume =
+        (BucketsResponse) responsePrevKeyWithoutVolume.getEntity();
+    Assertions.assertEquals(3,
+        bucketResponsePrevKeyWithoutVolume.getTotalCount());
+    Assertions.assertEquals(3,
+        bucketResponsePrevKeyWithoutVolume.getBuckets().size());
+    Assertions.assertEquals("sampleVol",
+        bucketResponsePrevKeyWithoutVolume.getBuckets()
+            .stream().findFirst().get().getVolumeName());
+    Assertions.assertEquals("bucketOne",
+        bucketResponsePrevKeyWithoutVolume.getBuckets()
+            .stream().findFirst().get().getName());
+
+    // Case 2: Volume is specified (prevKey is not ignored)
+    // Test limit
+    Response responseLimitWithVolume = bucketEndpoint.getBuckets(
+        "sampleVol2", 1, null);
+
+    Assertions.assertEquals(200, responseLimitWithVolume.getStatus());
+
+    BucketsResponse bucketResponseLimitWithVolume =
+        (BucketsResponse) responseLimitWithVolume.getEntity();
+    Assertions.assertEquals(1,
+        bucketResponseLimitWithVolume.getTotalCount());
+    Assertions.assertEquals(1,
+        bucketResponseLimitWithVolume.getBuckets().size());
+
+    // Case 3: Test prevKey (it should not be ignored)
+    Response responsePrevKeyWithVolume = bucketEndpoint.getBuckets(
+        "sampleVol2", 1, "bucketOne");
+
+    Assertions.assertEquals(200, responsePrevKeyWithVolume.getStatus());
+
+    BucketsResponse bucketPrevKeyResponseWithVolume =
+        (BucketsResponse) responsePrevKeyWithVolume.getEntity();
+    Assertions.assertEquals(1,
+        bucketPrevKeyResponseWithVolume.getTotalCount());
+    Assertions.assertEquals(1,
+        bucketPrevKeyResponseWithVolume.getBuckets().size());
+    Assertions.assertEquals("sampleVol2",
+        bucketPrevKeyResponseWithVolume.getBuckets()
+            .stream().findFirst().get().getVolumeName());
+    Assertions.assertEquals("bucketTwo",
+        bucketPrevKeyResponseWithVolume.getBuckets()
+            .stream().findFirst().get().getName());
+
+    // Case 4: Test volume does not exist
+    Response responseVolumeNotExist = bucketEndpoint.getBuckets(
+        "sampleVol3", 100, "bucketOne");
+
+    Assertions.assertEquals(200, responseVolumeNotExist.getStatus());
+
+    BucketsResponse bucketVolumeNotExistResponse  =
+        (BucketsResponse) responseVolumeNotExist.getEntity();
+    Assertions.assertEquals(0,
+        bucketVolumeNotExistResponse.getTotalCount());
+    Assertions.assertEquals(0,
+        bucketVolumeNotExistResponse.getBuckets().size());
+
+  }
+
   private void waitAndCheckConditionAfterHeartbeat(Callable<Boolean> check)
       throws Exception {
     // if container report is processed first, and pipeline does not exist
