@@ -18,7 +18,11 @@
 
 package org.apache.hadoop.ozone.recon.api;
 
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.ONE;
+import static org.apache.hadoop.ozone.om.helpers.QuotaUtil.getReplicatedSize;
+
 import org.apache.hadoop.hdds.client.BlockID;
+import org.apache.hadoop.hdds.client.StandaloneReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReplicaProto.State;
@@ -26,6 +30,7 @@ import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerManager;
 import org.apache.hadoop.hdds.scm.container.ContainerNotFoundException;
 import org.apache.hadoop.hdds.scm.container.ContainerReplica;
+import org.apache.hadoop.hdds.scm.container.placement.metrics.SCMNodeStat;
 import org.apache.hadoop.hdds.scm.server.OzoneStorageContainerManager;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.om.OMConfigKeys;
@@ -40,29 +45,28 @@ import org.apache.hadoop.ozone.recon.ReconConstants;
 import org.apache.hadoop.ozone.recon.ReconTestInjector;
 import org.apache.hadoop.ozone.recon.api.handlers.BucketHandler;
 import org.apache.hadoop.ozone.recon.api.handlers.EntityHandler;
-import org.apache.hadoop.ozone.recon.api.types.NamespaceSummaryResponse;
 import org.apache.hadoop.ozone.recon.api.types.DUResponse;
-import org.apache.hadoop.ozone.recon.api.types.EntityType;
 import org.apache.hadoop.ozone.recon.api.types.FileSizeDistributionResponse;
 import org.apache.hadoop.ozone.recon.api.types.ResponseStatus;
 import org.apache.hadoop.ozone.recon.api.types.QuotaUsageResponse;
+import org.apache.hadoop.ozone.recon.common.CommonUtils;
 import org.apache.hadoop.ozone.recon.recovery.ReconOMMetadataManager;
+import org.apache.hadoop.ozone.recon.scm.ReconNodeManager;
 import org.apache.hadoop.ozone.recon.scm.ReconStorageContainerManagerFacade;
 import org.apache.hadoop.ozone.recon.spi.ReconNamespaceSummaryManager;
 import org.apache.hadoop.ozone.recon.spi.StorageContainerServiceProvider;
 import org.apache.hadoop.ozone.recon.spi.impl.OzoneManagerServiceProviderImpl;
 import org.apache.hadoop.ozone.recon.spi.impl.StorageContainerServiceProviderImpl;
 import org.apache.hadoop.ozone.recon.tasks.NSSummaryTaskWithLegacy;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Rule;
-import org.junit.Test;
-import org.junit.rules.TemporaryFolder;
-
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.io.TempDir;
 import javax.ws.rs.core.Response;
 
 import java.io.File;
 import java.io.IOException;
+import java.nio.file.Files;
+import java.nio.file.Path;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.List;
@@ -70,6 +74,8 @@ import java.util.ArrayList;
 import java.util.Set;
 import java.util.HashSet;
 
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertArrayEquals;
 import static org.apache.hadoop.hdds.protocol.MockDatanodeDetails.randomDatanodeDetails;
 import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_DB_DIRS;
 import static org.apache.hadoop.ozone.OzoneConsts.OM_KEY_PREFIX;
@@ -104,12 +110,13 @@ import static org.mockito.Mockito.when;
  * so there is no need to test process() on DB's updates
  */
 public class TestNSSummaryEndpointWithLegacy {
-  @Rule
-  public TemporaryFolder temporaryFolder = new TemporaryFolder();
+  @TempDir
+  private Path temporaryFolder;
 
   private ReconOMMetadataManager reconOMMetadataManager;
   private NSSummaryEndpoint nsSummaryEndpoint;
   private OzoneConfiguration conf;
+  private CommonUtils commonUtils;
 
   private static final String TEST_PATH_UTILITY =
       "/vol1/buck1/a/b/c/d/e/file1.txt";
@@ -213,46 +220,48 @@ public class TestNSSummaryEndpointWithLegacy {
   private static final long KEY_FOUR_SIZE = 2 * OzoneConsts.KB + 1; // bin 2
   private static final long KEY_FIVE_SIZE = 100L; // bin 0
   private static final long KEY_SIX_SIZE = 2 * OzoneConsts.KB + 1; // bin 2
+  private static final long KEY_SEVEN_SIZE = 4 * OzoneConsts.KB + 1;
   private static final long KEY_EIGHT_SIZE = OzoneConsts.KB + 1; // bin 1
   private static final long KEY_NINE_SIZE = 2 * OzoneConsts.KB + 1; // bin 2
   private static final long KEY_TEN_SIZE = 2 * OzoneConsts.KB + 1; // bin 2
   private static final long KEY_ELEVEN_SIZE = OzoneConsts.KB + 1; // bin 1
-  private static final long LOCATION_INFO_GROUP_ONE_SIZE
-          = CONTAINER_ONE_REPLICA_COUNT * BLOCK_ONE_LENGTH
-          + CONTAINER_TWO_REPLICA_COUNT * BLOCK_TWO_LENGTH
-          + CONTAINER_THREE_REPLICA_COUNT * BLOCK_THREE_LENGTH;
-
-  private static final long MULTI_BLOCK_KEY_SIZE_WITH_REPLICA
-          = LOCATION_INFO_GROUP_ONE_SIZE;
-
-  private static final long LOCATION_INFO_GROUP_TWO_SIZE
-      = CONTAINER_FOUR_REPLICA_COUNT * BLOCK_FOUR_LENGTH
-      + CONTAINER_FIVE_REPLICA_COUNT * BLOCK_FIVE_LENGTH
-      + CONTAINER_SIX_REPLICA_COUNT * BLOCK_SIX_LENGTH;
 
   private static final long FILE1_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_ONE_SIZE;
+      getReplicatedSize(KEY_ONE_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
   private static final long FILE2_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_TWO_SIZE;
+      getReplicatedSize(KEY_TWO_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
   private static final long FILE3_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_ONE_SIZE;
+      getReplicatedSize(KEY_THREE_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
   private static final long FILE4_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_TWO_SIZE;
+      getReplicatedSize(KEY_FOUR_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
   private static final long FILE5_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_ONE_SIZE;
+      getReplicatedSize(KEY_FIVE_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
   private static final long FILE6_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_TWO_SIZE;
+      getReplicatedSize(KEY_SIX_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));;
   private static final long FILE7_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_ONE_SIZE;
+      getReplicatedSize(KEY_SEVEN_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));;
   private static final long FILE8_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_TWO_SIZE;
+      getReplicatedSize(KEY_EIGHT_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
   private static final long FILE9_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_ONE_SIZE;
+      getReplicatedSize(KEY_NINE_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
   private static final long FILE10_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_TWO_SIZE;
+      getReplicatedSize(KEY_TEN_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
   private static final long FILE11_SIZE_WITH_REPLICA =
-      LOCATION_INFO_GROUP_ONE_SIZE;
+      getReplicatedSize(KEY_ELEVEN_SIZE,
+              StandaloneReplicationConfig.getInstance(ONE));
 
+  private static final long MULTI_BLOCK_KEY_SIZE_WITH_REPLICA
+          = FILE7_SIZE_WITH_REPLICA;
   private static final long
       MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_ROOT
       = FILE1_SIZE_WITH_REPLICA
@@ -343,18 +352,20 @@ public class TestNSSummaryEndpointWithLegacy {
   private static final long DIR_ONE_DATA_SIZE = KEY_TWO_SIZE +
       KEY_THREE_SIZE + KEY_SIX_SIZE;
 
-  @Before
+  @BeforeEach
   public void setUp() throws Exception {
     conf = new OzoneConfiguration();
     OMMetadataManager omMetadataManager = initializeNewOmMetadataManager(
-        temporaryFolder.newFolder(), conf);
+        Files.createDirectory(temporaryFolder.resolve(
+            "JunitOmDBDir")).toFile(), conf);
     OzoneManagerServiceProviderImpl ozoneManagerServiceProvider =
         getMockOzoneManagerServiceProvider();
-    reconOMMetadataManager = getTestReconOmMetadataManager(omMetadataManager, 
-        temporaryFolder.newFolder());
+    reconOMMetadataManager = getTestReconOmMetadataManager(omMetadataManager,
+        Files.createDirectory(temporaryFolder.resolve(
+            "omMetadatDir")).toFile());
 
     ReconTestInjector reconTestInjector =
-        new ReconTestInjector.Builder(temporaryFolder)
+        new ReconTestInjector.Builder(temporaryFolder.toFile())
             .withReconOm(reconOMMetadataManager)
             .withOmServiceProvider(ozoneManagerServiceProvider)
             .withReconSqlDb()
@@ -375,95 +386,63 @@ public class TestNSSummaryEndpointWithLegacy {
         new NSSummaryTaskWithLegacy(reconNamespaceSummaryManager, 
                                     reconOMMetadataManager, conf);
     nsSummaryTaskWithLegacy.reprocessWithLegacy(reconOMMetadataManager);
+    commonUtils = new CommonUtils();
   }
 
   @Test
   public void testUtility() {
     String[] names = EntityHandler.parseRequestPath(TEST_PATH_UTILITY);
-    Assert.assertArrayEquals(TEST_NAMES, names);
+    assertArrayEquals(TEST_NAMES, names);
     String keyName = BucketHandler.getKeyName(names);
-    Assert.assertEquals(TEST_KEY_NAMES, keyName);
+    assertEquals(TEST_KEY_NAMES, keyName);
     String subpath = BucketHandler.buildSubpath(PARENT_DIR, "file1.txt");
-    Assert.assertEquals(TEST_PATH_UTILITY, subpath);
+    assertEquals(TEST_PATH_UTILITY, subpath);
   }
 
   @Test
   public void testGetBasicInfoRoot() throws Exception {
-    // Test root basics
-    Response rootResponse = nsSummaryEndpoint.getBasicInfo(ROOT_PATH);
-    NamespaceSummaryResponse rootResponseObj =
-        (NamespaceSummaryResponse) rootResponse.getEntity();
-    Assert.assertEquals(EntityType.ROOT, rootResponseObj.getEntityType());
-    Assert.assertEquals(2, rootResponseObj.getNumVolume());
-    Assert.assertEquals(4, rootResponseObj.getNumBucket());
-    Assert.assertEquals(5, rootResponseObj.getNumTotalDir());
-    Assert.assertEquals(10, rootResponseObj.getNumTotalKey());
+    commonUtils.testNSSummaryBasicInfoRoot(
+        nsSummaryEndpoint, reconOMMetadataManager);
   }
 
   @Test
   public void testGetBasicInfoVol() throws Exception {
     // Test volume basics
-    Response volResponse = nsSummaryEndpoint.getBasicInfo(VOL_PATH);
-    NamespaceSummaryResponse volResponseObj =
-        (NamespaceSummaryResponse) volResponse.getEntity();
-    Assert.assertEquals(EntityType.VOLUME, volResponseObj.getEntityType());
-    Assert.assertEquals(2, volResponseObj.getNumBucket());
-    Assert.assertEquals(4, volResponseObj.getNumTotalDir());
-    Assert.assertEquals(6, volResponseObj.getNumTotalKey());
+    commonUtils.testNSSummaryBasicInfoVolume(nsSummaryEndpoint);
   }
 
   @Test
   public void testGetBasicInfoBucketOne() throws Exception {
     // Test bucket 1's basics
-    Response bucketOneResponse =
-        nsSummaryEndpoint.getBasicInfo(BUCKET_ONE_PATH);
-    NamespaceSummaryResponse bucketOneObj =
-        (NamespaceSummaryResponse) bucketOneResponse.getEntity();
-    Assert.assertEquals(EntityType.BUCKET, bucketOneObj.getEntityType());
-    Assert.assertEquals(4, bucketOneObj.getNumTotalDir());
-    Assert.assertEquals(4, bucketOneObj.getNumTotalKey());
+    commonUtils.testNSSummaryBasicInfoBucketOne(
+        BucketLayout.LEGACY,
+        nsSummaryEndpoint);
   }
 
   @Test
   public void testGetBasicInfoBucketTwo() throws Exception {
     // Test bucket 2's basics
-    Response bucketTwoResponse =
-        nsSummaryEndpoint.getBasicInfo(BUCKET_TWO_PATH);
-    NamespaceSummaryResponse bucketTwoObj =
-        (NamespaceSummaryResponse) bucketTwoResponse.getEntity();
-    Assert.assertEquals(EntityType.BUCKET, bucketTwoObj.getEntityType());
-    Assert.assertEquals(0, bucketTwoObj.getNumTotalDir());
-    Assert.assertEquals(2, bucketTwoObj.getNumTotalKey());
+    commonUtils.testNSSummaryBasicInfoBucketTwo(
+        BucketLayout.LEGACY,
+        nsSummaryEndpoint);
   }
 
   @Test
   public void testGetBasicInfoDir() throws Exception {
     // Test intermediate directory basics
-    Response dirOneResponse = nsSummaryEndpoint.getBasicInfo(DIR_ONE_PATH);
-    NamespaceSummaryResponse dirOneObj =
-        (NamespaceSummaryResponse) dirOneResponse.getEntity();
-    Assert.assertEquals(EntityType.DIRECTORY, dirOneObj.getEntityType());
-    Assert.assertEquals(3, dirOneObj.getNumTotalDir());
-    Assert.assertEquals(3, dirOneObj.getNumTotalKey());
+    commonUtils.testNSSummaryBasicInfoDir(nsSummaryEndpoint);
   }
 
   @Test
   public void testGetBasicInfoNoPath() throws Exception {
     // Test invalid path
-    Response invalidResponse = nsSummaryEndpoint.getBasicInfo(INVALID_PATH);
-    NamespaceSummaryResponse invalidObj =
-        (NamespaceSummaryResponse) invalidResponse.getEntity();
-    Assert.assertEquals(ResponseStatus.PATH_NOT_FOUND,
-        invalidObj.getStatus());
+    commonUtils.testNSSummaryBasicInfoNoPath(nsSummaryEndpoint);
   }
 
   @Test
   public void testGetBasicInfoKey() throws Exception {
     // Test key
-    Response keyResponse = nsSummaryEndpoint.getBasicInfo(KEY_PATH);
-    NamespaceSummaryResponse keyResObj =
-        (NamespaceSummaryResponse) keyResponse.getEntity();
-    Assert.assertEquals(EntityType.KEY, keyResObj.getEntityType());
+    commonUtils.testNSSummaryBasicInfoKey(nsSummaryEndpoint);
   }
 
   @Test
@@ -472,17 +451,17 @@ public class TestNSSummaryEndpointWithLegacy {
     Response rootResponse = nsSummaryEndpoint.getDiskUsage(ROOT_PATH,
         false, false);
     DUResponse duRootRes = (DUResponse) rootResponse.getEntity();
-    Assert.assertEquals(2, duRootRes.getCount());
+    assertEquals(2, duRootRes.getCount());
     List<DUResponse.DiskUsage> duRootData = duRootRes.getDuData();
     // sort based on subpath
     Collections.sort(duRootData,
         Comparator.comparing(DUResponse.DiskUsage::getSubpath));
     DUResponse.DiskUsage duVol1 = duRootData.get(0);
     DUResponse.DiskUsage duVol2 = duRootData.get(1);
-    Assert.assertEquals(VOL_PATH, duVol1.getSubpath());
-    Assert.assertEquals(VOL_TWO_PATH, duVol2.getSubpath());
-    Assert.assertEquals(VOL_DATA_SIZE, duVol1.getSize());
-    Assert.assertEquals(VOL_TWO_DATA_SIZE, duVol2.getSize());
+    assertEquals(VOL_PATH, duVol1.getSubpath());
+    assertEquals(VOL_TWO_PATH, duVol2.getSubpath());
+    assertEquals(VOL_DATA_SIZE, duVol1.getSize());
+    assertEquals(VOL_TWO_DATA_SIZE, duVol2.getSize());
   }
 
   @Test
@@ -491,17 +470,17 @@ public class TestNSSummaryEndpointWithLegacy {
     Response volResponse = nsSummaryEndpoint.getDiskUsage(VOL_PATH,
         false, false);
     DUResponse duVolRes = (DUResponse) volResponse.getEntity();
-    Assert.assertEquals(2, duVolRes.getCount());
+    assertEquals(2, duVolRes.getCount());
     List<DUResponse.DiskUsage> duData = duVolRes.getDuData();
     // sort based on subpath
     Collections.sort(duData,
         Comparator.comparing(DUResponse.DiskUsage::getSubpath));
     DUResponse.DiskUsage duBucket1 = duData.get(0);
     DUResponse.DiskUsage duBucket2 = duData.get(1);
-    Assert.assertEquals(BUCKET_ONE_PATH, duBucket1.getSubpath());
-    Assert.assertEquals(BUCKET_TWO_PATH, duBucket2.getSubpath());
-    Assert.assertEquals(BUCKET_ONE_DATA_SIZE, duBucket1.getSize());
-    Assert.assertEquals(BUCKET_TWO_DATA_SIZE, duBucket2.getSize());
+    assertEquals(BUCKET_ONE_PATH, duBucket1.getSubpath());
+    assertEquals(BUCKET_TWO_PATH, duBucket2.getSubpath());
+    assertEquals(BUCKET_ONE_DATA_SIZE, duBucket1.getSize());
+    assertEquals(BUCKET_TWO_DATA_SIZE, duBucket2.getSize());
   }
 
   @Test
@@ -510,10 +489,10 @@ public class TestNSSummaryEndpointWithLegacy {
     Response bucketResponse = nsSummaryEndpoint.getDiskUsage(BUCKET_ONE_PATH,
         false, false);
     DUResponse duBucketResponse = (DUResponse) bucketResponse.getEntity();
-    Assert.assertEquals(1, duBucketResponse.getCount());
+    assertEquals(1, duBucketResponse.getCount());
     DUResponse.DiskUsage duDir1 = duBucketResponse.getDuData().get(0);
-    Assert.assertEquals(DIR_ONE_PATH, duDir1.getSubpath());
-    Assert.assertEquals(DIR_ONE_DATA_SIZE, duDir1.getSize());
+    assertEquals(DIR_ONE_PATH, duDir1.getSubpath());
+    assertEquals(DIR_ONE_DATA_SIZE, duDir1.getSize());
   }
 
   @Test
@@ -522,21 +501,21 @@ public class TestNSSummaryEndpointWithLegacy {
     Response dirResponse = nsSummaryEndpoint.getDiskUsage(DIR_ONE_PATH,
         false, false);
     DUResponse duDirReponse = (DUResponse) dirResponse.getEntity();
-    Assert.assertEquals(3, duDirReponse.getCount());
+    assertEquals(3, duDirReponse.getCount());
     List<DUResponse.DiskUsage> duSubDir = duDirReponse.getDuData();
     Collections.sort(duSubDir,
         Comparator.comparing(DUResponse.DiskUsage::getSubpath));
     DUResponse.DiskUsage duDir2 = duSubDir.get(0);
     DUResponse.DiskUsage duDir3 = duSubDir.get(1);
     DUResponse.DiskUsage duDir4 = duSubDir.get(2);
-    Assert.assertEquals(DIR_TWO_PATH, duDir2.getSubpath());
-    Assert.assertEquals(KEY_TWO_SIZE, duDir2.getSize());
+    assertEquals(DIR_TWO_PATH, duDir2.getSubpath());
+    assertEquals(KEY_TWO_SIZE, duDir2.getSize());
 
-    Assert.assertEquals(DIR_THREE_PATH, duDir3.getSubpath());
-    Assert.assertEquals(KEY_THREE_SIZE, duDir3.getSize());
+    assertEquals(DIR_THREE_PATH, duDir3.getSubpath());
+    assertEquals(KEY_THREE_SIZE, duDir3.getSize());
 
-    Assert.assertEquals(DIR_FOUR_PATH, duDir4.getSubpath());
-    Assert.assertEquals(KEY_SIX_SIZE, duDir4.getSize());
+    assertEquals(DIR_FOUR_PATH, duDir4.getSubpath());
+    assertEquals(KEY_SIX_SIZE, duDir4.getSize());
   }
 
   @Test
@@ -545,8 +524,8 @@ public class TestNSSummaryEndpointWithLegacy {
     Response keyResponse = nsSummaryEndpoint.getDiskUsage(KEY_PATH,
         false, false);
     DUResponse keyObj = (DUResponse) keyResponse.getEntity();
-    Assert.assertEquals(0, keyObj.getCount());
-    Assert.assertEquals(KEY_FOUR_SIZE, keyObj.getSize());
+    assertEquals(0, keyObj.getCount());
+    assertEquals(KEY_FOUR_SIZE, keyObj.getSize());
   }
 
   @Test
@@ -555,7 +534,7 @@ public class TestNSSummaryEndpointWithLegacy {
     Response invalidResponse = nsSummaryEndpoint.getDiskUsage(INVALID_PATH,
         false, false);
     DUResponse invalidObj = (DUResponse) invalidResponse.getEntity();
-    Assert.assertEquals(ResponseStatus.PATH_NOT_FOUND,
+    assertEquals(ResponseStatus.PATH_NOT_FOUND,
         invalidObj.getStatus());
   }
 
@@ -565,8 +544,8 @@ public class TestNSSummaryEndpointWithLegacy {
     Response keyResponse = nsSummaryEndpoint.getDiskUsage(MULTI_BLOCK_KEY_PATH,
         false, true);
     DUResponse replicaDUResponse = (DUResponse) keyResponse.getEntity();
-    Assert.assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
-    Assert.assertEquals(MULTI_BLOCK_KEY_SIZE_WITH_REPLICA,
+    assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
+    assertEquals(MULTI_BLOCK_KEY_SIZE_WITH_REPLICA,
         replicaDUResponse.getSizeWithReplica());
   }
 
@@ -577,10 +556,10 @@ public class TestNSSummaryEndpointWithLegacy {
     Response rootResponse = nsSummaryEndpoint.getDiskUsage(ROOT_PATH,
         false, true);
     DUResponse replicaDUResponse = (DUResponse) rootResponse.getEntity();
-    Assert.assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_ROOT,
+    assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_ROOT,
         replicaDUResponse.getSizeWithReplica());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_VOL,
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_VOL,
         replicaDUResponse.getDuData().get(0).getSizeWithReplica());
 
   }
@@ -591,10 +570,10 @@ public class TestNSSummaryEndpointWithLegacy {
     Response volResponse = nsSummaryEndpoint.getDiskUsage(VOL_PATH,
         false, true);
     DUResponse replicaDUResponse = (DUResponse) volResponse.getEntity();
-    Assert.assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_VOL,
+    assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_VOL,
         replicaDUResponse.getSizeWithReplica());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_BUCKET1,
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_BUCKET1,
         replicaDUResponse.getDuData().get(0).getSizeWithReplica());
   }
 
@@ -604,10 +583,10 @@ public class TestNSSummaryEndpointWithLegacy {
     Response bucketResponse = nsSummaryEndpoint.getDiskUsage(BUCKET_ONE_PATH,
         false, true);
     DUResponse replicaDUResponse = (DUResponse) bucketResponse.getEntity();
-    Assert.assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_BUCKET1,
+    assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_BUCKET1,
         replicaDUResponse.getSizeWithReplica());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_DIR1,
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_DIR1,
         replicaDUResponse.getDuData().get(0).getSizeWithReplica());
   }
 
@@ -623,10 +602,10 @@ public class TestNSSummaryEndpointWithLegacy {
     Response dir1Response = nsSummaryEndpoint.getDiskUsage(DIR_ONE_PATH,
         false, true);
     DUResponse replicaDUResponse = (DUResponse) dir1Response.getEntity();
-    Assert.assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_DIR1,
+    assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_DIR1,
         replicaDUResponse.getSizeWithReplica());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_DIR2,
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_DIR2,
         replicaDUResponse.getDuData().get(0).getSizeWithReplica());
   }
 
@@ -636,8 +615,8 @@ public class TestNSSummaryEndpointWithLegacy {
     Response keyResponse = nsSummaryEndpoint.getDiskUsage(KEY_PATH,
         false, true);
     DUResponse replicaDUResponse = (DUResponse) keyResponse.getEntity();
-    Assert.assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
-    Assert.assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_KEY,
+    assertEquals(ResponseStatus.OK, replicaDUResponse.getStatus());
+    assertEquals(MULTI_BLOCK_TOTAL_SIZE_WITH_REPLICA_UNDER_KEY,
         replicaDUResponse.getSizeWithReplica());
   }
 
@@ -647,45 +626,45 @@ public class TestNSSummaryEndpointWithLegacy {
     Response rootResponse = nsSummaryEndpoint.getQuotaUsage(ROOT_PATH);
     QuotaUsageResponse quRootRes =
         (QuotaUsageResponse) rootResponse.getEntity();
-    Assert.assertEquals(ROOT_QUOTA, quRootRes.getQuota());
-    Assert.assertEquals(ROOT_DATA_SIZE, quRootRes.getQuotaUsed());
+    assertEquals(ROOT_QUOTA, quRootRes.getQuota());
+    assertEquals(ROOT_DATA_SIZE, quRootRes.getQuotaUsed());
 
     // volume level quota usage
     Response volResponse = nsSummaryEndpoint.getQuotaUsage(VOL_PATH);
     QuotaUsageResponse quVolRes = (QuotaUsageResponse) volResponse.getEntity();
-    Assert.assertEquals(VOL_QUOTA, quVolRes.getQuota());
-    Assert.assertEquals(VOL_DATA_SIZE, quVolRes.getQuotaUsed());
+    assertEquals(VOL_QUOTA, quVolRes.getQuota());
+    assertEquals(VOL_DATA_SIZE, quVolRes.getQuotaUsed());
 
     // bucket level quota usage
     Response bucketRes = nsSummaryEndpoint.getQuotaUsage(BUCKET_ONE_PATH);
     QuotaUsageResponse quBucketRes = (QuotaUsageResponse) bucketRes.getEntity();
-    Assert.assertEquals(BUCKET_ONE_QUOTA, quBucketRes.getQuota());
-    Assert.assertEquals(BUCKET_ONE_DATA_SIZE, quBucketRes.getQuotaUsed());
+    assertEquals(BUCKET_ONE_QUOTA, quBucketRes.getQuota());
+    assertEquals(BUCKET_ONE_DATA_SIZE, quBucketRes.getQuotaUsed());
 
     Response bucketRes2 = nsSummaryEndpoint.getQuotaUsage(BUCKET_TWO_PATH);
     QuotaUsageResponse quBucketRes2 =
         (QuotaUsageResponse) bucketRes2.getEntity();
-    Assert.assertEquals(BUCKET_TWO_QUOTA, quBucketRes2.getQuota());
-    Assert.assertEquals(BUCKET_TWO_DATA_SIZE, quBucketRes2.getQuotaUsed());
+    assertEquals(BUCKET_TWO_QUOTA, quBucketRes2.getQuota());
+    assertEquals(BUCKET_TWO_DATA_SIZE, quBucketRes2.getQuotaUsed());
 
     // other level not applicable
     Response naResponse1 = nsSummaryEndpoint.getQuotaUsage(DIR_ONE_PATH);
     QuotaUsageResponse quotaUsageResponse1 =
         (QuotaUsageResponse) naResponse1.getEntity();
-    Assert.assertEquals(ResponseStatus.TYPE_NOT_APPLICABLE,
+    assertEquals(ResponseStatus.TYPE_NOT_APPLICABLE,
         quotaUsageResponse1.getResponseCode());
 
     Response naResponse2 = nsSummaryEndpoint.getQuotaUsage(KEY_PATH);
     QuotaUsageResponse quotaUsageResponse2 =
         (QuotaUsageResponse) naResponse2.getEntity();
-    Assert.assertEquals(ResponseStatus.TYPE_NOT_APPLICABLE,
+    assertEquals(ResponseStatus.TYPE_NOT_APPLICABLE,
         quotaUsageResponse2.getResponseCode());
 
     // invalid path request
     Response invalidRes = nsSummaryEndpoint.getQuotaUsage(INVALID_PATH);
     QuotaUsageResponse invalidResObj =
         (QuotaUsageResponse) invalidRes.getEntity();
-    Assert.assertEquals(ResponseStatus.PATH_NOT_FOUND,
+    assertEquals(ResponseStatus.PATH_NOT_FOUND,
         invalidResObj.getResponseCode());
   }
 
@@ -704,12 +683,12 @@ public class TestNSSummaryEndpointWithLegacy {
     FileSizeDistributionResponse fileSizeDistResObj =
             (FileSizeDistributionResponse) res.getEntity();
     int[] fileSizeDist = fileSizeDistResObj.getFileSizeDist();
-    Assert.assertEquals(bin0, fileSizeDist[0]);
-    Assert.assertEquals(bin1, fileSizeDist[1]);
-    Assert.assertEquals(bin2, fileSizeDist[2]);
-    Assert.assertEquals(bin3, fileSizeDist[3]);
-    for (int i = 4; i < ReconConstants.NUM_OF_BINS; ++i) {
-      Assert.assertEquals(0, fileSizeDist[i]);
+    assertEquals(bin0, fileSizeDist[0]);
+    assertEquals(bin1, fileSizeDist[1]);
+    assertEquals(bin2, fileSizeDist[2]);
+    assertEquals(bin3, fileSizeDist[3]);
+    for (int i = 4; i < ReconConstants.NUM_OF_FILE_SIZE_BINS; ++i) {
+      assertEquals(0, fileSizeDist[i]);
     }
   }
 
@@ -897,7 +876,7 @@ public class TestNSSummaryEndpointWithLegacy {
     omConfiguration.set(OMConfigKeys
         .OZONE_OM_ENABLE_FILESYSTEM_PATHS, "true");
     OMMetadataManager omMetadataManager = new OmMetadataManagerImpl(
-        omConfiguration);
+        omConfiguration, null);
 
     String volumeKey = omMetadataManager.getVolumeKey(VOL);
     OmVolumeArgs args =
@@ -986,7 +965,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_ONE_OBJECT_ID,
         VOL_OBJECT_ID,
         Collections.singletonList(locationInfoGroup),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_SEVEN_SIZE);
   }
 
   private OmKeyLocationInfoGroup getLocationInfoGroup1() {
@@ -1079,7 +1059,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_ONE_OBJECT_ID,
         VOL_OBJECT_ID,
         Collections.singletonList(locationInfoGroup1),
-          getBucketLayout());
+          getBucketLayout(),
+          KEY_ONE_SIZE);
 
     //vol/bucket1/dir1/dir2/file2
     writeKeyToOm(reconOMMetadataManager,
@@ -1092,7 +1073,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_ONE_OBJECT_ID,
         VOL_OBJECT_ID,
         Collections.singletonList(locationInfoGroup2),
-          getBucketLayout());
+          getBucketLayout(),
+          KEY_TWO_SIZE);
 
     //vol/bucket1/dir1/dir3/file3
     writeKeyToOm(reconOMMetadataManager,
@@ -1105,7 +1087,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_ONE_OBJECT_ID,
         VOL_OBJECT_ID,
         Collections.singletonList(locationInfoGroup1),
-          getBucketLayout());
+          getBucketLayout(),
+          KEY_THREE_SIZE);
 
     //vol/bucket2/file4
     writeKeyToOm(reconOMMetadataManager,
@@ -1118,7 +1101,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_TWO_OBJECT_ID,
         VOL_OBJECT_ID,
         Collections.singletonList(locationInfoGroup2),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_FOUR_SIZE);
 
     //vol/bucket2/file5
     writeKeyToOm(reconOMMetadataManager,
@@ -1131,7 +1115,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_TWO_OBJECT_ID,
         VOL_OBJECT_ID,
         Collections.singletonList(locationInfoGroup1),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_FIVE_SIZE);
 
     //vol/bucket1/dir1/dir4/file6
     writeKeyToOm(reconOMMetadataManager,
@@ -1144,7 +1129,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_ONE_OBJECT_ID,
         VOL_OBJECT_ID,
         Collections.singletonList(locationInfoGroup2),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_SIX_SIZE);
 
     //vol/bucket1/dir1/file7
     writeKeyToOm(reconOMMetadataManager,
@@ -1157,7 +1143,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_ONE_OBJECT_ID,
         VOL_OBJECT_ID,
         Collections.singletonList(locationInfoGroup1),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_SEVEN_SIZE);
 
     //vol2/bucket3/file8
     writeKeyToOm(reconOMMetadataManager,
@@ -1170,7 +1157,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_THREE_OBJECT_ID,
         VOL_TWO_OBJECT_ID,
         Collections.singletonList(locationInfoGroup2),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_EIGHT_SIZE);
 
     //vol2/bucket3/dir5/file9
     writeKeyToOm(reconOMMetadataManager,
@@ -1183,7 +1171,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_THREE_OBJECT_ID,
         VOL_TWO_OBJECT_ID,
         Collections.singletonList(locationInfoGroup1),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_NINE_SIZE);
 
     //vol2/bucket3/dir5/file10
     writeKeyToOm(reconOMMetadataManager,
@@ -1196,7 +1185,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_THREE_OBJECT_ID,
         VOL_TWO_OBJECT_ID,
         Collections.singletonList(locationInfoGroup2),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_TEN_SIZE);
 
     //vol2/bucket4/file11
     writeKeyToOm(reconOMMetadataManager,
@@ -1209,7 +1199,8 @@ public class TestNSSummaryEndpointWithLegacy {
         BUCKET_FOUR_OBJECT_ID,
         VOL_TWO_OBJECT_ID,
         Collections.singletonList(locationInfoGroup1),
-        getBucketLayout());
+        getBucketLayout(),
+        KEY_ELEVEN_SIZE);
   }
 
   /**
@@ -1283,10 +1274,18 @@ public class TestNSSummaryEndpointWithLegacy {
         .thenReturn(containerReplicas6);
 
     when(reconSCM.getContainerManager()).thenReturn(containerManager);
+    ReconNodeManager mockReconNodeManager = mock(ReconNodeManager.class);
+    when(mockReconNodeManager.getStats()).thenReturn(getMockSCMRootStat());
+    when(reconSCM.getScmNodeManager()).thenReturn(mockReconNodeManager);
     return reconSCM;
   }
 
   private static BucketLayout getBucketLayout() {
     return BucketLayout.LEGACY;
   }
-}
\ No newline at end of file
+
+  private static SCMNodeStat getMockSCMRootStat() {
+    return new SCMNodeStat(ROOT_QUOTA, ROOT_DATA_SIZE, 
+        ROOT_QUOTA - ROOT_DATA_SIZE);
+  }
+}
