@@ -64,4 +64,8 @@ public final class S3Consts {
 
   public static final String CUSTOM_METADATA_HEADER_PREFIX = "x-amz-meta-";
 
+
+  public static final String DECODED_CONTENT_LENGTH_HEADER =
+      "x-amz-decoded-content-length";
+
 }
