@@ -28,14 +28,17 @@ import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientStub;
 import org.apache.hadoop.ozone.s3.exception.OS3Exception;
 
+import static java.net.HttpURLConnection.HTTP_CONFLICT;
 import static java.net.HttpURLConnection.HTTP_NOT_FOUND;
+import static org.apache.hadoop.ozone.s3.exception.S3ErrorTable.BUCKET_ALREADY_EXISTS;
 import static org.apache.hadoop.ozone.s3.exception.S3ErrorTable.MALFORMED_HEADER;
 import static org.apache.hadoop.ozone.s3.signature.SignatureProcessor.DATE_FORMATTER;
-import org.junit.Assert;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
-import org.junit.Before;
-import org.junit.Test;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 /**
  * This class test Create Bucket functionality.
@@ -46,7 +49,7 @@ public class TestBucketPut {
   private OzoneClient clientStub;
   private BucketEndpoint bucketEndpoint;
 
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
 
     //Create client stub and object store stub.
@@ -59,12 +62,11 @@ public class TestBucketPut {
 
   @Test
   public void testBucketFailWithAuthHeaderMissing() throws Exception {
-
     try {
       bucketEndpoint.put(bucketName, null, null, null);
     } catch (OS3Exception ex) {
-      Assert.assertEquals(HTTP_NOT_FOUND, ex.getHttpCode());
-      Assert.assertEquals(MALFORMED_HEADER.getCode(), ex.getCode());
+      assertEquals(HTTP_NOT_FOUND, ex.getHttpCode());
+      assertEquals(MALFORMED_HEADER.getCode(), ex.getCode());
     }
   }
 
@@ -73,6 +75,12 @@ public class TestBucketPut {
     Response response = bucketEndpoint.put(bucketName, null, null, null);
     assertEquals(200, response.getStatus());
     assertNotNull(response.getLocation());
+
+    // Create-bucket on an existing bucket fails
+    OS3Exception e = assertThrows(OS3Exception.class, () -> bucketEndpoint.put(
+        bucketName, null, null, null));
+    assertEquals(HTTP_CONFLICT, e.getHttpCode());
+    assertEquals(BUCKET_ALREADY_EXISTS.getCode(), e.getCode());
   }
 
   @Test
@@ -80,8 +88,8 @@ public class TestBucketPut {
     try {
       bucketEndpoint.put(bucketName, null, null, null);
     } catch (OS3Exception ex) {
-      Assert.assertEquals(HTTP_NOT_FOUND, ex.getHttpCode());
-      Assert.assertEquals(MALFORMED_HEADER.getCode(), ex.getCode());
+      assertEquals(HTTP_NOT_FOUND, ex.getHttpCode());
+      assertEquals(MALFORMED_HEADER.getCode(), ex.getCode());
     }
   }
 
