@@ -22,7 +22,7 @@ package org.apache.hadoop.ozone.s3.endpoint;
 import javax.xml.bind.JAXBContext;
 import javax.xml.bind.JAXBException;
 
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
 
 /**
  * Testing JAXB serialization.
@@ -35,4 +35,4 @@ public class TestBucketResponse {
     context.createMarshaller().marshal(new ListObjectResponse(), System.out);
   }
 
-}
\ No newline at end of file
+}
