@@ -24,9 +24,8 @@ import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientStub;
 import org.apache.hadoop.ozone.s3.exception.OS3Exception;
 import org.apache.hadoop.ozone.s3.exception.S3ErrorTable;
-import org.junit.Assert;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 
 import javax.ws.rs.core.HttpHeaders;
@@ -36,9 +35,11 @@ import java.io.ByteArrayInputStream;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.hadoop.ozone.s3.util.S3Consts.STORAGE_CLASS_HEADER;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotNull;
 import static org.mockito.Mockito.when;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
 
 /**
  * This class test list parts request.
@@ -49,7 +50,7 @@ public class TestListParts {
   private static final ObjectEndpoint REST = new ObjectEndpoint();
   private static String uploadID;
 
-  @BeforeClass
+  @BeforeAll
   public static void setUp() throws Exception {
 
     OzoneClient client = new OzoneClientStub();
@@ -93,33 +94,33 @@ public class TestListParts {
 
   @Test
   public void testListParts() throws Exception {
-    Response response = REST.get(OzoneConsts.S3_BUCKET, OzoneConsts.KEY,
-        uploadID, 3, "0", null);
+    Response response = REST.get(OzoneConsts.S3_BUCKET, OzoneConsts.KEY, 0,
+        uploadID, 3, "0");
 
     ListPartsResponse listPartsResponse =
         (ListPartsResponse) response.getEntity();
 
-    Assert.assertFalse(listPartsResponse.getTruncated());
+    assertFalse(listPartsResponse.getTruncated());
     assertEquals(3, listPartsResponse.getPartList().size());
 
   }
 
   @Test
   public void testListPartsContinuation() throws Exception {
-    Response response = REST.get(OzoneConsts.S3_BUCKET, OzoneConsts.KEY,
-        uploadID, 2, "0", null);
+    Response response = REST.get(OzoneConsts.S3_BUCKET, OzoneConsts.KEY, 0,
+        uploadID, 2, "0");
     ListPartsResponse listPartsResponse =
         (ListPartsResponse) response.getEntity();
 
-    Assert.assertTrue(listPartsResponse.getTruncated());
+    assertTrue(listPartsResponse.getTruncated());
     assertEquals(2, listPartsResponse.getPartList().size());
 
     // Continue
-    response = REST.get(OzoneConsts.S3_BUCKET, OzoneConsts.KEY, uploadID, 2,
-        Integer.toString(listPartsResponse.getNextPartNumberMarker()), null);
+    response = REST.get(OzoneConsts.S3_BUCKET, OzoneConsts.KEY, 0, uploadID, 2,
+        Integer.toString(listPartsResponse.getNextPartNumberMarker()));
     listPartsResponse = (ListPartsResponse) response.getEntity();
 
-    Assert.assertFalse(listPartsResponse.getTruncated());
+    assertFalse(listPartsResponse.getTruncated());
     assertEquals(1, listPartsResponse.getPartList().size());
 
   }
@@ -127,10 +128,10 @@ public class TestListParts {
   @Test
   public void testListPartsWithUnknownUploadID() throws Exception {
     try {
-      REST.get(OzoneConsts.S3_BUCKET, OzoneConsts.KEY,
-          uploadID, 2, "0", null);
+      REST.get(OzoneConsts.S3_BUCKET, OzoneConsts.KEY, 0,
+          uploadID, 2, "0");
     } catch (OS3Exception ex) {
-      Assert.assertEquals(S3ErrorTable.NO_SUCH_UPLOAD.getErrorMessage(),
+      assertEquals(S3ErrorTable.NO_SUCH_UPLOAD.getErrorMessage(),
           ex.getErrorMessage());
     }
   }
