@@ -27,8 +27,8 @@ import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientStub;
 import org.apache.hadoop.ozone.s3.exception.OS3Exception;
 
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Assertions;
 
 /**
  * Test delete object.
@@ -54,7 +54,7 @@ public class TestObjectDelete {
     rest.delete("b1", "key1", null);
 
     //THEN
-    Assert.assertFalse("Bucket Should not contain any key after delete",
-        bucket.listKeys("").hasNext());
+    Assertions.assertFalse(bucket.listKeys("").hasNext(),
+        "Bucket Should not contain any key after delete");
   }
 }
