@@ -20,33 +20,44 @@
 
 package org.apache.hadoop.ozone.s3.endpoint;
 
-import javax.ws.rs.core.HttpHeaders;
-import javax.ws.rs.core.Response;
 import java.io.ByteArrayInputStream;
 import java.io.IOException;
-
+import java.io.InputStream;
+import javax.ws.rs.core.HttpHeaders;
+import javax.ws.rs.core.Response;
+import org.apache.commons.io.IOUtils;
 import org.apache.hadoop.hdds.client.ECReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationType;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
+import org.apache.hadoop.ozone.client.ObjectStore;
+import org.apache.hadoop.ozone.client.OzoneBucket;
 import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientStub;
 import org.apache.hadoop.ozone.client.OzoneKeyDetails;
+import org.apache.hadoop.ozone.client.OzoneVolume;
 import org.apache.hadoop.ozone.client.io.OzoneInputStream;
+import org.apache.hadoop.ozone.client.protocol.ClientProtocol;
+import org.apache.hadoop.ozone.om.exceptions.OMException;
+import org.apache.hadoop.ozone.om.helpers.BucketLayout;
 import org.apache.hadoop.ozone.s3.exception.OS3Exception;
-
-import org.apache.commons.io.IOUtils;
 import org.apache.hadoop.ozone.s3.exception.S3ErrorTable;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
+import org.apache.http.HttpStatus;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 
 import static java.nio.charset.StandardCharsets.UTF_8;
+import static org.apache.hadoop.ozone.s3.util.S3Consts.DECODED_CONTENT_LENGTH_HEADER;
 import static org.apache.hadoop.ozone.s3.util.S3Consts.COPY_SOURCE_HEADER;
 import static org.apache.hadoop.ozone.s3.util.S3Consts.STORAGE_CLASS_HEADER;
 import static org.apache.hadoop.ozone.s3.util.S3Utils.urlEncode;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.fail;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertThrows;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.ArgumentMatchers.eq;
+import static org.mockito.Mockito.doThrow;
+import static org.mockito.Mockito.times;
 import static org.mockito.Mockito.when;
 
 /**
@@ -62,7 +73,7 @@ public class TestObjectPut {
   private OzoneClient clientStub;
   private ObjectEndpoint objectEndpoint;
 
-  @Before
+  @BeforeEach
   public void setup() throws IOException {
     //Create client stub and object store stub.
     clientStub = new OzoneClientStub();
@@ -97,8 +108,8 @@ public class TestObjectPut {
     String keyContent =
         IOUtils.toString(ozoneInputStream, UTF_8);
 
-    Assert.assertEquals(200, response.getStatus());
-    Assert.assertEquals(CONTENT, keyContent);
+    assertEquals(200, response.getStatus());
+    assertEquals(CONTENT, keyContent);
   }
 
   @Test
@@ -116,7 +127,7 @@ public class TestObjectPut {
     Response response = objectEndpoint.put(bucketName, keyName, CONTENT
         .length(), 1, null, body);
 
-    Assert.assertEquals(ecReplicationConfig,
+    assertEquals(ecReplicationConfig,
         clientStub.getObjectStore().getS3Bucket(bucketName).getKey(keyName)
             .getReplicationConfig());
     OzoneInputStream ozoneInputStream =
@@ -125,8 +136,49 @@ public class TestObjectPut {
     String keyContent =
         IOUtils.toString(ozoneInputStream, UTF_8);
 
-    Assert.assertEquals(200, response.getStatus());
-    Assert.assertEquals(CONTENT, keyContent);
+    assertEquals(200, response.getStatus());
+    assertEquals(CONTENT, keyContent);
+  }
+
+  @Test
+  public void testPutObjectContentLength() throws IOException, OS3Exception {
+    // The contentLength specified when creating the Key should be the same as
+    // the Content-Length, the key Commit will compare the Content-Length with
+    // the actual length of the data written.
+    HttpHeaders headers = Mockito.mock(HttpHeaders.class);
+    ByteArrayInputStream body =
+        new ByteArrayInputStream(CONTENT.getBytes(UTF_8));
+    objectEndpoint.setHeaders(headers);
+    long dataSize = CONTENT.length();
+
+    objectEndpoint.put(bucketName, keyName, dataSize, 0, null, body);
+    assertEquals(dataSize, getKeyDataSize(keyName));
+  }
+
+  @Test
+  public void testPutObjectContentLengthForStreaming()
+      throws IOException, OS3Exception {
+    HttpHeaders headers = Mockito.mock(HttpHeaders.class);
+    objectEndpoint.setHeaders(headers);
+
+    String chunkedContent = "0a;chunk-signature=signature\r\n"
+        + "1234567890\r\n"
+        + "05;chunk-signature=signature\r\n"
+        + "abcde\r\n";
+
+    when(headers.getHeaderString("x-amz-content-sha256"))
+        .thenReturn("STREAMING-AWS4-HMAC-SHA256-PAYLOAD");
+
+    when(headers.getHeaderString(DECODED_CONTENT_LENGTH_HEADER))
+        .thenReturn("15");
+    objectEndpoint.put(bucketName, keyName, chunkedContent.length(), 0, null,
+        new ByteArrayInputStream(chunkedContent.getBytes(UTF_8)));
+    assertEquals(15, getKeyDataSize(keyName));
+  }
+
+  private long getKeyDataSize(String key) throws IOException {
+    return clientStub.getObjectStore().getS3Bucket(bucketName)
+        .getKey(key).getDataSize();
   }
 
   @Test
@@ -142,6 +194,8 @@ public class TestObjectPut {
 
     when(headers.getHeaderString("x-amz-content-sha256"))
         .thenReturn("STREAMING-AWS4-HMAC-SHA256-PAYLOAD");
+    when(headers.getHeaderString(DECODED_CONTENT_LENGTH_HEADER))
+        .thenReturn("15");
 
     //WHEN
     Response response = objectEndpoint.put(bucketName, keyName,
@@ -154,8 +208,8 @@ public class TestObjectPut {
             .readKey(keyName);
     String keyContent = IOUtils.toString(ozoneInputStream, UTF_8);
 
-    Assert.assertEquals(200, response.getStatus());
-    Assert.assertEquals("1234567890abcde", keyContent);
+    assertEquals(200, response.getStatus());
+    assertEquals("1234567890abcde", keyContent);
   }
 
   @Test
@@ -176,8 +230,8 @@ public class TestObjectPut {
 
     String keyContent = IOUtils.toString(ozoneInputStream, UTF_8);
 
-    Assert.assertEquals(200, response.getStatus());
-    Assert.assertEquals(CONTENT, keyContent);
+    assertEquals(200, response.getStatus());
+    assertEquals(CONTENT, keyContent);
 
 
     // Add copy header, and then call put
@@ -193,60 +247,43 @@ public class TestObjectPut {
 
     keyContent = IOUtils.toString(ozoneInputStream, UTF_8);
 
-    Assert.assertEquals(200, response.getStatus());
-    Assert.assertEquals(CONTENT, keyContent);
+    assertEquals(200, response.getStatus());
+    assertEquals(CONTENT, keyContent);
 
     // source and dest same
-    try {
-      objectEndpoint.put(bucketName, keyName, CONTENT.length(), 1, null, body);
-      fail("test copy object failed");
-    } catch (OS3Exception ex) {
-      Assert.assertTrue(ex.getErrorMessage().contains("This copy request is " +
-          "illegal"));
-    }
+    OS3Exception e = assertThrows(OS3Exception.class, () -> objectEndpoint.put(
+        bucketName, keyName, CONTENT.length(), 1, null, body),
+        "test copy object failed");
+    assertTrue(e.getErrorMessage().contains("This copy request is illegal"));
 
     // source bucket not found
-    try {
-      when(headers.getHeaderString(COPY_SOURCE_HEADER)).thenReturn(
-          nonexist + "/"  + urlEncode(keyName));
-      objectEndpoint.put(destBucket, destkey, CONTENT.length(), 1, null,
-          body);
-      fail("test copy object failed");
-    } catch (OS3Exception ex) {
-      Assert.assertTrue(ex.getCode().contains("NoSuchBucket"));
-    }
+    when(headers.getHeaderString(COPY_SOURCE_HEADER)).thenReturn(
+        nonexist + "/"  + urlEncode(keyName));
+    e = assertThrows(OS3Exception.class, () -> objectEndpoint.put(destBucket,
+        destkey, CONTENT.length(), 1, null, body), "test copy object failed");
+    assertTrue(e.getCode().contains("NoSuchBucket"));
 
     // dest bucket not found
-    try {
-      when(headers.getHeaderString(COPY_SOURCE_HEADER)).thenReturn(
-          bucketName + "/" + urlEncode(keyName));
-      objectEndpoint.put(nonexist, destkey, CONTENT.length(), 1, null, body);
-      fail("test copy object failed");
-    } catch (OS3Exception ex) {
-      Assert.assertTrue(ex.getCode().contains("NoSuchBucket"));
-    }
+    when(headers.getHeaderString(COPY_SOURCE_HEADER)).thenReturn(
+        bucketName + "/" + urlEncode(keyName));
+    e = assertThrows(OS3Exception.class, () -> objectEndpoint.put(nonexist,
+        destkey, CONTENT.length(), 1, null, body), "test copy object failed");
+    assertTrue(e.getCode().contains("NoSuchBucket"));
 
     //Both source and dest bucket not found
-    try {
-      when(headers.getHeaderString(COPY_SOURCE_HEADER)).thenReturn(
-          nonexist + "/" + urlEncode(keyName));
-      objectEndpoint.put(nonexist, destkey, CONTENT.length(), 1, null, body);
-      fail("test copy object failed");
-    } catch (OS3Exception ex) {
-      Assert.assertTrue(ex.getCode().contains("NoSuchBucket"));
-    }
+    when(headers.getHeaderString(COPY_SOURCE_HEADER)).thenReturn(
+        nonexist + "/" + urlEncode(keyName));
+    e = assertThrows(OS3Exception.class, () -> objectEndpoint.put(nonexist,
+        destkey, CONTENT.length(), 1, null, body), "test copy object failed");
+    assertTrue(e.getCode().contains("NoSuchBucket"));
 
     // source key not found
-    try {
-      when(headers.getHeaderString(COPY_SOURCE_HEADER)).thenReturn(
-          bucketName + "/" + urlEncode(nonexist));
-      objectEndpoint.put("nonexistent", keyName, CONTENT.length(), 1,
-          null, body);
-      fail("test copy object failed");
-    } catch (OS3Exception ex) {
-      Assert.assertTrue(ex.getCode().contains("NoSuchBucket"));
-    }
-
+    when(headers.getHeaderString(COPY_SOURCE_HEADER)).thenReturn(
+        bucketName + "/" + urlEncode(nonexist));
+    e = assertThrows(OS3Exception.class, () -> objectEndpoint.put(
+        "nonexistent", keyName, CONTENT.length(), 1, null, body),
+        "test copy object failed");
+    assertTrue(e.getCode().contains("NoSuchBucket"));
   }
 
   @Test
@@ -258,15 +295,11 @@ public class TestObjectPut {
     keyName = "sourceKey";
     when(headers.getHeaderString(STORAGE_CLASS_HEADER)).thenReturn("random");
 
-    try {
-      objectEndpoint.put(bucketName, keyName,
-          CONTENT.length(), 1, null, body);
-      fail("testInvalidStorageType");
-    } catch (OS3Exception ex) {
-      assertEquals(S3ErrorTable.INVALID_ARGUMENT.getErrorMessage(),
-          ex.getErrorMessage());
-      assertEquals("random", ex.getResource());
-    }
+    OS3Exception e = assertThrows(OS3Exception.class, () -> objectEndpoint.put(
+        bucketName, keyName, CONTENT.length(), 1, null, body));
+    assertEquals(S3ErrorTable.INVALID_ARGUMENT.getErrorMessage(),
+        e.getErrorMessage());
+    assertEquals("random", e.getResource());
   }
 
   @Test
@@ -286,6 +319,81 @@ public class TestObjectPut {
 
 
     //default type is set
-    Assert.assertEquals(ReplicationType.RATIS, key.getReplicationType());
+    assertEquals(ReplicationType.RATIS, key.getReplicationType());
+  }
+
+  @Test
+  public void testDirectoryCreation() throws IOException,
+      OS3Exception {
+    // GIVEN
+    final String path = "dir";
+    final long length = 0L;
+    final int partNumber = 0;
+    final String uploadId = "";
+    final InputStream body = null;
+    final HttpHeaders headers = Mockito.mock(HttpHeaders.class);
+    final ObjectEndpoint objEndpoint = new ObjectEndpoint();
+    objEndpoint.setOzoneConfiguration(new OzoneConfiguration());
+    objEndpoint.setHeaders(headers);
+    final OzoneClient client = Mockito.mock(OzoneClient.class);
+    objEndpoint.setClient(client);
+    final ObjectStore objectStore = Mockito.mock(ObjectStore.class);
+    final OzoneVolume volume = Mockito.mock(OzoneVolume.class);
+    final OzoneBucket bucket = Mockito.mock(OzoneBucket.class);
+    final ClientProtocol protocol = Mockito.mock(ClientProtocol.class);
+
+    // WHEN
+    when(client.getObjectStore()).thenReturn(objectStore);
+    when(client.getObjectStore().getS3Volume()).thenReturn(volume);
+    when(volume.getBucket(bucketName)).thenReturn(bucket);
+    when(bucket.getBucketLayout())
+        .thenReturn(BucketLayout.FILE_SYSTEM_OPTIMIZED);
+    when(client.getProxy()).thenReturn(protocol);
+    final Response response = objEndpoint.put(bucketName, path, length,
+        partNumber, uploadId, body);
+
+    // THEN
+    assertEquals(HttpStatus.SC_OK, response.getStatus());
+    Mockito.verify(protocol).createDirectory(any(), eq(bucketName), eq(path));
+  }
+
+  @Test
+  public void testDirectoryCreationOverFile() throws IOException {
+    // GIVEN
+    final String path = "key";
+    final long length = 0L;
+    final int partNumber = 0;
+    final String uploadId = "";
+    final ByteArrayInputStream body =
+        new ByteArrayInputStream("content".getBytes(UTF_8));
+    final HttpHeaders headers = Mockito.mock(HttpHeaders.class);
+    final ObjectEndpoint objEndpoint = new ObjectEndpoint();
+    objEndpoint.setOzoneConfiguration(new OzoneConfiguration());
+    objEndpoint.setHeaders(headers);
+    final OzoneClient client = Mockito.mock(OzoneClient.class);
+    objEndpoint.setClient(client);
+    final ObjectStore objectStore = Mockito.mock(ObjectStore.class);
+    final OzoneVolume volume = Mockito.mock(OzoneVolume.class);
+    final OzoneBucket bucket = Mockito.mock(OzoneBucket.class);
+    final ClientProtocol protocol = Mockito.mock(ClientProtocol.class);
+
+    // WHEN
+    when(client.getObjectStore()).thenReturn(objectStore);
+    when(client.getObjectStore().getS3Volume()).thenReturn(volume);
+    when(volume.getBucket(bucketName)).thenReturn(bucket);
+    when(bucket.getBucketLayout())
+        .thenReturn(BucketLayout.FILE_SYSTEM_OPTIMIZED);
+    when(client.getProxy()).thenReturn(protocol);
+    doThrow(new OMException(OMException.ResultCodes.FILE_ALREADY_EXISTS))
+        .when(protocol)
+        .createDirectory(any(), any(), any());
+
+    // THEN
+    final OS3Exception exception = assertThrows(OS3Exception.class,
+        () -> objEndpoint
+            .put(bucketName, path, length, partNumber, uploadId, body));
+    assertEquals("Conflict", exception.getCode());
+    assertEquals(409, exception.getHttpCode());
+    Mockito.verify(protocol, times(1)).createDirectory(any(), any(), any());
   }
 }
