@@ -24,23 +24,27 @@ import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientStub;
+import org.apache.hadoop.ozone.client.OzoneMultipartUploadPartListParts;
 import org.apache.hadoop.ozone.s3.exception.OS3Exception;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mockito;
 
 import javax.ws.rs.core.HttpHeaders;
 import javax.ws.rs.core.Response;
 
 import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.util.UUID;
 
 import static java.net.HttpURLConnection.HTTP_NOT_FOUND;
 import static java.nio.charset.StandardCharsets.UTF_8;
+import static org.apache.hadoop.ozone.s3.util.S3Consts.DECODED_CONTENT_LENGTH_HEADER;
 import static org.apache.hadoop.ozone.s3.util.S3Consts.STORAGE_CLASS_HEADER;
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.assertNotEquals;
-import static org.junit.Assert.assertNotNull;
-import static org.junit.Assert.fail;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNotEquals;
+import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.fail;
 import static org.mockito.Mockito.when;
 
 /**
@@ -49,11 +53,12 @@ import static org.mockito.Mockito.when;
 public class TestPartUpload {
 
   private static final ObjectEndpoint REST = new ObjectEndpoint();
+  private static OzoneClient client;
 
-  @BeforeClass
+  @BeforeAll
   public static void setUp() throws Exception {
 
-    OzoneClient client = new OzoneClientStub();
+    client = new OzoneClientStub();
     client.getObjectStore().createS3Bucket(OzoneConsts.S3_BUCKET);
 
 
@@ -135,4 +140,69 @@ public class TestPartUpload {
       assertEquals(HTTP_NOT_FOUND, ex.getHttpCode());
     }
   }
+
+  @Test
+  public void testPartUploadStreamContentLength()
+      throws IOException, OS3Exception {
+    HttpHeaders headers = Mockito.mock(HttpHeaders.class);
+    ObjectEndpoint objectEndpoint = new ObjectEndpoint();
+    objectEndpoint.setHeaders(headers);
+    objectEndpoint.setClient(client);
+    objectEndpoint.setOzoneConfiguration(new OzoneConfiguration());
+    String keyName = UUID.randomUUID().toString();
+
+    String chunkedContent = "0a;chunk-signature=signature\r\n"
+        + "1234567890\r\n"
+        + "05;chunk-signature=signature\r\n"
+        + "abcde\r\n";
+    when(headers.getHeaderString("x-amz-content-sha256"))
+        .thenReturn("STREAMING-AWS4-HMAC-SHA256-PAYLOAD");
+    when(headers.getHeaderString(DECODED_CONTENT_LENGTH_HEADER))
+        .thenReturn("15");
+
+    Response response = objectEndpoint.initializeMultipartUpload(
+        OzoneConsts.S3_BUCKET, keyName);
+    MultipartUploadInitiateResponse multipartUploadInitiateResponse =
+        (MultipartUploadInitiateResponse) response.getEntity();
+    assertNotNull(multipartUploadInitiateResponse.getUploadID());
+    String uploadID = multipartUploadInitiateResponse.getUploadID();
+    long contentLength = chunkedContent.length();
+
+    objectEndpoint.put(OzoneConsts.S3_BUCKET, keyName, contentLength, 1,
+        uploadID, new ByteArrayInputStream(chunkedContent.getBytes(UTF_8)));
+    assertContentLength(uploadID, keyName, 15);
+  }
+
+  @Test
+  public void testPartUploadContentLength() throws IOException, OS3Exception {
+    // The contentLength specified when creating the Key should be the same as
+    // the Content-Length, the key Commit will compare the Content-Length with
+    // the actual length of the data written.
+
+    String keyName = UUID.randomUUID().toString();
+    Response response = REST.initializeMultipartUpload(OzoneConsts.S3_BUCKET,
+        keyName);
+    MultipartUploadInitiateResponse multipartUploadInitiateResponse =
+        (MultipartUploadInitiateResponse) response.getEntity();
+    assertNotNull(multipartUploadInitiateResponse.getUploadID());
+    String uploadID = multipartUploadInitiateResponse.getUploadID();
+    String content = "Multipart Upload";
+    long contentLength = content.length();
+
+    ByteArrayInputStream body =
+        new ByteArrayInputStream(content.getBytes(UTF_8));
+    REST.put(OzoneConsts.S3_BUCKET, keyName,
+        contentLength, 1, uploadID, body);
+    assertContentLength(uploadID, keyName, content.length());
+  }
+
+  private void assertContentLength(String uploadID, String key,
+      long contentLength) throws IOException {
+    OzoneMultipartUploadPartListParts parts =
+        client.getObjectStore().getS3Bucket(OzoneConsts.S3_BUCKET)
+            .listParts(key, uploadID, 0, 100);
+    assertEquals(1, parts.getPartInfoList().size());
+    assertEquals(contentLength,
+        parts.getPartInfoList().get(0).getSize());
+  }
 }
