@@ -19,18 +19,14 @@
 package org.apache.hadoop.ozone.s3.signature;
 
 import java.time.LocalDate;
-
-import org.apache.ozone.test.LambdaTestUtils;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 
 import static java.time.temporal.ChronoUnit.DAYS;
 import static org.apache.hadoop.ozone.s3.signature.SignatureProcessor.DATE_FORMATTER;
-
-import org.junit.Assert;
-
-import static org.junit.Assert.assertEquals;
-
-import org.junit.Before;
-import org.junit.Test;
+import static org.junit.jupiter.api.Assertions.assertEquals;
+import static org.junit.jupiter.api.Assertions.assertNull;
+import static org.junit.jupiter.api.Assertions.assertThrows;
 
 /**
  * This class tests Authorization header format v2.
@@ -42,7 +38,7 @@ public class TestAuthorizationV4HeaderParser {
 
   private String curDate;
 
-  @Before
+  @BeforeEach
   public void setup() {
     LocalDate now = LocalDate.now();
     curDate = DATE_FORMATTER.format(now);
@@ -64,26 +60,25 @@ public class TestAuthorizationV4HeaderParser {
         signatureInfo.getSignature());
   }
 
-  @Test(expected = MalformedResourceException.class)
-  public void testV4HeaderMissingParts() throws MalformedResourceException {
+  @Test
+  public void testV4HeaderMissingParts() {
     String auth = "AWS4-HMAC-SHA256 " +
         "Credential=ozone/" + curDate + "/us-east-1/s3/aws4_request, " +
         "SignedHeaders=host;range;x-amz-date,";
     AuthorizationV4HeaderParser v4 =
         new AuthorizationV4HeaderParser(auth, SAMPLE_DATE);
-    v4.parseSignature();
+    assertThrows(MalformedResourceException.class, () -> v4.parseSignature());
   }
 
-  @Test(expected = MalformedResourceException.class)
-  public void testV4HeaderInvalidCredential()
-      throws MalformedResourceException {
+  @Test
+  public void testV4HeaderInvalidCredential() {
     String auth = "AWS4-HMAC-SHA256 " +
         "Credential=" + curDate + "/us-east-1/s3/aws4_request, " +
         "SignedHeaders=host;range;x-amz-date, " +
         "Signature=fe5f80f77d5fa3beca038a248ff027";
     AuthorizationV4HeaderParser v4 =
         new AuthorizationV4HeaderParser(auth, SAMPLE_DATE);
-    v4.parseSignature();
+    assertThrows(MalformedResourceException.class, () -> v4.parseSignature());
   }
 
   @Test
@@ -127,22 +122,27 @@ public class TestAuthorizationV4HeaderParser {
   }
 
   @Test
-  public void testV4HeaderDateValidationFailure() throws Exception {
+  public void testV4HeaderDateValidationFailure() {
     // Case 1: Empty date.
     LocalDate now = LocalDate.now();
     String dateStr = "";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> testRequestWithSpecificDate(dateStr));
 
     // Case 2: Date after yesterday.
     String dateStr2 = DATE_FORMATTER.format(now.plus(2, DAYS));
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> testRequestWithSpecificDate(dateStr2));
 
     // Case 3: Date before yesterday.
     String dateStr3 = DATE_FORMATTER.format(now.minus(2, DAYS));
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> testRequestWithSpecificDate(dateStr3));
+
+    // Case 4: Invalid date format
+    String dateStr4 = now.toString();
+    assertThrows(MalformedResourceException.class,
+        () -> testRequestWithSpecificDate(dateStr4));
   }
 
   private void testRequestWithSpecificDate(String dateStr)
@@ -173,7 +173,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027%";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth, SAMPLE_DATE)
             .parseSignature());
     String auth2 =
@@ -181,7 +181,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027%";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth2, SAMPLE_DATE)
             .parseSignature());
   }
@@ -194,7 +194,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth, SAMPLE_DATE)
             .parseSignature());
 
@@ -204,7 +204,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth2, SAMPLE_DATE)
             .parseSignature());
   }
@@ -217,7 +217,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth, SAMPLE_DATE)
             .parseSignature());
 
@@ -227,7 +227,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth2, SAMPLE_DATE)
             .parseSignature());
 
@@ -237,9 +237,19 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth3, SAMPLE_DATE)
             .parseSignature());
+
+    String auth4 =
+        "AWS4-HMAC-SHA256 Credential=ozone/" + curDate + "/us-east-1/s3" +
+            "/invalid_request,"
+            + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
+            + "Signature"
+            + "=fe5f80f77d5fa3beca038a248ff027";
+    assertThrows(MalformedResourceException.class,
+            () -> new AuthorizationV4HeaderParser(auth4, SAMPLE_DATE)
+                .parseSignature());
   }
 
   @Test
@@ -250,7 +260,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=;;,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth, SAMPLE_DATE)
             .parseSignature());
 
@@ -260,7 +270,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth2, SAMPLE_DATE)
             .parseSignature());
 
@@ -270,7 +280,7 @@ public class TestAuthorizationV4HeaderParser {
             + "=x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth3, SAMPLE_DATE)
             .parseSignature());
 
@@ -280,7 +290,7 @@ public class TestAuthorizationV4HeaderParser {
             + "=,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth4, SAMPLE_DATE)
             .parseSignature());
   }
@@ -293,7 +303,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027%";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth, SAMPLE_DATE)
             .parseSignature());
 
@@ -303,7 +313,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth2, SAMPLE_DATE)
             .parseSignature());
 
@@ -313,7 +323,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + ""
             + "=";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth3, SAMPLE_DATE)
             .parseSignature());
   }
@@ -326,7 +336,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth, SAMPLE_DATE)
             .parseSignature());
 
@@ -336,7 +346,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    Assert.assertNull(new AuthorizationV4HeaderParser(auth2, SAMPLE_DATE)
+    assertNull(new AuthorizationV4HeaderParser(auth2, SAMPLE_DATE)
         .parseSignature());
 
     String auth3 =
@@ -345,8 +355,17 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    Assert.assertNull(new AuthorizationV4HeaderParser(auth3, SAMPLE_DATE)
+    assertNull(new AuthorizationV4HeaderParser(auth3, SAMPLE_DATE)
         .parseSignature());
+
+    // Invalid algorithm
+    String auth4 = "AWS4-ZAVC-HJUA123 " +
+        "Credential=" + curDate + "/us-east-1/s3/aws4_request, " +
+        "SignedHeaders=host;range;x-amz-date, " +
+        "Signature=fe5f80f77d5fa3beca038a248ff027";
+    assertThrows(MalformedResourceException.class,
+        () -> new AuthorizationV4HeaderParser(auth4, SAMPLE_DATE)
+            .parseSignature());
   }
 
   @Test
@@ -357,7 +376,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth, SAMPLE_DATE)
             .parseSignature());
 
@@ -367,7 +386,7 @@ public class TestAuthorizationV4HeaderParser {
             + "SignedHeaders=host;x-amz-content-sha256;x-amz-date,"
             + "Signature"
             + "=fe5f80f77d5fa3beca038a248ff027";
-    LambdaTestUtils.intercept(MalformedResourceException.class, "",
+    assertThrows(MalformedResourceException.class,
         () -> new AuthorizationV4HeaderParser(auth2, SAMPLE_DATE)
             .parseSignature());
   }
