@@ -32,8 +32,9 @@ import org.apache.hadoop.ozone.recon.spi.impl.ReconDBDefinition;
 
 import static org.apache.hadoop.ozone.recon.ReconConstants.RECON_CONTAINER_KEY_DB;
 import static org.apache.hadoop.ozone.recon.ReconConstants.RECON_OM_SNAPSHOT_DB;
-import static org.junit.Assert.assertTrue;
-import org.junit.Test;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+
+import org.junit.jupiter.api.Test;
 
 /**
  * Simple factory unit test.
@@ -77,4 +78,4 @@ public class TestDBDefinitionFactory {
             new OzoneConfiguration());
     assertTrue(definition instanceof DatanodeSchemaThreeDBDefinition);
   }
-}
\ No newline at end of file
+}
